# ----------------------------------------------------------------------------
# pyglet
# Copyright (c) 2006-2008 Alex Holkner
# Copyright (c) 2008-2020 pyglet contributors
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#  * Neither the name of pyglet nor the names of its
#    contributors may be used to endorse or promote products
#    derived from this software without specific prior written
#    permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# ----------------------------------------------------------------------------

"""Wrapper for http://developer.download.nvidia.com/opengl/includes/glext.h

Generated by tools/gengl.py.
Do not modify this file.
"""

from ctypes import *
from pyglet.gl.lib import link_GL as _link_function
from pyglet.gl.lib import c_ptrdiff_t

# BEGIN GENERATED CONTENT (do not edit below this line)

# This content is generated by tools/gengl.py.
# Wrapper for http://developer.download.nvidia.com/opengl/includes/glext.h


# ARB_multitexture (/usr/include/GL/gl.h:1932)
GL_GLEXT_VERSION = 44 	# GL/glext.h:74
# VERSION_1_2 (GL/glext.h:76)
# ARB_imaging (GL/glext.h:120)
# VERSION_1_3 (GL/glext.h:198)
# VERSION_1_4 (GL/glext.h:297)
GL_BLEND_DST_RGB = 32968 	# GL/glext.h:298
GL_BLEND_SRC_RGB = 32969 	# GL/glext.h:299
GL_BLEND_DST_ALPHA = 32970 	# GL/glext.h:300
GL_BLEND_SRC_ALPHA = 32971 	# GL/glext.h:301
GL_POINT_SIZE_MIN = 33062 	# GL/glext.h:302
GL_POINT_SIZE_MAX = 33063 	# GL/glext.h:303
GL_POINT_FADE_THRESHOLD_SIZE = 33064 	# GL/glext.h:304
GL_POINT_DISTANCE_ATTENUATION = 33065 	# GL/glext.h:305
GL_GENERATE_MIPMAP = 33169 	# GL/glext.h:306
GL_GENERATE_MIPMAP_HINT = 33170 	# GL/glext.h:307
GL_DEPTH_COMPONENT16 = 33189 	# GL/glext.h:308
GL_DEPTH_COMPONENT24 = 33190 	# GL/glext.h:309
GL_DEPTH_COMPONENT32 = 33191 	# GL/glext.h:310
GL_MIRRORED_REPEAT = 33648 	# GL/glext.h:311
GL_FOG_COORDINATE_SOURCE = 33872 	# GL/glext.h:312
GL_FOG_COORDINATE = 33873 	# GL/glext.h:313
GL_FRAGMENT_DEPTH = 33874 	# GL/glext.h:314
GL_CURRENT_FOG_COORDINATE = 33875 	# GL/glext.h:315
GL_FOG_COORDINATE_ARRAY_TYPE = 33876 	# GL/glext.h:316
GL_FOG_COORDINATE_ARRAY_STRIDE = 33877 	# GL/glext.h:317
GL_FOG_COORDINATE_ARRAY_POINTER = 33878 	# GL/glext.h:318
GL_FOG_COORDINATE_ARRAY = 33879 	# GL/glext.h:319
GL_COLOR_SUM = 33880 	# GL/glext.h:320
GL_CURRENT_SECONDARY_COLOR = 33881 	# GL/glext.h:321
GL_SECONDARY_COLOR_ARRAY_SIZE = 33882 	# GL/glext.h:322
GL_SECONDARY_COLOR_ARRAY_TYPE = 33883 	# GL/glext.h:323
GL_SECONDARY_COLOR_ARRAY_STRIDE = 33884 	# GL/glext.h:324
GL_SECONDARY_COLOR_ARRAY_POINTER = 33885 	# GL/glext.h:325
GL_SECONDARY_COLOR_ARRAY = 33886 	# GL/glext.h:326
GL_MAX_TEXTURE_LOD_BIAS = 34045 	# GL/glext.h:327
GL_TEXTURE_FILTER_CONTROL = 34048 	# GL/glext.h:328
GL_TEXTURE_LOD_BIAS = 34049 	# GL/glext.h:329
GL_INCR_WRAP = 34055 	# GL/glext.h:330
GL_DECR_WRAP = 34056 	# GL/glext.h:331
GL_TEXTURE_DEPTH_SIZE = 34890 	# GL/glext.h:332
GL_DEPTH_TEXTURE_MODE = 34891 	# GL/glext.h:333
GL_TEXTURE_COMPARE_MODE = 34892 	# GL/glext.h:334
GL_TEXTURE_COMPARE_FUNC = 34893 	# GL/glext.h:335
GL_COMPARE_R_TO_TEXTURE = 34894 	# GL/glext.h:336
# VERSION_1_5 (GL/glext.h:339)
GL_BUFFER_SIZE = 34660 	# GL/glext.h:340
GL_BUFFER_USAGE = 34661 	# GL/glext.h:341
GL_QUERY_COUNTER_BITS = 34916 	# GL/glext.h:342
GL_CURRENT_QUERY = 34917 	# GL/glext.h:343
GL_QUERY_RESULT = 34918 	# GL/glext.h:344
GL_QUERY_RESULT_AVAILABLE = 34919 	# GL/glext.h:345
GL_ARRAY_BUFFER = 34962 	# GL/glext.h:346
GL_ELEMENT_ARRAY_BUFFER = 34963 	# GL/glext.h:347
GL_ARRAY_BUFFER_BINDING = 34964 	# GL/glext.h:348
GL_ELEMENT_ARRAY_BUFFER_BINDING = 34965 	# GL/glext.h:349
GL_VERTEX_ARRAY_BUFFER_BINDING = 34966 	# GL/glext.h:350
GL_NORMAL_ARRAY_BUFFER_BINDING = 34967 	# GL/glext.h:351
GL_COLOR_ARRAY_BUFFER_BINDING = 34968 	# GL/glext.h:352
GL_INDEX_ARRAY_BUFFER_BINDING = 34969 	# GL/glext.h:353
GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING = 34970 	# GL/glext.h:354
GL_EDGE_FLAG_ARRAY_BUFFER_BINDING = 34971 	# GL/glext.h:355
GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING = 34972 	# GL/glext.h:356
GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING = 34973 	# GL/glext.h:357
GL_WEIGHT_ARRAY_BUFFER_BINDING = 34974 	# GL/glext.h:358
GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING = 34975 	# GL/glext.h:359
GL_READ_ONLY = 35000 	# GL/glext.h:360
GL_WRITE_ONLY = 35001 	# GL/glext.h:361
GL_READ_WRITE = 35002 	# GL/glext.h:362
GL_BUFFER_ACCESS = 35003 	# GL/glext.h:363
GL_BUFFER_MAPPED = 35004 	# GL/glext.h:364
GL_BUFFER_MAP_POINTER = 35005 	# GL/glext.h:365
GL_STREAM_DRAW = 35040 	# GL/glext.h:366
GL_STREAM_READ = 35041 	# GL/glext.h:367
GL_STREAM_COPY = 35042 	# GL/glext.h:368
GL_STATIC_DRAW = 35044 	# GL/glext.h:369
GL_STATIC_READ = 35045 	# GL/glext.h:370
GL_STATIC_COPY = 35046 	# GL/glext.h:371
GL_DYNAMIC_DRAW = 35048 	# GL/glext.h:372
GL_DYNAMIC_READ = 35049 	# GL/glext.h:373
GL_DYNAMIC_COPY = 35050 	# GL/glext.h:374
GL_SAMPLES_PASSED = 35092 	# GL/glext.h:375
GL_FOG_COORD_SRC = 33872 	# GL/glext.h:376
GL_FOG_COORD = 33873 	# GL/glext.h:377
GL_CURRENT_FOG_COORD = 33875 	# GL/glext.h:378
GL_FOG_COORD_ARRAY_TYPE = 33876 	# GL/glext.h:379
GL_FOG_COORD_ARRAY_STRIDE = 33877 	# GL/glext.h:380
GL_FOG_COORD_ARRAY_POINTER = 33878 	# GL/glext.h:381
GL_FOG_COORD_ARRAY = 33879 	# GL/glext.h:382
GL_FOG_COORD_ARRAY_BUFFER_BINDING = 34973 	# GL/glext.h:383
GL_SRC0_RGB = 34176 	# GL/glext.h:384
GL_SRC1_RGB = 34177 	# GL/glext.h:385
GL_SRC2_RGB = 34178 	# GL/glext.h:386
GL_SRC0_ALPHA = 34184 	# GL/glext.h:387
GL_SRC1_ALPHA = 34185 	# GL/glext.h:388
GL_SRC2_ALPHA = 34186 	# GL/glext.h:389
# VERSION_2_0 (GL/glext.h:392)
GL_BLEND_EQUATION_RGB = 32777 	# GL/glext.h:393
GL_VERTEX_ATTRIB_ARRAY_ENABLED = 34338 	# GL/glext.h:394
GL_VERTEX_ATTRIB_ARRAY_SIZE = 34339 	# GL/glext.h:395
GL_VERTEX_ATTRIB_ARRAY_STRIDE = 34340 	# GL/glext.h:396
GL_VERTEX_ATTRIB_ARRAY_TYPE = 34341 	# GL/glext.h:397
GL_CURRENT_VERTEX_ATTRIB = 34342 	# GL/glext.h:398
GL_VERTEX_PROGRAM_POINT_SIZE = 34370 	# GL/glext.h:399
GL_VERTEX_PROGRAM_TWO_SIDE = 34371 	# GL/glext.h:400
GL_VERTEX_ATTRIB_ARRAY_POINTER = 34373 	# GL/glext.h:401
GL_STENCIL_BACK_FUNC = 34816 	# GL/glext.h:402
GL_STENCIL_BACK_FAIL = 34817 	# GL/glext.h:403
GL_STENCIL_BACK_PASS_DEPTH_FAIL = 34818 	# GL/glext.h:404
GL_STENCIL_BACK_PASS_DEPTH_PASS = 34819 	# GL/glext.h:405
GL_MAX_DRAW_BUFFERS = 34852 	# GL/glext.h:406
GL_DRAW_BUFFER0 = 34853 	# GL/glext.h:407
GL_DRAW_BUFFER1 = 34854 	# GL/glext.h:408
GL_DRAW_BUFFER2 = 34855 	# GL/glext.h:409
GL_DRAW_BUFFER3 = 34856 	# GL/glext.h:410
GL_DRAW_BUFFER4 = 34857 	# GL/glext.h:411
GL_DRAW_BUFFER5 = 34858 	# GL/glext.h:412
GL_DRAW_BUFFER6 = 34859 	# GL/glext.h:413
GL_DRAW_BUFFER7 = 34860 	# GL/glext.h:414
GL_DRAW_BUFFER8 = 34861 	# GL/glext.h:415
GL_DRAW_BUFFER9 = 34862 	# GL/glext.h:416
GL_DRAW_BUFFER10 = 34863 	# GL/glext.h:417
GL_DRAW_BUFFER11 = 34864 	# GL/glext.h:418
GL_DRAW_BUFFER12 = 34865 	# GL/glext.h:419
GL_DRAW_BUFFER13 = 34866 	# GL/glext.h:420
GL_DRAW_BUFFER14 = 34867 	# GL/glext.h:421
GL_DRAW_BUFFER15 = 34868 	# GL/glext.h:422
GL_BLEND_EQUATION_ALPHA = 34877 	# GL/glext.h:423
GL_POINT_SPRITE = 34913 	# GL/glext.h:424
GL_COORD_REPLACE = 34914 	# GL/glext.h:425
GL_MAX_VERTEX_ATTRIBS = 34921 	# GL/glext.h:426
GL_VERTEX_ATTRIB_ARRAY_NORMALIZED = 34922 	# GL/glext.h:427
GL_MAX_TEXTURE_COORDS = 34929 	# GL/glext.h:428
GL_MAX_TEXTURE_IMAGE_UNITS = 34930 	# GL/glext.h:429
GL_FRAGMENT_SHADER = 35632 	# GL/glext.h:430
GL_VERTEX_SHADER = 35633 	# GL/glext.h:431
GL_MAX_FRAGMENT_UNIFORM_COMPONENTS = 35657 	# GL/glext.h:432
GL_MAX_VERTEX_UNIFORM_COMPONENTS = 35658 	# GL/glext.h:433
GL_MAX_VARYING_FLOATS = 35659 	# GL/glext.h:434
GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS = 35660 	# GL/glext.h:435
GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS = 35661 	# GL/glext.h:436
GL_SHADER_TYPE = 35663 	# GL/glext.h:437
GL_FLOAT_VEC2 = 35664 	# GL/glext.h:438
GL_FLOAT_VEC3 = 35665 	# GL/glext.h:439
GL_FLOAT_VEC4 = 35666 	# GL/glext.h:440
GL_INT_VEC2 = 35667 	# GL/glext.h:441
GL_INT_VEC3 = 35668 	# GL/glext.h:442
GL_INT_VEC4 = 35669 	# GL/glext.h:443
GL_BOOL = 35670 	# GL/glext.h:444
GL_BOOL_VEC2 = 35671 	# GL/glext.h:445
GL_BOOL_VEC3 = 35672 	# GL/glext.h:446
GL_BOOL_VEC4 = 35673 	# GL/glext.h:447
GL_FLOAT_MAT2 = 35674 	# GL/glext.h:448
GL_FLOAT_MAT3 = 35675 	# GL/glext.h:449
GL_FLOAT_MAT4 = 35676 	# GL/glext.h:450
GL_SAMPLER_1D = 35677 	# GL/glext.h:451
GL_SAMPLER_2D = 35678 	# GL/glext.h:452
GL_SAMPLER_3D = 35679 	# GL/glext.h:453
GL_SAMPLER_CUBE = 35680 	# GL/glext.h:454
GL_SAMPLER_1D_SHADOW = 35681 	# GL/glext.h:455
GL_SAMPLER_2D_SHADOW = 35682 	# GL/glext.h:456
GL_DELETE_STATUS = 35712 	# GL/glext.h:457
GL_COMPILE_STATUS = 35713 	# GL/glext.h:458
GL_LINK_STATUS = 35714 	# GL/glext.h:459
GL_VALIDATE_STATUS = 35715 	# GL/glext.h:460
GL_INFO_LOG_LENGTH = 35716 	# GL/glext.h:461
GL_ATTACHED_SHADERS = 35717 	# GL/glext.h:462
GL_ACTIVE_UNIFORMS = 35718 	# GL/glext.h:463
GL_ACTIVE_UNIFORM_MAX_LENGTH = 35719 	# GL/glext.h:464
GL_SHADER_SOURCE_LENGTH = 35720 	# GL/glext.h:465
GL_ACTIVE_ATTRIBUTES = 35721 	# GL/glext.h:466
GL_ACTIVE_ATTRIBUTE_MAX_LENGTH = 35722 	# GL/glext.h:467
GL_FRAGMENT_SHADER_DERIVATIVE_HINT = 35723 	# GL/glext.h:468
GL_SHADING_LANGUAGE_VERSION = 35724 	# GL/glext.h:469
GL_CURRENT_PROGRAM = 35725 	# GL/glext.h:470
GL_POINT_SPRITE_COORD_ORIGIN = 36000 	# GL/glext.h:471
GL_LOWER_LEFT = 36001 	# GL/glext.h:472
GL_UPPER_LEFT = 36002 	# GL/glext.h:473
GL_STENCIL_BACK_REF = 36003 	# GL/glext.h:474
GL_STENCIL_BACK_VALUE_MASK = 36004 	# GL/glext.h:475
GL_STENCIL_BACK_WRITEMASK = 36005 	# GL/glext.h:476
# VERSION_2_1 (GL/glext.h:479)
GL_PIXEL_PACK_BUFFER = 35051 	# GL/glext.h:480
GL_PIXEL_UNPACK_BUFFER = 35052 	# GL/glext.h:481
GL_PIXEL_PACK_BUFFER_BINDING = 35053 	# GL/glext.h:482
GL_PIXEL_UNPACK_BUFFER_BINDING = 35055 	# GL/glext.h:483
GL_SRGB = 35904 	# GL/glext.h:484
GL_SRGB8 = 35905 	# GL/glext.h:485
GL_SRGB_ALPHA = 35906 	# GL/glext.h:486
GL_SRGB8_ALPHA8 = 35907 	# GL/glext.h:487
GL_SLUMINANCE_ALPHA = 35908 	# GL/glext.h:488
GL_SLUMINANCE8_ALPHA8 = 35909 	# GL/glext.h:489
GL_SLUMINANCE = 35910 	# GL/glext.h:490
GL_SLUMINANCE8 = 35911 	# GL/glext.h:491
GL_COMPRESSED_SRGB = 35912 	# GL/glext.h:492
GL_COMPRESSED_SRGB_ALPHA = 35913 	# GL/glext.h:493
GL_COMPRESSED_SLUMINANCE = 35914 	# GL/glext.h:494
GL_COMPRESSED_SLUMINANCE_ALPHA = 35915 	# GL/glext.h:495
GL_FLOAT_MAT2x3 = 35685 	# GL/glext.h:496
GL_FLOAT_MAT2x4 = 35686 	# GL/glext.h:497
GL_FLOAT_MAT3x2 = 35687 	# GL/glext.h:498
GL_FLOAT_MAT3x4 = 35688 	# GL/glext.h:499
GL_FLOAT_MAT4x2 = 35689 	# GL/glext.h:500
GL_FLOAT_MAT4x3 = 35690 	# GL/glext.h:501
GL_CURRENT_RASTER_SECONDARY_COLOR = 33887 	# GL/glext.h:502
# VERSION_3_0 (GL/glext.h:505)
GL_CLIP_DISTANCE0 = 12288 	# GL/glext.h:507
GL_CLIP_DISTANCE1 = 12289 	# GL/glext.h:508
GL_CLIP_DISTANCE2 = 12290 	# GL/glext.h:509
GL_CLIP_DISTANCE3 = 12291 	# GL/glext.h:510
GL_CLIP_DISTANCE4 = 12292 	# GL/glext.h:511
GL_CLIP_DISTANCE5 = 12293 	# GL/glext.h:512
GL_MAX_CLIP_DISTANCES = 3378 	# GL/glext.h:513
GL_MAJOR_VERSION = 33307 	# GL/glext.h:514
GL_MINOR_VERSION = 33308 	# GL/glext.h:515
GL_NUM_EXTENSIONS = 33309 	# GL/glext.h:516
GL_CONTEXT_FLAGS = 33310 	# GL/glext.h:517
GL_DEPTH_BUFFER = 33315 	# GL/glext.h:518
GL_STENCIL_BUFFER = 33316 	# GL/glext.h:519
GL_COMPRESSED_RED = 33317 	# GL/glext.h:520
GL_COMPRESSED_RG = 33318 	# GL/glext.h:521
GL_CONTEXT_FLAG_FORWARD_COMPATIBLE_BIT = 1 	# GL/glext.h:522
GL_RGBA32F = 34836 	# GL/glext.h:523
GL_RGB32F = 34837 	# GL/glext.h:524
GL_RGBA16F = 34842 	# GL/glext.h:525
GL_RGB16F = 34843 	# GL/glext.h:526
GL_VERTEX_ATTRIB_ARRAY_INTEGER = 35069 	# GL/glext.h:527
GL_MAX_ARRAY_TEXTURE_LAYERS = 35071 	# GL/glext.h:528
GL_MIN_PROGRAM_TEXEL_OFFSET = 35076 	# GL/glext.h:529
GL_MAX_PROGRAM_TEXEL_OFFSET = 35077 	# GL/glext.h:530
GL_CLAMP_VERTEX_COLOR = 35098 	# GL/glext.h:531
GL_CLAMP_FRAGMENT_COLOR = 35099 	# GL/glext.h:532
GL_CLAMP_READ_COLOR = 35100 	# GL/glext.h:533
GL_FIXED_ONLY = 35101 	# GL/glext.h:534
GL_MAX_VARYING_COMPONENTS = 35659 	# GL/glext.h:535
GL_TEXTURE_RED_TYPE = 35856 	# GL/glext.h:536
GL_TEXTURE_GREEN_TYPE = 35857 	# GL/glext.h:537
GL_TEXTURE_BLUE_TYPE = 35858 	# GL/glext.h:538
GL_TEXTURE_ALPHA_TYPE = 35859 	# GL/glext.h:539
GL_TEXTURE_LUMINANCE_TYPE = 35860 	# GL/glext.h:540
GL_TEXTURE_INTENSITY_TYPE = 35861 	# GL/glext.h:541
GL_TEXTURE_DEPTH_TYPE = 35862 	# GL/glext.h:542
GL_UNSIGNED_NORMALIZED = 35863 	# GL/glext.h:543
GL_TEXTURE_1D_ARRAY = 35864 	# GL/glext.h:544
GL_PROXY_TEXTURE_1D_ARRAY = 35865 	# GL/glext.h:545
GL_TEXTURE_2D_ARRAY = 35866 	# GL/glext.h:546
GL_PROXY_TEXTURE_2D_ARRAY = 35867 	# GL/glext.h:547
GL_TEXTURE_BINDING_1D_ARRAY = 35868 	# GL/glext.h:548
GL_TEXTURE_BINDING_2D_ARRAY = 35869 	# GL/glext.h:549
GL_R11F_G11F_B10F = 35898 	# GL/glext.h:550
GL_UNSIGNED_INT_10F_11F_11F_REV = 35899 	# GL/glext.h:551
GL_RGB9_E5 = 35901 	# GL/glext.h:552
GL_UNSIGNED_INT_5_9_9_9_REV = 35902 	# GL/glext.h:553
GL_TEXTURE_SHARED_SIZE = 35903 	# GL/glext.h:554
GL_TRANSFORM_FEEDBACK_VARYING_MAX_LENGTH = 35958 	# GL/glext.h:555
GL_TRANSFORM_FEEDBACK_BUFFER_MODE = 35967 	# GL/glext.h:556
GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS = 35968 	# GL/glext.h:557
GL_TRANSFORM_FEEDBACK_VARYINGS = 35971 	# GL/glext.h:558
GL_TRANSFORM_FEEDBACK_BUFFER_START = 35972 	# GL/glext.h:559
GL_TRANSFORM_FEEDBACK_BUFFER_SIZE = 35973 	# GL/glext.h:560
GL_PRIMITIVES_GENERATED = 35975 	# GL/glext.h:561
GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN = 35976 	# GL/glext.h:562
GL_RASTERIZER_DISCARD = 35977 	# GL/glext.h:563
GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_COMPONENTS = 35978 	# GL/glext.h:564
GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS = 35979 	# GL/glext.h:565
GL_INTERLEAVED_ATTRIBS = 35980 	# GL/glext.h:566
GL_SEPARATE_ATTRIBS = 35981 	# GL/glext.h:567
GL_TRANSFORM_FEEDBACK_BUFFER = 35982 	# GL/glext.h:568
GL_TRANSFORM_FEEDBACK_BUFFER_BINDING = 35983 	# GL/glext.h:569
GL_RGBA32UI = 36208 	# GL/glext.h:570
GL_RGB32UI = 36209 	# GL/glext.h:571
GL_RGBA16UI = 36214 	# GL/glext.h:572
GL_RGB16UI = 36215 	# GL/glext.h:573
GL_RGBA8UI = 36220 	# GL/glext.h:574
GL_RGB8UI = 36221 	# GL/glext.h:575
GL_RGBA32I = 36226 	# GL/glext.h:576
GL_RGB32I = 36227 	# GL/glext.h:577
GL_RGBA16I = 36232 	# GL/glext.h:578
GL_RGB16I = 36233 	# GL/glext.h:579
GL_RGBA8I = 36238 	# GL/glext.h:580
GL_RGB8I = 36239 	# GL/glext.h:581
GL_RED_INTEGER = 36244 	# GL/glext.h:582
GL_GREEN_INTEGER = 36245 	# GL/glext.h:583
GL_BLUE_INTEGER = 36246 	# GL/glext.h:584
GL_ALPHA_INTEGER = 36247 	# GL/glext.h:585
GL_RGB_INTEGER = 36248 	# GL/glext.h:586
GL_RGBA_INTEGER = 36249 	# GL/glext.h:587
GL_BGR_INTEGER = 36250 	# GL/glext.h:588
GL_BGRA_INTEGER = 36251 	# GL/glext.h:589
GL_SAMPLER_1D_ARRAY = 36288 	# GL/glext.h:590
GL_SAMPLER_2D_ARRAY = 36289 	# GL/glext.h:591
GL_SAMPLER_1D_ARRAY_SHADOW = 36291 	# GL/glext.h:592
GL_SAMPLER_2D_ARRAY_SHADOW = 36292 	# GL/glext.h:593
GL_SAMPLER_CUBE_SHADOW = 36293 	# GL/glext.h:594
GL_UNSIGNED_INT_VEC2 = 36294 	# GL/glext.h:595
GL_UNSIGNED_INT_VEC3 = 36295 	# GL/glext.h:596
GL_UNSIGNED_INT_VEC4 = 36296 	# GL/glext.h:597
GL_INT_SAMPLER_1D = 36297 	# GL/glext.h:598
GL_INT_SAMPLER_2D = 36298 	# GL/glext.h:599
GL_INT_SAMPLER_3D = 36299 	# GL/glext.h:600
GL_INT_SAMPLER_CUBE = 36300 	# GL/glext.h:601
GL_INT_SAMPLER_1D_ARRAY = 36302 	# GL/glext.h:602
GL_INT_SAMPLER_2D_ARRAY = 36303 	# GL/glext.h:603
GL_UNSIGNED_INT_SAMPLER_1D = 36305 	# GL/glext.h:604
GL_UNSIGNED_INT_SAMPLER_2D = 36306 	# GL/glext.h:605
GL_UNSIGNED_INT_SAMPLER_3D = 36307 	# GL/glext.h:606
GL_UNSIGNED_INT_SAMPLER_CUBE = 36308 	# GL/glext.h:607
GL_UNSIGNED_INT_SAMPLER_1D_ARRAY = 36310 	# GL/glext.h:608
GL_UNSIGNED_INT_SAMPLER_2D_ARRAY = 36311 	# GL/glext.h:609
GL_QUERY_WAIT = 36371 	# GL/glext.h:610
GL_QUERY_NO_WAIT = 36372 	# GL/glext.h:611
GL_QUERY_BY_REGION_WAIT = 36373 	# GL/glext.h:612
GL_QUERY_BY_REGION_NO_WAIT = 36374 	# GL/glext.h:613
# ARB_multitexture (GL/glext.h:742)
# ARB_transpose_matrix (GL/glext.h:780)
GL_TRANSPOSE_MODELVIEW_MATRIX_ARB = 34019 	# GL/glext.h:781
GL_TRANSPOSE_PROJECTION_MATRIX_ARB = 34020 	# GL/glext.h:782
GL_TRANSPOSE_TEXTURE_MATRIX_ARB = 34021 	# GL/glext.h:783
GL_TRANSPOSE_COLOR_MATRIX_ARB = 34022 	# GL/glext.h:784
# ARB_multisample (GL/glext.h:787)
GL_MULTISAMPLE_ARB = 32925 	# GL/glext.h:788
GL_SAMPLE_ALPHA_TO_COVERAGE_ARB = 32926 	# GL/glext.h:789
GL_SAMPLE_ALPHA_TO_ONE_ARB = 32927 	# GL/glext.h:790
GL_SAMPLE_COVERAGE_ARB = 32928 	# GL/glext.h:791
GL_SAMPLE_BUFFERS_ARB = 32936 	# GL/glext.h:792
GL_SAMPLES_ARB = 32937 	# GL/glext.h:793
GL_SAMPLE_COVERAGE_VALUE_ARB = 32938 	# GL/glext.h:794
GL_SAMPLE_COVERAGE_INVERT_ARB = 32939 	# GL/glext.h:795
GL_MULTISAMPLE_BIT_ARB = 536870912 	# GL/glext.h:796
# ARB_texture_env_add (GL/glext.h:799)
# ARB_texture_cube_map (GL/glext.h:802)
GL_NORMAL_MAP_ARB = 34065 	# GL/glext.h:803
GL_REFLECTION_MAP_ARB = 34066 	# GL/glext.h:804
GL_TEXTURE_CUBE_MAP_ARB = 34067 	# GL/glext.h:805
GL_TEXTURE_BINDING_CUBE_MAP_ARB = 34068 	# GL/glext.h:806
GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB = 34069 	# GL/glext.h:807
GL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB = 34070 	# GL/glext.h:808
GL_TEXTURE_CUBE_MAP_POSITIVE_Y_ARB = 34071 	# GL/glext.h:809
GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_ARB = 34072 	# GL/glext.h:810
GL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB = 34073 	# GL/glext.h:811
GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB = 34074 	# GL/glext.h:812
GL_PROXY_TEXTURE_CUBE_MAP_ARB = 34075 	# GL/glext.h:813
GL_MAX_CUBE_MAP_TEXTURE_SIZE_ARB = 34076 	# GL/glext.h:814
# ARB_texture_compression (GL/glext.h:817)
GL_COMPRESSED_ALPHA_ARB = 34025 	# GL/glext.h:818
GL_COMPRESSED_LUMINANCE_ARB = 34026 	# GL/glext.h:819
GL_COMPRESSED_LUMINANCE_ALPHA_ARB = 34027 	# GL/glext.h:820
GL_COMPRESSED_INTENSITY_ARB = 34028 	# GL/glext.h:821
GL_COMPRESSED_RGB_ARB = 34029 	# GL/glext.h:822
GL_COMPRESSED_RGBA_ARB = 34030 	# GL/glext.h:823
GL_TEXTURE_COMPRESSION_HINT_ARB = 34031 	# GL/glext.h:824
GL_TEXTURE_COMPRESSED_IMAGE_SIZE_ARB = 34464 	# GL/glext.h:825
GL_TEXTURE_COMPRESSED_ARB = 34465 	# GL/glext.h:826
GL_NUM_COMPRESSED_TEXTURE_FORMATS_ARB = 34466 	# GL/glext.h:827
GL_COMPRESSED_TEXTURE_FORMATS_ARB = 34467 	# GL/glext.h:828
# ARB_texture_border_clamp (GL/glext.h:831)
GL_CLAMP_TO_BORDER_ARB = 33069 	# GL/glext.h:832
# ARB_point_parameters (GL/glext.h:835)
GL_POINT_SIZE_MIN_ARB = 33062 	# GL/glext.h:836
GL_POINT_SIZE_MAX_ARB = 33063 	# GL/glext.h:837
GL_POINT_FADE_THRESHOLD_SIZE_ARB = 33064 	# GL/glext.h:838
GL_POINT_DISTANCE_ATTENUATION_ARB = 33065 	# GL/glext.h:839
# ARB_vertex_blend (GL/glext.h:842)
GL_MAX_VERTEX_UNITS_ARB = 34468 	# GL/glext.h:843
GL_ACTIVE_VERTEX_UNITS_ARB = 34469 	# GL/glext.h:844
GL_WEIGHT_SUM_UNITY_ARB = 34470 	# GL/glext.h:845
GL_VERTEX_BLEND_ARB = 34471 	# GL/glext.h:846
GL_CURRENT_WEIGHT_ARB = 34472 	# GL/glext.h:847
GL_WEIGHT_ARRAY_TYPE_ARB = 34473 	# GL/glext.h:848
GL_WEIGHT_ARRAY_STRIDE_ARB = 34474 	# GL/glext.h:849
GL_WEIGHT_ARRAY_SIZE_ARB = 34475 	# GL/glext.h:850
GL_WEIGHT_ARRAY_POINTER_ARB = 34476 	# GL/glext.h:851
GL_WEIGHT_ARRAY_ARB = 34477 	# GL/glext.h:852
GL_MODELVIEW0_ARB = 5888 	# GL/glext.h:853
GL_MODELVIEW1_ARB = 34058 	# GL/glext.h:854
GL_MODELVIEW2_ARB = 34594 	# GL/glext.h:855
GL_MODELVIEW3_ARB = 34595 	# GL/glext.h:856
GL_MODELVIEW4_ARB = 34596 	# GL/glext.h:857
GL_MODELVIEW5_ARB = 34597 	# GL/glext.h:858
GL_MODELVIEW6_ARB = 34598 	# GL/glext.h:859
GL_MODELVIEW7_ARB = 34599 	# GL/glext.h:860
GL_MODELVIEW8_ARB = 34600 	# GL/glext.h:861
GL_MODELVIEW9_ARB = 34601 	# GL/glext.h:862
GL_MODELVIEW10_ARB = 34602 	# GL/glext.h:863
GL_MODELVIEW11_ARB = 34603 	# GL/glext.h:864
GL_MODELVIEW12_ARB = 34604 	# GL/glext.h:865
GL_MODELVIEW13_ARB = 34605 	# GL/glext.h:866
GL_MODELVIEW14_ARB = 34606 	# GL/glext.h:867
GL_MODELVIEW15_ARB = 34607 	# GL/glext.h:868
GL_MODELVIEW16_ARB = 34608 	# GL/glext.h:869
GL_MODELVIEW17_ARB = 34609 	# GL/glext.h:870
GL_MODELVIEW18_ARB = 34610 	# GL/glext.h:871
GL_MODELVIEW19_ARB = 34611 	# GL/glext.h:872
GL_MODELVIEW20_ARB = 34612 	# GL/glext.h:873
GL_MODELVIEW21_ARB = 34613 	# GL/glext.h:874
GL_MODELVIEW22_ARB = 34614 	# GL/glext.h:875
GL_MODELVIEW23_ARB = 34615 	# GL/glext.h:876
GL_MODELVIEW24_ARB = 34616 	# GL/glext.h:877
GL_MODELVIEW25_ARB = 34617 	# GL/glext.h:878
GL_MODELVIEW26_ARB = 34618 	# GL/glext.h:879
GL_MODELVIEW27_ARB = 34619 	# GL/glext.h:880
GL_MODELVIEW28_ARB = 34620 	# GL/glext.h:881
GL_MODELVIEW29_ARB = 34621 	# GL/glext.h:882
GL_MODELVIEW30_ARB = 34622 	# GL/glext.h:883
GL_MODELVIEW31_ARB = 34623 	# GL/glext.h:884
# ARB_matrix_palette (GL/glext.h:887)
GL_MATRIX_PALETTE_ARB = 34880 	# GL/glext.h:888
GL_MAX_MATRIX_PALETTE_STACK_DEPTH_ARB = 34881 	# GL/glext.h:889
GL_MAX_PALETTE_MATRICES_ARB = 34882 	# GL/glext.h:890
GL_CURRENT_PALETTE_MATRIX_ARB = 34883 	# GL/glext.h:891
GL_MATRIX_INDEX_ARRAY_ARB = 34884 	# GL/glext.h:892
GL_CURRENT_MATRIX_INDEX_ARB = 34885 	# GL/glext.h:893
GL_MATRIX_INDEX_ARRAY_SIZE_ARB = 34886 	# GL/glext.h:894
GL_MATRIX_INDEX_ARRAY_TYPE_ARB = 34887 	# GL/glext.h:895
GL_MATRIX_INDEX_ARRAY_STRIDE_ARB = 34888 	# GL/glext.h:896
GL_MATRIX_INDEX_ARRAY_POINTER_ARB = 34889 	# GL/glext.h:897
# ARB_texture_env_combine (GL/glext.h:900)
GL_COMBINE_ARB = 34160 	# GL/glext.h:901
GL_COMBINE_RGB_ARB = 34161 	# GL/glext.h:902
GL_COMBINE_ALPHA_ARB = 34162 	# GL/glext.h:903
GL_SOURCE0_RGB_ARB = 34176 	# GL/glext.h:904
GL_SOURCE1_RGB_ARB = 34177 	# GL/glext.h:905
GL_SOURCE2_RGB_ARB = 34178 	# GL/glext.h:906
GL_SOURCE0_ALPHA_ARB = 34184 	# GL/glext.h:907
GL_SOURCE1_ALPHA_ARB = 34185 	# GL/glext.h:908
GL_SOURCE2_ALPHA_ARB = 34186 	# GL/glext.h:909
GL_OPERAND0_RGB_ARB = 34192 	# GL/glext.h:910
GL_OPERAND1_RGB_ARB = 34193 	# GL/glext.h:911
GL_OPERAND2_RGB_ARB = 34194 	# GL/glext.h:912
GL_OPERAND0_ALPHA_ARB = 34200 	# GL/glext.h:913
GL_OPERAND1_ALPHA_ARB = 34201 	# GL/glext.h:914
GL_OPERAND2_ALPHA_ARB = 34202 	# GL/glext.h:915
GL_RGB_SCALE_ARB = 34163 	# GL/glext.h:916
GL_ADD_SIGNED_ARB = 34164 	# GL/glext.h:917
GL_INTERPOLATE_ARB = 34165 	# GL/glext.h:918
GL_SUBTRACT_ARB = 34023 	# GL/glext.h:919
GL_CONSTANT_ARB = 34166 	# GL/glext.h:920
GL_PRIMARY_COLOR_ARB = 34167 	# GL/glext.h:921
GL_PREVIOUS_ARB = 34168 	# GL/glext.h:922
# ARB_texture_env_crossbar (GL/glext.h:925)
# ARB_texture_env_dot3 (GL/glext.h:928)
GL_DOT3_RGB_ARB = 34478 	# GL/glext.h:929
GL_DOT3_RGBA_ARB = 34479 	# GL/glext.h:930
# ARB_texture_mirrored_repeat (GL/glext.h:933)
GL_MIRRORED_REPEAT_ARB = 33648 	# GL/glext.h:934
# ARB_depth_texture (GL/glext.h:937)
GL_DEPTH_COMPONENT16_ARB = 33189 	# GL/glext.h:938
GL_DEPTH_COMPONENT24_ARB = 33190 	# GL/glext.h:939
GL_DEPTH_COMPONENT32_ARB = 33191 	# GL/glext.h:940
GL_TEXTURE_DEPTH_SIZE_ARB = 34890 	# GL/glext.h:941
GL_DEPTH_TEXTURE_MODE_ARB = 34891 	# GL/glext.h:942
# ARB_shadow (GL/glext.h:945)
GL_TEXTURE_COMPARE_MODE_ARB = 34892 	# GL/glext.h:946
GL_TEXTURE_COMPARE_FUNC_ARB = 34893 	# GL/glext.h:947
GL_COMPARE_R_TO_TEXTURE_ARB = 34894 	# GL/glext.h:948
# ARB_shadow_ambient (GL/glext.h:951)
GL_TEXTURE_COMPARE_FAIL_VALUE_ARB = 32959 	# GL/glext.h:952
# ARB_window_pos (GL/glext.h:955)
# ARB_vertex_program (GL/glext.h:958)
GL_COLOR_SUM_ARB = 33880 	# GL/glext.h:959
GL_VERTEX_PROGRAM_ARB = 34336 	# GL/glext.h:960
GL_VERTEX_ATTRIB_ARRAY_ENABLED_ARB = 34338 	# GL/glext.h:961
GL_VERTEX_ATTRIB_ARRAY_SIZE_ARB = 34339 	# GL/glext.h:962
GL_VERTEX_ATTRIB_ARRAY_STRIDE_ARB = 34340 	# GL/glext.h:963
GL_VERTEX_ATTRIB_ARRAY_TYPE_ARB = 34341 	# GL/glext.h:964
GL_CURRENT_VERTEX_ATTRIB_ARB = 34342 	# GL/glext.h:965
GL_PROGRAM_LENGTH_ARB = 34343 	# GL/glext.h:966
GL_PROGRAM_STRING_ARB = 34344 	# GL/glext.h:967
GL_MAX_PROGRAM_MATRIX_STACK_DEPTH_ARB = 34350 	# GL/glext.h:968
GL_MAX_PROGRAM_MATRICES_ARB = 34351 	# GL/glext.h:969
GL_CURRENT_MATRIX_STACK_DEPTH_ARB = 34368 	# GL/glext.h:970
GL_CURRENT_MATRIX_ARB = 34369 	# GL/glext.h:971
GL_VERTEX_PROGRAM_POINT_SIZE_ARB = 34370 	# GL/glext.h:972
GL_VERTEX_PROGRAM_TWO_SIDE_ARB = 34371 	# GL/glext.h:973
GL_VERTEX_ATTRIB_ARRAY_POINTER_ARB = 34373 	# GL/glext.h:974
GL_PROGRAM_ERROR_POSITION_ARB = 34379 	# GL/glext.h:975
GL_PROGRAM_BINDING_ARB = 34423 	# GL/glext.h:976
GL_MAX_VERTEX_ATTRIBS_ARB = 34921 	# GL/glext.h:977
GL_VERTEX_ATTRIB_ARRAY_NORMALIZED_ARB = 34922 	# GL/glext.h:978
GL_PROGRAM_ERROR_STRING_ARB = 34932 	# GL/glext.h:979
GL_PROGRAM_FORMAT_ASCII_ARB = 34933 	# GL/glext.h:980
GL_PROGRAM_FORMAT_ARB = 34934 	# GL/glext.h:981
GL_PROGRAM_INSTRUCTIONS_ARB = 34976 	# GL/glext.h:982
GL_MAX_PROGRAM_INSTRUCTIONS_ARB = 34977 	# GL/glext.h:983
GL_PROGRAM_NATIVE_INSTRUCTIONS_ARB = 34978 	# GL/glext.h:984
GL_MAX_PROGRAM_NATIVE_INSTRUCTIONS_ARB = 34979 	# GL/glext.h:985
GL_PROGRAM_TEMPORARIES_ARB = 34980 	# GL/glext.h:986
GL_MAX_PROGRAM_TEMPORARIES_ARB = 34981 	# GL/glext.h:987
GL_PROGRAM_NATIVE_TEMPORARIES_ARB = 34982 	# GL/glext.h:988
GL_MAX_PROGRAM_NATIVE_TEMPORARIES_ARB = 34983 	# GL/glext.h:989
GL_PROGRAM_PARAMETERS_ARB = 34984 	# GL/glext.h:990
GL_MAX_PROGRAM_PARAMETERS_ARB = 34985 	# GL/glext.h:991
GL_PROGRAM_NATIVE_PARAMETERS_ARB = 34986 	# GL/glext.h:992
GL_MAX_PROGRAM_NATIVE_PARAMETERS_ARB = 34987 	# GL/glext.h:993
GL_PROGRAM_ATTRIBS_ARB = 34988 	# GL/glext.h:994
GL_MAX_PROGRAM_ATTRIBS_ARB = 34989 	# GL/glext.h:995
GL_PROGRAM_NATIVE_ATTRIBS_ARB = 34990 	# GL/glext.h:996
GL_MAX_PROGRAM_NATIVE_ATTRIBS_ARB = 34991 	# GL/glext.h:997
GL_PROGRAM_ADDRESS_REGISTERS_ARB = 34992 	# GL/glext.h:998
GL_MAX_PROGRAM_ADDRESS_REGISTERS_ARB = 34993 	# GL/glext.h:999
GL_PROGRAM_NATIVE_ADDRESS_REGISTERS_ARB = 34994 	# GL/glext.h:1000
GL_MAX_PROGRAM_NATIVE_ADDRESS_REGISTERS_ARB = 34995 	# GL/glext.h:1001
GL_MAX_PROGRAM_LOCAL_PARAMETERS_ARB = 34996 	# GL/glext.h:1002
GL_MAX_PROGRAM_ENV_PARAMETERS_ARB = 34997 	# GL/glext.h:1003
GL_PROGRAM_UNDER_NATIVE_LIMITS_ARB = 34998 	# GL/glext.h:1004
GL_TRANSPOSE_CURRENT_MATRIX_ARB = 34999 	# GL/glext.h:1005
GL_MATRIX0_ARB = 35008 	# GL/glext.h:1006
GL_MATRIX1_ARB = 35009 	# GL/glext.h:1007
GL_MATRIX2_ARB = 35010 	# GL/glext.h:1008
GL_MATRIX3_ARB = 35011 	# GL/glext.h:1009
GL_MATRIX4_ARB = 35012 	# GL/glext.h:1010
GL_MATRIX5_ARB = 35013 	# GL/glext.h:1011
GL_MATRIX6_ARB = 35014 	# GL/glext.h:1012
GL_MATRIX7_ARB = 35015 	# GL/glext.h:1013
GL_MATRIX8_ARB = 35016 	# GL/glext.h:1014
GL_MATRIX9_ARB = 35017 	# GL/glext.h:1015
GL_MATRIX10_ARB = 35018 	# GL/glext.h:1016
GL_MATRIX11_ARB = 35019 	# GL/glext.h:1017
GL_MATRIX12_ARB = 35020 	# GL/glext.h:1018
GL_MATRIX13_ARB = 35021 	# GL/glext.h:1019
GL_MATRIX14_ARB = 35022 	# GL/glext.h:1020
GL_MATRIX15_ARB = 35023 	# GL/glext.h:1021
GL_MATRIX16_ARB = 35024 	# GL/glext.h:1022
GL_MATRIX17_ARB = 35025 	# GL/glext.h:1023
GL_MATRIX18_ARB = 35026 	# GL/glext.h:1024
GL_MATRIX19_ARB = 35027 	# GL/glext.h:1025
GL_MATRIX20_ARB = 35028 	# GL/glext.h:1026
GL_MATRIX21_ARB = 35029 	# GL/glext.h:1027
GL_MATRIX22_ARB = 35030 	# GL/glext.h:1028
GL_MATRIX23_ARB = 35031 	# GL/glext.h:1029
GL_MATRIX24_ARB = 35032 	# GL/glext.h:1030
GL_MATRIX25_ARB = 35033 	# GL/glext.h:1031
GL_MATRIX26_ARB = 35034 	# GL/glext.h:1032
GL_MATRIX27_ARB = 35035 	# GL/glext.h:1033
GL_MATRIX28_ARB = 35036 	# GL/glext.h:1034
GL_MATRIX29_ARB = 35037 	# GL/glext.h:1035
GL_MATRIX30_ARB = 35038 	# GL/glext.h:1036
GL_MATRIX31_ARB = 35039 	# GL/glext.h:1037
# ARB_fragment_program (GL/glext.h:1040)
GL_FRAGMENT_PROGRAM_ARB = 34820 	# GL/glext.h:1041
GL_PROGRAM_ALU_INSTRUCTIONS_ARB = 34821 	# GL/glext.h:1042
GL_PROGRAM_TEX_INSTRUCTIONS_ARB = 34822 	# GL/glext.h:1043
GL_PROGRAM_TEX_INDIRECTIONS_ARB = 34823 	# GL/glext.h:1044
GL_PROGRAM_NATIVE_ALU_INSTRUCTIONS_ARB = 34824 	# GL/glext.h:1045
GL_PROGRAM_NATIVE_TEX_INSTRUCTIONS_ARB = 34825 	# GL/glext.h:1046
GL_PROGRAM_NATIVE_TEX_INDIRECTIONS_ARB = 34826 	# GL/glext.h:1047
GL_MAX_PROGRAM_ALU_INSTRUCTIONS_ARB = 34827 	# GL/glext.h:1048
GL_MAX_PROGRAM_TEX_INSTRUCTIONS_ARB = 34828 	# GL/glext.h:1049
GL_MAX_PROGRAM_TEX_INDIRECTIONS_ARB = 34829 	# GL/glext.h:1050
GL_MAX_PROGRAM_NATIVE_ALU_INSTRUCTIONS_ARB = 34830 	# GL/glext.h:1051
GL_MAX_PROGRAM_NATIVE_TEX_INSTRUCTIONS_ARB = 34831 	# GL/glext.h:1052
GL_MAX_PROGRAM_NATIVE_TEX_INDIRECTIONS_ARB = 34832 	# GL/glext.h:1053
GL_MAX_TEXTURE_COORDS_ARB = 34929 	# GL/glext.h:1054
GL_MAX_TEXTURE_IMAGE_UNITS_ARB = 34930 	# GL/glext.h:1055
# ARB_vertex_buffer_object (GL/glext.h:1058)
GL_BUFFER_SIZE_ARB = 34660 	# GL/glext.h:1059
GL_BUFFER_USAGE_ARB = 34661 	# GL/glext.h:1060
GL_ARRAY_BUFFER_ARB = 34962 	# GL/glext.h:1061
GL_ELEMENT_ARRAY_BUFFER_ARB = 34963 	# GL/glext.h:1062
GL_ARRAY_BUFFER_BINDING_ARB = 34964 	# GL/glext.h:1063
GL_ELEMENT_ARRAY_BUFFER_BINDING_ARB = 34965 	# GL/glext.h:1064
GL_VERTEX_ARRAY_BUFFER_BINDING_ARB = 34966 	# GL/glext.h:1065
GL_NORMAL_ARRAY_BUFFER_BINDING_ARB = 34967 	# GL/glext.h:1066
GL_COLOR_ARRAY_BUFFER_BINDING_ARB = 34968 	# GL/glext.h:1067
GL_INDEX_ARRAY_BUFFER_BINDING_ARB = 34969 	# GL/glext.h:1068
GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING_ARB = 34970 	# GL/glext.h:1069
GL_EDGE_FLAG_ARRAY_BUFFER_BINDING_ARB = 34971 	# GL/glext.h:1070
GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING_ARB = 34972 	# GL/glext.h:1071
GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING_ARB = 34973 	# GL/glext.h:1072
GL_WEIGHT_ARRAY_BUFFER_BINDING_ARB = 34974 	# GL/glext.h:1073
GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING_ARB = 34975 	# GL/glext.h:1074
GL_READ_ONLY_ARB = 35000 	# GL/glext.h:1075
GL_WRITE_ONLY_ARB = 35001 	# GL/glext.h:1076
GL_READ_WRITE_ARB = 35002 	# GL/glext.h:1077
GL_BUFFER_ACCESS_ARB = 35003 	# GL/glext.h:1078
GL_BUFFER_MAPPED_ARB = 35004 	# GL/glext.h:1079
GL_BUFFER_MAP_POINTER_ARB = 35005 	# GL/glext.h:1080
GL_STREAM_DRAW_ARB = 35040 	# GL/glext.h:1081
GL_STREAM_READ_ARB = 35041 	# GL/glext.h:1082
GL_STREAM_COPY_ARB = 35042 	# GL/glext.h:1083
GL_STATIC_DRAW_ARB = 35044 	# GL/glext.h:1084
GL_STATIC_READ_ARB = 35045 	# GL/glext.h:1085
GL_STATIC_COPY_ARB = 35046 	# GL/glext.h:1086
GL_DYNAMIC_DRAW_ARB = 35048 	# GL/glext.h:1087
GL_DYNAMIC_READ_ARB = 35049 	# GL/glext.h:1088
GL_DYNAMIC_COPY_ARB = 35050 	# GL/glext.h:1089
# ARB_occlusion_query (GL/glext.h:1092)
GL_QUERY_COUNTER_BITS_ARB = 34916 	# GL/glext.h:1093
GL_CURRENT_QUERY_ARB = 34917 	# GL/glext.h:1094
GL_QUERY_RESULT_ARB = 34918 	# GL/glext.h:1095
GL_QUERY_RESULT_AVAILABLE_ARB = 34919 	# GL/glext.h:1096
GL_SAMPLES_PASSED_ARB = 35092 	# GL/glext.h:1097
# ARB_shader_objects (GL/glext.h:1100)
GL_PROGRAM_OBJECT_ARB = 35648 	# GL/glext.h:1101
GL_SHADER_OBJECT_ARB = 35656 	# GL/glext.h:1102
GL_OBJECT_TYPE_ARB = 35662 	# GL/glext.h:1103
GL_OBJECT_SUBTYPE_ARB = 35663 	# GL/glext.h:1104
GL_FLOAT_VEC2_ARB = 35664 	# GL/glext.h:1105
GL_FLOAT_VEC3_ARB = 35665 	# GL/glext.h:1106
GL_FLOAT_VEC4_ARB = 35666 	# GL/glext.h:1107
GL_INT_VEC2_ARB = 35667 	# GL/glext.h:1108
GL_INT_VEC3_ARB = 35668 	# GL/glext.h:1109
GL_INT_VEC4_ARB = 35669 	# GL/glext.h:1110
GL_BOOL_ARB = 35670 	# GL/glext.h:1111
GL_BOOL_VEC2_ARB = 35671 	# GL/glext.h:1112
GL_BOOL_VEC3_ARB = 35672 	# GL/glext.h:1113
GL_BOOL_VEC4_ARB = 35673 	# GL/glext.h:1114
GL_FLOAT_MAT2_ARB = 35674 	# GL/glext.h:1115
GL_FLOAT_MAT3_ARB = 35675 	# GL/glext.h:1116
GL_FLOAT_MAT4_ARB = 35676 	# GL/glext.h:1117
GL_SAMPLER_1D_ARB = 35677 	# GL/glext.h:1118
GL_SAMPLER_2D_ARB = 35678 	# GL/glext.h:1119
GL_SAMPLER_3D_ARB = 35679 	# GL/glext.h:1120
GL_SAMPLER_CUBE_ARB = 35680 	# GL/glext.h:1121
GL_SAMPLER_1D_SHADOW_ARB = 35681 	# GL/glext.h:1122
GL_SAMPLER_2D_SHADOW_ARB = 35682 	# GL/glext.h:1123
GL_SAMPLER_2D_RECT_ARB = 35683 	# GL/glext.h:1124
GL_SAMPLER_2D_RECT_SHADOW_ARB = 35684 	# GL/glext.h:1125
GL_OBJECT_DELETE_STATUS_ARB = 35712 	# GL/glext.h:1126
GL_OBJECT_COMPILE_STATUS_ARB = 35713 	# GL/glext.h:1127
GL_OBJECT_LINK_STATUS_ARB = 35714 	# GL/glext.h:1128
GL_OBJECT_VALIDATE_STATUS_ARB = 35715 	# GL/glext.h:1129
GL_OBJECT_INFO_LOG_LENGTH_ARB = 35716 	# GL/glext.h:1130
GL_OBJECT_ATTACHED_OBJECTS_ARB = 35717 	# GL/glext.h:1131
GL_OBJECT_ACTIVE_UNIFORMS_ARB = 35718 	# GL/glext.h:1132
GL_OBJECT_ACTIVE_UNIFORM_MAX_LENGTH_ARB = 35719 	# GL/glext.h:1133
GL_OBJECT_SHADER_SOURCE_LENGTH_ARB = 35720 	# GL/glext.h:1134
# ARB_vertex_shader (GL/glext.h:1137)
GL_VERTEX_SHADER_ARB = 35633 	# GL/glext.h:1138
GL_MAX_VERTEX_UNIFORM_COMPONENTS_ARB = 35658 	# GL/glext.h:1139
GL_MAX_VARYING_FLOATS_ARB = 35659 	# GL/glext.h:1140
GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS_ARB = 35660 	# GL/glext.h:1141
GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS_ARB = 35661 	# GL/glext.h:1142
GL_OBJECT_ACTIVE_ATTRIBUTES_ARB = 35721 	# GL/glext.h:1143
GL_OBJECT_ACTIVE_ATTRIBUTE_MAX_LENGTH_ARB = 35722 	# GL/glext.h:1144
# ARB_fragment_shader (GL/glext.h:1147)
GL_FRAGMENT_SHADER_ARB = 35632 	# GL/glext.h:1148
GL_MAX_FRAGMENT_UNIFORM_COMPONENTS_ARB = 35657 	# GL/glext.h:1149
GL_FRAGMENT_SHADER_DERIVATIVE_HINT_ARB = 35723 	# GL/glext.h:1150
# ARB_shading_language_100 (GL/glext.h:1153)
GL_SHADING_LANGUAGE_VERSION_ARB = 35724 	# GL/glext.h:1154
# ARB_texture_non_power_of_two (GL/glext.h:1157)
# ARB_point_sprite (GL/glext.h:1160)
GL_POINT_SPRITE_ARB = 34913 	# GL/glext.h:1161
GL_COORD_REPLACE_ARB = 34914 	# GL/glext.h:1162
# ARB_fragment_program_shadow (GL/glext.h:1165)
# ARB_draw_buffers (GL/glext.h:1168)
GL_MAX_DRAW_BUFFERS_ARB = 34852 	# GL/glext.h:1169
GL_DRAW_BUFFER0_ARB = 34853 	# GL/glext.h:1170
GL_DRAW_BUFFER1_ARB = 34854 	# GL/glext.h:1171
GL_DRAW_BUFFER2_ARB = 34855 	# GL/glext.h:1172
GL_DRAW_BUFFER3_ARB = 34856 	# GL/glext.h:1173
GL_DRAW_BUFFER4_ARB = 34857 	# GL/glext.h:1174
GL_DRAW_BUFFER5_ARB = 34858 	# GL/glext.h:1175
GL_DRAW_BUFFER6_ARB = 34859 	# GL/glext.h:1176
GL_DRAW_BUFFER7_ARB = 34860 	# GL/glext.h:1177
GL_DRAW_BUFFER8_ARB = 34861 	# GL/glext.h:1178
GL_DRAW_BUFFER9_ARB = 34862 	# GL/glext.h:1179
GL_DRAW_BUFFER10_ARB = 34863 	# GL/glext.h:1180
GL_DRAW_BUFFER11_ARB = 34864 	# GL/glext.h:1181
GL_DRAW_BUFFER12_ARB = 34865 	# GL/glext.h:1182
GL_DRAW_BUFFER13_ARB = 34866 	# GL/glext.h:1183
GL_DRAW_BUFFER14_ARB = 34867 	# GL/glext.h:1184
GL_DRAW_BUFFER15_ARB = 34868 	# GL/glext.h:1185
# ARB_texture_rectangle (GL/glext.h:1188)
GL_TEXTURE_RECTANGLE_ARB = 34037 	# GL/glext.h:1189
GL_TEXTURE_BINDING_RECTANGLE_ARB = 34038 	# GL/glext.h:1190
GL_PROXY_TEXTURE_RECTANGLE_ARB = 34039 	# GL/glext.h:1191
GL_MAX_RECTANGLE_TEXTURE_SIZE_ARB = 34040 	# GL/glext.h:1192
# ARB_color_buffer_float (GL/glext.h:1195)
GL_RGBA_FLOAT_MODE_ARB = 34848 	# GL/glext.h:1196
GL_CLAMP_VERTEX_COLOR_ARB = 35098 	# GL/glext.h:1197
GL_CLAMP_FRAGMENT_COLOR_ARB = 35099 	# GL/glext.h:1198
GL_CLAMP_READ_COLOR_ARB = 35100 	# GL/glext.h:1199
GL_FIXED_ONLY_ARB = 35101 	# GL/glext.h:1200
# ARB_half_float_pixel (GL/glext.h:1203)
GL_HALF_FLOAT_ARB = 5131 	# GL/glext.h:1204
# ARB_texture_float (GL/glext.h:1207)
GL_TEXTURE_RED_TYPE_ARB = 35856 	# GL/glext.h:1208
GL_TEXTURE_GREEN_TYPE_ARB = 35857 	# GL/glext.h:1209
GL_TEXTURE_BLUE_TYPE_ARB = 35858 	# GL/glext.h:1210
GL_TEXTURE_ALPHA_TYPE_ARB = 35859 	# GL/glext.h:1211
GL_TEXTURE_LUMINANCE_TYPE_ARB = 35860 	# GL/glext.h:1212
GL_TEXTURE_INTENSITY_TYPE_ARB = 35861 	# GL/glext.h:1213
GL_TEXTURE_DEPTH_TYPE_ARB = 35862 	# GL/glext.h:1214
GL_UNSIGNED_NORMALIZED_ARB = 35863 	# GL/glext.h:1215
GL_RGBA32F_ARB = 34836 	# GL/glext.h:1216
GL_RGB32F_ARB = 34837 	# GL/glext.h:1217
GL_ALPHA32F_ARB = 34838 	# GL/glext.h:1218
GL_INTENSITY32F_ARB = 34839 	# GL/glext.h:1219
GL_LUMINANCE32F_ARB = 34840 	# GL/glext.h:1220
GL_LUMINANCE_ALPHA32F_ARB = 34841 	# GL/glext.h:1221
GL_RGBA16F_ARB = 34842 	# GL/glext.h:1222
GL_RGB16F_ARB = 34843 	# GL/glext.h:1223
GL_ALPHA16F_ARB = 34844 	# GL/glext.h:1224
GL_INTENSITY16F_ARB = 34845 	# GL/glext.h:1225
GL_LUMINANCE16F_ARB = 34846 	# GL/glext.h:1226
GL_LUMINANCE_ALPHA16F_ARB = 34847 	# GL/glext.h:1227
# ARB_pixel_buffer_object (GL/glext.h:1230)
GL_PIXEL_PACK_BUFFER_ARB = 35051 	# GL/glext.h:1231
GL_PIXEL_UNPACK_BUFFER_ARB = 35052 	# GL/glext.h:1232
GL_PIXEL_PACK_BUFFER_BINDING_ARB = 35053 	# GL/glext.h:1233
GL_PIXEL_UNPACK_BUFFER_BINDING_ARB = 35055 	# GL/glext.h:1234
# ARB_depth_buffer_float (GL/glext.h:1237)
GL_DEPTH_COMPONENT32F = 36012 	# GL/glext.h:1238
GL_DEPTH32F_STENCIL8 = 36013 	# GL/glext.h:1239
GL_FLOAT_32_UNSIGNED_INT_24_8_REV = 36269 	# GL/glext.h:1240
# ARB_draw_instanced (GL/glext.h:1243)
# ARB_framebuffer_object (GL/glext.h:1246)
GL_INVALID_FRAMEBUFFER_OPERATION = 1286 	# GL/glext.h:1247
GL_FRAMEBUFFER_ATTACHMENT_COLOR_ENCODING = 33296 	# GL/glext.h:1248
GL_FRAMEBUFFER_ATTACHMENT_COMPONENT_TYPE = 33297 	# GL/glext.h:1249
GL_FRAMEBUFFER_ATTACHMENT_RED_SIZE = 33298 	# GL/glext.h:1250
GL_FRAMEBUFFER_ATTACHMENT_GREEN_SIZE = 33299 	# GL/glext.h:1251
GL_FRAMEBUFFER_ATTACHMENT_BLUE_SIZE = 33300 	# GL/glext.h:1252
GL_FRAMEBUFFER_ATTACHMENT_ALPHA_SIZE = 33301 	# GL/glext.h:1253
GL_FRAMEBUFFER_ATTACHMENT_DEPTH_SIZE = 33302 	# GL/glext.h:1254
GL_FRAMEBUFFER_ATTACHMENT_STENCIL_SIZE = 33303 	# GL/glext.h:1255
GL_FRAMEBUFFER_DEFAULT = 33304 	# GL/glext.h:1256
GL_FRAMEBUFFER_UNDEFINED = 33305 	# GL/glext.h:1257
GL_DEPTH_STENCIL_ATTACHMENT = 33306 	# GL/glext.h:1258
GL_INDEX = 33314 	# GL/glext.h:1259
GL_MAX_RENDERBUFFER_SIZE = 34024 	# GL/glext.h:1260
GL_DEPTH_STENCIL = 34041 	# GL/glext.h:1261
GL_UNSIGNED_INT_24_8 = 34042 	# GL/glext.h:1262
GL_DEPTH24_STENCIL8 = 35056 	# GL/glext.h:1263
GL_TEXTURE_STENCIL_SIZE = 35057 	# GL/glext.h:1264
GL_FRAMEBUFFER_BINDING = 36006 	# GL/glext.h:1265
GL_DRAW_FRAMEBUFFER_BINDING = 36006 	# GL/glext.h:1266
GL_RENDERBUFFER_BINDING = 36007 	# GL/glext.h:1267
GL_READ_FRAMEBUFFER = 36008 	# GL/glext.h:1268
GL_DRAW_FRAMEBUFFER = 36009 	# GL/glext.h:1269
GL_READ_FRAMEBUFFER_BINDING = 36010 	# GL/glext.h:1270
GL_RENDERBUFFER_SAMPLES = 36011 	# GL/glext.h:1271
GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE = 36048 	# GL/glext.h:1272
GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME = 36049 	# GL/glext.h:1273
GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL = 36050 	# GL/glext.h:1274
GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE = 36051 	# GL/glext.h:1275
GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LAYER = 36052 	# GL/glext.h:1276
GL_FRAMEBUFFER_COMPLETE = 36053 	# GL/glext.h:1277
GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT = 36054 	# GL/glext.h:1278
GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT = 36055 	# GL/glext.h:1279
GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER = 36059 	# GL/glext.h:1280
GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER = 36060 	# GL/glext.h:1281
GL_FRAMEBUFFER_UNSUPPORTED = 36061 	# GL/glext.h:1282
GL_MAX_COLOR_ATTACHMENTS = 36063 	# GL/glext.h:1283
GL_COLOR_ATTACHMENT0 = 36064 	# GL/glext.h:1284
GL_COLOR_ATTACHMENT1 = 36065 	# GL/glext.h:1285
GL_COLOR_ATTACHMENT2 = 36066 	# GL/glext.h:1286
GL_COLOR_ATTACHMENT3 = 36067 	# GL/glext.h:1287
GL_COLOR_ATTACHMENT4 = 36068 	# GL/glext.h:1288
GL_COLOR_ATTACHMENT5 = 36069 	# GL/glext.h:1289
GL_COLOR_ATTACHMENT6 = 36070 	# GL/glext.h:1290
GL_COLOR_ATTACHMENT7 = 36071 	# GL/glext.h:1291
GL_COLOR_ATTACHMENT8 = 36072 	# GL/glext.h:1292
GL_COLOR_ATTACHMENT9 = 36073 	# GL/glext.h:1293
GL_COLOR_ATTACHMENT10 = 36074 	# GL/glext.h:1294
GL_COLOR_ATTACHMENT11 = 36075 	# GL/glext.h:1295
GL_COLOR_ATTACHMENT12 = 36076 	# GL/glext.h:1296
GL_COLOR_ATTACHMENT13 = 36077 	# GL/glext.h:1297
GL_COLOR_ATTACHMENT14 = 36078 	# GL/glext.h:1298
GL_COLOR_ATTACHMENT15 = 36079 	# GL/glext.h:1299
GL_DEPTH_ATTACHMENT = 36096 	# GL/glext.h:1300
GL_STENCIL_ATTACHMENT = 36128 	# GL/glext.h:1301
GL_FRAMEBUFFER = 36160 	# GL/glext.h:1302
GL_RENDERBUFFER = 36161 	# GL/glext.h:1303
GL_RENDERBUFFER_WIDTH = 36162 	# GL/glext.h:1304
GL_RENDERBUFFER_HEIGHT = 36163 	# GL/glext.h:1305
GL_RENDERBUFFER_INTERNAL_FORMAT = 36164 	# GL/glext.h:1306
GL_STENCIL_INDEX1 = 36166 	# GL/glext.h:1307
GL_STENCIL_INDEX4 = 36167 	# GL/glext.h:1308
GL_STENCIL_INDEX8 = 36168 	# GL/glext.h:1309
GL_STENCIL_INDEX16 = 36169 	# GL/glext.h:1310
GL_RENDERBUFFER_RED_SIZE = 36176 	# GL/glext.h:1311
GL_RENDERBUFFER_GREEN_SIZE = 36177 	# GL/glext.h:1312
GL_RENDERBUFFER_BLUE_SIZE = 36178 	# GL/glext.h:1313
GL_RENDERBUFFER_ALPHA_SIZE = 36179 	# GL/glext.h:1314
GL_RENDERBUFFER_DEPTH_SIZE = 36180 	# GL/glext.h:1315
GL_RENDERBUFFER_STENCIL_SIZE = 36181 	# GL/glext.h:1316
GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE = 36182 	# GL/glext.h:1317
GL_MAX_SAMPLES = 36183 	# GL/glext.h:1318
# ARB_framebuffer_sRGB (GL/glext.h:1321)
GL_FRAMEBUFFER_SRGB = 36281 	# GL/glext.h:1322
# ARB_geometry_shader4 (GL/glext.h:1325)
GL_LINES_ADJACENCY_ARB = 10 	# GL/glext.h:1326
GL_LINE_STRIP_ADJACENCY_ARB = 11 	# GL/glext.h:1327
GL_TRIANGLES_ADJACENCY_ARB = 12 	# GL/glext.h:1328
GL_TRIANGLE_STRIP_ADJACENCY_ARB = 13 	# GL/glext.h:1329
GL_PROGRAM_POINT_SIZE_ARB = 34370 	# GL/glext.h:1330
GL_MAX_GEOMETRY_TEXTURE_IMAGE_UNITS_ARB = 35881 	# GL/glext.h:1331
GL_FRAMEBUFFER_ATTACHMENT_LAYERED_ARB = 36263 	# GL/glext.h:1332
GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS_ARB = 36264 	# GL/glext.h:1333
GL_FRAMEBUFFER_INCOMPLETE_LAYER_COUNT_ARB = 36265 	# GL/glext.h:1334
GL_GEOMETRY_SHADER_ARB = 36313 	# GL/glext.h:1335
GL_GEOMETRY_VERTICES_OUT_ARB = 36314 	# GL/glext.h:1336
GL_GEOMETRY_INPUT_TYPE_ARB = 36315 	# GL/glext.h:1337
GL_GEOMETRY_OUTPUT_TYPE_ARB = 36316 	# GL/glext.h:1338
GL_MAX_GEOMETRY_VARYING_COMPONENTS_ARB = 36317 	# GL/glext.h:1339
GL_MAX_VERTEX_VARYING_COMPONENTS_ARB = 36318 	# GL/glext.h:1340
GL_MAX_GEOMETRY_UNIFORM_COMPONENTS_ARB = 36319 	# GL/glext.h:1341
GL_MAX_GEOMETRY_OUTPUT_VERTICES_ARB = 36320 	# GL/glext.h:1342
GL_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS_ARB = 36321 	# GL/glext.h:1343
# ARB_half_float_vertex (GL/glext.h:1348)
GL_HALF_FLOAT = 5131 	# GL/glext.h:1349
# ARB_instanced_arrays (GL/glext.h:1352)
# ARB_map_buffer_range (GL/glext.h:1355)
GL_MAP_READ_BIT = 1 	# GL/glext.h:1356
GL_MAP_WRITE_BIT = 2 	# GL/glext.h:1357
GL_MAP_INVALIDATE_RANGE_BIT = 4 	# GL/glext.h:1358
GL_MAP_INVALIDATE_BUFFER_BIT = 8 	# GL/glext.h:1359
GL_MAP_FLUSH_EXPLICIT_BIT = 16 	# GL/glext.h:1360
GL_MAP_UNSYNCHRONIZED_BIT = 32 	# GL/glext.h:1361
# ARB_texture_buffer_object (GL/glext.h:1364)
GL_TEXTURE_BUFFER_ARB = 35882 	# GL/glext.h:1365
GL_MAX_TEXTURE_BUFFER_SIZE_ARB = 35883 	# GL/glext.h:1366
GL_TEXTURE_BINDING_BUFFER_ARB = 35884 	# GL/glext.h:1367
GL_TEXTURE_BUFFER_DATA_STORE_BINDING_ARB = 35885 	# GL/glext.h:1368
GL_TEXTURE_BUFFER_FORMAT_ARB = 35886 	# GL/glext.h:1369
# ARB_texture_compression_rgtc (GL/glext.h:1372)
GL_COMPRESSED_RED_RGTC1 = 36283 	# GL/glext.h:1373
GL_COMPRESSED_SIGNED_RED_RGTC1 = 36284 	# GL/glext.h:1374
GL_COMPRESSED_RG_RGTC2 = 36285 	# GL/glext.h:1375
GL_COMPRESSED_SIGNED_RG_RGTC2 = 36286 	# GL/glext.h:1376
# ARB_texture_rg (GL/glext.h:1379)
GL_RG = 33319 	# GL/glext.h:1380
GL_RG_INTEGER = 33320 	# GL/glext.h:1381
GL_R8 = 33321 	# GL/glext.h:1382
GL_R16 = 33322 	# GL/glext.h:1383
GL_RG8 = 33323 	# GL/glext.h:1384
GL_RG16 = 33324 	# GL/glext.h:1385
GL_R16F = 33325 	# GL/glext.h:1386
GL_R32F = 33326 	# GL/glext.h:1387
GL_RG16F = 33327 	# GL/glext.h:1388
GL_RG32F = 33328 	# GL/glext.h:1389
GL_R8I = 33329 	# GL/glext.h:1390
GL_R8UI = 33330 	# GL/glext.h:1391
GL_R16I = 33331 	# GL/glext.h:1392
GL_R16UI = 33332 	# GL/glext.h:1393
GL_R32I = 33333 	# GL/glext.h:1394
GL_R32UI = 33334 	# GL/glext.h:1395
GL_RG8I = 33335 	# GL/glext.h:1396
GL_RG8UI = 33336 	# GL/glext.h:1397
GL_RG16I = 33337 	# GL/glext.h:1398
GL_RG16UI = 33338 	# GL/glext.h:1399
GL_RG32I = 33339 	# GL/glext.h:1400
GL_RG32UI = 33340 	# GL/glext.h:1401
# ARB_vertex_array_object (GL/glext.h:1404)
GL_VERTEX_ARRAY_BINDING = 34229 	# GL/glext.h:1405
# EXT_abgr (GL/glext.h:1408)
GL_ABGR_EXT = 32768 	# GL/glext.h:1409
# EXT_blend_color (GL/glext.h:1412)
GL_CONSTANT_COLOR_EXT = 32769 	# GL/glext.h:1413
GL_ONE_MINUS_CONSTANT_COLOR_EXT = 32770 	# GL/glext.h:1414
GL_CONSTANT_ALPHA_EXT = 32771 	# GL/glext.h:1415
GL_ONE_MINUS_CONSTANT_ALPHA_EXT = 32772 	# GL/glext.h:1416
GL_BLEND_COLOR_EXT = 32773 	# GL/glext.h:1417
# EXT_polygon_offset (GL/glext.h:1420)
GL_POLYGON_OFFSET_EXT = 32823 	# GL/glext.h:1421
GL_POLYGON_OFFSET_FACTOR_EXT = 32824 	# GL/glext.h:1422
GL_POLYGON_OFFSET_BIAS_EXT = 32825 	# GL/glext.h:1423
# EXT_texture (GL/glext.h:1426)
GL_ALPHA4_EXT = 32827 	# GL/glext.h:1427
GL_ALPHA8_EXT = 32828 	# GL/glext.h:1428
GL_ALPHA12_EXT = 32829 	# GL/glext.h:1429
GL_ALPHA16_EXT = 32830 	# GL/glext.h:1430
GL_LUMINANCE4_EXT = 32831 	# GL/glext.h:1431
GL_LUMINANCE8_EXT = 32832 	# GL/glext.h:1432
GL_LUMINANCE12_EXT = 32833 	# GL/glext.h:1433
GL_LUMINANCE16_EXT = 32834 	# GL/glext.h:1434
GL_LUMINANCE4_ALPHA4_EXT = 32835 	# GL/glext.h:1435
GL_LUMINANCE6_ALPHA2_EXT = 32836 	# GL/glext.h:1436
GL_LUMINANCE8_ALPHA8_EXT = 32837 	# GL/glext.h:1437
GL_LUMINANCE12_ALPHA4_EXT = 32838 	# GL/glext.h:1438
GL_LUMINANCE12_ALPHA12_EXT = 32839 	# GL/glext.h:1439
GL_LUMINANCE16_ALPHA16_EXT = 32840 	# GL/glext.h:1440
GL_INTENSITY_EXT = 32841 	# GL/glext.h:1441
GL_INTENSITY4_EXT = 32842 	# GL/glext.h:1442
GL_INTENSITY8_EXT = 32843 	# GL/glext.h:1443
GL_INTENSITY12_EXT = 32844 	# GL/glext.h:1444
GL_INTENSITY16_EXT = 32845 	# GL/glext.h:1445
GL_RGB2_EXT = 32846 	# GL/glext.h:1446
GL_RGB4_EXT = 32847 	# GL/glext.h:1447
GL_RGB5_EXT = 32848 	# GL/glext.h:1448
GL_RGB8_EXT = 32849 	# GL/glext.h:1449
GL_RGB10_EXT = 32850 	# GL/glext.h:1450
GL_RGB12_EXT = 32851 	# GL/glext.h:1451
GL_RGB16_EXT = 32852 	# GL/glext.h:1452
GL_RGBA2_EXT = 32853 	# GL/glext.h:1453
GL_RGBA4_EXT = 32854 	# GL/glext.h:1454
GL_RGB5_A1_EXT = 32855 	# GL/glext.h:1455
GL_RGBA8_EXT = 32856 	# GL/glext.h:1456
GL_RGB10_A2_EXT = 32857 	# GL/glext.h:1457
GL_RGBA12_EXT = 32858 	# GL/glext.h:1458
GL_RGBA16_EXT = 32859 	# GL/glext.h:1459
GL_TEXTURE_RED_SIZE_EXT = 32860 	# GL/glext.h:1460
GL_TEXTURE_GREEN_SIZE_EXT = 32861 	# GL/glext.h:1461
GL_TEXTURE_BLUE_SIZE_EXT = 32862 	# GL/glext.h:1462
GL_TEXTURE_ALPHA_SIZE_EXT = 32863 	# GL/glext.h:1463
GL_TEXTURE_LUMINANCE_SIZE_EXT = 32864 	# GL/glext.h:1464
GL_TEXTURE_INTENSITY_SIZE_EXT = 32865 	# GL/glext.h:1465
GL_REPLACE_EXT = 32866 	# GL/glext.h:1466
GL_PROXY_TEXTURE_1D_EXT = 32867 	# GL/glext.h:1467
GL_PROXY_TEXTURE_2D_EXT = 32868 	# GL/glext.h:1468
GL_TEXTURE_TOO_LARGE_EXT = 32869 	# GL/glext.h:1469
# EXT_texture3D (GL/glext.h:1472)
GL_PACK_SKIP_IMAGES_EXT = 32875 	# GL/glext.h:1473
GL_PACK_IMAGE_HEIGHT_EXT = 32876 	# GL/glext.h:1474
GL_UNPACK_SKIP_IMAGES_EXT = 32877 	# GL/glext.h:1475
GL_UNPACK_IMAGE_HEIGHT_EXT = 32878 	# GL/glext.h:1476
GL_TEXTURE_3D_EXT = 32879 	# GL/glext.h:1477
GL_PROXY_TEXTURE_3D_EXT = 32880 	# GL/glext.h:1478
GL_TEXTURE_DEPTH_EXT = 32881 	# GL/glext.h:1479
GL_TEXTURE_WRAP_R_EXT = 32882 	# GL/glext.h:1480
GL_MAX_3D_TEXTURE_SIZE_EXT = 32883 	# GL/glext.h:1481
# SGIS_texture_filter4 (GL/glext.h:1484)
GL_FILTER4_SGIS = 33094 	# GL/glext.h:1485
GL_TEXTURE_FILTER4_SIZE_SGIS = 33095 	# GL/glext.h:1486
# EXT_subtexture (GL/glext.h:1489)
# EXT_copy_texture (GL/glext.h:1492)
# EXT_histogram (GL/glext.h:1495)
GL_HISTOGRAM_EXT = 32804 	# GL/glext.h:1496
GL_PROXY_HISTOGRAM_EXT = 32805 	# GL/glext.h:1497
GL_HISTOGRAM_WIDTH_EXT = 32806 	# GL/glext.h:1498
GL_HISTOGRAM_FORMAT_EXT = 32807 	# GL/glext.h:1499
GL_HISTOGRAM_RED_SIZE_EXT = 32808 	# GL/glext.h:1500
GL_HISTOGRAM_GREEN_SIZE_EXT = 32809 	# GL/glext.h:1501
GL_HISTOGRAM_BLUE_SIZE_EXT = 32810 	# GL/glext.h:1502
GL_HISTOGRAM_ALPHA_SIZE_EXT = 32811 	# GL/glext.h:1503
GL_HISTOGRAM_LUMINANCE_SIZE_EXT = 32812 	# GL/glext.h:1504
GL_HISTOGRAM_SINK_EXT = 32813 	# GL/glext.h:1505
GL_MINMAX_EXT = 32814 	# GL/glext.h:1506
GL_MINMAX_FORMAT_EXT = 32815 	# GL/glext.h:1507
GL_MINMAX_SINK_EXT = 32816 	# GL/glext.h:1508
GL_TABLE_TOO_LARGE_EXT = 32817 	# GL/glext.h:1509
# EXT_convolution (GL/glext.h:1512)
GL_CONVOLUTION_1D_EXT = 32784 	# GL/glext.h:1513
GL_CONVOLUTION_2D_EXT = 32785 	# GL/glext.h:1514
GL_SEPARABLE_2D_EXT = 32786 	# GL/glext.h:1515
GL_CONVOLUTION_BORDER_MODE_EXT = 32787 	# GL/glext.h:1516
GL_CONVOLUTION_FILTER_SCALE_EXT = 32788 	# GL/glext.h:1517
GL_CONVOLUTION_FILTER_BIAS_EXT = 32789 	# GL/glext.h:1518
GL_REDUCE_EXT = 32790 	# GL/glext.h:1519
GL_CONVOLUTION_FORMAT_EXT = 32791 	# GL/glext.h:1520
GL_CONVOLUTION_WIDTH_EXT = 32792 	# GL/glext.h:1521
GL_CONVOLUTION_HEIGHT_EXT = 32793 	# GL/glext.h:1522
GL_MAX_CONVOLUTION_WIDTH_EXT = 32794 	# GL/glext.h:1523
GL_MAX_CONVOLUTION_HEIGHT_EXT = 32795 	# GL/glext.h:1524
GL_POST_CONVOLUTION_RED_SCALE_EXT = 32796 	# GL/glext.h:1525
GL_POST_CONVOLUTION_GREEN_SCALE_EXT = 32797 	# GL/glext.h:1526
GL_POST_CONVOLUTION_BLUE_SCALE_EXT = 32798 	# GL/glext.h:1527
GL_POST_CONVOLUTION_ALPHA_SCALE_EXT = 32799 	# GL/glext.h:1528
GL_POST_CONVOLUTION_RED_BIAS_EXT = 32800 	# GL/glext.h:1529
GL_POST_CONVOLUTION_GREEN_BIAS_EXT = 32801 	# GL/glext.h:1530
GL_POST_CONVOLUTION_BLUE_BIAS_EXT = 32802 	# GL/glext.h:1531
GL_POST_CONVOLUTION_ALPHA_BIAS_EXT = 32803 	# GL/glext.h:1532
# SGI_color_matrix (GL/glext.h:1535)
GL_COLOR_MATRIX_SGI = 32945 	# GL/glext.h:1536
GL_COLOR_MATRIX_STACK_DEPTH_SGI = 32946 	# GL/glext.h:1537
GL_MAX_COLOR_MATRIX_STACK_DEPTH_SGI = 32947 	# GL/glext.h:1538
GL_POST_COLOR_MATRIX_RED_SCALE_SGI = 32948 	# GL/glext.h:1539
GL_POST_COLOR_MATRIX_GREEN_SCALE_SGI = 32949 	# GL/glext.h:1540
GL_POST_COLOR_MATRIX_BLUE_SCALE_SGI = 32950 	# GL/glext.h:1541
GL_POST_COLOR_MATRIX_ALPHA_SCALE_SGI = 32951 	# GL/glext.h:1542
GL_POST_COLOR_MATRIX_RED_BIAS_SGI = 32952 	# GL/glext.h:1543
GL_POST_COLOR_MATRIX_GREEN_BIAS_SGI = 32953 	# GL/glext.h:1544
GL_POST_COLOR_MATRIX_BLUE_BIAS_SGI = 32954 	# GL/glext.h:1545
GL_POST_COLOR_MATRIX_ALPHA_BIAS_SGI = 32955 	# GL/glext.h:1546
# SGI_color_table (GL/glext.h:1549)
GL_COLOR_TABLE_SGI = 32976 	# GL/glext.h:1550
GL_POST_CONVOLUTION_COLOR_TABLE_SGI = 32977 	# GL/glext.h:1551
GL_POST_COLOR_MATRIX_COLOR_TABLE_SGI = 32978 	# GL/glext.h:1552
GL_PROXY_COLOR_TABLE_SGI = 32979 	# GL/glext.h:1553
GL_PROXY_POST_CONVOLUTION_COLOR_TABLE_SGI = 32980 	# GL/glext.h:1554
GL_PROXY_POST_COLOR_MATRIX_COLOR_TABLE_SGI = 32981 	# GL/glext.h:1555
GL_COLOR_TABLE_SCALE_SGI = 32982 	# GL/glext.h:1556
GL_COLOR_TABLE_BIAS_SGI = 32983 	# GL/glext.h:1557
GL_COLOR_TABLE_FORMAT_SGI = 32984 	# GL/glext.h:1558
GL_COLOR_TABLE_WIDTH_SGI = 32985 	# GL/glext.h:1559
GL_COLOR_TABLE_RED_SIZE_SGI = 32986 	# GL/glext.h:1560
GL_COLOR_TABLE_GREEN_SIZE_SGI = 32987 	# GL/glext.h:1561
GL_COLOR_TABLE_BLUE_SIZE_SGI = 32988 	# GL/glext.h:1562
GL_COLOR_TABLE_ALPHA_SIZE_SGI = 32989 	# GL/glext.h:1563
GL_COLOR_TABLE_LUMINANCE_SIZE_SGI = 32990 	# GL/glext.h:1564
GL_COLOR_TABLE_INTENSITY_SIZE_SGI = 32991 	# GL/glext.h:1565
# SGIS_pixel_texture (GL/glext.h:1568)
GL_PIXEL_TEXTURE_SGIS = 33619 	# GL/glext.h:1569
GL_PIXEL_FRAGMENT_RGB_SOURCE_SGIS = 33620 	# GL/glext.h:1570
GL_PIXEL_FRAGMENT_ALPHA_SOURCE_SGIS = 33621 	# GL/glext.h:1571
GL_PIXEL_GROUP_COLOR_SGIS = 33622 	# GL/glext.h:1572
# SGIX_pixel_texture (GL/glext.h:1575)
GL_PIXEL_TEX_GEN_SGIX = 33081 	# GL/glext.h:1576
GL_PIXEL_TEX_GEN_MODE_SGIX = 33579 	# GL/glext.h:1577
# SGIS_texture4D (GL/glext.h:1580)
GL_PACK_SKIP_VOLUMES_SGIS = 33072 	# GL/glext.h:1581
GL_PACK_IMAGE_DEPTH_SGIS = 33073 	# GL/glext.h:1582
GL_UNPACK_SKIP_VOLUMES_SGIS = 33074 	# GL/glext.h:1583
GL_UNPACK_IMAGE_DEPTH_SGIS = 33075 	# GL/glext.h:1584
GL_TEXTURE_4D_SGIS = 33076 	# GL/glext.h:1585
GL_PROXY_TEXTURE_4D_SGIS = 33077 	# GL/glext.h:1586
GL_TEXTURE_4DSIZE_SGIS = 33078 	# GL/glext.h:1587
GL_TEXTURE_WRAP_Q_SGIS = 33079 	# GL/glext.h:1588
GL_MAX_4D_TEXTURE_SIZE_SGIS = 33080 	# GL/glext.h:1589
GL_TEXTURE_4D_BINDING_SGIS = 33103 	# GL/glext.h:1590
# SGI_texture_color_table (GL/glext.h:1593)
GL_TEXTURE_COLOR_TABLE_SGI = 32956 	# GL/glext.h:1594
GL_PROXY_TEXTURE_COLOR_TABLE_SGI = 32957 	# GL/glext.h:1595
# EXT_cmyka (GL/glext.h:1598)
GL_CMYK_EXT = 32780 	# GL/glext.h:1599
GL_CMYKA_EXT = 32781 	# GL/glext.h:1600
GL_PACK_CMYK_HINT_EXT = 32782 	# GL/glext.h:1601
GL_UNPACK_CMYK_HINT_EXT = 32783 	# GL/glext.h:1602
# EXT_texture_object (GL/glext.h:1605)
GL_TEXTURE_PRIORITY_EXT = 32870 	# GL/glext.h:1606
GL_TEXTURE_RESIDENT_EXT = 32871 	# GL/glext.h:1607
GL_TEXTURE_1D_BINDING_EXT = 32872 	# GL/glext.h:1608
GL_TEXTURE_2D_BINDING_EXT = 32873 	# GL/glext.h:1609
GL_TEXTURE_3D_BINDING_EXT = 32874 	# GL/glext.h:1610
# SGIS_detail_texture (GL/glext.h:1613)
GL_DETAIL_TEXTURE_2D_SGIS = 32917 	# GL/glext.h:1614
GL_DETAIL_TEXTURE_2D_BINDING_SGIS = 32918 	# GL/glext.h:1615
GL_LINEAR_DETAIL_SGIS = 32919 	# GL/glext.h:1616
GL_LINEAR_DETAIL_ALPHA_SGIS = 32920 	# GL/glext.h:1617
GL_LINEAR_DETAIL_COLOR_SGIS = 32921 	# GL/glext.h:1618
GL_DETAIL_TEXTURE_LEVEL_SGIS = 32922 	# GL/glext.h:1619
GL_DETAIL_TEXTURE_MODE_SGIS = 32923 	# GL/glext.h:1620
GL_DETAIL_TEXTURE_FUNC_POINTS_SGIS = 32924 	# GL/glext.h:1621
# SGIS_sharpen_texture (GL/glext.h:1624)
GL_LINEAR_SHARPEN_SGIS = 32941 	# GL/glext.h:1625
GL_LINEAR_SHARPEN_ALPHA_SGIS = 32942 	# GL/glext.h:1626
GL_LINEAR_SHARPEN_COLOR_SGIS = 32943 	# GL/glext.h:1627
GL_SHARPEN_TEXTURE_FUNC_POINTS_SGIS = 32944 	# GL/glext.h:1628
# EXT_packed_pixels (GL/glext.h:1631)
GL_UNSIGNED_BYTE_3_3_2_EXT = 32818 	# GL/glext.h:1632
GL_UNSIGNED_SHORT_4_4_4_4_EXT = 32819 	# GL/glext.h:1633
GL_UNSIGNED_SHORT_5_5_5_1_EXT = 32820 	# GL/glext.h:1634
GL_UNSIGNED_INT_8_8_8_8_EXT = 32821 	# GL/glext.h:1635
GL_UNSIGNED_INT_10_10_10_2_EXT = 32822 	# GL/glext.h:1636
# SGIS_texture_lod (GL/glext.h:1639)
GL_TEXTURE_MIN_LOD_SGIS = 33082 	# GL/glext.h:1640
GL_TEXTURE_MAX_LOD_SGIS = 33083 	# GL/glext.h:1641
GL_TEXTURE_BASE_LEVEL_SGIS = 33084 	# GL/glext.h:1642
GL_TEXTURE_MAX_LEVEL_SGIS = 33085 	# GL/glext.h:1643
# SGIS_multisample (GL/glext.h:1646)
GL_MULTISAMPLE_SGIS = 32925 	# GL/glext.h:1647
GL_SAMPLE_ALPHA_TO_MASK_SGIS = 32926 	# GL/glext.h:1648
GL_SAMPLE_ALPHA_TO_ONE_SGIS = 32927 	# GL/glext.h:1649
GL_SAMPLE_MASK_SGIS = 32928 	# GL/glext.h:1650
GL_1PASS_SGIS = 32929 	# GL/glext.h:1651
GL_2PASS_0_SGIS = 32930 	# GL/glext.h:1652
GL_2PASS_1_SGIS = 32931 	# GL/glext.h:1653
GL_4PASS_0_SGIS = 32932 	# GL/glext.h:1654
GL_4PASS_1_SGIS = 32933 	# GL/glext.h:1655
GL_4PASS_2_SGIS = 32934 	# GL/glext.h:1656
GL_4PASS_3_SGIS = 32935 	# GL/glext.h:1657
GL_SAMPLE_BUFFERS_SGIS = 32936 	# GL/glext.h:1658
GL_SAMPLES_SGIS = 32937 	# GL/glext.h:1659
GL_SAMPLE_MASK_VALUE_SGIS = 32938 	# GL/glext.h:1660
GL_SAMPLE_MASK_INVERT_SGIS = 32939 	# GL/glext.h:1661
GL_SAMPLE_PATTERN_SGIS = 32940 	# GL/glext.h:1662
# EXT_rescale_normal (GL/glext.h:1665)
GL_RESCALE_NORMAL_EXT = 32826 	# GL/glext.h:1666
# EXT_vertex_array (GL/glext.h:1669)
GL_VERTEX_ARRAY_EXT = 32884 	# GL/glext.h:1670
GL_NORMAL_ARRAY_EXT = 32885 	# GL/glext.h:1671
GL_COLOR_ARRAY_EXT = 32886 	# GL/glext.h:1672
GL_INDEX_ARRAY_EXT = 32887 	# GL/glext.h:1673
GL_TEXTURE_COORD_ARRAY_EXT = 32888 	# GL/glext.h:1674
GL_EDGE_FLAG_ARRAY_EXT = 32889 	# GL/glext.h:1675
GL_VERTEX_ARRAY_SIZE_EXT = 32890 	# GL/glext.h:1676
GL_VERTEX_ARRAY_TYPE_EXT = 32891 	# GL/glext.h:1677
GL_VERTEX_ARRAY_STRIDE_EXT = 32892 	# GL/glext.h:1678
GL_VERTEX_ARRAY_COUNT_EXT = 32893 	# GL/glext.h:1679
GL_NORMAL_ARRAY_TYPE_EXT = 32894 	# GL/glext.h:1680
GL_NORMAL_ARRAY_STRIDE_EXT = 32895 	# GL/glext.h:1681
GL_NORMAL_ARRAY_COUNT_EXT = 32896 	# GL/glext.h:1682
GL_COLOR_ARRAY_SIZE_EXT = 32897 	# GL/glext.h:1683
GL_COLOR_ARRAY_TYPE_EXT = 32898 	# GL/glext.h:1684
GL_COLOR_ARRAY_STRIDE_EXT = 32899 	# GL/glext.h:1685
GL_COLOR_ARRAY_COUNT_EXT = 32900 	# GL/glext.h:1686
GL_INDEX_ARRAY_TYPE_EXT = 32901 	# GL/glext.h:1687
GL_INDEX_ARRAY_STRIDE_EXT = 32902 	# GL/glext.h:1688
GL_INDEX_ARRAY_COUNT_EXT = 32903 	# GL/glext.h:1689
GL_TEXTURE_COORD_ARRAY_SIZE_EXT = 32904 	# GL/glext.h:1690
GL_TEXTURE_COORD_ARRAY_TYPE_EXT = 32905 	# GL/glext.h:1691
GL_TEXTURE_COORD_ARRAY_STRIDE_EXT = 32906 	# GL/glext.h:1692
GL_TEXTURE_COORD_ARRAY_COUNT_EXT = 32907 	# GL/glext.h:1693
GL_EDGE_FLAG_ARRAY_STRIDE_EXT = 32908 	# GL/glext.h:1694
GL_EDGE_FLAG_ARRAY_COUNT_EXT = 32909 	# GL/glext.h:1695
GL_VERTEX_ARRAY_POINTER_EXT = 32910 	# GL/glext.h:1696
GL_NORMAL_ARRAY_POINTER_EXT = 32911 	# GL/glext.h:1697
GL_COLOR_ARRAY_POINTER_EXT = 32912 	# GL/glext.h:1698
GL_INDEX_ARRAY_POINTER_EXT = 32913 	# GL/glext.h:1699
GL_TEXTURE_COORD_ARRAY_POINTER_EXT = 32914 	# GL/glext.h:1700
GL_EDGE_FLAG_ARRAY_POINTER_EXT = 32915 	# GL/glext.h:1701
# EXT_misc_attribute (GL/glext.h:1704)
# SGIS_generate_mipmap (GL/glext.h:1707)
GL_GENERATE_MIPMAP_SGIS = 33169 	# GL/glext.h:1708
GL_GENERATE_MIPMAP_HINT_SGIS = 33170 	# GL/glext.h:1709
# SGIX_clipmap (GL/glext.h:1712)
GL_LINEAR_CLIPMAP_LINEAR_SGIX = 33136 	# GL/glext.h:1713
GL_TEXTURE_CLIPMAP_CENTER_SGIX = 33137 	# GL/glext.h:1714
GL_TEXTURE_CLIPMAP_FRAME_SGIX = 33138 	# GL/glext.h:1715
GL_TEXTURE_CLIPMAP_OFFSET_SGIX = 33139 	# GL/glext.h:1716
GL_TEXTURE_CLIPMAP_VIRTUAL_DEPTH_SGIX = 33140 	# GL/glext.h:1717
GL_TEXTURE_CLIPMAP_LOD_OFFSET_SGIX = 33141 	# GL/glext.h:1718
GL_TEXTURE_CLIPMAP_DEPTH_SGIX = 33142 	# GL/glext.h:1719
GL_MAX_CLIPMAP_DEPTH_SGIX = 33143 	# GL/glext.h:1720
GL_MAX_CLIPMAP_VIRTUAL_DEPTH_SGIX = 33144 	# GL/glext.h:1721
GL_NEAREST_CLIPMAP_NEAREST_SGIX = 33869 	# GL/glext.h:1722
GL_NEAREST_CLIPMAP_LINEAR_SGIX = 33870 	# GL/glext.h:1723
GL_LINEAR_CLIPMAP_NEAREST_SGIX = 33871 	# GL/glext.h:1724
# SGIX_shadow (GL/glext.h:1727)
GL_TEXTURE_COMPARE_SGIX = 33178 	# GL/glext.h:1728
GL_TEXTURE_COMPARE_OPERATOR_SGIX = 33179 	# GL/glext.h:1729
GL_TEXTURE_LEQUAL_R_SGIX = 33180 	# GL/glext.h:1730
GL_TEXTURE_GEQUAL_R_SGIX = 33181 	# GL/glext.h:1731
# SGIS_texture_edge_clamp (GL/glext.h:1734)
GL_CLAMP_TO_EDGE_SGIS = 33071 	# GL/glext.h:1735
# SGIS_texture_border_clamp (GL/glext.h:1738)
GL_CLAMP_TO_BORDER_SGIS = 33069 	# GL/glext.h:1739
# EXT_blend_minmax (GL/glext.h:1742)
GL_FUNC_ADD_EXT = 32774 	# GL/glext.h:1743
GL_MIN_EXT = 32775 	# GL/glext.h:1744
GL_MAX_EXT = 32776 	# GL/glext.h:1745
GL_BLEND_EQUATION_EXT = 32777 	# GL/glext.h:1746
# EXT_blend_subtract (GL/glext.h:1749)
GL_FUNC_SUBTRACT_EXT = 32778 	# GL/glext.h:1750
GL_FUNC_REVERSE_SUBTRACT_EXT = 32779 	# GL/glext.h:1751
# EXT_blend_logic_op (GL/glext.h:1754)
# SGIX_interlace (GL/glext.h:1757)
GL_INTERLACE_SGIX = 32916 	# GL/glext.h:1758
# SGIX_pixel_tiles (GL/glext.h:1761)
GL_PIXEL_TILE_BEST_ALIGNMENT_SGIX = 33086 	# GL/glext.h:1762
GL_PIXEL_TILE_CACHE_INCREMENT_SGIX = 33087 	# GL/glext.h:1763
GL_PIXEL_TILE_WIDTH_SGIX = 33088 	# GL/glext.h:1764
GL_PIXEL_TILE_HEIGHT_SGIX = 33089 	# GL/glext.h:1765
GL_PIXEL_TILE_GRID_WIDTH_SGIX = 33090 	# GL/glext.h:1766
GL_PIXEL_TILE_GRID_HEIGHT_SGIX = 33091 	# GL/glext.h:1767
GL_PIXEL_TILE_GRID_DEPTH_SGIX = 33092 	# GL/glext.h:1768
GL_PIXEL_TILE_CACHE_SIZE_SGIX = 33093 	# GL/glext.h:1769
# SGIS_texture_select (GL/glext.h:1772)
GL_DUAL_ALPHA4_SGIS = 33040 	# GL/glext.h:1773
GL_DUAL_ALPHA8_SGIS = 33041 	# GL/glext.h:1774
GL_DUAL_ALPHA12_SGIS = 33042 	# GL/glext.h:1775
GL_DUAL_ALPHA16_SGIS = 33043 	# GL/glext.h:1776
GL_DUAL_LUMINANCE4_SGIS = 33044 	# GL/glext.h:1777
GL_DUAL_LUMINANCE8_SGIS = 33045 	# GL/glext.h:1778
GL_DUAL_LUMINANCE12_SGIS = 33046 	# GL/glext.h:1779
GL_DUAL_LUMINANCE16_SGIS = 33047 	# GL/glext.h:1780
GL_DUAL_INTENSITY4_SGIS = 33048 	# GL/glext.h:1781
GL_DUAL_INTENSITY8_SGIS = 33049 	# GL/glext.h:1782
GL_DUAL_INTENSITY12_SGIS = 33050 	# GL/glext.h:1783
GL_DUAL_INTENSITY16_SGIS = 33051 	# GL/glext.h:1784
GL_DUAL_LUMINANCE_ALPHA4_SGIS = 33052 	# GL/glext.h:1785
GL_DUAL_LUMINANCE_ALPHA8_SGIS = 33053 	# GL/glext.h:1786
GL_QUAD_ALPHA4_SGIS = 33054 	# GL/glext.h:1787
GL_QUAD_ALPHA8_SGIS = 33055 	# GL/glext.h:1788
GL_QUAD_LUMINANCE4_SGIS = 33056 	# GL/glext.h:1789
GL_QUAD_LUMINANCE8_SGIS = 33057 	# GL/glext.h:1790
GL_QUAD_INTENSITY4_SGIS = 33058 	# GL/glext.h:1791
GL_QUAD_INTENSITY8_SGIS = 33059 	# GL/glext.h:1792
GL_DUAL_TEXTURE_SELECT_SGIS = 33060 	# GL/glext.h:1793
GL_QUAD_TEXTURE_SELECT_SGIS = 33061 	# GL/glext.h:1794
# SGIX_sprite (GL/glext.h:1797)
GL_SPRITE_SGIX = 33096 	# GL/glext.h:1798
GL_SPRITE_MODE_SGIX = 33097 	# GL/glext.h:1799
GL_SPRITE_AXIS_SGIX = 33098 	# GL/glext.h:1800
GL_SPRITE_TRANSLATION_SGIX = 33099 	# GL/glext.h:1801
GL_SPRITE_AXIAL_SGIX = 33100 	# GL/glext.h:1802
GL_SPRITE_OBJECT_ALIGNED_SGIX = 33101 	# GL/glext.h:1803
GL_SPRITE_EYE_ALIGNED_SGIX = 33102 	# GL/glext.h:1804
# SGIX_texture_multi_buffer (GL/glext.h:1807)
GL_TEXTURE_MULTI_BUFFER_HINT_SGIX = 33070 	# GL/glext.h:1808
# EXT_point_parameters (GL/glext.h:1811)
GL_POINT_SIZE_MIN_EXT = 33062 	# GL/glext.h:1812
GL_POINT_SIZE_MAX_EXT = 33063 	# GL/glext.h:1813
GL_POINT_FADE_THRESHOLD_SIZE_EXT = 33064 	# GL/glext.h:1814
GL_DISTANCE_ATTENUATION_EXT = 33065 	# GL/glext.h:1815
# SGIS_point_parameters (GL/glext.h:1818)
GL_POINT_SIZE_MIN_SGIS = 33062 	# GL/glext.h:1819
GL_POINT_SIZE_MAX_SGIS = 33063 	# GL/glext.h:1820
GL_POINT_FADE_THRESHOLD_SIZE_SGIS = 33064 	# GL/glext.h:1821
GL_DISTANCE_ATTENUATION_SGIS = 33065 	# GL/glext.h:1822
# SGIX_instruments (GL/glext.h:1825)
GL_INSTRUMENT_BUFFER_POINTER_SGIX = 33152 	# GL/glext.h:1826
GL_INSTRUMENT_MEASUREMENTS_SGIX = 33153 	# GL/glext.h:1827
# SGIX_texture_scale_bias (GL/glext.h:1830)
GL_POST_TEXTURE_FILTER_BIAS_SGIX = 33145 	# GL/glext.h:1831
GL_POST_TEXTURE_FILTER_SCALE_SGIX = 33146 	# GL/glext.h:1832
GL_POST_TEXTURE_FILTER_BIAS_RANGE_SGIX = 33147 	# GL/glext.h:1833
GL_POST_TEXTURE_FILTER_SCALE_RANGE_SGIX = 33148 	# GL/glext.h:1834
# SGIX_framezoom (GL/glext.h:1837)
GL_FRAMEZOOM_SGIX = 33163 	# GL/glext.h:1838
GL_FRAMEZOOM_FACTOR_SGIX = 33164 	# GL/glext.h:1839
GL_MAX_FRAMEZOOM_FACTOR_SGIX = 33165 	# GL/glext.h:1840
# SGIX_tag_sample_buffer (GL/glext.h:1843)
# FfdMaskSGIX (GL/glext.h:1846)
GL_TEXTURE_DEFORMATION_BIT_SGIX = 1 	# GL/glext.h:1847
GL_GEOMETRY_DEFORMATION_BIT_SGIX = 2 	# GL/glext.h:1848
# SGIX_polynomial_ffd (GL/glext.h:1851)
GL_GEOMETRY_DEFORMATION_SGIX = 33172 	# GL/glext.h:1852
GL_TEXTURE_DEFORMATION_SGIX = 33173 	# GL/glext.h:1853
GL_DEFORMATIONS_MASK_SGIX = 33174 	# GL/glext.h:1854
GL_MAX_DEFORMATION_ORDER_SGIX = 33175 	# GL/glext.h:1855
# SGIX_reference_plane (GL/glext.h:1858)
GL_REFERENCE_PLANE_SGIX = 33149 	# GL/glext.h:1859
GL_REFERENCE_PLANE_EQUATION_SGIX = 33150 	# GL/glext.h:1860
# SGIX_flush_raster (GL/glext.h:1863)
# SGIX_depth_texture (GL/glext.h:1866)
GL_DEPTH_COMPONENT16_SGIX = 33189 	# GL/glext.h:1867
GL_DEPTH_COMPONENT24_SGIX = 33190 	# GL/glext.h:1868
GL_DEPTH_COMPONENT32_SGIX = 33191 	# GL/glext.h:1869
# SGIS_fog_function (GL/glext.h:1872)
GL_FOG_FUNC_SGIS = 33066 	# GL/glext.h:1873
GL_FOG_FUNC_POINTS_SGIS = 33067 	# GL/glext.h:1874
GL_MAX_FOG_FUNC_POINTS_SGIS = 33068 	# GL/glext.h:1875
# SGIX_fog_offset (GL/glext.h:1878)
GL_FOG_OFFSET_SGIX = 33176 	# GL/glext.h:1879
GL_FOG_OFFSET_VALUE_SGIX = 33177 	# GL/glext.h:1880
# HP_image_transform (GL/glext.h:1883)
GL_IMAGE_SCALE_X_HP = 33109 	# GL/glext.h:1884
GL_IMAGE_SCALE_Y_HP = 33110 	# GL/glext.h:1885
GL_IMAGE_TRANSLATE_X_HP = 33111 	# GL/glext.h:1886
GL_IMAGE_TRANSLATE_Y_HP = 33112 	# GL/glext.h:1887
GL_IMAGE_ROTATE_ANGLE_HP = 33113 	# GL/glext.h:1888
GL_IMAGE_ROTATE_ORIGIN_X_HP = 33114 	# GL/glext.h:1889
GL_IMAGE_ROTATE_ORIGIN_Y_HP = 33115 	# GL/glext.h:1890
GL_IMAGE_MAG_FILTER_HP = 33116 	# GL/glext.h:1891
GL_IMAGE_MIN_FILTER_HP = 33117 	# GL/glext.h:1892
GL_IMAGE_CUBIC_WEIGHT_HP = 33118 	# GL/glext.h:1893
GL_CUBIC_HP = 33119 	# GL/glext.h:1894
GL_AVERAGE_HP = 33120 	# GL/glext.h:1895
GL_IMAGE_TRANSFORM_2D_HP = 33121 	# GL/glext.h:1896
GL_POST_IMAGE_TRANSFORM_COLOR_TABLE_HP = 33122 	# GL/glext.h:1897
GL_PROXY_POST_IMAGE_TRANSFORM_COLOR_TABLE_HP = 33123 	# GL/glext.h:1898
# HP_convolution_border_modes (GL/glext.h:1901)
GL_IGNORE_BORDER_HP = 33104 	# GL/glext.h:1902
GL_CONSTANT_BORDER_HP = 33105 	# GL/glext.h:1903
GL_REPLICATE_BORDER_HP = 33107 	# GL/glext.h:1904
GL_CONVOLUTION_BORDER_COLOR_HP = 33108 	# GL/glext.h:1905
# INGR_palette_buffer (GL/glext.h:1908)
# SGIX_texture_add_env (GL/glext.h:1911)
GL_TEXTURE_ENV_BIAS_SGIX = 32958 	# GL/glext.h:1912
# EXT_color_subtable (GL/glext.h:1915)
# PGI_vertex_hints (GL/glext.h:1918)
GL_VERTEX_DATA_HINT_PGI = 107050 	# GL/glext.h:1919
GL_VERTEX_CONSISTENT_HINT_PGI = 107051 	# GL/glext.h:1920
GL_MATERIAL_SIDE_HINT_PGI = 107052 	# GL/glext.h:1921
GL_MAX_VERTEX_HINT_PGI = 107053 	# GL/glext.h:1922
GL_COLOR3_BIT_PGI = 65536 	# GL/glext.h:1923
GL_COLOR4_BIT_PGI = 131072 	# GL/glext.h:1924
GL_EDGEFLAG_BIT_PGI = 262144 	# GL/glext.h:1925
GL_INDEX_BIT_PGI = 524288 	# GL/glext.h:1926
GL_MAT_AMBIENT_BIT_PGI = 1048576 	# GL/glext.h:1927
GL_MAT_AMBIENT_AND_DIFFUSE_BIT_PGI = 2097152 	# GL/glext.h:1928
GL_MAT_DIFFUSE_BIT_PGI = 4194304 	# GL/glext.h:1929
GL_MAT_EMISSION_BIT_PGI = 8388608 	# GL/glext.h:1930
GL_MAT_COLOR_INDEXES_BIT_PGI = 16777216 	# GL/glext.h:1931
GL_MAT_SHININESS_BIT_PGI = 33554432 	# GL/glext.h:1932
GL_MAT_SPECULAR_BIT_PGI = 67108864 	# GL/glext.h:1933
GL_NORMAL_BIT_PGI = 134217728 	# GL/glext.h:1934
GL_TEXCOORD1_BIT_PGI = 268435456 	# GL/glext.h:1935
GL_TEXCOORD2_BIT_PGI = 536870912 	# GL/glext.h:1936
GL_TEXCOORD3_BIT_PGI = 1073741824 	# GL/glext.h:1937
GL_TEXCOORD4_BIT_PGI = 2147483648 	# GL/glext.h:1938
GL_VERTEX23_BIT_PGI = 4 	# GL/glext.h:1939
GL_VERTEX4_BIT_PGI = 8 	# GL/glext.h:1940
# PGI_misc_hints (GL/glext.h:1943)
GL_PREFER_DOUBLEBUFFER_HINT_PGI = 107000 	# GL/glext.h:1944
GL_CONSERVE_MEMORY_HINT_PGI = 107005 	# GL/glext.h:1945
GL_RECLAIM_MEMORY_HINT_PGI = 107006 	# GL/glext.h:1946
GL_NATIVE_GRAPHICS_HANDLE_PGI = 107010 	# GL/glext.h:1947
GL_NATIVE_GRAPHICS_BEGIN_HINT_PGI = 107011 	# GL/glext.h:1948
GL_NATIVE_GRAPHICS_END_HINT_PGI = 107012 	# GL/glext.h:1949
GL_ALWAYS_FAST_HINT_PGI = 107020 	# GL/glext.h:1950
GL_ALWAYS_SOFT_HINT_PGI = 107021 	# GL/glext.h:1951
GL_ALLOW_DRAW_OBJ_HINT_PGI = 107022 	# GL/glext.h:1952
GL_ALLOW_DRAW_WIN_HINT_PGI = 107023 	# GL/glext.h:1953
GL_ALLOW_DRAW_FRG_HINT_PGI = 107024 	# GL/glext.h:1954
GL_ALLOW_DRAW_MEM_HINT_PGI = 107025 	# GL/glext.h:1955
GL_STRICT_DEPTHFUNC_HINT_PGI = 107030 	# GL/glext.h:1956
GL_STRICT_LIGHTING_HINT_PGI = 107031 	# GL/glext.h:1957
GL_STRICT_SCISSOR_HINT_PGI = 107032 	# GL/glext.h:1958
GL_FULL_STIPPLE_HINT_PGI = 107033 	# GL/glext.h:1959
GL_CLIP_NEAR_HINT_PGI = 107040 	# GL/glext.h:1960
GL_CLIP_FAR_HINT_PGI = 107041 	# GL/glext.h:1961
GL_WIDE_LINE_HINT_PGI = 107042 	# GL/glext.h:1962
GL_BACK_NORMALS_HINT_PGI = 107043 	# GL/glext.h:1963
# EXT_paletted_texture (GL/glext.h:1966)
GL_COLOR_INDEX1_EXT = 32994 	# GL/glext.h:1967
GL_COLOR_INDEX2_EXT = 32995 	# GL/glext.h:1968
GL_COLOR_INDEX4_EXT = 32996 	# GL/glext.h:1969
GL_COLOR_INDEX8_EXT = 32997 	# GL/glext.h:1970
GL_COLOR_INDEX12_EXT = 32998 	# GL/glext.h:1971
GL_COLOR_INDEX16_EXT = 32999 	# GL/glext.h:1972
GL_TEXTURE_INDEX_SIZE_EXT = 33005 	# GL/glext.h:1973
# EXT_clip_volume_hint (GL/glext.h:1976)
GL_CLIP_VOLUME_CLIPPING_HINT_EXT = 33008 	# GL/glext.h:1977
# SGIX_list_priority (GL/glext.h:1980)
GL_LIST_PRIORITY_SGIX = 33154 	# GL/glext.h:1981
# SGIX_ir_instrument1 (GL/glext.h:1984)
GL_IR_INSTRUMENT1_SGIX = 33151 	# GL/glext.h:1985
# SGIX_calligraphic_fragment (GL/glext.h:1988)
GL_CALLIGRAPHIC_FRAGMENT_SGIX = 33155 	# GL/glext.h:1989
# SGIX_texture_lod_bias (GL/glext.h:1992)
GL_TEXTURE_LOD_BIAS_S_SGIX = 33166 	# GL/glext.h:1993
GL_TEXTURE_LOD_BIAS_T_SGIX = 33167 	# GL/glext.h:1994
GL_TEXTURE_LOD_BIAS_R_SGIX = 33168 	# GL/glext.h:1995
# SGIX_shadow_ambient (GL/glext.h:1998)
GL_SHADOW_AMBIENT_SGIX = 32959 	# GL/glext.h:1999
# EXT_index_texture (GL/glext.h:2002)
# EXT_index_material (GL/glext.h:2005)
GL_INDEX_MATERIAL_EXT = 33208 	# GL/glext.h:2006
GL_INDEX_MATERIAL_PARAMETER_EXT = 33209 	# GL/glext.h:2007
GL_INDEX_MATERIAL_FACE_EXT = 33210 	# GL/glext.h:2008
# EXT_index_func (GL/glext.h:2011)
GL_INDEX_TEST_EXT = 33205 	# GL/glext.h:2012
GL_INDEX_TEST_FUNC_EXT = 33206 	# GL/glext.h:2013
GL_INDEX_TEST_REF_EXT = 33207 	# GL/glext.h:2014
# EXT_index_array_formats (GL/glext.h:2017)
GL_IUI_V2F_EXT = 33197 	# GL/glext.h:2018
GL_IUI_V3F_EXT = 33198 	# GL/glext.h:2019
GL_IUI_N3F_V2F_EXT = 33199 	# GL/glext.h:2020
GL_IUI_N3F_V3F_EXT = 33200 	# GL/glext.h:2021
GL_T2F_IUI_V2F_EXT = 33201 	# GL/glext.h:2022
GL_T2F_IUI_V3F_EXT = 33202 	# GL/glext.h:2023
GL_T2F_IUI_N3F_V2F_EXT = 33203 	# GL/glext.h:2024
GL_T2F_IUI_N3F_V3F_EXT = 33204 	# GL/glext.h:2025
# EXT_compiled_vertex_array (GL/glext.h:2028)
GL_ARRAY_ELEMENT_LOCK_FIRST_EXT = 33192 	# GL/glext.h:2029
GL_ARRAY_ELEMENT_LOCK_COUNT_EXT = 33193 	# GL/glext.h:2030
# EXT_cull_vertex (GL/glext.h:2033)
GL_CULL_VERTEX_EXT = 33194 	# GL/glext.h:2034
GL_CULL_VERTEX_EYE_POSITION_EXT = 33195 	# GL/glext.h:2035
GL_CULL_VERTEX_OBJECT_POSITION_EXT = 33196 	# GL/glext.h:2036
# SGIX_ycrcb (GL/glext.h:2039)
GL_YCRCB_422_SGIX = 33211 	# GL/glext.h:2040
GL_YCRCB_444_SGIX = 33212 	# GL/glext.h:2041
# SGIX_fragment_lighting (GL/glext.h:2044)
GL_FRAGMENT_LIGHTING_SGIX = 33792 	# GL/glext.h:2045
GL_FRAGMENT_COLOR_MATERIAL_SGIX = 33793 	# GL/glext.h:2046
GL_FRAGMENT_COLOR_MATERIAL_FACE_SGIX = 33794 	# GL/glext.h:2047
GL_FRAGMENT_COLOR_MATERIAL_PARAMETER_SGIX = 33795 	# GL/glext.h:2048
GL_MAX_FRAGMENT_LIGHTS_SGIX = 33796 	# GL/glext.h:2049
GL_MAX_ACTIVE_LIGHTS_SGIX = 33797 	# GL/glext.h:2050
GL_CURRENT_RASTER_NORMAL_SGIX = 33798 	# GL/glext.h:2051
GL_LIGHT_ENV_MODE_SGIX = 33799 	# GL/glext.h:2052
GL_FRAGMENT_LIGHT_MODEL_LOCAL_VIEWER_SGIX = 33800 	# GL/glext.h:2053
GL_FRAGMENT_LIGHT_MODEL_TWO_SIDE_SGIX = 33801 	# GL/glext.h:2054
GL_FRAGMENT_LIGHT_MODEL_AMBIENT_SGIX = 33802 	# GL/glext.h:2055
GL_FRAGMENT_LIGHT_MODEL_NORMAL_INTERPOLATION_SGIX = 33803 	# GL/glext.h:2056
GL_FRAGMENT_LIGHT0_SGIX = 33804 	# GL/glext.h:2057
GL_FRAGMENT_LIGHT1_SGIX = 33805 	# GL/glext.h:2058
GL_FRAGMENT_LIGHT2_SGIX = 33806 	# GL/glext.h:2059
GL_FRAGMENT_LIGHT3_SGIX = 33807 	# GL/glext.h:2060
GL_FRAGMENT_LIGHT4_SGIX = 33808 	# GL/glext.h:2061
GL_FRAGMENT_LIGHT5_SGIX = 33809 	# GL/glext.h:2062
GL_FRAGMENT_LIGHT6_SGIX = 33810 	# GL/glext.h:2063
GL_FRAGMENT_LIGHT7_SGIX = 33811 	# GL/glext.h:2064
# IBM_rasterpos_clip (GL/glext.h:2067)
GL_RASTER_POSITION_UNCLIPPED_IBM = 103010 	# GL/glext.h:2068
# HP_texture_lighting (GL/glext.h:2071)
GL_TEXTURE_LIGHTING_MODE_HP = 33127 	# GL/glext.h:2072
GL_TEXTURE_POST_SPECULAR_HP = 33128 	# GL/glext.h:2073
GL_TEXTURE_PRE_SPECULAR_HP = 33129 	# GL/glext.h:2074
# EXT_draw_range_elements (GL/glext.h:2077)
GL_MAX_ELEMENTS_VERTICES_EXT = 33000 	# GL/glext.h:2078
GL_MAX_ELEMENTS_INDICES_EXT = 33001 	# GL/glext.h:2079
# WIN_phong_shading (GL/glext.h:2082)
GL_PHONG_WIN = 33002 	# GL/glext.h:2083
GL_PHONG_HINT_WIN = 33003 	# GL/glext.h:2084
# WIN_specular_fog (GL/glext.h:2087)
GL_FOG_SPECULAR_TEXTURE_WIN = 33004 	# GL/glext.h:2088
# EXT_light_texture (GL/glext.h:2091)
GL_FRAGMENT_MATERIAL_EXT = 33609 	# GL/glext.h:2092
GL_FRAGMENT_NORMAL_EXT = 33610 	# GL/glext.h:2093
GL_FRAGMENT_COLOR_EXT = 33612 	# GL/glext.h:2094
GL_ATTENUATION_EXT = 33613 	# GL/glext.h:2095
GL_SHADOW_ATTENUATION_EXT = 33614 	# GL/glext.h:2096
GL_TEXTURE_APPLICATION_MODE_EXT = 33615 	# GL/glext.h:2097
GL_TEXTURE_LIGHT_EXT = 33616 	# GL/glext.h:2098
GL_TEXTURE_MATERIAL_FACE_EXT = 33617 	# GL/glext.h:2099
GL_TEXTURE_MATERIAL_PARAMETER_EXT = 33618 	# GL/glext.h:2100
# SGIX_blend_alpha_minmax (GL/glext.h:2104)
GL_ALPHA_MIN_SGIX = 33568 	# GL/glext.h:2105
GL_ALPHA_MAX_SGIX = 33569 	# GL/glext.h:2106
# SGIX_impact_pixel_texture (GL/glext.h:2109)
GL_PIXEL_TEX_GEN_Q_CEILING_SGIX = 33156 	# GL/glext.h:2110
GL_PIXEL_TEX_GEN_Q_ROUND_SGIX = 33157 	# GL/glext.h:2111
GL_PIXEL_TEX_GEN_Q_FLOOR_SGIX = 33158 	# GL/glext.h:2112
GL_PIXEL_TEX_GEN_ALPHA_REPLACE_SGIX = 33159 	# GL/glext.h:2113
GL_PIXEL_TEX_GEN_ALPHA_NO_REPLACE_SGIX = 33160 	# GL/glext.h:2114
GL_PIXEL_TEX_GEN_ALPHA_LS_SGIX = 33161 	# GL/glext.h:2115
GL_PIXEL_TEX_GEN_ALPHA_MS_SGIX = 33162 	# GL/glext.h:2116
# EXT_bgra (GL/glext.h:2119)
GL_BGR_EXT = 32992 	# GL/glext.h:2120
GL_BGRA_EXT = 32993 	# GL/glext.h:2121
# SGIX_async (GL/glext.h:2124)
GL_ASYNC_MARKER_SGIX = 33577 	# GL/glext.h:2125
# SGIX_async_pixel (GL/glext.h:2128)
GL_ASYNC_TEX_IMAGE_SGIX = 33628 	# GL/glext.h:2129
GL_ASYNC_DRAW_PIXELS_SGIX = 33629 	# GL/glext.h:2130
GL_ASYNC_READ_PIXELS_SGIX = 33630 	# GL/glext.h:2131
GL_MAX_ASYNC_TEX_IMAGE_SGIX = 33631 	# GL/glext.h:2132
GL_MAX_ASYNC_DRAW_PIXELS_SGIX = 33632 	# GL/glext.h:2133
GL_MAX_ASYNC_READ_PIXELS_SGIX = 33633 	# GL/glext.h:2134
# SGIX_async_histogram (GL/glext.h:2137)
GL_ASYNC_HISTOGRAM_SGIX = 33580 	# GL/glext.h:2138
GL_MAX_ASYNC_HISTOGRAM_SGIX = 33581 	# GL/glext.h:2139
# INTEL_texture_scissor (GL/glext.h:2142)
# INTEL_parallel_arrays (GL/glext.h:2145)
GL_PARALLEL_ARRAYS_INTEL = 33780 	# GL/glext.h:2146
GL_VERTEX_ARRAY_PARALLEL_POINTERS_INTEL = 33781 	# GL/glext.h:2147
GL_NORMAL_ARRAY_PARALLEL_POINTERS_INTEL = 33782 	# GL/glext.h:2148
GL_COLOR_ARRAY_PARALLEL_POINTERS_INTEL = 33783 	# GL/glext.h:2149
GL_TEXTURE_COORD_ARRAY_PARALLEL_POINTERS_INTEL = 33784 	# GL/glext.h:2150
# HP_occlusion_test (GL/glext.h:2153)
GL_OCCLUSION_TEST_HP = 33125 	# GL/glext.h:2154
GL_OCCLUSION_TEST_RESULT_HP = 33126 	# GL/glext.h:2155
# EXT_pixel_transform (GL/glext.h:2158)
GL_PIXEL_TRANSFORM_2D_EXT = 33584 	# GL/glext.h:2159
GL_PIXEL_MAG_FILTER_EXT = 33585 	# GL/glext.h:2160
GL_PIXEL_MIN_FILTER_EXT = 33586 	# GL/glext.h:2161
GL_PIXEL_CUBIC_WEIGHT_EXT = 33587 	# GL/glext.h:2162
GL_CUBIC_EXT = 33588 	# GL/glext.h:2163
GL_AVERAGE_EXT = 33589 	# GL/glext.h:2164
GL_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT = 33590 	# GL/glext.h:2165
GL_MAX_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT = 33591 	# GL/glext.h:2166
GL_PIXEL_TRANSFORM_2D_MATRIX_EXT = 33592 	# GL/glext.h:2167
# EXT_pixel_transform_color_table (GL/glext.h:2170)
# EXT_shared_texture_palette (GL/glext.h:2173)
GL_SHARED_TEXTURE_PALETTE_EXT = 33275 	# GL/glext.h:2174
# EXT_separate_specular_color (GL/glext.h:2177)
GL_LIGHT_MODEL_COLOR_CONTROL_EXT = 33272 	# GL/glext.h:2178
GL_SINGLE_COLOR_EXT = 33273 	# GL/glext.h:2179
GL_SEPARATE_SPECULAR_COLOR_EXT = 33274 	# GL/glext.h:2180
# EXT_secondary_color (GL/glext.h:2183)
GL_COLOR_SUM_EXT = 33880 	# GL/glext.h:2184
GL_CURRENT_SECONDARY_COLOR_EXT = 33881 	# GL/glext.h:2185
GL_SECONDARY_COLOR_ARRAY_SIZE_EXT = 33882 	# GL/glext.h:2186
GL_SECONDARY_COLOR_ARRAY_TYPE_EXT = 33883 	# GL/glext.h:2187
GL_SECONDARY_COLOR_ARRAY_STRIDE_EXT = 33884 	# GL/glext.h:2188
GL_SECONDARY_COLOR_ARRAY_POINTER_EXT = 33885 	# GL/glext.h:2189
GL_SECONDARY_COLOR_ARRAY_EXT = 33886 	# GL/glext.h:2190
# EXT_texture_perturb_normal (GL/glext.h:2193)
GL_PERTURB_EXT = 34222 	# GL/glext.h:2194
GL_TEXTURE_NORMAL_EXT = 34223 	# GL/glext.h:2195
# EXT_multi_draw_arrays (GL/glext.h:2198)
# EXT_fog_coord (GL/glext.h:2201)
GL_FOG_COORDINATE_SOURCE_EXT = 33872 	# GL/glext.h:2202
GL_FOG_COORDINATE_EXT = 33873 	# GL/glext.h:2203
GL_FRAGMENT_DEPTH_EXT = 33874 	# GL/glext.h:2204
GL_CURRENT_FOG_COORDINATE_EXT = 33875 	# GL/glext.h:2205
GL_FOG_COORDINATE_ARRAY_TYPE_EXT = 33876 	# GL/glext.h:2206
GL_FOG_COORDINATE_ARRAY_STRIDE_EXT = 33877 	# GL/glext.h:2207
GL_FOG_COORDINATE_ARRAY_POINTER_EXT = 33878 	# GL/glext.h:2208
GL_FOG_COORDINATE_ARRAY_EXT = 33879 	# GL/glext.h:2209
# REND_screen_coordinates (GL/glext.h:2212)
GL_SCREEN_COORDINATES_REND = 33936 	# GL/glext.h:2213
GL_INVERTED_SCREEN_W_REND = 33937 	# GL/glext.h:2214
# EXT_coordinate_frame (GL/glext.h:2217)
GL_TANGENT_ARRAY_EXT = 33849 	# GL/glext.h:2218
GL_BINORMAL_ARRAY_EXT = 33850 	# GL/glext.h:2219
GL_CURRENT_TANGENT_EXT = 33851 	# GL/glext.h:2220
GL_CURRENT_BINORMAL_EXT = 33852 	# GL/glext.h:2221
GL_TANGENT_ARRAY_TYPE_EXT = 33854 	# GL/glext.h:2222
GL_TANGENT_ARRAY_STRIDE_EXT = 33855 	# GL/glext.h:2223
GL_BINORMAL_ARRAY_TYPE_EXT = 33856 	# GL/glext.h:2224
GL_BINORMAL_ARRAY_STRIDE_EXT = 33857 	# GL/glext.h:2225
GL_TANGENT_ARRAY_POINTER_EXT = 33858 	# GL/glext.h:2226
GL_BINORMAL_ARRAY_POINTER_EXT = 33859 	# GL/glext.h:2227
GL_MAP1_TANGENT_EXT = 33860 	# GL/glext.h:2228
GL_MAP2_TANGENT_EXT = 33861 	# GL/glext.h:2229
GL_MAP1_BINORMAL_EXT = 33862 	# GL/glext.h:2230
GL_MAP2_BINORMAL_EXT = 33863 	# GL/glext.h:2231
# EXT_texture_env_combine (GL/glext.h:2234)
GL_COMBINE_EXT = 34160 	# GL/glext.h:2235
GL_COMBINE_RGB_EXT = 34161 	# GL/glext.h:2236
GL_COMBINE_ALPHA_EXT = 34162 	# GL/glext.h:2237
GL_RGB_SCALE_EXT = 34163 	# GL/glext.h:2238
GL_ADD_SIGNED_EXT = 34164 	# GL/glext.h:2239
GL_INTERPOLATE_EXT = 34165 	# GL/glext.h:2240
GL_CONSTANT_EXT = 34166 	# GL/glext.h:2241
GL_PRIMARY_COLOR_EXT = 34167 	# GL/glext.h:2242
GL_PREVIOUS_EXT = 34168 	# GL/glext.h:2243
GL_SOURCE0_RGB_EXT = 34176 	# GL/glext.h:2244
GL_SOURCE1_RGB_EXT = 34177 	# GL/glext.h:2245
GL_SOURCE2_RGB_EXT = 34178 	# GL/glext.h:2246
GL_SOURCE0_ALPHA_EXT = 34184 	# GL/glext.h:2247
GL_SOURCE1_ALPHA_EXT = 34185 	# GL/glext.h:2248
GL_SOURCE2_ALPHA_EXT = 34186 	# GL/glext.h:2249
GL_OPERAND0_RGB_EXT = 34192 	# GL/glext.h:2250
GL_OPERAND1_RGB_EXT = 34193 	# GL/glext.h:2251
GL_OPERAND2_RGB_EXT = 34194 	# GL/glext.h:2252
GL_OPERAND0_ALPHA_EXT = 34200 	# GL/glext.h:2253
GL_OPERAND1_ALPHA_EXT = 34201 	# GL/glext.h:2254
GL_OPERAND2_ALPHA_EXT = 34202 	# GL/glext.h:2255
# APPLE_specular_vector (GL/glext.h:2258)
GL_LIGHT_MODEL_SPECULAR_VECTOR_APPLE = 34224 	# GL/glext.h:2259
# APPLE_transform_hint (GL/glext.h:2262)
GL_TRANSFORM_HINT_APPLE = 34225 	# GL/glext.h:2263
# SGIX_fog_scale (GL/glext.h:2266)
GL_FOG_SCALE_SGIX = 33276 	# GL/glext.h:2267
GL_FOG_SCALE_VALUE_SGIX = 33277 	# GL/glext.h:2268
# SUNX_constant_data (GL/glext.h:2271)
GL_UNPACK_CONSTANT_DATA_SUNX = 33237 	# GL/glext.h:2272
GL_TEXTURE_CONSTANT_DATA_SUNX = 33238 	# GL/glext.h:2273
# SUN_global_alpha (GL/glext.h:2276)
GL_GLOBAL_ALPHA_SUN = 33241 	# GL/glext.h:2277
GL_GLOBAL_ALPHA_FACTOR_SUN = 33242 	# GL/glext.h:2278
# SUN_triangle_list (GL/glext.h:2281)
GL_RESTART_SUN = 1 	# GL/glext.h:2282
GL_REPLACE_MIDDLE_SUN = 2 	# GL/glext.h:2283
GL_REPLACE_OLDEST_SUN = 3 	# GL/glext.h:2284
GL_TRIANGLE_LIST_SUN = 33239 	# GL/glext.h:2285
GL_REPLACEMENT_CODE_SUN = 33240 	# GL/glext.h:2286
GL_REPLACEMENT_CODE_ARRAY_SUN = 34240 	# GL/glext.h:2287
GL_REPLACEMENT_CODE_ARRAY_TYPE_SUN = 34241 	# GL/glext.h:2288
GL_REPLACEMENT_CODE_ARRAY_STRIDE_SUN = 34242 	# GL/glext.h:2289
GL_REPLACEMENT_CODE_ARRAY_POINTER_SUN = 34243 	# GL/glext.h:2290
GL_R1UI_V3F_SUN = 34244 	# GL/glext.h:2291
GL_R1UI_C4UB_V3F_SUN = 34245 	# GL/glext.h:2292
GL_R1UI_C3F_V3F_SUN = 34246 	# GL/glext.h:2293
GL_R1UI_N3F_V3F_SUN = 34247 	# GL/glext.h:2294
GL_R1UI_C4F_N3F_V3F_SUN = 34248 	# GL/glext.h:2295
GL_R1UI_T2F_V3F_SUN = 34249 	# GL/glext.h:2296
GL_R1UI_T2F_N3F_V3F_SUN = 34250 	# GL/glext.h:2297
GL_R1UI_T2F_C4F_N3F_V3F_SUN = 34251 	# GL/glext.h:2298
# SUN_vertex (GL/glext.h:2301)
# EXT_blend_func_separate (GL/glext.h:2304)
GL_BLEND_DST_RGB_EXT = 32968 	# GL/glext.h:2305
GL_BLEND_SRC_RGB_EXT = 32969 	# GL/glext.h:2306
GL_BLEND_DST_ALPHA_EXT = 32970 	# GL/glext.h:2307
GL_BLEND_SRC_ALPHA_EXT = 32971 	# GL/glext.h:2308
# INGR_color_clamp (GL/glext.h:2311)
GL_RED_MIN_CLAMP_INGR = 34144 	# GL/glext.h:2312
GL_GREEN_MIN_CLAMP_INGR = 34145 	# GL/glext.h:2313
GL_BLUE_MIN_CLAMP_INGR = 34146 	# GL/glext.h:2314
GL_ALPHA_MIN_CLAMP_INGR = 34147 	# GL/glext.h:2315
GL_RED_MAX_CLAMP_INGR = 34148 	# GL/glext.h:2316
GL_GREEN_MAX_CLAMP_INGR = 34149 	# GL/glext.h:2317
GL_BLUE_MAX_CLAMP_INGR = 34150 	# GL/glext.h:2318
GL_ALPHA_MAX_CLAMP_INGR = 34151 	# GL/glext.h:2319
# INGR_interlace_read (GL/glext.h:2322)
GL_INTERLACE_READ_INGR = 34152 	# GL/glext.h:2323
# EXT_stencil_wrap (GL/glext.h:2326)
GL_INCR_WRAP_EXT = 34055 	# GL/glext.h:2327
GL_DECR_WRAP_EXT = 34056 	# GL/glext.h:2328
# EXT_422_pixels (GL/glext.h:2331)
GL_422_EXT = 32972 	# GL/glext.h:2332
GL_422_REV_EXT = 32973 	# GL/glext.h:2333
GL_422_AVERAGE_EXT = 32974 	# GL/glext.h:2334
GL_422_REV_AVERAGE_EXT = 32975 	# GL/glext.h:2335
# NV_texgen_reflection (GL/glext.h:2338)
GL_NORMAL_MAP_NV = 34065 	# GL/glext.h:2339
GL_REFLECTION_MAP_NV = 34066 	# GL/glext.h:2340
# EXT_texture_cube_map (GL/glext.h:2343)
GL_NORMAL_MAP_EXT = 34065 	# GL/glext.h:2344
GL_REFLECTION_MAP_EXT = 34066 	# GL/glext.h:2345
GL_TEXTURE_CUBE_MAP_EXT = 34067 	# GL/glext.h:2346
GL_TEXTURE_BINDING_CUBE_MAP_EXT = 34068 	# GL/glext.h:2347
GL_TEXTURE_CUBE_MAP_POSITIVE_X_EXT = 34069 	# GL/glext.h:2348
GL_TEXTURE_CUBE_MAP_NEGATIVE_X_EXT = 34070 	# GL/glext.h:2349
GL_TEXTURE_CUBE_MAP_POSITIVE_Y_EXT = 34071 	# GL/glext.h:2350
GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_EXT = 34072 	# GL/glext.h:2351
GL_TEXTURE_CUBE_MAP_POSITIVE_Z_EXT = 34073 	# GL/glext.h:2352
GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_EXT = 34074 	# GL/glext.h:2353
GL_PROXY_TEXTURE_CUBE_MAP_EXT = 34075 	# GL/glext.h:2354
GL_MAX_CUBE_MAP_TEXTURE_SIZE_EXT = 34076 	# GL/glext.h:2355
# SUN_convolution_border_modes (GL/glext.h:2358)
GL_WRAP_BORDER_SUN = 33236 	# GL/glext.h:2359
# EXT_texture_env_add (GL/glext.h:2362)
# EXT_texture_lod_bias (GL/glext.h:2365)
GL_MAX_TEXTURE_LOD_BIAS_EXT = 34045 	# GL/glext.h:2366
GL_TEXTURE_FILTER_CONTROL_EXT = 34048 	# GL/glext.h:2367
GL_TEXTURE_LOD_BIAS_EXT = 34049 	# GL/glext.h:2368
# EXT_texture_filter_anisotropic (GL/glext.h:2371)
GL_TEXTURE_MAX_ANISOTROPY_EXT = 34046 	# GL/glext.h:2372
GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT = 34047 	# GL/glext.h:2373
# EXT_vertex_weighting (GL/glext.h:2376)
GL_MODELVIEW0_STACK_DEPTH_EXT = 2979 	# GL/glext.h:2377
GL_MODELVIEW1_STACK_DEPTH_EXT = 34050 	# GL/glext.h:2378
GL_MODELVIEW0_MATRIX_EXT = 2982 	# GL/glext.h:2379
GL_MODELVIEW1_MATRIX_EXT = 34054 	# GL/glext.h:2380
GL_VERTEX_WEIGHTING_EXT = 34057 	# GL/glext.h:2381
GL_MODELVIEW0_EXT = 5888 	# GL/glext.h:2382
GL_MODELVIEW1_EXT = 34058 	# GL/glext.h:2383
GL_CURRENT_VERTEX_WEIGHT_EXT = 34059 	# GL/glext.h:2384
GL_VERTEX_WEIGHT_ARRAY_EXT = 34060 	# GL/glext.h:2385
GL_VERTEX_WEIGHT_ARRAY_SIZE_EXT = 34061 	# GL/glext.h:2386
GL_VERTEX_WEIGHT_ARRAY_TYPE_EXT = 34062 	# GL/glext.h:2387
GL_VERTEX_WEIGHT_ARRAY_STRIDE_EXT = 34063 	# GL/glext.h:2388
GL_VERTEX_WEIGHT_ARRAY_POINTER_EXT = 34064 	# GL/glext.h:2389
# NV_light_max_exponent (GL/glext.h:2392)
GL_MAX_SHININESS_NV = 34052 	# GL/glext.h:2393
GL_MAX_SPOT_EXPONENT_NV = 34053 	# GL/glext.h:2394
# NV_vertex_array_range (GL/glext.h:2397)
GL_VERTEX_ARRAY_RANGE_NV = 34077 	# GL/glext.h:2398
GL_VERTEX_ARRAY_RANGE_LENGTH_NV = 34078 	# GL/glext.h:2399
GL_VERTEX_ARRAY_RANGE_VALID_NV = 34079 	# GL/glext.h:2400
GL_MAX_VERTEX_ARRAY_RANGE_ELEMENT_NV = 34080 	# GL/glext.h:2401
GL_VERTEX_ARRAY_RANGE_POINTER_NV = 34081 	# GL/glext.h:2402
# NV_register_combiners (GL/glext.h:2405)
GL_REGISTER_COMBINERS_NV = 34082 	# GL/glext.h:2406
GL_VARIABLE_A_NV = 34083 	# GL/glext.h:2407
GL_VARIABLE_B_NV = 34084 	# GL/glext.h:2408
GL_VARIABLE_C_NV = 34085 	# GL/glext.h:2409
GL_VARIABLE_D_NV = 34086 	# GL/glext.h:2410
GL_VARIABLE_E_NV = 34087 	# GL/glext.h:2411
GL_VARIABLE_F_NV = 34088 	# GL/glext.h:2412
GL_VARIABLE_G_NV = 34089 	# GL/glext.h:2413
GL_CONSTANT_COLOR0_NV = 34090 	# GL/glext.h:2414
GL_CONSTANT_COLOR1_NV = 34091 	# GL/glext.h:2415
GL_PRIMARY_COLOR_NV = 34092 	# GL/glext.h:2416
GL_SECONDARY_COLOR_NV = 34093 	# GL/glext.h:2417
GL_SPARE0_NV = 34094 	# GL/glext.h:2418
GL_SPARE1_NV = 34095 	# GL/glext.h:2419
GL_DISCARD_NV = 34096 	# GL/glext.h:2420
GL_E_TIMES_F_NV = 34097 	# GL/glext.h:2421
GL_SPARE0_PLUS_SECONDARY_COLOR_NV = 34098 	# GL/glext.h:2422
GL_UNSIGNED_IDENTITY_NV = 34102 	# GL/glext.h:2423
GL_UNSIGNED_INVERT_NV = 34103 	# GL/glext.h:2424
GL_EXPAND_NORMAL_NV = 34104 	# GL/glext.h:2425
GL_EXPAND_NEGATE_NV = 34105 	# GL/glext.h:2426
GL_HALF_BIAS_NORMAL_NV = 34106 	# GL/glext.h:2427
GL_HALF_BIAS_NEGATE_NV = 34107 	# GL/glext.h:2428
GL_SIGNED_IDENTITY_NV = 34108 	# GL/glext.h:2429
GL_SIGNED_NEGATE_NV = 34109 	# GL/glext.h:2430
GL_SCALE_BY_TWO_NV = 34110 	# GL/glext.h:2431
GL_SCALE_BY_FOUR_NV = 34111 	# GL/glext.h:2432
GL_SCALE_BY_ONE_HALF_NV = 34112 	# GL/glext.h:2433
GL_BIAS_BY_NEGATIVE_ONE_HALF_NV = 34113 	# GL/glext.h:2434
GL_COMBINER_INPUT_NV = 34114 	# GL/glext.h:2435
GL_COMBINER_MAPPING_NV = 34115 	# GL/glext.h:2436
GL_COMBINER_COMPONENT_USAGE_NV = 34116 	# GL/glext.h:2437
GL_COMBINER_AB_DOT_PRODUCT_NV = 34117 	# GL/glext.h:2438
GL_COMBINER_CD_DOT_PRODUCT_NV = 34118 	# GL/glext.h:2439
GL_COMBINER_MUX_SUM_NV = 34119 	# GL/glext.h:2440
GL_COMBINER_SCALE_NV = 34120 	# GL/glext.h:2441
GL_COMBINER_BIAS_NV = 34121 	# GL/glext.h:2442
GL_COMBINER_AB_OUTPUT_NV = 34122 	# GL/glext.h:2443
GL_COMBINER_CD_OUTPUT_NV = 34123 	# GL/glext.h:2444
GL_COMBINER_SUM_OUTPUT_NV = 34124 	# GL/glext.h:2445
GL_MAX_GENERAL_COMBINERS_NV = 34125 	# GL/glext.h:2446
GL_NUM_GENERAL_COMBINERS_NV = 34126 	# GL/glext.h:2447
GL_COLOR_SUM_CLAMP_NV = 34127 	# GL/glext.h:2448
GL_COMBINER0_NV = 34128 	# GL/glext.h:2449
GL_COMBINER1_NV = 34129 	# GL/glext.h:2450
GL_COMBINER2_NV = 34130 	# GL/glext.h:2451
GL_COMBINER3_NV = 34131 	# GL/glext.h:2452
GL_COMBINER4_NV = 34132 	# GL/glext.h:2453
GL_COMBINER5_NV = 34133 	# GL/glext.h:2454
GL_COMBINER6_NV = 34134 	# GL/glext.h:2455
GL_COMBINER7_NV = 34135 	# GL/glext.h:2456
# NV_fog_distance (GL/glext.h:2464)
GL_FOG_DISTANCE_MODE_NV = 34138 	# GL/glext.h:2465
GL_EYE_RADIAL_NV = 34139 	# GL/glext.h:2466
GL_EYE_PLANE_ABSOLUTE_NV = 34140 	# GL/glext.h:2467
# NV_texgen_emboss (GL/glext.h:2471)
GL_EMBOSS_LIGHT_NV = 34141 	# GL/glext.h:2472
GL_EMBOSS_CONSTANT_NV = 34142 	# GL/glext.h:2473
GL_EMBOSS_MAP_NV = 34143 	# GL/glext.h:2474
# NV_blend_square (GL/glext.h:2477)
# NV_texture_env_combine4 (GL/glext.h:2480)
GL_COMBINE4_NV = 34051 	# GL/glext.h:2481
GL_SOURCE3_RGB_NV = 34179 	# GL/glext.h:2482
GL_SOURCE3_ALPHA_NV = 34187 	# GL/glext.h:2483
GL_OPERAND3_RGB_NV = 34195 	# GL/glext.h:2484
GL_OPERAND3_ALPHA_NV = 34203 	# GL/glext.h:2485
# MESA_resize_buffers (GL/glext.h:2488)
# MESA_window_pos (GL/glext.h:2491)
# EXT_texture_compression_s3tc (GL/glext.h:2494)
GL_COMPRESSED_RGB_S3TC_DXT1_EXT = 33776 	# GL/glext.h:2495
GL_COMPRESSED_RGBA_S3TC_DXT1_EXT = 33777 	# GL/glext.h:2496
GL_COMPRESSED_RGBA_S3TC_DXT3_EXT = 33778 	# GL/glext.h:2497
GL_COMPRESSED_RGBA_S3TC_DXT5_EXT = 33779 	# GL/glext.h:2498
# IBM_cull_vertex (GL/glext.h:2501)
GL_CULL_VERTEX_IBM = 103050 	# GL/glext.h:2502
# IBM_multimode_draw_arrays (GL/glext.h:2505)
# IBM_vertex_array_lists (GL/glext.h:2508)
GL_VERTEX_ARRAY_LIST_IBM = 103070 	# GL/glext.h:2509
GL_NORMAL_ARRAY_LIST_IBM = 103071 	# GL/glext.h:2510
GL_COLOR_ARRAY_LIST_IBM = 103072 	# GL/glext.h:2511
GL_INDEX_ARRAY_LIST_IBM = 103073 	# GL/glext.h:2512
GL_TEXTURE_COORD_ARRAY_LIST_IBM = 103074 	# GL/glext.h:2513
GL_EDGE_FLAG_ARRAY_LIST_IBM = 103075 	# GL/glext.h:2514
GL_FOG_COORDINATE_ARRAY_LIST_IBM = 103076 	# GL/glext.h:2515
GL_SECONDARY_COLOR_ARRAY_LIST_IBM = 103077 	# GL/glext.h:2516
GL_VERTEX_ARRAY_LIST_STRIDE_IBM = 103080 	# GL/glext.h:2517
GL_NORMAL_ARRAY_LIST_STRIDE_IBM = 103081 	# GL/glext.h:2518
GL_COLOR_ARRAY_LIST_STRIDE_IBM = 103082 	# GL/glext.h:2519
GL_INDEX_ARRAY_LIST_STRIDE_IBM = 103083 	# GL/glext.h:2520
GL_TEXTURE_COORD_ARRAY_LIST_STRIDE_IBM = 103084 	# GL/glext.h:2521
GL_EDGE_FLAG_ARRAY_LIST_STRIDE_IBM = 103085 	# GL/glext.h:2522
GL_FOG_COORDINATE_ARRAY_LIST_STRIDE_IBM = 103086 	# GL/glext.h:2523
GL_SECONDARY_COLOR_ARRAY_LIST_STRIDE_IBM = 103087 	# GL/glext.h:2524
# SGIX_subsample (GL/glext.h:2527)
GL_PACK_SUBSAMPLE_RATE_SGIX = 34208 	# GL/glext.h:2528
GL_UNPACK_SUBSAMPLE_RATE_SGIX = 34209 	# GL/glext.h:2529
GL_PIXEL_SUBSAMPLE_4444_SGIX = 34210 	# GL/glext.h:2530
GL_PIXEL_SUBSAMPLE_2424_SGIX = 34211 	# GL/glext.h:2531
GL_PIXEL_SUBSAMPLE_4242_SGIX = 34212 	# GL/glext.h:2532
# SGIX_ycrcb_subsample (GL/glext.h:2535)
# SGIX_ycrcba (GL/glext.h:2538)
GL_YCRCB_SGIX = 33560 	# GL/glext.h:2539
GL_YCRCBA_SGIX = 33561 	# GL/glext.h:2540
# SGI_depth_pass_instrument (GL/glext.h:2543)
GL_DEPTH_PASS_INSTRUMENT_SGIX = 33552 	# GL/glext.h:2544
GL_DEPTH_PASS_INSTRUMENT_COUNTERS_SGIX = 33553 	# GL/glext.h:2545
GL_DEPTH_PASS_INSTRUMENT_MAX_SGIX = 33554 	# GL/glext.h:2546
# 3DFX_texture_compression_FXT1 (GL/glext.h:2549)
GL_COMPRESSED_RGB_FXT1_3DFX = 34480 	# GL/glext.h:2550
GL_COMPRESSED_RGBA_FXT1_3DFX = 34481 	# GL/glext.h:2551
# 3DFX_multisample (GL/glext.h:2554)
GL_MULTISAMPLE_3DFX = 34482 	# GL/glext.h:2555
GL_SAMPLE_BUFFERS_3DFX = 34483 	# GL/glext.h:2556
GL_SAMPLES_3DFX = 34484 	# GL/glext.h:2557
GL_MULTISAMPLE_BIT_3DFX = 536870912 	# GL/glext.h:2558
# 3DFX_tbuffer (GL/glext.h:2561)
# EXT_multisample (GL/glext.h:2564)
GL_MULTISAMPLE_EXT = 32925 	# GL/glext.h:2565
GL_SAMPLE_ALPHA_TO_MASK_EXT = 32926 	# GL/glext.h:2566
GL_SAMPLE_ALPHA_TO_ONE_EXT = 32927 	# GL/glext.h:2567
GL_SAMPLE_MASK_EXT = 32928 	# GL/glext.h:2568
GL_1PASS_EXT = 32929 	# GL/glext.h:2569
GL_2PASS_0_EXT = 32930 	# GL/glext.h:2570
GL_2PASS_1_EXT = 32931 	# GL/glext.h:2571
GL_4PASS_0_EXT = 32932 	# GL/glext.h:2572
GL_4PASS_1_EXT = 32933 	# GL/glext.h:2573
GL_4PASS_2_EXT = 32934 	# GL/glext.h:2574
GL_4PASS_3_EXT = 32935 	# GL/glext.h:2575
GL_SAMPLE_BUFFERS_EXT = 32936 	# GL/glext.h:2576
GL_SAMPLES_EXT = 32937 	# GL/glext.h:2577
GL_SAMPLE_MASK_VALUE_EXT = 32938 	# GL/glext.h:2578
GL_SAMPLE_MASK_INVERT_EXT = 32939 	# GL/glext.h:2579
GL_SAMPLE_PATTERN_EXT = 32940 	# GL/glext.h:2580
GL_MULTISAMPLE_BIT_EXT = 536870912 	# GL/glext.h:2581
# SGIX_vertex_preclip (GL/glext.h:2584)
GL_VERTEX_PRECLIP_SGIX = 33774 	# GL/glext.h:2585
GL_VERTEX_PRECLIP_HINT_SGIX = 33775 	# GL/glext.h:2586
# SGIX_convolution_accuracy (GL/glext.h:2589)
GL_CONVOLUTION_HINT_SGIX = 33558 	# GL/glext.h:2590
# SGIX_resample (GL/glext.h:2593)
GL_PACK_RESAMPLE_SGIX = 33836 	# GL/glext.h:2594
GL_UNPACK_RESAMPLE_SGIX = 33837 	# GL/glext.h:2595
GL_RESAMPLE_REPLICATE_SGIX = 33838 	# GL/glext.h:2596
GL_RESAMPLE_ZERO_FILL_SGIX = 33839 	# GL/glext.h:2597
GL_RESAMPLE_DECIMATE_SGIX = 33840 	# GL/glext.h:2598
# SGIS_point_line_texgen (GL/glext.h:2601)
GL_EYE_DISTANCE_TO_POINT_SGIS = 33264 	# GL/glext.h:2602
GL_OBJECT_DISTANCE_TO_POINT_SGIS = 33265 	# GL/glext.h:2603
GL_EYE_DISTANCE_TO_LINE_SGIS = 33266 	# GL/glext.h:2604
GL_OBJECT_DISTANCE_TO_LINE_SGIS = 33267 	# GL/glext.h:2605
GL_EYE_POINT_SGIS = 33268 	# GL/glext.h:2606
GL_OBJECT_POINT_SGIS = 33269 	# GL/glext.h:2607
GL_EYE_LINE_SGIS = 33270 	# GL/glext.h:2608
GL_OBJECT_LINE_SGIS = 33271 	# GL/glext.h:2609
# SGIS_texture_color_mask (GL/glext.h:2612)
GL_TEXTURE_COLOR_WRITEMASK_SGIS = 33263 	# GL/glext.h:2613
# EXT_texture_env_dot3 (GL/glext.h:2616)
GL_DOT3_RGB_EXT = 34624 	# GL/glext.h:2617
GL_DOT3_RGBA_EXT = 34625 	# GL/glext.h:2618
# ATI_texture_mirror_once (GL/glext.h:2621)
GL_MIRROR_CLAMP_ATI = 34626 	# GL/glext.h:2622
GL_MIRROR_CLAMP_TO_EDGE_ATI = 34627 	# GL/glext.h:2623
# NV_fence (GL/glext.h:2626)
GL_ALL_COMPLETED_NV = 34034 	# GL/glext.h:2627
GL_FENCE_STATUS_NV = 34035 	# GL/glext.h:2628
GL_FENCE_CONDITION_NV = 34036 	# GL/glext.h:2629
# IBM_texture_mirrored_repeat (GL/glext.h:2632)
GL_MIRRORED_REPEAT_IBM = 33648 	# GL/glext.h:2633
# NV_evaluators (GL/glext.h:2636)
GL_EVAL_2D_NV = 34496 	# GL/glext.h:2637
GL_EVAL_TRIANGULAR_2D_NV = 34497 	# GL/glext.h:2638
GL_MAP_TESSELLATION_NV = 34498 	# GL/glext.h:2639
GL_MAP_ATTRIB_U_ORDER_NV = 34499 	# GL/glext.h:2640
GL_MAP_ATTRIB_V_ORDER_NV = 34500 	# GL/glext.h:2641
GL_EVAL_FRACTIONAL_TESSELLATION_NV = 34501 	# GL/glext.h:2642
GL_EVAL_VERTEX_ATTRIB0_NV = 34502 	# GL/glext.h:2643
GL_EVAL_VERTEX_ATTRIB1_NV = 34503 	# GL/glext.h:2644
GL_EVAL_VERTEX_ATTRIB2_NV = 34504 	# GL/glext.h:2645
GL_EVAL_VERTEX_ATTRIB3_NV = 34505 	# GL/glext.h:2646
GL_EVAL_VERTEX_ATTRIB4_NV = 34506 	# GL/glext.h:2647
GL_EVAL_VERTEX_ATTRIB5_NV = 34507 	# GL/glext.h:2648
GL_EVAL_VERTEX_ATTRIB6_NV = 34508 	# GL/glext.h:2649
GL_EVAL_VERTEX_ATTRIB7_NV = 34509 	# GL/glext.h:2650
GL_EVAL_VERTEX_ATTRIB8_NV = 34510 	# GL/glext.h:2651
GL_EVAL_VERTEX_ATTRIB9_NV = 34511 	# GL/glext.h:2652
GL_EVAL_VERTEX_ATTRIB10_NV = 34512 	# GL/glext.h:2653
GL_EVAL_VERTEX_ATTRIB11_NV = 34513 	# GL/glext.h:2654
GL_EVAL_VERTEX_ATTRIB12_NV = 34514 	# GL/glext.h:2655
GL_EVAL_VERTEX_ATTRIB13_NV = 34515 	# GL/glext.h:2656
GL_EVAL_VERTEX_ATTRIB14_NV = 34516 	# GL/glext.h:2657
GL_EVAL_VERTEX_ATTRIB15_NV = 34517 	# GL/glext.h:2658
GL_MAX_MAP_TESSELLATION_NV = 34518 	# GL/glext.h:2659
GL_MAX_RATIONAL_EVAL_ORDER_NV = 34519 	# GL/glext.h:2660
# NV_packed_depth_stencil (GL/glext.h:2663)
GL_DEPTH_STENCIL_NV = 34041 	# GL/glext.h:2664
GL_UNSIGNED_INT_24_8_NV = 34042 	# GL/glext.h:2665
# EXT_packed_depth_stencil (GL/glext.h:2668)
GL_DEPTH_STENCIL_EXT = 34041 	# GL/glext.h:2669
GL_DEPTH24_STENCIL8_EXT = 35056 	# GL/glext.h:2670
GL_TEXTURE_STENCIL_SIZE_EXT = 35057 	# GL/glext.h:2671
GL_UNSIGNED_INT_24_8_EXT = 34042 	# GL/glext.h:2672
# NV_register_combiners2 (GL/glext.h:2675)
GL_PER_STAGE_CONSTANTS_NV = 34101 	# GL/glext.h:2676
# NV_texture_compression_vtc (GL/glext.h:2679)
# NV_texture_rectangle (GL/glext.h:2682)
GL_TEXTURE_RECTANGLE_NV = 34037 	# GL/glext.h:2683
GL_TEXTURE_BINDING_RECTANGLE_NV = 34038 	# GL/glext.h:2684
GL_PROXY_TEXTURE_RECTANGLE_NV = 34039 	# GL/glext.h:2685
GL_MAX_RECTANGLE_TEXTURE_SIZE_NV = 34040 	# GL/glext.h:2686
# NV_texture_shader (GL/glext.h:2689)
GL_OFFSET_TEXTURE_RECTANGLE_NV = 34380 	# GL/glext.h:2690
GL_OFFSET_TEXTURE_RECTANGLE_SCALE_NV = 34381 	# GL/glext.h:2691
GL_DOT_PRODUCT_TEXTURE_RECTANGLE_NV = 34382 	# GL/glext.h:2692
GL_RGBA_UNSIGNED_DOT_PRODUCT_MAPPING_NV = 34521 	# GL/glext.h:2693
GL_UNSIGNED_INT_S8_S8_8_8_NV = 34522 	# GL/glext.h:2694
GL_UNSIGNED_INT_8_8_S8_S8_REV_NV = 34523 	# GL/glext.h:2695
GL_DSDT_MAG_INTENSITY_NV = 34524 	# GL/glext.h:2696
GL_SHADER_CONSISTENT_NV = 34525 	# GL/glext.h:2697
GL_TEXTURE_SHADER_NV = 34526 	# GL/glext.h:2698
GL_SHADER_OPERATION_NV = 34527 	# GL/glext.h:2699
GL_CULL_MODES_NV = 34528 	# GL/glext.h:2700
GL_OFFSET_TEXTURE_MATRIX_NV = 34529 	# GL/glext.h:2701
GL_OFFSET_TEXTURE_SCALE_NV = 34530 	# GL/glext.h:2702
GL_OFFSET_TEXTURE_BIAS_NV = 34531 	# GL/glext.h:2703
GL_OFFSET_TEXTURE_2D_MATRIX_NV = 34529 	# GL/glext.h:2704
GL_OFFSET_TEXTURE_2D_SCALE_NV = 34530 	# GL/glext.h:2705
GL_OFFSET_TEXTURE_2D_BIAS_NV = 34531 	# GL/glext.h:2706
GL_PREVIOUS_TEXTURE_INPUT_NV = 34532 	# GL/glext.h:2707
GL_CONST_EYE_NV = 34533 	# GL/glext.h:2708
GL_PASS_THROUGH_NV = 34534 	# GL/glext.h:2709
GL_CULL_FRAGMENT_NV = 34535 	# GL/glext.h:2710
GL_OFFSET_TEXTURE_2D_NV = 34536 	# GL/glext.h:2711
GL_DEPENDENT_AR_TEXTURE_2D_NV = 34537 	# GL/glext.h:2712
GL_DEPENDENT_GB_TEXTURE_2D_NV = 34538 	# GL/glext.h:2713
GL_DOT_PRODUCT_NV = 34540 	# GL/glext.h:2714
GL_DOT_PRODUCT_DEPTH_REPLACE_NV = 34541 	# GL/glext.h:2715
GL_DOT_PRODUCT_TEXTURE_2D_NV = 34542 	# GL/glext.h:2716
GL_DOT_PRODUCT_TEXTURE_CUBE_MAP_NV = 34544 	# GL/glext.h:2717
GL_DOT_PRODUCT_DIFFUSE_CUBE_MAP_NV = 34545 	# GL/glext.h:2718
GL_DOT_PRODUCT_REFLECT_CUBE_MAP_NV = 34546 	# GL/glext.h:2719
GL_DOT_PRODUCT_CONST_EYE_REFLECT_CUBE_MAP_NV = 34547 	# GL/glext.h:2720
GL_HILO_NV = 34548 	# GL/glext.h:2721
GL_DSDT_NV = 34549 	# GL/glext.h:2722
GL_DSDT_MAG_NV = 34550 	# GL/glext.h:2723
GL_DSDT_MAG_VIB_NV = 34551 	# GL/glext.h:2724
GL_HILO16_NV = 34552 	# GL/glext.h:2725
GL_SIGNED_HILO_NV = 34553 	# GL/glext.h:2726
GL_SIGNED_HILO16_NV = 34554 	# GL/glext.h:2727
GL_SIGNED_RGBA_NV = 34555 	# GL/glext.h:2728
GL_SIGNED_RGBA8_NV = 34556 	# GL/glext.h:2729
GL_SIGNED_RGB_NV = 34558 	# GL/glext.h:2730
GL_SIGNED_RGB8_NV = 34559 	# GL/glext.h:2731
GL_SIGNED_LUMINANCE_NV = 34561 	# GL/glext.h:2732
GL_SIGNED_LUMINANCE8_NV = 34562 	# GL/glext.h:2733
GL_SIGNED_LUMINANCE_ALPHA_NV = 34563 	# GL/glext.h:2734
GL_SIGNED_LUMINANCE8_ALPHA8_NV = 34564 	# GL/glext.h:2735
GL_SIGNED_ALPHA_NV = 34565 	# GL/glext.h:2736
GL_SIGNED_ALPHA8_NV = 34566 	# GL/glext.h:2737
GL_SIGNED_INTENSITY_NV = 34567 	# GL/glext.h:2738
GL_SIGNED_INTENSITY8_NV = 34568 	# GL/glext.h:2739
GL_DSDT8_NV = 34569 	# GL/glext.h:2740
GL_DSDT8_MAG8_NV = 34570 	# GL/glext.h:2741
GL_DSDT8_MAG8_INTENSITY8_NV = 34571 	# GL/glext.h:2742
GL_SIGNED_RGB_UNSIGNED_ALPHA_NV = 34572 	# GL/glext.h:2743
GL_SIGNED_RGB8_UNSIGNED_ALPHA8_NV = 34573 	# GL/glext.h:2744
GL_HI_SCALE_NV = 34574 	# GL/glext.h:2745
GL_LO_SCALE_NV = 34575 	# GL/glext.h:2746
GL_DS_SCALE_NV = 34576 	# GL/glext.h:2747
GL_DT_SCALE_NV = 34577 	# GL/glext.h:2748
GL_MAGNITUDE_SCALE_NV = 34578 	# GL/glext.h:2749
GL_VIBRANCE_SCALE_NV = 34579 	# GL/glext.h:2750
GL_HI_BIAS_NV = 34580 	# GL/glext.h:2751
GL_LO_BIAS_NV = 34581 	# GL/glext.h:2752
GL_DS_BIAS_NV = 34582 	# GL/glext.h:2753
GL_DT_BIAS_NV = 34583 	# GL/glext.h:2754
GL_MAGNITUDE_BIAS_NV = 34584 	# GL/glext.h:2755
GL_VIBRANCE_BIAS_NV = 34585 	# GL/glext.h:2756
GL_TEXTURE_BORDER_VALUES_NV = 34586 	# GL/glext.h:2757
GL_TEXTURE_HI_SIZE_NV = 34587 	# GL/glext.h:2758
GL_TEXTURE_LO_SIZE_NV = 34588 	# GL/glext.h:2759
GL_TEXTURE_DS_SIZE_NV = 34589 	# GL/glext.h:2760
GL_TEXTURE_DT_SIZE_NV = 34590 	# GL/glext.h:2761
GL_TEXTURE_MAG_SIZE_NV = 34591 	# GL/glext.h:2762
# NV_texture_shader2 (GL/glext.h:2765)
GL_DOT_PRODUCT_TEXTURE_3D_NV = 34543 	# GL/glext.h:2766
# NV_vertex_array_range2 (GL/glext.h:2769)
GL_VERTEX_ARRAY_RANGE_WITHOUT_FLUSH_NV = 34099 	# GL/glext.h:2770
# NV_vertex_program (GL/glext.h:2773)
GL_VERTEX_PROGRAM_NV = 34336 	# GL/glext.h:2774
GL_VERTEX_STATE_PROGRAM_NV = 34337 	# GL/glext.h:2775
GL_ATTRIB_ARRAY_SIZE_NV = 34339 	# GL/glext.h:2776
GL_ATTRIB_ARRAY_STRIDE_NV = 34340 	# GL/glext.h:2777
GL_ATTRIB_ARRAY_TYPE_NV = 34341 	# GL/glext.h:2778
GL_CURRENT_ATTRIB_NV = 34342 	# GL/glext.h:2779
GL_PROGRAM_LENGTH_NV = 34343 	# GL/glext.h:2780
GL_PROGRAM_STRING_NV = 34344 	# GL/glext.h:2781
GL_MODELVIEW_PROJECTION_NV = 34345 	# GL/glext.h:2782
GL_IDENTITY_NV = 34346 	# GL/glext.h:2783
GL_INVERSE_NV = 34347 	# GL/glext.h:2784
GL_TRANSPOSE_NV = 34348 	# GL/glext.h:2785
GL_INVERSE_TRANSPOSE_NV = 34349 	# GL/glext.h:2786
GL_MAX_TRACK_MATRIX_STACK_DEPTH_NV = 34350 	# GL/glext.h:2787
GL_MAX_TRACK_MATRICES_NV = 34351 	# GL/glext.h:2788
GL_MATRIX0_NV = 34352 	# GL/glext.h:2789
GL_MATRIX1_NV = 34353 	# GL/glext.h:2790
GL_MATRIX2_NV = 34354 	# GL/glext.h:2791
GL_MATRIX3_NV = 34355 	# GL/glext.h:2792
GL_MATRIX4_NV = 34356 	# GL/glext.h:2793
GL_MATRIX5_NV = 34357 	# GL/glext.h:2794
GL_MATRIX6_NV = 34358 	# GL/glext.h:2795
GL_MATRIX7_NV = 34359 	# GL/glext.h:2796
GL_CURRENT_MATRIX_STACK_DEPTH_NV = 34368 	# GL/glext.h:2797
GL_CURRENT_MATRIX_NV = 34369 	# GL/glext.h:2798
GL_VERTEX_PROGRAM_POINT_SIZE_NV = 34370 	# GL/glext.h:2799
GL_VERTEX_PROGRAM_TWO_SIDE_NV = 34371 	# GL/glext.h:2800
GL_PROGRAM_PARAMETER_NV = 34372 	# GL/glext.h:2801
GL_ATTRIB_ARRAY_POINTER_NV = 34373 	# GL/glext.h:2802
GL_PROGRAM_TARGET_NV = 34374 	# GL/glext.h:2803
GL_PROGRAM_RESIDENT_NV = 34375 	# GL/glext.h:2804
GL_TRACK_MATRIX_NV = 34376 	# GL/glext.h:2805
GL_TRACK_MATRIX_TRANSFORM_NV = 34377 	# GL/glext.h:2806
GL_VERTEX_PROGRAM_BINDING_NV = 34378 	# GL/glext.h:2807
GL_PROGRAM_ERROR_POSITION_NV = 34379 	# GL/glext.h:2808
GL_VERTEX_ATTRIB_ARRAY0_NV = 34384 	# GL/glext.h:2809
GL_VERTEX_ATTRIB_ARRAY1_NV = 34385 	# GL/glext.h:2810
GL_VERTEX_ATTRIB_ARRAY2_NV = 34386 	# GL/glext.h:2811
GL_VERTEX_ATTRIB_ARRAY3_NV = 34387 	# GL/glext.h:2812
GL_VERTEX_ATTRIB_ARRAY4_NV = 34388 	# GL/glext.h:2813
GL_VERTEX_ATTRIB_ARRAY5_NV = 34389 	# GL/glext.h:2814
GL_VERTEX_ATTRIB_ARRAY6_NV = 34390 	# GL/glext.h:2815
GL_VERTEX_ATTRIB_ARRAY7_NV = 34391 	# GL/glext.h:2816
GL_VERTEX_ATTRIB_ARRAY8_NV = 34392 	# GL/glext.h:2817
GL_VERTEX_ATTRIB_ARRAY9_NV = 34393 	# GL/glext.h:2818
GL_VERTEX_ATTRIB_ARRAY10_NV = 34394 	# GL/glext.h:2819
GL_VERTEX_ATTRIB_ARRAY11_NV = 34395 	# GL/glext.h:2820
GL_VERTEX_ATTRIB_ARRAY12_NV = 34396 	# GL/glext.h:2821
GL_VERTEX_ATTRIB_ARRAY13_NV = 34397 	# GL/glext.h:2822
GL_VERTEX_ATTRIB_ARRAY14_NV = 34398 	# GL/glext.h:2823
GL_VERTEX_ATTRIB_ARRAY15_NV = 34399 	# GL/glext.h:2824
GL_MAP1_VERTEX_ATTRIB0_4_NV = 34400 	# GL/glext.h:2825
GL_MAP1_VERTEX_ATTRIB1_4_NV = 34401 	# GL/glext.h:2826
GL_MAP1_VERTEX_ATTRIB2_4_NV = 34402 	# GL/glext.h:2827
GL_MAP1_VERTEX_ATTRIB3_4_NV = 34403 	# GL/glext.h:2828
GL_MAP1_VERTEX_ATTRIB4_4_NV = 34404 	# GL/glext.h:2829
GL_MAP1_VERTEX_ATTRIB5_4_NV = 34405 	# GL/glext.h:2830
GL_MAP1_VERTEX_ATTRIB6_4_NV = 34406 	# GL/glext.h:2831
GL_MAP1_VERTEX_ATTRIB7_4_NV = 34407 	# GL/glext.h:2832
GL_MAP1_VERTEX_ATTRIB8_4_NV = 34408 	# GL/glext.h:2833
GL_MAP1_VERTEX_ATTRIB9_4_NV = 34409 	# GL/glext.h:2834
GL_MAP1_VERTEX_ATTRIB10_4_NV = 34410 	# GL/glext.h:2835
GL_MAP1_VERTEX_ATTRIB11_4_NV = 34411 	# GL/glext.h:2836
GL_MAP1_VERTEX_ATTRIB12_4_NV = 34412 	# GL/glext.h:2837
GL_MAP1_VERTEX_ATTRIB13_4_NV = 34413 	# GL/glext.h:2838
GL_MAP1_VERTEX_ATTRIB14_4_NV = 34414 	# GL/glext.h:2839
GL_MAP1_VERTEX_ATTRIB15_4_NV = 34415 	# GL/glext.h:2840
GL_MAP2_VERTEX_ATTRIB0_4_NV = 34416 	# GL/glext.h:2841
GL_MAP2_VERTEX_ATTRIB1_4_NV = 34417 	# GL/glext.h:2842
GL_MAP2_VERTEX_ATTRIB2_4_NV = 34418 	# GL/glext.h:2843
GL_MAP2_VERTEX_ATTRIB3_4_NV = 34419 	# GL/glext.h:2844
GL_MAP2_VERTEX_ATTRIB4_4_NV = 34420 	# GL/glext.h:2845
GL_MAP2_VERTEX_ATTRIB5_4_NV = 34421 	# GL/glext.h:2846
GL_MAP2_VERTEX_ATTRIB6_4_NV = 34422 	# GL/glext.h:2847
GL_MAP2_VERTEX_ATTRIB7_4_NV = 34423 	# GL/glext.h:2848
GL_MAP2_VERTEX_ATTRIB8_4_NV = 34424 	# GL/glext.h:2849
GL_MAP2_VERTEX_ATTRIB9_4_NV = 34425 	# GL/glext.h:2850
GL_MAP2_VERTEX_ATTRIB10_4_NV = 34426 	# GL/glext.h:2851
GL_MAP2_VERTEX_ATTRIB11_4_NV = 34427 	# GL/glext.h:2852
GL_MAP2_VERTEX_ATTRIB12_4_NV = 34428 	# GL/glext.h:2853
GL_MAP2_VERTEX_ATTRIB13_4_NV = 34429 	# GL/glext.h:2854
GL_MAP2_VERTEX_ATTRIB14_4_NV = 34430 	# GL/glext.h:2855
GL_MAP2_VERTEX_ATTRIB15_4_NV = 34431 	# GL/glext.h:2856
# SGIX_texture_coordinate_clamp (GL/glext.h:2859)
GL_TEXTURE_MAX_CLAMP_S_SGIX = 33641 	# GL/glext.h:2860
GL_TEXTURE_MAX_CLAMP_T_SGIX = 33642 	# GL/glext.h:2861
GL_TEXTURE_MAX_CLAMP_R_SGIX = 33643 	# GL/glext.h:2862
# SGIX_scalebias_hint (GL/glext.h:2865)
GL_SCALEBIAS_HINT_SGIX = 33570 	# GL/glext.h:2866
# OML_interlace (GL/glext.h:2869)
GL_INTERLACE_OML = 35200 	# GL/glext.h:2870
GL_INTERLACE_READ_OML = 35201 	# GL/glext.h:2871
# OML_subsample (GL/glext.h:2874)
GL_FORMAT_SUBSAMPLE_24_24_OML = 35202 	# GL/glext.h:2875
GL_FORMAT_SUBSAMPLE_244_244_OML = 35203 	# GL/glext.h:2876
# OML_resample (GL/glext.h:2879)
GL_PACK_RESAMPLE_OML = 35204 	# GL/glext.h:2880
GL_UNPACK_RESAMPLE_OML = 35205 	# GL/glext.h:2881
GL_RESAMPLE_REPLICATE_OML = 35206 	# GL/glext.h:2882
GL_RESAMPLE_ZERO_FILL_OML = 35207 	# GL/glext.h:2883
GL_RESAMPLE_AVERAGE_OML = 35208 	# GL/glext.h:2884
GL_RESAMPLE_DECIMATE_OML = 35209 	# GL/glext.h:2885
# NV_copy_depth_to_color (GL/glext.h:2888)
GL_DEPTH_STENCIL_TO_RGBA_NV = 34926 	# GL/glext.h:2889
GL_DEPTH_STENCIL_TO_BGRA_NV = 34927 	# GL/glext.h:2890
# ATI_envmap_bumpmap (GL/glext.h:2893)
GL_BUMP_ROT_MATRIX_ATI = 34677 	# GL/glext.h:2894
GL_BUMP_ROT_MATRIX_SIZE_ATI = 34678 	# GL/glext.h:2895
GL_BUMP_NUM_TEX_UNITS_ATI = 34679 	# GL/glext.h:2896
GL_BUMP_TEX_UNITS_ATI = 34680 	# GL/glext.h:2897
GL_DUDV_ATI = 34681 	# GL/glext.h:2898
GL_DU8DV8_ATI = 34682 	# GL/glext.h:2899
GL_BUMP_ENVMAP_ATI = 34683 	# GL/glext.h:2900
GL_BUMP_TARGET_ATI = 34684 	# GL/glext.h:2901
# ATI_fragment_shader (GL/glext.h:2904)
GL_FRAGMENT_SHADER_ATI = 35104 	# GL/glext.h:2905
GL_REG_0_ATI = 35105 	# GL/glext.h:2906
GL_REG_1_ATI = 35106 	# GL/glext.h:2907
GL_REG_2_ATI = 35107 	# GL/glext.h:2908
GL_REG_3_ATI = 35108 	# GL/glext.h:2909
GL_REG_4_ATI = 35109 	# GL/glext.h:2910
GL_REG_5_ATI = 35110 	# GL/glext.h:2911
GL_REG_6_ATI = 35111 	# GL/glext.h:2912
GL_REG_7_ATI = 35112 	# GL/glext.h:2913
GL_REG_8_ATI = 35113 	# GL/glext.h:2914
GL_REG_9_ATI = 35114 	# GL/glext.h:2915
GL_REG_10_ATI = 35115 	# GL/glext.h:2916
GL_REG_11_ATI = 35116 	# GL/glext.h:2917
GL_REG_12_ATI = 35117 	# GL/glext.h:2918
GL_REG_13_ATI = 35118 	# GL/glext.h:2919
GL_REG_14_ATI = 35119 	# GL/glext.h:2920
GL_REG_15_ATI = 35120 	# GL/glext.h:2921
GL_REG_16_ATI = 35121 	# GL/glext.h:2922
GL_REG_17_ATI = 35122 	# GL/glext.h:2923
GL_REG_18_ATI = 35123 	# GL/glext.h:2924
GL_REG_19_ATI = 35124 	# GL/glext.h:2925
GL_REG_20_ATI = 35125 	# GL/glext.h:2926
GL_REG_21_ATI = 35126 	# GL/glext.h:2927
GL_REG_22_ATI = 35127 	# GL/glext.h:2928
GL_REG_23_ATI = 35128 	# GL/glext.h:2929
GL_REG_24_ATI = 35129 	# GL/glext.h:2930
GL_REG_25_ATI = 35130 	# GL/glext.h:2931
GL_REG_26_ATI = 35131 	# GL/glext.h:2932
GL_REG_27_ATI = 35132 	# GL/glext.h:2933
GL_REG_28_ATI = 35133 	# GL/glext.h:2934
GL_REG_29_ATI = 35134 	# GL/glext.h:2935
GL_REG_30_ATI = 35135 	# GL/glext.h:2936
GL_REG_31_ATI = 35136 	# GL/glext.h:2937
GL_CON_0_ATI = 35137 	# GL/glext.h:2938
GL_CON_1_ATI = 35138 	# GL/glext.h:2939
GL_CON_2_ATI = 35139 	# GL/glext.h:2940
GL_CON_3_ATI = 35140 	# GL/glext.h:2941
GL_CON_4_ATI = 35141 	# GL/glext.h:2942
GL_CON_5_ATI = 35142 	# GL/glext.h:2943
GL_CON_6_ATI = 35143 	# GL/glext.h:2944
GL_CON_7_ATI = 35144 	# GL/glext.h:2945
GL_CON_8_ATI = 35145 	# GL/glext.h:2946
GL_CON_9_ATI = 35146 	# GL/glext.h:2947
GL_CON_10_ATI = 35147 	# GL/glext.h:2948
GL_CON_11_ATI = 35148 	# GL/glext.h:2949
GL_CON_12_ATI = 35149 	# GL/glext.h:2950
GL_CON_13_ATI = 35150 	# GL/glext.h:2951
GL_CON_14_ATI = 35151 	# GL/glext.h:2952
GL_CON_15_ATI = 35152 	# GL/glext.h:2953
GL_CON_16_ATI = 35153 	# GL/glext.h:2954
GL_CON_17_ATI = 35154 	# GL/glext.h:2955
GL_CON_18_ATI = 35155 	# GL/glext.h:2956
GL_CON_19_ATI = 35156 	# GL/glext.h:2957
GL_CON_20_ATI = 35157 	# GL/glext.h:2958
GL_CON_21_ATI = 35158 	# GL/glext.h:2959
GL_CON_22_ATI = 35159 	# GL/glext.h:2960
GL_CON_23_ATI = 35160 	# GL/glext.h:2961
GL_CON_24_ATI = 35161 	# GL/glext.h:2962
GL_CON_25_ATI = 35162 	# GL/glext.h:2963
GL_CON_26_ATI = 35163 	# GL/glext.h:2964
GL_CON_27_ATI = 35164 	# GL/glext.h:2965
GL_CON_28_ATI = 35165 	# GL/glext.h:2966
GL_CON_29_ATI = 35166 	# GL/glext.h:2967
GL_CON_30_ATI = 35167 	# GL/glext.h:2968
GL_CON_31_ATI = 35168 	# GL/glext.h:2969
GL_MOV_ATI = 35169 	# GL/glext.h:2970
GL_ADD_ATI = 35171 	# GL/glext.h:2971
GL_MUL_ATI = 35172 	# GL/glext.h:2972
GL_SUB_ATI = 35173 	# GL/glext.h:2973
GL_DOT3_ATI = 35174 	# GL/glext.h:2974
GL_DOT4_ATI = 35175 	# GL/glext.h:2975
GL_MAD_ATI = 35176 	# GL/glext.h:2976
GL_LERP_ATI = 35177 	# GL/glext.h:2977
GL_CND_ATI = 35178 	# GL/glext.h:2978
GL_CND0_ATI = 35179 	# GL/glext.h:2979
GL_DOT2_ADD_ATI = 35180 	# GL/glext.h:2980
GL_SECONDARY_INTERPOLATOR_ATI = 35181 	# GL/glext.h:2981
GL_NUM_FRAGMENT_REGISTERS_ATI = 35182 	# GL/glext.h:2982
GL_NUM_FRAGMENT_CONSTANTS_ATI = 35183 	# GL/glext.h:2983
GL_NUM_PASSES_ATI = 35184 	# GL/glext.h:2984
GL_NUM_INSTRUCTIONS_PER_PASS_ATI = 35185 	# GL/glext.h:2985
GL_NUM_INSTRUCTIONS_TOTAL_ATI = 35186 	# GL/glext.h:2986
GL_NUM_INPUT_INTERPOLATOR_COMPONENTS_ATI = 35187 	# GL/glext.h:2987
GL_NUM_LOOPBACK_COMPONENTS_ATI = 35188 	# GL/glext.h:2988
GL_COLOR_ALPHA_PAIRING_ATI = 35189 	# GL/glext.h:2989
GL_SWIZZLE_STR_ATI = 35190 	# GL/glext.h:2990
GL_SWIZZLE_STQ_ATI = 35191 	# GL/glext.h:2991
GL_SWIZZLE_STR_DR_ATI = 35192 	# GL/glext.h:2992
GL_SWIZZLE_STQ_DQ_ATI = 35193 	# GL/glext.h:2993
GL_SWIZZLE_STRQ_ATI = 35194 	# GL/glext.h:2994
GL_SWIZZLE_STRQ_DQ_ATI = 35195 	# GL/glext.h:2995
GL_RED_BIT_ATI = 1 	# GL/glext.h:2996
GL_GREEN_BIT_ATI = 2 	# GL/glext.h:2997
GL_BLUE_BIT_ATI = 4 	# GL/glext.h:2998
GL_2X_BIT_ATI = 1 	# GL/glext.h:2999
GL_4X_BIT_ATI = 2 	# GL/glext.h:3000
GL_8X_BIT_ATI = 4 	# GL/glext.h:3001
GL_HALF_BIT_ATI = 8 	# GL/glext.h:3002
GL_QUARTER_BIT_ATI = 16 	# GL/glext.h:3003
GL_EIGHTH_BIT_ATI = 32 	# GL/glext.h:3004
GL_SATURATE_BIT_ATI = 64 	# GL/glext.h:3005
GL_COMP_BIT_ATI = 2 	# GL/glext.h:3006
GL_NEGATE_BIT_ATI = 4 	# GL/glext.h:3007
GL_BIAS_BIT_ATI = 8 	# GL/glext.h:3008
# ATI_pn_triangles (GL/glext.h:3011)
GL_PN_TRIANGLES_ATI = 34800 	# GL/glext.h:3012
GL_MAX_PN_TRIANGLES_TESSELATION_LEVEL_ATI = 34801 	# GL/glext.h:3013
GL_PN_TRIANGLES_POINT_MODE_ATI = 34802 	# GL/glext.h:3014
GL_PN_TRIANGLES_NORMAL_MODE_ATI = 34803 	# GL/glext.h:3015
GL_PN_TRIANGLES_TESSELATION_LEVEL_ATI = 34804 	# GL/glext.h:3016
GL_PN_TRIANGLES_POINT_MODE_LINEAR_ATI = 34805 	# GL/glext.h:3017
GL_PN_TRIANGLES_POINT_MODE_CUBIC_ATI = 34806 	# GL/glext.h:3018
GL_PN_TRIANGLES_NORMAL_MODE_LINEAR_ATI = 34807 	# GL/glext.h:3019
GL_PN_TRIANGLES_NORMAL_MODE_QUADRATIC_ATI = 34808 	# GL/glext.h:3020
# ATI_vertex_array_object (GL/glext.h:3023)
GL_STATIC_ATI = 34656 	# GL/glext.h:3024
GL_DYNAMIC_ATI = 34657 	# GL/glext.h:3025
GL_PRESERVE_ATI = 34658 	# GL/glext.h:3026
GL_DISCARD_ATI = 34659 	# GL/glext.h:3027
GL_OBJECT_BUFFER_SIZE_ATI = 34660 	# GL/glext.h:3028
GL_OBJECT_BUFFER_USAGE_ATI = 34661 	# GL/glext.h:3029
GL_ARRAY_OBJECT_BUFFER_ATI = 34662 	# GL/glext.h:3030
GL_ARRAY_OBJECT_OFFSET_ATI = 34663 	# GL/glext.h:3031
# EXT_vertex_shader (GL/glext.h:3034)
GL_VERTEX_SHADER_EXT = 34688 	# GL/glext.h:3035
GL_VERTEX_SHADER_BINDING_EXT = 34689 	# GL/glext.h:3036
GL_OP_INDEX_EXT = 34690 	# GL/glext.h:3037
GL_OP_NEGATE_EXT = 34691 	# GL/glext.h:3038
GL_OP_DOT3_EXT = 34692 	# GL/glext.h:3039
GL_OP_DOT4_EXT = 34693 	# GL/glext.h:3040
GL_OP_MUL_EXT = 34694 	# GL/glext.h:3041
GL_OP_ADD_EXT = 34695 	# GL/glext.h:3042
GL_OP_MADD_EXT = 34696 	# GL/glext.h:3043
GL_OP_FRAC_EXT = 34697 	# GL/glext.h:3044
GL_OP_MAX_EXT = 34698 	# GL/glext.h:3045
GL_OP_MIN_EXT = 34699 	# GL/glext.h:3046
GL_OP_SET_GE_EXT = 34700 	# GL/glext.h:3047
GL_OP_SET_LT_EXT = 34701 	# GL/glext.h:3048
GL_OP_CLAMP_EXT = 34702 	# GL/glext.h:3049
GL_OP_FLOOR_EXT = 34703 	# GL/glext.h:3050
GL_OP_ROUND_EXT = 34704 	# GL/glext.h:3051
GL_OP_EXP_BASE_2_EXT = 34705 	# GL/glext.h:3052
GL_OP_LOG_BASE_2_EXT = 34706 	# GL/glext.h:3053
GL_OP_POWER_EXT = 34707 	# GL/glext.h:3054
GL_OP_RECIP_EXT = 34708 	# GL/glext.h:3055
GL_OP_RECIP_SQRT_EXT = 34709 	# GL/glext.h:3056
GL_OP_SUB_EXT = 34710 	# GL/glext.h:3057
GL_OP_CROSS_PRODUCT_EXT = 34711 	# GL/glext.h:3058
GL_OP_MULTIPLY_MATRIX_EXT = 34712 	# GL/glext.h:3059
GL_OP_MOV_EXT = 34713 	# GL/glext.h:3060
GL_OUTPUT_VERTEX_EXT = 34714 	# GL/glext.h:3061
GL_OUTPUT_COLOR0_EXT = 34715 	# GL/glext.h:3062
GL_OUTPUT_COLOR1_EXT = 34716 	# GL/glext.h:3063
GL_OUTPUT_TEXTURE_COORD0_EXT = 34717 	# GL/glext.h:3064
GL_OUTPUT_TEXTURE_COORD1_EXT = 34718 	# GL/glext.h:3065
GL_OUTPUT_TEXTURE_COORD2_EXT = 34719 	# GL/glext.h:3066
GL_OUTPUT_TEXTURE_COORD3_EXT = 34720 	# GL/glext.h:3067
GL_OUTPUT_TEXTURE_COORD4_EXT = 34721 	# GL/glext.h:3068
GL_OUTPUT_TEXTURE_COORD5_EXT = 34722 	# GL/glext.h:3069
GL_OUTPUT_TEXTURE_COORD6_EXT = 34723 	# GL/glext.h:3070
GL_OUTPUT_TEXTURE_COORD7_EXT = 34724 	# GL/glext.h:3071
GL_OUTPUT_TEXTURE_COORD8_EXT = 34725 	# GL/glext.h:3072
GL_OUTPUT_TEXTURE_COORD9_EXT = 34726 	# GL/glext.h:3073
GL_OUTPUT_TEXTURE_COORD10_EXT = 34727 	# GL/glext.h:3074
GL_OUTPUT_TEXTURE_COORD11_EXT = 34728 	# GL/glext.h:3075
GL_OUTPUT_TEXTURE_COORD12_EXT = 34729 	# GL/glext.h:3076
GL_OUTPUT_TEXTURE_COORD13_EXT = 34730 	# GL/glext.h:3077
GL_OUTPUT_TEXTURE_COORD14_EXT = 34731 	# GL/glext.h:3078
GL_OUTPUT_TEXTURE_COORD15_EXT = 34732 	# GL/glext.h:3079
GL_OUTPUT_TEXTURE_COORD16_EXT = 34733 	# GL/glext.h:3080
GL_OUTPUT_TEXTURE_COORD17_EXT = 34734 	# GL/glext.h:3081
GL_OUTPUT_TEXTURE_COORD18_EXT = 34735 	# GL/glext.h:3082
GL_OUTPUT_TEXTURE_COORD19_EXT = 34736 	# GL/glext.h:3083
GL_OUTPUT_TEXTURE_COORD20_EXT = 34737 	# GL/glext.h:3084
GL_OUTPUT_TEXTURE_COORD21_EXT = 34738 	# GL/glext.h:3085
GL_OUTPUT_TEXTURE_COORD22_EXT = 34739 	# GL/glext.h:3086
GL_OUTPUT_TEXTURE_COORD23_EXT = 34740 	# GL/glext.h:3087
GL_OUTPUT_TEXTURE_COORD24_EXT = 34741 	# GL/glext.h:3088
GL_OUTPUT_TEXTURE_COORD25_EXT = 34742 	# GL/glext.h:3089
GL_OUTPUT_TEXTURE_COORD26_EXT = 34743 	# GL/glext.h:3090
GL_OUTPUT_TEXTURE_COORD27_EXT = 34744 	# GL/glext.h:3091
GL_OUTPUT_TEXTURE_COORD28_EXT = 34745 	# GL/glext.h:3092
GL_OUTPUT_TEXTURE_COORD29_EXT = 34746 	# GL/glext.h:3093
GL_OUTPUT_TEXTURE_COORD30_EXT = 34747 	# GL/glext.h:3094
GL_OUTPUT_TEXTURE_COORD31_EXT = 34748 	# GL/glext.h:3095
GL_OUTPUT_FOG_EXT = 34749 	# GL/glext.h:3096
GL_SCALAR_EXT = 34750 	# GL/glext.h:3097
GL_VECTOR_EXT = 34751 	# GL/glext.h:3098
GL_MATRIX_EXT = 34752 	# GL/glext.h:3099
GL_VARIANT_EXT = 34753 	# GL/glext.h:3100
GL_INVARIANT_EXT = 34754 	# GL/glext.h:3101
GL_LOCAL_CONSTANT_EXT = 34755 	# GL/glext.h:3102
GL_LOCAL_EXT = 34756 	# GL/glext.h:3103
GL_MAX_VERTEX_SHADER_INSTRUCTIONS_EXT = 34757 	# GL/glext.h:3104
GL_MAX_VERTEX_SHADER_VARIANTS_EXT = 34758 	# GL/glext.h:3105
GL_MAX_VERTEX_SHADER_INVARIANTS_EXT = 34759 	# GL/glext.h:3106
GL_MAX_VERTEX_SHADER_LOCAL_CONSTANTS_EXT = 34760 	# GL/glext.h:3107
GL_MAX_VERTEX_SHADER_LOCALS_EXT = 34761 	# GL/glext.h:3108
GL_MAX_OPTIMIZED_VERTEX_SHADER_INSTRUCTIONS_EXT = 34762 	# GL/glext.h:3109
GL_MAX_OPTIMIZED_VERTEX_SHADER_VARIANTS_EXT = 34763 	# GL/glext.h:3110
GL_MAX_OPTIMIZED_VERTEX_SHADER_LOCAL_CONSTANTS_EXT = 34764 	# GL/glext.h:3111
GL_MAX_OPTIMIZED_VERTEX_SHADER_INVARIANTS_EXT = 34765 	# GL/glext.h:3112
GL_MAX_OPTIMIZED_VERTEX_SHADER_LOCALS_EXT = 34766 	# GL/glext.h:3113
GL_VERTEX_SHADER_INSTRUCTIONS_EXT = 34767 	# GL/glext.h:3114
GL_VERTEX_SHADER_VARIANTS_EXT = 34768 	# GL/glext.h:3115
GL_VERTEX_SHADER_INVARIANTS_EXT = 34769 	# GL/glext.h:3116
GL_VERTEX_SHADER_LOCAL_CONSTANTS_EXT = 34770 	# GL/glext.h:3117
GL_VERTEX_SHADER_LOCALS_EXT = 34771 	# GL/glext.h:3118
GL_VERTEX_SHADER_OPTIMIZED_EXT = 34772 	# GL/glext.h:3119
GL_X_EXT = 34773 	# GL/glext.h:3120
GL_Y_EXT = 34774 	# GL/glext.h:3121
GL_Z_EXT = 34775 	# GL/glext.h:3122
GL_W_EXT = 34776 	# GL/glext.h:3123
GL_NEGATIVE_X_EXT = 34777 	# GL/glext.h:3124
GL_NEGATIVE_Y_EXT = 34778 	# GL/glext.h:3125
GL_NEGATIVE_Z_EXT = 34779 	# GL/glext.h:3126
GL_NEGATIVE_W_EXT = 34780 	# GL/glext.h:3127
GL_ZERO_EXT = 34781 	# GL/glext.h:3128
GL_ONE_EXT = 34782 	# GL/glext.h:3129
GL_NEGATIVE_ONE_EXT = 34783 	# GL/glext.h:3130
GL_NORMALIZED_RANGE_EXT = 34784 	# GL/glext.h:3131
GL_FULL_RANGE_EXT = 34785 	# GL/glext.h:3132
GL_CURRENT_VERTEX_EXT = 34786 	# GL/glext.h:3133
GL_MVP_MATRIX_EXT = 34787 	# GL/glext.h:3134
GL_VARIANT_VALUE_EXT = 34788 	# GL/glext.h:3135
GL_VARIANT_DATATYPE_EXT = 34789 	# GL/glext.h:3136
GL_VARIANT_ARRAY_STRIDE_EXT = 34790 	# GL/glext.h:3137
GL_VARIANT_ARRAY_TYPE_EXT = 34791 	# GL/glext.h:3138
GL_VARIANT_ARRAY_EXT = 34792 	# GL/glext.h:3139
GL_VARIANT_ARRAY_POINTER_EXT = 34793 	# GL/glext.h:3140
GL_INVARIANT_VALUE_EXT = 34794 	# GL/glext.h:3141
GL_INVARIANT_DATATYPE_EXT = 34795 	# GL/glext.h:3142
GL_LOCAL_CONSTANT_VALUE_EXT = 34796 	# GL/glext.h:3143
GL_LOCAL_CONSTANT_DATATYPE_EXT = 34797 	# GL/glext.h:3144
# ATI_vertex_streams (GL/glext.h:3147)
GL_MAX_VERTEX_STREAMS_ATI = 34667 	# GL/glext.h:3148
GL_VERTEX_STREAM0_ATI = 34668 	# GL/glext.h:3149
GL_VERTEX_STREAM1_ATI = 34669 	# GL/glext.h:3150
GL_VERTEX_STREAM2_ATI = 34670 	# GL/glext.h:3151
GL_VERTEX_STREAM3_ATI = 34671 	# GL/glext.h:3152
GL_VERTEX_STREAM4_ATI = 34672 	# GL/glext.h:3153
GL_VERTEX_STREAM5_ATI = 34673 	# GL/glext.h:3154
GL_VERTEX_STREAM6_ATI = 34674 	# GL/glext.h:3155
GL_VERTEX_STREAM7_ATI = 34675 	# GL/glext.h:3156
GL_VERTEX_SOURCE_ATI = 34676 	# GL/glext.h:3157
# ATI_element_array (GL/glext.h:3160)
GL_ELEMENT_ARRAY_ATI = 34664 	# GL/glext.h:3161
GL_ELEMENT_ARRAY_TYPE_ATI = 34665 	# GL/glext.h:3162
GL_ELEMENT_ARRAY_POINTER_ATI = 34666 	# GL/glext.h:3163
# SUN_mesh_array (GL/glext.h:3166)
GL_QUAD_MESH_SUN = 34324 	# GL/glext.h:3167
GL_TRIANGLE_MESH_SUN = 34325 	# GL/glext.h:3168
# SUN_slice_accum (GL/glext.h:3171)
GL_SLICE_ACCUM_SUN = 34252 	# GL/glext.h:3172
# NV_multisample_filter_hint (GL/glext.h:3175)
GL_MULTISAMPLE_FILTER_HINT_NV = 34100 	# GL/glext.h:3176
# NV_depth_clamp (GL/glext.h:3179)
GL_DEPTH_CLAMP_NV = 34383 	# GL/glext.h:3180
# NV_occlusion_query (GL/glext.h:3183)
GL_PIXEL_COUNTER_BITS_NV = 34916 	# GL/glext.h:3184
GL_CURRENT_OCCLUSION_QUERY_ID_NV = 34917 	# GL/glext.h:3185
GL_PIXEL_COUNT_NV = 34918 	# GL/glext.h:3186
GL_PIXEL_COUNT_AVAILABLE_NV = 34919 	# GL/glext.h:3187
# NV_point_sprite (GL/glext.h:3190)
GL_POINT_SPRITE_NV = 34913 	# GL/glext.h:3191
GL_COORD_REPLACE_NV = 34914 	# GL/glext.h:3192
GL_POINT_SPRITE_R_MODE_NV = 34915 	# GL/glext.h:3193
# NV_texture_shader3 (GL/glext.h:3196)
GL_OFFSET_PROJECTIVE_TEXTURE_2D_NV = 34896 	# GL/glext.h:3197
GL_OFFSET_PROJECTIVE_TEXTURE_2D_SCALE_NV = 34897 	# GL/glext.h:3198
GL_OFFSET_PROJECTIVE_TEXTURE_RECTANGLE_NV = 34898 	# GL/glext.h:3199
GL_OFFSET_PROJECTIVE_TEXTURE_RECTANGLE_SCALE_NV = 34899 	# GL/glext.h:3200
GL_OFFSET_HILO_TEXTURE_2D_NV = 34900 	# GL/glext.h:3201
GL_OFFSET_HILO_TEXTURE_RECTANGLE_NV = 34901 	# GL/glext.h:3202
GL_OFFSET_HILO_PROJECTIVE_TEXTURE_2D_NV = 34902 	# GL/glext.h:3203
GL_OFFSET_HILO_PROJECTIVE_TEXTURE_RECTANGLE_NV = 34903 	# GL/glext.h:3204
GL_DEPENDENT_HILO_TEXTURE_2D_NV = 34904 	# GL/glext.h:3205
GL_DEPENDENT_RGB_TEXTURE_3D_NV = 34905 	# GL/glext.h:3206
GL_DEPENDENT_RGB_TEXTURE_CUBE_MAP_NV = 34906 	# GL/glext.h:3207
GL_DOT_PRODUCT_PASS_THROUGH_NV = 34907 	# GL/glext.h:3208
GL_DOT_PRODUCT_TEXTURE_1D_NV = 34908 	# GL/glext.h:3209
GL_DOT_PRODUCT_AFFINE_DEPTH_REPLACE_NV = 34909 	# GL/glext.h:3210
GL_HILO8_NV = 34910 	# GL/glext.h:3211
GL_SIGNED_HILO8_NV = 34911 	# GL/glext.h:3212
GL_FORCE_BLUE_TO_ONE_NV = 34912 	# GL/glext.h:3213
# NV_vertex_program1_1 (GL/glext.h:3216)
# EXT_shadow_funcs (GL/glext.h:3219)
# EXT_stencil_two_side (GL/glext.h:3222)
GL_STENCIL_TEST_TWO_SIDE_EXT = 35088 	# GL/glext.h:3223
GL_ACTIVE_STENCIL_FACE_EXT = 35089 	# GL/glext.h:3224
# ATI_text_fragment_shader (GL/glext.h:3227)
GL_TEXT_FRAGMENT_SHADER_ATI = 33280 	# GL/glext.h:3228
# APPLE_client_storage (GL/glext.h:3231)
GL_UNPACK_CLIENT_STORAGE_APPLE = 34226 	# GL/glext.h:3232
# APPLE_element_array (GL/glext.h:3235)
GL_ELEMENT_ARRAY_APPLE = 34664 	# GL/glext.h:3236
GL_ELEMENT_ARRAY_TYPE_APPLE = 34665 	# GL/glext.h:3237
GL_ELEMENT_ARRAY_POINTER_APPLE = 34666 	# GL/glext.h:3238
# APPLE_fence (GL/glext.h:3241)
GL_DRAW_PIXELS_APPLE = 35338 	# GL/glext.h:3242
GL_FENCE_APPLE = 35339 	# GL/glext.h:3243
# APPLE_vertex_array_object (GL/glext.h:3246)
GL_VERTEX_ARRAY_BINDING_APPLE = 34229 	# GL/glext.h:3247
# APPLE_vertex_array_range (GL/glext.h:3250)
GL_VERTEX_ARRAY_RANGE_APPLE = 34077 	# GL/glext.h:3251
GL_VERTEX_ARRAY_RANGE_LENGTH_APPLE = 34078 	# GL/glext.h:3252
GL_VERTEX_ARRAY_STORAGE_HINT_APPLE = 34079 	# GL/glext.h:3253
GL_VERTEX_ARRAY_RANGE_POINTER_APPLE = 34081 	# GL/glext.h:3254
GL_STORAGE_CACHED_APPLE = 34238 	# GL/glext.h:3255
GL_STORAGE_SHARED_APPLE = 34239 	# GL/glext.h:3256
# APPLE_ycbcr_422 (GL/glext.h:3259)
GL_YCBCR_422_APPLE = 34233 	# GL/glext.h:3260
GL_UNSIGNED_SHORT_8_8_APPLE = 34234 	# GL/glext.h:3261
GL_UNSIGNED_SHORT_8_8_REV_APPLE = 34235 	# GL/glext.h:3262
# S3_s3tc (GL/glext.h:3265)
GL_RGB_S3TC = 33696 	# GL/glext.h:3266
GL_RGB4_S3TC = 33697 	# GL/glext.h:3267
GL_RGBA_S3TC = 33698 	# GL/glext.h:3268
GL_RGBA4_S3TC = 33699 	# GL/glext.h:3269
# ATI_draw_buffers (GL/glext.h:3272)
GL_MAX_DRAW_BUFFERS_ATI = 34852 	# GL/glext.h:3273
GL_DRAW_BUFFER0_ATI = 34853 	# GL/glext.h:3274
GL_DRAW_BUFFER1_ATI = 34854 	# GL/glext.h:3275
GL_DRAW_BUFFER2_ATI = 34855 	# GL/glext.h:3276
GL_DRAW_BUFFER3_ATI = 34856 	# GL/glext.h:3277
GL_DRAW_BUFFER4_ATI = 34857 	# GL/glext.h:3278
GL_DRAW_BUFFER5_ATI = 34858 	# GL/glext.h:3279
GL_DRAW_BUFFER6_ATI = 34859 	# GL/glext.h:3280
GL_DRAW_BUFFER7_ATI = 34860 	# GL/glext.h:3281
GL_DRAW_BUFFER8_ATI = 34861 	# GL/glext.h:3282
GL_DRAW_BUFFER9_ATI = 34862 	# GL/glext.h:3283
GL_DRAW_BUFFER10_ATI = 34863 	# GL/glext.h:3284
GL_DRAW_BUFFER11_ATI = 34864 	# GL/glext.h:3285
GL_DRAW_BUFFER12_ATI = 34865 	# GL/glext.h:3286
GL_DRAW_BUFFER13_ATI = 34866 	# GL/glext.h:3287
GL_DRAW_BUFFER14_ATI = 34867 	# GL/glext.h:3288
GL_DRAW_BUFFER15_ATI = 34868 	# GL/glext.h:3289
# ATI_pixel_format_float (GL/glext.h:3292)
GL_TYPE_RGBA_FLOAT_ATI = 34848 	# GL/glext.h:3293
GL_COLOR_CLEAR_UNCLAMPED_VALUE_ATI = 34869 	# GL/glext.h:3294
# ATI_texture_env_combine3 (GL/glext.h:3297)
GL_MODULATE_ADD_ATI = 34628 	# GL/glext.h:3298
GL_MODULATE_SIGNED_ADD_ATI = 34629 	# GL/glext.h:3299
GL_MODULATE_SUBTRACT_ATI = 34630 	# GL/glext.h:3300
# ATI_texture_float (GL/glext.h:3303)
GL_RGBA_FLOAT32_ATI = 34836 	# GL/glext.h:3304
GL_RGB_FLOAT32_ATI = 34837 	# GL/glext.h:3305
GL_ALPHA_FLOAT32_ATI = 34838 	# GL/glext.h:3306
GL_INTENSITY_FLOAT32_ATI = 34839 	# GL/glext.h:3307
GL_LUMINANCE_FLOAT32_ATI = 34840 	# GL/glext.h:3308
GL_LUMINANCE_ALPHA_FLOAT32_ATI = 34841 	# GL/glext.h:3309
GL_RGBA_FLOAT16_ATI = 34842 	# GL/glext.h:3310
GL_RGB_FLOAT16_ATI = 34843 	# GL/glext.h:3311
GL_ALPHA_FLOAT16_ATI = 34844 	# GL/glext.h:3312
GL_INTENSITY_FLOAT16_ATI = 34845 	# GL/glext.h:3313
GL_LUMINANCE_FLOAT16_ATI = 34846 	# GL/glext.h:3314
GL_LUMINANCE_ALPHA_FLOAT16_ATI = 34847 	# GL/glext.h:3315
# NV_float_buffer (GL/glext.h:3318)
GL_FLOAT_R_NV = 34944 	# GL/glext.h:3319
GL_FLOAT_RG_NV = 34945 	# GL/glext.h:3320
GL_FLOAT_RGB_NV = 34946 	# GL/glext.h:3321
GL_FLOAT_RGBA_NV = 34947 	# GL/glext.h:3322
GL_FLOAT_R16_NV = 34948 	# GL/glext.h:3323
GL_FLOAT_R32_NV = 34949 	# GL/glext.h:3324
GL_FLOAT_RG16_NV = 34950 	# GL/glext.h:3325
GL_FLOAT_RG32_NV = 34951 	# GL/glext.h:3326
GL_FLOAT_RGB16_NV = 34952 	# GL/glext.h:3327
GL_FLOAT_RGB32_NV = 34953 	# GL/glext.h:3328
GL_FLOAT_RGBA16_NV = 34954 	# GL/glext.h:3329
GL_FLOAT_RGBA32_NV = 34955 	# GL/glext.h:3330
GL_TEXTURE_FLOAT_COMPONENTS_NV = 34956 	# GL/glext.h:3331
GL_FLOAT_CLEAR_COLOR_VALUE_NV = 34957 	# GL/glext.h:3332
GL_FLOAT_RGBA_MODE_NV = 34958 	# GL/glext.h:3333
# NV_fragment_program (GL/glext.h:3336)
GL_MAX_FRAGMENT_PROGRAM_LOCAL_PARAMETERS_NV = 34920 	# GL/glext.h:3337
GL_FRAGMENT_PROGRAM_NV = 34928 	# GL/glext.h:3338
GL_MAX_TEXTURE_COORDS_NV = 34929 	# GL/glext.h:3339
GL_MAX_TEXTURE_IMAGE_UNITS_NV = 34930 	# GL/glext.h:3340
GL_FRAGMENT_PROGRAM_BINDING_NV = 34931 	# GL/glext.h:3341
GL_PROGRAM_ERROR_STRING_NV = 34932 	# GL/glext.h:3342
# NV_half_float (GL/glext.h:3345)
GL_HALF_FLOAT_NV = 5131 	# GL/glext.h:3346
# NV_pixel_data_range (GL/glext.h:3349)
GL_WRITE_PIXEL_DATA_RANGE_NV = 34936 	# GL/glext.h:3350
GL_READ_PIXEL_DATA_RANGE_NV = 34937 	# GL/glext.h:3351
GL_WRITE_PIXEL_DATA_RANGE_LENGTH_NV = 34938 	# GL/glext.h:3352
GL_READ_PIXEL_DATA_RANGE_LENGTH_NV = 34939 	# GL/glext.h:3353
GL_WRITE_PIXEL_DATA_RANGE_POINTER_NV = 34940 	# GL/glext.h:3354
GL_READ_PIXEL_DATA_RANGE_POINTER_NV = 34941 	# GL/glext.h:3355
# NV_primitive_restart (GL/glext.h:3358)
GL_PRIMITIVE_RESTART_NV = 34136 	# GL/glext.h:3359
GL_PRIMITIVE_RESTART_INDEX_NV = 34137 	# GL/glext.h:3360
# NV_texture_expand_normal (GL/glext.h:3363)
GL_TEXTURE_UNSIGNED_REMAP_MODE_NV = 34959 	# GL/glext.h:3364
# NV_vertex_program2 (GL/glext.h:3367)
# ATI_map_object_buffer (GL/glext.h:3370)
# ATI_separate_stencil (GL/glext.h:3373)
GL_STENCIL_BACK_FUNC_ATI = 34816 	# GL/glext.h:3374
GL_STENCIL_BACK_FAIL_ATI = 34817 	# GL/glext.h:3375
GL_STENCIL_BACK_PASS_DEPTH_FAIL_ATI = 34818 	# GL/glext.h:3376
GL_STENCIL_BACK_PASS_DEPTH_PASS_ATI = 34819 	# GL/glext.h:3377
# ATI_vertex_attrib_array_object (GL/glext.h:3380)
# OES_read_format (GL/glext.h:3383)
GL_IMPLEMENTATION_COLOR_READ_TYPE_OES = 35738 	# GL/glext.h:3384
GL_IMPLEMENTATION_COLOR_READ_FORMAT_OES = 35739 	# GL/glext.h:3385
# EXT_depth_bounds_test (GL/glext.h:3388)
GL_DEPTH_BOUNDS_TEST_EXT = 34960 	# GL/glext.h:3389
GL_DEPTH_BOUNDS_EXT = 34961 	# GL/glext.h:3390
# EXT_texture_mirror_clamp (GL/glext.h:3393)
GL_MIRROR_CLAMP_EXT = 34626 	# GL/glext.h:3394
GL_MIRROR_CLAMP_TO_EDGE_EXT = 34627 	# GL/glext.h:3395
GL_MIRROR_CLAMP_TO_BORDER_EXT = 35090 	# GL/glext.h:3396
# EXT_blend_equation_separate (GL/glext.h:3399)
GL_BLEND_EQUATION_RGB_EXT = 32777 	# GL/glext.h:3400
GL_BLEND_EQUATION_ALPHA_EXT = 34877 	# GL/glext.h:3401
# MESA_pack_invert (GL/glext.h:3404)
GL_PACK_INVERT_MESA = 34648 	# GL/glext.h:3405
# MESA_ycbcr_texture (GL/glext.h:3408)
GL_UNSIGNED_SHORT_8_8_MESA = 34234 	# GL/glext.h:3409
GL_UNSIGNED_SHORT_8_8_REV_MESA = 34235 	# GL/glext.h:3410
GL_YCBCR_MESA = 34647 	# GL/glext.h:3411
# EXT_pixel_buffer_object (GL/glext.h:3414)
GL_PIXEL_PACK_BUFFER_EXT = 35051 	# GL/glext.h:3415
GL_PIXEL_UNPACK_BUFFER_EXT = 35052 	# GL/glext.h:3416
GL_PIXEL_PACK_BUFFER_BINDING_EXT = 35053 	# GL/glext.h:3417
GL_PIXEL_UNPACK_BUFFER_BINDING_EXT = 35055 	# GL/glext.h:3418
# NV_fragment_program_option (GL/glext.h:3421)
# NV_fragment_program2 (GL/glext.h:3424)
GL_MAX_PROGRAM_EXEC_INSTRUCTIONS_NV = 35060 	# GL/glext.h:3425
GL_MAX_PROGRAM_CALL_DEPTH_NV = 35061 	# GL/glext.h:3426
GL_MAX_PROGRAM_IF_DEPTH_NV = 35062 	# GL/glext.h:3427
GL_MAX_PROGRAM_LOOP_DEPTH_NV = 35063 	# GL/glext.h:3428
GL_MAX_PROGRAM_LOOP_COUNT_NV = 35064 	# GL/glext.h:3429
# NV_vertex_program2_option (GL/glext.h:3432)
# NV_vertex_program3 (GL/glext.h:3437)
# EXT_framebuffer_object (GL/glext.h:3441)
GL_INVALID_FRAMEBUFFER_OPERATION_EXT = 1286 	# GL/glext.h:3442
GL_MAX_RENDERBUFFER_SIZE_EXT = 34024 	# GL/glext.h:3443
GL_FRAMEBUFFER_BINDING_EXT = 36006 	# GL/glext.h:3444
GL_RENDERBUFFER_BINDING_EXT = 36007 	# GL/glext.h:3445
GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE_EXT = 36048 	# GL/glext.h:3446
GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME_EXT = 36049 	# GL/glext.h:3447
GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL_EXT = 36050 	# GL/glext.h:3448
GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE_EXT = 36051 	# GL/glext.h:3449
GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_3D_ZOFFSET_EXT = 36052 	# GL/glext.h:3450
GL_FRAMEBUFFER_COMPLETE_EXT = 36053 	# GL/glext.h:3451
GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT_EXT = 36054 	# GL/glext.h:3452
GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT_EXT = 36055 	# GL/glext.h:3453
GL_FRAMEBUFFER_INCOMPLETE_DUPLICATE_ATTACHMENT_EXT = 36056 	# GL/glext.h:3454
GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_EXT = 36057 	# GL/glext.h:3455
GL_FRAMEBUFFER_INCOMPLETE_FORMATS_EXT = 36058 	# GL/glext.h:3456
GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER_EXT = 36059 	# GL/glext.h:3457
GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER_EXT = 36060 	# GL/glext.h:3458
GL_FRAMEBUFFER_UNSUPPORTED_EXT = 36061 	# GL/glext.h:3459
GL_MAX_COLOR_ATTACHMENTS_EXT = 36063 	# GL/glext.h:3460
GL_COLOR_ATTACHMENT0_EXT = 36064 	# GL/glext.h:3461
GL_COLOR_ATTACHMENT1_EXT = 36065 	# GL/glext.h:3462
GL_COLOR_ATTACHMENT2_EXT = 36066 	# GL/glext.h:3463
GL_COLOR_ATTACHMENT3_EXT = 36067 	# GL/glext.h:3464
GL_COLOR_ATTACHMENT4_EXT = 36068 	# GL/glext.h:3465
GL_COLOR_ATTACHMENT5_EXT = 36069 	# GL/glext.h:3466
GL_COLOR_ATTACHMENT6_EXT = 36070 	# GL/glext.h:3467
GL_COLOR_ATTACHMENT7_EXT = 36071 	# GL/glext.h:3468
GL_COLOR_ATTACHMENT8_EXT = 36072 	# GL/glext.h:3469
GL_COLOR_ATTACHMENT9_EXT = 36073 	# GL/glext.h:3470
GL_COLOR_ATTACHMENT10_EXT = 36074 	# GL/glext.h:3471
GL_COLOR_ATTACHMENT11_EXT = 36075 	# GL/glext.h:3472
GL_COLOR_ATTACHMENT12_EXT = 36076 	# GL/glext.h:3473
GL_COLOR_ATTACHMENT13_EXT = 36077 	# GL/glext.h:3474
GL_COLOR_ATTACHMENT14_EXT = 36078 	# GL/glext.h:3475
GL_COLOR_ATTACHMENT15_EXT = 36079 	# GL/glext.h:3476
GL_DEPTH_ATTACHMENT_EXT = 36096 	# GL/glext.h:3477
GL_STENCIL_ATTACHMENT_EXT = 36128 	# GL/glext.h:3478
GL_FRAMEBUFFER_EXT = 36160 	# GL/glext.h:3479
GL_RENDERBUFFER_EXT = 36161 	# GL/glext.h:3480
GL_RENDERBUFFER_WIDTH_EXT = 36162 	# GL/glext.h:3481
GL_RENDERBUFFER_HEIGHT_EXT = 36163 	# GL/glext.h:3482
GL_RENDERBUFFER_INTERNAL_FORMAT_EXT = 36164 	# GL/glext.h:3483
GL_STENCIL_INDEX_EXT = 36165 	# GL/glext.h:3484
GL_STENCIL_INDEX1_EXT = 36166 	# GL/glext.h:3485
GL_STENCIL_INDEX4_EXT = 36167 	# GL/glext.h:3486
GL_STENCIL_INDEX8_EXT = 36168 	# GL/glext.h:3487
GL_STENCIL_INDEX16_EXT = 36169 	# GL/glext.h:3488
GL_RENDERBUFFER_RED_SIZE_EXT = 36176 	# GL/glext.h:3489
GL_RENDERBUFFER_GREEN_SIZE_EXT = 36177 	# GL/glext.h:3490
GL_RENDERBUFFER_BLUE_SIZE_EXT = 36178 	# GL/glext.h:3491
GL_RENDERBUFFER_ALPHA_SIZE_EXT = 36179 	# GL/glext.h:3492
GL_RENDERBUFFER_DEPTH_SIZE_EXT = 36180 	# GL/glext.h:3493
GL_RENDERBUFFER_STENCIL_SIZE_EXT = 36181 	# GL/glext.h:3494
# GREMEDY_string_marker (GL/glext.h:3497)
# EXT_Cg_shader (GL/glext.h:3500)
GL_CG_VERTEX_SHADER_EXT = 35086 	# GL/glext.h:3501
GL_CG_FRAGMENT_SHADER_EXT = 35087 	# GL/glext.h:3502
# EXT_timer_query (GL/glext.h:3505)
GL_TIME_ELAPSED_EXT = 35007 	# GL/glext.h:3506
# EXT_texture_buffer_object (GL/glext.h:3509)
GL_TEXTURE_BUFFER_EXT = 35882 	# GL/glext.h:3510
GL_MAX_TEXTURE_BUFFER_SIZE_EXT = 35883 	# GL/glext.h:3511
GL_TEXTURE_BINDING_BUFFER_EXT = 35884 	# GL/glext.h:3512
GL_TEXTURE_BUFFER_DATA_STORE_BINDING_EXT = 35885 	# GL/glext.h:3513
GL_TEXTURE_BUFFER_FORMAT_EXT = 35886 	# GL/glext.h:3514
# EXT_gpu_shader4 (GL/glext.h:3517)
GL_SAMPLER_1D_ARRAY_EXT = 36288 	# GL/glext.h:3518
GL_SAMPLER_2D_ARRAY_EXT = 36289 	# GL/glext.h:3519
GL_SAMPLER_BUFFER_EXT = 36290 	# GL/glext.h:3520
GL_SAMPLER_1D_ARRAY_SHADOW_EXT = 36291 	# GL/glext.h:3521
GL_SAMPLER_2D_ARRAY_SHADOW_EXT = 36292 	# GL/glext.h:3522
GL_SAMPLER_CUBE_SHADOW_EXT = 36293 	# GL/glext.h:3523
GL_UNSIGNED_INT_VEC2_EXT = 36294 	# GL/glext.h:3524
GL_UNSIGNED_INT_VEC3_EXT = 36295 	# GL/glext.h:3525
GL_UNSIGNED_INT_VEC4_EXT = 36296 	# GL/glext.h:3526
GL_INT_SAMPLER_1D_EXT = 36297 	# GL/glext.h:3527
GL_INT_SAMPLER_2D_EXT = 36298 	# GL/glext.h:3528
GL_INT_SAMPLER_3D_EXT = 36299 	# GL/glext.h:3529
GL_INT_SAMPLER_CUBE_EXT = 36300 	# GL/glext.h:3530
GL_INT_SAMPLER_2D_RECT_EXT = 36301 	# GL/glext.h:3531
GL_INT_SAMPLER_1D_ARRAY_EXT = 36302 	# GL/glext.h:3532
GL_INT_SAMPLER_2D_ARRAY_EXT = 36303 	# GL/glext.h:3533
GL_INT_SAMPLER_BUFFER_EXT = 36304 	# GL/glext.h:3534
GL_UNSIGNED_INT_SAMPLER_1D_EXT = 36305 	# GL/glext.h:3535
GL_UNSIGNED_INT_SAMPLER_2D_EXT = 36306 	# GL/glext.h:3536
GL_UNSIGNED_INT_SAMPLER_3D_EXT = 36307 	# GL/glext.h:3537
GL_UNSIGNED_INT_SAMPLER_CUBE_EXT = 36308 	# GL/glext.h:3538
GL_UNSIGNED_INT_SAMPLER_2D_RECT_EXT = 36309 	# GL/glext.h:3539
GL_UNSIGNED_INT_SAMPLER_1D_ARRAY_EXT = 36310 	# GL/glext.h:3540
GL_UNSIGNED_INT_SAMPLER_2D_ARRAY_EXT = 36311 	# GL/glext.h:3541
GL_UNSIGNED_INT_SAMPLER_BUFFER_EXT = 36312 	# GL/glext.h:3542
GL_VERTEX_ATTRIB_ARRAY_INTEGER_EXT = 35069 	# GL/glext.h:3543
# EXT_geometry_shader4 (GL/glext.h:3546)
GL_GEOMETRY_SHADER_EXT = 36313 	# GL/glext.h:3547
GL_MAX_GEOMETRY_VARYING_COMPONENTS_EXT = 36317 	# GL/glext.h:3548
GL_MAX_VERTEX_VARYING_COMPONENTS_EXT = 36318 	# GL/glext.h:3549
GL_MAX_VARYING_COMPONENTS_EXT = 35659 	# GL/glext.h:3550
GL_MAX_GEOMETRY_UNIFORM_COMPONENTS_EXT = 36319 	# GL/glext.h:3551
GL_MAX_GEOMETRY_OUTPUT_VERTICES_EXT = 36320 	# GL/glext.h:3552
GL_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS_EXT = 36321 	# GL/glext.h:3553
GL_GEOMETRY_VERTICES_OUT_EXT = 36314 	# GL/glext.h:3554
GL_GEOMETRY_INPUT_TYPE_EXT = 36315 	# GL/glext.h:3555
GL_GEOMETRY_OUTPUT_TYPE_EXT = 36316 	# GL/glext.h:3556
GL_MAX_GEOMETRY_TEXTURE_IMAGE_UNITS_EXT = 35881 	# GL/glext.h:3557
GL_LINES_ADJACENCY_EXT = 10 	# GL/glext.h:3558
GL_LINE_STRIP_ADJACENCY_EXT = 11 	# GL/glext.h:3559
GL_TRIANGLES_ADJACENCY_EXT = 12 	# GL/glext.h:3560
GL_TRIANGLE_STRIP_ADJACENCY_EXT = 13 	# GL/glext.h:3561
GL_FRAMEBUFFER_ATTACHMENT_LAYERED_EXT = 36263 	# GL/glext.h:3562
GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS_EXT = 36264 	# GL/glext.h:3563
GL_FRAMEBUFFER_INCOMPLETE_LAYER_COUNT_EXT = 36265 	# GL/glext.h:3564
GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LAYER_EXT = 36052 	# GL/glext.h:3565
GL_PROGRAM_POINT_SIZE_EXT = 34370 	# GL/glext.h:3566
# EXT_bindable_uniform (GL/glext.h:3569)
GL_MAX_VERTEX_BINDABLE_UNIFORMS_EXT = 36322 	# GL/glext.h:3570
GL_MAX_FRAGMENT_BINDABLE_UNIFORMS_EXT = 36323 	# GL/glext.h:3571
GL_MAX_GEOMETRY_BINDABLE_UNIFORMS_EXT = 36324 	# GL/glext.h:3572
GL_MAX_BINDABLE_UNIFORM_SIZE_EXT = 36333 	# GL/glext.h:3573
GL_UNIFORM_BUFFER_EXT = 36334 	# GL/glext.h:3574
GL_UNIFORM_BUFFER_BINDING_EXT = 36335 	# GL/glext.h:3575
# EXT_framebuffer_sRGB (GL/glext.h:3578)
GL_FRAMEBUFFER_SRGB_EXT = 36281 	# GL/glext.h:3579
GL_FRAMEBUFFER_SRGB_CAPABLE_EXT = 36282 	# GL/glext.h:3580
# EXT_texture_shared_exponent (GL/glext.h:3583)
GL_RGB9_E5_EXT = 35901 	# GL/glext.h:3584
GL_UNSIGNED_INT_5_9_9_9_REV_EXT = 35902 	# GL/glext.h:3585
GL_TEXTURE_SHARED_SIZE_EXT = 35903 	# GL/glext.h:3586
# EXT_packed_float (GL/glext.h:3589)
GL_R11F_G11F_B10F_EXT = 35898 	# GL/glext.h:3590
GL_UNSIGNED_INT_10F_11F_11F_REV_EXT = 35899 	# GL/glext.h:3591
GL_RGBA_SIGNED_COMPONENTS_EXT = 35900 	# GL/glext.h:3592
# EXT_texture_array (GL/glext.h:3595)
GL_TEXTURE_1D_ARRAY_EXT = 35864 	# GL/glext.h:3596
GL_PROXY_TEXTURE_1D_ARRAY_EXT = 35865 	# GL/glext.h:3597
GL_TEXTURE_2D_ARRAY_EXT = 35866 	# GL/glext.h:3598
GL_PROXY_TEXTURE_2D_ARRAY_EXT = 35867 	# GL/glext.h:3599
GL_TEXTURE_BINDING_1D_ARRAY_EXT = 35868 	# GL/glext.h:3600
GL_TEXTURE_BINDING_2D_ARRAY_EXT = 35869 	# GL/glext.h:3601
GL_MAX_ARRAY_TEXTURE_LAYERS_EXT = 35071 	# GL/glext.h:3602
GL_COMPARE_REF_DEPTH_TO_TEXTURE_EXT = 34894 	# GL/glext.h:3603
# EXT_texture_integer (GL/glext.h:3607)
GL_RGBA32UI_EXT = 36208 	# GL/glext.h:3608
GL_RGB32UI_EXT = 36209 	# GL/glext.h:3609
GL_ALPHA32UI_EXT = 36210 	# GL/glext.h:3610
GL_INTENSITY32UI_EXT = 36211 	# GL/glext.h:3611
GL_LUMINANCE32UI_EXT = 36212 	# GL/glext.h:3612
GL_LUMINANCE_ALPHA32UI_EXT = 36213 	# GL/glext.h:3613
GL_RGBA16UI_EXT = 36214 	# GL/glext.h:3614
GL_RGB16UI_EXT = 36215 	# GL/glext.h:3615
GL_ALPHA16UI_EXT = 36216 	# GL/glext.h:3616
GL_INTENSITY16UI_EXT = 36217 	# GL/glext.h:3617
GL_LUMINANCE16UI_EXT = 36218 	# GL/glext.h:3618
GL_LUMINANCE_ALPHA16UI_EXT = 36219 	# GL/glext.h:3619
GL_RGBA8UI_EXT = 36220 	# GL/glext.h:3620
GL_RGB8UI_EXT = 36221 	# GL/glext.h:3621
GL_ALPHA8UI_EXT = 36222 	# GL/glext.h:3622
GL_INTENSITY8UI_EXT = 36223 	# GL/glext.h:3623
GL_LUMINANCE8UI_EXT = 36224 	# GL/glext.h:3624
GL_LUMINANCE_ALPHA8UI_EXT = 36225 	# GL/glext.h:3625
GL_RGBA32I_EXT = 36226 	# GL/glext.h:3626
GL_RGB32I_EXT = 36227 	# GL/glext.h:3627
GL_ALPHA32I_EXT = 36228 	# GL/glext.h:3628
GL_INTENSITY32I_EXT = 36229 	# GL/glext.h:3629
GL_LUMINANCE32I_EXT = 36230 	# GL/glext.h:3630
GL_LUMINANCE_ALPHA32I_EXT = 36231 	# GL/glext.h:3631
GL_RGBA16I_EXT = 36232 	# GL/glext.h:3632
GL_RGB16I_EXT = 36233 	# GL/glext.h:3633
GL_ALPHA16I_EXT = 36234 	# GL/glext.h:3634
GL_INTENSITY16I_EXT = 36235 	# GL/glext.h:3635
GL_LUMINANCE16I_EXT = 36236 	# GL/glext.h:3636
GL_LUMINANCE_ALPHA16I_EXT = 36237 	# GL/glext.h:3637
GL_RGBA8I_EXT = 36238 	# GL/glext.h:3638
GL_RGB8I_EXT = 36239 	# GL/glext.h:3639
GL_ALPHA8I_EXT = 36240 	# GL/glext.h:3640
GL_INTENSITY8I_EXT = 36241 	# GL/glext.h:3641
GL_LUMINANCE8I_EXT = 36242 	# GL/glext.h:3642
GL_LUMINANCE_ALPHA8I_EXT = 36243 	# GL/glext.h:3643
GL_RED_INTEGER_EXT = 36244 	# GL/glext.h:3644
GL_GREEN_INTEGER_EXT = 36245 	# GL/glext.h:3645
GL_BLUE_INTEGER_EXT = 36246 	# GL/glext.h:3646
GL_ALPHA_INTEGER_EXT = 36247 	# GL/glext.h:3647
GL_RGB_INTEGER_EXT = 36248 	# GL/glext.h:3648
GL_RGBA_INTEGER_EXT = 36249 	# GL/glext.h:3649
GL_BGR_INTEGER_EXT = 36250 	# GL/glext.h:3650
GL_BGRA_INTEGER_EXT = 36251 	# GL/glext.h:3651
GL_LUMINANCE_INTEGER_EXT = 36252 	# GL/glext.h:3652
GL_LUMINANCE_ALPHA_INTEGER_EXT = 36253 	# GL/glext.h:3653
GL_RGBA_INTEGER_MODE_EXT = 36254 	# GL/glext.h:3654
# NV_depth_buffer_float (GL/glext.h:3657)
GL_DEPTH_COMPONENT32F_NV = 36267 	# GL/glext.h:3658
GL_DEPTH32F_STENCIL8_NV = 36268 	# GL/glext.h:3659
GL_FLOAT_32_UNSIGNED_INT_24_8_REV_NV = 36269 	# GL/glext.h:3660
GL_DEPTH_BUFFER_FLOAT_MODE_NV = 36271 	# GL/glext.h:3661
# EXT_texture_compression_latc (GL/glext.h:3664)
GL_COMPRESSED_LUMINANCE_LATC1_EXT = 35952 	# GL/glext.h:3665
GL_COMPRESSED_SIGNED_LUMINANCE_LATC1_EXT = 35953 	# GL/glext.h:3666
GL_COMPRESSED_LUMINANCE_ALPHA_LATC2_EXT = 35954 	# GL/glext.h:3667
GL_COMPRESSED_SIGNED_LUMINANCE_ALPHA_LATC2_EXT = 35955 	# GL/glext.h:3668
# NV_transform_feedback (GL/glext.h:3671)
GL_BACK_PRIMARY_COLOR_NV = 35959 	# GL/glext.h:3672
GL_BACK_SECONDARY_COLOR_NV = 35960 	# GL/glext.h:3673
GL_TEXTURE_COORD_NV = 35961 	# GL/glext.h:3674
GL_CLIP_DISTANCE_NV = 35962 	# GL/glext.h:3675
GL_VERTEX_ID_NV = 35963 	# GL/glext.h:3676
GL_PRIMITIVE_ID_NV = 35964 	# GL/glext.h:3677
GL_GENERIC_ATTRIB_NV = 35965 	# GL/glext.h:3678
GL_TRANSFORM_FEEDBACK_ATTRIBS_NV = 35966 	# GL/glext.h:3679
GL_TRANSFORM_FEEDBACK_BUFFER_MODE_NV = 35967 	# GL/glext.h:3680
GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS_NV = 35968 	# GL/glext.h:3681
GL_ACTIVE_VARYINGS_NV = 35969 	# GL/glext.h:3682
GL_ACTIVE_VARYING_MAX_LENGTH_NV = 35970 	# GL/glext.h:3683
GL_TRANSFORM_FEEDBACK_VARYINGS_NV = 35971 	# GL/glext.h:3684
GL_TRANSFORM_FEEDBACK_BUFFER_START_NV = 35972 	# GL/glext.h:3685
GL_TRANSFORM_FEEDBACK_BUFFER_SIZE_NV = 35973 	# GL/glext.h:3686
GL_TRANSFORM_FEEDBACK_RECORD_NV = 35974 	# GL/glext.h:3687
GL_PRIMITIVES_GENERATED_NV = 35975 	# GL/glext.h:3688
GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN_NV = 35976 	# GL/glext.h:3689
GL_RASTERIZER_DISCARD_NV = 35977 	# GL/glext.h:3690
GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_ATTRIBS_NV = 35978 	# GL/glext.h:3691
GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS_NV = 35979 	# GL/glext.h:3692
GL_INTERLEAVED_ATTRIBS_NV = 35980 	# GL/glext.h:3693
GL_SEPARATE_ATTRIBS_NV = 35981 	# GL/glext.h:3694
GL_TRANSFORM_FEEDBACK_BUFFER_NV = 35982 	# GL/glext.h:3695
GL_TRANSFORM_FEEDBACK_BUFFER_BINDING_NV = 35983 	# GL/glext.h:3696
# NV_geometry_program4 (GL/glext.h:3699)
GL_GEOMETRY_PROGRAM_NV = 35878 	# GL/glext.h:3700
GL_MAX_PROGRAM_OUTPUT_VERTICES_NV = 35879 	# GL/glext.h:3701
GL_MAX_PROGRAM_TOTAL_OUTPUT_COMPONENTS_NV = 35880 	# GL/glext.h:3702
# NV_gpu_program4 (GL/glext.h:3705)
GL_MIN_PROGRAM_TEXEL_OFFSET_NV = 35076 	# GL/glext.h:3706
GL_MAX_PROGRAM_TEXEL_OFFSET_NV = 35077 	# GL/glext.h:3707
GL_PROGRAM_ATTRIB_COMPONENTS_NV = 35078 	# GL/glext.h:3708
GL_PROGRAM_RESULT_COMPONENTS_NV = 35079 	# GL/glext.h:3709
GL_MAX_PROGRAM_ATTRIB_COMPONENTS_NV = 35080 	# GL/glext.h:3710
GL_MAX_PROGRAM_RESULT_COMPONENTS_NV = 35081 	# GL/glext.h:3711
GL_MAX_PROGRAM_GENERIC_ATTRIBS_NV = 36261 	# GL/glext.h:3712
GL_MAX_PROGRAM_GENERIC_RESULTS_NV = 36262 	# GL/glext.h:3713
GL_MAX_PROGRAM_PARAMETER_BUFFER_BINDINGS_NV = 36256 	# GL/glext.h:3717
GL_MAX_PROGRAM_PARAMETER_BUFFER_SIZE_NV = 36257 	# GL/glext.h:3718
GL_VERTEX_PROGRAM_PARAMETER_BUFFER_NV = 36258 	# GL/glext.h:3719
GL_GEOMETRY_PROGRAM_PARAMETER_BUFFER_NV = 36259 	# GL/glext.h:3720
GL_FRAGMENT_PROGRAM_PARAMETER_BUFFER_NV = 36260 	# GL/glext.h:3721
# NV_framebuffer_multisample_coverage (GL/glext.h:3724)
GL_RENDERBUFFER_COVERAGE_SAMPLES_NV = 36011 	# GL/glext.h:3725
GL_RENDERBUFFER_COLOR_SAMPLES_NV = 36368 	# GL/glext.h:3726
GL_MAX_RENDERBUFFER_COVERAGE_SAMPLES_NV = 36183 	# GL/glext.h:3727
GL_MAX_RENDERBUFFER_COLOR_SAMPLES_NV = 36369 	# GL/glext.h:3728
GL_MAX_MULTISAMPLE_COVERAGE_MODES_NV = 36370 	# GL/glext.h:3729
GL_MULTISAMPLE_COVERAGE_MODES_NV = 36371 	# GL/glext.h:3730
# EXT_framebuffer_multisample (GL/glext.h:3733)
GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE_EXT = 36182 	# GL/glext.h:3734
GL_MAX_SAMPLES_EXT = 36183 	# GL/glext.h:3735
GL_RENDERBUFFER_SAMPLES_EXT = 36011 	# GL/glext.h:3736
# EXT_framebuffer_blit (GL/glext.h:3739)
GL_READ_FRAMEBUFFER_EXT = 36008 	# GL/glext.h:3740
GL_DRAW_FRAMEBUFFER_EXT = 36009 	# GL/glext.h:3741
GL_DRAW_FRAMEBUFFER_BINDING_EXT = 36006 	# GL/glext.h:3742
GL_READ_FRAMEBUFFER_BINDING_EXT = 36010 	# GL/glext.h:3743
# EXT_texture_compression_rgtc (GL/glext.h:3746)
GL_COMPRESSED_RED_RGTC1_EXT = 36283 	# GL/glext.h:3747
GL_COMPRESSED_SIGNED_RED_RGTC1_EXT = 36284 	# GL/glext.h:3748
GL_COMPRESSED_RED_GREEN_RGTC2_EXT = 36285 	# GL/glext.h:3749
GL_COMPRESSED_SIGNED_RED_GREEN_RGTC2_EXT = 36286 	# GL/glext.h:3750
# NV_present_video (GL/glext.h:3753)
GL_FRAME_NV = 36390 	# GL/glext.h:3754
GL_FIELDS_NV = 36391 	# GL/glext.h:3755
GL_CURRENT_TIME_NV = 36392 	# GL/glext.h:3756
GL_NUM_FILL_STREAMS_NV = 36393 	# GL/glext.h:3757
GL_PRESENT_TIME_NV = 36394 	# GL/glext.h:3758
GL_PRESENT_DURATION_NV = 36395 	# GL/glext.h:3759
# NV_conditional_render (GL/glext.h:3762)
GL_QUERY_WAIT_NV = 36371 	# GL/glext.h:3763
GL_QUERY_NO_WAIT_NV = 36372 	# GL/glext.h:3764
GL_QUERY_BY_REGION_WAIT_NV = 36373 	# GL/glext.h:3765
GL_QUERY_BY_REGION_NO_WAIT_NV = 36374 	# GL/glext.h:3766
# EXT_transform_feedback (GL/glext.h:3769)
GL_TRANSFORM_FEEDBACK_BUFFER_EXT = 35982 	# GL/glext.h:3770
GL_TRANSFORM_FEEDBACK_BUFFER_START_EXT = 35972 	# GL/glext.h:3771
GL_TRANSFORM_FEEDBACK_BUFFER_SIZE_EXT = 35973 	# GL/glext.h:3772
GL_TRANSFORM_FEEDBACK_BUFFER_BINDING_EXT = 35983 	# GL/glext.h:3773
GL_INTERLEAVED_ATTRIBS_EXT = 35980 	# GL/glext.h:3774
GL_SEPARATE_ATTRIBS_EXT = 35981 	# GL/glext.h:3775
GL_PRIMITIVES_GENERATED_EXT = 35975 	# GL/glext.h:3776
GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN_EXT = 35976 	# GL/glext.h:3777
GL_RASTERIZER_DISCARD_EXT = 35977 	# GL/glext.h:3778
GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_COMPONENTS_EXT = 35978 	# GL/glext.h:3779
GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS_EXT = 35979 	# GL/glext.h:3780
GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS_EXT = 35968 	# GL/glext.h:3781
GL_TRANSFORM_FEEDBACK_VARYINGS_EXT = 35971 	# GL/glext.h:3782
GL_TRANSFORM_FEEDBACK_BUFFER_MODE_EXT = 35967 	# GL/glext.h:3783
GL_TRANSFORM_FEEDBACK_VARYING_MAX_LENGTH_EXT = 35958 	# GL/glext.h:3784
# EXT_direct_state_access (GL/glext.h:3787)
GL_PROGRAM_MATRIX_EXT = 36397 	# GL/glext.h:3788
GL_TRANSPOSE_PROGRAM_MATRIX_EXT = 36398 	# GL/glext.h:3789
GL_PROGRAM_MATRIX_STACK_DEPTH_EXT = 36399 	# GL/glext.h:3790
# EXT_vertex_array_bgra (GL/glext.h:3793)
# EXT_texture_swizzle (GL/glext.h:3797)
GL_TEXTURE_SWIZZLE_R_EXT = 36418 	# GL/glext.h:3798
GL_TEXTURE_SWIZZLE_G_EXT = 36419 	# GL/glext.h:3799
GL_TEXTURE_SWIZZLE_B_EXT = 36420 	# GL/glext.h:3800
GL_TEXTURE_SWIZZLE_A_EXT = 36421 	# GL/glext.h:3801
GL_TEXTURE_SWIZZLE_RGBA_EXT = 36422 	# GL/glext.h:3802
# NV_explicit_multisample (GL/glext.h:3805)
GL_SAMPLE_POSITION_NV = 36432 	# GL/glext.h:3806
GL_SAMPLE_MASK_NV = 36433 	# GL/glext.h:3807
GL_SAMPLE_MASK_VALUE_NV = 36434 	# GL/glext.h:3808
GL_TEXTURE_BINDING_RENDERBUFFER_NV = 36435 	# GL/glext.h:3809
GL_TEXTURE_RENDERBUFFER_DATA_STORE_BINDING_NV = 36436 	# GL/glext.h:3810
GL_MAX_SAMPLE_MASK_WORDS_NV = 36441 	# GL/glext.h:3811
GL_TEXTURE_RENDERBUFFER_NV = 36437 	# GL/glext.h:3812
GL_SAMPLER_RENDERBUFFER_NV = 36438 	# GL/glext.h:3813
GL_INT_SAMPLER_RENDERBUFFER_NV = 36439 	# GL/glext.h:3814
GL_UNSIGNED_INT_SAMPLER_RENDERBUFFER_NV = 36440 	# GL/glext.h:3815
# NV_transform_feedback2 (GL/glext.h:3818)
GL_TRANSFORM_FEEDBACK_NV = 36386 	# GL/glext.h:3819
GL_TRANSFORM_FEEDBACK_BUFFER_PAUSED_NV = 36387 	# GL/glext.h:3820
GL_TRANSFORM_FEEDBACK_BUFFER_ACTIVE_NV = 36388 	# GL/glext.h:3821
GL_TRANSFORM_FEEDBACK_BINDING_NV = 36389 	# GL/glext.h:3822
# NV_vertex_buffer_unified_memory (GL/glext.h:3825)
GL_VERTEX_ATTRIB_ARRAY_UNIFIED_NV = 36638 	# GL/glext.h:3826
GL_ELEMENT_ARRAY_UNIFIED_NV = 36639 	# GL/glext.h:3827
GL_VERTEX_ATTRIB_ARRAY_ADDRESS_NV = 36640 	# GL/glext.h:3828
GL_VERTEX_ARRAY_ADDRESS_NV = 36641 	# GL/glext.h:3829
GL_NORMAL_ARRAY_ADDRESS_NV = 36642 	# GL/glext.h:3830
GL_COLOR_ARRAY_ADDRESS_NV = 36643 	# GL/glext.h:3831
GL_INDEX_ARRAY_ADDRESS_NV = 36644 	# GL/glext.h:3832
GL_TEXTURE_COORD_ARRAY_ADDRESS_NV = 36645 	# GL/glext.h:3833
GL_EDGE_FLAG_ARRAY_ADDRESS_NV = 36646 	# GL/glext.h:3834
GL_SECONDARY_COLOR_ARRAY_ADDRESS_NV = 36647 	# GL/glext.h:3835
GL_FOG_COORD_ARRAY_ADDRESS_NV = 36648 	# GL/glext.h:3836
GL_ELEMENT_ARRAY_ADDRESS_NV = 36649 	# GL/glext.h:3837
GL_VERTEX_ATTRIB_ARRAY_LENGTH_NV = 36650 	# GL/glext.h:3838
GL_VERTEX_ARRAY_LENGTH_NV = 36651 	# GL/glext.h:3839
GL_NORMAL_ARRAY_LENGTH_NV = 36652 	# GL/glext.h:3840
GL_COLOR_ARRAY_LENGTH_NV = 36653 	# GL/glext.h:3841
GL_INDEX_ARRAY_LENGTH_NV = 36654 	# GL/glext.h:3842
GL_TEXTURE_COORD_ARRAY_LENGTH_NV = 36655 	# GL/glext.h:3843
GL_EDGE_FLAG_ARRAY_LENGTH_NV = 36656 	# GL/glext.h:3844
GL_SECONDARY_COLOR_ARRAY_LENGTH_NV = 36657 	# GL/glext.h:3845
GL_FOG_COORD_ARRAY_LENGTH_NV = 36658 	# GL/glext.h:3846
GL_ELEMENT_ARRAY_LENGTH_NV = 36659 	# GL/glext.h:3847
# NV_shader_buffer_load (GL/glext.h:3850)
GL_BUFFER_GPU_ADDRESS_NV = 36637 	# GL/glext.h:3851
GL_GPU_ADDRESS_NV = 36660 	# GL/glext.h:3852
GL_MAX_SHADER_BUFFER_ADDRESS_NV = 36661 	# GL/glext.h:3853
# VERSION_2_0 (GL/glext.h:3859)
GLchar = c_char 	# GL/glext.h:3861
# VERSION_1_5 (GL/glext.h:3864)
GLintptr = c_ptrdiff_t 	# GL/glext.h:3866
GLsizeiptr = c_ptrdiff_t 	# GL/glext.h:3867
# ARB_vertex_buffer_object (GL/glext.h:3870)
GLintptrARB = c_ptrdiff_t 	# GL/glext.h:3872
GLsizeiptrARB = c_ptrdiff_t 	# GL/glext.h:3873
# ARB_shader_objects (GL/glext.h:3876)
GLcharARB = c_char 	# GL/glext.h:3878
GLhandleARB = c_uint 	# GL/glext.h:3879
# ARB_half_float_pixel (GL/glext.h:3883)
GLhalfARB = c_ushort 	# GL/glext.h:3884
# NV_half_float (GL/glext.h:3887)
GLhalfNV = c_ushort 	# GL/glext.h:3888
# EXT_timer_query (GL/glext.h:3891)
GLint64EXT = c_longlong 	# GL/glext.h:3892
GLuint64EXT = c_ulonglong 	# GL/glext.h:3893
# VERSION_1_2 (GL/glext.h:3896)
# VERSION_1_3 (GL/glext.h:3978)
# VERSION_1_4 (GL/glext.h:4076)
GL_VERSION_1_4 = 1 	# GL/glext.h:4077
GLenum = c_uint 	# /usr/include/GL/gl.h:121
# GL/glext.h:4079
glBlendFuncSeparate = _link_function('glBlendFuncSeparate', None, [GLenum, GLenum, GLenum, GLenum], 'VERSION_1_4')

GLfloat = c_float 	# /usr/include/GL/gl.h:132
# GL/glext.h:4080
glFogCoordf = _link_function('glFogCoordf', None, [GLfloat], 'VERSION_1_4')

# GL/glext.h:4081
glFogCoordfv = _link_function('glFogCoordfv', None, [POINTER(GLfloat)], 'VERSION_1_4')

GLdouble = c_double 	# /usr/include/GL/gl.h:134
# GL/glext.h:4082
glFogCoordd = _link_function('glFogCoordd', None, [GLdouble], 'VERSION_1_4')

# GL/glext.h:4083
glFogCoorddv = _link_function('glFogCoorddv', None, [POINTER(GLdouble)], 'VERSION_1_4')

GLsizei = c_int 	# /usr/include/GL/gl.h:131
GLvoid = None 	# /usr/include/GL/gl.h:124
# GL/glext.h:4084
glFogCoordPointer = _link_function('glFogCoordPointer', None, [GLenum, GLsizei, POINTER(GLvoid)], 'VERSION_1_4')

GLint = c_int 	# /usr/include/GL/gl.h:127
# GL/glext.h:4085
glMultiDrawArrays = _link_function('glMultiDrawArrays', None, [GLenum, POINTER(GLint), POINTER(GLsizei), GLsizei], 'VERSION_1_4')

# GL/glext.h:4086
glMultiDrawElements = _link_function('glMultiDrawElements', None, [GLenum, POINTER(GLsizei), GLenum, POINTER(POINTER(GLvoid)), GLsizei], 'VERSION_1_4')

# GL/glext.h:4087
glPointParameterf = _link_function('glPointParameterf', None, [GLenum, GLfloat], 'VERSION_1_4')

# GL/glext.h:4088
glPointParameterfv = _link_function('glPointParameterfv', None, [GLenum, POINTER(GLfloat)], 'VERSION_1_4')

# GL/glext.h:4089
glPointParameteri = _link_function('glPointParameteri', None, [GLenum, GLint], 'VERSION_1_4')

# GL/glext.h:4090
glPointParameteriv = _link_function('glPointParameteriv', None, [GLenum, POINTER(GLint)], 'VERSION_1_4')

GLbyte = c_char 	# /usr/include/GL/gl.h:125
# GL/glext.h:4091
glSecondaryColor3b = _link_function('glSecondaryColor3b', None, [GLbyte, GLbyte, GLbyte], 'VERSION_1_4')

# GL/glext.h:4092
glSecondaryColor3bv = _link_function('glSecondaryColor3bv', None, [POINTER(GLbyte)], 'VERSION_1_4')

# GL/glext.h:4093
glSecondaryColor3d = _link_function('glSecondaryColor3d', None, [GLdouble, GLdouble, GLdouble], 'VERSION_1_4')

# GL/glext.h:4094
glSecondaryColor3dv = _link_function('glSecondaryColor3dv', None, [POINTER(GLdouble)], 'VERSION_1_4')

# GL/glext.h:4095
glSecondaryColor3f = _link_function('glSecondaryColor3f', None, [GLfloat, GLfloat, GLfloat], 'VERSION_1_4')

# GL/glext.h:4096
glSecondaryColor3fv = _link_function('glSecondaryColor3fv', None, [POINTER(GLfloat)], 'VERSION_1_4')

# GL/glext.h:4097
glSecondaryColor3i = _link_function('glSecondaryColor3i', None, [GLint, GLint, GLint], 'VERSION_1_4')

# GL/glext.h:4098
glSecondaryColor3iv = _link_function('glSecondaryColor3iv', None, [POINTER(GLint)], 'VERSION_1_4')

GLshort = c_short 	# /usr/include/GL/gl.h:126
# GL/glext.h:4099
glSecondaryColor3s = _link_function('glSecondaryColor3s', None, [GLshort, GLshort, GLshort], 'VERSION_1_4')

# GL/glext.h:4100
glSecondaryColor3sv = _link_function('glSecondaryColor3sv', None, [POINTER(GLshort)], 'VERSION_1_4')

GLubyte = c_ubyte 	# /usr/include/GL/gl.h:128
# GL/glext.h:4101
glSecondaryColor3ub = _link_function('glSecondaryColor3ub', None, [GLubyte, GLubyte, GLubyte], 'VERSION_1_4')

# GL/glext.h:4102
glSecondaryColor3ubv = _link_function('glSecondaryColor3ubv', None, [POINTER(GLubyte)], 'VERSION_1_4')

GLuint = c_uint 	# /usr/include/GL/gl.h:130
# GL/glext.h:4103
glSecondaryColor3ui = _link_function('glSecondaryColor3ui', None, [GLuint, GLuint, GLuint], 'VERSION_1_4')

# GL/glext.h:4104
glSecondaryColor3uiv = _link_function('glSecondaryColor3uiv', None, [POINTER(GLuint)], 'VERSION_1_4')

GLushort = c_ushort 	# /usr/include/GL/gl.h:129
# GL/glext.h:4105
glSecondaryColor3us = _link_function('glSecondaryColor3us', None, [GLushort, GLushort, GLushort], 'VERSION_1_4')

# GL/glext.h:4106
glSecondaryColor3usv = _link_function('glSecondaryColor3usv', None, [POINTER(GLushort)], 'VERSION_1_4')

# GL/glext.h:4107
glSecondaryColorPointer = _link_function('glSecondaryColorPointer', None, [GLint, GLenum, GLsizei, POINTER(GLvoid)], 'VERSION_1_4')

# GL/glext.h:4108
glWindowPos2d = _link_function('glWindowPos2d', None, [GLdouble, GLdouble], 'VERSION_1_4')

# GL/glext.h:4109
glWindowPos2dv = _link_function('glWindowPos2dv', None, [POINTER(GLdouble)], 'VERSION_1_4')

# GL/glext.h:4110
glWindowPos2f = _link_function('glWindowPos2f', None, [GLfloat, GLfloat], 'VERSION_1_4')

# GL/glext.h:4111
glWindowPos2fv = _link_function('glWindowPos2fv', None, [POINTER(GLfloat)], 'VERSION_1_4')

# GL/glext.h:4112
glWindowPos2i = _link_function('glWindowPos2i', None, [GLint, GLint], 'VERSION_1_4')

# GL/glext.h:4113
glWindowPos2iv = _link_function('glWindowPos2iv', None, [POINTER(GLint)], 'VERSION_1_4')

# GL/glext.h:4114
glWindowPos2s = _link_function('glWindowPos2s', None, [GLshort, GLshort], 'VERSION_1_4')

# GL/glext.h:4115
glWindowPos2sv = _link_function('glWindowPos2sv', None, [POINTER(GLshort)], 'VERSION_1_4')

# GL/glext.h:4116
glWindowPos3d = _link_function('glWindowPos3d', None, [GLdouble, GLdouble, GLdouble], 'VERSION_1_4')

# GL/glext.h:4117
glWindowPos3dv = _link_function('glWindowPos3dv', None, [POINTER(GLdouble)], 'VERSION_1_4')

# GL/glext.h:4118
glWindowPos3f = _link_function('glWindowPos3f', None, [GLfloat, GLfloat, GLfloat], 'VERSION_1_4')

# GL/glext.h:4119
glWindowPos3fv = _link_function('glWindowPos3fv', None, [POINTER(GLfloat)], 'VERSION_1_4')

# GL/glext.h:4120
glWindowPos3i = _link_function('glWindowPos3i', None, [GLint, GLint, GLint], 'VERSION_1_4')

# GL/glext.h:4121
glWindowPos3iv = _link_function('glWindowPos3iv', None, [POINTER(GLint)], 'VERSION_1_4')

# GL/glext.h:4122
glWindowPos3s = _link_function('glWindowPos3s', None, [GLshort, GLshort, GLshort], 'VERSION_1_4')

# GL/glext.h:4123
glWindowPos3sv = _link_function('glWindowPos3sv', None, [POINTER(GLshort)], 'VERSION_1_4')

PFNGLBLENDFUNCSEPARATEPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:4125
PFNGLFOGCOORDFPROC = CFUNCTYPE(None, GLfloat) 	# GL/glext.h:4126
PFNGLFOGCOORDFVPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:4127
PFNGLFOGCOORDDPROC = CFUNCTYPE(None, GLdouble) 	# GL/glext.h:4128
PFNGLFOGCOORDDVPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:4129
PFNGLFOGCOORDPOINTERPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:4130
PFNGLMULTIDRAWARRAYSPROC = CFUNCTYPE(None, GLenum, POINTER(GLint), POINTER(GLsizei), GLsizei) 	# GL/glext.h:4131
PFNGLMULTIDRAWELEMENTSPROC = CFUNCTYPE(None, GLenum, POINTER(GLsizei), GLenum, POINTER(POINTER(GLvoid)), GLsizei) 	# GL/glext.h:4132
PFNGLPOINTPARAMETERFPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:4133
PFNGLPOINTPARAMETERFVPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4134
PFNGLPOINTPARAMETERIPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:4135
PFNGLPOINTPARAMETERIVPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:4136
PFNGLSECONDARYCOLOR3BPROC = CFUNCTYPE(None, GLbyte, GLbyte, GLbyte) 	# GL/glext.h:4137
PFNGLSECONDARYCOLOR3BVPROC = CFUNCTYPE(None, POINTER(GLbyte)) 	# GL/glext.h:4138
PFNGLSECONDARYCOLOR3DPROC = CFUNCTYPE(None, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:4139
PFNGLSECONDARYCOLOR3DVPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:4140
PFNGLSECONDARYCOLOR3FPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:4141
PFNGLSECONDARYCOLOR3FVPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:4142
PFNGLSECONDARYCOLOR3IPROC = CFUNCTYPE(None, GLint, GLint, GLint) 	# GL/glext.h:4143
PFNGLSECONDARYCOLOR3IVPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:4144
PFNGLSECONDARYCOLOR3SPROC = CFUNCTYPE(None, GLshort, GLshort, GLshort) 	# GL/glext.h:4145
PFNGLSECONDARYCOLOR3SVPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:4146
PFNGLSECONDARYCOLOR3UBPROC = CFUNCTYPE(None, GLubyte, GLubyte, GLubyte) 	# GL/glext.h:4147
PFNGLSECONDARYCOLOR3UBVPROC = CFUNCTYPE(None, POINTER(GLubyte)) 	# GL/glext.h:4148
PFNGLSECONDARYCOLOR3UIPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint) 	# GL/glext.h:4149
PFNGLSECONDARYCOLOR3UIVPROC = CFUNCTYPE(None, POINTER(GLuint)) 	# GL/glext.h:4150
PFNGLSECONDARYCOLOR3USPROC = CFUNCTYPE(None, GLushort, GLushort, GLushort) 	# GL/glext.h:4151
PFNGLSECONDARYCOLOR3USVPROC = CFUNCTYPE(None, POINTER(GLushort)) 	# GL/glext.h:4152
PFNGLSECONDARYCOLORPOINTERPROC = CFUNCTYPE(None, GLint, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:4153
PFNGLWINDOWPOS2DPROC = CFUNCTYPE(None, GLdouble, GLdouble) 	# GL/glext.h:4154
PFNGLWINDOWPOS2DVPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:4155
PFNGLWINDOWPOS2FPROC = CFUNCTYPE(None, GLfloat, GLfloat) 	# GL/glext.h:4156
PFNGLWINDOWPOS2FVPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:4157
PFNGLWINDOWPOS2IPROC = CFUNCTYPE(None, GLint, GLint) 	# GL/glext.h:4158
PFNGLWINDOWPOS2IVPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:4159
PFNGLWINDOWPOS2SPROC = CFUNCTYPE(None, GLshort, GLshort) 	# GL/glext.h:4160
PFNGLWINDOWPOS2SVPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:4161
PFNGLWINDOWPOS3DPROC = CFUNCTYPE(None, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:4162
PFNGLWINDOWPOS3DVPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:4163
PFNGLWINDOWPOS3FPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:4164
PFNGLWINDOWPOS3FVPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:4165
PFNGLWINDOWPOS3IPROC = CFUNCTYPE(None, GLint, GLint, GLint) 	# GL/glext.h:4166
PFNGLWINDOWPOS3IVPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:4167
PFNGLWINDOWPOS3SPROC = CFUNCTYPE(None, GLshort, GLshort, GLshort) 	# GL/glext.h:4168
PFNGLWINDOWPOS3SVPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:4169
# VERSION_1_5 (GL/glext.h:4172)
GL_VERSION_1_5 = 1 	# GL/glext.h:4173
# GL/glext.h:4175
glGenQueries = _link_function('glGenQueries', None, [GLsizei, POINTER(GLuint)], 'VERSION_1_5')

# GL/glext.h:4176
glDeleteQueries = _link_function('glDeleteQueries', None, [GLsizei, POINTER(GLuint)], 'VERSION_1_5')

GLboolean = c_ubyte 	# /usr/include/GL/gl.h:122
# GL/glext.h:4177
glIsQuery = _link_function('glIsQuery', GLboolean, [GLuint], 'VERSION_1_5')

# GL/glext.h:4178
glBeginQuery = _link_function('glBeginQuery', None, [GLenum, GLuint], 'VERSION_1_5')

# GL/glext.h:4179
glEndQuery = _link_function('glEndQuery', None, [GLenum], 'VERSION_1_5')

# GL/glext.h:4180
glGetQueryiv = _link_function('glGetQueryiv', None, [GLenum, GLenum, POINTER(GLint)], 'VERSION_1_5')

# GL/glext.h:4181
glGetQueryObjectiv = _link_function('glGetQueryObjectiv', None, [GLuint, GLenum, POINTER(GLint)], 'VERSION_1_5')

# GL/glext.h:4182
glGetQueryObjectuiv = _link_function('glGetQueryObjectuiv', None, [GLuint, GLenum, POINTER(GLuint)], 'VERSION_1_5')

# GL/glext.h:4183
glBindBuffer = _link_function('glBindBuffer', None, [GLenum, GLuint], 'VERSION_1_5')

# GL/glext.h:4184
glDeleteBuffers = _link_function('glDeleteBuffers', None, [GLsizei, POINTER(GLuint)], 'VERSION_1_5')

# GL/glext.h:4185
glGenBuffers = _link_function('glGenBuffers', None, [GLsizei, POINTER(GLuint)], 'VERSION_1_5')

# GL/glext.h:4186
glIsBuffer = _link_function('glIsBuffer', GLboolean, [GLuint], 'VERSION_1_5')

# GL/glext.h:4187
glBufferData = _link_function('glBufferData', None, [GLenum, GLsizeiptr, POINTER(GLvoid), GLenum], 'VERSION_1_5')

# GL/glext.h:4188
glBufferSubData = _link_function('glBufferSubData', None, [GLenum, GLintptr, GLsizeiptr, POINTER(GLvoid)], 'VERSION_1_5')

# GL/glext.h:4189
glGetBufferSubData = _link_function('glGetBufferSubData', None, [GLenum, GLintptr, GLsizeiptr, POINTER(GLvoid)], 'VERSION_1_5')

# GL/glext.h:4190
glMapBuffer = _link_function('glMapBuffer', POINTER(GLvoid), [GLenum, GLenum], 'VERSION_1_5')

# GL/glext.h:4191
glUnmapBuffer = _link_function('glUnmapBuffer', GLboolean, [GLenum], 'VERSION_1_5')

# GL/glext.h:4192
glGetBufferParameteriv = _link_function('glGetBufferParameteriv', None, [GLenum, GLenum, POINTER(GLint)], 'VERSION_1_5')

# GL/glext.h:4193
glGetBufferPointerv = _link_function('glGetBufferPointerv', None, [GLenum, GLenum, POINTER(POINTER(GLvoid))], 'VERSION_1_5')

PFNGLGENQUERIESPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:4195
PFNGLDELETEQUERIESPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:4196
PFNGLISQUERYPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:4197
PFNGLBEGINQUERYPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:4198
PFNGLENDQUERYPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:4199
PFNGLGETQUERYIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:4200
PFNGLGETQUERYOBJECTIVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:4201
PFNGLGETQUERYOBJECTUIVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLuint)) 	# GL/glext.h:4202
PFNGLBINDBUFFERPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:4203
PFNGLDELETEBUFFERSPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:4204
PFNGLGENBUFFERSPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:4205
PFNGLISBUFFERPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:4206
PFNGLBUFFERDATAPROC = CFUNCTYPE(None, GLenum, GLsizeiptr, POINTER(GLvoid), GLenum) 	# GL/glext.h:4207
PFNGLBUFFERSUBDATAPROC = CFUNCTYPE(None, GLenum, GLintptr, GLsizeiptr, POINTER(GLvoid)) 	# GL/glext.h:4208
PFNGLGETBUFFERSUBDATAPROC = CFUNCTYPE(None, GLenum, GLintptr, GLsizeiptr, POINTER(GLvoid)) 	# GL/glext.h:4209
PFNGLMAPBUFFERPROC = CFUNCTYPE(POINTER(GLvoid), GLenum, GLenum) 	# GL/glext.h:4210
PFNGLUNMAPBUFFERPROC = CFUNCTYPE(GLboolean, GLenum) 	# GL/glext.h:4211
PFNGLGETBUFFERPARAMETERIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:4212
PFNGLGETBUFFERPOINTERVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:4213
# VERSION_2_0 (GL/glext.h:4216)
GL_VERSION_2_0 = 1 	# GL/glext.h:4217
# GL/glext.h:4219
glBlendEquationSeparate = _link_function('glBlendEquationSeparate', None, [GLenum, GLenum], 'VERSION_2_0')

# GL/glext.h:4220
glDrawBuffers = _link_function('glDrawBuffers', None, [GLsizei, POINTER(GLenum)], 'VERSION_2_0')

# GL/glext.h:4221
glStencilOpSeparate = _link_function('glStencilOpSeparate', None, [GLenum, GLenum, GLenum, GLenum], 'VERSION_2_0')

# GL/glext.h:4222
glStencilFuncSeparate = _link_function('glStencilFuncSeparate', None, [GLenum, GLenum, GLint, GLuint], 'VERSION_2_0')

# GL/glext.h:4223
glStencilMaskSeparate = _link_function('glStencilMaskSeparate', None, [GLenum, GLuint], 'VERSION_2_0')

# GL/glext.h:4224
glAttachShader = _link_function('glAttachShader', None, [GLuint, GLuint], 'VERSION_2_0')

# GL/glext.h:4225
glBindAttribLocation = _link_function('glBindAttribLocation', None, [GLuint, GLuint, POINTER(GLchar)], 'VERSION_2_0')

# GL/glext.h:4226
glCompileShader = _link_function('glCompileShader', None, [GLuint], 'VERSION_2_0')

# GL/glext.h:4227
glCreateProgram = _link_function('glCreateProgram', GLuint, [], 'VERSION_2_0')

# GL/glext.h:4228
glCreateShader = _link_function('glCreateShader', GLuint, [GLenum], 'VERSION_2_0')

# GL/glext.h:4229
glDeleteProgram = _link_function('glDeleteProgram', None, [GLuint], 'VERSION_2_0')

# GL/glext.h:4230
glDeleteShader = _link_function('glDeleteShader', None, [GLuint], 'VERSION_2_0')

# GL/glext.h:4231
glDetachShader = _link_function('glDetachShader', None, [GLuint, GLuint], 'VERSION_2_0')

# GL/glext.h:4232
glDisableVertexAttribArray = _link_function('glDisableVertexAttribArray', None, [GLuint], 'VERSION_2_0')

# GL/glext.h:4233
glEnableVertexAttribArray = _link_function('glEnableVertexAttribArray', None, [GLuint], 'VERSION_2_0')

# GL/glext.h:4234
glGetActiveAttrib = _link_function('glGetActiveAttrib', None, [GLuint, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLint), POINTER(GLenum), POINTER(GLchar)], 'VERSION_2_0')

# GL/glext.h:4235
glGetActiveUniform = _link_function('glGetActiveUniform', None, [GLuint, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLint), POINTER(GLenum), POINTER(GLchar)], 'VERSION_2_0')

# GL/glext.h:4236
glGetAttachedShaders = _link_function('glGetAttachedShaders', None, [GLuint, GLsizei, POINTER(GLsizei), POINTER(GLuint)], 'VERSION_2_0')

# GL/glext.h:4237
glGetAttribLocation = _link_function('glGetAttribLocation', GLint, [GLuint, POINTER(GLchar)], 'VERSION_2_0')

# GL/glext.h:4238
glGetProgramiv = _link_function('glGetProgramiv', None, [GLuint, GLenum, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:4239
glGetProgramInfoLog = _link_function('glGetProgramInfoLog', None, [GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)], 'VERSION_2_0')

# GL/glext.h:4240
glGetShaderiv = _link_function('glGetShaderiv', None, [GLuint, GLenum, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:4241
glGetShaderInfoLog = _link_function('glGetShaderInfoLog', None, [GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)], 'VERSION_2_0')

# GL/glext.h:4242
glGetShaderSource = _link_function('glGetShaderSource', None, [GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)], 'VERSION_2_0')

# GL/glext.h:4243
glGetUniformLocation = _link_function('glGetUniformLocation', GLint, [GLuint, POINTER(GLchar)], 'VERSION_2_0')

# GL/glext.h:4244
glGetUniformfv = _link_function('glGetUniformfv', None, [GLuint, GLint, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:4245
glGetUniformiv = _link_function('glGetUniformiv', None, [GLuint, GLint, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:4246
glGetVertexAttribdv = _link_function('glGetVertexAttribdv', None, [GLuint, GLenum, POINTER(GLdouble)], 'VERSION_2_0')

# GL/glext.h:4247
glGetVertexAttribfv = _link_function('glGetVertexAttribfv', None, [GLuint, GLenum, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:4248
glGetVertexAttribiv = _link_function('glGetVertexAttribiv', None, [GLuint, GLenum, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:4249
glGetVertexAttribPointerv = _link_function('glGetVertexAttribPointerv', None, [GLuint, GLenum, POINTER(POINTER(GLvoid))], 'VERSION_2_0')

# GL/glext.h:4250
glIsProgram = _link_function('glIsProgram', GLboolean, [GLuint], 'VERSION_2_0')

# GL/glext.h:4251
glIsShader = _link_function('glIsShader', GLboolean, [GLuint], 'VERSION_2_0')

# GL/glext.h:4252
glLinkProgram = _link_function('glLinkProgram', None, [GLuint], 'VERSION_2_0')

# GL/glext.h:4253
glShaderSource = _link_function('glShaderSource', None, [GLuint, GLsizei, POINTER(POINTER(GLchar)), POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:4254
glUseProgram = _link_function('glUseProgram', None, [GLuint], 'VERSION_2_0')

# GL/glext.h:4255
glUniform1f = _link_function('glUniform1f', None, [GLint, GLfloat], 'VERSION_2_0')

# GL/glext.h:4256
glUniform2f = _link_function('glUniform2f', None, [GLint, GLfloat, GLfloat], 'VERSION_2_0')

# GL/glext.h:4257
glUniform3f = _link_function('glUniform3f', None, [GLint, GLfloat, GLfloat, GLfloat], 'VERSION_2_0')

# GL/glext.h:4258
glUniform4f = _link_function('glUniform4f', None, [GLint, GLfloat, GLfloat, GLfloat, GLfloat], 'VERSION_2_0')

# GL/glext.h:4259
glUniform1i = _link_function('glUniform1i', None, [GLint, GLint], 'VERSION_2_0')

# GL/glext.h:4260
glUniform2i = _link_function('glUniform2i', None, [GLint, GLint, GLint], 'VERSION_2_0')

# GL/glext.h:4261
glUniform3i = _link_function('glUniform3i', None, [GLint, GLint, GLint, GLint], 'VERSION_2_0')

# GL/glext.h:4262
glUniform4i = _link_function('glUniform4i', None, [GLint, GLint, GLint, GLint, GLint], 'VERSION_2_0')

# GL/glext.h:4263
glUniform1fv = _link_function('glUniform1fv', None, [GLint, GLsizei, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:4264
glUniform2fv = _link_function('glUniform2fv', None, [GLint, GLsizei, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:4265
glUniform3fv = _link_function('glUniform3fv', None, [GLint, GLsizei, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:4266
glUniform4fv = _link_function('glUniform4fv', None, [GLint, GLsizei, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:4267
glUniform1iv = _link_function('glUniform1iv', None, [GLint, GLsizei, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:4268
glUniform2iv = _link_function('glUniform2iv', None, [GLint, GLsizei, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:4269
glUniform3iv = _link_function('glUniform3iv', None, [GLint, GLsizei, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:4270
glUniform4iv = _link_function('glUniform4iv', None, [GLint, GLsizei, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:4271
glUniformMatrix2fv = _link_function('glUniformMatrix2fv', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:4272
glUniformMatrix3fv = _link_function('glUniformMatrix3fv', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:4273
glUniformMatrix4fv = _link_function('glUniformMatrix4fv', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:4274
glValidateProgram = _link_function('glValidateProgram', None, [GLuint], 'VERSION_2_0')

# GL/glext.h:4275
glVertexAttrib1d = _link_function('glVertexAttrib1d', None, [GLuint, GLdouble], 'VERSION_2_0')

# GL/glext.h:4276
glVertexAttrib1dv = _link_function('glVertexAttrib1dv', None, [GLuint, POINTER(GLdouble)], 'VERSION_2_0')

# GL/glext.h:4277
glVertexAttrib1f = _link_function('glVertexAttrib1f', None, [GLuint, GLfloat], 'VERSION_2_0')

# GL/glext.h:4278
glVertexAttrib1fv = _link_function('glVertexAttrib1fv', None, [GLuint, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:4279
glVertexAttrib1s = _link_function('glVertexAttrib1s', None, [GLuint, GLshort], 'VERSION_2_0')

# GL/glext.h:4280
glVertexAttrib1sv = _link_function('glVertexAttrib1sv', None, [GLuint, POINTER(GLshort)], 'VERSION_2_0')

# GL/glext.h:4281
glVertexAttrib2d = _link_function('glVertexAttrib2d', None, [GLuint, GLdouble, GLdouble], 'VERSION_2_0')

# GL/glext.h:4282
glVertexAttrib2dv = _link_function('glVertexAttrib2dv', None, [GLuint, POINTER(GLdouble)], 'VERSION_2_0')

# GL/glext.h:4283
glVertexAttrib2f = _link_function('glVertexAttrib2f', None, [GLuint, GLfloat, GLfloat], 'VERSION_2_0')

# GL/glext.h:4284
glVertexAttrib2fv = _link_function('glVertexAttrib2fv', None, [GLuint, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:4285
glVertexAttrib2s = _link_function('glVertexAttrib2s', None, [GLuint, GLshort, GLshort], 'VERSION_2_0')

# GL/glext.h:4286
glVertexAttrib2sv = _link_function('glVertexAttrib2sv', None, [GLuint, POINTER(GLshort)], 'VERSION_2_0')

# GL/glext.h:4287
glVertexAttrib3d = _link_function('glVertexAttrib3d', None, [GLuint, GLdouble, GLdouble, GLdouble], 'VERSION_2_0')

# GL/glext.h:4288
glVertexAttrib3dv = _link_function('glVertexAttrib3dv', None, [GLuint, POINTER(GLdouble)], 'VERSION_2_0')

# GL/glext.h:4289
glVertexAttrib3f = _link_function('glVertexAttrib3f', None, [GLuint, GLfloat, GLfloat, GLfloat], 'VERSION_2_0')

# GL/glext.h:4290
glVertexAttrib3fv = _link_function('glVertexAttrib3fv', None, [GLuint, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:4291
glVertexAttrib3s = _link_function('glVertexAttrib3s', None, [GLuint, GLshort, GLshort, GLshort], 'VERSION_2_0')

# GL/glext.h:4292
glVertexAttrib3sv = _link_function('glVertexAttrib3sv', None, [GLuint, POINTER(GLshort)], 'VERSION_2_0')

# GL/glext.h:4293
glVertexAttrib4Nbv = _link_function('glVertexAttrib4Nbv', None, [GLuint, POINTER(GLbyte)], 'VERSION_2_0')

# GL/glext.h:4294
glVertexAttrib4Niv = _link_function('glVertexAttrib4Niv', None, [GLuint, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:4295
glVertexAttrib4Nsv = _link_function('glVertexAttrib4Nsv', None, [GLuint, POINTER(GLshort)], 'VERSION_2_0')

# GL/glext.h:4296
glVertexAttrib4Nub = _link_function('glVertexAttrib4Nub', None, [GLuint, GLubyte, GLubyte, GLubyte, GLubyte], 'VERSION_2_0')

# GL/glext.h:4297
glVertexAttrib4Nubv = _link_function('glVertexAttrib4Nubv', None, [GLuint, POINTER(GLubyte)], 'VERSION_2_0')

# GL/glext.h:4298
glVertexAttrib4Nuiv = _link_function('glVertexAttrib4Nuiv', None, [GLuint, POINTER(GLuint)], 'VERSION_2_0')

# GL/glext.h:4299
glVertexAttrib4Nusv = _link_function('glVertexAttrib4Nusv', None, [GLuint, POINTER(GLushort)], 'VERSION_2_0')

# GL/glext.h:4300
glVertexAttrib4bv = _link_function('glVertexAttrib4bv', None, [GLuint, POINTER(GLbyte)], 'VERSION_2_0')

# GL/glext.h:4301
glVertexAttrib4d = _link_function('glVertexAttrib4d', None, [GLuint, GLdouble, GLdouble, GLdouble, GLdouble], 'VERSION_2_0')

# GL/glext.h:4302
glVertexAttrib4dv = _link_function('glVertexAttrib4dv', None, [GLuint, POINTER(GLdouble)], 'VERSION_2_0')

# GL/glext.h:4303
glVertexAttrib4f = _link_function('glVertexAttrib4f', None, [GLuint, GLfloat, GLfloat, GLfloat, GLfloat], 'VERSION_2_0')

# GL/glext.h:4304
glVertexAttrib4fv = _link_function('glVertexAttrib4fv', None, [GLuint, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:4305
glVertexAttrib4iv = _link_function('glVertexAttrib4iv', None, [GLuint, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:4306
glVertexAttrib4s = _link_function('glVertexAttrib4s', None, [GLuint, GLshort, GLshort, GLshort, GLshort], 'VERSION_2_0')

# GL/glext.h:4307
glVertexAttrib4sv = _link_function('glVertexAttrib4sv', None, [GLuint, POINTER(GLshort)], 'VERSION_2_0')

# GL/glext.h:4308
glVertexAttrib4ubv = _link_function('glVertexAttrib4ubv', None, [GLuint, POINTER(GLubyte)], 'VERSION_2_0')

# GL/glext.h:4309
glVertexAttrib4uiv = _link_function('glVertexAttrib4uiv', None, [GLuint, POINTER(GLuint)], 'VERSION_2_0')

# GL/glext.h:4310
glVertexAttrib4usv = _link_function('glVertexAttrib4usv', None, [GLuint, POINTER(GLushort)], 'VERSION_2_0')

# GL/glext.h:4311
glVertexAttribPointer = _link_function('glVertexAttribPointer', None, [GLuint, GLint, GLenum, GLboolean, GLsizei, POINTER(GLvoid)], 'VERSION_2_0')

PFNGLBLENDEQUATIONSEPARATEPROC = CFUNCTYPE(None, GLenum, GLenum) 	# GL/glext.h:4313
PFNGLDRAWBUFFERSPROC = CFUNCTYPE(None, GLsizei, POINTER(GLenum)) 	# GL/glext.h:4314
PFNGLSTENCILOPSEPARATEPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:4315
PFNGLSTENCILFUNCSEPARATEPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLuint) 	# GL/glext.h:4316
PFNGLSTENCILMASKSEPARATEPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:4317
PFNGLATTACHSHADERPROC = CFUNCTYPE(None, GLuint, GLuint) 	# GL/glext.h:4318
PFNGLBINDATTRIBLOCATIONPROC = CFUNCTYPE(None, GLuint, GLuint, POINTER(GLchar)) 	# GL/glext.h:4319
PFNGLCOMPILESHADERPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:4320
PFNGLCREATEPROGRAMPROC = CFUNCTYPE(GLuint) 	# GL/glext.h:4321
PFNGLCREATESHADERPROC = CFUNCTYPE(GLuint, GLenum) 	# GL/glext.h:4322
PFNGLDELETEPROGRAMPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:4323
PFNGLDELETESHADERPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:4324
PFNGLDETACHSHADERPROC = CFUNCTYPE(None, GLuint, GLuint) 	# GL/glext.h:4325
PFNGLDISABLEVERTEXATTRIBARRAYPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:4326
PFNGLENABLEVERTEXATTRIBARRAYPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:4327
PFNGLGETACTIVEATTRIBPROC = CFUNCTYPE(None, GLuint, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLint), POINTER(GLenum), POINTER(GLchar)) 	# GL/glext.h:4328
PFNGLGETACTIVEUNIFORMPROC = CFUNCTYPE(None, GLuint, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLint), POINTER(GLenum), POINTER(GLchar)) 	# GL/glext.h:4329
PFNGLGETATTACHEDSHADERSPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLuint)) 	# GL/glext.h:4330
PFNGLGETATTRIBLOCATIONPROC = CFUNCTYPE(GLint, GLuint, POINTER(GLchar)) 	# GL/glext.h:4331
PFNGLGETPROGRAMIVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:4332
PFNGLGETPROGRAMINFOLOGPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)) 	# GL/glext.h:4333
PFNGLGETSHADERIVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:4334
PFNGLGETSHADERINFOLOGPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)) 	# GL/glext.h:4335
PFNGLGETSHADERSOURCEPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)) 	# GL/glext.h:4336
PFNGLGETUNIFORMLOCATIONPROC = CFUNCTYPE(GLint, GLuint, POINTER(GLchar)) 	# GL/glext.h:4337
PFNGLGETUNIFORMFVPROC = CFUNCTYPE(None, GLuint, GLint, POINTER(GLfloat)) 	# GL/glext.h:4338
PFNGLGETUNIFORMIVPROC = CFUNCTYPE(None, GLuint, GLint, POINTER(GLint)) 	# GL/glext.h:4339
PFNGLGETVERTEXATTRIBDVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLdouble)) 	# GL/glext.h:4340
PFNGLGETVERTEXATTRIBFVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4341
PFNGLGETVERTEXATTRIBIVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:4342
PFNGLGETVERTEXATTRIBPOINTERVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:4343
PFNGLISPROGRAMPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:4344
PFNGLISSHADERPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:4345
PFNGLLINKPROGRAMPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:4346
PFNGLSHADERSOURCEPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(POINTER(GLchar)), POINTER(GLint)) 	# GL/glext.h:4347
PFNGLUSEPROGRAMPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:4348
PFNGLUNIFORM1FPROC = CFUNCTYPE(None, GLint, GLfloat) 	# GL/glext.h:4349
PFNGLUNIFORM2FPROC = CFUNCTYPE(None, GLint, GLfloat, GLfloat) 	# GL/glext.h:4350
PFNGLUNIFORM3FPROC = CFUNCTYPE(None, GLint, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:4351
PFNGLUNIFORM4FPROC = CFUNCTYPE(None, GLint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:4352
PFNGLUNIFORM1IPROC = CFUNCTYPE(None, GLint, GLint) 	# GL/glext.h:4353
PFNGLUNIFORM2IPROC = CFUNCTYPE(None, GLint, GLint, GLint) 	# GL/glext.h:4354
PFNGLUNIFORM3IPROC = CFUNCTYPE(None, GLint, GLint, GLint, GLint) 	# GL/glext.h:4355
PFNGLUNIFORM4IPROC = CFUNCTYPE(None, GLint, GLint, GLint, GLint, GLint) 	# GL/glext.h:4356
PFNGLUNIFORM1FVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:4357
PFNGLUNIFORM2FVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:4358
PFNGLUNIFORM3FVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:4359
PFNGLUNIFORM4FVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:4360
PFNGLUNIFORM1IVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:4361
PFNGLUNIFORM2IVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:4362
PFNGLUNIFORM3IVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:4363
PFNGLUNIFORM4IVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:4364
PFNGLUNIFORMMATRIX2FVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:4365
PFNGLUNIFORMMATRIX3FVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:4366
PFNGLUNIFORMMATRIX4FVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:4367
PFNGLVALIDATEPROGRAMPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:4368
PFNGLVERTEXATTRIB1DPROC = CFUNCTYPE(None, GLuint, GLdouble) 	# GL/glext.h:4369
PFNGLVERTEXATTRIB1DVPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:4370
PFNGLVERTEXATTRIB1FPROC = CFUNCTYPE(None, GLuint, GLfloat) 	# GL/glext.h:4371
PFNGLVERTEXATTRIB1FVPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:4372
PFNGLVERTEXATTRIB1SPROC = CFUNCTYPE(None, GLuint, GLshort) 	# GL/glext.h:4373
PFNGLVERTEXATTRIB1SVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:4374
PFNGLVERTEXATTRIB2DPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble) 	# GL/glext.h:4375
PFNGLVERTEXATTRIB2DVPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:4376
PFNGLVERTEXATTRIB2FPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat) 	# GL/glext.h:4377
PFNGLVERTEXATTRIB2FVPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:4378
PFNGLVERTEXATTRIB2SPROC = CFUNCTYPE(None, GLuint, GLshort, GLshort) 	# GL/glext.h:4379
PFNGLVERTEXATTRIB2SVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:4380
PFNGLVERTEXATTRIB3DPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:4381
PFNGLVERTEXATTRIB3DVPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:4382
PFNGLVERTEXATTRIB3FPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:4383
PFNGLVERTEXATTRIB3FVPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:4384
PFNGLVERTEXATTRIB3SPROC = CFUNCTYPE(None, GLuint, GLshort, GLshort, GLshort) 	# GL/glext.h:4385
PFNGLVERTEXATTRIB3SVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:4386
PFNGLVERTEXATTRIB4NBVPROC = CFUNCTYPE(None, GLuint, POINTER(GLbyte)) 	# GL/glext.h:4387
PFNGLVERTEXATTRIB4NIVPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:4388
PFNGLVERTEXATTRIB4NSVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:4389
PFNGLVERTEXATTRIB4NUBPROC = CFUNCTYPE(None, GLuint, GLubyte, GLubyte, GLubyte, GLubyte) 	# GL/glext.h:4390
PFNGLVERTEXATTRIB4NUBVPROC = CFUNCTYPE(None, GLuint, POINTER(GLubyte)) 	# GL/glext.h:4391
PFNGLVERTEXATTRIB4NUIVPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:4392
PFNGLVERTEXATTRIB4NUSVPROC = CFUNCTYPE(None, GLuint, POINTER(GLushort)) 	# GL/glext.h:4393
PFNGLVERTEXATTRIB4BVPROC = CFUNCTYPE(None, GLuint, POINTER(GLbyte)) 	# GL/glext.h:4394
PFNGLVERTEXATTRIB4DPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:4395
PFNGLVERTEXATTRIB4DVPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:4396
PFNGLVERTEXATTRIB4FPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:4397
PFNGLVERTEXATTRIB4FVPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:4398
PFNGLVERTEXATTRIB4IVPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:4399
PFNGLVERTEXATTRIB4SPROC = CFUNCTYPE(None, GLuint, GLshort, GLshort, GLshort, GLshort) 	# GL/glext.h:4400
PFNGLVERTEXATTRIB4SVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:4401
PFNGLVERTEXATTRIB4UBVPROC = CFUNCTYPE(None, GLuint, POINTER(GLubyte)) 	# GL/glext.h:4402
PFNGLVERTEXATTRIB4UIVPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:4403
PFNGLVERTEXATTRIB4USVPROC = CFUNCTYPE(None, GLuint, POINTER(GLushort)) 	# GL/glext.h:4404
PFNGLVERTEXATTRIBPOINTERPROC = CFUNCTYPE(None, GLuint, GLint, GLenum, GLboolean, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:4405
# VERSION_2_1 (GL/glext.h:4408)
GL_VERSION_2_1 = 1 	# GL/glext.h:4409
# GL/glext.h:4411
glUniformMatrix2x3fv = _link_function('glUniformMatrix2x3fv', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'VERSION_2_1')

# GL/glext.h:4412
glUniformMatrix3x2fv = _link_function('glUniformMatrix3x2fv', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'VERSION_2_1')

# GL/glext.h:4413
glUniformMatrix2x4fv = _link_function('glUniformMatrix2x4fv', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'VERSION_2_1')

# GL/glext.h:4414
glUniformMatrix4x2fv = _link_function('glUniformMatrix4x2fv', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'VERSION_2_1')

# GL/glext.h:4415
glUniformMatrix3x4fv = _link_function('glUniformMatrix3x4fv', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'VERSION_2_1')

# GL/glext.h:4416
glUniformMatrix4x3fv = _link_function('glUniformMatrix4x3fv', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'VERSION_2_1')

PFNGLUNIFORMMATRIX2X3FVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:4418
PFNGLUNIFORMMATRIX3X2FVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:4419
PFNGLUNIFORMMATRIX2X4FVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:4420
PFNGLUNIFORMMATRIX4X2FVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:4421
PFNGLUNIFORMMATRIX3X4FVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:4422
PFNGLUNIFORMMATRIX4X3FVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:4423
# VERSION_3_0 (GL/glext.h:4426)
GL_VERSION_3_0 = 1 	# GL/glext.h:4427
# GL/glext.h:4433
glColorMaski = _link_function('glColorMaski', None, [GLuint, GLboolean, GLboolean, GLboolean, GLboolean], 'VERSION_3_0')

# GL/glext.h:4434
glGetBooleani_v = _link_function('glGetBooleani_v', None, [GLenum, GLuint, POINTER(GLboolean)], 'VERSION_3_0')

# GL/glext.h:4435
glGetIntegeri_v = _link_function('glGetIntegeri_v', None, [GLenum, GLuint, POINTER(GLint)], 'VERSION_3_0')

# GL/glext.h:4436
glEnablei = _link_function('glEnablei', None, [GLenum, GLuint], 'VERSION_3_0')

# GL/glext.h:4437
glDisablei = _link_function('glDisablei', None, [GLenum, GLuint], 'VERSION_3_0')

# GL/glext.h:4438
glIsEnabledi = _link_function('glIsEnabledi', GLboolean, [GLenum, GLuint], 'VERSION_3_0')

# GL/glext.h:4439
glBeginTransformFeedback = _link_function('glBeginTransformFeedback', None, [GLenum], 'VERSION_3_0')

# GL/glext.h:4440
glEndTransformFeedback = _link_function('glEndTransformFeedback', None, [], 'VERSION_3_0')

# GL/glext.h:4441
glBindBufferRange = _link_function('glBindBufferRange', None, [GLenum, GLuint, GLuint, GLintptr, GLsizeiptr], 'VERSION_3_0')

# GL/glext.h:4442
glBindBufferBase = _link_function('glBindBufferBase', None, [GLenum, GLuint, GLuint], 'VERSION_3_0')

# GL/glext.h:4443
glTransformFeedbackVaryings = _link_function('glTransformFeedbackVaryings', None, [GLuint, GLsizei, POINTER(POINTER(GLchar)), GLenum], 'VERSION_3_0')

# GL/glext.h:4444
glGetTransformFeedbackVarying = _link_function('glGetTransformFeedbackVarying', None, [GLuint, GLuint, POINTER(GLint)], 'VERSION_3_0')

# GL/glext.h:4445
glClampColor = _link_function('glClampColor', None, [GLenum, GLenum], 'VERSION_3_0')

# GL/glext.h:4446
glBeginConditionalRender = _link_function('glBeginConditionalRender', None, [GLuint, GLenum], 'VERSION_3_0')

# GL/glext.h:4447
glEndConditionalRender = _link_function('glEndConditionalRender', None, [], 'VERSION_3_0')

# GL/glext.h:4448
glVertexAttribI1i = _link_function('glVertexAttribI1i', None, [GLuint, GLint], 'VERSION_3_0')

# GL/glext.h:4449
glVertexAttribI2i = _link_function('glVertexAttribI2i', None, [GLuint, GLint, GLint], 'VERSION_3_0')

# GL/glext.h:4450
glVertexAttribI3i = _link_function('glVertexAttribI3i', None, [GLuint, GLint, GLint, GLint], 'VERSION_3_0')

# GL/glext.h:4451
glVertexAttribI4i = _link_function('glVertexAttribI4i', None, [GLuint, GLint, GLint, GLint, GLint], 'VERSION_3_0')

# GL/glext.h:4452
glVertexAttribI1ui = _link_function('glVertexAttribI1ui', None, [GLuint, GLuint], 'VERSION_3_0')

# GL/glext.h:4453
glVertexAttribI2ui = _link_function('glVertexAttribI2ui', None, [GLuint, GLuint, GLuint], 'VERSION_3_0')

# GL/glext.h:4454
glVertexAttribI3ui = _link_function('glVertexAttribI3ui', None, [GLuint, GLuint, GLuint, GLuint], 'VERSION_3_0')

# GL/glext.h:4455
glVertexAttribI4ui = _link_function('glVertexAttribI4ui', None, [GLuint, GLuint, GLuint, GLuint, GLuint], 'VERSION_3_0')

# GL/glext.h:4456
glVertexAttribI1iv = _link_function('glVertexAttribI1iv', None, [GLuint, POINTER(GLint)], 'VERSION_3_0')

# GL/glext.h:4457
glVertexAttribI2iv = _link_function('glVertexAttribI2iv', None, [GLuint, POINTER(GLint)], 'VERSION_3_0')

# GL/glext.h:4458
glVertexAttribI3iv = _link_function('glVertexAttribI3iv', None, [GLuint, POINTER(GLint)], 'VERSION_3_0')

# GL/glext.h:4459
glVertexAttribI4iv = _link_function('glVertexAttribI4iv', None, [GLuint, POINTER(GLint)], 'VERSION_3_0')

# GL/glext.h:4460
glVertexAttribI1uiv = _link_function('glVertexAttribI1uiv', None, [GLuint, POINTER(GLuint)], 'VERSION_3_0')

# GL/glext.h:4461
glVertexAttribI2uiv = _link_function('glVertexAttribI2uiv', None, [GLuint, POINTER(GLuint)], 'VERSION_3_0')

# GL/glext.h:4462
glVertexAttribI3uiv = _link_function('glVertexAttribI3uiv', None, [GLuint, POINTER(GLuint)], 'VERSION_3_0')

# GL/glext.h:4463
glVertexAttribI4uiv = _link_function('glVertexAttribI4uiv', None, [GLuint, POINTER(GLuint)], 'VERSION_3_0')

# GL/glext.h:4464
glVertexAttribI4bv = _link_function('glVertexAttribI4bv', None, [GLuint, POINTER(GLbyte)], 'VERSION_3_0')

# GL/glext.h:4465
glVertexAttribI4sv = _link_function('glVertexAttribI4sv', None, [GLuint, POINTER(GLshort)], 'VERSION_3_0')

# GL/glext.h:4466
glVertexAttribI4ubv = _link_function('glVertexAttribI4ubv', None, [GLuint, POINTER(GLubyte)], 'VERSION_3_0')

# GL/glext.h:4467
glVertexAttribI4usv = _link_function('glVertexAttribI4usv', None, [GLuint, POINTER(GLushort)], 'VERSION_3_0')

# GL/glext.h:4468
glVertexAttribIPointer = _link_function('glVertexAttribIPointer', None, [GLuint, GLint, GLenum, GLsizei, POINTER(GLvoid)], 'VERSION_3_0')

# GL/glext.h:4469
glGetVertexAttribIiv = _link_function('glGetVertexAttribIiv', None, [GLuint, GLenum, POINTER(GLint)], 'VERSION_3_0')

# GL/glext.h:4470
glGetVertexAttribIuiv = _link_function('glGetVertexAttribIuiv', None, [GLuint, GLenum, POINTER(GLuint)], 'VERSION_3_0')

# GL/glext.h:4471
glGetUniformuiv = _link_function('glGetUniformuiv', None, [GLuint, GLint, POINTER(GLuint)], 'VERSION_3_0')

# GL/glext.h:4472
glBindFragDataLocation = _link_function('glBindFragDataLocation', None, [GLuint, GLuint, POINTER(GLchar)], 'VERSION_3_0')

# GL/glext.h:4473
glGetFragDataLocation = _link_function('glGetFragDataLocation', GLint, [GLuint, POINTER(GLchar)], 'VERSION_3_0')

# GL/glext.h:4474
glUniform1ui = _link_function('glUniform1ui', None, [GLint, GLuint], 'VERSION_3_0')

# GL/glext.h:4475
glUniform2ui = _link_function('glUniform2ui', None, [GLint, GLuint, GLuint], 'VERSION_3_0')

# GL/glext.h:4476
glUniform3ui = _link_function('glUniform3ui', None, [GLint, GLuint, GLuint, GLuint], 'VERSION_3_0')

# GL/glext.h:4477
glUniform4ui = _link_function('glUniform4ui', None, [GLint, GLuint, GLuint, GLuint, GLuint], 'VERSION_3_0')

# GL/glext.h:4478
glUniform1uiv = _link_function('glUniform1uiv', None, [GLint, GLsizei, POINTER(GLuint)], 'VERSION_3_0')

# GL/glext.h:4479
glUniform2uiv = _link_function('glUniform2uiv', None, [GLint, GLsizei, POINTER(GLuint)], 'VERSION_3_0')

# GL/glext.h:4480
glUniform3uiv = _link_function('glUniform3uiv', None, [GLint, GLsizei, POINTER(GLuint)], 'VERSION_3_0')

# GL/glext.h:4481
glUniform4uiv = _link_function('glUniform4uiv', None, [GLint, GLsizei, POINTER(GLuint)], 'VERSION_3_0')

# GL/glext.h:4482
glTexParameterIiv = _link_function('glTexParameterIiv', None, [GLenum, GLenum, POINTER(GLint)], 'VERSION_3_0')

# GL/glext.h:4483
glTexParameterIuiv = _link_function('glTexParameterIuiv', None, [GLenum, GLenum, POINTER(GLuint)], 'VERSION_3_0')

# GL/glext.h:4484
glGetTexParameterIiv = _link_function('glGetTexParameterIiv', None, [GLenum, GLenum, POINTER(GLint)], 'VERSION_3_0')

# GL/glext.h:4485
glGetTexParameterIuiv = _link_function('glGetTexParameterIuiv', None, [GLenum, GLenum, POINTER(GLuint)], 'VERSION_3_0')

# GL/glext.h:4486
glClearBufferiv = _link_function('glClearBufferiv', None, [GLenum, GLint, POINTER(GLint)], 'VERSION_3_0')

# GL/glext.h:4487
glClearBufferuiv = _link_function('glClearBufferuiv', None, [GLenum, GLint, POINTER(GLuint)], 'VERSION_3_0')

# GL/glext.h:4488
glClearBufferfv = _link_function('glClearBufferfv', None, [GLenum, GLint, POINTER(GLfloat)], 'VERSION_3_0')

# GL/glext.h:4489
glClearBufferfi = _link_function('glClearBufferfi', None, [GLenum, GLint, GLfloat, GLint], 'VERSION_3_0')

# GL/glext.h:4490
glGetStringi = _link_function('glGetStringi', POINTER(GLubyte), [GLenum, GLuint], 'VERSION_3_0')

PFNGLCOLORMASKIPROC = CFUNCTYPE(None, GLuint, GLboolean, GLboolean, GLboolean, GLboolean) 	# GL/glext.h:4492
PFNGLGETBOOLEANI_VPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLboolean)) 	# GL/glext.h:4493
PFNGLGETINTEGERI_VPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLint)) 	# GL/glext.h:4494
PFNGLENABLEIPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:4495
PFNGLDISABLEIPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:4496
PFNGLISENABLEDIPROC = CFUNCTYPE(GLboolean, GLenum, GLuint) 	# GL/glext.h:4497
PFNGLBEGINTRANSFORMFEEDBACKPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:4498
PFNGLENDTRANSFORMFEEDBACKPROC = CFUNCTYPE(None) 	# GL/glext.h:4499
PFNGLBINDBUFFERRANGEPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLintptr, GLsizeiptr) 	# GL/glext.h:4500
PFNGLBINDBUFFERBASEPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint) 	# GL/glext.h:4501
PFNGLTRANSFORMFEEDBACKVARYINGSPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLint), GLenum) 	# GL/glext.h:4502
PFNGLGETTRANSFORMFEEDBACKVARYINGPROC = CFUNCTYPE(None, GLuint, GLuint, POINTER(GLint)) 	# GL/glext.h:4503
PFNGLCLAMPCOLORPROC = CFUNCTYPE(None, GLenum, GLenum) 	# GL/glext.h:4504
PFNGLBEGINCONDITIONALRENDERPROC = CFUNCTYPE(None, GLuint, GLenum) 	# GL/glext.h:4505
PFNGLENDCONDITIONALRENDERPROC = CFUNCTYPE(None) 	# GL/glext.h:4506
PFNGLVERTEXATTRIBI1IPROC = CFUNCTYPE(None, GLuint, GLint) 	# GL/glext.h:4507
PFNGLVERTEXATTRIBI2IPROC = CFUNCTYPE(None, GLuint, GLint, GLint) 	# GL/glext.h:4508
PFNGLVERTEXATTRIBI3IPROC = CFUNCTYPE(None, GLuint, GLint, GLint, GLint) 	# GL/glext.h:4509
PFNGLVERTEXATTRIBI4IPROC = CFUNCTYPE(None, GLuint, GLint, GLint, GLint, GLint) 	# GL/glext.h:4510
PFNGLVERTEXATTRIBI1UIPROC = CFUNCTYPE(None, GLuint, GLuint) 	# GL/glext.h:4511
PFNGLVERTEXATTRIBI2UIPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint) 	# GL/glext.h:4512
PFNGLVERTEXATTRIBI3UIPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:4513
PFNGLVERTEXATTRIBI4UIPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:4514
PFNGLVERTEXATTRIBI1IVPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:4515
PFNGLVERTEXATTRIBI2IVPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:4516
PFNGLVERTEXATTRIBI3IVPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:4517
PFNGLVERTEXATTRIBI4IVPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:4518
PFNGLVERTEXATTRIBI1UIVPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:4519
PFNGLVERTEXATTRIBI2UIVPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:4520
PFNGLVERTEXATTRIBI3UIVPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:4521
PFNGLVERTEXATTRIBI4UIVPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:4522
PFNGLVERTEXATTRIBI4BVPROC = CFUNCTYPE(None, GLuint, POINTER(GLbyte)) 	# GL/glext.h:4523
PFNGLVERTEXATTRIBI4SVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:4524
PFNGLVERTEXATTRIBI4UBVPROC = CFUNCTYPE(None, GLuint, POINTER(GLubyte)) 	# GL/glext.h:4525
PFNGLVERTEXATTRIBI4USVPROC = CFUNCTYPE(None, GLuint, POINTER(GLushort)) 	# GL/glext.h:4526
PFNGLVERTEXATTRIBIPOINTERPROC = CFUNCTYPE(None, GLuint, GLint, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:4527
PFNGLGETVERTEXATTRIBIIVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:4528
PFNGLGETVERTEXATTRIBIUIVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLuint)) 	# GL/glext.h:4529
PFNGLGETUNIFORMUIVPROC = CFUNCTYPE(None, GLuint, GLint, POINTER(GLuint)) 	# GL/glext.h:4530
PFNGLBINDFRAGDATALOCATIONPROC = CFUNCTYPE(None, GLuint, GLuint, POINTER(GLchar)) 	# GL/glext.h:4531
PFNGLGETFRAGDATALOCATIONPROC = CFUNCTYPE(GLint, GLuint, POINTER(GLchar)) 	# GL/glext.h:4532
PFNGLUNIFORM1UIPROC = CFUNCTYPE(None, GLint, GLuint) 	# GL/glext.h:4533
PFNGLUNIFORM2UIPROC = CFUNCTYPE(None, GLint, GLuint, GLuint) 	# GL/glext.h:4534
PFNGLUNIFORM3UIPROC = CFUNCTYPE(None, GLint, GLuint, GLuint, GLuint) 	# GL/glext.h:4535
PFNGLUNIFORM4UIPROC = CFUNCTYPE(None, GLint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:4536
PFNGLUNIFORM1UIVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:4537
PFNGLUNIFORM2UIVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:4538
PFNGLUNIFORM3UIVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:4539
PFNGLUNIFORM4UIVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:4540
PFNGLTEXPARAMETERIIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:4541
PFNGLTEXPARAMETERIUIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLuint)) 	# GL/glext.h:4542
PFNGLGETTEXPARAMETERIIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:4543
PFNGLGETTEXPARAMETERIUIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLuint)) 	# GL/glext.h:4544
PFNGLCLEARBUFFERIVPROC = CFUNCTYPE(None, GLenum, GLint, POINTER(GLint)) 	# GL/glext.h:4545
PFNGLCLEARBUFFERUIVPROC = CFUNCTYPE(None, GLenum, GLint, POINTER(GLuint)) 	# GL/glext.h:4546
PFNGLCLEARBUFFERFVPROC = CFUNCTYPE(None, GLenum, GLint, POINTER(GLfloat)) 	# GL/glext.h:4547
PFNGLCLEARBUFFERFIPROC = CFUNCTYPE(None, GLenum, GLint, GLfloat, GLint) 	# GL/glext.h:4548
PFNGLGETSTRINGIPROC = CFUNCTYPE(POINTER(GLubyte), GLenum, GLuint) 	# GL/glext.h:4549
# ARB_multitexture (GL/glext.h:4552)
# ARB_transpose_matrix (GL/glext.h:4626)
GL_ARB_transpose_matrix = 1 	# GL/glext.h:4627
# GL/glext.h:4629
glLoadTransposeMatrixfARB = _link_function('glLoadTransposeMatrixfARB', None, [POINTER(GLfloat)], 'ARB_transpose_matrix')

# GL/glext.h:4630
glLoadTransposeMatrixdARB = _link_function('glLoadTransposeMatrixdARB', None, [POINTER(GLdouble)], 'ARB_transpose_matrix')

# GL/glext.h:4631
glMultTransposeMatrixfARB = _link_function('glMultTransposeMatrixfARB', None, [POINTER(GLfloat)], 'ARB_transpose_matrix')

# GL/glext.h:4632
glMultTransposeMatrixdARB = _link_function('glMultTransposeMatrixdARB', None, [POINTER(GLdouble)], 'ARB_transpose_matrix')

PFNGLLOADTRANSPOSEMATRIXFARBPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:4634
PFNGLLOADTRANSPOSEMATRIXDARBPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:4635
PFNGLMULTTRANSPOSEMATRIXFARBPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:4636
PFNGLMULTTRANSPOSEMATRIXDARBPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:4637
# ARB_multisample (GL/glext.h:4640)
GL_ARB_multisample = 1 	# GL/glext.h:4641
GLclampf = c_float 	# /usr/include/GL/gl.h:133
# GL/glext.h:4643
glSampleCoverageARB = _link_function('glSampleCoverageARB', None, [GLclampf, GLboolean], 'ARB_multisample')

PFNGLSAMPLECOVERAGEARBPROC = CFUNCTYPE(None, GLclampf, GLboolean) 	# GL/glext.h:4645
# ARB_texture_env_add (GL/glext.h:4648)
GL_ARB_texture_env_add = 1 	# GL/glext.h:4649
# ARB_texture_cube_map (GL/glext.h:4652)
GL_ARB_texture_cube_map = 1 	# GL/glext.h:4653
# ARB_texture_compression (GL/glext.h:4656)
GL_ARB_texture_compression = 1 	# GL/glext.h:4657
# GL/glext.h:4659
glCompressedTexImage3DARB = _link_function('glCompressedTexImage3DARB', None, [GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)], 'ARB_texture_compression')

# GL/glext.h:4660
glCompressedTexImage2DARB = _link_function('glCompressedTexImage2DARB', None, [GLenum, GLint, GLenum, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)], 'ARB_texture_compression')

# GL/glext.h:4661
glCompressedTexImage1DARB = _link_function('glCompressedTexImage1DARB', None, [GLenum, GLint, GLenum, GLsizei, GLint, GLsizei, POINTER(GLvoid)], 'ARB_texture_compression')

# GL/glext.h:4662
glCompressedTexSubImage3DARB = _link_function('glCompressedTexSubImage3DARB', None, [GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)], 'ARB_texture_compression')

# GL/glext.h:4663
glCompressedTexSubImage2DARB = _link_function('glCompressedTexSubImage2DARB', None, [GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)], 'ARB_texture_compression')

# GL/glext.h:4664
glCompressedTexSubImage1DARB = _link_function('glCompressedTexSubImage1DARB', None, [GLenum, GLint, GLint, GLsizei, GLenum, GLsizei, POINTER(GLvoid)], 'ARB_texture_compression')

# GL/glext.h:4665
glGetCompressedTexImageARB = _link_function('glGetCompressedTexImageARB', None, [GLenum, GLint, POINTER(GLvoid)], 'ARB_texture_compression')

PFNGLCOMPRESSEDTEXIMAGE3DARBPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:4667
PFNGLCOMPRESSEDTEXIMAGE2DARBPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:4668
PFNGLCOMPRESSEDTEXIMAGE1DARBPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLsizei, GLint, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:4669
PFNGLCOMPRESSEDTEXSUBIMAGE3DARBPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:4670
PFNGLCOMPRESSEDTEXSUBIMAGE2DARBPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:4671
PFNGLCOMPRESSEDTEXSUBIMAGE1DARBPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLsizei, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:4672
PFNGLGETCOMPRESSEDTEXIMAGEARBPROC = CFUNCTYPE(None, GLenum, GLint, POINTER(GLvoid)) 	# GL/glext.h:4673
# ARB_texture_border_clamp (GL/glext.h:4676)
GL_ARB_texture_border_clamp = 1 	# GL/glext.h:4677
# ARB_point_parameters (GL/glext.h:4680)
GL_ARB_point_parameters = 1 	# GL/glext.h:4681
# GL/glext.h:4683
glPointParameterfARB = _link_function('glPointParameterfARB', None, [GLenum, GLfloat], 'ARB_point_parameters')

# GL/glext.h:4684
glPointParameterfvARB = _link_function('glPointParameterfvARB', None, [GLenum, POINTER(GLfloat)], 'ARB_point_parameters')

PFNGLPOINTPARAMETERFARBPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:4686
PFNGLPOINTPARAMETERFVARBPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4687
# ARB_vertex_blend (GL/glext.h:4690)
GL_ARB_vertex_blend = 1 	# GL/glext.h:4691
# GL/glext.h:4693
glWeightbvARB = _link_function('glWeightbvARB', None, [GLint, POINTER(GLbyte)], 'ARB_vertex_blend')

# GL/glext.h:4694
glWeightsvARB = _link_function('glWeightsvARB', None, [GLint, POINTER(GLshort)], 'ARB_vertex_blend')

# GL/glext.h:4695
glWeightivARB = _link_function('glWeightivARB', None, [GLint, POINTER(GLint)], 'ARB_vertex_blend')

# GL/glext.h:4696
glWeightfvARB = _link_function('glWeightfvARB', None, [GLint, POINTER(GLfloat)], 'ARB_vertex_blend')

# GL/glext.h:4697
glWeightdvARB = _link_function('glWeightdvARB', None, [GLint, POINTER(GLdouble)], 'ARB_vertex_blend')

# GL/glext.h:4698
glWeightubvARB = _link_function('glWeightubvARB', None, [GLint, POINTER(GLubyte)], 'ARB_vertex_blend')

# GL/glext.h:4699
glWeightusvARB = _link_function('glWeightusvARB', None, [GLint, POINTER(GLushort)], 'ARB_vertex_blend')

# GL/glext.h:4700
glWeightuivARB = _link_function('glWeightuivARB', None, [GLint, POINTER(GLuint)], 'ARB_vertex_blend')

# GL/glext.h:4701
glWeightPointerARB = _link_function('glWeightPointerARB', None, [GLint, GLenum, GLsizei, POINTER(GLvoid)], 'ARB_vertex_blend')

# GL/glext.h:4702
glVertexBlendARB = _link_function('glVertexBlendARB', None, [GLint], 'ARB_vertex_blend')

PFNGLWEIGHTBVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLbyte)) 	# GL/glext.h:4704
PFNGLWEIGHTSVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLshort)) 	# GL/glext.h:4705
PFNGLWEIGHTIVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLint)) 	# GL/glext.h:4706
PFNGLWEIGHTFVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLfloat)) 	# GL/glext.h:4707
PFNGLWEIGHTDVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLdouble)) 	# GL/glext.h:4708
PFNGLWEIGHTUBVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLubyte)) 	# GL/glext.h:4709
PFNGLWEIGHTUSVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLushort)) 	# GL/glext.h:4710
PFNGLWEIGHTUIVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLuint)) 	# GL/glext.h:4711
PFNGLWEIGHTPOINTERARBPROC = CFUNCTYPE(None, GLint, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:4712
PFNGLVERTEXBLENDARBPROC = CFUNCTYPE(None, GLint) 	# GL/glext.h:4713
# ARB_matrix_palette (GL/glext.h:4716)
GL_ARB_matrix_palette = 1 	# GL/glext.h:4717
# GL/glext.h:4719
glCurrentPaletteMatrixARB = _link_function('glCurrentPaletteMatrixARB', None, [GLint], 'ARB_matrix_palette')

# GL/glext.h:4720
glMatrixIndexubvARB = _link_function('glMatrixIndexubvARB', None, [GLint, POINTER(GLubyte)], 'ARB_matrix_palette')

# GL/glext.h:4721
glMatrixIndexusvARB = _link_function('glMatrixIndexusvARB', None, [GLint, POINTER(GLushort)], 'ARB_matrix_palette')

# GL/glext.h:4722
glMatrixIndexuivARB = _link_function('glMatrixIndexuivARB', None, [GLint, POINTER(GLuint)], 'ARB_matrix_palette')

# GL/glext.h:4723
glMatrixIndexPointerARB = _link_function('glMatrixIndexPointerARB', None, [GLint, GLenum, GLsizei, POINTER(GLvoid)], 'ARB_matrix_palette')

PFNGLCURRENTPALETTEMATRIXARBPROC = CFUNCTYPE(None, GLint) 	# GL/glext.h:4725
PFNGLMATRIXINDEXUBVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLubyte)) 	# GL/glext.h:4726
PFNGLMATRIXINDEXUSVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLushort)) 	# GL/glext.h:4727
PFNGLMATRIXINDEXUIVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLuint)) 	# GL/glext.h:4728
PFNGLMATRIXINDEXPOINTERARBPROC = CFUNCTYPE(None, GLint, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:4729
# ARB_texture_env_combine (GL/glext.h:4732)
GL_ARB_texture_env_combine = 1 	# GL/glext.h:4733
# ARB_texture_env_crossbar (GL/glext.h:4736)
GL_ARB_texture_env_crossbar = 1 	# GL/glext.h:4737
# ARB_texture_env_dot3 (GL/glext.h:4740)
GL_ARB_texture_env_dot3 = 1 	# GL/glext.h:4741
# ARB_texture_mirrored_repeat (GL/glext.h:4744)
GL_ARB_texture_mirrored_repeat = 1 	# GL/glext.h:4745
# ARB_depth_texture (GL/glext.h:4748)
GL_ARB_depth_texture = 1 	# GL/glext.h:4749
# ARB_shadow (GL/glext.h:4752)
GL_ARB_shadow = 1 	# GL/glext.h:4753
# ARB_shadow_ambient (GL/glext.h:4756)
GL_ARB_shadow_ambient = 1 	# GL/glext.h:4757
# ARB_window_pos (GL/glext.h:4760)
GL_ARB_window_pos = 1 	# GL/glext.h:4761
# GL/glext.h:4763
glWindowPos2dARB = _link_function('glWindowPos2dARB', None, [GLdouble, GLdouble], 'ARB_window_pos')

# GL/glext.h:4764
glWindowPos2dvARB = _link_function('glWindowPos2dvARB', None, [POINTER(GLdouble)], 'ARB_window_pos')

# GL/glext.h:4765
glWindowPos2fARB = _link_function('glWindowPos2fARB', None, [GLfloat, GLfloat], 'ARB_window_pos')

# GL/glext.h:4766
glWindowPos2fvARB = _link_function('glWindowPos2fvARB', None, [POINTER(GLfloat)], 'ARB_window_pos')

# GL/glext.h:4767
glWindowPos2iARB = _link_function('glWindowPos2iARB', None, [GLint, GLint], 'ARB_window_pos')

# GL/glext.h:4768
glWindowPos2ivARB = _link_function('glWindowPos2ivARB', None, [POINTER(GLint)], 'ARB_window_pos')

# GL/glext.h:4769
glWindowPos2sARB = _link_function('glWindowPos2sARB', None, [GLshort, GLshort], 'ARB_window_pos')

# GL/glext.h:4770
glWindowPos2svARB = _link_function('glWindowPos2svARB', None, [POINTER(GLshort)], 'ARB_window_pos')

# GL/glext.h:4771
glWindowPos3dARB = _link_function('glWindowPos3dARB', None, [GLdouble, GLdouble, GLdouble], 'ARB_window_pos')

# GL/glext.h:4772
glWindowPos3dvARB = _link_function('glWindowPos3dvARB', None, [POINTER(GLdouble)], 'ARB_window_pos')

# GL/glext.h:4773
glWindowPos3fARB = _link_function('glWindowPos3fARB', None, [GLfloat, GLfloat, GLfloat], 'ARB_window_pos')

# GL/glext.h:4774
glWindowPos3fvARB = _link_function('glWindowPos3fvARB', None, [POINTER(GLfloat)], 'ARB_window_pos')

# GL/glext.h:4775
glWindowPos3iARB = _link_function('glWindowPos3iARB', None, [GLint, GLint, GLint], 'ARB_window_pos')

# GL/glext.h:4776
glWindowPos3ivARB = _link_function('glWindowPos3ivARB', None, [POINTER(GLint)], 'ARB_window_pos')

# GL/glext.h:4777
glWindowPos3sARB = _link_function('glWindowPos3sARB', None, [GLshort, GLshort, GLshort], 'ARB_window_pos')

# GL/glext.h:4778
glWindowPos3svARB = _link_function('glWindowPos3svARB', None, [POINTER(GLshort)], 'ARB_window_pos')

PFNGLWINDOWPOS2DARBPROC = CFUNCTYPE(None, GLdouble, GLdouble) 	# GL/glext.h:4780
PFNGLWINDOWPOS2DVARBPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:4781
PFNGLWINDOWPOS2FARBPROC = CFUNCTYPE(None, GLfloat, GLfloat) 	# GL/glext.h:4782
PFNGLWINDOWPOS2FVARBPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:4783
PFNGLWINDOWPOS2IARBPROC = CFUNCTYPE(None, GLint, GLint) 	# GL/glext.h:4784
PFNGLWINDOWPOS2IVARBPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:4785
PFNGLWINDOWPOS2SARBPROC = CFUNCTYPE(None, GLshort, GLshort) 	# GL/glext.h:4786
PFNGLWINDOWPOS2SVARBPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:4787
PFNGLWINDOWPOS3DARBPROC = CFUNCTYPE(None, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:4788
PFNGLWINDOWPOS3DVARBPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:4789
PFNGLWINDOWPOS3FARBPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:4790
PFNGLWINDOWPOS3FVARBPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:4791
PFNGLWINDOWPOS3IARBPROC = CFUNCTYPE(None, GLint, GLint, GLint) 	# GL/glext.h:4792
PFNGLWINDOWPOS3IVARBPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:4793
PFNGLWINDOWPOS3SARBPROC = CFUNCTYPE(None, GLshort, GLshort, GLshort) 	# GL/glext.h:4794
PFNGLWINDOWPOS3SVARBPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:4795
# ARB_vertex_program (GL/glext.h:4798)
GL_ARB_vertex_program = 1 	# GL/glext.h:4799
# GL/glext.h:4801
glVertexAttrib1dARB = _link_function('glVertexAttrib1dARB', None, [GLuint, GLdouble], 'ARB_vertex_program')

# GL/glext.h:4802
glVertexAttrib1dvARB = _link_function('glVertexAttrib1dvARB', None, [GLuint, POINTER(GLdouble)], 'ARB_vertex_program')

# GL/glext.h:4803
glVertexAttrib1fARB = _link_function('glVertexAttrib1fARB', None, [GLuint, GLfloat], 'ARB_vertex_program')

# GL/glext.h:4804
glVertexAttrib1fvARB = _link_function('glVertexAttrib1fvARB', None, [GLuint, POINTER(GLfloat)], 'ARB_vertex_program')

# GL/glext.h:4805
glVertexAttrib1sARB = _link_function('glVertexAttrib1sARB', None, [GLuint, GLshort], 'ARB_vertex_program')

# GL/glext.h:4806
glVertexAttrib1svARB = _link_function('glVertexAttrib1svARB', None, [GLuint, POINTER(GLshort)], 'ARB_vertex_program')

# GL/glext.h:4807
glVertexAttrib2dARB = _link_function('glVertexAttrib2dARB', None, [GLuint, GLdouble, GLdouble], 'ARB_vertex_program')

# GL/glext.h:4808
glVertexAttrib2dvARB = _link_function('glVertexAttrib2dvARB', None, [GLuint, POINTER(GLdouble)], 'ARB_vertex_program')

# GL/glext.h:4809
glVertexAttrib2fARB = _link_function('glVertexAttrib2fARB', None, [GLuint, GLfloat, GLfloat], 'ARB_vertex_program')

# GL/glext.h:4810
glVertexAttrib2fvARB = _link_function('glVertexAttrib2fvARB', None, [GLuint, POINTER(GLfloat)], 'ARB_vertex_program')

# GL/glext.h:4811
glVertexAttrib2sARB = _link_function('glVertexAttrib2sARB', None, [GLuint, GLshort, GLshort], 'ARB_vertex_program')

# GL/glext.h:4812
glVertexAttrib2svARB = _link_function('glVertexAttrib2svARB', None, [GLuint, POINTER(GLshort)], 'ARB_vertex_program')

# GL/glext.h:4813
glVertexAttrib3dARB = _link_function('glVertexAttrib3dARB', None, [GLuint, GLdouble, GLdouble, GLdouble], 'ARB_vertex_program')

# GL/glext.h:4814
glVertexAttrib3dvARB = _link_function('glVertexAttrib3dvARB', None, [GLuint, POINTER(GLdouble)], 'ARB_vertex_program')

# GL/glext.h:4815
glVertexAttrib3fARB = _link_function('glVertexAttrib3fARB', None, [GLuint, GLfloat, GLfloat, GLfloat], 'ARB_vertex_program')

# GL/glext.h:4816
glVertexAttrib3fvARB = _link_function('glVertexAttrib3fvARB', None, [GLuint, POINTER(GLfloat)], 'ARB_vertex_program')

# GL/glext.h:4817
glVertexAttrib3sARB = _link_function('glVertexAttrib3sARB', None, [GLuint, GLshort, GLshort, GLshort], 'ARB_vertex_program')

# GL/glext.h:4818
glVertexAttrib3svARB = _link_function('glVertexAttrib3svARB', None, [GLuint, POINTER(GLshort)], 'ARB_vertex_program')

# GL/glext.h:4819
glVertexAttrib4NbvARB = _link_function('glVertexAttrib4NbvARB', None, [GLuint, POINTER(GLbyte)], 'ARB_vertex_program')

# GL/glext.h:4820
glVertexAttrib4NivARB = _link_function('glVertexAttrib4NivARB', None, [GLuint, POINTER(GLint)], 'ARB_vertex_program')

# GL/glext.h:4821
glVertexAttrib4NsvARB = _link_function('glVertexAttrib4NsvARB', None, [GLuint, POINTER(GLshort)], 'ARB_vertex_program')

# GL/glext.h:4822
glVertexAttrib4NubARB = _link_function('glVertexAttrib4NubARB', None, [GLuint, GLubyte, GLubyte, GLubyte, GLubyte], 'ARB_vertex_program')

# GL/glext.h:4823
glVertexAttrib4NubvARB = _link_function('glVertexAttrib4NubvARB', None, [GLuint, POINTER(GLubyte)], 'ARB_vertex_program')

# GL/glext.h:4824
glVertexAttrib4NuivARB = _link_function('glVertexAttrib4NuivARB', None, [GLuint, POINTER(GLuint)], 'ARB_vertex_program')

# GL/glext.h:4825
glVertexAttrib4NusvARB = _link_function('glVertexAttrib4NusvARB', None, [GLuint, POINTER(GLushort)], 'ARB_vertex_program')

# GL/glext.h:4826
glVertexAttrib4bvARB = _link_function('glVertexAttrib4bvARB', None, [GLuint, POINTER(GLbyte)], 'ARB_vertex_program')

# GL/glext.h:4827
glVertexAttrib4dARB = _link_function('glVertexAttrib4dARB', None, [GLuint, GLdouble, GLdouble, GLdouble, GLdouble], 'ARB_vertex_program')

# GL/glext.h:4828
glVertexAttrib4dvARB = _link_function('glVertexAttrib4dvARB', None, [GLuint, POINTER(GLdouble)], 'ARB_vertex_program')

# GL/glext.h:4829
glVertexAttrib4fARB = _link_function('glVertexAttrib4fARB', None, [GLuint, GLfloat, GLfloat, GLfloat, GLfloat], 'ARB_vertex_program')

# GL/glext.h:4830
glVertexAttrib4fvARB = _link_function('glVertexAttrib4fvARB', None, [GLuint, POINTER(GLfloat)], 'ARB_vertex_program')

# GL/glext.h:4831
glVertexAttrib4ivARB = _link_function('glVertexAttrib4ivARB', None, [GLuint, POINTER(GLint)], 'ARB_vertex_program')

# GL/glext.h:4832
glVertexAttrib4sARB = _link_function('glVertexAttrib4sARB', None, [GLuint, GLshort, GLshort, GLshort, GLshort], 'ARB_vertex_program')

# GL/glext.h:4833
glVertexAttrib4svARB = _link_function('glVertexAttrib4svARB', None, [GLuint, POINTER(GLshort)], 'ARB_vertex_program')

# GL/glext.h:4834
glVertexAttrib4ubvARB = _link_function('glVertexAttrib4ubvARB', None, [GLuint, POINTER(GLubyte)], 'ARB_vertex_program')

# GL/glext.h:4835
glVertexAttrib4uivARB = _link_function('glVertexAttrib4uivARB', None, [GLuint, POINTER(GLuint)], 'ARB_vertex_program')

# GL/glext.h:4836
glVertexAttrib4usvARB = _link_function('glVertexAttrib4usvARB', None, [GLuint, POINTER(GLushort)], 'ARB_vertex_program')

# GL/glext.h:4837
glVertexAttribPointerARB = _link_function('glVertexAttribPointerARB', None, [GLuint, GLint, GLenum, GLboolean, GLsizei, POINTER(GLvoid)], 'ARB_vertex_program')

# GL/glext.h:4838
glEnableVertexAttribArrayARB = _link_function('glEnableVertexAttribArrayARB', None, [GLuint], 'ARB_vertex_program')

# GL/glext.h:4839
glDisableVertexAttribArrayARB = _link_function('glDisableVertexAttribArrayARB', None, [GLuint], 'ARB_vertex_program')

# GL/glext.h:4840
glProgramStringARB = _link_function('glProgramStringARB', None, [GLenum, GLenum, GLsizei, POINTER(GLvoid)], 'ARB_vertex_program')

# GL/glext.h:4841
glBindProgramARB = _link_function('glBindProgramARB', None, [GLenum, GLuint], 'ARB_vertex_program')

# GL/glext.h:4842
glDeleteProgramsARB = _link_function('glDeleteProgramsARB', None, [GLsizei, POINTER(GLuint)], 'ARB_vertex_program')

# GL/glext.h:4843
glGenProgramsARB = _link_function('glGenProgramsARB', None, [GLsizei, POINTER(GLuint)], 'ARB_vertex_program')

# GL/glext.h:4844
glProgramEnvParameter4dARB = _link_function('glProgramEnvParameter4dARB', None, [GLenum, GLuint, GLdouble, GLdouble, GLdouble, GLdouble], 'ARB_vertex_program')

# GL/glext.h:4845
glProgramEnvParameter4dvARB = _link_function('glProgramEnvParameter4dvARB', None, [GLenum, GLuint, POINTER(GLdouble)], 'ARB_vertex_program')

# GL/glext.h:4846
glProgramEnvParameter4fARB = _link_function('glProgramEnvParameter4fARB', None, [GLenum, GLuint, GLfloat, GLfloat, GLfloat, GLfloat], 'ARB_vertex_program')

# GL/glext.h:4847
glProgramEnvParameter4fvARB = _link_function('glProgramEnvParameter4fvARB', None, [GLenum, GLuint, POINTER(GLfloat)], 'ARB_vertex_program')

# GL/glext.h:4848
glProgramLocalParameter4dARB = _link_function('glProgramLocalParameter4dARB', None, [GLenum, GLuint, GLdouble, GLdouble, GLdouble, GLdouble], 'ARB_vertex_program')

# GL/glext.h:4849
glProgramLocalParameter4dvARB = _link_function('glProgramLocalParameter4dvARB', None, [GLenum, GLuint, POINTER(GLdouble)], 'ARB_vertex_program')

# GL/glext.h:4850
glProgramLocalParameter4fARB = _link_function('glProgramLocalParameter4fARB', None, [GLenum, GLuint, GLfloat, GLfloat, GLfloat, GLfloat], 'ARB_vertex_program')

# GL/glext.h:4851
glProgramLocalParameter4fvARB = _link_function('glProgramLocalParameter4fvARB', None, [GLenum, GLuint, POINTER(GLfloat)], 'ARB_vertex_program')

# GL/glext.h:4852
glGetProgramEnvParameterdvARB = _link_function('glGetProgramEnvParameterdvARB', None, [GLenum, GLuint, POINTER(GLdouble)], 'ARB_vertex_program')

# GL/glext.h:4853
glGetProgramEnvParameterfvARB = _link_function('glGetProgramEnvParameterfvARB', None, [GLenum, GLuint, POINTER(GLfloat)], 'ARB_vertex_program')

# GL/glext.h:4854
glGetProgramLocalParameterdvARB = _link_function('glGetProgramLocalParameterdvARB', None, [GLenum, GLuint, POINTER(GLdouble)], 'ARB_vertex_program')

# GL/glext.h:4855
glGetProgramLocalParameterfvARB = _link_function('glGetProgramLocalParameterfvARB', None, [GLenum, GLuint, POINTER(GLfloat)], 'ARB_vertex_program')

# GL/glext.h:4856
glGetProgramivARB = _link_function('glGetProgramivARB', None, [GLenum, GLenum, POINTER(GLint)], 'ARB_vertex_program')

# GL/glext.h:4857
glGetProgramStringARB = _link_function('glGetProgramStringARB', None, [GLenum, GLenum, POINTER(GLvoid)], 'ARB_vertex_program')

# GL/glext.h:4858
glGetVertexAttribdvARB = _link_function('glGetVertexAttribdvARB', None, [GLuint, GLenum, POINTER(GLdouble)], 'ARB_vertex_program')

# GL/glext.h:4859
glGetVertexAttribfvARB = _link_function('glGetVertexAttribfvARB', None, [GLuint, GLenum, POINTER(GLfloat)], 'ARB_vertex_program')

# GL/glext.h:4860
glGetVertexAttribivARB = _link_function('glGetVertexAttribivARB', None, [GLuint, GLenum, POINTER(GLint)], 'ARB_vertex_program')

# GL/glext.h:4861
glGetVertexAttribPointervARB = _link_function('glGetVertexAttribPointervARB', None, [GLuint, GLenum, POINTER(POINTER(GLvoid))], 'ARB_vertex_program')

# GL/glext.h:4862
glIsProgramARB = _link_function('glIsProgramARB', GLboolean, [GLuint], 'ARB_vertex_program')

PFNGLVERTEXATTRIB1DARBPROC = CFUNCTYPE(None, GLuint, GLdouble) 	# GL/glext.h:4864
PFNGLVERTEXATTRIB1DVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:4865
PFNGLVERTEXATTRIB1FARBPROC = CFUNCTYPE(None, GLuint, GLfloat) 	# GL/glext.h:4866
PFNGLVERTEXATTRIB1FVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:4867
PFNGLVERTEXATTRIB1SARBPROC = CFUNCTYPE(None, GLuint, GLshort) 	# GL/glext.h:4868
PFNGLVERTEXATTRIB1SVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:4869
PFNGLVERTEXATTRIB2DARBPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble) 	# GL/glext.h:4870
PFNGLVERTEXATTRIB2DVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:4871
PFNGLVERTEXATTRIB2FARBPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat) 	# GL/glext.h:4872
PFNGLVERTEXATTRIB2FVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:4873
PFNGLVERTEXATTRIB2SARBPROC = CFUNCTYPE(None, GLuint, GLshort, GLshort) 	# GL/glext.h:4874
PFNGLVERTEXATTRIB2SVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:4875
PFNGLVERTEXATTRIB3DARBPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:4876
PFNGLVERTEXATTRIB3DVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:4877
PFNGLVERTEXATTRIB3FARBPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:4878
PFNGLVERTEXATTRIB3FVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:4879
PFNGLVERTEXATTRIB3SARBPROC = CFUNCTYPE(None, GLuint, GLshort, GLshort, GLshort) 	# GL/glext.h:4880
PFNGLVERTEXATTRIB3SVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:4881
PFNGLVERTEXATTRIB4NBVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLbyte)) 	# GL/glext.h:4882
PFNGLVERTEXATTRIB4NIVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:4883
PFNGLVERTEXATTRIB4NSVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:4884
PFNGLVERTEXATTRIB4NUBARBPROC = CFUNCTYPE(None, GLuint, GLubyte, GLubyte, GLubyte, GLubyte) 	# GL/glext.h:4885
PFNGLVERTEXATTRIB4NUBVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLubyte)) 	# GL/glext.h:4886
PFNGLVERTEXATTRIB4NUIVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:4887
PFNGLVERTEXATTRIB4NUSVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLushort)) 	# GL/glext.h:4888
PFNGLVERTEXATTRIB4BVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLbyte)) 	# GL/glext.h:4889
PFNGLVERTEXATTRIB4DARBPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:4890
PFNGLVERTEXATTRIB4DVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:4891
PFNGLVERTEXATTRIB4FARBPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:4892
PFNGLVERTEXATTRIB4FVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:4893
PFNGLVERTEXATTRIB4IVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:4894
PFNGLVERTEXATTRIB4SARBPROC = CFUNCTYPE(None, GLuint, GLshort, GLshort, GLshort, GLshort) 	# GL/glext.h:4895
PFNGLVERTEXATTRIB4SVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:4896
PFNGLVERTEXATTRIB4UBVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLubyte)) 	# GL/glext.h:4897
PFNGLVERTEXATTRIB4UIVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:4898
PFNGLVERTEXATTRIB4USVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLushort)) 	# GL/glext.h:4899
PFNGLVERTEXATTRIBPOINTERARBPROC = CFUNCTYPE(None, GLuint, GLint, GLenum, GLboolean, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:4900
PFNGLENABLEVERTEXATTRIBARRAYARBPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:4901
PFNGLDISABLEVERTEXATTRIBARRAYARBPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:4902
PFNGLPROGRAMSTRINGARBPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:4903
PFNGLBINDPROGRAMARBPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:4904
PFNGLDELETEPROGRAMSARBPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:4905
PFNGLGENPROGRAMSARBPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:4906
PFNGLPROGRAMENVPARAMETER4DARBPROC = CFUNCTYPE(None, GLenum, GLuint, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:4907
PFNGLPROGRAMENVPARAMETER4DVARBPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLdouble)) 	# GL/glext.h:4908
PFNGLPROGRAMENVPARAMETER4FARBPROC = CFUNCTYPE(None, GLenum, GLuint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:4909
PFNGLPROGRAMENVPARAMETER4FVARBPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLfloat)) 	# GL/glext.h:4910
PFNGLPROGRAMLOCALPARAMETER4DARBPROC = CFUNCTYPE(None, GLenum, GLuint, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:4911
PFNGLPROGRAMLOCALPARAMETER4DVARBPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLdouble)) 	# GL/glext.h:4912
PFNGLPROGRAMLOCALPARAMETER4FARBPROC = CFUNCTYPE(None, GLenum, GLuint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:4913
PFNGLPROGRAMLOCALPARAMETER4FVARBPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLfloat)) 	# GL/glext.h:4914
PFNGLGETPROGRAMENVPARAMETERDVARBPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLdouble)) 	# GL/glext.h:4915
PFNGLGETPROGRAMENVPARAMETERFVARBPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLfloat)) 	# GL/glext.h:4916
PFNGLGETPROGRAMLOCALPARAMETERDVARBPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLdouble)) 	# GL/glext.h:4917
PFNGLGETPROGRAMLOCALPARAMETERFVARBPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLfloat)) 	# GL/glext.h:4918
PFNGLGETPROGRAMIVARBPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:4919
PFNGLGETPROGRAMSTRINGARBPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:4920
PFNGLGETVERTEXATTRIBDVARBPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLdouble)) 	# GL/glext.h:4921
PFNGLGETVERTEXATTRIBFVARBPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4922
PFNGLGETVERTEXATTRIBIVARBPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:4923
PFNGLGETVERTEXATTRIBPOINTERVARBPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:4924
PFNGLISPROGRAMARBPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:4925
# ARB_fragment_program (GL/glext.h:4928)
GL_ARB_fragment_program = 1 	# GL/glext.h:4929
# ARB_vertex_buffer_object (GL/glext.h:4933)
GL_ARB_vertex_buffer_object = 1 	# GL/glext.h:4934
# GL/glext.h:4936
glBindBufferARB = _link_function('glBindBufferARB', None, [GLenum, GLuint], 'ARB_vertex_buffer_object')

# GL/glext.h:4937
glDeleteBuffersARB = _link_function('glDeleteBuffersARB', None, [GLsizei, POINTER(GLuint)], 'ARB_vertex_buffer_object')

# GL/glext.h:4938
glGenBuffersARB = _link_function('glGenBuffersARB', None, [GLsizei, POINTER(GLuint)], 'ARB_vertex_buffer_object')

# GL/glext.h:4939
glIsBufferARB = _link_function('glIsBufferARB', GLboolean, [GLuint], 'ARB_vertex_buffer_object')

# GL/glext.h:4940
glBufferDataARB = _link_function('glBufferDataARB', None, [GLenum, GLsizeiptrARB, POINTER(GLvoid), GLenum], 'ARB_vertex_buffer_object')

# GL/glext.h:4941
glBufferSubDataARB = _link_function('glBufferSubDataARB', None, [GLenum, GLintptrARB, GLsizeiptrARB, POINTER(GLvoid)], 'ARB_vertex_buffer_object')

# GL/glext.h:4942
glGetBufferSubDataARB = _link_function('glGetBufferSubDataARB', None, [GLenum, GLintptrARB, GLsizeiptrARB, POINTER(GLvoid)], 'ARB_vertex_buffer_object')

# GL/glext.h:4943
glMapBufferARB = _link_function('glMapBufferARB', POINTER(GLvoid), [GLenum, GLenum], 'ARB_vertex_buffer_object')

# GL/glext.h:4944
glUnmapBufferARB = _link_function('glUnmapBufferARB', GLboolean, [GLenum], 'ARB_vertex_buffer_object')

# GL/glext.h:4945
glGetBufferParameterivARB = _link_function('glGetBufferParameterivARB', None, [GLenum, GLenum, POINTER(GLint)], 'ARB_vertex_buffer_object')

# GL/glext.h:4946
glGetBufferPointervARB = _link_function('glGetBufferPointervARB', None, [GLenum, GLenum, POINTER(POINTER(GLvoid))], 'ARB_vertex_buffer_object')

PFNGLBINDBUFFERARBPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:4948
PFNGLDELETEBUFFERSARBPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:4949
PFNGLGENBUFFERSARBPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:4950
PFNGLISBUFFERARBPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:4951
PFNGLBUFFERDATAARBPROC = CFUNCTYPE(None, GLenum, GLsizeiptrARB, POINTER(GLvoid), GLenum) 	# GL/glext.h:4952
PFNGLBUFFERSUBDATAARBPROC = CFUNCTYPE(None, GLenum, GLintptrARB, GLsizeiptrARB, POINTER(GLvoid)) 	# GL/glext.h:4953
PFNGLGETBUFFERSUBDATAARBPROC = CFUNCTYPE(None, GLenum, GLintptrARB, GLsizeiptrARB, POINTER(GLvoid)) 	# GL/glext.h:4954
PFNGLMAPBUFFERARBPROC = CFUNCTYPE(POINTER(GLvoid), GLenum, GLenum) 	# GL/glext.h:4955
PFNGLUNMAPBUFFERARBPROC = CFUNCTYPE(GLboolean, GLenum) 	# GL/glext.h:4956
PFNGLGETBUFFERPARAMETERIVARBPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:4957
PFNGLGETBUFFERPOINTERVARBPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:4958
# ARB_occlusion_query (GL/glext.h:4961)
GL_ARB_occlusion_query = 1 	# GL/glext.h:4962
# GL/glext.h:4964
glGenQueriesARB = _link_function('glGenQueriesARB', None, [GLsizei, POINTER(GLuint)], 'ARB_occlusion_query')

# GL/glext.h:4965
glDeleteQueriesARB = _link_function('glDeleteQueriesARB', None, [GLsizei, POINTER(GLuint)], 'ARB_occlusion_query')

# GL/glext.h:4966
glIsQueryARB = _link_function('glIsQueryARB', GLboolean, [GLuint], 'ARB_occlusion_query')

# GL/glext.h:4967
glBeginQueryARB = _link_function('glBeginQueryARB', None, [GLenum, GLuint], 'ARB_occlusion_query')

# GL/glext.h:4968
glEndQueryARB = _link_function('glEndQueryARB', None, [GLenum], 'ARB_occlusion_query')

# GL/glext.h:4969
glGetQueryivARB = _link_function('glGetQueryivARB', None, [GLenum, GLenum, POINTER(GLint)], 'ARB_occlusion_query')

# GL/glext.h:4970
glGetQueryObjectivARB = _link_function('glGetQueryObjectivARB', None, [GLuint, GLenum, POINTER(GLint)], 'ARB_occlusion_query')

# GL/glext.h:4971
glGetQueryObjectuivARB = _link_function('glGetQueryObjectuivARB', None, [GLuint, GLenum, POINTER(GLuint)], 'ARB_occlusion_query')

PFNGLGENQUERIESARBPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:4973
PFNGLDELETEQUERIESARBPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:4974
PFNGLISQUERYARBPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:4975
PFNGLBEGINQUERYARBPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:4976
PFNGLENDQUERYARBPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:4977
PFNGLGETQUERYIVARBPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:4978
PFNGLGETQUERYOBJECTIVARBPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:4979
PFNGLGETQUERYOBJECTUIVARBPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLuint)) 	# GL/glext.h:4980
# ARB_shader_objects (GL/glext.h:4983)
GL_ARB_shader_objects = 1 	# GL/glext.h:4984
# GL/glext.h:4986
glDeleteObjectARB = _link_function('glDeleteObjectARB', None, [GLhandleARB], 'ARB_shader_objects')

# GL/glext.h:4987
glGetHandleARB = _link_function('glGetHandleARB', GLhandleARB, [GLenum], 'ARB_shader_objects')

# GL/glext.h:4988
glDetachObjectARB = _link_function('glDetachObjectARB', None, [GLhandleARB, GLhandleARB], 'ARB_shader_objects')

# GL/glext.h:4989
glCreateShaderObjectARB = _link_function('glCreateShaderObjectARB', GLhandleARB, [GLenum], 'ARB_shader_objects')

# GL/glext.h:4990
glShaderSourceARB = _link_function('glShaderSourceARB', None, [GLhandleARB, GLsizei, POINTER(POINTER(GLcharARB)), POINTER(GLint)], 'ARB_shader_objects')

# GL/glext.h:4991
glCompileShaderARB = _link_function('glCompileShaderARB', None, [GLhandleARB], 'ARB_shader_objects')

# GL/glext.h:4992
glCreateProgramObjectARB = _link_function('glCreateProgramObjectARB', GLhandleARB, [], 'ARB_shader_objects')

# GL/glext.h:4993
glAttachObjectARB = _link_function('glAttachObjectARB', None, [GLhandleARB, GLhandleARB], 'ARB_shader_objects')

# GL/glext.h:4994
glLinkProgramARB = _link_function('glLinkProgramARB', None, [GLhandleARB], 'ARB_shader_objects')

# GL/glext.h:4995
glUseProgramObjectARB = _link_function('glUseProgramObjectARB', None, [GLhandleARB], 'ARB_shader_objects')

# GL/glext.h:4996
glValidateProgramARB = _link_function('glValidateProgramARB', None, [GLhandleARB], 'ARB_shader_objects')

# GL/glext.h:4997
glUniform1fARB = _link_function('glUniform1fARB', None, [GLint, GLfloat], 'ARB_shader_objects')

# GL/glext.h:4998
glUniform2fARB = _link_function('glUniform2fARB', None, [GLint, GLfloat, GLfloat], 'ARB_shader_objects')

# GL/glext.h:4999
glUniform3fARB = _link_function('glUniform3fARB', None, [GLint, GLfloat, GLfloat, GLfloat], 'ARB_shader_objects')

# GL/glext.h:5000
glUniform4fARB = _link_function('glUniform4fARB', None, [GLint, GLfloat, GLfloat, GLfloat, GLfloat], 'ARB_shader_objects')

# GL/glext.h:5001
glUniform1iARB = _link_function('glUniform1iARB', None, [GLint, GLint], 'ARB_shader_objects')

# GL/glext.h:5002
glUniform2iARB = _link_function('glUniform2iARB', None, [GLint, GLint, GLint], 'ARB_shader_objects')

# GL/glext.h:5003
glUniform3iARB = _link_function('glUniform3iARB', None, [GLint, GLint, GLint, GLint], 'ARB_shader_objects')

# GL/glext.h:5004
glUniform4iARB = _link_function('glUniform4iARB', None, [GLint, GLint, GLint, GLint, GLint], 'ARB_shader_objects')

# GL/glext.h:5005
glUniform1fvARB = _link_function('glUniform1fvARB', None, [GLint, GLsizei, POINTER(GLfloat)], 'ARB_shader_objects')

# GL/glext.h:5006
glUniform2fvARB = _link_function('glUniform2fvARB', None, [GLint, GLsizei, POINTER(GLfloat)], 'ARB_shader_objects')

# GL/glext.h:5007
glUniform3fvARB = _link_function('glUniform3fvARB', None, [GLint, GLsizei, POINTER(GLfloat)], 'ARB_shader_objects')

# GL/glext.h:5008
glUniform4fvARB = _link_function('glUniform4fvARB', None, [GLint, GLsizei, POINTER(GLfloat)], 'ARB_shader_objects')

# GL/glext.h:5009
glUniform1ivARB = _link_function('glUniform1ivARB', None, [GLint, GLsizei, POINTER(GLint)], 'ARB_shader_objects')

# GL/glext.h:5010
glUniform2ivARB = _link_function('glUniform2ivARB', None, [GLint, GLsizei, POINTER(GLint)], 'ARB_shader_objects')

# GL/glext.h:5011
glUniform3ivARB = _link_function('glUniform3ivARB', None, [GLint, GLsizei, POINTER(GLint)], 'ARB_shader_objects')

# GL/glext.h:5012
glUniform4ivARB = _link_function('glUniform4ivARB', None, [GLint, GLsizei, POINTER(GLint)], 'ARB_shader_objects')

# GL/glext.h:5013
glUniformMatrix2fvARB = _link_function('glUniformMatrix2fvARB', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'ARB_shader_objects')

# GL/glext.h:5014
glUniformMatrix3fvARB = _link_function('glUniformMatrix3fvARB', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'ARB_shader_objects')

# GL/glext.h:5015
glUniformMatrix4fvARB = _link_function('glUniformMatrix4fvARB', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'ARB_shader_objects')

# GL/glext.h:5016
glGetObjectParameterfvARB = _link_function('glGetObjectParameterfvARB', None, [GLhandleARB, GLenum, POINTER(GLfloat)], 'ARB_shader_objects')

# GL/glext.h:5017
glGetObjectParameterivARB = _link_function('glGetObjectParameterivARB', None, [GLhandleARB, GLenum, POINTER(GLint)], 'ARB_shader_objects')

# GL/glext.h:5018
glGetInfoLogARB = _link_function('glGetInfoLogARB', None, [GLhandleARB, GLsizei, POINTER(GLsizei), POINTER(GLcharARB)], 'ARB_shader_objects')

# GL/glext.h:5019
glGetAttachedObjectsARB = _link_function('glGetAttachedObjectsARB', None, [GLhandleARB, GLsizei, POINTER(GLsizei), POINTER(GLhandleARB)], 'ARB_shader_objects')

# GL/glext.h:5020
glGetUniformLocationARB = _link_function('glGetUniformLocationARB', GLint, [GLhandleARB, POINTER(GLcharARB)], 'ARB_shader_objects')

# GL/glext.h:5021
glGetActiveUniformARB = _link_function('glGetActiveUniformARB', None, [GLhandleARB, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLint), POINTER(GLenum), POINTER(GLcharARB)], 'ARB_shader_objects')

# GL/glext.h:5022
glGetUniformfvARB = _link_function('glGetUniformfvARB', None, [GLhandleARB, GLint, POINTER(GLfloat)], 'ARB_shader_objects')

# GL/glext.h:5023
glGetUniformivARB = _link_function('glGetUniformivARB', None, [GLhandleARB, GLint, POINTER(GLint)], 'ARB_shader_objects')

# GL/glext.h:5024
glGetShaderSourceARB = _link_function('glGetShaderSourceARB', None, [GLhandleARB, GLsizei, POINTER(GLsizei), POINTER(GLcharARB)], 'ARB_shader_objects')

PFNGLDELETEOBJECTARBPROC = CFUNCTYPE(None, GLhandleARB) 	# GL/glext.h:5026
PFNGLGETHANDLEARBPROC = CFUNCTYPE(GLhandleARB, GLenum) 	# GL/glext.h:5027
PFNGLDETACHOBJECTARBPROC = CFUNCTYPE(None, GLhandleARB, GLhandleARB) 	# GL/glext.h:5028
PFNGLCREATESHADEROBJECTARBPROC = CFUNCTYPE(GLhandleARB, GLenum) 	# GL/glext.h:5029
PFNGLSHADERSOURCEARBPROC = CFUNCTYPE(None, GLhandleARB, GLsizei, POINTER(POINTER(GLcharARB)), POINTER(GLint)) 	# GL/glext.h:5030
PFNGLCOMPILESHADERARBPROC = CFUNCTYPE(None, GLhandleARB) 	# GL/glext.h:5031
PFNGLCREATEPROGRAMOBJECTARBPROC = CFUNCTYPE(GLhandleARB) 	# GL/glext.h:5032
PFNGLATTACHOBJECTARBPROC = CFUNCTYPE(None, GLhandleARB, GLhandleARB) 	# GL/glext.h:5033
PFNGLLINKPROGRAMARBPROC = CFUNCTYPE(None, GLhandleARB) 	# GL/glext.h:5034
PFNGLUSEPROGRAMOBJECTARBPROC = CFUNCTYPE(None, GLhandleARB) 	# GL/glext.h:5035
PFNGLVALIDATEPROGRAMARBPROC = CFUNCTYPE(None, GLhandleARB) 	# GL/glext.h:5036
PFNGLUNIFORM1FARBPROC = CFUNCTYPE(None, GLint, GLfloat) 	# GL/glext.h:5037
PFNGLUNIFORM2FARBPROC = CFUNCTYPE(None, GLint, GLfloat, GLfloat) 	# GL/glext.h:5038
PFNGLUNIFORM3FARBPROC = CFUNCTYPE(None, GLint, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5039
PFNGLUNIFORM4FARBPROC = CFUNCTYPE(None, GLint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5040
PFNGLUNIFORM1IARBPROC = CFUNCTYPE(None, GLint, GLint) 	# GL/glext.h:5041
PFNGLUNIFORM2IARBPROC = CFUNCTYPE(None, GLint, GLint, GLint) 	# GL/glext.h:5042
PFNGLUNIFORM3IARBPROC = CFUNCTYPE(None, GLint, GLint, GLint, GLint) 	# GL/glext.h:5043
PFNGLUNIFORM4IARBPROC = CFUNCTYPE(None, GLint, GLint, GLint, GLint, GLint) 	# GL/glext.h:5044
PFNGLUNIFORM1FVARBPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:5045
PFNGLUNIFORM2FVARBPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:5046
PFNGLUNIFORM3FVARBPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:5047
PFNGLUNIFORM4FVARBPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:5048
PFNGLUNIFORM1IVARBPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:5049
PFNGLUNIFORM2IVARBPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:5050
PFNGLUNIFORM3IVARBPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:5051
PFNGLUNIFORM4IVARBPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:5052
PFNGLUNIFORMMATRIX2FVARBPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:5053
PFNGLUNIFORMMATRIX3FVARBPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:5054
PFNGLUNIFORMMATRIX4FVARBPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:5055
PFNGLGETOBJECTPARAMETERFVARBPROC = CFUNCTYPE(None, GLhandleARB, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5056
PFNGLGETOBJECTPARAMETERIVARBPROC = CFUNCTYPE(None, GLhandleARB, GLenum, POINTER(GLint)) 	# GL/glext.h:5057
PFNGLGETINFOLOGARBPROC = CFUNCTYPE(None, GLhandleARB, GLsizei, POINTER(GLsizei), POINTER(GLcharARB)) 	# GL/glext.h:5058
PFNGLGETATTACHEDOBJECTSARBPROC = CFUNCTYPE(None, GLhandleARB, GLsizei, POINTER(GLsizei), POINTER(GLhandleARB)) 	# GL/glext.h:5059
PFNGLGETUNIFORMLOCATIONARBPROC = CFUNCTYPE(GLint, GLhandleARB, POINTER(GLcharARB)) 	# GL/glext.h:5060
PFNGLGETACTIVEUNIFORMARBPROC = CFUNCTYPE(None, GLhandleARB, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLint), POINTER(GLenum), POINTER(GLcharARB)) 	# GL/glext.h:5061
PFNGLGETUNIFORMFVARBPROC = CFUNCTYPE(None, GLhandleARB, GLint, POINTER(GLfloat)) 	# GL/glext.h:5062
PFNGLGETUNIFORMIVARBPROC = CFUNCTYPE(None, GLhandleARB, GLint, POINTER(GLint)) 	# GL/glext.h:5063
PFNGLGETSHADERSOURCEARBPROC = CFUNCTYPE(None, GLhandleARB, GLsizei, POINTER(GLsizei), POINTER(GLcharARB)) 	# GL/glext.h:5064
# ARB_vertex_shader (GL/glext.h:5067)
GL_ARB_vertex_shader = 1 	# GL/glext.h:5068
# GL/glext.h:5070
glBindAttribLocationARB = _link_function('glBindAttribLocationARB', None, [GLhandleARB, GLuint, POINTER(GLcharARB)], 'ARB_vertex_shader')

# GL/glext.h:5071
glGetActiveAttribARB = _link_function('glGetActiveAttribARB', None, [GLhandleARB, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLint), POINTER(GLenum), POINTER(GLcharARB)], 'ARB_vertex_shader')

# GL/glext.h:5072
glGetAttribLocationARB = _link_function('glGetAttribLocationARB', GLint, [GLhandleARB, POINTER(GLcharARB)], 'ARB_vertex_shader')

PFNGLBINDATTRIBLOCATIONARBPROC = CFUNCTYPE(None, GLhandleARB, GLuint, POINTER(GLcharARB)) 	# GL/glext.h:5074
PFNGLGETACTIVEATTRIBARBPROC = CFUNCTYPE(None, GLhandleARB, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLint), POINTER(GLenum), POINTER(GLcharARB)) 	# GL/glext.h:5075
PFNGLGETATTRIBLOCATIONARBPROC = CFUNCTYPE(GLint, GLhandleARB, POINTER(GLcharARB)) 	# GL/glext.h:5076
# ARB_fragment_shader (GL/glext.h:5079)
GL_ARB_fragment_shader = 1 	# GL/glext.h:5080
# ARB_shading_language_100 (GL/glext.h:5083)
GL_ARB_shading_language_100 = 1 	# GL/glext.h:5084
# ARB_texture_non_power_of_two (GL/glext.h:5087)
GL_ARB_texture_non_power_of_two = 1 	# GL/glext.h:5088
# ARB_point_sprite (GL/glext.h:5091)
GL_ARB_point_sprite = 1 	# GL/glext.h:5092
# ARB_fragment_program_shadow (GL/glext.h:5095)
GL_ARB_fragment_program_shadow = 1 	# GL/glext.h:5096
# ARB_draw_buffers (GL/glext.h:5099)
GL_ARB_draw_buffers = 1 	# GL/glext.h:5100
# GL/glext.h:5102
glDrawBuffersARB = _link_function('glDrawBuffersARB', None, [GLsizei, POINTER(GLenum)], 'ARB_draw_buffers')

PFNGLDRAWBUFFERSARBPROC = CFUNCTYPE(None, GLsizei, POINTER(GLenum)) 	# GL/glext.h:5104
# ARB_texture_rectangle (GL/glext.h:5107)
GL_ARB_texture_rectangle = 1 	# GL/glext.h:5108
# ARB_color_buffer_float (GL/glext.h:5111)
GL_ARB_color_buffer_float = 1 	# GL/glext.h:5112
# GL/glext.h:5114
glClampColorARB = _link_function('glClampColorARB', None, [GLenum, GLenum], 'ARB_color_buffer_float')

PFNGLCLAMPCOLORARBPROC = CFUNCTYPE(None, GLenum, GLenum) 	# GL/glext.h:5116
# ARB_half_float_pixel (GL/glext.h:5119)
GL_ARB_half_float_pixel = 1 	# GL/glext.h:5120
# ARB_texture_float (GL/glext.h:5123)
GL_ARB_texture_float = 1 	# GL/glext.h:5124
# ARB_pixel_buffer_object (GL/glext.h:5127)
GL_ARB_pixel_buffer_object = 1 	# GL/glext.h:5128
# ARB_depth_buffer_float (GL/glext.h:5131)
GL_ARB_depth_buffer_float = 1 	# GL/glext.h:5132
# ARB_draw_instanced (GL/glext.h:5135)
GL_ARB_draw_instanced = 1 	# GL/glext.h:5136
# GL/glext.h:5138
glDrawArraysInstancedARB = _link_function('glDrawArraysInstancedARB', None, [GLenum, GLint, GLsizei, GLsizei], 'ARB_draw_instanced')

# GL/glext.h:5139
glDrawElementsInstancedARB = _link_function('glDrawElementsInstancedARB', None, [GLenum, GLsizei, GLenum, POINTER(GLvoid), GLsizei], 'ARB_draw_instanced')

PFNGLDRAWARRAYSINSTANCEDARBPROC = CFUNCTYPE(None, GLenum, GLint, GLsizei, GLsizei) 	# GL/glext.h:5141
PFNGLDRAWELEMENTSINSTANCEDARBPROC = CFUNCTYPE(None, GLenum, GLsizei, GLenum, POINTER(GLvoid), GLsizei) 	# GL/glext.h:5142
# ARB_framebuffer_object (GL/glext.h:5145)
GL_ARB_framebuffer_object = 1 	# GL/glext.h:5146
# GL/glext.h:5148
glIsRenderbuffer = _link_function('glIsRenderbuffer', GLboolean, [GLuint], 'ARB_framebuffer_object')

# GL/glext.h:5149
glBindRenderbuffer = _link_function('glBindRenderbuffer', None, [GLenum, GLuint], 'ARB_framebuffer_object')

# GL/glext.h:5150
glDeleteRenderbuffers = _link_function('glDeleteRenderbuffers', None, [GLsizei, POINTER(GLuint)], 'ARB_framebuffer_object')

# GL/glext.h:5151
glGenRenderbuffers = _link_function('glGenRenderbuffers', None, [GLsizei, POINTER(GLuint)], 'ARB_framebuffer_object')

# GL/glext.h:5152
glRenderbufferStorage = _link_function('glRenderbufferStorage', None, [GLenum, GLenum, GLsizei, GLsizei], 'ARB_framebuffer_object')

# GL/glext.h:5153
glGetRenderbufferParameteriv = _link_function('glGetRenderbufferParameteriv', None, [GLenum, GLenum, POINTER(GLint)], 'ARB_framebuffer_object')

# GL/glext.h:5154
glIsFramebuffer = _link_function('glIsFramebuffer', GLboolean, [GLuint], 'ARB_framebuffer_object')

# GL/glext.h:5155
glBindFramebuffer = _link_function('glBindFramebuffer', None, [GLenum, GLuint], 'ARB_framebuffer_object')

# GL/glext.h:5156
glDeleteFramebuffers = _link_function('glDeleteFramebuffers', None, [GLsizei, POINTER(GLuint)], 'ARB_framebuffer_object')

# GL/glext.h:5157
glGenFramebuffers = _link_function('glGenFramebuffers', None, [GLsizei, POINTER(GLuint)], 'ARB_framebuffer_object')

# GL/glext.h:5158
glCheckFramebufferStatus = _link_function('glCheckFramebufferStatus', GLenum, [GLenum], 'ARB_framebuffer_object')

# GL/glext.h:5159
glFramebufferTexture1D = _link_function('glFramebufferTexture1D', None, [GLenum, GLenum, GLenum, GLuint, GLint], 'ARB_framebuffer_object')

# GL/glext.h:5160
glFramebufferTexture2D = _link_function('glFramebufferTexture2D', None, [GLenum, GLenum, GLenum, GLuint, GLint], 'ARB_framebuffer_object')

# GL/glext.h:5161
glFramebufferTexture3D = _link_function('glFramebufferTexture3D', None, [GLenum, GLenum, GLenum, GLuint, GLint, GLint], 'ARB_framebuffer_object')

# GL/glext.h:5162
glFramebufferRenderbuffer = _link_function('glFramebufferRenderbuffer', None, [GLenum, GLenum, GLenum, GLuint], 'ARB_framebuffer_object')

# GL/glext.h:5163
glGetFramebufferAttachmentParameteriv = _link_function('glGetFramebufferAttachmentParameteriv', None, [GLenum, GLenum, GLenum, POINTER(GLint)], 'ARB_framebuffer_object')

# GL/glext.h:5164
glGenerateMipmap = _link_function('glGenerateMipmap', None, [GLenum], 'ARB_framebuffer_object')

GLbitfield = c_uint 	# /usr/include/GL/gl.h:123
# GL/glext.h:5165
glBlitFramebuffer = _link_function('glBlitFramebuffer', None, [GLint, GLint, GLint, GLint, GLint, GLint, GLint, GLint, GLbitfield, GLenum], 'ARB_framebuffer_object')

# GL/glext.h:5166
glRenderbufferStorageMultisample = _link_function('glRenderbufferStorageMultisample', None, [GLenum, GLsizei, GLenum, GLsizei, GLsizei], 'ARB_framebuffer_object')

# GL/glext.h:5167
glFramebufferTextureLayer = _link_function('glFramebufferTextureLayer', None, [GLenum, GLenum, GLuint, GLint, GLint], 'ARB_framebuffer_object')

PFNGLISRENDERBUFFERPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:5169
PFNGLBINDRENDERBUFFERPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:5170
PFNGLDELETERENDERBUFFERSPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:5171
PFNGLGENRENDERBUFFERSPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:5172
PFNGLRENDERBUFFERSTORAGEPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLsizei) 	# GL/glext.h:5173
PFNGLGETRENDERBUFFERPARAMETERIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5174
PFNGLISFRAMEBUFFERPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:5175
PFNGLBINDFRAMEBUFFERPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:5176
PFNGLDELETEFRAMEBUFFERSPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:5177
PFNGLGENFRAMEBUFFERSPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:5178
PFNGLCHECKFRAMEBUFFERSTATUSPROC = CFUNCTYPE(GLenum, GLenum) 	# GL/glext.h:5179
PFNGLFRAMEBUFFERTEXTURE1DPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLuint, GLint) 	# GL/glext.h:5180
PFNGLFRAMEBUFFERTEXTURE2DPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLuint, GLint) 	# GL/glext.h:5181
PFNGLFRAMEBUFFERTEXTURE3DPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLuint, GLint, GLint) 	# GL/glext.h:5182
PFNGLFRAMEBUFFERRENDERBUFFERPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLuint) 	# GL/glext.h:5183
PFNGLGETFRAMEBUFFERATTACHMENTPARAMETERIVPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5184
PFNGLGENERATEMIPMAPPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:5185
PFNGLBLITFRAMEBUFFERPROC = CFUNCTYPE(None, GLint, GLint, GLint, GLint, GLint, GLint, GLint, GLint, GLbitfield, GLenum) 	# GL/glext.h:5186
PFNGLRENDERBUFFERSTORAGEMULTISAMPLEPROC = CFUNCTYPE(None, GLenum, GLsizei, GLenum, GLsizei, GLsizei) 	# GL/glext.h:5187
PFNGLFRAMEBUFFERTEXTURELAYERPROC = CFUNCTYPE(None, GLenum, GLenum, GLuint, GLint, GLint) 	# GL/glext.h:5188
# ARB_framebuffer_sRGB (GL/glext.h:5191)
GL_ARB_framebuffer_sRGB = 1 	# GL/glext.h:5192
# ARB_geometry_shader4 (GL/glext.h:5195)
GL_ARB_geometry_shader4 = 1 	# GL/glext.h:5196
# GL/glext.h:5198
glProgramParameteriARB = _link_function('glProgramParameteriARB', None, [GLuint, GLenum, GLint], 'ARB_geometry_shader4')

# GL/glext.h:5199
glFramebufferTextureARB = _link_function('glFramebufferTextureARB', None, [GLenum, GLenum, GLuint, GLint], 'ARB_geometry_shader4')

# GL/glext.h:5200
glFramebufferTextureLayerARB = _link_function('glFramebufferTextureLayerARB', None, [GLenum, GLenum, GLuint, GLint, GLint], 'ARB_geometry_shader4')

# GL/glext.h:5201
glFramebufferTextureFaceARB = _link_function('glFramebufferTextureFaceARB', None, [GLenum, GLenum, GLuint, GLint, GLenum], 'ARB_geometry_shader4')

PFNGLPROGRAMPARAMETERIARBPROC = CFUNCTYPE(None, GLuint, GLenum, GLint) 	# GL/glext.h:5203
PFNGLFRAMEBUFFERTEXTUREARBPROC = CFUNCTYPE(None, GLenum, GLenum, GLuint, GLint) 	# GL/glext.h:5204
PFNGLFRAMEBUFFERTEXTURELAYERARBPROC = CFUNCTYPE(None, GLenum, GLenum, GLuint, GLint, GLint) 	# GL/glext.h:5205
PFNGLFRAMEBUFFERTEXTUREFACEARBPROC = CFUNCTYPE(None, GLenum, GLenum, GLuint, GLint, GLenum) 	# GL/glext.h:5206
# ARB_half_float_vertex (GL/glext.h:5209)
GL_ARB_half_float_vertex = 1 	# GL/glext.h:5210
# ARB_instanced_arrays (GL/glext.h:5213)
GL_ARB_instanced_arrays = 1 	# GL/glext.h:5214
# GL/glext.h:5216
glVertexAttribDivisorARB = _link_function('glVertexAttribDivisorARB', None, [GLuint, GLuint], 'ARB_instanced_arrays')

PFNGLVERTEXATTRIBDIVISORARBPROC = CFUNCTYPE(None, GLuint, GLuint) 	# GL/glext.h:5218
# ARB_map_buffer_range (GL/glext.h:5221)
GL_ARB_map_buffer_range = 1 	# GL/glext.h:5222
# GL/glext.h:5224
glMapBufferRange = _link_function('glMapBufferRange', None, [GLenum, GLintptr, GLsizeiptr, GLbitfield], 'ARB_map_buffer_range')

# GL/glext.h:5225
glFlushMappedBufferRange = _link_function('glFlushMappedBufferRange', None, [GLenum, GLintptr, GLsizeiptr], 'ARB_map_buffer_range')

PFNGLMAPBUFFERRANGEPROC = CFUNCTYPE(None, GLenum, GLintptr, GLsizeiptr, GLbitfield) 	# GL/glext.h:5227
PFNGLFLUSHMAPPEDBUFFERRANGEPROC = CFUNCTYPE(None, GLenum, GLintptr, GLsizeiptr) 	# GL/glext.h:5228
# ARB_texture_buffer_object (GL/glext.h:5231)
GL_ARB_texture_buffer_object = 1 	# GL/glext.h:5232
# GL/glext.h:5234
glTexBufferARB = _link_function('glTexBufferARB', None, [GLenum, GLenum, GLuint], 'ARB_texture_buffer_object')

PFNGLTEXBUFFERARBPROC = CFUNCTYPE(None, GLenum, GLenum, GLuint) 	# GL/glext.h:5236
# ARB_texture_compression_rgtc (GL/glext.h:5239)
GL_ARB_texture_compression_rgtc = 1 	# GL/glext.h:5240
# ARB_texture_rg (GL/glext.h:5243)
GL_ARB_texture_rg = 1 	# GL/glext.h:5244
# ARB_vertex_array_object (GL/glext.h:5247)
GL_ARB_vertex_array_object = 1 	# GL/glext.h:5248
# GL/glext.h:5250
glBindVertexArray = _link_function('glBindVertexArray', None, [GLuint], 'ARB_vertex_array_object')

# GL/glext.h:5251
glDeleteVertexArrays = _link_function('glDeleteVertexArrays', None, [GLsizei, POINTER(GLuint)], 'ARB_vertex_array_object')

# GL/glext.h:5252
glGenVertexArrays = _link_function('glGenVertexArrays', None, [GLsizei, POINTER(GLuint)], 'ARB_vertex_array_object')

# GL/glext.h:5253
glIsVertexArray = _link_function('glIsVertexArray', GLboolean, [GLuint], 'ARB_vertex_array_object')

PFNGLBINDVERTEXARRAYPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:5255
PFNGLDELETEVERTEXARRAYSPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:5256
PFNGLGENVERTEXARRAYSPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:5257
PFNGLISVERTEXARRAYPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:5258
# EXT_abgr (GL/glext.h:5261)
GL_EXT_abgr = 1 	# GL/glext.h:5262
# EXT_blend_color (GL/glext.h:5265)
GL_EXT_blend_color = 1 	# GL/glext.h:5266
# GL/glext.h:5268
glBlendColorEXT = _link_function('glBlendColorEXT', None, [GLclampf, GLclampf, GLclampf, GLclampf], 'EXT_blend_color')

PFNGLBLENDCOLOREXTPROC = CFUNCTYPE(None, GLclampf, GLclampf, GLclampf, GLclampf) 	# GL/glext.h:5270
# EXT_polygon_offset (GL/glext.h:5273)
GL_EXT_polygon_offset = 1 	# GL/glext.h:5274
# GL/glext.h:5276
glPolygonOffsetEXT = _link_function('glPolygonOffsetEXT', None, [GLfloat, GLfloat], 'EXT_polygon_offset')

PFNGLPOLYGONOFFSETEXTPROC = CFUNCTYPE(None, GLfloat, GLfloat) 	# GL/glext.h:5278
# EXT_texture (GL/glext.h:5281)
GL_EXT_texture = 1 	# GL/glext.h:5282
# EXT_texture3D (GL/glext.h:5285)
GL_EXT_texture3D = 1 	# GL/glext.h:5286
# GL/glext.h:5288
glTexImage3DEXT = _link_function('glTexImage3DEXT', None, [GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)], 'EXT_texture3D')

# GL/glext.h:5289
glTexSubImage3DEXT = _link_function('glTexSubImage3DEXT', None, [GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_texture3D')

PFNGLTEXIMAGE3DEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:5291
PFNGLTEXSUBIMAGE3DEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:5292
# SGIS_texture_filter4 (GL/glext.h:5295)
GL_SGIS_texture_filter4 = 1 	# GL/glext.h:5296
# GL/glext.h:5298
glGetTexFilterFuncSGIS = _link_function('glGetTexFilterFuncSGIS', None, [GLenum, GLenum, POINTER(GLfloat)], 'SGIS_texture_filter4')

# GL/glext.h:5299
glTexFilterFuncSGIS = _link_function('glTexFilterFuncSGIS', None, [GLenum, GLenum, GLsizei, POINTER(GLfloat)], 'SGIS_texture_filter4')

PFNGLGETTEXFILTERFUNCSGISPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5301
PFNGLTEXFILTERFUNCSGISPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:5302
# EXT_subtexture (GL/glext.h:5305)
GL_EXT_subtexture = 1 	# GL/glext.h:5306
# GL/glext.h:5308
glTexSubImage1DEXT = _link_function('glTexSubImage1DEXT', None, [GLenum, GLint, GLint, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_subtexture')

# GL/glext.h:5309
glTexSubImage2DEXT = _link_function('glTexSubImage2DEXT', None, [GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_subtexture')

PFNGLTEXSUBIMAGE1DEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:5311
PFNGLTEXSUBIMAGE2DEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:5312
# EXT_copy_texture (GL/glext.h:5315)
GL_EXT_copy_texture = 1 	# GL/glext.h:5316
# GL/glext.h:5318
glCopyTexImage1DEXT = _link_function('glCopyTexImage1DEXT', None, [GLenum, GLint, GLenum, GLint, GLint, GLsizei, GLint], 'EXT_copy_texture')

# GL/glext.h:5319
glCopyTexImage2DEXT = _link_function('glCopyTexImage2DEXT', None, [GLenum, GLint, GLenum, GLint, GLint, GLsizei, GLsizei, GLint], 'EXT_copy_texture')

# GL/glext.h:5320
glCopyTexSubImage1DEXT = _link_function('glCopyTexSubImage1DEXT', None, [GLenum, GLint, GLint, GLint, GLint, GLsizei], 'EXT_copy_texture')

# GL/glext.h:5321
glCopyTexSubImage2DEXT = _link_function('glCopyTexSubImage2DEXT', None, [GLenum, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei], 'EXT_copy_texture')

# GL/glext.h:5322
glCopyTexSubImage3DEXT = _link_function('glCopyTexSubImage3DEXT', None, [GLenum, GLint, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei], 'EXT_copy_texture')

PFNGLCOPYTEXIMAGE1DEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLint, GLint, GLsizei, GLint) 	# GL/glext.h:5324
PFNGLCOPYTEXIMAGE2DEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLint, GLint, GLsizei, GLsizei, GLint) 	# GL/glext.h:5325
PFNGLCOPYTEXSUBIMAGE1DEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint, GLsizei) 	# GL/glext.h:5326
PFNGLCOPYTEXSUBIMAGE2DEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei) 	# GL/glext.h:5327
PFNGLCOPYTEXSUBIMAGE3DEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei) 	# GL/glext.h:5328
# EXT_histogram (GL/glext.h:5331)
GL_EXT_histogram = 1 	# GL/glext.h:5332
# GL/glext.h:5334
glGetHistogramEXT = _link_function('glGetHistogramEXT', None, [GLenum, GLboolean, GLenum, GLenum, POINTER(GLvoid)], 'EXT_histogram')

# GL/glext.h:5335
glGetHistogramParameterfvEXT = _link_function('glGetHistogramParameterfvEXT', None, [GLenum, GLenum, POINTER(GLfloat)], 'EXT_histogram')

# GL/glext.h:5336
glGetHistogramParameterivEXT = _link_function('glGetHistogramParameterivEXT', None, [GLenum, GLenum, POINTER(GLint)], 'EXT_histogram')

# GL/glext.h:5337
glGetMinmaxEXT = _link_function('glGetMinmaxEXT', None, [GLenum, GLboolean, GLenum, GLenum, POINTER(GLvoid)], 'EXT_histogram')

# GL/glext.h:5338
glGetMinmaxParameterfvEXT = _link_function('glGetMinmaxParameterfvEXT', None, [GLenum, GLenum, POINTER(GLfloat)], 'EXT_histogram')

# GL/glext.h:5339
glGetMinmaxParameterivEXT = _link_function('glGetMinmaxParameterivEXT', None, [GLenum, GLenum, POINTER(GLint)], 'EXT_histogram')

# GL/glext.h:5340
glHistogramEXT = _link_function('glHistogramEXT', None, [GLenum, GLsizei, GLenum, GLboolean], 'EXT_histogram')

# GL/glext.h:5341
glMinmaxEXT = _link_function('glMinmaxEXT', None, [GLenum, GLenum, GLboolean], 'EXT_histogram')

# GL/glext.h:5342
glResetHistogramEXT = _link_function('glResetHistogramEXT', None, [GLenum], 'EXT_histogram')

# GL/glext.h:5343
glResetMinmaxEXT = _link_function('glResetMinmaxEXT', None, [GLenum], 'EXT_histogram')

PFNGLGETHISTOGRAMEXTPROC = CFUNCTYPE(None, GLenum, GLboolean, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:5345
PFNGLGETHISTOGRAMPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5346
PFNGLGETHISTOGRAMPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5347
PFNGLGETMINMAXEXTPROC = CFUNCTYPE(None, GLenum, GLboolean, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:5348
PFNGLGETMINMAXPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5349
PFNGLGETMINMAXPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5350
PFNGLHISTOGRAMEXTPROC = CFUNCTYPE(None, GLenum, GLsizei, GLenum, GLboolean) 	# GL/glext.h:5351
PFNGLMINMAXEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLboolean) 	# GL/glext.h:5352
PFNGLRESETHISTOGRAMEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:5353
PFNGLRESETMINMAXEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:5354
# EXT_convolution (GL/glext.h:5357)
GL_EXT_convolution = 1 	# GL/glext.h:5358
# GL/glext.h:5360
glConvolutionFilter1DEXT = _link_function('glConvolutionFilter1DEXT', None, [GLenum, GLenum, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_convolution')

# GL/glext.h:5361
glConvolutionFilter2DEXT = _link_function('glConvolutionFilter2DEXT', None, [GLenum, GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_convolution')

# GL/glext.h:5362
glConvolutionParameterfEXT = _link_function('glConvolutionParameterfEXT', None, [GLenum, GLenum, GLfloat], 'EXT_convolution')

# GL/glext.h:5363
glConvolutionParameterfvEXT = _link_function('glConvolutionParameterfvEXT', None, [GLenum, GLenum, POINTER(GLfloat)], 'EXT_convolution')

# GL/glext.h:5364
glConvolutionParameteriEXT = _link_function('glConvolutionParameteriEXT', None, [GLenum, GLenum, GLint], 'EXT_convolution')

# GL/glext.h:5365
glConvolutionParameterivEXT = _link_function('glConvolutionParameterivEXT', None, [GLenum, GLenum, POINTER(GLint)], 'EXT_convolution')

# GL/glext.h:5366
glCopyConvolutionFilter1DEXT = _link_function('glCopyConvolutionFilter1DEXT', None, [GLenum, GLenum, GLint, GLint, GLsizei], 'EXT_convolution')

# GL/glext.h:5367
glCopyConvolutionFilter2DEXT = _link_function('glCopyConvolutionFilter2DEXT', None, [GLenum, GLenum, GLint, GLint, GLsizei, GLsizei], 'EXT_convolution')

# GL/glext.h:5368
glGetConvolutionFilterEXT = _link_function('glGetConvolutionFilterEXT', None, [GLenum, GLenum, GLenum, POINTER(GLvoid)], 'EXT_convolution')

# GL/glext.h:5369
glGetConvolutionParameterfvEXT = _link_function('glGetConvolutionParameterfvEXT', None, [GLenum, GLenum, POINTER(GLfloat)], 'EXT_convolution')

# GL/glext.h:5370
glGetConvolutionParameterivEXT = _link_function('glGetConvolutionParameterivEXT', None, [GLenum, GLenum, POINTER(GLint)], 'EXT_convolution')

# GL/glext.h:5371
glGetSeparableFilterEXT = _link_function('glGetSeparableFilterEXT', None, [GLenum, GLenum, GLenum, POINTER(GLvoid), POINTER(GLvoid), POINTER(GLvoid)], 'EXT_convolution')

# GL/glext.h:5372
glSeparableFilter2DEXT = _link_function('glSeparableFilter2DEXT', None, [GLenum, GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid), POINTER(GLvoid)], 'EXT_convolution')

PFNGLCONVOLUTIONFILTER1DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:5374
PFNGLCONVOLUTIONFILTER2DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:5375
PFNGLCONVOLUTIONPARAMETERFEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLfloat) 	# GL/glext.h:5376
PFNGLCONVOLUTIONPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5377
PFNGLCONVOLUTIONPARAMETERIEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint) 	# GL/glext.h:5378
PFNGLCONVOLUTIONPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5379
PFNGLCOPYCONVOLUTIONFILTER1DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLsizei) 	# GL/glext.h:5380
PFNGLCOPYCONVOLUTIONFILTER2DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLsizei, GLsizei) 	# GL/glext.h:5381
PFNGLGETCONVOLUTIONFILTEREXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:5382
PFNGLGETCONVOLUTIONPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5383
PFNGLGETCONVOLUTIONPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5384
PFNGLGETSEPARABLEFILTEREXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLvoid), POINTER(GLvoid), POINTER(GLvoid)) 	# GL/glext.h:5385
PFNGLSEPARABLEFILTER2DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid), POINTER(GLvoid)) 	# GL/glext.h:5386
# EXT_color_matrix (GL/glext.h:5389)
GL_EXT_color_matrix = 1 	# GL/glext.h:5390
# SGI_color_table (GL/glext.h:5393)
GL_SGI_color_table = 1 	# GL/glext.h:5394
# GL/glext.h:5396
glColorTableSGI = _link_function('glColorTableSGI', None, [GLenum, GLenum, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'SGI_color_table')

# GL/glext.h:5397
glColorTableParameterfvSGI = _link_function('glColorTableParameterfvSGI', None, [GLenum, GLenum, POINTER(GLfloat)], 'SGI_color_table')

# GL/glext.h:5398
glColorTableParameterivSGI = _link_function('glColorTableParameterivSGI', None, [GLenum, GLenum, POINTER(GLint)], 'SGI_color_table')

# GL/glext.h:5399
glCopyColorTableSGI = _link_function('glCopyColorTableSGI', None, [GLenum, GLenum, GLint, GLint, GLsizei], 'SGI_color_table')

# GL/glext.h:5400
glGetColorTableSGI = _link_function('glGetColorTableSGI', None, [GLenum, GLenum, GLenum, POINTER(GLvoid)], 'SGI_color_table')

# GL/glext.h:5401
glGetColorTableParameterfvSGI = _link_function('glGetColorTableParameterfvSGI', None, [GLenum, GLenum, POINTER(GLfloat)], 'SGI_color_table')

# GL/glext.h:5402
glGetColorTableParameterivSGI = _link_function('glGetColorTableParameterivSGI', None, [GLenum, GLenum, POINTER(GLint)], 'SGI_color_table')

PFNGLCOLORTABLESGIPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:5404
PFNGLCOLORTABLEPARAMETERFVSGIPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5405
PFNGLCOLORTABLEPARAMETERIVSGIPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5406
PFNGLCOPYCOLORTABLESGIPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLsizei) 	# GL/glext.h:5407
PFNGLGETCOLORTABLESGIPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:5408
PFNGLGETCOLORTABLEPARAMETERFVSGIPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5409
PFNGLGETCOLORTABLEPARAMETERIVSGIPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5410
# SGIX_pixel_texture (GL/glext.h:5413)
GL_SGIX_pixel_texture = 1 	# GL/glext.h:5414
# GL/glext.h:5416
glPixelTexGenSGIX = _link_function('glPixelTexGenSGIX', None, [GLenum], 'SGIX_pixel_texture')

PFNGLPIXELTEXGENSGIXPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:5418
# SGIS_pixel_texture (GL/glext.h:5421)
GL_SGIS_pixel_texture = 1 	# GL/glext.h:5422
# GL/glext.h:5424
glPixelTexGenParameteriSGIS = _link_function('glPixelTexGenParameteriSGIS', None, [GLenum, GLint], 'SGIS_pixel_texture')

# GL/glext.h:5425
glPixelTexGenParameterivSGIS = _link_function('glPixelTexGenParameterivSGIS', None, [GLenum, POINTER(GLint)], 'SGIS_pixel_texture')

# GL/glext.h:5426
glPixelTexGenParameterfSGIS = _link_function('glPixelTexGenParameterfSGIS', None, [GLenum, GLfloat], 'SGIS_pixel_texture')

# GL/glext.h:5427
glPixelTexGenParameterfvSGIS = _link_function('glPixelTexGenParameterfvSGIS', None, [GLenum, POINTER(GLfloat)], 'SGIS_pixel_texture')

# GL/glext.h:5428
glGetPixelTexGenParameterivSGIS = _link_function('glGetPixelTexGenParameterivSGIS', None, [GLenum, POINTER(GLint)], 'SGIS_pixel_texture')

# GL/glext.h:5429
glGetPixelTexGenParameterfvSGIS = _link_function('glGetPixelTexGenParameterfvSGIS', None, [GLenum, POINTER(GLfloat)], 'SGIS_pixel_texture')

PFNGLPIXELTEXGENPARAMETERISGISPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:5431
PFNGLPIXELTEXGENPARAMETERIVSGISPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:5432
PFNGLPIXELTEXGENPARAMETERFSGISPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:5433
PFNGLPIXELTEXGENPARAMETERFVSGISPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5434
PFNGLGETPIXELTEXGENPARAMETERIVSGISPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:5435
PFNGLGETPIXELTEXGENPARAMETERFVSGISPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5436
# SGIS_texture4D (GL/glext.h:5439)
GL_SGIS_texture4D = 1 	# GL/glext.h:5440
# GL/glext.h:5442
glTexImage4DSGIS = _link_function('glTexImage4DSGIS', None, [GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)], 'SGIS_texture4D')

# GL/glext.h:5443
glTexSubImage4DSGIS = _link_function('glTexSubImage4DSGIS', None, [GLenum, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'SGIS_texture4D')

PFNGLTEXIMAGE4DSGISPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:5445
PFNGLTEXSUBIMAGE4DSGISPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:5446
# SGI_texture_color_table (GL/glext.h:5449)
GL_SGI_texture_color_table = 1 	# GL/glext.h:5450
# EXT_cmyka (GL/glext.h:5453)
GL_EXT_cmyka = 1 	# GL/glext.h:5454
# EXT_texture_object (GL/glext.h:5457)
GL_EXT_texture_object = 1 	# GL/glext.h:5458
# GL/glext.h:5460
glAreTexturesResidentEXT = _link_function('glAreTexturesResidentEXT', GLboolean, [GLsizei, POINTER(GLuint), POINTER(GLboolean)], 'EXT_texture_object')

# GL/glext.h:5461
glBindTextureEXT = _link_function('glBindTextureEXT', None, [GLenum, GLuint], 'EXT_texture_object')

# GL/glext.h:5462
glDeleteTexturesEXT = _link_function('glDeleteTexturesEXT', None, [GLsizei, POINTER(GLuint)], 'EXT_texture_object')

# GL/glext.h:5463
glGenTexturesEXT = _link_function('glGenTexturesEXT', None, [GLsizei, POINTER(GLuint)], 'EXT_texture_object')

# GL/glext.h:5464
glIsTextureEXT = _link_function('glIsTextureEXT', GLboolean, [GLuint], 'EXT_texture_object')

# GL/glext.h:5465
glPrioritizeTexturesEXT = _link_function('glPrioritizeTexturesEXT', None, [GLsizei, POINTER(GLuint), POINTER(GLclampf)], 'EXT_texture_object')

PFNGLARETEXTURESRESIDENTEXTPROC = CFUNCTYPE(GLboolean, GLsizei, POINTER(GLuint), POINTER(GLboolean)) 	# GL/glext.h:5467
PFNGLBINDTEXTUREEXTPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:5468
PFNGLDELETETEXTURESEXTPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:5469
PFNGLGENTEXTURESEXTPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:5470
PFNGLISTEXTUREEXTPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:5471
PFNGLPRIORITIZETEXTURESEXTPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint), POINTER(GLclampf)) 	# GL/glext.h:5472
# SGIS_detail_texture (GL/glext.h:5475)
GL_SGIS_detail_texture = 1 	# GL/glext.h:5476
# GL/glext.h:5478
glDetailTexFuncSGIS = _link_function('glDetailTexFuncSGIS', None, [GLenum, GLsizei, POINTER(GLfloat)], 'SGIS_detail_texture')

# GL/glext.h:5479
glGetDetailTexFuncSGIS = _link_function('glGetDetailTexFuncSGIS', None, [GLenum, POINTER(GLfloat)], 'SGIS_detail_texture')

PFNGLDETAILTEXFUNCSGISPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:5481
PFNGLGETDETAILTEXFUNCSGISPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5482
# SGIS_sharpen_texture (GL/glext.h:5485)
GL_SGIS_sharpen_texture = 1 	# GL/glext.h:5486
# GL/glext.h:5488
glSharpenTexFuncSGIS = _link_function('glSharpenTexFuncSGIS', None, [GLenum, GLsizei, POINTER(GLfloat)], 'SGIS_sharpen_texture')

# GL/glext.h:5489
glGetSharpenTexFuncSGIS = _link_function('glGetSharpenTexFuncSGIS', None, [GLenum, POINTER(GLfloat)], 'SGIS_sharpen_texture')

PFNGLSHARPENTEXFUNCSGISPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:5491
PFNGLGETSHARPENTEXFUNCSGISPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5492
# EXT_packed_pixels (GL/glext.h:5495)
GL_EXT_packed_pixels = 1 	# GL/glext.h:5496
# SGIS_texture_lod (GL/glext.h:5499)
GL_SGIS_texture_lod = 1 	# GL/glext.h:5500
# SGIS_multisample (GL/glext.h:5503)
GL_SGIS_multisample = 1 	# GL/glext.h:5504
# GL/glext.h:5506
glSampleMaskSGIS = _link_function('glSampleMaskSGIS', None, [GLclampf, GLboolean], 'SGIS_multisample')

# GL/glext.h:5507
glSamplePatternSGIS = _link_function('glSamplePatternSGIS', None, [GLenum], 'SGIS_multisample')

PFNGLSAMPLEMASKSGISPROC = CFUNCTYPE(None, GLclampf, GLboolean) 	# GL/glext.h:5509
PFNGLSAMPLEPATTERNSGISPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:5510
# EXT_rescale_normal (GL/glext.h:5513)
GL_EXT_rescale_normal = 1 	# GL/glext.h:5514
# EXT_vertex_array (GL/glext.h:5517)
GL_EXT_vertex_array = 1 	# GL/glext.h:5518
# GL/glext.h:5520
glArrayElementEXT = _link_function('glArrayElementEXT', None, [GLint], 'EXT_vertex_array')

# GL/glext.h:5521
glColorPointerEXT = _link_function('glColorPointerEXT', None, [GLint, GLenum, GLsizei, GLsizei, POINTER(GLvoid)], 'EXT_vertex_array')

# GL/glext.h:5522
glDrawArraysEXT = _link_function('glDrawArraysEXT', None, [GLenum, GLint, GLsizei], 'EXT_vertex_array')

# GL/glext.h:5523
glEdgeFlagPointerEXT = _link_function('glEdgeFlagPointerEXT', None, [GLsizei, GLsizei, POINTER(GLboolean)], 'EXT_vertex_array')

# GL/glext.h:5524
glGetPointervEXT = _link_function('glGetPointervEXT', None, [GLenum, POINTER(POINTER(GLvoid))], 'EXT_vertex_array')

# GL/glext.h:5525
glIndexPointerEXT = _link_function('glIndexPointerEXT', None, [GLenum, GLsizei, GLsizei, POINTER(GLvoid)], 'EXT_vertex_array')

# GL/glext.h:5526
glNormalPointerEXT = _link_function('glNormalPointerEXT', None, [GLenum, GLsizei, GLsizei, POINTER(GLvoid)], 'EXT_vertex_array')

# GL/glext.h:5527
glTexCoordPointerEXT = _link_function('glTexCoordPointerEXT', None, [GLint, GLenum, GLsizei, GLsizei, POINTER(GLvoid)], 'EXT_vertex_array')

# GL/glext.h:5528
glVertexPointerEXT = _link_function('glVertexPointerEXT', None, [GLint, GLenum, GLsizei, GLsizei, POINTER(GLvoid)], 'EXT_vertex_array')

PFNGLARRAYELEMENTEXTPROC = CFUNCTYPE(None, GLint) 	# GL/glext.h:5530
PFNGLCOLORPOINTEREXTPROC = CFUNCTYPE(None, GLint, GLenum, GLsizei, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:5531
PFNGLDRAWARRAYSEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLsizei) 	# GL/glext.h:5532
PFNGLEDGEFLAGPOINTEREXTPROC = CFUNCTYPE(None, GLsizei, GLsizei, POINTER(GLboolean)) 	# GL/glext.h:5533
PFNGLGETPOINTERVEXTPROC = CFUNCTYPE(None, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:5534
PFNGLINDEXPOINTEREXTPROC = CFUNCTYPE(None, GLenum, GLsizei, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:5535
PFNGLNORMALPOINTEREXTPROC = CFUNCTYPE(None, GLenum, GLsizei, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:5536
PFNGLTEXCOORDPOINTEREXTPROC = CFUNCTYPE(None, GLint, GLenum, GLsizei, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:5537
PFNGLVERTEXPOINTEREXTPROC = CFUNCTYPE(None, GLint, GLenum, GLsizei, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:5538
# EXT_misc_attribute (GL/glext.h:5541)
GL_EXT_misc_attribute = 1 	# GL/glext.h:5542
# SGIS_generate_mipmap (GL/glext.h:5545)
GL_SGIS_generate_mipmap = 1 	# GL/glext.h:5546
# SGIX_clipmap (GL/glext.h:5549)
GL_SGIX_clipmap = 1 	# GL/glext.h:5550
# SGIX_shadow (GL/glext.h:5553)
GL_SGIX_shadow = 1 	# GL/glext.h:5554
# SGIS_texture_edge_clamp (GL/glext.h:5557)
GL_SGIS_texture_edge_clamp = 1 	# GL/glext.h:5558
# SGIS_texture_border_clamp (GL/glext.h:5561)
GL_SGIS_texture_border_clamp = 1 	# GL/glext.h:5562
# EXT_blend_minmax (GL/glext.h:5565)
GL_EXT_blend_minmax = 1 	# GL/glext.h:5566
# GL/glext.h:5568
glBlendEquationEXT = _link_function('glBlendEquationEXT', None, [GLenum], 'EXT_blend_minmax')

PFNGLBLENDEQUATIONEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:5570
# EXT_blend_subtract (GL/glext.h:5573)
GL_EXT_blend_subtract = 1 	# GL/glext.h:5574
# EXT_blend_logic_op (GL/glext.h:5577)
GL_EXT_blend_logic_op = 1 	# GL/glext.h:5578
# SGIX_interlace (GL/glext.h:5581)
GL_SGIX_interlace = 1 	# GL/glext.h:5582
# SGIX_pixel_tiles (GL/glext.h:5585)
GL_SGIX_pixel_tiles = 1 	# GL/glext.h:5586
# SGIX_texture_select (GL/glext.h:5589)
GL_SGIX_texture_select = 1 	# GL/glext.h:5590
# SGIX_sprite (GL/glext.h:5593)
GL_SGIX_sprite = 1 	# GL/glext.h:5594
# GL/glext.h:5596
glSpriteParameterfSGIX = _link_function('glSpriteParameterfSGIX', None, [GLenum, GLfloat], 'SGIX_sprite')

# GL/glext.h:5597
glSpriteParameterfvSGIX = _link_function('glSpriteParameterfvSGIX', None, [GLenum, POINTER(GLfloat)], 'SGIX_sprite')

# GL/glext.h:5598
glSpriteParameteriSGIX = _link_function('glSpriteParameteriSGIX', None, [GLenum, GLint], 'SGIX_sprite')

# GL/glext.h:5599
glSpriteParameterivSGIX = _link_function('glSpriteParameterivSGIX', None, [GLenum, POINTER(GLint)], 'SGIX_sprite')

PFNGLSPRITEPARAMETERFSGIXPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:5601
PFNGLSPRITEPARAMETERFVSGIXPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5602
PFNGLSPRITEPARAMETERISGIXPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:5603
PFNGLSPRITEPARAMETERIVSGIXPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:5604
# SGIX_texture_multi_buffer (GL/glext.h:5607)
GL_SGIX_texture_multi_buffer = 1 	# GL/glext.h:5608
# EXT_point_parameters (GL/glext.h:5611)
GL_EXT_point_parameters = 1 	# GL/glext.h:5612
# GL/glext.h:5614
glPointParameterfEXT = _link_function('glPointParameterfEXT', None, [GLenum, GLfloat], 'EXT_point_parameters')

# GL/glext.h:5615
glPointParameterfvEXT = _link_function('glPointParameterfvEXT', None, [GLenum, POINTER(GLfloat)], 'EXT_point_parameters')

PFNGLPOINTPARAMETERFEXTPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:5617
PFNGLPOINTPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5618
# SGIS_point_parameters (GL/glext.h:5621)
GL_SGIS_point_parameters = 1 	# GL/glext.h:5622
# GL/glext.h:5624
glPointParameterfSGIS = _link_function('glPointParameterfSGIS', None, [GLenum, GLfloat], 'SGIS_point_parameters')

# GL/glext.h:5625
glPointParameterfvSGIS = _link_function('glPointParameterfvSGIS', None, [GLenum, POINTER(GLfloat)], 'SGIS_point_parameters')

PFNGLPOINTPARAMETERFSGISPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:5627
PFNGLPOINTPARAMETERFVSGISPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5628
# SGIX_instruments (GL/glext.h:5631)
GL_SGIX_instruments = 1 	# GL/glext.h:5632
# GL/glext.h:5634
glGetInstrumentsSGIX = _link_function('glGetInstrumentsSGIX', GLint, [], 'SGIX_instruments')

# GL/glext.h:5635
glInstrumentsBufferSGIX = _link_function('glInstrumentsBufferSGIX', None, [GLsizei, POINTER(GLint)], 'SGIX_instruments')

# GL/glext.h:5636
glPollInstrumentsSGIX = _link_function('glPollInstrumentsSGIX', GLint, [POINTER(GLint)], 'SGIX_instruments')

# GL/glext.h:5637
glReadInstrumentsSGIX = _link_function('glReadInstrumentsSGIX', None, [GLint], 'SGIX_instruments')

# GL/glext.h:5638
glStartInstrumentsSGIX = _link_function('glStartInstrumentsSGIX', None, [], 'SGIX_instruments')

# GL/glext.h:5639
glStopInstrumentsSGIX = _link_function('glStopInstrumentsSGIX', None, [GLint], 'SGIX_instruments')

PFNGLGETINSTRUMENTSSGIXPROC = CFUNCTYPE(GLint) 	# GL/glext.h:5641
PFNGLINSTRUMENTSBUFFERSGIXPROC = CFUNCTYPE(None, GLsizei, POINTER(GLint)) 	# GL/glext.h:5642
PFNGLPOLLINSTRUMENTSSGIXPROC = CFUNCTYPE(GLint, POINTER(GLint)) 	# GL/glext.h:5643
PFNGLREADINSTRUMENTSSGIXPROC = CFUNCTYPE(None, GLint) 	# GL/glext.h:5644
PFNGLSTARTINSTRUMENTSSGIXPROC = CFUNCTYPE(None) 	# GL/glext.h:5645
PFNGLSTOPINSTRUMENTSSGIXPROC = CFUNCTYPE(None, GLint) 	# GL/glext.h:5646
# SGIX_texture_scale_bias (GL/glext.h:5649)
GL_SGIX_texture_scale_bias = 1 	# GL/glext.h:5650
# SGIX_framezoom (GL/glext.h:5653)
GL_SGIX_framezoom = 1 	# GL/glext.h:5654
# GL/glext.h:5656
glFrameZoomSGIX = _link_function('glFrameZoomSGIX', None, [GLint], 'SGIX_framezoom')

PFNGLFRAMEZOOMSGIXPROC = CFUNCTYPE(None, GLint) 	# GL/glext.h:5658
# SGIX_tag_sample_buffer (GL/glext.h:5661)
GL_SGIX_tag_sample_buffer = 1 	# GL/glext.h:5662
# GL/glext.h:5664
glTagSampleBufferSGIX = _link_function('glTagSampleBufferSGIX', None, [], 'SGIX_tag_sample_buffer')

PFNGLTAGSAMPLEBUFFERSGIXPROC = CFUNCTYPE(None) 	# GL/glext.h:5666
# SGIX_polynomial_ffd (GL/glext.h:5669)
GL_SGIX_polynomial_ffd = 1 	# GL/glext.h:5670
# GL/glext.h:5672
glDeformationMap3dSGIX = _link_function('glDeformationMap3dSGIX', None, [GLenum, GLdouble, GLdouble, GLint, GLint, GLdouble, GLdouble, GLint, GLint, GLdouble, GLdouble, GLint, GLint, POINTER(GLdouble)], 'SGIX_polynomial_ffd')

# GL/glext.h:5673
glDeformationMap3fSGIX = _link_function('glDeformationMap3fSGIX', None, [GLenum, GLfloat, GLfloat, GLint, GLint, GLfloat, GLfloat, GLint, GLint, GLfloat, GLfloat, GLint, GLint, POINTER(GLfloat)], 'SGIX_polynomial_ffd')

# GL/glext.h:5674
glDeformSGIX = _link_function('glDeformSGIX', None, [GLbitfield], 'SGIX_polynomial_ffd')

# GL/glext.h:5675
glLoadIdentityDeformationMapSGIX = _link_function('glLoadIdentityDeformationMapSGIX', None, [GLbitfield], 'SGIX_polynomial_ffd')

PFNGLDEFORMATIONMAP3DSGIXPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble, GLint, GLint, GLdouble, GLdouble, GLint, GLint, GLdouble, GLdouble, GLint, GLint, POINTER(GLdouble)) 	# GL/glext.h:5677
PFNGLDEFORMATIONMAP3FSGIXPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat, GLint, GLint, GLfloat, GLfloat, GLint, GLint, GLfloat, GLfloat, GLint, GLint, POINTER(GLfloat)) 	# GL/glext.h:5678
PFNGLDEFORMSGIXPROC = CFUNCTYPE(None, GLbitfield) 	# GL/glext.h:5679
PFNGLLOADIDENTITYDEFORMATIONMAPSGIXPROC = CFUNCTYPE(None, GLbitfield) 	# GL/glext.h:5680
# SGIX_reference_plane (GL/glext.h:5683)
GL_SGIX_reference_plane = 1 	# GL/glext.h:5684
# GL/glext.h:5686
glReferencePlaneSGIX = _link_function('glReferencePlaneSGIX', None, [POINTER(GLdouble)], 'SGIX_reference_plane')

PFNGLREFERENCEPLANESGIXPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:5688
# SGIX_flush_raster (GL/glext.h:5691)
GL_SGIX_flush_raster = 1 	# GL/glext.h:5692
# GL/glext.h:5694
glFlushRasterSGIX = _link_function('glFlushRasterSGIX', None, [], 'SGIX_flush_raster')

PFNGLFLUSHRASTERSGIXPROC = CFUNCTYPE(None) 	# GL/glext.h:5696
# SGIX_depth_texture (GL/glext.h:5699)
GL_SGIX_depth_texture = 1 	# GL/glext.h:5700
# SGIS_fog_function (GL/glext.h:5703)
GL_SGIS_fog_function = 1 	# GL/glext.h:5704
# GL/glext.h:5706
glFogFuncSGIS = _link_function('glFogFuncSGIS', None, [GLsizei, POINTER(GLfloat)], 'SGIS_fog_function')

# GL/glext.h:5707
glGetFogFuncSGIS = _link_function('glGetFogFuncSGIS', None, [POINTER(GLfloat)], 'SGIS_fog_function')

PFNGLFOGFUNCSGISPROC = CFUNCTYPE(None, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:5709
PFNGLGETFOGFUNCSGISPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:5710
# SGIX_fog_offset (GL/glext.h:5713)
GL_SGIX_fog_offset = 1 	# GL/glext.h:5714
# HP_image_transform (GL/glext.h:5717)
GL_HP_image_transform = 1 	# GL/glext.h:5718
# GL/glext.h:5720
glImageTransformParameteriHP = _link_function('glImageTransformParameteriHP', None, [GLenum, GLenum, GLint], 'HP_image_transform')

# GL/glext.h:5721
glImageTransformParameterfHP = _link_function('glImageTransformParameterfHP', None, [GLenum, GLenum, GLfloat], 'HP_image_transform')

# GL/glext.h:5722
glImageTransformParameterivHP = _link_function('glImageTransformParameterivHP', None, [GLenum, GLenum, POINTER(GLint)], 'HP_image_transform')

# GL/glext.h:5723
glImageTransformParameterfvHP = _link_function('glImageTransformParameterfvHP', None, [GLenum, GLenum, POINTER(GLfloat)], 'HP_image_transform')

# GL/glext.h:5724
glGetImageTransformParameterivHP = _link_function('glGetImageTransformParameterivHP', None, [GLenum, GLenum, POINTER(GLint)], 'HP_image_transform')

# GL/glext.h:5725
glGetImageTransformParameterfvHP = _link_function('glGetImageTransformParameterfvHP', None, [GLenum, GLenum, POINTER(GLfloat)], 'HP_image_transform')

PFNGLIMAGETRANSFORMPARAMETERIHPPROC = CFUNCTYPE(None, GLenum, GLenum, GLint) 	# GL/glext.h:5727
PFNGLIMAGETRANSFORMPARAMETERFHPPROC = CFUNCTYPE(None, GLenum, GLenum, GLfloat) 	# GL/glext.h:5728
PFNGLIMAGETRANSFORMPARAMETERIVHPPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5729
PFNGLIMAGETRANSFORMPARAMETERFVHPPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5730
PFNGLGETIMAGETRANSFORMPARAMETERIVHPPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5731
PFNGLGETIMAGETRANSFORMPARAMETERFVHPPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5732
# HP_convolution_border_modes (GL/glext.h:5735)
GL_HP_convolution_border_modes = 1 	# GL/glext.h:5736
# SGIX_texture_add_env (GL/glext.h:5739)
GL_SGIX_texture_add_env = 1 	# GL/glext.h:5740
# EXT_color_subtable (GL/glext.h:5743)
GL_EXT_color_subtable = 1 	# GL/glext.h:5744
# GL/glext.h:5746
glColorSubTableEXT = _link_function('glColorSubTableEXT', None, [GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_color_subtable')

# GL/glext.h:5747
glCopyColorSubTableEXT = _link_function('glCopyColorSubTableEXT', None, [GLenum, GLsizei, GLint, GLint, GLsizei], 'EXT_color_subtable')

PFNGLCOLORSUBTABLEEXTPROC = CFUNCTYPE(None, GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:5749
PFNGLCOPYCOLORSUBTABLEEXTPROC = CFUNCTYPE(None, GLenum, GLsizei, GLint, GLint, GLsizei) 	# GL/glext.h:5750
# PGI_vertex_hints (GL/glext.h:5753)
GL_PGI_vertex_hints = 1 	# GL/glext.h:5754
# PGI_misc_hints (GL/glext.h:5757)
GL_PGI_misc_hints = 1 	# GL/glext.h:5758
# GL/glext.h:5760
glHintPGI = _link_function('glHintPGI', None, [GLenum, GLint], 'PGI_misc_hints')

PFNGLHINTPGIPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:5762
# EXT_paletted_texture (GL/glext.h:5765)
GL_EXT_paletted_texture = 1 	# GL/glext.h:5766
# GL/glext.h:5768
glColorTableEXT = _link_function('glColorTableEXT', None, [GLenum, GLenum, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_paletted_texture')

# GL/glext.h:5769
glGetColorTableEXT = _link_function('glGetColorTableEXT', None, [GLenum, GLenum, GLenum, POINTER(GLvoid)], 'EXT_paletted_texture')

# GL/glext.h:5770
glGetColorTableParameterivEXT = _link_function('glGetColorTableParameterivEXT', None, [GLenum, GLenum, POINTER(GLint)], 'EXT_paletted_texture')

# GL/glext.h:5771
glGetColorTableParameterfvEXT = _link_function('glGetColorTableParameterfvEXT', None, [GLenum, GLenum, POINTER(GLfloat)], 'EXT_paletted_texture')

PFNGLCOLORTABLEEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:5773
PFNGLGETCOLORTABLEEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:5774
PFNGLGETCOLORTABLEPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5775
PFNGLGETCOLORTABLEPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5776
# EXT_clip_volume_hint (GL/glext.h:5779)
GL_EXT_clip_volume_hint = 1 	# GL/glext.h:5780
# SGIX_list_priority (GL/glext.h:5783)
GL_SGIX_list_priority = 1 	# GL/glext.h:5784
# GL/glext.h:5786
glGetListParameterfvSGIX = _link_function('glGetListParameterfvSGIX', None, [GLuint, GLenum, POINTER(GLfloat)], 'SGIX_list_priority')

# GL/glext.h:5787
glGetListParameterivSGIX = _link_function('glGetListParameterivSGIX', None, [GLuint, GLenum, POINTER(GLint)], 'SGIX_list_priority')

# GL/glext.h:5788
glListParameterfSGIX = _link_function('glListParameterfSGIX', None, [GLuint, GLenum, GLfloat], 'SGIX_list_priority')

# GL/glext.h:5789
glListParameterfvSGIX = _link_function('glListParameterfvSGIX', None, [GLuint, GLenum, POINTER(GLfloat)], 'SGIX_list_priority')

# GL/glext.h:5790
glListParameteriSGIX = _link_function('glListParameteriSGIX', None, [GLuint, GLenum, GLint], 'SGIX_list_priority')

# GL/glext.h:5791
glListParameterivSGIX = _link_function('glListParameterivSGIX', None, [GLuint, GLenum, POINTER(GLint)], 'SGIX_list_priority')

PFNGLGETLISTPARAMETERFVSGIXPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5793
PFNGLGETLISTPARAMETERIVSGIXPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:5794
PFNGLLISTPARAMETERFSGIXPROC = CFUNCTYPE(None, GLuint, GLenum, GLfloat) 	# GL/glext.h:5795
PFNGLLISTPARAMETERFVSGIXPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5796
PFNGLLISTPARAMETERISGIXPROC = CFUNCTYPE(None, GLuint, GLenum, GLint) 	# GL/glext.h:5797
PFNGLLISTPARAMETERIVSGIXPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:5798
# SGIX_ir_instrument1 (GL/glext.h:5801)
GL_SGIX_ir_instrument1 = 1 	# GL/glext.h:5802
# SGIX_calligraphic_fragment (GL/glext.h:5805)
GL_SGIX_calligraphic_fragment = 1 	# GL/glext.h:5806
# SGIX_texture_lod_bias (GL/glext.h:5809)
GL_SGIX_texture_lod_bias = 1 	# GL/glext.h:5810
# SGIX_shadow_ambient (GL/glext.h:5813)
GL_SGIX_shadow_ambient = 1 	# GL/glext.h:5814
# EXT_index_texture (GL/glext.h:5817)
GL_EXT_index_texture = 1 	# GL/glext.h:5818
# EXT_index_material (GL/glext.h:5821)
GL_EXT_index_material = 1 	# GL/glext.h:5822
# GL/glext.h:5824
glIndexMaterialEXT = _link_function('glIndexMaterialEXT', None, [GLenum, GLenum], 'EXT_index_material')

PFNGLINDEXMATERIALEXTPROC = CFUNCTYPE(None, GLenum, GLenum) 	# GL/glext.h:5826
# EXT_index_func (GL/glext.h:5829)
GL_EXT_index_func = 1 	# GL/glext.h:5830
# GL/glext.h:5832
glIndexFuncEXT = _link_function('glIndexFuncEXT', None, [GLenum, GLclampf], 'EXT_index_func')

PFNGLINDEXFUNCEXTPROC = CFUNCTYPE(None, GLenum, GLclampf) 	# GL/glext.h:5834
# EXT_index_array_formats (GL/glext.h:5837)
GL_EXT_index_array_formats = 1 	# GL/glext.h:5838
# EXT_compiled_vertex_array (GL/glext.h:5841)
GL_EXT_compiled_vertex_array = 1 	# GL/glext.h:5842
# GL/glext.h:5844
glLockArraysEXT = _link_function('glLockArraysEXT', None, [GLint, GLsizei], 'EXT_compiled_vertex_array')

# GL/glext.h:5845
glUnlockArraysEXT = _link_function('glUnlockArraysEXT', None, [], 'EXT_compiled_vertex_array')

PFNGLLOCKARRAYSEXTPROC = CFUNCTYPE(None, GLint, GLsizei) 	# GL/glext.h:5847
PFNGLUNLOCKARRAYSEXTPROC = CFUNCTYPE(None) 	# GL/glext.h:5848
# EXT_cull_vertex (GL/glext.h:5851)
GL_EXT_cull_vertex = 1 	# GL/glext.h:5852
# GL/glext.h:5854
glCullParameterdvEXT = _link_function('glCullParameterdvEXT', None, [GLenum, POINTER(GLdouble)], 'EXT_cull_vertex')

# GL/glext.h:5855
glCullParameterfvEXT = _link_function('glCullParameterfvEXT', None, [GLenum, POINTER(GLfloat)], 'EXT_cull_vertex')

PFNGLCULLPARAMETERDVEXTPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:5857
PFNGLCULLPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5858
# SGIX_ycrcb (GL/glext.h:5861)
GL_SGIX_ycrcb = 1 	# GL/glext.h:5862
# SGIX_fragment_lighting (GL/glext.h:5865)
GL_SGIX_fragment_lighting = 1 	# GL/glext.h:5866
# GL/glext.h:5868
glFragmentColorMaterialSGIX = _link_function('glFragmentColorMaterialSGIX', None, [GLenum, GLenum], 'SGIX_fragment_lighting')

# GL/glext.h:5869
glFragmentLightfSGIX = _link_function('glFragmentLightfSGIX', None, [GLenum, GLenum, GLfloat], 'SGIX_fragment_lighting')

# GL/glext.h:5870
glFragmentLightfvSGIX = _link_function('glFragmentLightfvSGIX', None, [GLenum, GLenum, POINTER(GLfloat)], 'SGIX_fragment_lighting')

# GL/glext.h:5871
glFragmentLightiSGIX = _link_function('glFragmentLightiSGIX', None, [GLenum, GLenum, GLint], 'SGIX_fragment_lighting')

# GL/glext.h:5872
glFragmentLightivSGIX = _link_function('glFragmentLightivSGIX', None, [GLenum, GLenum, POINTER(GLint)], 'SGIX_fragment_lighting')

# GL/glext.h:5873
glFragmentLightModelfSGIX = _link_function('glFragmentLightModelfSGIX', None, [GLenum, GLfloat], 'SGIX_fragment_lighting')

# GL/glext.h:5874
glFragmentLightModelfvSGIX = _link_function('glFragmentLightModelfvSGIX', None, [GLenum, POINTER(GLfloat)], 'SGIX_fragment_lighting')

# GL/glext.h:5875
glFragmentLightModeliSGIX = _link_function('glFragmentLightModeliSGIX', None, [GLenum, GLint], 'SGIX_fragment_lighting')

# GL/glext.h:5876
glFragmentLightModelivSGIX = _link_function('glFragmentLightModelivSGIX', None, [GLenum, POINTER(GLint)], 'SGIX_fragment_lighting')

# GL/glext.h:5877
glFragmentMaterialfSGIX = _link_function('glFragmentMaterialfSGIX', None, [GLenum, GLenum, GLfloat], 'SGIX_fragment_lighting')

# GL/glext.h:5878
glFragmentMaterialfvSGIX = _link_function('glFragmentMaterialfvSGIX', None, [GLenum, GLenum, POINTER(GLfloat)], 'SGIX_fragment_lighting')

# GL/glext.h:5879
glFragmentMaterialiSGIX = _link_function('glFragmentMaterialiSGIX', None, [GLenum, GLenum, GLint], 'SGIX_fragment_lighting')

# GL/glext.h:5880
glFragmentMaterialivSGIX = _link_function('glFragmentMaterialivSGIX', None, [GLenum, GLenum, POINTER(GLint)], 'SGIX_fragment_lighting')

# GL/glext.h:5881
glGetFragmentLightfvSGIX = _link_function('glGetFragmentLightfvSGIX', None, [GLenum, GLenum, POINTER(GLfloat)], 'SGIX_fragment_lighting')

# GL/glext.h:5882
glGetFragmentLightivSGIX = _link_function('glGetFragmentLightivSGIX', None, [GLenum, GLenum, POINTER(GLint)], 'SGIX_fragment_lighting')

# GL/glext.h:5883
glGetFragmentMaterialfvSGIX = _link_function('glGetFragmentMaterialfvSGIX', None, [GLenum, GLenum, POINTER(GLfloat)], 'SGIX_fragment_lighting')

# GL/glext.h:5884
glGetFragmentMaterialivSGIX = _link_function('glGetFragmentMaterialivSGIX', None, [GLenum, GLenum, POINTER(GLint)], 'SGIX_fragment_lighting')

# GL/glext.h:5885
glLightEnviSGIX = _link_function('glLightEnviSGIX', None, [GLenum, GLint], 'SGIX_fragment_lighting')

PFNGLFRAGMENTCOLORMATERIALSGIXPROC = CFUNCTYPE(None, GLenum, GLenum) 	# GL/glext.h:5887
PFNGLFRAGMENTLIGHTFSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, GLfloat) 	# GL/glext.h:5888
PFNGLFRAGMENTLIGHTFVSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5889
PFNGLFRAGMENTLIGHTISGIXPROC = CFUNCTYPE(None, GLenum, GLenum, GLint) 	# GL/glext.h:5890
PFNGLFRAGMENTLIGHTIVSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5891
PFNGLFRAGMENTLIGHTMODELFSGIXPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:5892
PFNGLFRAGMENTLIGHTMODELFVSGIXPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5893
PFNGLFRAGMENTLIGHTMODELISGIXPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:5894
PFNGLFRAGMENTLIGHTMODELIVSGIXPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:5895
PFNGLFRAGMENTMATERIALFSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, GLfloat) 	# GL/glext.h:5896
PFNGLFRAGMENTMATERIALFVSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5897
PFNGLFRAGMENTMATERIALISGIXPROC = CFUNCTYPE(None, GLenum, GLenum, GLint) 	# GL/glext.h:5898
PFNGLFRAGMENTMATERIALIVSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5899
PFNGLGETFRAGMENTLIGHTFVSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5900
PFNGLGETFRAGMENTLIGHTIVSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5901
PFNGLGETFRAGMENTMATERIALFVSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5902
PFNGLGETFRAGMENTMATERIALIVSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5903
PFNGLLIGHTENVISGIXPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:5904
# IBM_rasterpos_clip (GL/glext.h:5907)
GL_IBM_rasterpos_clip = 1 	# GL/glext.h:5908
# HP_texture_lighting (GL/glext.h:5911)
GL_HP_texture_lighting = 1 	# GL/glext.h:5912
# EXT_draw_range_elements (GL/glext.h:5915)
GL_EXT_draw_range_elements = 1 	# GL/glext.h:5916
# GL/glext.h:5918
glDrawRangeElementsEXT = _link_function('glDrawRangeElementsEXT', None, [GLenum, GLuint, GLuint, GLsizei, GLenum, POINTER(GLvoid)], 'EXT_draw_range_elements')

PFNGLDRAWRANGEELEMENTSEXTPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLsizei, GLenum, POINTER(GLvoid)) 	# GL/glext.h:5920
# WIN_phong_shading (GL/glext.h:5923)
GL_WIN_phong_shading = 1 	# GL/glext.h:5924
# WIN_specular_fog (GL/glext.h:5927)
GL_WIN_specular_fog = 1 	# GL/glext.h:5928
# EXT_light_texture (GL/glext.h:5931)
GL_EXT_light_texture = 1 	# GL/glext.h:5932
# GL/glext.h:5934
glApplyTextureEXT = _link_function('glApplyTextureEXT', None, [GLenum], 'EXT_light_texture')

# GL/glext.h:5935
glTextureLightEXT = _link_function('glTextureLightEXT', None, [GLenum], 'EXT_light_texture')

# GL/glext.h:5936
glTextureMaterialEXT = _link_function('glTextureMaterialEXT', None, [GLenum, GLenum], 'EXT_light_texture')

PFNGLAPPLYTEXTUREEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:5938
PFNGLTEXTURELIGHTEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:5939
PFNGLTEXTUREMATERIALEXTPROC = CFUNCTYPE(None, GLenum, GLenum) 	# GL/glext.h:5940
# SGIX_blend_alpha_minmax (GL/glext.h:5943)
GL_SGIX_blend_alpha_minmax = 1 	# GL/glext.h:5944
# EXT_bgra (GL/glext.h:5947)
GL_EXT_bgra = 1 	# GL/glext.h:5948
# SGIX_async (GL/glext.h:5951)
GL_SGIX_async = 1 	# GL/glext.h:5952
# GL/glext.h:5954
glAsyncMarkerSGIX = _link_function('glAsyncMarkerSGIX', None, [GLuint], 'SGIX_async')

# GL/glext.h:5955
glFinishAsyncSGIX = _link_function('glFinishAsyncSGIX', GLint, [POINTER(GLuint)], 'SGIX_async')

# GL/glext.h:5956
glPollAsyncSGIX = _link_function('glPollAsyncSGIX', GLint, [POINTER(GLuint)], 'SGIX_async')

# GL/glext.h:5957
glGenAsyncMarkersSGIX = _link_function('glGenAsyncMarkersSGIX', GLuint, [GLsizei], 'SGIX_async')

# GL/glext.h:5958
glDeleteAsyncMarkersSGIX = _link_function('glDeleteAsyncMarkersSGIX', None, [GLuint, GLsizei], 'SGIX_async')

# GL/glext.h:5959
glIsAsyncMarkerSGIX = _link_function('glIsAsyncMarkerSGIX', GLboolean, [GLuint], 'SGIX_async')

PFNGLASYNCMARKERSGIXPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:5961
PFNGLFINISHASYNCSGIXPROC = CFUNCTYPE(GLint, POINTER(GLuint)) 	# GL/glext.h:5962
PFNGLPOLLASYNCSGIXPROC = CFUNCTYPE(GLint, POINTER(GLuint)) 	# GL/glext.h:5963
PFNGLGENASYNCMARKERSSGIXPROC = CFUNCTYPE(GLuint, GLsizei) 	# GL/glext.h:5964
PFNGLDELETEASYNCMARKERSSGIXPROC = CFUNCTYPE(None, GLuint, GLsizei) 	# GL/glext.h:5965
PFNGLISASYNCMARKERSGIXPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:5966
# SGIX_async_pixel (GL/glext.h:5969)
GL_SGIX_async_pixel = 1 	# GL/glext.h:5970
# SGIX_async_histogram (GL/glext.h:5973)
GL_SGIX_async_histogram = 1 	# GL/glext.h:5974
# INTEL_parallel_arrays (GL/glext.h:5977)
GL_INTEL_parallel_arrays = 1 	# GL/glext.h:5978
# GL/glext.h:5980
glVertexPointervINTEL = _link_function('glVertexPointervINTEL', None, [GLint, GLenum, POINTER(POINTER(GLvoid))], 'INTEL_parallel_arrays')

# GL/glext.h:5981
glNormalPointervINTEL = _link_function('glNormalPointervINTEL', None, [GLenum, POINTER(POINTER(GLvoid))], 'INTEL_parallel_arrays')

# GL/glext.h:5982
glColorPointervINTEL = _link_function('glColorPointervINTEL', None, [GLint, GLenum, POINTER(POINTER(GLvoid))], 'INTEL_parallel_arrays')

# GL/glext.h:5983
glTexCoordPointervINTEL = _link_function('glTexCoordPointervINTEL', None, [GLint, GLenum, POINTER(POINTER(GLvoid))], 'INTEL_parallel_arrays')

PFNGLVERTEXPOINTERVINTELPROC = CFUNCTYPE(None, GLint, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:5985
PFNGLNORMALPOINTERVINTELPROC = CFUNCTYPE(None, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:5986
PFNGLCOLORPOINTERVINTELPROC = CFUNCTYPE(None, GLint, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:5987
PFNGLTEXCOORDPOINTERVINTELPROC = CFUNCTYPE(None, GLint, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:5988
# HP_occlusion_test (GL/glext.h:5991)
GL_HP_occlusion_test = 1 	# GL/glext.h:5992
# EXT_pixel_transform (GL/glext.h:5995)
GL_EXT_pixel_transform = 1 	# GL/glext.h:5996
# GL/glext.h:5998
glPixelTransformParameteriEXT = _link_function('glPixelTransformParameteriEXT', None, [GLenum, GLenum, GLint], 'EXT_pixel_transform')

# GL/glext.h:5999
glPixelTransformParameterfEXT = _link_function('glPixelTransformParameterfEXT', None, [GLenum, GLenum, GLfloat], 'EXT_pixel_transform')

# GL/glext.h:6000
glPixelTransformParameterivEXT = _link_function('glPixelTransformParameterivEXT', None, [GLenum, GLenum, POINTER(GLint)], 'EXT_pixel_transform')

# GL/glext.h:6001
glPixelTransformParameterfvEXT = _link_function('glPixelTransformParameterfvEXT', None, [GLenum, GLenum, POINTER(GLfloat)], 'EXT_pixel_transform')

PFNGLPIXELTRANSFORMPARAMETERIEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint) 	# GL/glext.h:6003
PFNGLPIXELTRANSFORMPARAMETERFEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLfloat) 	# GL/glext.h:6004
PFNGLPIXELTRANSFORMPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:6005
PFNGLPIXELTRANSFORMPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6006
# EXT_pixel_transform_color_table (GL/glext.h:6009)
GL_EXT_pixel_transform_color_table = 1 	# GL/glext.h:6010
# EXT_shared_texture_palette (GL/glext.h:6013)
GL_EXT_shared_texture_palette = 1 	# GL/glext.h:6014
# EXT_separate_specular_color (GL/glext.h:6017)
GL_EXT_separate_specular_color = 1 	# GL/glext.h:6018
# EXT_secondary_color (GL/glext.h:6021)
GL_EXT_secondary_color = 1 	# GL/glext.h:6022
# GL/glext.h:6024
glSecondaryColor3bEXT = _link_function('glSecondaryColor3bEXT', None, [GLbyte, GLbyte, GLbyte], 'EXT_secondary_color')

# GL/glext.h:6025
glSecondaryColor3bvEXT = _link_function('glSecondaryColor3bvEXT', None, [POINTER(GLbyte)], 'EXT_secondary_color')

# GL/glext.h:6026
glSecondaryColor3dEXT = _link_function('glSecondaryColor3dEXT', None, [GLdouble, GLdouble, GLdouble], 'EXT_secondary_color')

# GL/glext.h:6027
glSecondaryColor3dvEXT = _link_function('glSecondaryColor3dvEXT', None, [POINTER(GLdouble)], 'EXT_secondary_color')

# GL/glext.h:6028
glSecondaryColor3fEXT = _link_function('glSecondaryColor3fEXT', None, [GLfloat, GLfloat, GLfloat], 'EXT_secondary_color')

# GL/glext.h:6029
glSecondaryColor3fvEXT = _link_function('glSecondaryColor3fvEXT', None, [POINTER(GLfloat)], 'EXT_secondary_color')

# GL/glext.h:6030
glSecondaryColor3iEXT = _link_function('glSecondaryColor3iEXT', None, [GLint, GLint, GLint], 'EXT_secondary_color')

# GL/glext.h:6031
glSecondaryColor3ivEXT = _link_function('glSecondaryColor3ivEXT', None, [POINTER(GLint)], 'EXT_secondary_color')

# GL/glext.h:6032
glSecondaryColor3sEXT = _link_function('glSecondaryColor3sEXT', None, [GLshort, GLshort, GLshort], 'EXT_secondary_color')

# GL/glext.h:6033
glSecondaryColor3svEXT = _link_function('glSecondaryColor3svEXT', None, [POINTER(GLshort)], 'EXT_secondary_color')

# GL/glext.h:6034
glSecondaryColor3ubEXT = _link_function('glSecondaryColor3ubEXT', None, [GLubyte, GLubyte, GLubyte], 'EXT_secondary_color')

# GL/glext.h:6035
glSecondaryColor3ubvEXT = _link_function('glSecondaryColor3ubvEXT', None, [POINTER(GLubyte)], 'EXT_secondary_color')

# GL/glext.h:6036
glSecondaryColor3uiEXT = _link_function('glSecondaryColor3uiEXT', None, [GLuint, GLuint, GLuint], 'EXT_secondary_color')

# GL/glext.h:6037
glSecondaryColor3uivEXT = _link_function('glSecondaryColor3uivEXT', None, [POINTER(GLuint)], 'EXT_secondary_color')

# GL/glext.h:6038
glSecondaryColor3usEXT = _link_function('glSecondaryColor3usEXT', None, [GLushort, GLushort, GLushort], 'EXT_secondary_color')

# GL/glext.h:6039
glSecondaryColor3usvEXT = _link_function('glSecondaryColor3usvEXT', None, [POINTER(GLushort)], 'EXT_secondary_color')

# GL/glext.h:6040
glSecondaryColorPointerEXT = _link_function('glSecondaryColorPointerEXT', None, [GLint, GLenum, GLsizei, POINTER(GLvoid)], 'EXT_secondary_color')

PFNGLSECONDARYCOLOR3BEXTPROC = CFUNCTYPE(None, GLbyte, GLbyte, GLbyte) 	# GL/glext.h:6042
PFNGLSECONDARYCOLOR3BVEXTPROC = CFUNCTYPE(None, POINTER(GLbyte)) 	# GL/glext.h:6043
PFNGLSECONDARYCOLOR3DEXTPROC = CFUNCTYPE(None, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:6044
PFNGLSECONDARYCOLOR3DVEXTPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:6045
PFNGLSECONDARYCOLOR3FEXTPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6046
PFNGLSECONDARYCOLOR3FVEXTPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:6047
PFNGLSECONDARYCOLOR3IEXTPROC = CFUNCTYPE(None, GLint, GLint, GLint) 	# GL/glext.h:6048
PFNGLSECONDARYCOLOR3IVEXTPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:6049
PFNGLSECONDARYCOLOR3SEXTPROC = CFUNCTYPE(None, GLshort, GLshort, GLshort) 	# GL/glext.h:6050
PFNGLSECONDARYCOLOR3SVEXTPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:6051
PFNGLSECONDARYCOLOR3UBEXTPROC = CFUNCTYPE(None, GLubyte, GLubyte, GLubyte) 	# GL/glext.h:6052
PFNGLSECONDARYCOLOR3UBVEXTPROC = CFUNCTYPE(None, POINTER(GLubyte)) 	# GL/glext.h:6053
PFNGLSECONDARYCOLOR3UIEXTPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint) 	# GL/glext.h:6054
PFNGLSECONDARYCOLOR3UIVEXTPROC = CFUNCTYPE(None, POINTER(GLuint)) 	# GL/glext.h:6055
PFNGLSECONDARYCOLOR3USEXTPROC = CFUNCTYPE(None, GLushort, GLushort, GLushort) 	# GL/glext.h:6056
PFNGLSECONDARYCOLOR3USVEXTPROC = CFUNCTYPE(None, POINTER(GLushort)) 	# GL/glext.h:6057
PFNGLSECONDARYCOLORPOINTEREXTPROC = CFUNCTYPE(None, GLint, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:6058
# EXT_texture_perturb_normal (GL/glext.h:6061)
GL_EXT_texture_perturb_normal = 1 	# GL/glext.h:6062
# GL/glext.h:6064
glTextureNormalEXT = _link_function('glTextureNormalEXT', None, [GLenum], 'EXT_texture_perturb_normal')

PFNGLTEXTURENORMALEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:6066
# EXT_multi_draw_arrays (GL/glext.h:6069)
GL_EXT_multi_draw_arrays = 1 	# GL/glext.h:6070
# GL/glext.h:6072
glMultiDrawArraysEXT = _link_function('glMultiDrawArraysEXT', None, [GLenum, POINTER(GLint), POINTER(GLsizei), GLsizei], 'EXT_multi_draw_arrays')

# GL/glext.h:6073
glMultiDrawElementsEXT = _link_function('glMultiDrawElementsEXT', None, [GLenum, POINTER(GLsizei), GLenum, POINTER(POINTER(GLvoid)), GLsizei], 'EXT_multi_draw_arrays')

PFNGLMULTIDRAWARRAYSEXTPROC = CFUNCTYPE(None, GLenum, POINTER(GLint), POINTER(GLsizei), GLsizei) 	# GL/glext.h:6075
PFNGLMULTIDRAWELEMENTSEXTPROC = CFUNCTYPE(None, GLenum, POINTER(GLsizei), GLenum, POINTER(POINTER(GLvoid)), GLsizei) 	# GL/glext.h:6076
# EXT_fog_coord (GL/glext.h:6079)
GL_EXT_fog_coord = 1 	# GL/glext.h:6080
# GL/glext.h:6082
glFogCoordfEXT = _link_function('glFogCoordfEXT', None, [GLfloat], 'EXT_fog_coord')

# GL/glext.h:6083
glFogCoordfvEXT = _link_function('glFogCoordfvEXT', None, [POINTER(GLfloat)], 'EXT_fog_coord')

# GL/glext.h:6084
glFogCoorddEXT = _link_function('glFogCoorddEXT', None, [GLdouble], 'EXT_fog_coord')

# GL/glext.h:6085
glFogCoorddvEXT = _link_function('glFogCoorddvEXT', None, [POINTER(GLdouble)], 'EXT_fog_coord')

# GL/glext.h:6086
glFogCoordPointerEXT = _link_function('glFogCoordPointerEXT', None, [GLenum, GLsizei, POINTER(GLvoid)], 'EXT_fog_coord')

PFNGLFOGCOORDFEXTPROC = CFUNCTYPE(None, GLfloat) 	# GL/glext.h:6088
PFNGLFOGCOORDFVEXTPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:6089
PFNGLFOGCOORDDEXTPROC = CFUNCTYPE(None, GLdouble) 	# GL/glext.h:6090
PFNGLFOGCOORDDVEXTPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:6091
PFNGLFOGCOORDPOINTEREXTPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:6092
# REND_screen_coordinates (GL/glext.h:6095)
GL_REND_screen_coordinates = 1 	# GL/glext.h:6096
# EXT_coordinate_frame (GL/glext.h:6099)
GL_EXT_coordinate_frame = 1 	# GL/glext.h:6100
# GL/glext.h:6102
glTangent3bEXT = _link_function('glTangent3bEXT', None, [GLbyte, GLbyte, GLbyte], 'EXT_coordinate_frame')

# GL/glext.h:6103
glTangent3bvEXT = _link_function('glTangent3bvEXT', None, [POINTER(GLbyte)], 'EXT_coordinate_frame')

# GL/glext.h:6104
glTangent3dEXT = _link_function('glTangent3dEXT', None, [GLdouble, GLdouble, GLdouble], 'EXT_coordinate_frame')

# GL/glext.h:6105
glTangent3dvEXT = _link_function('glTangent3dvEXT', None, [POINTER(GLdouble)], 'EXT_coordinate_frame')

# GL/glext.h:6106
glTangent3fEXT = _link_function('glTangent3fEXT', None, [GLfloat, GLfloat, GLfloat], 'EXT_coordinate_frame')

# GL/glext.h:6107
glTangent3fvEXT = _link_function('glTangent3fvEXT', None, [POINTER(GLfloat)], 'EXT_coordinate_frame')

# GL/glext.h:6108
glTangent3iEXT = _link_function('glTangent3iEXT', None, [GLint, GLint, GLint], 'EXT_coordinate_frame')

# GL/glext.h:6109
glTangent3ivEXT = _link_function('glTangent3ivEXT', None, [POINTER(GLint)], 'EXT_coordinate_frame')

# GL/glext.h:6110
glTangent3sEXT = _link_function('glTangent3sEXT', None, [GLshort, GLshort, GLshort], 'EXT_coordinate_frame')

# GL/glext.h:6111
glTangent3svEXT = _link_function('glTangent3svEXT', None, [POINTER(GLshort)], 'EXT_coordinate_frame')

# GL/glext.h:6112
glBinormal3bEXT = _link_function('glBinormal3bEXT', None, [GLbyte, GLbyte, GLbyte], 'EXT_coordinate_frame')

# GL/glext.h:6113
glBinormal3bvEXT = _link_function('glBinormal3bvEXT', None, [POINTER(GLbyte)], 'EXT_coordinate_frame')

# GL/glext.h:6114
glBinormal3dEXT = _link_function('glBinormal3dEXT', None, [GLdouble, GLdouble, GLdouble], 'EXT_coordinate_frame')

# GL/glext.h:6115
glBinormal3dvEXT = _link_function('glBinormal3dvEXT', None, [POINTER(GLdouble)], 'EXT_coordinate_frame')

# GL/glext.h:6116
glBinormal3fEXT = _link_function('glBinormal3fEXT', None, [GLfloat, GLfloat, GLfloat], 'EXT_coordinate_frame')

# GL/glext.h:6117
glBinormal3fvEXT = _link_function('glBinormal3fvEXT', None, [POINTER(GLfloat)], 'EXT_coordinate_frame')

# GL/glext.h:6118
glBinormal3iEXT = _link_function('glBinormal3iEXT', None, [GLint, GLint, GLint], 'EXT_coordinate_frame')

# GL/glext.h:6119
glBinormal3ivEXT = _link_function('glBinormal3ivEXT', None, [POINTER(GLint)], 'EXT_coordinate_frame')

# GL/glext.h:6120
glBinormal3sEXT = _link_function('glBinormal3sEXT', None, [GLshort, GLshort, GLshort], 'EXT_coordinate_frame')

# GL/glext.h:6121
glBinormal3svEXT = _link_function('glBinormal3svEXT', None, [POINTER(GLshort)], 'EXT_coordinate_frame')

# GL/glext.h:6122
glTangentPointerEXT = _link_function('glTangentPointerEXT', None, [GLenum, GLsizei, POINTER(GLvoid)], 'EXT_coordinate_frame')

# GL/glext.h:6123
glBinormalPointerEXT = _link_function('glBinormalPointerEXT', None, [GLenum, GLsizei, POINTER(GLvoid)], 'EXT_coordinate_frame')

PFNGLTANGENT3BEXTPROC = CFUNCTYPE(None, GLbyte, GLbyte, GLbyte) 	# GL/glext.h:6125
PFNGLTANGENT3BVEXTPROC = CFUNCTYPE(None, POINTER(GLbyte)) 	# GL/glext.h:6126
PFNGLTANGENT3DEXTPROC = CFUNCTYPE(None, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:6127
PFNGLTANGENT3DVEXTPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:6128
PFNGLTANGENT3FEXTPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6129
PFNGLTANGENT3FVEXTPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:6130
PFNGLTANGENT3IEXTPROC = CFUNCTYPE(None, GLint, GLint, GLint) 	# GL/glext.h:6131
PFNGLTANGENT3IVEXTPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:6132
PFNGLTANGENT3SEXTPROC = CFUNCTYPE(None, GLshort, GLshort, GLshort) 	# GL/glext.h:6133
PFNGLTANGENT3SVEXTPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:6134
PFNGLBINORMAL3BEXTPROC = CFUNCTYPE(None, GLbyte, GLbyte, GLbyte) 	# GL/glext.h:6135
PFNGLBINORMAL3BVEXTPROC = CFUNCTYPE(None, POINTER(GLbyte)) 	# GL/glext.h:6136
PFNGLBINORMAL3DEXTPROC = CFUNCTYPE(None, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:6137
PFNGLBINORMAL3DVEXTPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:6138
PFNGLBINORMAL3FEXTPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6139
PFNGLBINORMAL3FVEXTPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:6140
PFNGLBINORMAL3IEXTPROC = CFUNCTYPE(None, GLint, GLint, GLint) 	# GL/glext.h:6141
PFNGLBINORMAL3IVEXTPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:6142
PFNGLBINORMAL3SEXTPROC = CFUNCTYPE(None, GLshort, GLshort, GLshort) 	# GL/glext.h:6143
PFNGLBINORMAL3SVEXTPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:6144
PFNGLTANGENTPOINTEREXTPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:6145
PFNGLBINORMALPOINTEREXTPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:6146
# EXT_texture_env_combine (GL/glext.h:6149)
GL_EXT_texture_env_combine = 1 	# GL/glext.h:6150
# APPLE_specular_vector (GL/glext.h:6153)
GL_APPLE_specular_vector = 1 	# GL/glext.h:6154
# APPLE_transform_hint (GL/glext.h:6157)
GL_APPLE_transform_hint = 1 	# GL/glext.h:6158
# SGIX_fog_scale (GL/glext.h:6161)
GL_SGIX_fog_scale = 1 	# GL/glext.h:6162
# SUNX_constant_data (GL/glext.h:6165)
GL_SUNX_constant_data = 1 	# GL/glext.h:6166
# GL/glext.h:6168
glFinishTextureSUNX = _link_function('glFinishTextureSUNX', None, [], 'SUNX_constant_data')

PFNGLFINISHTEXTURESUNXPROC = CFUNCTYPE(None) 	# GL/glext.h:6170
# SUN_global_alpha (GL/glext.h:6173)
GL_SUN_global_alpha = 1 	# GL/glext.h:6174
# GL/glext.h:6176
glGlobalAlphaFactorbSUN = _link_function('glGlobalAlphaFactorbSUN', None, [GLbyte], 'SUN_global_alpha')

# GL/glext.h:6177
glGlobalAlphaFactorsSUN = _link_function('glGlobalAlphaFactorsSUN', None, [GLshort], 'SUN_global_alpha')

# GL/glext.h:6178
glGlobalAlphaFactoriSUN = _link_function('glGlobalAlphaFactoriSUN', None, [GLint], 'SUN_global_alpha')

# GL/glext.h:6179
glGlobalAlphaFactorfSUN = _link_function('glGlobalAlphaFactorfSUN', None, [GLfloat], 'SUN_global_alpha')

# GL/glext.h:6180
glGlobalAlphaFactordSUN = _link_function('glGlobalAlphaFactordSUN', None, [GLdouble], 'SUN_global_alpha')

# GL/glext.h:6181
glGlobalAlphaFactorubSUN = _link_function('glGlobalAlphaFactorubSUN', None, [GLubyte], 'SUN_global_alpha')

# GL/glext.h:6182
glGlobalAlphaFactorusSUN = _link_function('glGlobalAlphaFactorusSUN', None, [GLushort], 'SUN_global_alpha')

# GL/glext.h:6183
glGlobalAlphaFactoruiSUN = _link_function('glGlobalAlphaFactoruiSUN', None, [GLuint], 'SUN_global_alpha')

PFNGLGLOBALALPHAFACTORBSUNPROC = CFUNCTYPE(None, GLbyte) 	# GL/glext.h:6185
PFNGLGLOBALALPHAFACTORSSUNPROC = CFUNCTYPE(None, GLshort) 	# GL/glext.h:6186
PFNGLGLOBALALPHAFACTORISUNPROC = CFUNCTYPE(None, GLint) 	# GL/glext.h:6187
PFNGLGLOBALALPHAFACTORFSUNPROC = CFUNCTYPE(None, GLfloat) 	# GL/glext.h:6188
PFNGLGLOBALALPHAFACTORDSUNPROC = CFUNCTYPE(None, GLdouble) 	# GL/glext.h:6189
PFNGLGLOBALALPHAFACTORUBSUNPROC = CFUNCTYPE(None, GLubyte) 	# GL/glext.h:6190
PFNGLGLOBALALPHAFACTORUSSUNPROC = CFUNCTYPE(None, GLushort) 	# GL/glext.h:6191
PFNGLGLOBALALPHAFACTORUISUNPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6192
# SUN_triangle_list (GL/glext.h:6195)
GL_SUN_triangle_list = 1 	# GL/glext.h:6196
# GL/glext.h:6198
glReplacementCodeuiSUN = _link_function('glReplacementCodeuiSUN', None, [GLuint], 'SUN_triangle_list')

# GL/glext.h:6199
glReplacementCodeusSUN = _link_function('glReplacementCodeusSUN', None, [GLushort], 'SUN_triangle_list')

# GL/glext.h:6200
glReplacementCodeubSUN = _link_function('glReplacementCodeubSUN', None, [GLubyte], 'SUN_triangle_list')

# GL/glext.h:6201
glReplacementCodeuivSUN = _link_function('glReplacementCodeuivSUN', None, [POINTER(GLuint)], 'SUN_triangle_list')

# GL/glext.h:6202
glReplacementCodeusvSUN = _link_function('glReplacementCodeusvSUN', None, [POINTER(GLushort)], 'SUN_triangle_list')

# GL/glext.h:6203
glReplacementCodeubvSUN = _link_function('glReplacementCodeubvSUN', None, [POINTER(GLubyte)], 'SUN_triangle_list')

# GL/glext.h:6204
glReplacementCodePointerSUN = _link_function('glReplacementCodePointerSUN', None, [GLenum, GLsizei, POINTER(POINTER(GLvoid))], 'SUN_triangle_list')

PFNGLREPLACEMENTCODEUISUNPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6206
PFNGLREPLACEMENTCODEUSSUNPROC = CFUNCTYPE(None, GLushort) 	# GL/glext.h:6207
PFNGLREPLACEMENTCODEUBSUNPROC = CFUNCTYPE(None, GLubyte) 	# GL/glext.h:6208
PFNGLREPLACEMENTCODEUIVSUNPROC = CFUNCTYPE(None, POINTER(GLuint)) 	# GL/glext.h:6209
PFNGLREPLACEMENTCODEUSVSUNPROC = CFUNCTYPE(None, POINTER(GLushort)) 	# GL/glext.h:6210
PFNGLREPLACEMENTCODEUBVSUNPROC = CFUNCTYPE(None, POINTER(GLubyte)) 	# GL/glext.h:6211
PFNGLREPLACEMENTCODEPOINTERSUNPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(POINTER(GLvoid))) 	# GL/glext.h:6212
# SUN_vertex (GL/glext.h:6215)
GL_SUN_vertex = 1 	# GL/glext.h:6216
# GL/glext.h:6218
glColor4ubVertex2fSUN = _link_function('glColor4ubVertex2fSUN', None, [GLubyte, GLubyte, GLubyte, GLubyte, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:6219
glColor4ubVertex2fvSUN = _link_function('glColor4ubVertex2fvSUN', None, [POINTER(GLubyte), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:6220
glColor4ubVertex3fSUN = _link_function('glColor4ubVertex3fSUN', None, [GLubyte, GLubyte, GLubyte, GLubyte, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:6221
glColor4ubVertex3fvSUN = _link_function('glColor4ubVertex3fvSUN', None, [POINTER(GLubyte), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:6222
glColor3fVertex3fSUN = _link_function('glColor3fVertex3fSUN', None, [GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:6223
glColor3fVertex3fvSUN = _link_function('glColor3fVertex3fvSUN', None, [POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:6224
glNormal3fVertex3fSUN = _link_function('glNormal3fVertex3fSUN', None, [GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:6225
glNormal3fVertex3fvSUN = _link_function('glNormal3fVertex3fvSUN', None, [POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:6226
glColor4fNormal3fVertex3fSUN = _link_function('glColor4fNormal3fVertex3fSUN', None, [GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:6227
glColor4fNormal3fVertex3fvSUN = _link_function('glColor4fNormal3fVertex3fvSUN', None, [POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:6228
glTexCoord2fVertex3fSUN = _link_function('glTexCoord2fVertex3fSUN', None, [GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:6229
glTexCoord2fVertex3fvSUN = _link_function('glTexCoord2fVertex3fvSUN', None, [POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:6230
glTexCoord4fVertex4fSUN = _link_function('glTexCoord4fVertex4fSUN', None, [GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:6231
glTexCoord4fVertex4fvSUN = _link_function('glTexCoord4fVertex4fvSUN', None, [POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:6232
glTexCoord2fColor4ubVertex3fSUN = _link_function('glTexCoord2fColor4ubVertex3fSUN', None, [GLfloat, GLfloat, GLubyte, GLubyte, GLubyte, GLubyte, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:6233
glTexCoord2fColor4ubVertex3fvSUN = _link_function('glTexCoord2fColor4ubVertex3fvSUN', None, [POINTER(GLfloat), POINTER(GLubyte), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:6234
glTexCoord2fColor3fVertex3fSUN = _link_function('glTexCoord2fColor3fVertex3fSUN', None, [GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:6235
glTexCoord2fColor3fVertex3fvSUN = _link_function('glTexCoord2fColor3fVertex3fvSUN', None, [POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:6236
glTexCoord2fNormal3fVertex3fSUN = _link_function('glTexCoord2fNormal3fVertex3fSUN', None, [GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:6237
glTexCoord2fNormal3fVertex3fvSUN = _link_function('glTexCoord2fNormal3fVertex3fvSUN', None, [POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:6238
glTexCoord2fColor4fNormal3fVertex3fSUN = _link_function('glTexCoord2fColor4fNormal3fVertex3fSUN', None, [GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:6239
glTexCoord2fColor4fNormal3fVertex3fvSUN = _link_function('glTexCoord2fColor4fNormal3fVertex3fvSUN', None, [POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:6240
glTexCoord4fColor4fNormal3fVertex4fSUN = _link_function('glTexCoord4fColor4fNormal3fVertex4fSUN', None, [GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:6241
glTexCoord4fColor4fNormal3fVertex4fvSUN = _link_function('glTexCoord4fColor4fNormal3fVertex4fvSUN', None, [POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:6242
glReplacementCodeuiVertex3fSUN = _link_function('glReplacementCodeuiVertex3fSUN', None, [GLuint, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:6243
glReplacementCodeuiVertex3fvSUN = _link_function('glReplacementCodeuiVertex3fvSUN', None, [POINTER(GLuint), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:6244
glReplacementCodeuiColor4ubVertex3fSUN = _link_function('glReplacementCodeuiColor4ubVertex3fSUN', None, [GLuint, GLubyte, GLubyte, GLubyte, GLubyte, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:6245
glReplacementCodeuiColor4ubVertex3fvSUN = _link_function('glReplacementCodeuiColor4ubVertex3fvSUN', None, [POINTER(GLuint), POINTER(GLubyte), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:6246
glReplacementCodeuiColor3fVertex3fSUN = _link_function('glReplacementCodeuiColor3fVertex3fSUN', None, [GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:6247
glReplacementCodeuiColor3fVertex3fvSUN = _link_function('glReplacementCodeuiColor3fVertex3fvSUN', None, [POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:6248
glReplacementCodeuiNormal3fVertex3fSUN = _link_function('glReplacementCodeuiNormal3fVertex3fSUN', None, [GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:6249
glReplacementCodeuiNormal3fVertex3fvSUN = _link_function('glReplacementCodeuiNormal3fVertex3fvSUN', None, [POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:6250
glReplacementCodeuiColor4fNormal3fVertex3fSUN = _link_function('glReplacementCodeuiColor4fNormal3fVertex3fSUN', None, [GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:6251
glReplacementCodeuiColor4fNormal3fVertex3fvSUN = _link_function('glReplacementCodeuiColor4fNormal3fVertex3fvSUN', None, [POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:6252
glReplacementCodeuiTexCoord2fVertex3fSUN = _link_function('glReplacementCodeuiTexCoord2fVertex3fSUN', None, [GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:6253
glReplacementCodeuiTexCoord2fVertex3fvSUN = _link_function('glReplacementCodeuiTexCoord2fVertex3fvSUN', None, [POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:6254
glReplacementCodeuiTexCoord2fNormal3fVertex3fSUN = _link_function('glReplacementCodeuiTexCoord2fNormal3fVertex3fSUN', None, [GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:6255
glReplacementCodeuiTexCoord2fNormal3fVertex3fvSUN = _link_function('glReplacementCodeuiTexCoord2fNormal3fVertex3fvSUN', None, [POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:6256
glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fSUN = _link_function('glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fSUN', None, [GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:6257
glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fvSUN = _link_function('glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fvSUN', None, [POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

PFNGLCOLOR4UBVERTEX2FSUNPROC = CFUNCTYPE(None, GLubyte, GLubyte, GLubyte, GLubyte, GLfloat, GLfloat) 	# GL/glext.h:6259
PFNGLCOLOR4UBVERTEX2FVSUNPROC = CFUNCTYPE(None, POINTER(GLubyte), POINTER(GLfloat)) 	# GL/glext.h:6260
PFNGLCOLOR4UBVERTEX3FSUNPROC = CFUNCTYPE(None, GLubyte, GLubyte, GLubyte, GLubyte, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6261
PFNGLCOLOR4UBVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLubyte), POINTER(GLfloat)) 	# GL/glext.h:6262
PFNGLCOLOR3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6263
PFNGLCOLOR3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:6264
PFNGLNORMAL3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6265
PFNGLNORMAL3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:6266
PFNGLCOLOR4FNORMAL3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6267
PFNGLCOLOR4FNORMAL3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:6268
PFNGLTEXCOORD2FVERTEX3FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6269
PFNGLTEXCOORD2FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:6270
PFNGLTEXCOORD4FVERTEX4FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6271
PFNGLTEXCOORD4FVERTEX4FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:6272
PFNGLTEXCOORD2FCOLOR4UBVERTEX3FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLubyte, GLubyte, GLubyte, GLubyte, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6273
PFNGLTEXCOORD2FCOLOR4UBVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLubyte), POINTER(GLfloat)) 	# GL/glext.h:6274
PFNGLTEXCOORD2FCOLOR3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6275
PFNGLTEXCOORD2FCOLOR3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:6276
PFNGLTEXCOORD2FNORMAL3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6277
PFNGLTEXCOORD2FNORMAL3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:6278
PFNGLTEXCOORD2FCOLOR4FNORMAL3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6279
PFNGLTEXCOORD2FCOLOR4FNORMAL3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:6280
PFNGLTEXCOORD4FCOLOR4FNORMAL3FVERTEX4FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6281
PFNGLTEXCOORD4FCOLOR4FNORMAL3FVERTEX4FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:6282
PFNGLREPLACEMENTCODEUIVERTEX3FSUNPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6283
PFNGLREPLACEMENTCODEUIVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLuint), POINTER(GLfloat)) 	# GL/glext.h:6284
PFNGLREPLACEMENTCODEUICOLOR4UBVERTEX3FSUNPROC = CFUNCTYPE(None, GLuint, GLubyte, GLubyte, GLubyte, GLubyte, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6285
PFNGLREPLACEMENTCODEUICOLOR4UBVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLuint), POINTER(GLubyte), POINTER(GLfloat)) 	# GL/glext.h:6286
PFNGLREPLACEMENTCODEUICOLOR3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6287
PFNGLREPLACEMENTCODEUICOLOR3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:6288
PFNGLREPLACEMENTCODEUINORMAL3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6289
PFNGLREPLACEMENTCODEUINORMAL3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:6290
PFNGLREPLACEMENTCODEUICOLOR4FNORMAL3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6291
PFNGLREPLACEMENTCODEUICOLOR4FNORMAL3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:6292
PFNGLREPLACEMENTCODEUITEXCOORD2FVERTEX3FSUNPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6293
PFNGLREPLACEMENTCODEUITEXCOORD2FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:6294
PFNGLREPLACEMENTCODEUITEXCOORD2FNORMAL3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6295
PFNGLREPLACEMENTCODEUITEXCOORD2FNORMAL3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:6296
PFNGLREPLACEMENTCODEUITEXCOORD2FCOLOR4FNORMAL3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6297
PFNGLREPLACEMENTCODEUITEXCOORD2FCOLOR4FNORMAL3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:6298
# EXT_blend_func_separate (GL/glext.h:6301)
GL_EXT_blend_func_separate = 1 	# GL/glext.h:6302
# GL/glext.h:6304
glBlendFuncSeparateEXT = _link_function('glBlendFuncSeparateEXT', None, [GLenum, GLenum, GLenum, GLenum], 'EXT_blend_func_separate')

PFNGLBLENDFUNCSEPARATEEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:6306
# INGR_blend_func_separate (GL/glext.h:6309)
GL_INGR_blend_func_separate = 1 	# GL/glext.h:6310
# GL/glext.h:6312
glBlendFuncSeparateINGR = _link_function('glBlendFuncSeparateINGR', None, [GLenum, GLenum, GLenum, GLenum], 'INGR_blend_func_separate')

PFNGLBLENDFUNCSEPARATEINGRPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:6314
# INGR_color_clamp (GL/glext.h:6317)
GL_INGR_color_clamp = 1 	# GL/glext.h:6318
# INGR_interlace_read (GL/glext.h:6321)
GL_INGR_interlace_read = 1 	# GL/glext.h:6322
# EXT_stencil_wrap (GL/glext.h:6325)
GL_EXT_stencil_wrap = 1 	# GL/glext.h:6326
# EXT_422_pixels (GL/glext.h:6329)
GL_EXT_422_pixels = 1 	# GL/glext.h:6330
# NV_texgen_reflection (GL/glext.h:6333)
GL_NV_texgen_reflection = 1 	# GL/glext.h:6334
# SUN_convolution_border_modes (GL/glext.h:6337)
GL_SUN_convolution_border_modes = 1 	# GL/glext.h:6338
# EXT_texture_env_add (GL/glext.h:6341)
GL_EXT_texture_env_add = 1 	# GL/glext.h:6342
# EXT_texture_lod_bias (GL/glext.h:6345)
GL_EXT_texture_lod_bias = 1 	# GL/glext.h:6346
# EXT_texture_filter_anisotropic (GL/glext.h:6349)
GL_EXT_texture_filter_anisotropic = 1 	# GL/glext.h:6350
# EXT_vertex_weighting (GL/glext.h:6353)
GL_EXT_vertex_weighting = 1 	# GL/glext.h:6354
# GL/glext.h:6356
glVertexWeightfEXT = _link_function('glVertexWeightfEXT', None, [GLfloat], 'EXT_vertex_weighting')

# GL/glext.h:6357
glVertexWeightfvEXT = _link_function('glVertexWeightfvEXT', None, [POINTER(GLfloat)], 'EXT_vertex_weighting')

# GL/glext.h:6358
glVertexWeightPointerEXT = _link_function('glVertexWeightPointerEXT', None, [GLsizei, GLenum, GLsizei, POINTER(GLvoid)], 'EXT_vertex_weighting')

PFNGLVERTEXWEIGHTFEXTPROC = CFUNCTYPE(None, GLfloat) 	# GL/glext.h:6360
PFNGLVERTEXWEIGHTFVEXTPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:6361
PFNGLVERTEXWEIGHTPOINTEREXTPROC = CFUNCTYPE(None, GLsizei, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:6362
# NV_light_max_exponent (GL/glext.h:6365)
GL_NV_light_max_exponent = 1 	# GL/glext.h:6366
# NV_vertex_array_range (GL/glext.h:6369)
GL_NV_vertex_array_range = 1 	# GL/glext.h:6370
# GL/glext.h:6372
glFlushVertexArrayRangeNV = _link_function('glFlushVertexArrayRangeNV', None, [], 'NV_vertex_array_range')

# GL/glext.h:6373
glVertexArrayRangeNV = _link_function('glVertexArrayRangeNV', None, [GLsizei, POINTER(GLvoid)], 'NV_vertex_array_range')

PFNGLFLUSHVERTEXARRAYRANGENVPROC = CFUNCTYPE(None) 	# GL/glext.h:6375
PFNGLVERTEXARRAYRANGENVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:6376
# NV_register_combiners (GL/glext.h:6379)
GL_NV_register_combiners = 1 	# GL/glext.h:6380
# GL/glext.h:6382
glCombinerParameterfvNV = _link_function('glCombinerParameterfvNV', None, [GLenum, POINTER(GLfloat)], 'NV_register_combiners')

# GL/glext.h:6383
glCombinerParameterfNV = _link_function('glCombinerParameterfNV', None, [GLenum, GLfloat], 'NV_register_combiners')

# GL/glext.h:6384
glCombinerParameterivNV = _link_function('glCombinerParameterivNV', None, [GLenum, POINTER(GLint)], 'NV_register_combiners')

# GL/glext.h:6385
glCombinerParameteriNV = _link_function('glCombinerParameteriNV', None, [GLenum, GLint], 'NV_register_combiners')

# GL/glext.h:6386
glCombinerInputNV = _link_function('glCombinerInputNV', None, [GLenum, GLenum, GLenum, GLenum, GLenum, GLenum], 'NV_register_combiners')

# GL/glext.h:6387
glCombinerOutputNV = _link_function('glCombinerOutputNV', None, [GLenum, GLenum, GLenum, GLenum, GLenum, GLenum, GLenum, GLboolean, GLboolean, GLboolean], 'NV_register_combiners')

# GL/glext.h:6388
glFinalCombinerInputNV = _link_function('glFinalCombinerInputNV', None, [GLenum, GLenum, GLenum, GLenum], 'NV_register_combiners')

# GL/glext.h:6389
glGetCombinerInputParameterfvNV = _link_function('glGetCombinerInputParameterfvNV', None, [GLenum, GLenum, GLenum, GLenum, POINTER(GLfloat)], 'NV_register_combiners')

# GL/glext.h:6390
glGetCombinerInputParameterivNV = _link_function('glGetCombinerInputParameterivNV', None, [GLenum, GLenum, GLenum, GLenum, POINTER(GLint)], 'NV_register_combiners')

# GL/glext.h:6391
glGetCombinerOutputParameterfvNV = _link_function('glGetCombinerOutputParameterfvNV', None, [GLenum, GLenum, GLenum, POINTER(GLfloat)], 'NV_register_combiners')

# GL/glext.h:6392
glGetCombinerOutputParameterivNV = _link_function('glGetCombinerOutputParameterivNV', None, [GLenum, GLenum, GLenum, POINTER(GLint)], 'NV_register_combiners')

# GL/glext.h:6393
glGetFinalCombinerInputParameterfvNV = _link_function('glGetFinalCombinerInputParameterfvNV', None, [GLenum, GLenum, POINTER(GLfloat)], 'NV_register_combiners')

# GL/glext.h:6394
glGetFinalCombinerInputParameterivNV = _link_function('glGetFinalCombinerInputParameterivNV', None, [GLenum, GLenum, POINTER(GLint)], 'NV_register_combiners')

PFNGLCOMBINERPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6396
PFNGLCOMBINERPARAMETERFNVPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:6397
PFNGLCOMBINERPARAMETERIVNVPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:6398
PFNGLCOMBINERPARAMETERINVPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:6399
PFNGLCOMBINERINPUTNVPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:6400
PFNGLCOMBINEROUTPUTNVPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum, GLenum, GLenum, GLenum, GLboolean, GLboolean, GLboolean) 	# GL/glext.h:6401
PFNGLFINALCOMBINERINPUTNVPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:6402
PFNGLGETCOMBINERINPUTPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6403
PFNGLGETCOMBINERINPUTPARAMETERIVNVPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:6404
PFNGLGETCOMBINEROUTPUTPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6405
PFNGLGETCOMBINEROUTPUTPARAMETERIVNVPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:6406
PFNGLGETFINALCOMBINERINPUTPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6407
PFNGLGETFINALCOMBINERINPUTPARAMETERIVNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:6408
# NV_fog_distance (GL/glext.h:6411)
GL_NV_fog_distance = 1 	# GL/glext.h:6412
# NV_texgen_emboss (GL/glext.h:6415)
GL_NV_texgen_emboss = 1 	# GL/glext.h:6416
# NV_blend_square (GL/glext.h:6419)
GL_NV_blend_square = 1 	# GL/glext.h:6420
# NV_texture_env_combine4 (GL/glext.h:6423)
GL_NV_texture_env_combine4 = 1 	# GL/glext.h:6424
# MESA_resize_buffers (GL/glext.h:6427)
GL_MESA_resize_buffers = 1 	# GL/glext.h:6428
# GL/glext.h:6430
glResizeBuffersMESA = _link_function('glResizeBuffersMESA', None, [], 'MESA_resize_buffers')

PFNGLRESIZEBUFFERSMESAPROC = CFUNCTYPE(None) 	# GL/glext.h:6432
# MESA_window_pos (GL/glext.h:6435)
GL_MESA_window_pos = 1 	# GL/glext.h:6436
# GL/glext.h:6438
glWindowPos2dMESA = _link_function('glWindowPos2dMESA', None, [GLdouble, GLdouble], 'MESA_window_pos')

# GL/glext.h:6439
glWindowPos2dvMESA = _link_function('glWindowPos2dvMESA', None, [POINTER(GLdouble)], 'MESA_window_pos')

# GL/glext.h:6440
glWindowPos2fMESA = _link_function('glWindowPos2fMESA', None, [GLfloat, GLfloat], 'MESA_window_pos')

# GL/glext.h:6441
glWindowPos2fvMESA = _link_function('glWindowPos2fvMESA', None, [POINTER(GLfloat)], 'MESA_window_pos')

# GL/glext.h:6442
glWindowPos2iMESA = _link_function('glWindowPos2iMESA', None, [GLint, GLint], 'MESA_window_pos')

# GL/glext.h:6443
glWindowPos2ivMESA = _link_function('glWindowPos2ivMESA', None, [POINTER(GLint)], 'MESA_window_pos')

# GL/glext.h:6444
glWindowPos2sMESA = _link_function('glWindowPos2sMESA', None, [GLshort, GLshort], 'MESA_window_pos')

# GL/glext.h:6445
glWindowPos2svMESA = _link_function('glWindowPos2svMESA', None, [POINTER(GLshort)], 'MESA_window_pos')

# GL/glext.h:6446
glWindowPos3dMESA = _link_function('glWindowPos3dMESA', None, [GLdouble, GLdouble, GLdouble], 'MESA_window_pos')

# GL/glext.h:6447
glWindowPos3dvMESA = _link_function('glWindowPos3dvMESA', None, [POINTER(GLdouble)], 'MESA_window_pos')

# GL/glext.h:6448
glWindowPos3fMESA = _link_function('glWindowPos3fMESA', None, [GLfloat, GLfloat, GLfloat], 'MESA_window_pos')

# GL/glext.h:6449
glWindowPos3fvMESA = _link_function('glWindowPos3fvMESA', None, [POINTER(GLfloat)], 'MESA_window_pos')

# GL/glext.h:6450
glWindowPos3iMESA = _link_function('glWindowPos3iMESA', None, [GLint, GLint, GLint], 'MESA_window_pos')

# GL/glext.h:6451
glWindowPos3ivMESA = _link_function('glWindowPos3ivMESA', None, [POINTER(GLint)], 'MESA_window_pos')

# GL/glext.h:6452
glWindowPos3sMESA = _link_function('glWindowPos3sMESA', None, [GLshort, GLshort, GLshort], 'MESA_window_pos')

# GL/glext.h:6453
glWindowPos3svMESA = _link_function('glWindowPos3svMESA', None, [POINTER(GLshort)], 'MESA_window_pos')

# GL/glext.h:6454
glWindowPos4dMESA = _link_function('glWindowPos4dMESA', None, [GLdouble, GLdouble, GLdouble, GLdouble], 'MESA_window_pos')

# GL/glext.h:6455
glWindowPos4dvMESA = _link_function('glWindowPos4dvMESA', None, [POINTER(GLdouble)], 'MESA_window_pos')

# GL/glext.h:6456
glWindowPos4fMESA = _link_function('glWindowPos4fMESA', None, [GLfloat, GLfloat, GLfloat, GLfloat], 'MESA_window_pos')

# GL/glext.h:6457
glWindowPos4fvMESA = _link_function('glWindowPos4fvMESA', None, [POINTER(GLfloat)], 'MESA_window_pos')

# GL/glext.h:6458
glWindowPos4iMESA = _link_function('glWindowPos4iMESA', None, [GLint, GLint, GLint, GLint], 'MESA_window_pos')

# GL/glext.h:6459
glWindowPos4ivMESA = _link_function('glWindowPos4ivMESA', None, [POINTER(GLint)], 'MESA_window_pos')

# GL/glext.h:6460
glWindowPos4sMESA = _link_function('glWindowPos4sMESA', None, [GLshort, GLshort, GLshort, GLshort], 'MESA_window_pos')

# GL/glext.h:6461
glWindowPos4svMESA = _link_function('glWindowPos4svMESA', None, [POINTER(GLshort)], 'MESA_window_pos')

PFNGLWINDOWPOS2DMESAPROC = CFUNCTYPE(None, GLdouble, GLdouble) 	# GL/glext.h:6463
PFNGLWINDOWPOS2DVMESAPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:6464
PFNGLWINDOWPOS2FMESAPROC = CFUNCTYPE(None, GLfloat, GLfloat) 	# GL/glext.h:6465
PFNGLWINDOWPOS2FVMESAPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:6466
PFNGLWINDOWPOS2IMESAPROC = CFUNCTYPE(None, GLint, GLint) 	# GL/glext.h:6467
PFNGLWINDOWPOS2IVMESAPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:6468
PFNGLWINDOWPOS2SMESAPROC = CFUNCTYPE(None, GLshort, GLshort) 	# GL/glext.h:6469
PFNGLWINDOWPOS2SVMESAPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:6470
PFNGLWINDOWPOS3DMESAPROC = CFUNCTYPE(None, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:6471
PFNGLWINDOWPOS3DVMESAPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:6472
PFNGLWINDOWPOS3FMESAPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6473
PFNGLWINDOWPOS3FVMESAPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:6474
PFNGLWINDOWPOS3IMESAPROC = CFUNCTYPE(None, GLint, GLint, GLint) 	# GL/glext.h:6475
PFNGLWINDOWPOS3IVMESAPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:6476
PFNGLWINDOWPOS3SMESAPROC = CFUNCTYPE(None, GLshort, GLshort, GLshort) 	# GL/glext.h:6477
PFNGLWINDOWPOS3SVMESAPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:6478
PFNGLWINDOWPOS4DMESAPROC = CFUNCTYPE(None, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:6479
PFNGLWINDOWPOS4DVMESAPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:6480
PFNGLWINDOWPOS4FMESAPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6481
PFNGLWINDOWPOS4FVMESAPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:6482
PFNGLWINDOWPOS4IMESAPROC = CFUNCTYPE(None, GLint, GLint, GLint, GLint) 	# GL/glext.h:6483
PFNGLWINDOWPOS4IVMESAPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:6484
PFNGLWINDOWPOS4SMESAPROC = CFUNCTYPE(None, GLshort, GLshort, GLshort, GLshort) 	# GL/glext.h:6485
PFNGLWINDOWPOS4SVMESAPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:6486
# EXT_texture_compression_s3tc (GL/glext.h:6489)
GL_EXT_texture_compression_s3tc = 1 	# GL/glext.h:6490
# IBM_cull_vertex (GL/glext.h:6493)
GL_IBM_cull_vertex = 1 	# GL/glext.h:6494
# IBM_multimode_draw_arrays (GL/glext.h:6497)
GL_IBM_multimode_draw_arrays = 1 	# GL/glext.h:6498
# GL/glext.h:6500
glMultiModeDrawArraysIBM = _link_function('glMultiModeDrawArraysIBM', None, [POINTER(GLenum), POINTER(GLint), POINTER(GLsizei), GLsizei, GLint], 'IBM_multimode_draw_arrays')

# GL/glext.h:6501
glMultiModeDrawElementsIBM = _link_function('glMultiModeDrawElementsIBM', None, [POINTER(GLenum), POINTER(GLsizei), GLenum, POINTER(POINTER(GLvoid)), GLsizei, GLint], 'IBM_multimode_draw_arrays')

PFNGLMULTIMODEDRAWARRAYSIBMPROC = CFUNCTYPE(None, POINTER(GLenum), POINTER(GLint), POINTER(GLsizei), GLsizei, GLint) 	# GL/glext.h:6503
PFNGLMULTIMODEDRAWELEMENTSIBMPROC = CFUNCTYPE(None, POINTER(GLenum), POINTER(GLsizei), GLenum, POINTER(POINTER(GLvoid)), GLsizei, GLint) 	# GL/glext.h:6504
# IBM_vertex_array_lists (GL/glext.h:6507)
GL_IBM_vertex_array_lists = 1 	# GL/glext.h:6508
# GL/glext.h:6510
glColorPointerListIBM = _link_function('glColorPointerListIBM', None, [GLint, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint], 'IBM_vertex_array_lists')

# GL/glext.h:6511
glSecondaryColorPointerListIBM = _link_function('glSecondaryColorPointerListIBM', None, [GLint, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint], 'IBM_vertex_array_lists')

# GL/glext.h:6512
glEdgeFlagPointerListIBM = _link_function('glEdgeFlagPointerListIBM', None, [GLint, POINTER(POINTER(GLboolean)), GLint], 'IBM_vertex_array_lists')

# GL/glext.h:6513
glFogCoordPointerListIBM = _link_function('glFogCoordPointerListIBM', None, [GLenum, GLint, POINTER(POINTER(GLvoid)), GLint], 'IBM_vertex_array_lists')

# GL/glext.h:6514
glIndexPointerListIBM = _link_function('glIndexPointerListIBM', None, [GLenum, GLint, POINTER(POINTER(GLvoid)), GLint], 'IBM_vertex_array_lists')

# GL/glext.h:6515
glNormalPointerListIBM = _link_function('glNormalPointerListIBM', None, [GLenum, GLint, POINTER(POINTER(GLvoid)), GLint], 'IBM_vertex_array_lists')

# GL/glext.h:6516
glTexCoordPointerListIBM = _link_function('glTexCoordPointerListIBM', None, [GLint, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint], 'IBM_vertex_array_lists')

# GL/glext.h:6517
glVertexPointerListIBM = _link_function('glVertexPointerListIBM', None, [GLint, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint], 'IBM_vertex_array_lists')

PFNGLCOLORPOINTERLISTIBMPROC = CFUNCTYPE(None, GLint, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint) 	# GL/glext.h:6519
PFNGLSECONDARYCOLORPOINTERLISTIBMPROC = CFUNCTYPE(None, GLint, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint) 	# GL/glext.h:6520
PFNGLEDGEFLAGPOINTERLISTIBMPROC = CFUNCTYPE(None, GLint, POINTER(POINTER(GLboolean)), GLint) 	# GL/glext.h:6521
PFNGLFOGCOORDPOINTERLISTIBMPROC = CFUNCTYPE(None, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint) 	# GL/glext.h:6522
PFNGLINDEXPOINTERLISTIBMPROC = CFUNCTYPE(None, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint) 	# GL/glext.h:6523
PFNGLNORMALPOINTERLISTIBMPROC = CFUNCTYPE(None, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint) 	# GL/glext.h:6524
PFNGLTEXCOORDPOINTERLISTIBMPROC = CFUNCTYPE(None, GLint, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint) 	# GL/glext.h:6525
PFNGLVERTEXPOINTERLISTIBMPROC = CFUNCTYPE(None, GLint, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint) 	# GL/glext.h:6526
# SGIX_subsample (GL/glext.h:6529)
GL_SGIX_subsample = 1 	# GL/glext.h:6530
# SGIX_ycrcba (GL/glext.h:6533)
GL_SGIX_ycrcba = 1 	# GL/glext.h:6534
# SGIX_ycrcb_subsample (GL/glext.h:6537)
GL_SGIX_ycrcb_subsample = 1 	# GL/glext.h:6538
# SGIX_depth_pass_instrument (GL/glext.h:6541)
GL_SGIX_depth_pass_instrument = 1 	# GL/glext.h:6542
# 3DFX_texture_compression_FXT1 (GL/glext.h:6545)
GL_3DFX_texture_compression_FXT1 = 1 	# GL/glext.h:6546
# 3DFX_multisample (GL/glext.h:6549)
GL_3DFX_multisample = 1 	# GL/glext.h:6550
# 3DFX_tbuffer (GL/glext.h:6553)
GL_3DFX_tbuffer = 1 	# GL/glext.h:6554
# GL/glext.h:6556
glTbufferMask3DFX = _link_function('glTbufferMask3DFX', None, [GLuint], '3DFX_tbuffer')

PFNGLTBUFFERMASK3DFXPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6558
# EXT_multisample (GL/glext.h:6561)
GL_EXT_multisample = 1 	# GL/glext.h:6562
# GL/glext.h:6564
glSampleMaskEXT = _link_function('glSampleMaskEXT', None, [GLclampf, GLboolean], 'EXT_multisample')

# GL/glext.h:6565
glSamplePatternEXT = _link_function('glSamplePatternEXT', None, [GLenum], 'EXT_multisample')

PFNGLSAMPLEMASKEXTPROC = CFUNCTYPE(None, GLclampf, GLboolean) 	# GL/glext.h:6567
PFNGLSAMPLEPATTERNEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:6568
# SGIX_vertex_preclip (GL/glext.h:6571)
GL_SGIX_vertex_preclip = 1 	# GL/glext.h:6572
# SGIX_convolution_accuracy (GL/glext.h:6575)
GL_SGIX_convolution_accuracy = 1 	# GL/glext.h:6576
# SGIX_resample (GL/glext.h:6579)
GL_SGIX_resample = 1 	# GL/glext.h:6580
# SGIS_point_line_texgen (GL/glext.h:6583)
GL_SGIS_point_line_texgen = 1 	# GL/glext.h:6584
# SGIS_texture_color_mask (GL/glext.h:6587)
GL_SGIS_texture_color_mask = 1 	# GL/glext.h:6588
# GL/glext.h:6590
glTextureColorMaskSGIS = _link_function('glTextureColorMaskSGIS', None, [GLboolean, GLboolean, GLboolean, GLboolean], 'SGIS_texture_color_mask')

PFNGLTEXTURECOLORMASKSGISPROC = CFUNCTYPE(None, GLboolean, GLboolean, GLboolean, GLboolean) 	# GL/glext.h:6592
# SGIX_igloo_interface (GL/glext.h:6595)
GL_SGIX_igloo_interface = 1 	# GL/glext.h:6596
# GL/glext.h:6598
glIglooInterfaceSGIX = _link_function('glIglooInterfaceSGIX', None, [GLenum, POINTER(GLvoid)], 'SGIX_igloo_interface')

PFNGLIGLOOINTERFACESGIXPROC = CFUNCTYPE(None, GLenum, POINTER(GLvoid)) 	# GL/glext.h:6600
# EXT_texture_env_dot3 (GL/glext.h:6603)
GL_EXT_texture_env_dot3 = 1 	# GL/glext.h:6604
# ATI_texture_mirror_once (GL/glext.h:6607)
GL_ATI_texture_mirror_once = 1 	# GL/glext.h:6608
# NV_fence (GL/glext.h:6611)
GL_NV_fence = 1 	# GL/glext.h:6612
# GL/glext.h:6614
glDeleteFencesNV = _link_function('glDeleteFencesNV', None, [GLsizei, POINTER(GLuint)], 'NV_fence')

# GL/glext.h:6615
glGenFencesNV = _link_function('glGenFencesNV', None, [GLsizei, POINTER(GLuint)], 'NV_fence')

# GL/glext.h:6616
glIsFenceNV = _link_function('glIsFenceNV', GLboolean, [GLuint], 'NV_fence')

# GL/glext.h:6617
glTestFenceNV = _link_function('glTestFenceNV', GLboolean, [GLuint], 'NV_fence')

# GL/glext.h:6618
glGetFenceivNV = _link_function('glGetFenceivNV', None, [GLuint, GLenum, POINTER(GLint)], 'NV_fence')

# GL/glext.h:6619
glFinishFenceNV = _link_function('glFinishFenceNV', None, [GLuint], 'NV_fence')

# GL/glext.h:6620
glSetFenceNV = _link_function('glSetFenceNV', None, [GLuint, GLenum], 'NV_fence')

PFNGLDELETEFENCESNVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6622
PFNGLGENFENCESNVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6623
PFNGLISFENCENVPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:6624
PFNGLTESTFENCENVPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:6625
PFNGLGETFENCEIVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:6626
PFNGLFINISHFENCENVPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6627
PFNGLSETFENCENVPROC = CFUNCTYPE(None, GLuint, GLenum) 	# GL/glext.h:6628
# NV_evaluators (GL/glext.h:6631)
GL_NV_evaluators = 1 	# GL/glext.h:6632
# GL/glext.h:6634
glMapControlPointsNV = _link_function('glMapControlPointsNV', None, [GLenum, GLuint, GLenum, GLsizei, GLsizei, GLint, GLint, GLboolean, POINTER(GLvoid)], 'NV_evaluators')

# GL/glext.h:6635
glMapParameterivNV = _link_function('glMapParameterivNV', None, [GLenum, GLenum, POINTER(GLint)], 'NV_evaluators')

# GL/glext.h:6636
glMapParameterfvNV = _link_function('glMapParameterfvNV', None, [GLenum, GLenum, POINTER(GLfloat)], 'NV_evaluators')

# GL/glext.h:6637
glGetMapControlPointsNV = _link_function('glGetMapControlPointsNV', None, [GLenum, GLuint, GLenum, GLsizei, GLsizei, GLboolean, POINTER(GLvoid)], 'NV_evaluators')

# GL/glext.h:6638
glGetMapParameterivNV = _link_function('glGetMapParameterivNV', None, [GLenum, GLenum, POINTER(GLint)], 'NV_evaluators')

# GL/glext.h:6639
glGetMapParameterfvNV = _link_function('glGetMapParameterfvNV', None, [GLenum, GLenum, POINTER(GLfloat)], 'NV_evaluators')

# GL/glext.h:6640
glGetMapAttribParameterivNV = _link_function('glGetMapAttribParameterivNV', None, [GLenum, GLuint, GLenum, POINTER(GLint)], 'NV_evaluators')

# GL/glext.h:6641
glGetMapAttribParameterfvNV = _link_function('glGetMapAttribParameterfvNV', None, [GLenum, GLuint, GLenum, POINTER(GLfloat)], 'NV_evaluators')

# GL/glext.h:6642
glEvalMapsNV = _link_function('glEvalMapsNV', None, [GLenum, GLenum], 'NV_evaluators')

PFNGLMAPCONTROLPOINTSNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLenum, GLsizei, GLsizei, GLint, GLint, GLboolean, POINTER(GLvoid)) 	# GL/glext.h:6644
PFNGLMAPPARAMETERIVNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:6645
PFNGLMAPPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6646
PFNGLGETMAPCONTROLPOINTSNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLenum, GLsizei, GLsizei, GLboolean, POINTER(GLvoid)) 	# GL/glext.h:6647
PFNGLGETMAPPARAMETERIVNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:6648
PFNGLGETMAPPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6649
PFNGLGETMAPATTRIBPARAMETERIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:6650
PFNGLGETMAPATTRIBPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6651
PFNGLEVALMAPSNVPROC = CFUNCTYPE(None, GLenum, GLenum) 	# GL/glext.h:6652
# NV_packed_depth_stencil (GL/glext.h:6655)
GL_NV_packed_depth_stencil = 1 	# GL/glext.h:6656
# EXT_packed_depth_stencil (GL/glext.h:6659)
GL_EXT_packed_depth_stencil = 1 	# GL/glext.h:6660
# NV_register_combiners2 (GL/glext.h:6663)
GL_NV_register_combiners2 = 1 	# GL/glext.h:6664
# GL/glext.h:6666
glCombinerStageParameterfvNV = _link_function('glCombinerStageParameterfvNV', None, [GLenum, GLenum, POINTER(GLfloat)], 'NV_register_combiners2')

# GL/glext.h:6667
glGetCombinerStageParameterfvNV = _link_function('glGetCombinerStageParameterfvNV', None, [GLenum, GLenum, POINTER(GLfloat)], 'NV_register_combiners2')

PFNGLCOMBINERSTAGEPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6669
PFNGLGETCOMBINERSTAGEPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6670
# NV_texture_compression_vtc (GL/glext.h:6673)
GL_NV_texture_compression_vtc = 1 	# GL/glext.h:6674
# NV_texture_rectangle (GL/glext.h:6677)
GL_NV_texture_rectangle = 1 	# GL/glext.h:6678
# NV_texture_shader (GL/glext.h:6681)
GL_NV_texture_shader = 1 	# GL/glext.h:6682
# NV_texture_shader2 (GL/glext.h:6685)
GL_NV_texture_shader2 = 1 	# GL/glext.h:6686
# NV_vertex_array_range2 (GL/glext.h:6689)
GL_NV_vertex_array_range2 = 1 	# GL/glext.h:6690
# NV_vertex_program (GL/glext.h:6693)
GL_NV_vertex_program = 1 	# GL/glext.h:6694
# GL/glext.h:6696
glAreProgramsResidentNV = _link_function('glAreProgramsResidentNV', GLboolean, [GLsizei, POINTER(GLuint), POINTER(GLboolean)], 'NV_vertex_program')

# GL/glext.h:6697
glBindProgramNV = _link_function('glBindProgramNV', None, [GLenum, GLuint], 'NV_vertex_program')

# GL/glext.h:6698
glDeleteProgramsNV = _link_function('glDeleteProgramsNV', None, [GLsizei, POINTER(GLuint)], 'NV_vertex_program')

# GL/glext.h:6699
glExecuteProgramNV = _link_function('glExecuteProgramNV', None, [GLenum, GLuint, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:6700
glGenProgramsNV = _link_function('glGenProgramsNV', None, [GLsizei, POINTER(GLuint)], 'NV_vertex_program')

# GL/glext.h:6701
glGetProgramParameterdvNV = _link_function('glGetProgramParameterdvNV', None, [GLenum, GLuint, GLenum, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:6702
glGetProgramParameterfvNV = _link_function('glGetProgramParameterfvNV', None, [GLenum, GLuint, GLenum, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:6703
glGetProgramivNV = _link_function('glGetProgramivNV', None, [GLuint, GLenum, POINTER(GLint)], 'NV_vertex_program')

# GL/glext.h:6704
glGetProgramStringNV = _link_function('glGetProgramStringNV', None, [GLuint, GLenum, POINTER(GLubyte)], 'NV_vertex_program')

# GL/glext.h:6705
glGetTrackMatrixivNV = _link_function('glGetTrackMatrixivNV', None, [GLenum, GLuint, GLenum, POINTER(GLint)], 'NV_vertex_program')

# GL/glext.h:6706
glGetVertexAttribdvNV = _link_function('glGetVertexAttribdvNV', None, [GLuint, GLenum, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:6707
glGetVertexAttribfvNV = _link_function('glGetVertexAttribfvNV', None, [GLuint, GLenum, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:6708
glGetVertexAttribivNV = _link_function('glGetVertexAttribivNV', None, [GLuint, GLenum, POINTER(GLint)], 'NV_vertex_program')

# GL/glext.h:6709
glGetVertexAttribPointervNV = _link_function('glGetVertexAttribPointervNV', None, [GLuint, GLenum, POINTER(POINTER(GLvoid))], 'NV_vertex_program')

# GL/glext.h:6710
glIsProgramNV = _link_function('glIsProgramNV', GLboolean, [GLuint], 'NV_vertex_program')

# GL/glext.h:6711
glLoadProgramNV = _link_function('glLoadProgramNV', None, [GLenum, GLuint, GLsizei, POINTER(GLubyte)], 'NV_vertex_program')

# GL/glext.h:6712
glProgramParameter4dNV = _link_function('glProgramParameter4dNV', None, [GLenum, GLuint, GLdouble, GLdouble, GLdouble, GLdouble], 'NV_vertex_program')

# GL/glext.h:6713
glProgramParameter4dvNV = _link_function('glProgramParameter4dvNV', None, [GLenum, GLuint, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:6714
glProgramParameter4fNV = _link_function('glProgramParameter4fNV', None, [GLenum, GLuint, GLfloat, GLfloat, GLfloat, GLfloat], 'NV_vertex_program')

# GL/glext.h:6715
glProgramParameter4fvNV = _link_function('glProgramParameter4fvNV', None, [GLenum, GLuint, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:6716
glProgramParameters4dvNV = _link_function('glProgramParameters4dvNV', None, [GLenum, GLuint, GLuint, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:6717
glProgramParameters4fvNV = _link_function('glProgramParameters4fvNV', None, [GLenum, GLuint, GLuint, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:6718
glRequestResidentProgramsNV = _link_function('glRequestResidentProgramsNV', None, [GLsizei, POINTER(GLuint)], 'NV_vertex_program')

# GL/glext.h:6719
glTrackMatrixNV = _link_function('glTrackMatrixNV', None, [GLenum, GLuint, GLenum, GLenum], 'NV_vertex_program')

# GL/glext.h:6720
glVertexAttribPointerNV = _link_function('glVertexAttribPointerNV', None, [GLuint, GLint, GLenum, GLsizei, POINTER(GLvoid)], 'NV_vertex_program')

# GL/glext.h:6721
glVertexAttrib1dNV = _link_function('glVertexAttrib1dNV', None, [GLuint, GLdouble], 'NV_vertex_program')

# GL/glext.h:6722
glVertexAttrib1dvNV = _link_function('glVertexAttrib1dvNV', None, [GLuint, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:6723
glVertexAttrib1fNV = _link_function('glVertexAttrib1fNV', None, [GLuint, GLfloat], 'NV_vertex_program')

# GL/glext.h:6724
glVertexAttrib1fvNV = _link_function('glVertexAttrib1fvNV', None, [GLuint, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:6725
glVertexAttrib1sNV = _link_function('glVertexAttrib1sNV', None, [GLuint, GLshort], 'NV_vertex_program')

# GL/glext.h:6726
glVertexAttrib1svNV = _link_function('glVertexAttrib1svNV', None, [GLuint, POINTER(GLshort)], 'NV_vertex_program')

# GL/glext.h:6727
glVertexAttrib2dNV = _link_function('glVertexAttrib2dNV', None, [GLuint, GLdouble, GLdouble], 'NV_vertex_program')

# GL/glext.h:6728
glVertexAttrib2dvNV = _link_function('glVertexAttrib2dvNV', None, [GLuint, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:6729
glVertexAttrib2fNV = _link_function('glVertexAttrib2fNV', None, [GLuint, GLfloat, GLfloat], 'NV_vertex_program')

# GL/glext.h:6730
glVertexAttrib2fvNV = _link_function('glVertexAttrib2fvNV', None, [GLuint, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:6731
glVertexAttrib2sNV = _link_function('glVertexAttrib2sNV', None, [GLuint, GLshort, GLshort], 'NV_vertex_program')

# GL/glext.h:6732
glVertexAttrib2svNV = _link_function('glVertexAttrib2svNV', None, [GLuint, POINTER(GLshort)], 'NV_vertex_program')

# GL/glext.h:6733
glVertexAttrib3dNV = _link_function('glVertexAttrib3dNV', None, [GLuint, GLdouble, GLdouble, GLdouble], 'NV_vertex_program')

# GL/glext.h:6734
glVertexAttrib3dvNV = _link_function('glVertexAttrib3dvNV', None, [GLuint, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:6735
glVertexAttrib3fNV = _link_function('glVertexAttrib3fNV', None, [GLuint, GLfloat, GLfloat, GLfloat], 'NV_vertex_program')

# GL/glext.h:6736
glVertexAttrib3fvNV = _link_function('glVertexAttrib3fvNV', None, [GLuint, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:6737
glVertexAttrib3sNV = _link_function('glVertexAttrib3sNV', None, [GLuint, GLshort, GLshort, GLshort], 'NV_vertex_program')

# GL/glext.h:6738
glVertexAttrib3svNV = _link_function('glVertexAttrib3svNV', None, [GLuint, POINTER(GLshort)], 'NV_vertex_program')

# GL/glext.h:6739
glVertexAttrib4dNV = _link_function('glVertexAttrib4dNV', None, [GLuint, GLdouble, GLdouble, GLdouble, GLdouble], 'NV_vertex_program')

# GL/glext.h:6740
glVertexAttrib4dvNV = _link_function('glVertexAttrib4dvNV', None, [GLuint, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:6741
glVertexAttrib4fNV = _link_function('glVertexAttrib4fNV', None, [GLuint, GLfloat, GLfloat, GLfloat, GLfloat], 'NV_vertex_program')

# GL/glext.h:6742
glVertexAttrib4fvNV = _link_function('glVertexAttrib4fvNV', None, [GLuint, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:6743
glVertexAttrib4sNV = _link_function('glVertexAttrib4sNV', None, [GLuint, GLshort, GLshort, GLshort, GLshort], 'NV_vertex_program')

# GL/glext.h:6744
glVertexAttrib4svNV = _link_function('glVertexAttrib4svNV', None, [GLuint, POINTER(GLshort)], 'NV_vertex_program')

# GL/glext.h:6745
glVertexAttrib4ubNV = _link_function('glVertexAttrib4ubNV', None, [GLuint, GLubyte, GLubyte, GLubyte, GLubyte], 'NV_vertex_program')

# GL/glext.h:6746
glVertexAttrib4ubvNV = _link_function('glVertexAttrib4ubvNV', None, [GLuint, POINTER(GLubyte)], 'NV_vertex_program')

# GL/glext.h:6747
glVertexAttribs1dvNV = _link_function('glVertexAttribs1dvNV', None, [GLuint, GLsizei, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:6748
glVertexAttribs1fvNV = _link_function('glVertexAttribs1fvNV', None, [GLuint, GLsizei, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:6749
glVertexAttribs1svNV = _link_function('glVertexAttribs1svNV', None, [GLuint, GLsizei, POINTER(GLshort)], 'NV_vertex_program')

# GL/glext.h:6750
glVertexAttribs2dvNV = _link_function('glVertexAttribs2dvNV', None, [GLuint, GLsizei, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:6751
glVertexAttribs2fvNV = _link_function('glVertexAttribs2fvNV', None, [GLuint, GLsizei, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:6752
glVertexAttribs2svNV = _link_function('glVertexAttribs2svNV', None, [GLuint, GLsizei, POINTER(GLshort)], 'NV_vertex_program')

# GL/glext.h:6753
glVertexAttribs3dvNV = _link_function('glVertexAttribs3dvNV', None, [GLuint, GLsizei, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:6754
glVertexAttribs3fvNV = _link_function('glVertexAttribs3fvNV', None, [GLuint, GLsizei, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:6755
glVertexAttribs3svNV = _link_function('glVertexAttribs3svNV', None, [GLuint, GLsizei, POINTER(GLshort)], 'NV_vertex_program')

# GL/glext.h:6756
glVertexAttribs4dvNV = _link_function('glVertexAttribs4dvNV', None, [GLuint, GLsizei, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:6757
glVertexAttribs4fvNV = _link_function('glVertexAttribs4fvNV', None, [GLuint, GLsizei, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:6758
glVertexAttribs4svNV = _link_function('glVertexAttribs4svNV', None, [GLuint, GLsizei, POINTER(GLshort)], 'NV_vertex_program')

# GL/glext.h:6759
glVertexAttribs4ubvNV = _link_function('glVertexAttribs4ubvNV', None, [GLuint, GLsizei, POINTER(GLubyte)], 'NV_vertex_program')

PFNGLAREPROGRAMSRESIDENTNVPROC = CFUNCTYPE(GLboolean, GLsizei, POINTER(GLuint), POINTER(GLboolean)) 	# GL/glext.h:6761
PFNGLBINDPROGRAMNVPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:6762
PFNGLDELETEPROGRAMSNVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6763
PFNGLEXECUTEPROGRAMNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLfloat)) 	# GL/glext.h:6764
PFNGLGENPROGRAMSNVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6765
PFNGLGETPROGRAMPARAMETERDVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLenum, POINTER(GLdouble)) 	# GL/glext.h:6766
PFNGLGETPROGRAMPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6767
PFNGLGETPROGRAMIVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:6768
PFNGLGETPROGRAMSTRINGNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLubyte)) 	# GL/glext.h:6769
PFNGLGETTRACKMATRIXIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:6770
PFNGLGETVERTEXATTRIBDVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLdouble)) 	# GL/glext.h:6771
PFNGLGETVERTEXATTRIBFVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6772
PFNGLGETVERTEXATTRIBIVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:6773
PFNGLGETVERTEXATTRIBPOINTERVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:6774
PFNGLISPROGRAMNVPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:6775
PFNGLLOADPROGRAMNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLsizei, POINTER(GLubyte)) 	# GL/glext.h:6776
PFNGLPROGRAMPARAMETER4DNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:6777
PFNGLPROGRAMPARAMETER4DVNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLdouble)) 	# GL/glext.h:6778
PFNGLPROGRAMPARAMETER4FNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6779
PFNGLPROGRAMPARAMETER4FVNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLfloat)) 	# GL/glext.h:6780
PFNGLPROGRAMPARAMETERS4DVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, POINTER(GLdouble)) 	# GL/glext.h:6781
PFNGLPROGRAMPARAMETERS4FVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, POINTER(GLfloat)) 	# GL/glext.h:6782
PFNGLREQUESTRESIDENTPROGRAMSNVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6783
PFNGLTRACKMATRIXNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLenum, GLenum) 	# GL/glext.h:6784
PFNGLVERTEXATTRIBPOINTERNVPROC = CFUNCTYPE(None, GLuint, GLint, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:6785
PFNGLVERTEXATTRIB1DNVPROC = CFUNCTYPE(None, GLuint, GLdouble) 	# GL/glext.h:6786
PFNGLVERTEXATTRIB1DVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:6787
PFNGLVERTEXATTRIB1FNVPROC = CFUNCTYPE(None, GLuint, GLfloat) 	# GL/glext.h:6788
PFNGLVERTEXATTRIB1FVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:6789
PFNGLVERTEXATTRIB1SNVPROC = CFUNCTYPE(None, GLuint, GLshort) 	# GL/glext.h:6790
PFNGLVERTEXATTRIB1SVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:6791
PFNGLVERTEXATTRIB2DNVPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble) 	# GL/glext.h:6792
PFNGLVERTEXATTRIB2DVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:6793
PFNGLVERTEXATTRIB2FNVPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat) 	# GL/glext.h:6794
PFNGLVERTEXATTRIB2FVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:6795
PFNGLVERTEXATTRIB2SNVPROC = CFUNCTYPE(None, GLuint, GLshort, GLshort) 	# GL/glext.h:6796
PFNGLVERTEXATTRIB2SVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:6797
PFNGLVERTEXATTRIB3DNVPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:6798
PFNGLVERTEXATTRIB3DVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:6799
PFNGLVERTEXATTRIB3FNVPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6800
PFNGLVERTEXATTRIB3FVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:6801
PFNGLVERTEXATTRIB3SNVPROC = CFUNCTYPE(None, GLuint, GLshort, GLshort, GLshort) 	# GL/glext.h:6802
PFNGLVERTEXATTRIB3SVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:6803
PFNGLVERTEXATTRIB4DNVPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:6804
PFNGLVERTEXATTRIB4DVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:6805
PFNGLVERTEXATTRIB4FNVPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6806
PFNGLVERTEXATTRIB4FVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:6807
PFNGLVERTEXATTRIB4SNVPROC = CFUNCTYPE(None, GLuint, GLshort, GLshort, GLshort, GLshort) 	# GL/glext.h:6808
PFNGLVERTEXATTRIB4SVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:6809
PFNGLVERTEXATTRIB4UBNVPROC = CFUNCTYPE(None, GLuint, GLubyte, GLubyte, GLubyte, GLubyte) 	# GL/glext.h:6810
PFNGLVERTEXATTRIB4UBVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLubyte)) 	# GL/glext.h:6811
PFNGLVERTEXATTRIBS1DVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLdouble)) 	# GL/glext.h:6812
PFNGLVERTEXATTRIBS1FVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:6813
PFNGLVERTEXATTRIBS1SVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLshort)) 	# GL/glext.h:6814
PFNGLVERTEXATTRIBS2DVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLdouble)) 	# GL/glext.h:6815
PFNGLVERTEXATTRIBS2FVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:6816
PFNGLVERTEXATTRIBS2SVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLshort)) 	# GL/glext.h:6817
PFNGLVERTEXATTRIBS3DVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLdouble)) 	# GL/glext.h:6818
PFNGLVERTEXATTRIBS3FVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:6819
PFNGLVERTEXATTRIBS3SVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLshort)) 	# GL/glext.h:6820
PFNGLVERTEXATTRIBS4DVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLdouble)) 	# GL/glext.h:6821
PFNGLVERTEXATTRIBS4FVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:6822
PFNGLVERTEXATTRIBS4SVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLshort)) 	# GL/glext.h:6823
PFNGLVERTEXATTRIBS4UBVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLubyte)) 	# GL/glext.h:6824
# SGIX_texture_coordinate_clamp (GL/glext.h:6827)
GL_SGIX_texture_coordinate_clamp = 1 	# GL/glext.h:6828
# SGIX_scalebias_hint (GL/glext.h:6831)
GL_SGIX_scalebias_hint = 1 	# GL/glext.h:6832
# OML_interlace (GL/glext.h:6835)
GL_OML_interlace = 1 	# GL/glext.h:6836
# OML_subsample (GL/glext.h:6839)
GL_OML_subsample = 1 	# GL/glext.h:6840
# OML_resample (GL/glext.h:6843)
GL_OML_resample = 1 	# GL/glext.h:6844
# NV_copy_depth_to_color (GL/glext.h:6847)
GL_NV_copy_depth_to_color = 1 	# GL/glext.h:6848
# ATI_envmap_bumpmap (GL/glext.h:6851)
GL_ATI_envmap_bumpmap = 1 	# GL/glext.h:6852
# GL/glext.h:6854
glTexBumpParameterivATI = _link_function('glTexBumpParameterivATI', None, [GLenum, POINTER(GLint)], 'ATI_envmap_bumpmap')

# GL/glext.h:6855
glTexBumpParameterfvATI = _link_function('glTexBumpParameterfvATI', None, [GLenum, POINTER(GLfloat)], 'ATI_envmap_bumpmap')

# GL/glext.h:6856
glGetTexBumpParameterivATI = _link_function('glGetTexBumpParameterivATI', None, [GLenum, POINTER(GLint)], 'ATI_envmap_bumpmap')

# GL/glext.h:6857
glGetTexBumpParameterfvATI = _link_function('glGetTexBumpParameterfvATI', None, [GLenum, POINTER(GLfloat)], 'ATI_envmap_bumpmap')

PFNGLTEXBUMPPARAMETERIVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:6859
PFNGLTEXBUMPPARAMETERFVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6860
PFNGLGETTEXBUMPPARAMETERIVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:6861
PFNGLGETTEXBUMPPARAMETERFVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6862
# ATI_fragment_shader (GL/glext.h:6865)
GL_ATI_fragment_shader = 1 	# GL/glext.h:6866
# GL/glext.h:6868
glGenFragmentShadersATI = _link_function('glGenFragmentShadersATI', GLuint, [GLuint], 'ATI_fragment_shader')

# GL/glext.h:6869
glBindFragmentShaderATI = _link_function('glBindFragmentShaderATI', None, [GLuint], 'ATI_fragment_shader')

# GL/glext.h:6870
glDeleteFragmentShaderATI = _link_function('glDeleteFragmentShaderATI', None, [GLuint], 'ATI_fragment_shader')

# GL/glext.h:6871
glBeginFragmentShaderATI = _link_function('glBeginFragmentShaderATI', None, [], 'ATI_fragment_shader')

# GL/glext.h:6872
glEndFragmentShaderATI = _link_function('glEndFragmentShaderATI', None, [], 'ATI_fragment_shader')

# GL/glext.h:6873
glPassTexCoordATI = _link_function('glPassTexCoordATI', None, [GLuint, GLuint, GLenum], 'ATI_fragment_shader')

# GL/glext.h:6874
glSampleMapATI = _link_function('glSampleMapATI', None, [GLuint, GLuint, GLenum], 'ATI_fragment_shader')

# GL/glext.h:6875
glColorFragmentOp1ATI = _link_function('glColorFragmentOp1ATI', None, [GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint], 'ATI_fragment_shader')

# GL/glext.h:6876
glColorFragmentOp2ATI = _link_function('glColorFragmentOp2ATI', None, [GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint], 'ATI_fragment_shader')

# GL/glext.h:6877
glColorFragmentOp3ATI = _link_function('glColorFragmentOp3ATI', None, [GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint], 'ATI_fragment_shader')

# GL/glext.h:6878
glAlphaFragmentOp1ATI = _link_function('glAlphaFragmentOp1ATI', None, [GLenum, GLuint, GLuint, GLuint, GLuint, GLuint], 'ATI_fragment_shader')

# GL/glext.h:6879
glAlphaFragmentOp2ATI = _link_function('glAlphaFragmentOp2ATI', None, [GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint], 'ATI_fragment_shader')

# GL/glext.h:6880
glAlphaFragmentOp3ATI = _link_function('glAlphaFragmentOp3ATI', None, [GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint], 'ATI_fragment_shader')

# GL/glext.h:6881
glSetFragmentShaderConstantATI = _link_function('glSetFragmentShaderConstantATI', None, [GLuint, POINTER(GLfloat)], 'ATI_fragment_shader')

PFNGLGENFRAGMENTSHADERSATIPROC = CFUNCTYPE(GLuint, GLuint) 	# GL/glext.h:6883
PFNGLBINDFRAGMENTSHADERATIPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6884
PFNGLDELETEFRAGMENTSHADERATIPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6885
PFNGLBEGINFRAGMENTSHADERATIPROC = CFUNCTYPE(None) 	# GL/glext.h:6886
PFNGLENDFRAGMENTSHADERATIPROC = CFUNCTYPE(None) 	# GL/glext.h:6887
PFNGLPASSTEXCOORDATIPROC = CFUNCTYPE(None, GLuint, GLuint, GLenum) 	# GL/glext.h:6888
PFNGLSAMPLEMAPATIPROC = CFUNCTYPE(None, GLuint, GLuint, GLenum) 	# GL/glext.h:6889
PFNGLCOLORFRAGMENTOP1ATIPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:6890
PFNGLCOLORFRAGMENTOP2ATIPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:6891
PFNGLCOLORFRAGMENTOP3ATIPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:6892
PFNGLALPHAFRAGMENTOP1ATIPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:6893
PFNGLALPHAFRAGMENTOP2ATIPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:6894
PFNGLALPHAFRAGMENTOP3ATIPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:6895
PFNGLSETFRAGMENTSHADERCONSTANTATIPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:6896
# ATI_pn_triangles (GL/glext.h:6899)
GL_ATI_pn_triangles = 1 	# GL/glext.h:6900
# GL/glext.h:6902
glPNTrianglesiATI = _link_function('glPNTrianglesiATI', None, [GLenum, GLint], 'ATI_pn_triangles')

# GL/glext.h:6903
glPNTrianglesfATI = _link_function('glPNTrianglesfATI', None, [GLenum, GLfloat], 'ATI_pn_triangles')

PFNGLPNTRIANGLESIATIPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:6905
PFNGLPNTRIANGLESFATIPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:6906
# ATI_vertex_array_object (GL/glext.h:6909)
GL_ATI_vertex_array_object = 1 	# GL/glext.h:6910
# GL/glext.h:6912
glNewObjectBufferATI = _link_function('glNewObjectBufferATI', GLuint, [GLsizei, POINTER(GLvoid), GLenum], 'ATI_vertex_array_object')

# GL/glext.h:6913
glIsObjectBufferATI = _link_function('glIsObjectBufferATI', GLboolean, [GLuint], 'ATI_vertex_array_object')

# GL/glext.h:6914
glUpdateObjectBufferATI = _link_function('glUpdateObjectBufferATI', None, [GLuint, GLuint, GLsizei, POINTER(GLvoid), GLenum], 'ATI_vertex_array_object')

# GL/glext.h:6915
glGetObjectBufferfvATI = _link_function('glGetObjectBufferfvATI', None, [GLuint, GLenum, POINTER(GLfloat)], 'ATI_vertex_array_object')

# GL/glext.h:6916
glGetObjectBufferivATI = _link_function('glGetObjectBufferivATI', None, [GLuint, GLenum, POINTER(GLint)], 'ATI_vertex_array_object')

# GL/glext.h:6917
glFreeObjectBufferATI = _link_function('glFreeObjectBufferATI', None, [GLuint], 'ATI_vertex_array_object')

# GL/glext.h:6918
glArrayObjectATI = _link_function('glArrayObjectATI', None, [GLenum, GLint, GLenum, GLsizei, GLuint, GLuint], 'ATI_vertex_array_object')

# GL/glext.h:6919
glGetArrayObjectfvATI = _link_function('glGetArrayObjectfvATI', None, [GLenum, GLenum, POINTER(GLfloat)], 'ATI_vertex_array_object')

# GL/glext.h:6920
glGetArrayObjectivATI = _link_function('glGetArrayObjectivATI', None, [GLenum, GLenum, POINTER(GLint)], 'ATI_vertex_array_object')

# GL/glext.h:6921
glVariantArrayObjectATI = _link_function('glVariantArrayObjectATI', None, [GLuint, GLenum, GLsizei, GLuint, GLuint], 'ATI_vertex_array_object')

# GL/glext.h:6922
glGetVariantArrayObjectfvATI = _link_function('glGetVariantArrayObjectfvATI', None, [GLuint, GLenum, POINTER(GLfloat)], 'ATI_vertex_array_object')

# GL/glext.h:6923
glGetVariantArrayObjectivATI = _link_function('glGetVariantArrayObjectivATI', None, [GLuint, GLenum, POINTER(GLint)], 'ATI_vertex_array_object')

PFNGLNEWOBJECTBUFFERATIPROC = CFUNCTYPE(GLuint, GLsizei, POINTER(GLvoid), GLenum) 	# GL/glext.h:6925
PFNGLISOBJECTBUFFERATIPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:6926
PFNGLUPDATEOBJECTBUFFERATIPROC = CFUNCTYPE(None, GLuint, GLuint, GLsizei, POINTER(GLvoid), GLenum) 	# GL/glext.h:6927
PFNGLGETOBJECTBUFFERFVATIPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6928
PFNGLGETOBJECTBUFFERIVATIPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:6929
PFNGLFREEOBJECTBUFFERATIPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6930
PFNGLARRAYOBJECTATIPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLsizei, GLuint, GLuint) 	# GL/glext.h:6931
PFNGLGETARRAYOBJECTFVATIPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6932
PFNGLGETARRAYOBJECTIVATIPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:6933
PFNGLVARIANTARRAYOBJECTATIPROC = CFUNCTYPE(None, GLuint, GLenum, GLsizei, GLuint, GLuint) 	# GL/glext.h:6934
PFNGLGETVARIANTARRAYOBJECTFVATIPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6935
PFNGLGETVARIANTARRAYOBJECTIVATIPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:6936
# EXT_vertex_shader (GL/glext.h:6939)
GL_EXT_vertex_shader = 1 	# GL/glext.h:6940
# GL/glext.h:6942
glBeginVertexShaderEXT = _link_function('glBeginVertexShaderEXT', None, [], 'EXT_vertex_shader')

# GL/glext.h:6943
glEndVertexShaderEXT = _link_function('glEndVertexShaderEXT', None, [], 'EXT_vertex_shader')

# GL/glext.h:6944
glBindVertexShaderEXT = _link_function('glBindVertexShaderEXT', None, [GLuint], 'EXT_vertex_shader')

# GL/glext.h:6945
glGenVertexShadersEXT = _link_function('glGenVertexShadersEXT', GLuint, [GLuint], 'EXT_vertex_shader')

# GL/glext.h:6946
glDeleteVertexShaderEXT = _link_function('glDeleteVertexShaderEXT', None, [GLuint], 'EXT_vertex_shader')

# GL/glext.h:6947
glShaderOp1EXT = _link_function('glShaderOp1EXT', None, [GLenum, GLuint, GLuint], 'EXT_vertex_shader')

# GL/glext.h:6948
glShaderOp2EXT = _link_function('glShaderOp2EXT', None, [GLenum, GLuint, GLuint, GLuint], 'EXT_vertex_shader')

# GL/glext.h:6949
glShaderOp3EXT = _link_function('glShaderOp3EXT', None, [GLenum, GLuint, GLuint, GLuint, GLuint], 'EXT_vertex_shader')

# GL/glext.h:6950
glSwizzleEXT = _link_function('glSwizzleEXT', None, [GLuint, GLuint, GLenum, GLenum, GLenum, GLenum], 'EXT_vertex_shader')

# GL/glext.h:6951
glWriteMaskEXT = _link_function('glWriteMaskEXT', None, [GLuint, GLuint, GLenum, GLenum, GLenum, GLenum], 'EXT_vertex_shader')

# GL/glext.h:6952
glInsertComponentEXT = _link_function('glInsertComponentEXT', None, [GLuint, GLuint, GLuint], 'EXT_vertex_shader')

# GL/glext.h:6953
glExtractComponentEXT = _link_function('glExtractComponentEXT', None, [GLuint, GLuint, GLuint], 'EXT_vertex_shader')

# GL/glext.h:6954
glGenSymbolsEXT = _link_function('glGenSymbolsEXT', GLuint, [GLenum, GLenum, GLenum, GLuint], 'EXT_vertex_shader')

# GL/glext.h:6955
glSetInvariantEXT = _link_function('glSetInvariantEXT', None, [GLuint, GLenum, POINTER(GLvoid)], 'EXT_vertex_shader')

# GL/glext.h:6956
glSetLocalConstantEXT = _link_function('glSetLocalConstantEXT', None, [GLuint, GLenum, POINTER(GLvoid)], 'EXT_vertex_shader')

# GL/glext.h:6957
glVariantbvEXT = _link_function('glVariantbvEXT', None, [GLuint, POINTER(GLbyte)], 'EXT_vertex_shader')

# GL/glext.h:6958
glVariantsvEXT = _link_function('glVariantsvEXT', None, [GLuint, POINTER(GLshort)], 'EXT_vertex_shader')

# GL/glext.h:6959
glVariantivEXT = _link_function('glVariantivEXT', None, [GLuint, POINTER(GLint)], 'EXT_vertex_shader')

# GL/glext.h:6960
glVariantfvEXT = _link_function('glVariantfvEXT', None, [GLuint, POINTER(GLfloat)], 'EXT_vertex_shader')

# GL/glext.h:6961
glVariantdvEXT = _link_function('glVariantdvEXT', None, [GLuint, POINTER(GLdouble)], 'EXT_vertex_shader')

# GL/glext.h:6962
glVariantubvEXT = _link_function('glVariantubvEXT', None, [GLuint, POINTER(GLubyte)], 'EXT_vertex_shader')

# GL/glext.h:6963
glVariantusvEXT = _link_function('glVariantusvEXT', None, [GLuint, POINTER(GLushort)], 'EXT_vertex_shader')

# GL/glext.h:6964
glVariantuivEXT = _link_function('glVariantuivEXT', None, [GLuint, POINTER(GLuint)], 'EXT_vertex_shader')

# GL/glext.h:6965
glVariantPointerEXT = _link_function('glVariantPointerEXT', None, [GLuint, GLenum, GLuint, POINTER(GLvoid)], 'EXT_vertex_shader')

# GL/glext.h:6966
glEnableVariantClientStateEXT = _link_function('glEnableVariantClientStateEXT', None, [GLuint], 'EXT_vertex_shader')

# GL/glext.h:6967
glDisableVariantClientStateEXT = _link_function('glDisableVariantClientStateEXT', None, [GLuint], 'EXT_vertex_shader')

# GL/glext.h:6968
glBindLightParameterEXT = _link_function('glBindLightParameterEXT', GLuint, [GLenum, GLenum], 'EXT_vertex_shader')

# GL/glext.h:6969
glBindMaterialParameterEXT = _link_function('glBindMaterialParameterEXT', GLuint, [GLenum, GLenum], 'EXT_vertex_shader')

# GL/glext.h:6970
glBindTexGenParameterEXT = _link_function('glBindTexGenParameterEXT', GLuint, [GLenum, GLenum, GLenum], 'EXT_vertex_shader')

# GL/glext.h:6971
glBindTextureUnitParameterEXT = _link_function('glBindTextureUnitParameterEXT', GLuint, [GLenum, GLenum], 'EXT_vertex_shader')

# GL/glext.h:6972
glBindParameterEXT = _link_function('glBindParameterEXT', GLuint, [GLenum], 'EXT_vertex_shader')

# GL/glext.h:6973
glIsVariantEnabledEXT = _link_function('glIsVariantEnabledEXT', GLboolean, [GLuint, GLenum], 'EXT_vertex_shader')

# GL/glext.h:6974
glGetVariantBooleanvEXT = _link_function('glGetVariantBooleanvEXT', None, [GLuint, GLenum, POINTER(GLboolean)], 'EXT_vertex_shader')

# GL/glext.h:6975
glGetVariantIntegervEXT = _link_function('glGetVariantIntegervEXT', None, [GLuint, GLenum, POINTER(GLint)], 'EXT_vertex_shader')

# GL/glext.h:6976
glGetVariantFloatvEXT = _link_function('glGetVariantFloatvEXT', None, [GLuint, GLenum, POINTER(GLfloat)], 'EXT_vertex_shader')

# GL/glext.h:6977
glGetVariantPointervEXT = _link_function('glGetVariantPointervEXT', None, [GLuint, GLenum, POINTER(POINTER(GLvoid))], 'EXT_vertex_shader')

# GL/glext.h:6978
glGetInvariantBooleanvEXT = _link_function('glGetInvariantBooleanvEXT', None, [GLuint, GLenum, POINTER(GLboolean)], 'EXT_vertex_shader')

# GL/glext.h:6979
glGetInvariantIntegervEXT = _link_function('glGetInvariantIntegervEXT', None, [GLuint, GLenum, POINTER(GLint)], 'EXT_vertex_shader')

# GL/glext.h:6980
glGetInvariantFloatvEXT = _link_function('glGetInvariantFloatvEXT', None, [GLuint, GLenum, POINTER(GLfloat)], 'EXT_vertex_shader')

# GL/glext.h:6981
glGetLocalConstantBooleanvEXT = _link_function('glGetLocalConstantBooleanvEXT', None, [GLuint, GLenum, POINTER(GLboolean)], 'EXT_vertex_shader')

# GL/glext.h:6982
glGetLocalConstantIntegervEXT = _link_function('glGetLocalConstantIntegervEXT', None, [GLuint, GLenum, POINTER(GLint)], 'EXT_vertex_shader')

# GL/glext.h:6983
glGetLocalConstantFloatvEXT = _link_function('glGetLocalConstantFloatvEXT', None, [GLuint, GLenum, POINTER(GLfloat)], 'EXT_vertex_shader')

PFNGLBEGINVERTEXSHADEREXTPROC = CFUNCTYPE(None) 	# GL/glext.h:6985
PFNGLENDVERTEXSHADEREXTPROC = CFUNCTYPE(None) 	# GL/glext.h:6986
PFNGLBINDVERTEXSHADEREXTPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6987
PFNGLGENVERTEXSHADERSEXTPROC = CFUNCTYPE(GLuint, GLuint) 	# GL/glext.h:6988
PFNGLDELETEVERTEXSHADEREXTPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6989
PFNGLSHADEROP1EXTPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint) 	# GL/glext.h:6990
PFNGLSHADEROP2EXTPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint) 	# GL/glext.h:6991
PFNGLSHADEROP3EXTPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:6992
PFNGLSWIZZLEEXTPROC = CFUNCTYPE(None, GLuint, GLuint, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:6993
PFNGLWRITEMASKEXTPROC = CFUNCTYPE(None, GLuint, GLuint, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:6994
PFNGLINSERTCOMPONENTEXTPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint) 	# GL/glext.h:6995
PFNGLEXTRACTCOMPONENTEXTPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint) 	# GL/glext.h:6996
PFNGLGENSYMBOLSEXTPROC = CFUNCTYPE(GLuint, GLenum, GLenum, GLenum, GLuint) 	# GL/glext.h:6997
PFNGLSETINVARIANTEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLvoid)) 	# GL/glext.h:6998
PFNGLSETLOCALCONSTANTEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLvoid)) 	# GL/glext.h:6999
PFNGLVARIANTBVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLbyte)) 	# GL/glext.h:7000
PFNGLVARIANTSVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:7001
PFNGLVARIANTIVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:7002
PFNGLVARIANTFVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:7003
PFNGLVARIANTDVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:7004
PFNGLVARIANTUBVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLubyte)) 	# GL/glext.h:7005
PFNGLVARIANTUSVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLushort)) 	# GL/glext.h:7006
PFNGLVARIANTUIVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:7007
PFNGLVARIANTPOINTEREXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, POINTER(GLvoid)) 	# GL/glext.h:7008
PFNGLENABLEVARIANTCLIENTSTATEEXTPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:7009
PFNGLDISABLEVARIANTCLIENTSTATEEXTPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:7010
PFNGLBINDLIGHTPARAMETEREXTPROC = CFUNCTYPE(GLuint, GLenum, GLenum) 	# GL/glext.h:7011
PFNGLBINDMATERIALPARAMETEREXTPROC = CFUNCTYPE(GLuint, GLenum, GLenum) 	# GL/glext.h:7012
PFNGLBINDTEXGENPARAMETEREXTPROC = CFUNCTYPE(GLuint, GLenum, GLenum, GLenum) 	# GL/glext.h:7013
PFNGLBINDTEXTUREUNITPARAMETEREXTPROC = CFUNCTYPE(GLuint, GLenum, GLenum) 	# GL/glext.h:7014
PFNGLBINDPARAMETEREXTPROC = CFUNCTYPE(GLuint, GLenum) 	# GL/glext.h:7015
PFNGLISVARIANTENABLEDEXTPROC = CFUNCTYPE(GLboolean, GLuint, GLenum) 	# GL/glext.h:7016
PFNGLGETVARIANTBOOLEANVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLboolean)) 	# GL/glext.h:7017
PFNGLGETVARIANTINTEGERVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:7018
PFNGLGETVARIANTFLOATVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:7019
PFNGLGETVARIANTPOINTERVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:7020
PFNGLGETINVARIANTBOOLEANVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLboolean)) 	# GL/glext.h:7021
PFNGLGETINVARIANTINTEGERVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:7022
PFNGLGETINVARIANTFLOATVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:7023
PFNGLGETLOCALCONSTANTBOOLEANVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLboolean)) 	# GL/glext.h:7024
PFNGLGETLOCALCONSTANTINTEGERVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:7025
PFNGLGETLOCALCONSTANTFLOATVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:7026
# ATI_vertex_streams (GL/glext.h:7029)
GL_ATI_vertex_streams = 1 	# GL/glext.h:7030
# GL/glext.h:7032
glVertexStream1sATI = _link_function('glVertexStream1sATI', None, [GLenum, GLshort], 'ATI_vertex_streams')

# GL/glext.h:7033
glVertexStream1svATI = _link_function('glVertexStream1svATI', None, [GLenum, POINTER(GLshort)], 'ATI_vertex_streams')

# GL/glext.h:7034
glVertexStream1iATI = _link_function('glVertexStream1iATI', None, [GLenum, GLint], 'ATI_vertex_streams')

# GL/glext.h:7035
glVertexStream1ivATI = _link_function('glVertexStream1ivATI', None, [GLenum, POINTER(GLint)], 'ATI_vertex_streams')

# GL/glext.h:7036
glVertexStream1fATI = _link_function('glVertexStream1fATI', None, [GLenum, GLfloat], 'ATI_vertex_streams')

# GL/glext.h:7037
glVertexStream1fvATI = _link_function('glVertexStream1fvATI', None, [GLenum, POINTER(GLfloat)], 'ATI_vertex_streams')

# GL/glext.h:7038
glVertexStream1dATI = _link_function('glVertexStream1dATI', None, [GLenum, GLdouble], 'ATI_vertex_streams')

# GL/glext.h:7039
glVertexStream1dvATI = _link_function('glVertexStream1dvATI', None, [GLenum, POINTER(GLdouble)], 'ATI_vertex_streams')

# GL/glext.h:7040
glVertexStream2sATI = _link_function('glVertexStream2sATI', None, [GLenum, GLshort, GLshort], 'ATI_vertex_streams')

# GL/glext.h:7041
glVertexStream2svATI = _link_function('glVertexStream2svATI', None, [GLenum, POINTER(GLshort)], 'ATI_vertex_streams')

# GL/glext.h:7042
glVertexStream2iATI = _link_function('glVertexStream2iATI', None, [GLenum, GLint, GLint], 'ATI_vertex_streams')

# GL/glext.h:7043
glVertexStream2ivATI = _link_function('glVertexStream2ivATI', None, [GLenum, POINTER(GLint)], 'ATI_vertex_streams')

# GL/glext.h:7044
glVertexStream2fATI = _link_function('glVertexStream2fATI', None, [GLenum, GLfloat, GLfloat], 'ATI_vertex_streams')

# GL/glext.h:7045
glVertexStream2fvATI = _link_function('glVertexStream2fvATI', None, [GLenum, POINTER(GLfloat)], 'ATI_vertex_streams')

# GL/glext.h:7046
glVertexStream2dATI = _link_function('glVertexStream2dATI', None, [GLenum, GLdouble, GLdouble], 'ATI_vertex_streams')

# GL/glext.h:7047
glVertexStream2dvATI = _link_function('glVertexStream2dvATI', None, [GLenum, POINTER(GLdouble)], 'ATI_vertex_streams')

# GL/glext.h:7048
glVertexStream3sATI = _link_function('glVertexStream3sATI', None, [GLenum, GLshort, GLshort, GLshort], 'ATI_vertex_streams')

# GL/glext.h:7049
glVertexStream3svATI = _link_function('glVertexStream3svATI', None, [GLenum, POINTER(GLshort)], 'ATI_vertex_streams')

# GL/glext.h:7050
glVertexStream3iATI = _link_function('glVertexStream3iATI', None, [GLenum, GLint, GLint, GLint], 'ATI_vertex_streams')

# GL/glext.h:7051
glVertexStream3ivATI = _link_function('glVertexStream3ivATI', None, [GLenum, POINTER(GLint)], 'ATI_vertex_streams')

# GL/glext.h:7052
glVertexStream3fATI = _link_function('glVertexStream3fATI', None, [GLenum, GLfloat, GLfloat, GLfloat], 'ATI_vertex_streams')

# GL/glext.h:7053
glVertexStream3fvATI = _link_function('glVertexStream3fvATI', None, [GLenum, POINTER(GLfloat)], 'ATI_vertex_streams')

# GL/glext.h:7054
glVertexStream3dATI = _link_function('glVertexStream3dATI', None, [GLenum, GLdouble, GLdouble, GLdouble], 'ATI_vertex_streams')

# GL/glext.h:7055
glVertexStream3dvATI = _link_function('glVertexStream3dvATI', None, [GLenum, POINTER(GLdouble)], 'ATI_vertex_streams')

# GL/glext.h:7056
glVertexStream4sATI = _link_function('glVertexStream4sATI', None, [GLenum, GLshort, GLshort, GLshort, GLshort], 'ATI_vertex_streams')

# GL/glext.h:7057
glVertexStream4svATI = _link_function('glVertexStream4svATI', None, [GLenum, POINTER(GLshort)], 'ATI_vertex_streams')

# GL/glext.h:7058
glVertexStream4iATI = _link_function('glVertexStream4iATI', None, [GLenum, GLint, GLint, GLint, GLint], 'ATI_vertex_streams')

# GL/glext.h:7059
glVertexStream4ivATI = _link_function('glVertexStream4ivATI', None, [GLenum, POINTER(GLint)], 'ATI_vertex_streams')

# GL/glext.h:7060
glVertexStream4fATI = _link_function('glVertexStream4fATI', None, [GLenum, GLfloat, GLfloat, GLfloat, GLfloat], 'ATI_vertex_streams')

# GL/glext.h:7061
glVertexStream4fvATI = _link_function('glVertexStream4fvATI', None, [GLenum, POINTER(GLfloat)], 'ATI_vertex_streams')

# GL/glext.h:7062
glVertexStream4dATI = _link_function('glVertexStream4dATI', None, [GLenum, GLdouble, GLdouble, GLdouble, GLdouble], 'ATI_vertex_streams')

# GL/glext.h:7063
glVertexStream4dvATI = _link_function('glVertexStream4dvATI', None, [GLenum, POINTER(GLdouble)], 'ATI_vertex_streams')

# GL/glext.h:7064
glNormalStream3bATI = _link_function('glNormalStream3bATI', None, [GLenum, GLbyte, GLbyte, GLbyte], 'ATI_vertex_streams')

# GL/glext.h:7065
glNormalStream3bvATI = _link_function('glNormalStream3bvATI', None, [GLenum, POINTER(GLbyte)], 'ATI_vertex_streams')

# GL/glext.h:7066
glNormalStream3sATI = _link_function('glNormalStream3sATI', None, [GLenum, GLshort, GLshort, GLshort], 'ATI_vertex_streams')

# GL/glext.h:7067
glNormalStream3svATI = _link_function('glNormalStream3svATI', None, [GLenum, POINTER(GLshort)], 'ATI_vertex_streams')

# GL/glext.h:7068
glNormalStream3iATI = _link_function('glNormalStream3iATI', None, [GLenum, GLint, GLint, GLint], 'ATI_vertex_streams')

# GL/glext.h:7069
glNormalStream3ivATI = _link_function('glNormalStream3ivATI', None, [GLenum, POINTER(GLint)], 'ATI_vertex_streams')

# GL/glext.h:7070
glNormalStream3fATI = _link_function('glNormalStream3fATI', None, [GLenum, GLfloat, GLfloat, GLfloat], 'ATI_vertex_streams')

# GL/glext.h:7071
glNormalStream3fvATI = _link_function('glNormalStream3fvATI', None, [GLenum, POINTER(GLfloat)], 'ATI_vertex_streams')

# GL/glext.h:7072
glNormalStream3dATI = _link_function('glNormalStream3dATI', None, [GLenum, GLdouble, GLdouble, GLdouble], 'ATI_vertex_streams')

# GL/glext.h:7073
glNormalStream3dvATI = _link_function('glNormalStream3dvATI', None, [GLenum, POINTER(GLdouble)], 'ATI_vertex_streams')

# GL/glext.h:7074
glClientActiveVertexStreamATI = _link_function('glClientActiveVertexStreamATI', None, [GLenum], 'ATI_vertex_streams')

# GL/glext.h:7075
glVertexBlendEnviATI = _link_function('glVertexBlendEnviATI', None, [GLenum, GLint], 'ATI_vertex_streams')

# GL/glext.h:7076
glVertexBlendEnvfATI = _link_function('glVertexBlendEnvfATI', None, [GLenum, GLfloat], 'ATI_vertex_streams')

PFNGLVERTEXSTREAM1SATIPROC = CFUNCTYPE(None, GLenum, GLshort) 	# GL/glext.h:7078
PFNGLVERTEXSTREAM1SVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:7079
PFNGLVERTEXSTREAM1IATIPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:7080
PFNGLVERTEXSTREAM1IVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:7081
PFNGLVERTEXSTREAM1FATIPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:7082
PFNGLVERTEXSTREAM1FVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:7083
PFNGLVERTEXSTREAM1DATIPROC = CFUNCTYPE(None, GLenum, GLdouble) 	# GL/glext.h:7084
PFNGLVERTEXSTREAM1DVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:7085
PFNGLVERTEXSTREAM2SATIPROC = CFUNCTYPE(None, GLenum, GLshort, GLshort) 	# GL/glext.h:7086
PFNGLVERTEXSTREAM2SVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:7087
PFNGLVERTEXSTREAM2IATIPROC = CFUNCTYPE(None, GLenum, GLint, GLint) 	# GL/glext.h:7088
PFNGLVERTEXSTREAM2IVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:7089
PFNGLVERTEXSTREAM2FATIPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat) 	# GL/glext.h:7090
PFNGLVERTEXSTREAM2FVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:7091
PFNGLVERTEXSTREAM2DATIPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble) 	# GL/glext.h:7092
PFNGLVERTEXSTREAM2DVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:7093
PFNGLVERTEXSTREAM3SATIPROC = CFUNCTYPE(None, GLenum, GLshort, GLshort, GLshort) 	# GL/glext.h:7094
PFNGLVERTEXSTREAM3SVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:7095
PFNGLVERTEXSTREAM3IATIPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint) 	# GL/glext.h:7096
PFNGLVERTEXSTREAM3IVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:7097
PFNGLVERTEXSTREAM3FATIPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:7098
PFNGLVERTEXSTREAM3FVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:7099
PFNGLVERTEXSTREAM3DATIPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:7100
PFNGLVERTEXSTREAM3DVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:7101
PFNGLVERTEXSTREAM4SATIPROC = CFUNCTYPE(None, GLenum, GLshort, GLshort, GLshort, GLshort) 	# GL/glext.h:7102
PFNGLVERTEXSTREAM4SVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:7103
PFNGLVERTEXSTREAM4IATIPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint) 	# GL/glext.h:7104
PFNGLVERTEXSTREAM4IVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:7105
PFNGLVERTEXSTREAM4FATIPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:7106
PFNGLVERTEXSTREAM4FVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:7107
PFNGLVERTEXSTREAM4DATIPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:7108
PFNGLVERTEXSTREAM4DVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:7109
PFNGLNORMALSTREAM3BATIPROC = CFUNCTYPE(None, GLenum, GLbyte, GLbyte, GLbyte) 	# GL/glext.h:7110
PFNGLNORMALSTREAM3BVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLbyte)) 	# GL/glext.h:7111
PFNGLNORMALSTREAM3SATIPROC = CFUNCTYPE(None, GLenum, GLshort, GLshort, GLshort) 	# GL/glext.h:7112
PFNGLNORMALSTREAM3SVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:7113
PFNGLNORMALSTREAM3IATIPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint) 	# GL/glext.h:7114
PFNGLNORMALSTREAM3IVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:7115
PFNGLNORMALSTREAM3FATIPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:7116
PFNGLNORMALSTREAM3FVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:7117
PFNGLNORMALSTREAM3DATIPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:7118
PFNGLNORMALSTREAM3DVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:7119
PFNGLCLIENTACTIVEVERTEXSTREAMATIPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:7120
PFNGLVERTEXBLENDENVIATIPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:7121
PFNGLVERTEXBLENDENVFATIPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:7122
# ATI_element_array (GL/glext.h:7125)
GL_ATI_element_array = 1 	# GL/glext.h:7126
# GL/glext.h:7128
glElementPointerATI = _link_function('glElementPointerATI', None, [GLenum, POINTER(GLvoid)], 'ATI_element_array')

# GL/glext.h:7129
glDrawElementArrayATI = _link_function('glDrawElementArrayATI', None, [GLenum, GLsizei], 'ATI_element_array')

# GL/glext.h:7130
glDrawRangeElementArrayATI = _link_function('glDrawRangeElementArrayATI', None, [GLenum, GLuint, GLuint, GLsizei], 'ATI_element_array')

PFNGLELEMENTPOINTERATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLvoid)) 	# GL/glext.h:7132
PFNGLDRAWELEMENTARRAYATIPROC = CFUNCTYPE(None, GLenum, GLsizei) 	# GL/glext.h:7133
PFNGLDRAWRANGEELEMENTARRAYATIPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLsizei) 	# GL/glext.h:7134
# SUN_mesh_array (GL/glext.h:7137)
GL_SUN_mesh_array = 1 	# GL/glext.h:7138
# GL/glext.h:7140
glDrawMeshArraysSUN = _link_function('glDrawMeshArraysSUN', None, [GLenum, GLint, GLsizei, GLsizei], 'SUN_mesh_array')

PFNGLDRAWMESHARRAYSSUNPROC = CFUNCTYPE(None, GLenum, GLint, GLsizei, GLsizei) 	# GL/glext.h:7142
# SUN_slice_accum (GL/glext.h:7145)
GL_SUN_slice_accum = 1 	# GL/glext.h:7146
# NV_multisample_filter_hint (GL/glext.h:7149)
GL_NV_multisample_filter_hint = 1 	# GL/glext.h:7150
# NV_depth_clamp (GL/glext.h:7153)
GL_NV_depth_clamp = 1 	# GL/glext.h:7154
# NV_occlusion_query (GL/glext.h:7157)
GL_NV_occlusion_query = 1 	# GL/glext.h:7158
# GL/glext.h:7160
glGenOcclusionQueriesNV = _link_function('glGenOcclusionQueriesNV', None, [GLsizei, POINTER(GLuint)], 'NV_occlusion_query')

# GL/glext.h:7161
glDeleteOcclusionQueriesNV = _link_function('glDeleteOcclusionQueriesNV', None, [GLsizei, POINTER(GLuint)], 'NV_occlusion_query')

# GL/glext.h:7162
glIsOcclusionQueryNV = _link_function('glIsOcclusionQueryNV', GLboolean, [GLuint], 'NV_occlusion_query')

# GL/glext.h:7163
glBeginOcclusionQueryNV = _link_function('glBeginOcclusionQueryNV', None, [GLuint], 'NV_occlusion_query')

# GL/glext.h:7164
glEndOcclusionQueryNV = _link_function('glEndOcclusionQueryNV', None, [], 'NV_occlusion_query')

# GL/glext.h:7165
glGetOcclusionQueryivNV = _link_function('glGetOcclusionQueryivNV', None, [GLuint, GLenum, POINTER(GLint)], 'NV_occlusion_query')

# GL/glext.h:7166
glGetOcclusionQueryuivNV = _link_function('glGetOcclusionQueryuivNV', None, [GLuint, GLenum, POINTER(GLuint)], 'NV_occlusion_query')

PFNGLGENOCCLUSIONQUERIESNVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:7168
PFNGLDELETEOCCLUSIONQUERIESNVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:7169
PFNGLISOCCLUSIONQUERYNVPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:7170
PFNGLBEGINOCCLUSIONQUERYNVPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:7171
PFNGLENDOCCLUSIONQUERYNVPROC = CFUNCTYPE(None) 	# GL/glext.h:7172
PFNGLGETOCCLUSIONQUERYIVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:7173
PFNGLGETOCCLUSIONQUERYUIVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLuint)) 	# GL/glext.h:7174
# NV_point_sprite (GL/glext.h:7177)
GL_NV_point_sprite = 1 	# GL/glext.h:7178
# GL/glext.h:7180
glPointParameteriNV = _link_function('glPointParameteriNV', None, [GLenum, GLint], 'NV_point_sprite')

# GL/glext.h:7181
glPointParameterivNV = _link_function('glPointParameterivNV', None, [GLenum, POINTER(GLint)], 'NV_point_sprite')

PFNGLPOINTPARAMETERINVPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:7183
PFNGLPOINTPARAMETERIVNVPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:7184
# NV_texture_shader3 (GL/glext.h:7187)
GL_NV_texture_shader3 = 1 	# GL/glext.h:7188
# NV_vertex_program1_1 (GL/glext.h:7191)
GL_NV_vertex_program1_1 = 1 	# GL/glext.h:7192
# EXT_shadow_funcs (GL/glext.h:7195)
GL_EXT_shadow_funcs = 1 	# GL/glext.h:7196
# EXT_stencil_two_side (GL/glext.h:7199)
GL_EXT_stencil_two_side = 1 	# GL/glext.h:7200
# GL/glext.h:7202
glActiveStencilFaceEXT = _link_function('glActiveStencilFaceEXT', None, [GLenum], 'EXT_stencil_two_side')

PFNGLACTIVESTENCILFACEEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:7204
# ATI_text_fragment_shader (GL/glext.h:7207)
GL_ATI_text_fragment_shader = 1 	# GL/glext.h:7208
# APPLE_client_storage (GL/glext.h:7211)
GL_APPLE_client_storage = 1 	# GL/glext.h:7212
# APPLE_element_array (GL/glext.h:7215)
GL_APPLE_element_array = 1 	# GL/glext.h:7216
# GL/glext.h:7218
glElementPointerAPPLE = _link_function('glElementPointerAPPLE', None, [GLenum, POINTER(GLvoid)], 'APPLE_element_array')

# GL/glext.h:7219
glDrawElementArrayAPPLE = _link_function('glDrawElementArrayAPPLE', None, [GLenum, GLint, GLsizei], 'APPLE_element_array')

# GL/glext.h:7220
glDrawRangeElementArrayAPPLE = _link_function('glDrawRangeElementArrayAPPLE', None, [GLenum, GLuint, GLuint, GLint, GLsizei], 'APPLE_element_array')

# GL/glext.h:7221
glMultiDrawElementArrayAPPLE = _link_function('glMultiDrawElementArrayAPPLE', None, [GLenum, POINTER(GLint), POINTER(GLsizei), GLsizei], 'APPLE_element_array')

# GL/glext.h:7222
glMultiDrawRangeElementArrayAPPLE = _link_function('glMultiDrawRangeElementArrayAPPLE', None, [GLenum, GLuint, GLuint, POINTER(GLint), POINTER(GLsizei), GLsizei], 'APPLE_element_array')

PFNGLELEMENTPOINTERAPPLEPROC = CFUNCTYPE(None, GLenum, POINTER(GLvoid)) 	# GL/glext.h:7224
PFNGLDRAWELEMENTARRAYAPPLEPROC = CFUNCTYPE(None, GLenum, GLint, GLsizei) 	# GL/glext.h:7225
PFNGLDRAWRANGEELEMENTARRAYAPPLEPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLint, GLsizei) 	# GL/glext.h:7226
PFNGLMULTIDRAWELEMENTARRAYAPPLEPROC = CFUNCTYPE(None, GLenum, POINTER(GLint), POINTER(GLsizei), GLsizei) 	# GL/glext.h:7227
PFNGLMULTIDRAWRANGEELEMENTARRAYAPPLEPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, POINTER(GLint), POINTER(GLsizei), GLsizei) 	# GL/glext.h:7228
# APPLE_fence (GL/glext.h:7231)
GL_APPLE_fence = 1 	# GL/glext.h:7232
# GL/glext.h:7234
glGenFencesAPPLE = _link_function('glGenFencesAPPLE', None, [GLsizei, POINTER(GLuint)], 'APPLE_fence')

# GL/glext.h:7235
glDeleteFencesAPPLE = _link_function('glDeleteFencesAPPLE', None, [GLsizei, POINTER(GLuint)], 'APPLE_fence')

# GL/glext.h:7236
glSetFenceAPPLE = _link_function('glSetFenceAPPLE', None, [GLuint], 'APPLE_fence')

# GL/glext.h:7237
glIsFenceAPPLE = _link_function('glIsFenceAPPLE', GLboolean, [GLuint], 'APPLE_fence')

# GL/glext.h:7238
glTestFenceAPPLE = _link_function('glTestFenceAPPLE', GLboolean, [GLuint], 'APPLE_fence')

# GL/glext.h:7239
glFinishFenceAPPLE = _link_function('glFinishFenceAPPLE', None, [GLuint], 'APPLE_fence')

# GL/glext.h:7240
glTestObjectAPPLE = _link_function('glTestObjectAPPLE', GLboolean, [GLenum, GLuint], 'APPLE_fence')

# GL/glext.h:7241
glFinishObjectAPPLE = _link_function('glFinishObjectAPPLE', None, [GLenum, GLint], 'APPLE_fence')

PFNGLGENFENCESAPPLEPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:7243
PFNGLDELETEFENCESAPPLEPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:7244
PFNGLSETFENCEAPPLEPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:7245
PFNGLISFENCEAPPLEPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:7246
PFNGLTESTFENCEAPPLEPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:7247
PFNGLFINISHFENCEAPPLEPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:7248
PFNGLTESTOBJECTAPPLEPROC = CFUNCTYPE(GLboolean, GLenum, GLuint) 	# GL/glext.h:7249
PFNGLFINISHOBJECTAPPLEPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:7250
# APPLE_vertex_array_object (GL/glext.h:7253)
GL_APPLE_vertex_array_object = 1 	# GL/glext.h:7254
# GL/glext.h:7256
glBindVertexArrayAPPLE = _link_function('glBindVertexArrayAPPLE', None, [GLuint], 'APPLE_vertex_array_object')

# GL/glext.h:7257
glDeleteVertexArraysAPPLE = _link_function('glDeleteVertexArraysAPPLE', None, [GLsizei, POINTER(GLuint)], 'APPLE_vertex_array_object')

# GL/glext.h:7258
glGenVertexArraysAPPLE = _link_function('glGenVertexArraysAPPLE', None, [GLsizei, POINTER(GLuint)], 'APPLE_vertex_array_object')

# GL/glext.h:7259
glIsVertexArrayAPPLE = _link_function('glIsVertexArrayAPPLE', GLboolean, [GLuint], 'APPLE_vertex_array_object')

PFNGLBINDVERTEXARRAYAPPLEPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:7261
PFNGLDELETEVERTEXARRAYSAPPLEPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:7262
PFNGLGENVERTEXARRAYSAPPLEPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:7263
PFNGLISVERTEXARRAYAPPLEPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:7264
# APPLE_vertex_array_range (GL/glext.h:7267)
GL_APPLE_vertex_array_range = 1 	# GL/glext.h:7268
# GL/glext.h:7270
glVertexArrayRangeAPPLE = _link_function('glVertexArrayRangeAPPLE', None, [GLsizei, POINTER(GLvoid)], 'APPLE_vertex_array_range')

# GL/glext.h:7271
glFlushVertexArrayRangeAPPLE = _link_function('glFlushVertexArrayRangeAPPLE', None, [GLsizei, POINTER(GLvoid)], 'APPLE_vertex_array_range')

# GL/glext.h:7272
glVertexArrayParameteriAPPLE = _link_function('glVertexArrayParameteriAPPLE', None, [GLenum, GLint], 'APPLE_vertex_array_range')

PFNGLVERTEXARRAYRANGEAPPLEPROC = CFUNCTYPE(None, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:7274
PFNGLFLUSHVERTEXARRAYRANGEAPPLEPROC = CFUNCTYPE(None, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:7275
PFNGLVERTEXARRAYPARAMETERIAPPLEPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:7276
# APPLE_ycbcr_422 (GL/glext.h:7279)
GL_APPLE_ycbcr_422 = 1 	# GL/glext.h:7280
# S3_s3tc (GL/glext.h:7283)
GL_S3_s3tc = 1 	# GL/glext.h:7284
# ATI_draw_buffers (GL/glext.h:7287)
GL_ATI_draw_buffers = 1 	# GL/glext.h:7288
# GL/glext.h:7290
glDrawBuffersATI = _link_function('glDrawBuffersATI', None, [GLsizei, POINTER(GLenum)], 'ATI_draw_buffers')

PFNGLDRAWBUFFERSATIPROC = CFUNCTYPE(None, GLsizei, POINTER(GLenum)) 	# GL/glext.h:7292
# ATI_pixel_format_float (GL/glext.h:7295)
GL_ATI_pixel_format_float = 1 	# GL/glext.h:7296
# ATI_texture_env_combine3 (GL/glext.h:7302)
GL_ATI_texture_env_combine3 = 1 	# GL/glext.h:7303
# ATI_texture_float (GL/glext.h:7306)
GL_ATI_texture_float = 1 	# GL/glext.h:7307
# NV_float_buffer (GL/glext.h:7310)
GL_NV_float_buffer = 1 	# GL/glext.h:7311
# NV_fragment_program (GL/glext.h:7314)
GL_NV_fragment_program = 1 	# GL/glext.h:7315
# GL/glext.h:7318
glProgramNamedParameter4fNV = _link_function('glProgramNamedParameter4fNV', None, [GLuint, GLsizei, POINTER(GLubyte), GLfloat, GLfloat, GLfloat, GLfloat], 'NV_fragment_program')

# GL/glext.h:7319
glProgramNamedParameter4dNV = _link_function('glProgramNamedParameter4dNV', None, [GLuint, GLsizei, POINTER(GLubyte), GLdouble, GLdouble, GLdouble, GLdouble], 'NV_fragment_program')

# GL/glext.h:7320
glProgramNamedParameter4fvNV = _link_function('glProgramNamedParameter4fvNV', None, [GLuint, GLsizei, POINTER(GLubyte), POINTER(GLfloat)], 'NV_fragment_program')

# GL/glext.h:7321
glProgramNamedParameter4dvNV = _link_function('glProgramNamedParameter4dvNV', None, [GLuint, GLsizei, POINTER(GLubyte), POINTER(GLdouble)], 'NV_fragment_program')

# GL/glext.h:7322
glGetProgramNamedParameterfvNV = _link_function('glGetProgramNamedParameterfvNV', None, [GLuint, GLsizei, POINTER(GLubyte), POINTER(GLfloat)], 'NV_fragment_program')

# GL/glext.h:7323
glGetProgramNamedParameterdvNV = _link_function('glGetProgramNamedParameterdvNV', None, [GLuint, GLsizei, POINTER(GLubyte), POINTER(GLdouble)], 'NV_fragment_program')

PFNGLPROGRAMNAMEDPARAMETER4FNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLubyte), GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:7325
PFNGLPROGRAMNAMEDPARAMETER4DNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLubyte), GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:7326
PFNGLPROGRAMNAMEDPARAMETER4FVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLubyte), POINTER(GLfloat)) 	# GL/glext.h:7327
PFNGLPROGRAMNAMEDPARAMETER4DVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLubyte), POINTER(GLdouble)) 	# GL/glext.h:7328
PFNGLGETPROGRAMNAMEDPARAMETERFVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLubyte), POINTER(GLfloat)) 	# GL/glext.h:7329
PFNGLGETPROGRAMNAMEDPARAMETERDVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLubyte), POINTER(GLdouble)) 	# GL/glext.h:7330
# NV_half_float (GL/glext.h:7333)
GL_NV_half_float = 1 	# GL/glext.h:7334
# GL/glext.h:7336
glVertex2hNV = _link_function('glVertex2hNV', None, [GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:7337
glVertex2hvNV = _link_function('glVertex2hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:7338
glVertex3hNV = _link_function('glVertex3hNV', None, [GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:7339
glVertex3hvNV = _link_function('glVertex3hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:7340
glVertex4hNV = _link_function('glVertex4hNV', None, [GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:7341
glVertex4hvNV = _link_function('glVertex4hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:7342
glNormal3hNV = _link_function('glNormal3hNV', None, [GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:7343
glNormal3hvNV = _link_function('glNormal3hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:7344
glColor3hNV = _link_function('glColor3hNV', None, [GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:7345
glColor3hvNV = _link_function('glColor3hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:7346
glColor4hNV = _link_function('glColor4hNV', None, [GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:7347
glColor4hvNV = _link_function('glColor4hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:7348
glTexCoord1hNV = _link_function('glTexCoord1hNV', None, [GLhalfNV], 'NV_half_float')

# GL/glext.h:7349
glTexCoord1hvNV = _link_function('glTexCoord1hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:7350
glTexCoord2hNV = _link_function('glTexCoord2hNV', None, [GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:7351
glTexCoord2hvNV = _link_function('glTexCoord2hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:7352
glTexCoord3hNV = _link_function('glTexCoord3hNV', None, [GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:7353
glTexCoord3hvNV = _link_function('glTexCoord3hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:7354
glTexCoord4hNV = _link_function('glTexCoord4hNV', None, [GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:7355
glTexCoord4hvNV = _link_function('glTexCoord4hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:7356
glMultiTexCoord1hNV = _link_function('glMultiTexCoord1hNV', None, [GLenum, GLhalfNV], 'NV_half_float')

# GL/glext.h:7357
glMultiTexCoord1hvNV = _link_function('glMultiTexCoord1hvNV', None, [GLenum, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:7358
glMultiTexCoord2hNV = _link_function('glMultiTexCoord2hNV', None, [GLenum, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:7359
glMultiTexCoord2hvNV = _link_function('glMultiTexCoord2hvNV', None, [GLenum, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:7360
glMultiTexCoord3hNV = _link_function('glMultiTexCoord3hNV', None, [GLenum, GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:7361
glMultiTexCoord3hvNV = _link_function('glMultiTexCoord3hvNV', None, [GLenum, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:7362
glMultiTexCoord4hNV = _link_function('glMultiTexCoord4hNV', None, [GLenum, GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:7363
glMultiTexCoord4hvNV = _link_function('glMultiTexCoord4hvNV', None, [GLenum, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:7364
glFogCoordhNV = _link_function('glFogCoordhNV', None, [GLhalfNV], 'NV_half_float')

# GL/glext.h:7365
glFogCoordhvNV = _link_function('glFogCoordhvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:7366
glSecondaryColor3hNV = _link_function('glSecondaryColor3hNV', None, [GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:7367
glSecondaryColor3hvNV = _link_function('glSecondaryColor3hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:7368
glVertexWeighthNV = _link_function('glVertexWeighthNV', None, [GLhalfNV], 'NV_half_float')

# GL/glext.h:7369
glVertexWeighthvNV = _link_function('glVertexWeighthvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:7370
glVertexAttrib1hNV = _link_function('glVertexAttrib1hNV', None, [GLuint, GLhalfNV], 'NV_half_float')

# GL/glext.h:7371
glVertexAttrib1hvNV = _link_function('glVertexAttrib1hvNV', None, [GLuint, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:7372
glVertexAttrib2hNV = _link_function('glVertexAttrib2hNV', None, [GLuint, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:7373
glVertexAttrib2hvNV = _link_function('glVertexAttrib2hvNV', None, [GLuint, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:7374
glVertexAttrib3hNV = _link_function('glVertexAttrib3hNV', None, [GLuint, GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:7375
glVertexAttrib3hvNV = _link_function('glVertexAttrib3hvNV', None, [GLuint, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:7376
glVertexAttrib4hNV = _link_function('glVertexAttrib4hNV', None, [GLuint, GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:7377
glVertexAttrib4hvNV = _link_function('glVertexAttrib4hvNV', None, [GLuint, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:7378
glVertexAttribs1hvNV = _link_function('glVertexAttribs1hvNV', None, [GLuint, GLsizei, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:7379
glVertexAttribs2hvNV = _link_function('glVertexAttribs2hvNV', None, [GLuint, GLsizei, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:7380
glVertexAttribs3hvNV = _link_function('glVertexAttribs3hvNV', None, [GLuint, GLsizei, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:7381
glVertexAttribs4hvNV = _link_function('glVertexAttribs4hvNV', None, [GLuint, GLsizei, POINTER(GLhalfNV)], 'NV_half_float')

PFNGLVERTEX2HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV) 	# GL/glext.h:7383
PFNGLVERTEX2HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:7384
PFNGLVERTEX3HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:7385
PFNGLVERTEX3HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:7386
PFNGLVERTEX4HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:7387
PFNGLVERTEX4HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:7388
PFNGLNORMAL3HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:7389
PFNGLNORMAL3HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:7390
PFNGLCOLOR3HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:7391
PFNGLCOLOR3HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:7392
PFNGLCOLOR4HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:7393
PFNGLCOLOR4HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:7394
PFNGLTEXCOORD1HNVPROC = CFUNCTYPE(None, GLhalfNV) 	# GL/glext.h:7395
PFNGLTEXCOORD1HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:7396
PFNGLTEXCOORD2HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV) 	# GL/glext.h:7397
PFNGLTEXCOORD2HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:7398
PFNGLTEXCOORD3HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:7399
PFNGLTEXCOORD3HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:7400
PFNGLTEXCOORD4HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:7401
PFNGLTEXCOORD4HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:7402
PFNGLMULTITEXCOORD1HNVPROC = CFUNCTYPE(None, GLenum, GLhalfNV) 	# GL/glext.h:7403
PFNGLMULTITEXCOORD1HVNVPROC = CFUNCTYPE(None, GLenum, POINTER(GLhalfNV)) 	# GL/glext.h:7404
PFNGLMULTITEXCOORD2HNVPROC = CFUNCTYPE(None, GLenum, GLhalfNV, GLhalfNV) 	# GL/glext.h:7405
PFNGLMULTITEXCOORD2HVNVPROC = CFUNCTYPE(None, GLenum, POINTER(GLhalfNV)) 	# GL/glext.h:7406
PFNGLMULTITEXCOORD3HNVPROC = CFUNCTYPE(None, GLenum, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:7407
PFNGLMULTITEXCOORD3HVNVPROC = CFUNCTYPE(None, GLenum, POINTER(GLhalfNV)) 	# GL/glext.h:7408
PFNGLMULTITEXCOORD4HNVPROC = CFUNCTYPE(None, GLenum, GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:7409
PFNGLMULTITEXCOORD4HVNVPROC = CFUNCTYPE(None, GLenum, POINTER(GLhalfNV)) 	# GL/glext.h:7410
PFNGLFOGCOORDHNVPROC = CFUNCTYPE(None, GLhalfNV) 	# GL/glext.h:7411
PFNGLFOGCOORDHVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:7412
PFNGLSECONDARYCOLOR3HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:7413
PFNGLSECONDARYCOLOR3HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:7414
PFNGLVERTEXWEIGHTHNVPROC = CFUNCTYPE(None, GLhalfNV) 	# GL/glext.h:7415
PFNGLVERTEXWEIGHTHVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:7416
PFNGLVERTEXATTRIB1HNVPROC = CFUNCTYPE(None, GLuint, GLhalfNV) 	# GL/glext.h:7417
PFNGLVERTEXATTRIB1HVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLhalfNV)) 	# GL/glext.h:7418
PFNGLVERTEXATTRIB2HNVPROC = CFUNCTYPE(None, GLuint, GLhalfNV, GLhalfNV) 	# GL/glext.h:7419
PFNGLVERTEXATTRIB2HVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLhalfNV)) 	# GL/glext.h:7420
PFNGLVERTEXATTRIB3HNVPROC = CFUNCTYPE(None, GLuint, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:7421
PFNGLVERTEXATTRIB3HVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLhalfNV)) 	# GL/glext.h:7422
PFNGLVERTEXATTRIB4HNVPROC = CFUNCTYPE(None, GLuint, GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:7423
PFNGLVERTEXATTRIB4HVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLhalfNV)) 	# GL/glext.h:7424
PFNGLVERTEXATTRIBS1HVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLhalfNV)) 	# GL/glext.h:7425
PFNGLVERTEXATTRIBS2HVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLhalfNV)) 	# GL/glext.h:7426
PFNGLVERTEXATTRIBS3HVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLhalfNV)) 	# GL/glext.h:7427
PFNGLVERTEXATTRIBS4HVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLhalfNV)) 	# GL/glext.h:7428
# NV_pixel_data_range (GL/glext.h:7431)
GL_NV_pixel_data_range = 1 	# GL/glext.h:7432
# GL/glext.h:7434
glPixelDataRangeNV = _link_function('glPixelDataRangeNV', None, [GLenum, GLsizei, POINTER(GLvoid)], 'NV_pixel_data_range')

# GL/glext.h:7435
glFlushPixelDataRangeNV = _link_function('glFlushPixelDataRangeNV', None, [GLenum], 'NV_pixel_data_range')

PFNGLPIXELDATARANGENVPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:7437
PFNGLFLUSHPIXELDATARANGENVPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:7438
# NV_primitive_restart (GL/glext.h:7441)
GL_NV_primitive_restart = 1 	# GL/glext.h:7442
# GL/glext.h:7444
glPrimitiveRestartNV = _link_function('glPrimitiveRestartNV', None, [], 'NV_primitive_restart')

# GL/glext.h:7445
glPrimitiveRestartIndexNV = _link_function('glPrimitiveRestartIndexNV', None, [GLuint], 'NV_primitive_restart')

PFNGLPRIMITIVERESTARTNVPROC = CFUNCTYPE(None) 	# GL/glext.h:7447
PFNGLPRIMITIVERESTARTINDEXNVPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:7448
# NV_texture_expand_normal (GL/glext.h:7451)
GL_NV_texture_expand_normal = 1 	# GL/glext.h:7452
# NV_vertex_program2 (GL/glext.h:7455)
GL_NV_vertex_program2 = 1 	# GL/glext.h:7456
# ATI_map_object_buffer (GL/glext.h:7459)
GL_ATI_map_object_buffer = 1 	# GL/glext.h:7460
# GL/glext.h:7462
glMapObjectBufferATI = _link_function('glMapObjectBufferATI', POINTER(GLvoid), [GLuint], 'ATI_map_object_buffer')

# GL/glext.h:7463
glUnmapObjectBufferATI = _link_function('glUnmapObjectBufferATI', None, [GLuint], 'ATI_map_object_buffer')

PFNGLMAPOBJECTBUFFERATIPROC = CFUNCTYPE(POINTER(GLvoid), GLuint) 	# GL/glext.h:7465
PFNGLUNMAPOBJECTBUFFERATIPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:7466
# ATI_separate_stencil (GL/glext.h:7469)
GL_ATI_separate_stencil = 1 	# GL/glext.h:7470
# GL/glext.h:7472
glStencilOpSeparateATI = _link_function('glStencilOpSeparateATI', None, [GLenum, GLenum, GLenum, GLenum], 'ATI_separate_stencil')

# GL/glext.h:7473
glStencilFuncSeparateATI = _link_function('glStencilFuncSeparateATI', None, [GLenum, GLenum, GLint, GLuint], 'ATI_separate_stencil')

PFNGLSTENCILOPSEPARATEATIPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:7475
PFNGLSTENCILFUNCSEPARATEATIPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLuint) 	# GL/glext.h:7476
# ATI_vertex_attrib_array_object (GL/glext.h:7479)
GL_ATI_vertex_attrib_array_object = 1 	# GL/glext.h:7480
# GL/glext.h:7482
glVertexAttribArrayObjectATI = _link_function('glVertexAttribArrayObjectATI', None, [GLuint, GLint, GLenum, GLboolean, GLsizei, GLuint, GLuint], 'ATI_vertex_attrib_array_object')

# GL/glext.h:7483
glGetVertexAttribArrayObjectfvATI = _link_function('glGetVertexAttribArrayObjectfvATI', None, [GLuint, GLenum, POINTER(GLfloat)], 'ATI_vertex_attrib_array_object')

# GL/glext.h:7484
glGetVertexAttribArrayObjectivATI = _link_function('glGetVertexAttribArrayObjectivATI', None, [GLuint, GLenum, POINTER(GLint)], 'ATI_vertex_attrib_array_object')

PFNGLVERTEXATTRIBARRAYOBJECTATIPROC = CFUNCTYPE(None, GLuint, GLint, GLenum, GLboolean, GLsizei, GLuint, GLuint) 	# GL/glext.h:7486
PFNGLGETVERTEXATTRIBARRAYOBJECTFVATIPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:7487
PFNGLGETVERTEXATTRIBARRAYOBJECTIVATIPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:7488
# OES_read_format (GL/glext.h:7491)
GL_OES_read_format = 1 	# GL/glext.h:7492
# EXT_depth_bounds_test (GL/glext.h:7495)
GL_EXT_depth_bounds_test = 1 	# GL/glext.h:7496
GLclampd = c_double 	# /usr/include/GL/gl.h:135
# GL/glext.h:7498
glDepthBoundsEXT = _link_function('glDepthBoundsEXT', None, [GLclampd, GLclampd], 'EXT_depth_bounds_test')

PFNGLDEPTHBOUNDSEXTPROC = CFUNCTYPE(None, GLclampd, GLclampd) 	# GL/glext.h:7500
# EXT_texture_mirror_clamp (GL/glext.h:7503)
GL_EXT_texture_mirror_clamp = 1 	# GL/glext.h:7504
# EXT_blend_equation_separate (GL/glext.h:7507)
GL_EXT_blend_equation_separate = 1 	# GL/glext.h:7508
# GL/glext.h:7510
glBlendEquationSeparateEXT = _link_function('glBlendEquationSeparateEXT', None, [GLenum, GLenum], 'EXT_blend_equation_separate')

PFNGLBLENDEQUATIONSEPARATEEXTPROC = CFUNCTYPE(None, GLenum, GLenum) 	# GL/glext.h:7512
# MESA_pack_invert (GL/glext.h:7515)
GL_MESA_pack_invert = 1 	# GL/glext.h:7516
# MESA_ycbcr_texture (GL/glext.h:7519)
GL_MESA_ycbcr_texture = 1 	# GL/glext.h:7520
# EXT_pixel_buffer_object (GL/glext.h:7523)
GL_EXT_pixel_buffer_object = 1 	# GL/glext.h:7524
# NV_fragment_program_option (GL/glext.h:7527)
GL_NV_fragment_program_option = 1 	# GL/glext.h:7528
# NV_fragment_program2 (GL/glext.h:7531)
GL_NV_fragment_program2 = 1 	# GL/glext.h:7532
# NV_vertex_program2_option (GL/glext.h:7535)
GL_NV_vertex_program2_option = 1 	# GL/glext.h:7536
# NV_vertex_program3 (GL/glext.h:7539)
GL_NV_vertex_program3 = 1 	# GL/glext.h:7540
# EXT_framebuffer_object (GL/glext.h:7543)
GL_EXT_framebuffer_object = 1 	# GL/glext.h:7544
# GL/glext.h:7546
glIsRenderbufferEXT = _link_function('glIsRenderbufferEXT', GLboolean, [GLuint], 'EXT_framebuffer_object')

# GL/glext.h:7547
glBindRenderbufferEXT = _link_function('glBindRenderbufferEXT', None, [GLenum, GLuint], 'EXT_framebuffer_object')

# GL/glext.h:7548
glDeleteRenderbuffersEXT = _link_function('glDeleteRenderbuffersEXT', None, [GLsizei, POINTER(GLuint)], 'EXT_framebuffer_object')

# GL/glext.h:7549
glGenRenderbuffersEXT = _link_function('glGenRenderbuffersEXT', None, [GLsizei, POINTER(GLuint)], 'EXT_framebuffer_object')

# GL/glext.h:7550
glRenderbufferStorageEXT = _link_function('glRenderbufferStorageEXT', None, [GLenum, GLenum, GLsizei, GLsizei], 'EXT_framebuffer_object')

# GL/glext.h:7551
glGetRenderbufferParameterivEXT = _link_function('glGetRenderbufferParameterivEXT', None, [GLenum, GLenum, POINTER(GLint)], 'EXT_framebuffer_object')

# GL/glext.h:7552
glIsFramebufferEXT = _link_function('glIsFramebufferEXT', GLboolean, [GLuint], 'EXT_framebuffer_object')

# GL/glext.h:7553
glBindFramebufferEXT = _link_function('glBindFramebufferEXT', None, [GLenum, GLuint], 'EXT_framebuffer_object')

# GL/glext.h:7554
glDeleteFramebuffersEXT = _link_function('glDeleteFramebuffersEXT', None, [GLsizei, POINTER(GLuint)], 'EXT_framebuffer_object')

# GL/glext.h:7555
glGenFramebuffersEXT = _link_function('glGenFramebuffersEXT', None, [GLsizei, POINTER(GLuint)], 'EXT_framebuffer_object')

# GL/glext.h:7556
glCheckFramebufferStatusEXT = _link_function('glCheckFramebufferStatusEXT', GLenum, [GLenum], 'EXT_framebuffer_object')

# GL/glext.h:7557
glFramebufferTexture1DEXT = _link_function('glFramebufferTexture1DEXT', None, [GLenum, GLenum, GLenum, GLuint, GLint], 'EXT_framebuffer_object')

# GL/glext.h:7558
glFramebufferTexture2DEXT = _link_function('glFramebufferTexture2DEXT', None, [GLenum, GLenum, GLenum, GLuint, GLint], 'EXT_framebuffer_object')

# GL/glext.h:7559
glFramebufferTexture3DEXT = _link_function('glFramebufferTexture3DEXT', None, [GLenum, GLenum, GLenum, GLuint, GLint, GLint], 'EXT_framebuffer_object')

# GL/glext.h:7560
glFramebufferRenderbufferEXT = _link_function('glFramebufferRenderbufferEXT', None, [GLenum, GLenum, GLenum, GLuint], 'EXT_framebuffer_object')

# GL/glext.h:7561
glGetFramebufferAttachmentParameterivEXT = _link_function('glGetFramebufferAttachmentParameterivEXT', None, [GLenum, GLenum, GLenum, POINTER(GLint)], 'EXT_framebuffer_object')

# GL/glext.h:7562
glGenerateMipmapEXT = _link_function('glGenerateMipmapEXT', None, [GLenum], 'EXT_framebuffer_object')

PFNGLISRENDERBUFFEREXTPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:7564
PFNGLBINDRENDERBUFFEREXTPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:7565
PFNGLDELETERENDERBUFFERSEXTPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:7566
PFNGLGENRENDERBUFFERSEXTPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:7567
PFNGLRENDERBUFFERSTORAGEEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLsizei) 	# GL/glext.h:7568
PFNGLGETRENDERBUFFERPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:7569
PFNGLISFRAMEBUFFEREXTPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:7570
PFNGLBINDFRAMEBUFFEREXTPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:7571
PFNGLDELETEFRAMEBUFFERSEXTPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:7572
PFNGLGENFRAMEBUFFERSEXTPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:7573
PFNGLCHECKFRAMEBUFFERSTATUSEXTPROC = CFUNCTYPE(GLenum, GLenum) 	# GL/glext.h:7574
PFNGLFRAMEBUFFERTEXTURE1DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLuint, GLint) 	# GL/glext.h:7575
PFNGLFRAMEBUFFERTEXTURE2DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLuint, GLint) 	# GL/glext.h:7576
PFNGLFRAMEBUFFERTEXTURE3DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLuint, GLint, GLint) 	# GL/glext.h:7577
PFNGLFRAMEBUFFERRENDERBUFFEREXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLuint) 	# GL/glext.h:7578
PFNGLGETFRAMEBUFFERATTACHMENTPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:7579
PFNGLGENERATEMIPMAPEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:7580
# GREMEDY_string_marker (GL/glext.h:7583)
GL_GREMEDY_string_marker = 1 	# GL/glext.h:7584
# GL/glext.h:7586
glStringMarkerGREMEDY = _link_function('glStringMarkerGREMEDY', None, [GLsizei, POINTER(GLvoid)], 'GREMEDY_string_marker')

PFNGLSTRINGMARKERGREMEDYPROC = CFUNCTYPE(None, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:7588
# EXT_Cg_shader (GL/glext.h:7591)
GL_EXT_Cg_shader = 1 	# GL/glext.h:7592
# EXT_timer_query (GL/glext.h:7595)
GL_EXT_timer_query = 1 	# GL/glext.h:7596
# GL/glext.h:7598
glGetQueryObjecti64vEXT = _link_function('glGetQueryObjecti64vEXT', None, [GLuint, GLenum, POINTER(GLint64EXT)], 'EXT_timer_query')

# GL/glext.h:7599
glGetQueryObjectui64vEXT = _link_function('glGetQueryObjectui64vEXT', None, [GLuint, GLenum, POINTER(GLuint64EXT)], 'EXT_timer_query')

PFNGLGETQUERYOBJECTI64VEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint64EXT)) 	# GL/glext.h:7601
PFNGLGETQUERYOBJECTUI64VEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLuint64EXT)) 	# GL/glext.h:7602
# EXT_texture_buffer_object (GL/glext.h:7605)
GL_EXT_texture_buffer_object = 1 	# GL/glext.h:7606
# GL/glext.h:7608
glTexBufferEXT = _link_function('glTexBufferEXT', None, [GLenum, GLenum, GLuint], 'EXT_texture_buffer_object')

PFNGLTEXBUFFEREXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLuint) 	# GL/glext.h:7610
# NV_transform_feedback (GL/glext.h:7613)
GL_NV_transform_feedback = 1 	# GL/glext.h:7614
# GL/glext.h:7616
glBeginTransformFeedbackNV = _link_function('glBeginTransformFeedbackNV', None, [GLenum], 'NV_transform_feedback')

# GL/glext.h:7617
glEndTransformFeedbackNV = _link_function('glEndTransformFeedbackNV', None, [], 'NV_transform_feedback')

# GL/glext.h:7618
glTransformFeedbackAttribsNV = _link_function('glTransformFeedbackAttribsNV', None, [GLuint, POINTER(GLint), GLenum], 'NV_transform_feedback')

# GL/glext.h:7619
glBindBufferRangeNV = _link_function('glBindBufferRangeNV', None, [GLenum, GLuint, GLuint, GLintptr, GLsizeiptr], 'NV_transform_feedback')

# GL/glext.h:7620
glBindBufferOffsetNV = _link_function('glBindBufferOffsetNV', None, [GLenum, GLuint, GLuint, GLintptr], 'NV_transform_feedback')

# GL/glext.h:7621
glBindBufferBaseNV = _link_function('glBindBufferBaseNV', None, [GLenum, GLuint, GLuint], 'NV_transform_feedback')

# GL/glext.h:7622
glTransformFeedbackVaryingsNV = _link_function('glTransformFeedbackVaryingsNV', None, [GLuint, GLsizei, POINTER(GLint), GLenum], 'NV_transform_feedback')

# GL/glext.h:7623
glActiveVaryingNV = _link_function('glActiveVaryingNV', None, [GLuint, POINTER(GLchar)], 'NV_transform_feedback')

# GL/glext.h:7624
glGetVaryingLocationNV = _link_function('glGetVaryingLocationNV', GLint, [GLuint, POINTER(GLchar)], 'NV_transform_feedback')

# GL/glext.h:7625
glGetActiveVaryingNV = _link_function('glGetActiveVaryingNV', None, [GLuint, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLsizei), POINTER(GLenum), POINTER(GLchar)], 'NV_transform_feedback')

# GL/glext.h:7626
glGetTransformFeedbackVaryingNV = _link_function('glGetTransformFeedbackVaryingNV', None, [GLuint, GLuint, POINTER(GLint)], 'NV_transform_feedback')

PFNGLBEGINTRANSFORMFEEDBACKNVPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:7628
PFNGLENDTRANSFORMFEEDBACKNVPROC = CFUNCTYPE(None) 	# GL/glext.h:7629
PFNGLTRANSFORMFEEDBACKATTRIBSNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLint), GLenum) 	# GL/glext.h:7630
PFNGLBINDBUFFERRANGENVPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLintptr, GLsizeiptr) 	# GL/glext.h:7631
PFNGLBINDBUFFEROFFSETNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLintptr) 	# GL/glext.h:7632
PFNGLBINDBUFFERBASENVPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint) 	# GL/glext.h:7633
PFNGLTRANSFORMFEEDBACKVARYINGSNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLint), GLenum) 	# GL/glext.h:7634
PFNGLACTIVEVARYINGNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLchar)) 	# GL/glext.h:7635
PFNGLGETVARYINGLOCATIONNVPROC = CFUNCTYPE(GLint, GLuint, POINTER(GLchar)) 	# GL/glext.h:7636
PFNGLGETACTIVEVARYINGNVPROC = CFUNCTYPE(None, GLuint, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLsizei), POINTER(GLenum), POINTER(GLchar)) 	# GL/glext.h:7637
PFNGLGETTRANSFORMFEEDBACKVARYINGNVPROC = CFUNCTYPE(None, GLuint, GLuint, POINTER(GLint)) 	# GL/glext.h:7638
# NV_depth_buffer_float (GL/glext.h:7642)
GL_NV_depth_buffer_float = 1 	# GL/glext.h:7643
# GL/glext.h:7645
glDepthRangedNV = _link_function('glDepthRangedNV', None, [GLdouble, GLdouble], 'NV_depth_buffer_float')

# GL/glext.h:7646
glClearDepthdNV = _link_function('glClearDepthdNV', None, [GLdouble], 'NV_depth_buffer_float')

# GL/glext.h:7647
glDepthBoundsdNV = _link_function('glDepthBoundsdNV', None, [GLdouble, GLdouble], 'NV_depth_buffer_float')

PFNGLDEPTHRANGEDNVPROC = CFUNCTYPE(None, GLdouble, GLdouble) 	# GL/glext.h:7649
PFNGLCLEARDEPTHDNVPROC = CFUNCTYPE(None, GLdouble) 	# GL/glext.h:7650
PFNGLDEPTHBOUNDSDNVPROC = CFUNCTYPE(None, GLdouble, GLdouble) 	# GL/glext.h:7651
# EXT_texture_compression_latc (GL/glext.h:7654)
GL_EXT_texture_compression_latc = 1 	# GL/glext.h:7655
# EXT_framebuffer_sRGB (GL/glext.h:7658)
GL_EXT_framebuffer_sRGB = 1 	# GL/glext.h:7659
# EXT_texture_shared_exponent (GL/glext.h:7662)
GL_EXT_texture_shared_exponent = 1 	# GL/glext.h:7663
# EXT_packed_float (GL/glext.h:7666)
GL_EXT_packed_float = 1 	# GL/glext.h:7667
# EXT_texture_array (GL/glext.h:7670)
GL_EXT_texture_array = 1 	# GL/glext.h:7671
# EXT_draw_buffers2 (GL/glext.h:7674)
GL_EXT_draw_buffers2 = 1 	# GL/glext.h:7675
# GL/glext.h:7677
glColorMaskIndexedEXT = _link_function('glColorMaskIndexedEXT', None, [GLuint, GLboolean, GLboolean, GLboolean, GLboolean], 'EXT_draw_buffers2')

# GL/glext.h:7678
glGetBooleanIndexedvEXT = _link_function('glGetBooleanIndexedvEXT', None, [GLenum, GLuint, POINTER(GLboolean)], 'EXT_draw_buffers2')

# GL/glext.h:7679
glGetIntegerIndexedvEXT = _link_function('glGetIntegerIndexedvEXT', None, [GLenum, GLuint, POINTER(GLint)], 'EXT_draw_buffers2')

# GL/glext.h:7680
glEnableIndexedEXT = _link_function('glEnableIndexedEXT', None, [GLenum, GLuint], 'EXT_draw_buffers2')

# GL/glext.h:7681
glDisableIndexedEXT = _link_function('glDisableIndexedEXT', None, [GLenum, GLuint], 'EXT_draw_buffers2')

# GL/glext.h:7682
glIsEnabledIndexedEXT = _link_function('glIsEnabledIndexedEXT', GLboolean, [GLenum, GLuint], 'EXT_draw_buffers2')

PFNGLCOLORMASKINDEXEDEXTPROC = CFUNCTYPE(None, GLuint, GLboolean, GLboolean, GLboolean, GLboolean) 	# GL/glext.h:7684
PFNGLGETBOOLEANINDEXEDVEXTPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLboolean)) 	# GL/glext.h:7685
PFNGLGETINTEGERINDEXEDVEXTPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLint)) 	# GL/glext.h:7686
PFNGLENABLEINDEXEDEXTPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:7687
PFNGLDISABLEINDEXEDEXTPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:7688
PFNGLISENABLEDINDEXEDEXTPROC = CFUNCTYPE(GLboolean, GLenum, GLuint) 	# GL/glext.h:7689
# EXT_texture_integer (GL/glext.h:7692)
GL_EXT_texture_integer = 1 	# GL/glext.h:7693
# GL/glext.h:7695
glTexParameterIivEXT = _link_function('glTexParameterIivEXT', None, [GLenum, GLenum, POINTER(GLint)], 'EXT_texture_integer')

# GL/glext.h:7696
glTexParameterIuivEXT = _link_function('glTexParameterIuivEXT', None, [GLenum, GLenum, POINTER(GLuint)], 'EXT_texture_integer')

# GL/glext.h:7697
glGetTexParameterIivEXT = _link_function('glGetTexParameterIivEXT', None, [GLenum, GLenum, POINTER(GLint)], 'EXT_texture_integer')

# GL/glext.h:7698
glGetTexParameterIuivEXT = _link_function('glGetTexParameterIuivEXT', None, [GLenum, GLenum, POINTER(GLuint)], 'EXT_texture_integer')

# GL/glext.h:7699
glClearColorIiEXT = _link_function('glClearColorIiEXT', None, [GLint, GLint, GLint, GLint], 'EXT_texture_integer')

# GL/glext.h:7700
glClearColorIuiEXT = _link_function('glClearColorIuiEXT', None, [GLuint, GLuint, GLuint, GLuint], 'EXT_texture_integer')

PFNGLTEXPARAMETERIIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:7702
PFNGLTEXPARAMETERIUIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLuint)) 	# GL/glext.h:7703
PFNGLGETTEXPARAMETERIIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:7704
PFNGLGETTEXPARAMETERIUIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLuint)) 	# GL/glext.h:7705
PFNGLCLEARCOLORIIEXTPROC = CFUNCTYPE(None, GLint, GLint, GLint, GLint) 	# GL/glext.h:7706
PFNGLCLEARCOLORIUIEXTPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:7707
# EXT_bindable_uniform (GL/glext.h:7710)
GL_EXT_bindable_uniform = 1 	# GL/glext.h:7711
# GL/glext.h:7713
glUniformBufferEXT = _link_function('glUniformBufferEXT', None, [GLuint, GLint, GLuint], 'EXT_bindable_uniform')

# GL/glext.h:7714
glGetUniformBufferSizeEXT = _link_function('glGetUniformBufferSizeEXT', GLint, [GLuint, GLint], 'EXT_bindable_uniform')

# GL/glext.h:7715
glGetUniformOffsetEXT = _link_function('glGetUniformOffsetEXT', GLintptr, [GLuint, GLint], 'EXT_bindable_uniform')

PFNGLUNIFORMBUFFEREXTPROC = CFUNCTYPE(None, GLuint, GLint, GLuint) 	# GL/glext.h:7717
PFNGLGETUNIFORMBUFFERSIZEEXTPROC = CFUNCTYPE(GLint, GLuint, GLint) 	# GL/glext.h:7718
PFNGLGETUNIFORMOFFSETEXTPROC = CFUNCTYPE(GLintptr, GLuint, GLint) 	# GL/glext.h:7719
# EXT_gpu_shader4 (GL/glext.h:7722)
GL_EXT_gpu_shader4 = 1 	# GL/glext.h:7723
# GL/glext.h:7725
glGetUniformuivEXT = _link_function('glGetUniformuivEXT', None, [GLuint, GLint, POINTER(GLuint)], 'EXT_gpu_shader4')

# GL/glext.h:7726
glBindFragDataLocationEXT = _link_function('glBindFragDataLocationEXT', None, [GLuint, GLuint, POINTER(GLchar)], 'EXT_gpu_shader4')

# GL/glext.h:7727
glGetFragDataLocationEXT = _link_function('glGetFragDataLocationEXT', GLint, [GLuint, POINTER(GLchar)], 'EXT_gpu_shader4')

# GL/glext.h:7728
glUniform1uiEXT = _link_function('glUniform1uiEXT', None, [GLint, GLuint], 'EXT_gpu_shader4')

# GL/glext.h:7729
glUniform2uiEXT = _link_function('glUniform2uiEXT', None, [GLint, GLuint, GLuint], 'EXT_gpu_shader4')

# GL/glext.h:7730
glUniform3uiEXT = _link_function('glUniform3uiEXT', None, [GLint, GLuint, GLuint, GLuint], 'EXT_gpu_shader4')

# GL/glext.h:7731
glUniform4uiEXT = _link_function('glUniform4uiEXT', None, [GLint, GLuint, GLuint, GLuint, GLuint], 'EXT_gpu_shader4')

# GL/glext.h:7732
glUniform1uivEXT = _link_function('glUniform1uivEXT', None, [GLint, GLsizei, POINTER(GLuint)], 'EXT_gpu_shader4')

# GL/glext.h:7733
glUniform2uivEXT = _link_function('glUniform2uivEXT', None, [GLint, GLsizei, POINTER(GLuint)], 'EXT_gpu_shader4')

# GL/glext.h:7734
glUniform3uivEXT = _link_function('glUniform3uivEXT', None, [GLint, GLsizei, POINTER(GLuint)], 'EXT_gpu_shader4')

# GL/glext.h:7735
glUniform4uivEXT = _link_function('glUniform4uivEXT', None, [GLint, GLsizei, POINTER(GLuint)], 'EXT_gpu_shader4')

# GL/glext.h:7736
glVertexAttribI1iEXT = _link_function('glVertexAttribI1iEXT', None, [GLuint, GLint], 'EXT_gpu_shader4')

# GL/glext.h:7737
glVertexAttribI2iEXT = _link_function('glVertexAttribI2iEXT', None, [GLuint, GLint, GLint], 'EXT_gpu_shader4')

# GL/glext.h:7738
glVertexAttribI3iEXT = _link_function('glVertexAttribI3iEXT', None, [GLuint, GLint, GLint, GLint], 'EXT_gpu_shader4')

# GL/glext.h:7739
glVertexAttribI4iEXT = _link_function('glVertexAttribI4iEXT', None, [GLuint, GLint, GLint, GLint, GLint], 'EXT_gpu_shader4')

# GL/glext.h:7740
glVertexAttribI1uiEXT = _link_function('glVertexAttribI1uiEXT', None, [GLuint, GLuint], 'EXT_gpu_shader4')

# GL/glext.h:7741
glVertexAttribI2uiEXT = _link_function('glVertexAttribI2uiEXT', None, [GLuint, GLuint, GLuint], 'EXT_gpu_shader4')

# GL/glext.h:7742
glVertexAttribI3uiEXT = _link_function('glVertexAttribI3uiEXT', None, [GLuint, GLuint, GLuint, GLuint], 'EXT_gpu_shader4')

# GL/glext.h:7743
glVertexAttribI4uiEXT = _link_function('glVertexAttribI4uiEXT', None, [GLuint, GLuint, GLuint, GLuint, GLuint], 'EXT_gpu_shader4')

# GL/glext.h:7744
glVertexAttribI1ivEXT = _link_function('glVertexAttribI1ivEXT', None, [GLuint, POINTER(GLint)], 'EXT_gpu_shader4')

# GL/glext.h:7745
glVertexAttribI2ivEXT = _link_function('glVertexAttribI2ivEXT', None, [GLuint, POINTER(GLint)], 'EXT_gpu_shader4')

# GL/glext.h:7746
glVertexAttribI3ivEXT = _link_function('glVertexAttribI3ivEXT', None, [GLuint, POINTER(GLint)], 'EXT_gpu_shader4')

# GL/glext.h:7747
glVertexAttribI4ivEXT = _link_function('glVertexAttribI4ivEXT', None, [GLuint, POINTER(GLint)], 'EXT_gpu_shader4')

# GL/glext.h:7748
glVertexAttribI1uivEXT = _link_function('glVertexAttribI1uivEXT', None, [GLuint, POINTER(GLuint)], 'EXT_gpu_shader4')

# GL/glext.h:7749
glVertexAttribI2uivEXT = _link_function('glVertexAttribI2uivEXT', None, [GLuint, POINTER(GLuint)], 'EXT_gpu_shader4')

# GL/glext.h:7750
glVertexAttribI3uivEXT = _link_function('glVertexAttribI3uivEXT', None, [GLuint, POINTER(GLuint)], 'EXT_gpu_shader4')

# GL/glext.h:7751
glVertexAttribI4uivEXT = _link_function('glVertexAttribI4uivEXT', None, [GLuint, POINTER(GLuint)], 'EXT_gpu_shader4')

# GL/glext.h:7752
glVertexAttribI4bvEXT = _link_function('glVertexAttribI4bvEXT', None, [GLuint, POINTER(GLbyte)], 'EXT_gpu_shader4')

# GL/glext.h:7753
glVertexAttribI4svEXT = _link_function('glVertexAttribI4svEXT', None, [GLuint, POINTER(GLshort)], 'EXT_gpu_shader4')

# GL/glext.h:7754
glVertexAttribI4ubvEXT = _link_function('glVertexAttribI4ubvEXT', None, [GLuint, POINTER(GLubyte)], 'EXT_gpu_shader4')

# GL/glext.h:7755
glVertexAttribI4usvEXT = _link_function('glVertexAttribI4usvEXT', None, [GLuint, POINTER(GLushort)], 'EXT_gpu_shader4')

# GL/glext.h:7756
glVertexAttribIPointerEXT = _link_function('glVertexAttribIPointerEXT', None, [GLuint, GLint, GLenum, GLsizei, POINTER(GLvoid)], 'EXT_gpu_shader4')

# GL/glext.h:7757
glGetVertexAttribIivEXT = _link_function('glGetVertexAttribIivEXT', None, [GLuint, GLenum, POINTER(GLint)], 'EXT_gpu_shader4')

# GL/glext.h:7758
glGetVertexAttribIuivEXT = _link_function('glGetVertexAttribIuivEXT', None, [GLuint, GLenum, POINTER(GLuint)], 'EXT_gpu_shader4')

PFNGLGETUNIFORMUIVEXTPROC = CFUNCTYPE(None, GLuint, GLint, POINTER(GLuint)) 	# GL/glext.h:7760
PFNGLBINDFRAGDATALOCATIONEXTPROC = CFUNCTYPE(None, GLuint, GLuint, POINTER(GLchar)) 	# GL/glext.h:7761
PFNGLGETFRAGDATALOCATIONEXTPROC = CFUNCTYPE(GLint, GLuint, POINTER(GLchar)) 	# GL/glext.h:7762
PFNGLUNIFORM1UIEXTPROC = CFUNCTYPE(None, GLint, GLuint) 	# GL/glext.h:7763
PFNGLUNIFORM2UIEXTPROC = CFUNCTYPE(None, GLint, GLuint, GLuint) 	# GL/glext.h:7764
PFNGLUNIFORM3UIEXTPROC = CFUNCTYPE(None, GLint, GLuint, GLuint, GLuint) 	# GL/glext.h:7765
PFNGLUNIFORM4UIEXTPROC = CFUNCTYPE(None, GLint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:7766
PFNGLUNIFORM1UIVEXTPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:7767
PFNGLUNIFORM2UIVEXTPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:7768
PFNGLUNIFORM3UIVEXTPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:7769
PFNGLUNIFORM4UIVEXTPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:7770
PFNGLVERTEXATTRIBI1IEXTPROC = CFUNCTYPE(None, GLuint, GLint) 	# GL/glext.h:7771
PFNGLVERTEXATTRIBI2IEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLint) 	# GL/glext.h:7772
PFNGLVERTEXATTRIBI3IEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLint, GLint) 	# GL/glext.h:7773
PFNGLVERTEXATTRIBI4IEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLint, GLint, GLint) 	# GL/glext.h:7774
PFNGLVERTEXATTRIBI1UIEXTPROC = CFUNCTYPE(None, GLuint, GLuint) 	# GL/glext.h:7775
PFNGLVERTEXATTRIBI2UIEXTPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint) 	# GL/glext.h:7776
PFNGLVERTEXATTRIBI3UIEXTPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:7777
PFNGLVERTEXATTRIBI4UIEXTPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:7778
PFNGLVERTEXATTRIBI1IVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:7779
PFNGLVERTEXATTRIBI2IVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:7780
PFNGLVERTEXATTRIBI3IVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:7781
PFNGLVERTEXATTRIBI4IVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:7782
PFNGLVERTEXATTRIBI1UIVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:7783
PFNGLVERTEXATTRIBI2UIVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:7784
PFNGLVERTEXATTRIBI3UIVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:7785
PFNGLVERTEXATTRIBI4UIVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:7786
PFNGLVERTEXATTRIBI4BVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLbyte)) 	# GL/glext.h:7787
PFNGLVERTEXATTRIBI4SVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:7788
PFNGLVERTEXATTRIBI4UBVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLubyte)) 	# GL/glext.h:7789
PFNGLVERTEXATTRIBI4USVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLushort)) 	# GL/glext.h:7790
PFNGLVERTEXATTRIBIPOINTEREXTPROC = CFUNCTYPE(None, GLuint, GLint, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:7791
PFNGLGETVERTEXATTRIBIIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:7792
PFNGLGETVERTEXATTRIBIUIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLuint)) 	# GL/glext.h:7793
# EXT_geometry_shader4 (GL/glext.h:7797)
GL_EXT_geometry_shader4 = 1 	# GL/glext.h:7798
# GL/glext.h:7800
glProgramParameteriEXT = _link_function('glProgramParameteriEXT', None, [GLuint, GLenum, GLint], 'EXT_geometry_shader4')

# GL/glext.h:7801
glFramebufferTextureEXT = _link_function('glFramebufferTextureEXT', None, [GLenum, GLenum, GLuint, GLint], 'EXT_geometry_shader4')

# GL/glext.h:7802
glFramebufferTextureLayerEXT = _link_function('glFramebufferTextureLayerEXT', None, [GLenum, GLenum, GLuint, GLint, GLint], 'EXT_geometry_shader4')

# GL/glext.h:7803
glFramebufferTextureFaceEXT = _link_function('glFramebufferTextureFaceEXT', None, [GLenum, GLenum, GLuint, GLint, GLenum], 'EXT_geometry_shader4')

PFNGLPROGRAMPARAMETERIEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint) 	# GL/glext.h:7805
PFNGLFRAMEBUFFERTEXTUREEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLuint, GLint) 	# GL/glext.h:7806
PFNGLFRAMEBUFFERTEXTURELAYEREXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLuint, GLint, GLint) 	# GL/glext.h:7807
PFNGLFRAMEBUFFERTEXTUREFACEEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLuint, GLint, GLenum) 	# GL/glext.h:7808
# NV_geometry_program4 (GL/glext.h:7811)
GL_NV_geometry_program4 = 1 	# GL/glext.h:7812
# GL/glext.h:7814
glProgramVertexLimitNV = _link_function('glProgramVertexLimitNV', None, [GLenum, GLint], 'NV_geometry_program4')

PFNGLPROGRAMVERTEXLIMITNVPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:7816
# NV_gpu_program4 (GL/glext.h:7819)
GL_NV_gpu_program4 = 1 	# GL/glext.h:7820
# GL/glext.h:7822
glProgramLocalParameterI4iNV = _link_function('glProgramLocalParameterI4iNV', None, [GLenum, GLuint, GLint, GLint, GLint, GLint], 'NV_gpu_program4')

# GL/glext.h:7823
glProgramLocalParameterI4ivNV = _link_function('glProgramLocalParameterI4ivNV', None, [GLenum, GLuint, POINTER(GLint)], 'NV_gpu_program4')

# GL/glext.h:7824
glProgramLocalParametersI4ivNV = _link_function('glProgramLocalParametersI4ivNV', None, [GLenum, GLuint, GLsizei, POINTER(GLint)], 'NV_gpu_program4')

# GL/glext.h:7825
glProgramLocalParameterI4uiNV = _link_function('glProgramLocalParameterI4uiNV', None, [GLenum, GLuint, GLuint, GLuint, GLuint, GLuint], 'NV_gpu_program4')

# GL/glext.h:7826
glProgramLocalParameterI4uivNV = _link_function('glProgramLocalParameterI4uivNV', None, [GLenum, GLuint, POINTER(GLuint)], 'NV_gpu_program4')

# GL/glext.h:7827
glProgramLocalParametersI4uivNV = _link_function('glProgramLocalParametersI4uivNV', None, [GLenum, GLuint, GLsizei, POINTER(GLuint)], 'NV_gpu_program4')

# GL/glext.h:7828
glProgramEnvParameterI4iNV = _link_function('glProgramEnvParameterI4iNV', None, [GLenum, GLuint, GLint, GLint, GLint, GLint], 'NV_gpu_program4')

# GL/glext.h:7829
glProgramEnvParameterI4ivNV = _link_function('glProgramEnvParameterI4ivNV', None, [GLenum, GLuint, POINTER(GLint)], 'NV_gpu_program4')

# GL/glext.h:7830
glProgramEnvParametersI4ivNV = _link_function('glProgramEnvParametersI4ivNV', None, [GLenum, GLuint, GLsizei, POINTER(GLint)], 'NV_gpu_program4')

# GL/glext.h:7831
glProgramEnvParameterI4uiNV = _link_function('glProgramEnvParameterI4uiNV', None, [GLenum, GLuint, GLuint, GLuint, GLuint, GLuint], 'NV_gpu_program4')

# GL/glext.h:7832
glProgramEnvParameterI4uivNV = _link_function('glProgramEnvParameterI4uivNV', None, [GLenum, GLuint, POINTER(GLuint)], 'NV_gpu_program4')

# GL/glext.h:7833
glProgramEnvParametersI4uivNV = _link_function('glProgramEnvParametersI4uivNV', None, [GLenum, GLuint, GLsizei, POINTER(GLuint)], 'NV_gpu_program4')

# GL/glext.h:7834
glGetProgramLocalParameterIivNV = _link_function('glGetProgramLocalParameterIivNV', None, [GLenum, GLuint, POINTER(GLint)], 'NV_gpu_program4')

# GL/glext.h:7835
glGetProgramLocalParameterIuivNV = _link_function('glGetProgramLocalParameterIuivNV', None, [GLenum, GLuint, POINTER(GLuint)], 'NV_gpu_program4')

# GL/glext.h:7836
glGetProgramEnvParameterIivNV = _link_function('glGetProgramEnvParameterIivNV', None, [GLenum, GLuint, POINTER(GLint)], 'NV_gpu_program4')

# GL/glext.h:7837
glGetProgramEnvParameterIuivNV = _link_function('glGetProgramEnvParameterIuivNV', None, [GLenum, GLuint, POINTER(GLuint)], 'NV_gpu_program4')

PFNGLPROGRAMLOCALPARAMETERI4INVPROC = CFUNCTYPE(None, GLenum, GLuint, GLint, GLint, GLint, GLint) 	# GL/glext.h:7839
PFNGLPROGRAMLOCALPARAMETERI4IVNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLint)) 	# GL/glext.h:7840
PFNGLPROGRAMLOCALPARAMETERSI4IVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLsizei, POINTER(GLint)) 	# GL/glext.h:7841
PFNGLPROGRAMLOCALPARAMETERI4UINVPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:7842
PFNGLPROGRAMLOCALPARAMETERI4UIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLuint)) 	# GL/glext.h:7843
PFNGLPROGRAMLOCALPARAMETERSI4UIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:7844
PFNGLPROGRAMENVPARAMETERI4INVPROC = CFUNCTYPE(None, GLenum, GLuint, GLint, GLint, GLint, GLint) 	# GL/glext.h:7845
PFNGLPROGRAMENVPARAMETERI4IVNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLint)) 	# GL/glext.h:7846
PFNGLPROGRAMENVPARAMETERSI4IVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLsizei, POINTER(GLint)) 	# GL/glext.h:7847
PFNGLPROGRAMENVPARAMETERI4UINVPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:7848
PFNGLPROGRAMENVPARAMETERI4UIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLuint)) 	# GL/glext.h:7849
PFNGLPROGRAMENVPARAMETERSI4UIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:7850
PFNGLGETPROGRAMLOCALPARAMETERIIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLint)) 	# GL/glext.h:7851
PFNGLGETPROGRAMLOCALPARAMETERIUIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLuint)) 	# GL/glext.h:7852
PFNGLGETPROGRAMENVPARAMETERIIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLint)) 	# GL/glext.h:7853
PFNGLGETPROGRAMENVPARAMETERIUIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLuint)) 	# GL/glext.h:7854
# NV_parameter_buffer_object (GL/glext.h:7857)
GL_NV_parameter_buffer_object = 1 	# GL/glext.h:7858
# GL/glext.h:7860
glProgramBufferParametersfvNV = _link_function('glProgramBufferParametersfvNV', None, [GLenum, GLuint, GLuint, GLsizei, POINTER(GLfloat)], 'NV_parameter_buffer_object')

# GL/glext.h:7861
glProgramBufferParametersIivNV = _link_function('glProgramBufferParametersIivNV', None, [GLenum, GLuint, GLuint, GLsizei, POINTER(GLint)], 'NV_parameter_buffer_object')

# GL/glext.h:7862
glProgramBufferParametersIuivNV = _link_function('glProgramBufferParametersIuivNV', None, [GLenum, GLuint, GLuint, GLsizei, POINTER(GLuint)], 'NV_parameter_buffer_object')

PFNGLPROGRAMBUFFERPARAMETERSFVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:7864
PFNGLPROGRAMBUFFERPARAMETERSIIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLsizei, POINTER(GLint)) 	# GL/glext.h:7865
PFNGLPROGRAMBUFFERPARAMETERSIUIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:7866
# EXT_framebuffer_multisample (GL/glext.h:7869)
GL_EXT_framebuffer_multisample = 1 	# GL/glext.h:7870
# GL/glext.h:7872
glRenderbufferStorageMultisampleEXT = _link_function('glRenderbufferStorageMultisampleEXT', None, [GLenum, GLsizei, GLenum, GLsizei, GLsizei], 'EXT_framebuffer_multisample')

PFNGLRENDERBUFFERSTORAGEMULTISAMPLEEXTPROC = CFUNCTYPE(None, GLenum, GLsizei, GLenum, GLsizei, GLsizei) 	# GL/glext.h:7874
# NV_framebuffer_multisample_coverage (GL/glext.h:7877)
GL_NV_framebuffer_multisample_coverage = 1 	# GL/glext.h:7878
# GL/glext.h:7880
glRenderbufferStorageMultisampleCoverageNV = _link_function('glRenderbufferStorageMultisampleCoverageNV', None, [GLenum, GLsizei, GLsizei, GLenum, GLsizei, GLsizei], 'NV_framebuffer_multisample_coverage')

PFNGLRENDERBUFFERSTORAGEMULTISAMPLECOVERAGENVPROC = CFUNCTYPE(None, GLenum, GLsizei, GLsizei, GLenum, GLsizei, GLsizei) 	# GL/glext.h:7882
# EXT_framebuffer_blit (GL/glext.h:7885)
GL_EXT_framebuffer_blit = 1 	# GL/glext.h:7886
# GL/glext.h:7888
glBlitFramebufferEXT = _link_function('glBlitFramebufferEXT', None, [GLint, GLint, GLint, GLint, GLint, GLint, GLint, GLint, GLbitfield, GLenum], 'EXT_framebuffer_blit')

PFNGLBLITFRAMEBUFFEREXTPROC = CFUNCTYPE(None, GLint, GLint, GLint, GLint, GLint, GLint, GLint, GLint, GLbitfield, GLenum) 	# GL/glext.h:7890
# EXT_draw_instanced (GL/glext.h:7893)
GL_EXT_draw_instanced = 1 	# GL/glext.h:7894
# GL/glext.h:7896
glDrawArraysInstancedEXT = _link_function('glDrawArraysInstancedEXT', None, [GLenum, GLint, GLsizei, GLsizei], 'EXT_draw_instanced')

# GL/glext.h:7897
glDrawElementsInstancedEXT = _link_function('glDrawElementsInstancedEXT', None, [GLenum, GLsizei, GLenum, POINTER(GLvoid), GLsizei], 'EXT_draw_instanced')

PFNGLDRAWARRAYSINSTANCEDEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLsizei, GLsizei) 	# GL/glext.h:7899
PFNGLDRAWELEMENTSINSTANCEDEXTPROC = CFUNCTYPE(None, GLenum, GLsizei, GLenum, POINTER(GLvoid), GLsizei) 	# GL/glext.h:7900
# EXT_texture_compression_rgtc (GL/glext.h:7903)
GL_EXT_texture_compression_rgtc = 1 	# GL/glext.h:7904
# NV_present_video (GL/glext.h:7907)
GL_NV_present_video = 1 	# GL/glext.h:7908
# GL/glext.h:7910
glPresentFrameKeyedNV = _link_function('glPresentFrameKeyedNV', None, [GLuint, GLuint64EXT, GLuint, GLuint, GLenum, GLenum, GLuint, GLuint, GLenum, GLuint, GLuint], 'NV_present_video')

# GL/glext.h:7917
glPresentFrameDualFillNV = _link_function('glPresentFrameDualFillNV', None, [GLuint, GLuint64EXT, GLuint, GLuint, GLenum, GLenum, GLuint, GLenum, GLuint, GLenum, GLuint, GLenum, GLuint], 'NV_present_video')

# GL/glext.h:7926
glGetVideoivNV = _link_function('glGetVideoivNV', None, [GLuint, GLenum, POINTER(GLint)], 'NV_present_video')

# GL/glext.h:7927
glGetVideouivNV = _link_function('glGetVideouivNV', None, [GLuint, GLenum, POINTER(GLuint)], 'NV_present_video')

# GL/glext.h:7928
glGetVideoi64vNV = _link_function('glGetVideoi64vNV', None, [GLuint, GLenum, POINTER(GLint64EXT)], 'NV_present_video')

# GL/glext.h:7929
glGetVideoui64vNV = _link_function('glGetVideoui64vNV', None, [GLuint, GLenum, POINTER(GLuint64EXT)], 'NV_present_video')

PFNGLPRESENTFRAMEKEYEDNVPROC = CFUNCTYPE(None, GLuint, GLuint64EXT, GLuint, GLuint, GLenum, GLenum, GLuint, GLuint, GLenum, GLuint, GLuint) 	# GL/glext.h:7931
PFNGLPRESENTFRAMEDUALFILLNVPROC = CFUNCTYPE(None, GLuint, GLuint64EXT, GLuint, GLuint, GLenum, GLenum, GLuint, GLenum, GLuint, GLenum, GLuint, GLenum, GLuint) 	# GL/glext.h:7938
PFNGLGETVIDEOIVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:7947
PFNGLGETVIDEOUIVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLuint)) 	# GL/glext.h:7948
PFNGLGETVIDEOI64VNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint64EXT)) 	# GL/glext.h:7949
PFNGLGETVIDEOUI64VNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLuint64EXT)) 	# GL/glext.h:7950
# NV_conditional_render (GL/glext.h:7953)
GL_NV_conditional_render = 1 	# GL/glext.h:7954
# GL/glext.h:7956
glBeginConditionalRenderNV = _link_function('glBeginConditionalRenderNV', None, [GLuint, GLenum], 'NV_conditional_render')

# GL/glext.h:7957
glEndConditionalRenderNV = _link_function('glEndConditionalRenderNV', None, [], 'NV_conditional_render')

PFNGLBEGINCONDITIONALRENDERNVPROC = CFUNCTYPE(None, GLuint, GLenum) 	# GL/glext.h:7959
PFNGLENDCONDITIONALRENDERNVPROC = CFUNCTYPE(None) 	# GL/glext.h:7960
# EXT_transform_feedback (GL/glext.h:7963)
GL_EXT_transform_feedback = 1 	# GL/glext.h:7964
# GL/glext.h:7966
glBeginTransformFeedbackEXT = _link_function('glBeginTransformFeedbackEXT', None, [GLenum], 'EXT_transform_feedback')

# GL/glext.h:7967
glEndTransformFeedbackEXT = _link_function('glEndTransformFeedbackEXT', None, [], 'EXT_transform_feedback')

# GL/glext.h:7968
glBindBufferRangeEXT = _link_function('glBindBufferRangeEXT', None, [GLenum, GLuint, GLuint, GLintptr, GLsizeiptr], 'EXT_transform_feedback')

# GL/glext.h:7969
glBindBufferOffsetEXT = _link_function('glBindBufferOffsetEXT', None, [GLenum, GLuint, GLuint, GLintptr], 'EXT_transform_feedback')

# GL/glext.h:7970
glBindBufferBaseEXT = _link_function('glBindBufferBaseEXT', None, [GLenum, GLuint, GLuint], 'EXT_transform_feedback')

# GL/glext.h:7971
glTransformFeedbackVaryingsEXT = _link_function('glTransformFeedbackVaryingsEXT', None, [GLuint, GLsizei, POINTER(POINTER(GLchar)), GLenum], 'EXT_transform_feedback')

# GL/glext.h:7972
glGetTransformFeedbackVaryingEXT = _link_function('glGetTransformFeedbackVaryingEXT', None, [GLuint, GLuint, POINTER(GLint)], 'EXT_transform_feedback')

PFNGLBEGINTRANSFORMFEEDBACKEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:7974
PFNGLENDTRANSFORMFEEDBACKEXTPROC = CFUNCTYPE(None) 	# GL/glext.h:7975
PFNGLBINDBUFFERRANGEEXTPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLintptr, GLsizeiptr) 	# GL/glext.h:7976
PFNGLBINDBUFFEROFFSETEXTPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLintptr) 	# GL/glext.h:7977
PFNGLBINDBUFFERBASEEXTPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint) 	# GL/glext.h:7978
PFNGLTRANSFORMFEEDBACKVARYINGSEXTPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLint), GLenum) 	# GL/glext.h:7979
PFNGLGETTRANSFORMFEEDBACKVARYINGEXTPROC = CFUNCTYPE(None, GLuint, GLuint, POINTER(GLint)) 	# GL/glext.h:7980
# EXT_direct_state_access (GL/glext.h:7983)
GL_EXT_direct_state_access = 1 	# GL/glext.h:7984
# GL/glext.h:7986
glClientAttribDefaultEXT = _link_function('glClientAttribDefaultEXT', None, [GLbitfield], 'EXT_direct_state_access')

# GL/glext.h:7987
glPushClientAttribDefaultEXT = _link_function('glPushClientAttribDefaultEXT', None, [GLbitfield], 'EXT_direct_state_access')

# GL/glext.h:7988
glMatrixLoadfEXT = _link_function('glMatrixLoadfEXT', None, [GLenum, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:7989
glMatrixLoaddEXT = _link_function('glMatrixLoaddEXT', None, [GLenum, POINTER(GLdouble)], 'EXT_direct_state_access')

# GL/glext.h:7990
glMatrixMultfEXT = _link_function('glMatrixMultfEXT', None, [GLenum, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:7991
glMatrixMultdEXT = _link_function('glMatrixMultdEXT', None, [GLenum, POINTER(GLdouble)], 'EXT_direct_state_access')

# GL/glext.h:7992
glMatrixLoadIdentityEXT = _link_function('glMatrixLoadIdentityEXT', None, [GLenum], 'EXT_direct_state_access')

# GL/glext.h:7993
glMatrixRotatefEXT = _link_function('glMatrixRotatefEXT', None, [GLenum, GLfloat, GLfloat, GLfloat, GLfloat], 'EXT_direct_state_access')

# GL/glext.h:7994
glMatrixRotatedEXT = _link_function('glMatrixRotatedEXT', None, [GLenum, GLdouble, GLdouble, GLdouble, GLdouble], 'EXT_direct_state_access')

# GL/glext.h:7995
glMatrixScalefEXT = _link_function('glMatrixScalefEXT', None, [GLenum, GLfloat, GLfloat, GLfloat], 'EXT_direct_state_access')

# GL/glext.h:7996
glMatrixScaledEXT = _link_function('glMatrixScaledEXT', None, [GLenum, GLdouble, GLdouble, GLdouble], 'EXT_direct_state_access')

# GL/glext.h:7997
glMatrixTranslatefEXT = _link_function('glMatrixTranslatefEXT', None, [GLenum, GLfloat, GLfloat, GLfloat], 'EXT_direct_state_access')

# GL/glext.h:7998
glMatrixTranslatedEXT = _link_function('glMatrixTranslatedEXT', None, [GLenum, GLdouble, GLdouble, GLdouble], 'EXT_direct_state_access')

# GL/glext.h:7999
glMatrixFrustumEXT = _link_function('glMatrixFrustumEXT', None, [GLenum, GLdouble, GLdouble, GLdouble, GLdouble, GLdouble, GLdouble], 'EXT_direct_state_access')

# GL/glext.h:8000
glMatrixOrthoEXT = _link_function('glMatrixOrthoEXT', None, [GLenum, GLdouble, GLdouble, GLdouble, GLdouble, GLdouble, GLdouble], 'EXT_direct_state_access')

# GL/glext.h:8001
glMatrixPopEXT = _link_function('glMatrixPopEXT', None, [GLenum], 'EXT_direct_state_access')

# GL/glext.h:8002
glMatrixPushEXT = _link_function('glMatrixPushEXT', None, [GLenum], 'EXT_direct_state_access')

# GL/glext.h:8003
glMatrixLoadTransposefEXT = _link_function('glMatrixLoadTransposefEXT', None, [GLenum, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:8004
glMatrixLoadTransposedEXT = _link_function('glMatrixLoadTransposedEXT', None, [GLenum, POINTER(GLdouble)], 'EXT_direct_state_access')

# GL/glext.h:8005
glMatrixMultTransposefEXT = _link_function('glMatrixMultTransposefEXT', None, [GLenum, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:8006
glMatrixMultTransposedEXT = _link_function('glMatrixMultTransposedEXT', None, [GLenum, POINTER(GLdouble)], 'EXT_direct_state_access')

# GL/glext.h:8007
glTextureParameterfEXT = _link_function('glTextureParameterfEXT', None, [GLuint, GLenum, GLenum, GLfloat], 'EXT_direct_state_access')

# GL/glext.h:8008
glTextureParameterfvEXT = _link_function('glTextureParameterfvEXT', None, [GLuint, GLenum, GLenum, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:8009
glTextureParameteriEXT = _link_function('glTextureParameteriEXT', None, [GLuint, GLenum, GLenum, GLint], 'EXT_direct_state_access')

# GL/glext.h:8010
glTextureParameterivEXT = _link_function('glTextureParameterivEXT', None, [GLuint, GLenum, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:8011
glTextureImage1DEXT = _link_function('glTextureImage1DEXT', None, [GLuint, GLenum, GLint, GLenum, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:8012
glTextureImage2DEXT = _link_function('glTextureImage2DEXT', None, [GLuint, GLenum, GLint, GLenum, GLsizei, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:8013
glTextureSubImage1DEXT = _link_function('glTextureSubImage1DEXT', None, [GLuint, GLenum, GLint, GLint, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:8014
glTextureSubImage2DEXT = _link_function('glTextureSubImage2DEXT', None, [GLuint, GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:8015
glCopyTextureImage1DEXT = _link_function('glCopyTextureImage1DEXT', None, [GLuint, GLenum, GLint, GLenum, GLint, GLint, GLsizei, GLint], 'EXT_direct_state_access')

# GL/glext.h:8016
glCopyTextureImage2DEXT = _link_function('glCopyTextureImage2DEXT', None, [GLuint, GLenum, GLint, GLenum, GLint, GLint, GLsizei, GLsizei, GLint], 'EXT_direct_state_access')

# GL/glext.h:8017
glCopyTextureSubImage1DEXT = _link_function('glCopyTextureSubImage1DEXT', None, [GLuint, GLenum, GLint, GLint, GLint, GLint, GLsizei], 'EXT_direct_state_access')

# GL/glext.h:8018
glCopyTextureSubImage2DEXT = _link_function('glCopyTextureSubImage2DEXT', None, [GLuint, GLenum, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei], 'EXT_direct_state_access')

# GL/glext.h:8019
glGetTextureImageEXT = _link_function('glGetTextureImageEXT', None, [GLuint, GLenum, GLint, GLenum, GLenum, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:8020
glGetTextureParameterfvEXT = _link_function('glGetTextureParameterfvEXT', None, [GLuint, GLenum, GLenum, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:8021
glGetTextureParameterivEXT = _link_function('glGetTextureParameterivEXT', None, [GLuint, GLenum, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:8022
glGetTextureLevelParameterfvEXT = _link_function('glGetTextureLevelParameterfvEXT', None, [GLuint, GLenum, GLint, GLenum, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:8023
glGetTextureLevelParameterivEXT = _link_function('glGetTextureLevelParameterivEXT', None, [GLuint, GLenum, GLint, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:8024
glTextureImage3DEXT = _link_function('glTextureImage3DEXT', None, [GLuint, GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:8025
glTextureSubImage3DEXT = _link_function('glTextureSubImage3DEXT', None, [GLuint, GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:8026
glCopyTextureSubImage3DEXT = _link_function('glCopyTextureSubImage3DEXT', None, [GLuint, GLenum, GLint, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei], 'EXT_direct_state_access')

# GL/glext.h:8027
glMultiTexParameterfEXT = _link_function('glMultiTexParameterfEXT', None, [GLenum, GLenum, GLenum, GLfloat], 'EXT_direct_state_access')

# GL/glext.h:8028
glMultiTexParameterfvEXT = _link_function('glMultiTexParameterfvEXT', None, [GLenum, GLenum, GLenum, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:8029
glMultiTexParameteriEXT = _link_function('glMultiTexParameteriEXT', None, [GLenum, GLenum, GLenum, GLint], 'EXT_direct_state_access')

# GL/glext.h:8030
glMultiTexParameterivEXT = _link_function('glMultiTexParameterivEXT', None, [GLenum, GLenum, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:8031
glMultiTexImage1DEXT = _link_function('glMultiTexImage1DEXT', None, [GLenum, GLenum, GLint, GLenum, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:8032
glMultiTexImage2DEXT = _link_function('glMultiTexImage2DEXT', None, [GLenum, GLenum, GLint, GLenum, GLsizei, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:8033
glMultiTexSubImage1DEXT = _link_function('glMultiTexSubImage1DEXT', None, [GLenum, GLenum, GLint, GLint, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:8034
glMultiTexSubImage2DEXT = _link_function('glMultiTexSubImage2DEXT', None, [GLenum, GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:8035
glCopyMultiTexImage1DEXT = _link_function('glCopyMultiTexImage1DEXT', None, [GLenum, GLenum, GLint, GLenum, GLint, GLint, GLsizei, GLint], 'EXT_direct_state_access')

# GL/glext.h:8036
glCopyMultiTexImage2DEXT = _link_function('glCopyMultiTexImage2DEXT', None, [GLenum, GLenum, GLint, GLenum, GLint, GLint, GLsizei, GLsizei, GLint], 'EXT_direct_state_access')

# GL/glext.h:8037
glCopyMultiTexSubImage1DEXT = _link_function('glCopyMultiTexSubImage1DEXT', None, [GLenum, GLenum, GLint, GLint, GLint, GLint, GLsizei], 'EXT_direct_state_access')

# GL/glext.h:8038
glCopyMultiTexSubImage2DEXT = _link_function('glCopyMultiTexSubImage2DEXT', None, [GLenum, GLenum, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei], 'EXT_direct_state_access')

# GL/glext.h:8039
glGetMultiTexImageEXT = _link_function('glGetMultiTexImageEXT', None, [GLenum, GLenum, GLint, GLenum, GLenum, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:8040
glGetMultiTexParameterfvEXT = _link_function('glGetMultiTexParameterfvEXT', None, [GLenum, GLenum, GLenum, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:8041
glGetMultiTexParameterivEXT = _link_function('glGetMultiTexParameterivEXT', None, [GLenum, GLenum, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:8042
glGetMultiTexLevelParameterfvEXT = _link_function('glGetMultiTexLevelParameterfvEXT', None, [GLenum, GLenum, GLint, GLenum, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:8043
glGetMultiTexLevelParameterivEXT = _link_function('glGetMultiTexLevelParameterivEXT', None, [GLenum, GLenum, GLint, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:8044
glMultiTexImage3DEXT = _link_function('glMultiTexImage3DEXT', None, [GLenum, GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:8045
glMultiTexSubImage3DEXT = _link_function('glMultiTexSubImage3DEXT', None, [GLenum, GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:8046
glCopyMultiTexSubImage3DEXT = _link_function('glCopyMultiTexSubImage3DEXT', None, [GLenum, GLenum, GLint, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei], 'EXT_direct_state_access')

# GL/glext.h:8047
glBindMultiTextureEXT = _link_function('glBindMultiTextureEXT', None, [GLenum, GLenum, GLuint], 'EXT_direct_state_access')

# GL/glext.h:8048
glEnableClientStateIndexedEXT = _link_function('glEnableClientStateIndexedEXT', None, [GLenum, GLuint], 'EXT_direct_state_access')

# GL/glext.h:8049
glDisableClientStateIndexedEXT = _link_function('glDisableClientStateIndexedEXT', None, [GLenum, GLuint], 'EXT_direct_state_access')

# GL/glext.h:8050
glMultiTexCoordPointerEXT = _link_function('glMultiTexCoordPointerEXT', None, [GLenum, GLint, GLenum, GLsizei, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:8051
glMultiTexEnvfEXT = _link_function('glMultiTexEnvfEXT', None, [GLenum, GLenum, GLenum, GLfloat], 'EXT_direct_state_access')

# GL/glext.h:8052
glMultiTexEnvfvEXT = _link_function('glMultiTexEnvfvEXT', None, [GLenum, GLenum, GLenum, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:8053
glMultiTexEnviEXT = _link_function('glMultiTexEnviEXT', None, [GLenum, GLenum, GLenum, GLint], 'EXT_direct_state_access')

# GL/glext.h:8054
glMultiTexEnvivEXT = _link_function('glMultiTexEnvivEXT', None, [GLenum, GLenum, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:8055
glMultiTexGendEXT = _link_function('glMultiTexGendEXT', None, [GLenum, GLenum, GLenum, GLdouble], 'EXT_direct_state_access')

# GL/glext.h:8056
glMultiTexGendvEXT = _link_function('glMultiTexGendvEXT', None, [GLenum, GLenum, GLenum, POINTER(GLdouble)], 'EXT_direct_state_access')

# GL/glext.h:8057
glMultiTexGenfEXT = _link_function('glMultiTexGenfEXT', None, [GLenum, GLenum, GLenum, GLfloat], 'EXT_direct_state_access')

# GL/glext.h:8058
glMultiTexGenfvEXT = _link_function('glMultiTexGenfvEXT', None, [GLenum, GLenum, GLenum, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:8059
glMultiTexGeniEXT = _link_function('glMultiTexGeniEXT', None, [GLenum, GLenum, GLenum, GLint], 'EXT_direct_state_access')

# GL/glext.h:8060
glMultiTexGenivEXT = _link_function('glMultiTexGenivEXT', None, [GLenum, GLenum, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:8061
glGetMultiTexEnvfvEXT = _link_function('glGetMultiTexEnvfvEXT', None, [GLenum, GLenum, GLenum, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:8062
glGetMultiTexEnvivEXT = _link_function('glGetMultiTexEnvivEXT', None, [GLenum, GLenum, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:8063
glGetMultiTexGendvEXT = _link_function('glGetMultiTexGendvEXT', None, [GLenum, GLenum, GLenum, POINTER(GLdouble)], 'EXT_direct_state_access')

# GL/glext.h:8064
glGetMultiTexGenfvEXT = _link_function('glGetMultiTexGenfvEXT', None, [GLenum, GLenum, GLenum, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:8065
glGetMultiTexGenivEXT = _link_function('glGetMultiTexGenivEXT', None, [GLenum, GLenum, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:8066
glGetFloatIndexedvEXT = _link_function('glGetFloatIndexedvEXT', None, [GLenum, GLuint, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:8067
glGetDoubleIndexedvEXT = _link_function('glGetDoubleIndexedvEXT', None, [GLenum, GLuint, POINTER(GLdouble)], 'EXT_direct_state_access')

# GL/glext.h:8068
glGetPointerIndexedvEXT = _link_function('glGetPointerIndexedvEXT', None, [GLenum, GLuint, POINTER(POINTER(GLvoid))], 'EXT_direct_state_access')

# GL/glext.h:8069
glCompressedTextureImage3DEXT = _link_function('glCompressedTextureImage3DEXT', None, [GLuint, GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:8070
glCompressedTextureImage2DEXT = _link_function('glCompressedTextureImage2DEXT', None, [GLuint, GLenum, GLint, GLenum, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:8071
glCompressedTextureImage1DEXT = _link_function('glCompressedTextureImage1DEXT', None, [GLuint, GLenum, GLint, GLenum, GLsizei, GLint, GLsizei, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:8072
glCompressedTextureSubImage3DEXT = _link_function('glCompressedTextureSubImage3DEXT', None, [GLuint, GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:8073
glCompressedTextureSubImage2DEXT = _link_function('glCompressedTextureSubImage2DEXT', None, [GLuint, GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:8074
glCompressedTextureSubImage1DEXT = _link_function('glCompressedTextureSubImage1DEXT', None, [GLuint, GLenum, GLint, GLint, GLsizei, GLenum, GLsizei, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:8075
glGetCompressedTextureImageEXT = _link_function('glGetCompressedTextureImageEXT', None, [GLuint, GLenum, GLint, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:8076
glCompressedMultiTexImage3DEXT = _link_function('glCompressedMultiTexImage3DEXT', None, [GLenum, GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:8077
glCompressedMultiTexImage2DEXT = _link_function('glCompressedMultiTexImage2DEXT', None, [GLenum, GLenum, GLint, GLenum, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:8078
glCompressedMultiTexImage1DEXT = _link_function('glCompressedMultiTexImage1DEXT', None, [GLenum, GLenum, GLint, GLenum, GLsizei, GLint, GLsizei, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:8079
glCompressedMultiTexSubImage3DEXT = _link_function('glCompressedMultiTexSubImage3DEXT', None, [GLenum, GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:8080
glCompressedMultiTexSubImage2DEXT = _link_function('glCompressedMultiTexSubImage2DEXT', None, [GLenum, GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:8081
glCompressedMultiTexSubImage1DEXT = _link_function('glCompressedMultiTexSubImage1DEXT', None, [GLenum, GLenum, GLint, GLint, GLsizei, GLenum, GLsizei, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:8082
glGetCompressedMultiTexImageEXT = _link_function('glGetCompressedMultiTexImageEXT', None, [GLenum, GLenum, GLint, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:8083
glNamedProgramStringEXT = _link_function('glNamedProgramStringEXT', None, [GLuint, GLenum, GLenum, GLsizei, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:8084
glNamedProgramLocalParameter4dEXT = _link_function('glNamedProgramLocalParameter4dEXT', None, [GLuint, GLenum, GLuint, GLdouble, GLdouble, GLdouble, GLdouble], 'EXT_direct_state_access')

# GL/glext.h:8085
glNamedProgramLocalParameter4dvEXT = _link_function('glNamedProgramLocalParameter4dvEXT', None, [GLuint, GLenum, GLuint, POINTER(GLdouble)], 'EXT_direct_state_access')

# GL/glext.h:8086
glNamedProgramLocalParameter4fEXT = _link_function('glNamedProgramLocalParameter4fEXT', None, [GLuint, GLenum, GLuint, GLfloat, GLfloat, GLfloat, GLfloat], 'EXT_direct_state_access')

# GL/glext.h:8087
glNamedProgramLocalParameter4fvEXT = _link_function('glNamedProgramLocalParameter4fvEXT', None, [GLuint, GLenum, GLuint, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:8088
glGetNamedProgramLocalParameterdvEXT = _link_function('glGetNamedProgramLocalParameterdvEXT', None, [GLuint, GLenum, GLuint, POINTER(GLdouble)], 'EXT_direct_state_access')

# GL/glext.h:8089
glGetNamedProgramLocalParameterfvEXT = _link_function('glGetNamedProgramLocalParameterfvEXT', None, [GLuint, GLenum, GLuint, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:8090
glGetNamedProgramivEXT = _link_function('glGetNamedProgramivEXT', None, [GLuint, GLenum, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:8091
glGetNamedProgramStringEXT = _link_function('glGetNamedProgramStringEXT', None, [GLuint, GLenum, GLenum, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:8092
glNamedProgramLocalParameters4fvEXT = _link_function('glNamedProgramLocalParameters4fvEXT', None, [GLuint, GLenum, GLuint, GLsizei, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:8093
glNamedProgramLocalParameterI4iEXT = _link_function('glNamedProgramLocalParameterI4iEXT', None, [GLuint, GLenum, GLuint, GLint, GLint, GLint, GLint], 'EXT_direct_state_access')

# GL/glext.h:8094
glNamedProgramLocalParameterI4ivEXT = _link_function('glNamedProgramLocalParameterI4ivEXT', None, [GLuint, GLenum, GLuint, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:8095
glNamedProgramLocalParametersI4ivEXT = _link_function('glNamedProgramLocalParametersI4ivEXT', None, [GLuint, GLenum, GLuint, GLsizei, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:8096
glNamedProgramLocalParameterI4uiEXT = _link_function('glNamedProgramLocalParameterI4uiEXT', None, [GLuint, GLenum, GLuint, GLuint, GLuint, GLuint, GLuint], 'EXT_direct_state_access')

# GL/glext.h:8097
glNamedProgramLocalParameterI4uivEXT = _link_function('glNamedProgramLocalParameterI4uivEXT', None, [GLuint, GLenum, GLuint, POINTER(GLuint)], 'EXT_direct_state_access')

# GL/glext.h:8098
glNamedProgramLocalParametersI4uivEXT = _link_function('glNamedProgramLocalParametersI4uivEXT', None, [GLuint, GLenum, GLuint, GLsizei, POINTER(GLuint)], 'EXT_direct_state_access')

# GL/glext.h:8099
glGetNamedProgramLocalParameterIivEXT = _link_function('glGetNamedProgramLocalParameterIivEXT', None, [GLuint, GLenum, GLuint, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:8100
glGetNamedProgramLocalParameterIuivEXT = _link_function('glGetNamedProgramLocalParameterIuivEXT', None, [GLuint, GLenum, GLuint, POINTER(GLuint)], 'EXT_direct_state_access')

# GL/glext.h:8101
glTextureParameterIivEXT = _link_function('glTextureParameterIivEXT', None, [GLuint, GLenum, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:8102
glTextureParameterIuivEXT = _link_function('glTextureParameterIuivEXT', None, [GLuint, GLenum, GLenum, POINTER(GLuint)], 'EXT_direct_state_access')

# GL/glext.h:8103
glGetTextureParameterIivEXT = _link_function('glGetTextureParameterIivEXT', None, [GLuint, GLenum, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:8104
glGetTextureParameterIuivEXT = _link_function('glGetTextureParameterIuivEXT', None, [GLuint, GLenum, GLenum, POINTER(GLuint)], 'EXT_direct_state_access')

# GL/glext.h:8105
glMultiTexParameterIivEXT = _link_function('glMultiTexParameterIivEXT', None, [GLenum, GLenum, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:8106
glMultiTexParameterIuivEXT = _link_function('glMultiTexParameterIuivEXT', None, [GLenum, GLenum, GLenum, POINTER(GLuint)], 'EXT_direct_state_access')

# GL/glext.h:8107
glGetMultiTexParameterIivEXT = _link_function('glGetMultiTexParameterIivEXT', None, [GLenum, GLenum, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:8108
glGetMultiTexParameterIuivEXT = _link_function('glGetMultiTexParameterIuivEXT', None, [GLenum, GLenum, GLenum, POINTER(GLuint)], 'EXT_direct_state_access')

# GL/glext.h:8109
glProgramUniform1fEXT = _link_function('glProgramUniform1fEXT', None, [GLuint, GLint, GLfloat], 'EXT_direct_state_access')

# GL/glext.h:8110
glProgramUniform2fEXT = _link_function('glProgramUniform2fEXT', None, [GLuint, GLint, GLfloat, GLfloat], 'EXT_direct_state_access')

# GL/glext.h:8111
glProgramUniform3fEXT = _link_function('glProgramUniform3fEXT', None, [GLuint, GLint, GLfloat, GLfloat, GLfloat], 'EXT_direct_state_access')

# GL/glext.h:8112
glProgramUniform4fEXT = _link_function('glProgramUniform4fEXT', None, [GLuint, GLint, GLfloat, GLfloat, GLfloat, GLfloat], 'EXT_direct_state_access')

# GL/glext.h:8113
glProgramUniform1iEXT = _link_function('glProgramUniform1iEXT', None, [GLuint, GLint, GLint], 'EXT_direct_state_access')

# GL/glext.h:8114
glProgramUniform2iEXT = _link_function('glProgramUniform2iEXT', None, [GLuint, GLint, GLint, GLint], 'EXT_direct_state_access')

# GL/glext.h:8115
glProgramUniform3iEXT = _link_function('glProgramUniform3iEXT', None, [GLuint, GLint, GLint, GLint, GLint], 'EXT_direct_state_access')

# GL/glext.h:8116
glProgramUniform4iEXT = _link_function('glProgramUniform4iEXT', None, [GLuint, GLint, GLint, GLint, GLint, GLint], 'EXT_direct_state_access')

# GL/glext.h:8117
glProgramUniform1fvEXT = _link_function('glProgramUniform1fvEXT', None, [GLuint, GLint, GLsizei, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:8118
glProgramUniform2fvEXT = _link_function('glProgramUniform2fvEXT', None, [GLuint, GLint, GLsizei, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:8119
glProgramUniform3fvEXT = _link_function('glProgramUniform3fvEXT', None, [GLuint, GLint, GLsizei, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:8120
glProgramUniform4fvEXT = _link_function('glProgramUniform4fvEXT', None, [GLuint, GLint, GLsizei, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:8121
glProgramUniform1ivEXT = _link_function('glProgramUniform1ivEXT', None, [GLuint, GLint, GLsizei, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:8122
glProgramUniform2ivEXT = _link_function('glProgramUniform2ivEXT', None, [GLuint, GLint, GLsizei, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:8123
glProgramUniform3ivEXT = _link_function('glProgramUniform3ivEXT', None, [GLuint, GLint, GLsizei, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:8124
glProgramUniform4ivEXT = _link_function('glProgramUniform4ivEXT', None, [GLuint, GLint, GLsizei, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:8125
glProgramUniformMatrix2fvEXT = _link_function('glProgramUniformMatrix2fvEXT', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:8126
glProgramUniformMatrix3fvEXT = _link_function('glProgramUniformMatrix3fvEXT', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:8127
glProgramUniformMatrix4fvEXT = _link_function('glProgramUniformMatrix4fvEXT', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:8128
glProgramUniformMatrix2x3fvEXT = _link_function('glProgramUniformMatrix2x3fvEXT', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:8129
glProgramUniformMatrix3x2fvEXT = _link_function('glProgramUniformMatrix3x2fvEXT', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:8130
glProgramUniformMatrix2x4fvEXT = _link_function('glProgramUniformMatrix2x4fvEXT', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:8131
glProgramUniformMatrix4x2fvEXT = _link_function('glProgramUniformMatrix4x2fvEXT', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:8132
glProgramUniformMatrix3x4fvEXT = _link_function('glProgramUniformMatrix3x4fvEXT', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:8133
glProgramUniformMatrix4x3fvEXT = _link_function('glProgramUniformMatrix4x3fvEXT', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:8134
glProgramUniform1uiEXT = _link_function('glProgramUniform1uiEXT', None, [GLuint, GLint, GLuint], 'EXT_direct_state_access')

# GL/glext.h:8135
glProgramUniform2uiEXT = _link_function('glProgramUniform2uiEXT', None, [GLuint, GLint, GLuint, GLuint], 'EXT_direct_state_access')

# GL/glext.h:8136
glProgramUniform3uiEXT = _link_function('glProgramUniform3uiEXT', None, [GLuint, GLint, GLuint, GLuint, GLuint], 'EXT_direct_state_access')

# GL/glext.h:8137
glProgramUniform4uiEXT = _link_function('glProgramUniform4uiEXT', None, [GLuint, GLint, GLuint, GLuint, GLuint, GLuint], 'EXT_direct_state_access')

# GL/glext.h:8138
glProgramUniform1uivEXT = _link_function('glProgramUniform1uivEXT', None, [GLuint, GLint, GLsizei, POINTER(GLuint)], 'EXT_direct_state_access')

# GL/glext.h:8139
glProgramUniform2uivEXT = _link_function('glProgramUniform2uivEXT', None, [GLuint, GLint, GLsizei, POINTER(GLuint)], 'EXT_direct_state_access')

# GL/glext.h:8140
glProgramUniform3uivEXT = _link_function('glProgramUniform3uivEXT', None, [GLuint, GLint, GLsizei, POINTER(GLuint)], 'EXT_direct_state_access')

# GL/glext.h:8141
glProgramUniform4uivEXT = _link_function('glProgramUniform4uivEXT', None, [GLuint, GLint, GLsizei, POINTER(GLuint)], 'EXT_direct_state_access')

# GL/glext.h:8142
glNamedBufferDataEXT = _link_function('glNamedBufferDataEXT', None, [GLuint, GLsizeiptr, POINTER(GLvoid), GLenum], 'EXT_direct_state_access')

# GL/glext.h:8143
glNamedBufferSubDataEXT = _link_function('glNamedBufferSubDataEXT', None, [GLuint, GLintptr, GLsizeiptr, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:8144
glMapNamedBufferEXT = _link_function('glMapNamedBufferEXT', POINTER(GLvoid), [GLuint, GLenum], 'EXT_direct_state_access')

# GL/glext.h:8145
glUnmapNamedBufferEXT = _link_function('glUnmapNamedBufferEXT', GLboolean, [GLuint], 'EXT_direct_state_access')

# GL/glext.h:8146
glGetNamedBufferParameterivEXT = _link_function('glGetNamedBufferParameterivEXT', None, [GLuint, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:8147
glGetNamedBufferPointervEXT = _link_function('glGetNamedBufferPointervEXT', None, [GLuint, GLenum, POINTER(POINTER(GLvoid))], 'EXT_direct_state_access')

# GL/glext.h:8148
glGetNamedBufferSubDataEXT = _link_function('glGetNamedBufferSubDataEXT', None, [GLuint, GLintptr, GLsizeiptr, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:8149
glTextureBufferEXT = _link_function('glTextureBufferEXT', None, [GLuint, GLenum, GLenum, GLuint], 'EXT_direct_state_access')

# GL/glext.h:8150
glMultiTexBufferEXT = _link_function('glMultiTexBufferEXT', None, [GLenum, GLenum, GLenum, GLuint], 'EXT_direct_state_access')

# GL/glext.h:8151
glNamedRenderbufferStorageEXT = _link_function('glNamedRenderbufferStorageEXT', None, [GLuint, GLenum, GLsizei, GLsizei], 'EXT_direct_state_access')

# GL/glext.h:8152
glGetNamedRenderbufferParameterivEXT = _link_function('glGetNamedRenderbufferParameterivEXT', None, [GLuint, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:8153
glCheckNamedFramebufferStatusEXT = _link_function('glCheckNamedFramebufferStatusEXT', GLenum, [GLuint, GLenum], 'EXT_direct_state_access')

# GL/glext.h:8154
glNamedFramebufferTexture1DEXT = _link_function('glNamedFramebufferTexture1DEXT', None, [GLuint, GLenum, GLenum, GLuint, GLint], 'EXT_direct_state_access')

# GL/glext.h:8155
glNamedFramebufferTexture2DEXT = _link_function('glNamedFramebufferTexture2DEXT', None, [GLuint, GLenum, GLenum, GLuint, GLint], 'EXT_direct_state_access')

# GL/glext.h:8156
glNamedFramebufferTexture3DEXT = _link_function('glNamedFramebufferTexture3DEXT', None, [GLuint, GLenum, GLenum, GLuint, GLint, GLint], 'EXT_direct_state_access')

# GL/glext.h:8157
glNamedFramebufferRenderbufferEXT = _link_function('glNamedFramebufferRenderbufferEXT', None, [GLuint, GLenum, GLenum, GLuint], 'EXT_direct_state_access')

# GL/glext.h:8158
glGetNamedFramebufferAttachmentParameterivEXT = _link_function('glGetNamedFramebufferAttachmentParameterivEXT', None, [GLuint, GLenum, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:8159
glGenerateTextureMipmapEXT = _link_function('glGenerateTextureMipmapEXT', None, [GLuint, GLenum], 'EXT_direct_state_access')

# GL/glext.h:8160
glGenerateMultiTexMipmapEXT = _link_function('glGenerateMultiTexMipmapEXT', None, [GLenum, GLenum], 'EXT_direct_state_access')

# GL/glext.h:8161
glFramebufferDrawBufferEXT = _link_function('glFramebufferDrawBufferEXT', None, [GLuint, GLenum], 'EXT_direct_state_access')

# GL/glext.h:8162
glFramebufferDrawBuffersEXT = _link_function('glFramebufferDrawBuffersEXT', None, [GLuint, GLsizei, POINTER(GLenum)], 'EXT_direct_state_access')

# GL/glext.h:8163
glFramebufferReadBufferEXT = _link_function('glFramebufferReadBufferEXT', None, [GLuint, GLenum], 'EXT_direct_state_access')

# GL/glext.h:8164
glGetFramebufferParameterivEXT = _link_function('glGetFramebufferParameterivEXT', None, [GLuint, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:8165
glNamedRenderbufferStorageMultisampleEXT = _link_function('glNamedRenderbufferStorageMultisampleEXT', None, [GLuint, GLsizei, GLenum, GLsizei, GLsizei], 'EXT_direct_state_access')

# GL/glext.h:8166
glNamedRenderbufferStorageMultisampleCoverageEXT = _link_function('glNamedRenderbufferStorageMultisampleCoverageEXT', None, [GLuint, GLsizei, GLsizei, GLenum, GLsizei, GLsizei], 'EXT_direct_state_access')

# GL/glext.h:8167
glNamedFramebufferTextureEXT = _link_function('glNamedFramebufferTextureEXT', None, [GLuint, GLenum, GLuint, GLint], 'EXT_direct_state_access')

# GL/glext.h:8168
glNamedFramebufferTextureLayerEXT = _link_function('glNamedFramebufferTextureLayerEXT', None, [GLuint, GLenum, GLuint, GLint, GLint], 'EXT_direct_state_access')

# GL/glext.h:8169
glNamedFramebufferTextureFaceEXT = _link_function('glNamedFramebufferTextureFaceEXT', None, [GLuint, GLenum, GLuint, GLint, GLenum], 'EXT_direct_state_access')

# GL/glext.h:8170
glTextureRenderbufferEXT = _link_function('glTextureRenderbufferEXT', None, [GLuint, GLenum, GLuint], 'EXT_direct_state_access')

# GL/glext.h:8171
glMultiTexRenderbufferEXT = _link_function('glMultiTexRenderbufferEXT', None, [GLenum, GLenum, GLuint], 'EXT_direct_state_access')

PFNGLCLIENTATTRIBDEFAULTEXTPROC = CFUNCTYPE(None, GLbitfield) 	# GL/glext.h:8173
PFNGLPUSHCLIENTATTRIBDEFAULTEXTPROC = CFUNCTYPE(None, GLbitfield) 	# GL/glext.h:8174
PFNGLMATRIXLOADFEXTPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:8175
PFNGLMATRIXLOADDEXTPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:8176
PFNGLMATRIXMULTFEXTPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:8177
PFNGLMATRIXMULTDEXTPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:8178
PFNGLMATRIXLOADIDENTITYEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:8179
PFNGLMATRIXROTATEFEXTPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:8180
PFNGLMATRIXROTATEDEXTPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:8181
PFNGLMATRIXSCALEFEXTPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:8182
PFNGLMATRIXSCALEDEXTPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:8183
PFNGLMATRIXTRANSLATEFEXTPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:8184
PFNGLMATRIXTRANSLATEDEXTPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:8185
PFNGLMATRIXFRUSTUMEXTPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:8186
PFNGLMATRIXORTHOEXTPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:8187
PFNGLMATRIXPOPEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:8188
PFNGLMATRIXPUSHEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:8189
PFNGLMATRIXLOADTRANSPOSEFEXTPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:8190
PFNGLMATRIXLOADTRANSPOSEDEXTPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:8191
PFNGLMATRIXMULTTRANSPOSEFEXTPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:8192
PFNGLMATRIXMULTTRANSPOSEDEXTPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:8193
PFNGLTEXTUREPARAMETERFEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, GLfloat) 	# GL/glext.h:8194
PFNGLTEXTUREPARAMETERFVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:8195
PFNGLTEXTUREPARAMETERIEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, GLint) 	# GL/glext.h:8196
PFNGLTEXTUREPARAMETERIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:8197
PFNGLTEXTUREIMAGE1DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLenum, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:8198
PFNGLTEXTUREIMAGE2DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLenum, GLsizei, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:8199
PFNGLTEXTURESUBIMAGE1DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLint, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:8200
PFNGLTEXTURESUBIMAGE2DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:8201
PFNGLCOPYTEXTUREIMAGE1DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLenum, GLint, GLint, GLsizei, GLint) 	# GL/glext.h:8202
PFNGLCOPYTEXTUREIMAGE2DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLenum, GLint, GLint, GLsizei, GLsizei, GLint) 	# GL/glext.h:8203
PFNGLCOPYTEXTURESUBIMAGE1DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLint, GLint, GLint, GLsizei) 	# GL/glext.h:8204
PFNGLCOPYTEXTURESUBIMAGE2DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei) 	# GL/glext.h:8205
PFNGLGETTEXTUREIMAGEEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:8206
PFNGLGETTEXTUREPARAMETERFVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:8207
PFNGLGETTEXTUREPARAMETERIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:8208
PFNGLGETTEXTURELEVELPARAMETERFVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:8209
PFNGLGETTEXTURELEVELPARAMETERIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLenum, POINTER(GLint)) 	# GL/glext.h:8210
PFNGLTEXTUREIMAGE3DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:8211
PFNGLTEXTURESUBIMAGE3DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:8212
PFNGLCOPYTEXTURESUBIMAGE3DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei) 	# GL/glext.h:8213
PFNGLMULTITEXPARAMETERFEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLfloat) 	# GL/glext.h:8214
PFNGLMULTITEXPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:8215
PFNGLMULTITEXPARAMETERIEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLint) 	# GL/glext.h:8216
PFNGLMULTITEXPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:8217
PFNGLMULTITEXIMAGE1DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLenum, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:8218
PFNGLMULTITEXIMAGE2DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLenum, GLsizei, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:8219
PFNGLMULTITEXSUBIMAGE1DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:8220
PFNGLMULTITEXSUBIMAGE2DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:8221
PFNGLCOPYMULTITEXIMAGE1DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLenum, GLint, GLint, GLsizei, GLint) 	# GL/glext.h:8222
PFNGLCOPYMULTITEXIMAGE2DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLenum, GLint, GLint, GLsizei, GLsizei, GLint) 	# GL/glext.h:8223
PFNGLCOPYMULTITEXSUBIMAGE1DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLint, GLint, GLsizei) 	# GL/glext.h:8224
PFNGLCOPYMULTITEXSUBIMAGE2DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei) 	# GL/glext.h:8225
PFNGLGETMULTITEXIMAGEEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:8226
PFNGLGETMULTITEXPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:8227
PFNGLGETMULTITEXPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:8228
PFNGLGETMULTITEXLEVELPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:8229
PFNGLGETMULTITEXLEVELPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLenum, POINTER(GLint)) 	# GL/glext.h:8230
PFNGLMULTITEXIMAGE3DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:8231
PFNGLMULTITEXSUBIMAGE3DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:8232
PFNGLCOPYMULTITEXSUBIMAGE3DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei) 	# GL/glext.h:8233
PFNGLBINDMULTITEXTUREEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLuint) 	# GL/glext.h:8234
PFNGLENABLECLIENTSTATEINDEXEDEXTPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:8235
PFNGLDISABLECLIENTSTATEINDEXEDEXTPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:8236
PFNGLMULTITEXCOORDPOINTEREXTPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:8237
PFNGLMULTITEXENVFEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLfloat) 	# GL/glext.h:8238
PFNGLMULTITEXENVFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:8239
PFNGLMULTITEXENVIEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLint) 	# GL/glext.h:8240
PFNGLMULTITEXENVIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:8241
PFNGLMULTITEXGENDEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLdouble) 	# GL/glext.h:8242
PFNGLMULTITEXGENDVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLdouble)) 	# GL/glext.h:8243
PFNGLMULTITEXGENFEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLfloat) 	# GL/glext.h:8244
PFNGLMULTITEXGENFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:8245
PFNGLMULTITEXGENIEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLint) 	# GL/glext.h:8246
PFNGLMULTITEXGENIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:8247
PFNGLGETMULTITEXENVFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:8248
PFNGLGETMULTITEXENVIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:8249
PFNGLGETMULTITEXGENDVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLdouble)) 	# GL/glext.h:8250
PFNGLGETMULTITEXGENFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:8251
PFNGLGETMULTITEXGENIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:8252
PFNGLGETFLOATINDEXEDVEXTPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLfloat)) 	# GL/glext.h:8253
PFNGLGETDOUBLEINDEXEDVEXTPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLdouble)) 	# GL/glext.h:8254
PFNGLGETPOINTERINDEXEDVEXTPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(POINTER(GLvoid))) 	# GL/glext.h:8255
PFNGLCOMPRESSEDTEXTUREIMAGE3DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:8256
PFNGLCOMPRESSEDTEXTUREIMAGE2DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLenum, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:8257
PFNGLCOMPRESSEDTEXTUREIMAGE1DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLenum, GLsizei, GLint, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:8258
PFNGLCOMPRESSEDTEXTURESUBIMAGE3DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:8259
PFNGLCOMPRESSEDTEXTURESUBIMAGE2DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:8260
PFNGLCOMPRESSEDTEXTURESUBIMAGE1DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLint, GLsizei, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:8261
PFNGLGETCOMPRESSEDTEXTUREIMAGEEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, POINTER(GLvoid)) 	# GL/glext.h:8262
PFNGLCOMPRESSEDMULTITEXIMAGE3DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:8263
PFNGLCOMPRESSEDMULTITEXIMAGE2DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLenum, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:8264
PFNGLCOMPRESSEDMULTITEXIMAGE1DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLenum, GLsizei, GLint, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:8265
PFNGLCOMPRESSEDMULTITEXSUBIMAGE3DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:8266
PFNGLCOMPRESSEDMULTITEXSUBIMAGE2DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:8267
PFNGLCOMPRESSEDMULTITEXSUBIMAGE1DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLsizei, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:8268
PFNGLGETCOMPRESSEDMULTITEXIMAGEEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, POINTER(GLvoid)) 	# GL/glext.h:8269
PFNGLNAMEDPROGRAMSTRINGEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:8270
PFNGLNAMEDPROGRAMLOCALPARAMETER4DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:8271
PFNGLNAMEDPROGRAMLOCALPARAMETER4DVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, POINTER(GLdouble)) 	# GL/glext.h:8272
PFNGLNAMEDPROGRAMLOCALPARAMETER4FEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:8273
PFNGLNAMEDPROGRAMLOCALPARAMETER4FVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, POINTER(GLfloat)) 	# GL/glext.h:8274
PFNGLGETNAMEDPROGRAMLOCALPARAMETERDVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, POINTER(GLdouble)) 	# GL/glext.h:8275
PFNGLGETNAMEDPROGRAMLOCALPARAMETERFVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, POINTER(GLfloat)) 	# GL/glext.h:8276
PFNGLGETNAMEDPROGRAMIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:8277
PFNGLGETNAMEDPROGRAMSTRINGEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:8278
PFNGLNAMEDPROGRAMLOCALPARAMETERS4FVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:8279
PFNGLNAMEDPROGRAMLOCALPARAMETERI4IEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, GLint, GLint, GLint, GLint) 	# GL/glext.h:8280
PFNGLNAMEDPROGRAMLOCALPARAMETERI4IVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, POINTER(GLint)) 	# GL/glext.h:8281
PFNGLNAMEDPROGRAMLOCALPARAMETERSI4IVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, GLsizei, POINTER(GLint)) 	# GL/glext.h:8282
PFNGLNAMEDPROGRAMLOCALPARAMETERI4UIEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:8283
PFNGLNAMEDPROGRAMLOCALPARAMETERI4UIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, POINTER(GLuint)) 	# GL/glext.h:8284
PFNGLNAMEDPROGRAMLOCALPARAMETERSI4UIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:8285
PFNGLGETNAMEDPROGRAMLOCALPARAMETERIIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, POINTER(GLint)) 	# GL/glext.h:8286
PFNGLGETNAMEDPROGRAMLOCALPARAMETERIUIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, POINTER(GLuint)) 	# GL/glext.h:8287
PFNGLTEXTUREPARAMETERIIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:8288
PFNGLTEXTUREPARAMETERIUIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, POINTER(GLuint)) 	# GL/glext.h:8289
PFNGLGETTEXTUREPARAMETERIIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:8290
PFNGLGETTEXTUREPARAMETERIUIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, POINTER(GLuint)) 	# GL/glext.h:8291
PFNGLMULTITEXPARAMETERIIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:8292
PFNGLMULTITEXPARAMETERIUIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLuint)) 	# GL/glext.h:8293
PFNGLGETMULTITEXPARAMETERIIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:8294
PFNGLGETMULTITEXPARAMETERIUIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLuint)) 	# GL/glext.h:8295
PFNGLPROGRAMUNIFORM1FEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLfloat) 	# GL/glext.h:8296
PFNGLPROGRAMUNIFORM2FEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLfloat, GLfloat) 	# GL/glext.h:8297
PFNGLPROGRAMUNIFORM3FEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:8298
PFNGLPROGRAMUNIFORM4FEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:8299
PFNGLPROGRAMUNIFORM1IEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLint) 	# GL/glext.h:8300
PFNGLPROGRAMUNIFORM2IEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLint, GLint) 	# GL/glext.h:8301
PFNGLPROGRAMUNIFORM3IEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLint, GLint, GLint) 	# GL/glext.h:8302
PFNGLPROGRAMUNIFORM4IEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLint, GLint, GLint, GLint) 	# GL/glext.h:8303
PFNGLPROGRAMUNIFORM1FVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:8304
PFNGLPROGRAMUNIFORM2FVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:8305
PFNGLPROGRAMUNIFORM3FVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:8306
PFNGLPROGRAMUNIFORM4FVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:8307
PFNGLPROGRAMUNIFORM1IVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:8308
PFNGLPROGRAMUNIFORM2IVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:8309
PFNGLPROGRAMUNIFORM3IVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:8310
PFNGLPROGRAMUNIFORM4IVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:8311
PFNGLPROGRAMUNIFORMMATRIX2FVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:8312
PFNGLPROGRAMUNIFORMMATRIX3FVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:8313
PFNGLPROGRAMUNIFORMMATRIX4FVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:8314
PFNGLPROGRAMUNIFORMMATRIX2X3FVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:8315
PFNGLPROGRAMUNIFORMMATRIX3X2FVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:8316
PFNGLPROGRAMUNIFORMMATRIX2X4FVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:8317
PFNGLPROGRAMUNIFORMMATRIX4X2FVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:8318
PFNGLPROGRAMUNIFORMMATRIX3X4FVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:8319
PFNGLPROGRAMUNIFORMMATRIX4X3FVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:8320
PFNGLPROGRAMUNIFORM1UIEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLuint) 	# GL/glext.h:8321
PFNGLPROGRAMUNIFORM2UIEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLuint, GLuint) 	# GL/glext.h:8322
PFNGLPROGRAMUNIFORM3UIEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLuint, GLuint, GLuint) 	# GL/glext.h:8323
PFNGLPROGRAMUNIFORM4UIEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:8324
PFNGLPROGRAMUNIFORM1UIVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:8325
PFNGLPROGRAMUNIFORM2UIVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:8326
PFNGLPROGRAMUNIFORM3UIVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:8327
PFNGLPROGRAMUNIFORM4UIVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:8328
PFNGLNAMEDBUFFERDATAEXTPROC = CFUNCTYPE(None, GLuint, GLsizeiptr, POINTER(GLvoid), GLenum) 	# GL/glext.h:8329
PFNGLNAMEDBUFFERSUBDATAEXTPROC = CFUNCTYPE(None, GLuint, GLintptr, GLsizeiptr, POINTER(GLvoid)) 	# GL/glext.h:8330
PFNGLMAPNAMEDBUFFEREXTPROC = CFUNCTYPE(POINTER(GLvoid), GLuint, GLenum) 	# GL/glext.h:8331
PFNGLUNMAPNAMEDBUFFEREXTPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:8332
PFNGLGETNAMEDBUFFERPARAMETERIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:8333
PFNGLGETNAMEDBUFFERPOINTERVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:8334
PFNGLGETNAMEDBUFFERSUBDATAEXTPROC = CFUNCTYPE(None, GLuint, GLintptr, GLsizeiptr, POINTER(GLvoid)) 	# GL/glext.h:8335
PFNGLTEXTUREBUFFEREXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, GLuint) 	# GL/glext.h:8336
PFNGLMULTITEXBUFFEREXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLuint) 	# GL/glext.h:8337
PFNGLNAMEDRENDERBUFFERSTORAGEEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLsizei, GLsizei) 	# GL/glext.h:8338
PFNGLGETNAMEDRENDERBUFFERPARAMETERIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:8339
PFNGLCHECKNAMEDFRAMEBUFFERSTATUSEXTPROC = CFUNCTYPE(GLenum, GLuint, GLenum) 	# GL/glext.h:8340
PFNGLNAMEDFRAMEBUFFERTEXTURE1DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, GLuint, GLint) 	# GL/glext.h:8341
PFNGLNAMEDFRAMEBUFFERTEXTURE2DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, GLuint, GLint) 	# GL/glext.h:8342
PFNGLNAMEDFRAMEBUFFERTEXTURE3DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, GLuint, GLint, GLint) 	# GL/glext.h:8343
PFNGLNAMEDFRAMEBUFFERRENDERBUFFEREXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, GLuint) 	# GL/glext.h:8344
PFNGLGETNAMEDFRAMEBUFFERATTACHMENTPARAMETERIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:8345
PFNGLGENERATETEXTUREMIPMAPEXTPROC = CFUNCTYPE(None, GLuint, GLenum) 	# GL/glext.h:8346
PFNGLGENERATEMULTITEXMIPMAPEXTPROC = CFUNCTYPE(None, GLenum, GLenum) 	# GL/glext.h:8347
PFNGLFRAMEBUFFERDRAWBUFFEREXTPROC = CFUNCTYPE(None, GLuint, GLenum) 	# GL/glext.h:8348
PFNGLFRAMEBUFFERDRAWBUFFERSEXTPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLenum)) 	# GL/glext.h:8349
PFNGLFRAMEBUFFERREADBUFFEREXTPROC = CFUNCTYPE(None, GLuint, GLenum) 	# GL/glext.h:8350
PFNGLGETFRAMEBUFFERPARAMETERIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:8351
PFNGLNAMEDRENDERBUFFERSTORAGEMULTISAMPLEEXTPROC = CFUNCTYPE(None, GLuint, GLsizei, GLenum, GLsizei, GLsizei) 	# GL/glext.h:8352
PFNGLNAMEDRENDERBUFFERSTORAGEMULTISAMPLECOVERAGEEXTPROC = CFUNCTYPE(None, GLuint, GLsizei, GLsizei, GLenum, GLsizei, GLsizei) 	# GL/glext.h:8353
PFNGLNAMEDFRAMEBUFFERTEXTUREEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, GLint) 	# GL/glext.h:8354
PFNGLNAMEDFRAMEBUFFERTEXTURELAYEREXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, GLint, GLint) 	# GL/glext.h:8355
PFNGLNAMEDFRAMEBUFFERTEXTUREFACEEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, GLint, GLenum) 	# GL/glext.h:8356
PFNGLTEXTURERENDERBUFFEREXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint) 	# GL/glext.h:8357
PFNGLMULTITEXRENDERBUFFEREXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLuint) 	# GL/glext.h:8358
# EXT_vertex_array_bgra (GL/glext.h:8361)
GL_EXT_vertex_array_bgra = 1 	# GL/glext.h:8362
# EXT_texture_swizzle (GL/glext.h:8365)
GL_EXT_texture_swizzle = 1 	# GL/glext.h:8366
# NV_explicit_multisample (GL/glext.h:8369)
GL_NV_explicit_multisample = 1 	# GL/glext.h:8370
# GL/glext.h:8372
glGetMultisamplefvNV = _link_function('glGetMultisamplefvNV', None, [GLenum, GLuint, POINTER(GLfloat)], 'NV_explicit_multisample')

# GL/glext.h:8373
glSampleMaskIndexedNV = _link_function('glSampleMaskIndexedNV', None, [GLuint, GLbitfield], 'NV_explicit_multisample')

# GL/glext.h:8374
glTexRenderbufferNV = _link_function('glTexRenderbufferNV', None, [GLenum, GLuint], 'NV_explicit_multisample')

PFNGLGETMULTISAMPLEFVNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLfloat)) 	# GL/glext.h:8376
PFNGLSAMPLEMASKINDEXEDNVPROC = CFUNCTYPE(None, GLuint, GLbitfield) 	# GL/glext.h:8377
PFNGLTEXRENDERBUFFERNVPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:8378
# NV_transform_feedback2 (GL/glext.h:8381)
GL_NV_transform_feedback2 = 1 	# GL/glext.h:8382
# GL/glext.h:8384
glBindTransformFeedbackNV = _link_function('glBindTransformFeedbackNV', None, [GLenum, GLuint], 'NV_transform_feedback2')

# GL/glext.h:8385
glDeleteTransformFeedbacksNV = _link_function('glDeleteTransformFeedbacksNV', None, [GLsizei, POINTER(GLuint)], 'NV_transform_feedback2')

# GL/glext.h:8386
glGenTransformFeedbacksNV = _link_function('glGenTransformFeedbacksNV', None, [GLsizei, POINTER(GLuint)], 'NV_transform_feedback2')

# GL/glext.h:8387
glIsTransformFeedbackNV = _link_function('glIsTransformFeedbackNV', GLboolean, [GLuint], 'NV_transform_feedback2')

# GL/glext.h:8388
glPauseTransformFeedbackNV = _link_function('glPauseTransformFeedbackNV', None, [], 'NV_transform_feedback2')

# GL/glext.h:8389
glResumeTransformFeedbackNV = _link_function('glResumeTransformFeedbackNV', None, [], 'NV_transform_feedback2')

# GL/glext.h:8390
glDrawTransformFeedbackNV = _link_function('glDrawTransformFeedbackNV', None, [GLenum, GLuint], 'NV_transform_feedback2')

PFNGLBINDTRANSFORMFEEDBACKNVPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:8392
PFNGLDELETETRANSFORMFEEDBACKSNVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:8393
PFNGLGENTRANSFORMFEEDBACKSNVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:8394
PFNGLISTRANSFORMFEEDBACKNVPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:8395
PFNGLPAUSETRANSFORMFEEDBACKNVPROC = CFUNCTYPE(None) 	# GL/glext.h:8396
PFNGLRESUMETRANSFORMFEEDBACKNVPROC = CFUNCTYPE(None) 	# GL/glext.h:8397
PFNGLDRAWTRANSFORMFEEDBACKNVPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:8398
# NV_vertex_buffer_unified_memory (GL/glext.h:8401)
GL_NV_vertex_buffer_unified_memory = 1 	# GL/glext.h:8402
# GL/glext.h:8404
glBufferAddressRangeNV = _link_function('glBufferAddressRangeNV', None, [GLenum, GLuint, GLuint64EXT, GLsizeiptr], 'NV_vertex_buffer_unified_memory')

# GL/glext.h:8405
glVertexFormatNV = _link_function('glVertexFormatNV', None, [GLint, GLenum, GLsizei], 'NV_vertex_buffer_unified_memory')

# GL/glext.h:8406
glNormalFormatNV = _link_function('glNormalFormatNV', None, [GLenum, GLsizei], 'NV_vertex_buffer_unified_memory')

# GL/glext.h:8407
glColorFormatNV = _link_function('glColorFormatNV', None, [GLint, GLenum, GLsizei], 'NV_vertex_buffer_unified_memory')

# GL/glext.h:8408
glIndexFormatNV = _link_function('glIndexFormatNV', None, [GLenum, GLsizei], 'NV_vertex_buffer_unified_memory')

# GL/glext.h:8409
glTexCoordFormatNV = _link_function('glTexCoordFormatNV', None, [GLint, GLenum, GLsizei], 'NV_vertex_buffer_unified_memory')

# GL/glext.h:8410
glEdgeFlagFormatNV = _link_function('glEdgeFlagFormatNV', None, [GLsizei], 'NV_vertex_buffer_unified_memory')

# GL/glext.h:8411
glSecondaryColorFormatNV = _link_function('glSecondaryColorFormatNV', None, [GLint, GLenum, GLsizei], 'NV_vertex_buffer_unified_memory')

# GL/glext.h:8412
glFogCoordFormatNV = _link_function('glFogCoordFormatNV', None, [GLenum, GLsizei], 'NV_vertex_buffer_unified_memory')

# GL/glext.h:8413
glVertexAttribFormatNV = _link_function('glVertexAttribFormatNV', None, [GLuint, GLint, GLenum, GLboolean, GLsizei], 'NV_vertex_buffer_unified_memory')

# GL/glext.h:8414
glVertexAttribIFormatNV = _link_function('glVertexAttribIFormatNV', None, [GLuint, GLint, GLenum, GLsizei], 'NV_vertex_buffer_unified_memory')

# GL/glext.h:8415
glGetIntegerui64i_vNV = _link_function('glGetIntegerui64i_vNV', None, [GLenum, GLuint, POINTER(GLuint64EXT)], 'NV_vertex_buffer_unified_memory')

PFNGLBUFFERADDRESSRANGENVPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint64EXT, GLsizeiptr) 	# GL/glext.h:8417
PFNGLVERTEXFORMATNVPROC = CFUNCTYPE(None, GLint, GLenum, GLsizei) 	# GL/glext.h:8418
PFNGLNORMALFORMATNVPROC = CFUNCTYPE(None, GLenum, GLsizei) 	# GL/glext.h:8419
PFNGLCOLORFORMATNVPROC = CFUNCTYPE(None, GLint, GLenum, GLsizei) 	# GL/glext.h:8420
PFNGLINDEXFORMATNVPROC = CFUNCTYPE(None, GLenum, GLsizei) 	# GL/glext.h:8421
PFNGLTEXCOORDFORMATNVPROC = CFUNCTYPE(None, GLint, GLenum, GLsizei) 	# GL/glext.h:8422
PFNGLEDGEFLAGFORMATNVPROC = CFUNCTYPE(None, GLsizei) 	# GL/glext.h:8423
PFNGLSECONDARYCOLORFORMATNVPROC = CFUNCTYPE(None, GLint, GLenum, GLsizei) 	# GL/glext.h:8424
PFNGLFOGCOORDFORMATNVPROC = CFUNCTYPE(None, GLenum, GLsizei) 	# GL/glext.h:8425
PFNGLVERTEXATTRIBFORMATNVPROC = CFUNCTYPE(None, GLuint, GLint, GLenum, GLboolean, GLsizei) 	# GL/glext.h:8426
PFNGLVERTEXATTRIBIFORMATNVPROC = CFUNCTYPE(None, GLuint, GLint, GLenum, GLsizei) 	# GL/glext.h:8427
PFNGLGETINTEGERUI64I_VNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLuint64EXT)) 	# GL/glext.h:8428
# NV_shader_buffer_load (GL/glext.h:8432)
GL_NV_shader_buffer_load = 1 	# GL/glext.h:8433
# GL/glext.h:8435
glGetBufferParameterui64vNV = _link_function('glGetBufferParameterui64vNV', None, [GLenum, GLenum, POINTER(GLuint64EXT)], 'NV_shader_buffer_load')

# GL/glext.h:8436
glGetIntegerui64vNV = _link_function('glGetIntegerui64vNV', None, [GLenum, POINTER(GLuint64EXT)], 'NV_shader_buffer_load')

# GL/glext.h:8437
glGetNamedBufferParameterui64vNV = _link_function('glGetNamedBufferParameterui64vNV', None, [GLuint, GLenum, POINTER(GLuint64EXT)], 'NV_shader_buffer_load')

# GL/glext.h:8438
glIsBufferResidentNV = _link_function('glIsBufferResidentNV', GLboolean, [GLenum], 'NV_shader_buffer_load')

# GL/glext.h:8439
glIsNamedBufferResidentNV = _link_function('glIsNamedBufferResidentNV', GLboolean, [GLuint], 'NV_shader_buffer_load')

# GL/glext.h:8440
glMakeBufferNonResidentNV = _link_function('glMakeBufferNonResidentNV', None, [GLenum], 'NV_shader_buffer_load')

# GL/glext.h:8441
glMakeBufferResidentNV = _link_function('glMakeBufferResidentNV', None, [GLenum, GLenum], 'NV_shader_buffer_load')

# GL/glext.h:8442
glMakeNamedBufferNonResidentNV = _link_function('glMakeNamedBufferNonResidentNV', None, [GLuint], 'NV_shader_buffer_load')

# GL/glext.h:8443
glMakeNamedBufferResidentNV = _link_function('glMakeNamedBufferResidentNV', None, [GLuint, GLenum], 'NV_shader_buffer_load')

# GL/glext.h:8444
glUniformui64NV = _link_function('glUniformui64NV', None, [GLint, GLuint64EXT], 'NV_shader_buffer_load')

# GL/glext.h:8445
glUniformui64vNV = _link_function('glUniformui64vNV', None, [GLint, GLsizei, POINTER(GLuint64EXT)], 'NV_shader_buffer_load')

# GL/glext.h:8446
glGetUniformui64vNV = _link_function('glGetUniformui64vNV', None, [GLuint, GLint, POINTER(GLuint64EXT)], 'NV_shader_buffer_load')

# GL/glext.h:8447
glProgramUniformui64NV = _link_function('glProgramUniformui64NV', None, [GLuint, GLint, GLuint64EXT], 'NV_shader_buffer_load')

# GL/glext.h:8448
glProgramUniformui64vNV = _link_function('glProgramUniformui64vNV', None, [GLuint, GLint, GLsizei, POINTER(GLuint64EXT)], 'NV_shader_buffer_load')

PFNGLGETBUFFERPARAMETERUI64VNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLuint64EXT)) 	# GL/glext.h:8450
PFNGLGETINTEGERUI64VNVPROC = CFUNCTYPE(None, GLenum, POINTER(GLuint64EXT)) 	# GL/glext.h:8451
PFNGLGETNAMEDBUFFERPARAMETERUI64VNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLuint64EXT)) 	# GL/glext.h:8452
PFNGLISBUFFERRESIDENTNVPROC = CFUNCTYPE(GLboolean, GLenum) 	# GL/glext.h:8453
PFNGLISNAMEDBUFFERRESIDENTNVPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:8454
PFNGLMAKEBUFFERNONRESIDENTNVPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:8455
PFNGLMAKEBUFFERRESIDENTNVPROC = CFUNCTYPE(None, GLenum, GLenum) 	# GL/glext.h:8456
PFNGLMAKENAMEDBUFFERNONRESIDENTNVPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:8457
PFNGLMAKENAMEDBUFFERRESIDENTNVPROC = CFUNCTYPE(None, GLuint, GLenum) 	# GL/glext.h:8458
PFNGLUNIFORMUI64NVPROC = CFUNCTYPE(None, GLint, GLuint64EXT) 	# GL/glext.h:8459
PFNGLUNIFORMUI64VNVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLuint64EXT)) 	# GL/glext.h:8460
PFNGLGETUNIFORMUI64VNVPROC = CFUNCTYPE(None, GLuint, GLint, POINTER(GLuint64EXT)) 	# GL/glext.h:8461
PFNGLPROGRAMUNIFORMUI64NVPROC = CFUNCTYPE(None, GLuint, GLint, GLuint64EXT) 	# GL/glext.h:8462
PFNGLPROGRAMUNIFORMUI64VNVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLuint64EXT)) 	# GL/glext.h:8463
# MESA_packed_depth_stencil (/usr/include/GL/gl.h:2065)
# ATI_blend_equation_separate (/usr/include/GL/gl.h:2077)
# OES_EGL_image (/usr/include/GL/gl.h:2089)
# OES_EGL_image (/usr/include/GL/gl.h:2093)

__all__ = ['GL_GLEXT_VERSION', 'GL_BLEND_DST_RGB', 'GL_BLEND_SRC_RGB',
'GL_BLEND_DST_ALPHA', 'GL_BLEND_SRC_ALPHA', 'GL_POINT_SIZE_MIN',
'GL_POINT_SIZE_MAX', 'GL_POINT_FADE_THRESHOLD_SIZE',
'GL_POINT_DISTANCE_ATTENUATION', 'GL_GENERATE_MIPMAP',
'GL_GENERATE_MIPMAP_HINT', 'GL_DEPTH_COMPONENT16', 'GL_DEPTH_COMPONENT24',
'GL_DEPTH_COMPONENT32', 'GL_MIRRORED_REPEAT', 'GL_FOG_COORDINATE_SOURCE',
'GL_FOG_COORDINATE', 'GL_FRAGMENT_DEPTH', 'GL_CURRENT_FOG_COORDINATE',
'GL_FOG_COORDINATE_ARRAY_TYPE', 'GL_FOG_COORDINATE_ARRAY_STRIDE',
'GL_FOG_COORDINATE_ARRAY_POINTER', 'GL_FOG_COORDINATE_ARRAY', 'GL_COLOR_SUM',
'GL_CURRENT_SECONDARY_COLOR', 'GL_SECONDARY_COLOR_ARRAY_SIZE',
'GL_SECONDARY_COLOR_ARRAY_TYPE', 'GL_SECONDARY_COLOR_ARRAY_STRIDE',
'GL_SECONDARY_COLOR_ARRAY_POINTER', 'GL_SECONDARY_COLOR_ARRAY',
'GL_MAX_TEXTURE_LOD_BIAS', 'GL_TEXTURE_FILTER_CONTROL', 'GL_TEXTURE_LOD_BIAS',
'GL_INCR_WRAP', 'GL_DECR_WRAP', 'GL_TEXTURE_DEPTH_SIZE',
'GL_DEPTH_TEXTURE_MODE', 'GL_TEXTURE_COMPARE_MODE', 'GL_TEXTURE_COMPARE_FUNC',
'GL_COMPARE_R_TO_TEXTURE', 'GL_BUFFER_SIZE', 'GL_BUFFER_USAGE',
'GL_QUERY_COUNTER_BITS', 'GL_CURRENT_QUERY', 'GL_QUERY_RESULT',
'GL_QUERY_RESULT_AVAILABLE', 'GL_ARRAY_BUFFER', 'GL_ELEMENT_ARRAY_BUFFER',
'GL_ARRAY_BUFFER_BINDING', 'GL_ELEMENT_ARRAY_BUFFER_BINDING',
'GL_VERTEX_ARRAY_BUFFER_BINDING', 'GL_NORMAL_ARRAY_BUFFER_BINDING',
'GL_COLOR_ARRAY_BUFFER_BINDING', 'GL_INDEX_ARRAY_BUFFER_BINDING',
'GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING', 'GL_EDGE_FLAG_ARRAY_BUFFER_BINDING',
'GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING',
'GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING', 'GL_WEIGHT_ARRAY_BUFFER_BINDING',
'GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING', 'GL_READ_ONLY', 'GL_WRITE_ONLY',
'GL_READ_WRITE', 'GL_BUFFER_ACCESS', 'GL_BUFFER_MAPPED',
'GL_BUFFER_MAP_POINTER', 'GL_STREAM_DRAW', 'GL_STREAM_READ', 'GL_STREAM_COPY',
'GL_STATIC_DRAW', 'GL_STATIC_READ', 'GL_STATIC_COPY', 'GL_DYNAMIC_DRAW',
'GL_DYNAMIC_READ', 'GL_DYNAMIC_COPY', 'GL_SAMPLES_PASSED', 'GL_FOG_COORD_SRC',
'GL_FOG_COORD', 'GL_CURRENT_FOG_COORD', 'GL_FOG_COORD_ARRAY_TYPE',
'GL_FOG_COORD_ARRAY_STRIDE', 'GL_FOG_COORD_ARRAY_POINTER',
'GL_FOG_COORD_ARRAY', 'GL_FOG_COORD_ARRAY_BUFFER_BINDING', 'GL_SRC0_RGB',
'GL_SRC1_RGB', 'GL_SRC2_RGB', 'GL_SRC0_ALPHA', 'GL_SRC1_ALPHA',
'GL_SRC2_ALPHA', 'GL_BLEND_EQUATION_RGB', 'GL_VERTEX_ATTRIB_ARRAY_ENABLED',
'GL_VERTEX_ATTRIB_ARRAY_SIZE', 'GL_VERTEX_ATTRIB_ARRAY_STRIDE',
'GL_VERTEX_ATTRIB_ARRAY_TYPE', 'GL_CURRENT_VERTEX_ATTRIB',
'GL_VERTEX_PROGRAM_POINT_SIZE', 'GL_VERTEX_PROGRAM_TWO_SIDE',
'GL_VERTEX_ATTRIB_ARRAY_POINTER', 'GL_STENCIL_BACK_FUNC',
'GL_STENCIL_BACK_FAIL', 'GL_STENCIL_BACK_PASS_DEPTH_FAIL',
'GL_STENCIL_BACK_PASS_DEPTH_PASS', 'GL_MAX_DRAW_BUFFERS', 'GL_DRAW_BUFFER0',
'GL_DRAW_BUFFER1', 'GL_DRAW_BUFFER2', 'GL_DRAW_BUFFER3', 'GL_DRAW_BUFFER4',
'GL_DRAW_BUFFER5', 'GL_DRAW_BUFFER6', 'GL_DRAW_BUFFER7', 'GL_DRAW_BUFFER8',
'GL_DRAW_BUFFER9', 'GL_DRAW_BUFFER10', 'GL_DRAW_BUFFER11', 'GL_DRAW_BUFFER12',
'GL_DRAW_BUFFER13', 'GL_DRAW_BUFFER14', 'GL_DRAW_BUFFER15',
'GL_BLEND_EQUATION_ALPHA', 'GL_POINT_SPRITE', 'GL_COORD_REPLACE',
'GL_MAX_VERTEX_ATTRIBS', 'GL_VERTEX_ATTRIB_ARRAY_NORMALIZED',
'GL_MAX_TEXTURE_COORDS', 'GL_MAX_TEXTURE_IMAGE_UNITS', 'GL_FRAGMENT_SHADER',
'GL_VERTEX_SHADER', 'GL_MAX_FRAGMENT_UNIFORM_COMPONENTS',
'GL_MAX_VERTEX_UNIFORM_COMPONENTS', 'GL_MAX_VARYING_FLOATS',
'GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS', 'GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS',
'GL_SHADER_TYPE', 'GL_FLOAT_VEC2', 'GL_FLOAT_VEC3', 'GL_FLOAT_VEC4',
'GL_INT_VEC2', 'GL_INT_VEC3', 'GL_INT_VEC4', 'GL_BOOL', 'GL_BOOL_VEC2',
'GL_BOOL_VEC3', 'GL_BOOL_VEC4', 'GL_FLOAT_MAT2', 'GL_FLOAT_MAT3',
'GL_FLOAT_MAT4', 'GL_SAMPLER_1D', 'GL_SAMPLER_2D', 'GL_SAMPLER_3D',
'GL_SAMPLER_CUBE', 'GL_SAMPLER_1D_SHADOW', 'GL_SAMPLER_2D_SHADOW',
'GL_DELETE_STATUS', 'GL_COMPILE_STATUS', 'GL_LINK_STATUS',
'GL_VALIDATE_STATUS', 'GL_INFO_LOG_LENGTH', 'GL_ATTACHED_SHADERS',
'GL_ACTIVE_UNIFORMS', 'GL_ACTIVE_UNIFORM_MAX_LENGTH',
'GL_SHADER_SOURCE_LENGTH', 'GL_ACTIVE_ATTRIBUTES',
'GL_ACTIVE_ATTRIBUTE_MAX_LENGTH', 'GL_FRAGMENT_SHADER_DERIVATIVE_HINT',
'GL_SHADING_LANGUAGE_VERSION', 'GL_CURRENT_PROGRAM',
'GL_POINT_SPRITE_COORD_ORIGIN', 'GL_LOWER_LEFT', 'GL_UPPER_LEFT',
'GL_STENCIL_BACK_REF', 'GL_STENCIL_BACK_VALUE_MASK',
'GL_STENCIL_BACK_WRITEMASK', 'GL_PIXEL_PACK_BUFFER', 'GL_PIXEL_UNPACK_BUFFER',
'GL_PIXEL_PACK_BUFFER_BINDING', 'GL_PIXEL_UNPACK_BUFFER_BINDING', 'GL_SRGB',
'GL_SRGB8', 'GL_SRGB_ALPHA', 'GL_SRGB8_ALPHA8', 'GL_SLUMINANCE_ALPHA',
'GL_SLUMINANCE8_ALPHA8', 'GL_SLUMINANCE', 'GL_SLUMINANCE8',
'GL_COMPRESSED_SRGB', 'GL_COMPRESSED_SRGB_ALPHA', 'GL_COMPRESSED_SLUMINANCE',
'GL_COMPRESSED_SLUMINANCE_ALPHA', 'GL_FLOAT_MAT2x3', 'GL_FLOAT_MAT2x4',
'GL_FLOAT_MAT3x2', 'GL_FLOAT_MAT3x4', 'GL_FLOAT_MAT4x2', 'GL_FLOAT_MAT4x3',
'GL_CURRENT_RASTER_SECONDARY_COLOR', 'GL_CLIP_DISTANCE0', 'GL_CLIP_DISTANCE1',
'GL_CLIP_DISTANCE2', 'GL_CLIP_DISTANCE3', 'GL_CLIP_DISTANCE4',
'GL_CLIP_DISTANCE5', 'GL_MAX_CLIP_DISTANCES', 'GL_MAJOR_VERSION',
'GL_MINOR_VERSION', 'GL_NUM_EXTENSIONS', 'GL_CONTEXT_FLAGS',
'GL_DEPTH_BUFFER', 'GL_STENCIL_BUFFER', 'GL_COMPRESSED_RED',
'GL_COMPRESSED_RG', 'GL_CONTEXT_FLAG_FORWARD_COMPATIBLE_BIT', 'GL_RGBA32F',
'GL_RGB32F', 'GL_RGBA16F', 'GL_RGB16F', 'GL_VERTEX_ATTRIB_ARRAY_INTEGER',
'GL_MAX_ARRAY_TEXTURE_LAYERS', 'GL_MIN_PROGRAM_TEXEL_OFFSET',
'GL_MAX_PROGRAM_TEXEL_OFFSET', 'GL_CLAMP_VERTEX_COLOR',
'GL_CLAMP_FRAGMENT_COLOR', 'GL_CLAMP_READ_COLOR', 'GL_FIXED_ONLY',
'GL_MAX_VARYING_COMPONENTS', 'GL_TEXTURE_RED_TYPE', 'GL_TEXTURE_GREEN_TYPE',
'GL_TEXTURE_BLUE_TYPE', 'GL_TEXTURE_ALPHA_TYPE', 'GL_TEXTURE_LUMINANCE_TYPE',
'GL_TEXTURE_INTENSITY_TYPE', 'GL_TEXTURE_DEPTH_TYPE',
'GL_UNSIGNED_NORMALIZED', 'GL_TEXTURE_1D_ARRAY', 'GL_PROXY_TEXTURE_1D_ARRAY',
'GL_TEXTURE_2D_ARRAY', 'GL_PROXY_TEXTURE_2D_ARRAY',
'GL_TEXTURE_BINDING_1D_ARRAY', 'GL_TEXTURE_BINDING_2D_ARRAY',
'GL_R11F_G11F_B10F', 'GL_UNSIGNED_INT_10F_11F_11F_REV', 'GL_RGB9_E5',
'GL_UNSIGNED_INT_5_9_9_9_REV', 'GL_TEXTURE_SHARED_SIZE',
'GL_TRANSFORM_FEEDBACK_VARYING_MAX_LENGTH',
'GL_TRANSFORM_FEEDBACK_BUFFER_MODE',
'GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS',
'GL_TRANSFORM_FEEDBACK_VARYINGS', 'GL_TRANSFORM_FEEDBACK_BUFFER_START',
'GL_TRANSFORM_FEEDBACK_BUFFER_SIZE', 'GL_PRIMITIVES_GENERATED',
'GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN', 'GL_RASTERIZER_DISCARD',
'GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_COMPONENTS',
'GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS', 'GL_INTERLEAVED_ATTRIBS',
'GL_SEPARATE_ATTRIBS', 'GL_TRANSFORM_FEEDBACK_BUFFER',
'GL_TRANSFORM_FEEDBACK_BUFFER_BINDING', 'GL_RGBA32UI', 'GL_RGB32UI',
'GL_RGBA16UI', 'GL_RGB16UI', 'GL_RGBA8UI', 'GL_RGB8UI', 'GL_RGBA32I',
'GL_RGB32I', 'GL_RGBA16I', 'GL_RGB16I', 'GL_RGBA8I', 'GL_RGB8I',
'GL_RED_INTEGER', 'GL_GREEN_INTEGER', 'GL_BLUE_INTEGER', 'GL_ALPHA_INTEGER',
'GL_RGB_INTEGER', 'GL_RGBA_INTEGER', 'GL_BGR_INTEGER', 'GL_BGRA_INTEGER',
'GL_SAMPLER_1D_ARRAY', 'GL_SAMPLER_2D_ARRAY', 'GL_SAMPLER_1D_ARRAY_SHADOW',
'GL_SAMPLER_2D_ARRAY_SHADOW', 'GL_SAMPLER_CUBE_SHADOW',
'GL_UNSIGNED_INT_VEC2', 'GL_UNSIGNED_INT_VEC3', 'GL_UNSIGNED_INT_VEC4',
'GL_INT_SAMPLER_1D', 'GL_INT_SAMPLER_2D', 'GL_INT_SAMPLER_3D',
'GL_INT_SAMPLER_CUBE', 'GL_INT_SAMPLER_1D_ARRAY', 'GL_INT_SAMPLER_2D_ARRAY',
'GL_UNSIGNED_INT_SAMPLER_1D', 'GL_UNSIGNED_INT_SAMPLER_2D',
'GL_UNSIGNED_INT_SAMPLER_3D', 'GL_UNSIGNED_INT_SAMPLER_CUBE',
'GL_UNSIGNED_INT_SAMPLER_1D_ARRAY', 'GL_UNSIGNED_INT_SAMPLER_2D_ARRAY',
'GL_QUERY_WAIT', 'GL_QUERY_NO_WAIT', 'GL_QUERY_BY_REGION_WAIT',
'GL_QUERY_BY_REGION_NO_WAIT', 'GL_TRANSPOSE_MODELVIEW_MATRIX_ARB',
'GL_TRANSPOSE_PROJECTION_MATRIX_ARB', 'GL_TRANSPOSE_TEXTURE_MATRIX_ARB',
'GL_TRANSPOSE_COLOR_MATRIX_ARB', 'GL_MULTISAMPLE_ARB',
'GL_SAMPLE_ALPHA_TO_COVERAGE_ARB', 'GL_SAMPLE_ALPHA_TO_ONE_ARB',
'GL_SAMPLE_COVERAGE_ARB', 'GL_SAMPLE_BUFFERS_ARB', 'GL_SAMPLES_ARB',
'GL_SAMPLE_COVERAGE_VALUE_ARB', 'GL_SAMPLE_COVERAGE_INVERT_ARB',
'GL_MULTISAMPLE_BIT_ARB', 'GL_NORMAL_MAP_ARB', 'GL_REFLECTION_MAP_ARB',
'GL_TEXTURE_CUBE_MAP_ARB', 'GL_TEXTURE_BINDING_CUBE_MAP_ARB',
'GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB', 'GL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB',
'GL_TEXTURE_CUBE_MAP_POSITIVE_Y_ARB', 'GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_ARB',
'GL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB', 'GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB',
'GL_PROXY_TEXTURE_CUBE_MAP_ARB', 'GL_MAX_CUBE_MAP_TEXTURE_SIZE_ARB',
'GL_COMPRESSED_ALPHA_ARB', 'GL_COMPRESSED_LUMINANCE_ARB',
'GL_COMPRESSED_LUMINANCE_ALPHA_ARB', 'GL_COMPRESSED_INTENSITY_ARB',
'GL_COMPRESSED_RGB_ARB', 'GL_COMPRESSED_RGBA_ARB',
'GL_TEXTURE_COMPRESSION_HINT_ARB', 'GL_TEXTURE_COMPRESSED_IMAGE_SIZE_ARB',
'GL_TEXTURE_COMPRESSED_ARB', 'GL_NUM_COMPRESSED_TEXTURE_FORMATS_ARB',
'GL_COMPRESSED_TEXTURE_FORMATS_ARB', 'GL_CLAMP_TO_BORDER_ARB',
'GL_POINT_SIZE_MIN_ARB', 'GL_POINT_SIZE_MAX_ARB',
'GL_POINT_FADE_THRESHOLD_SIZE_ARB', 'GL_POINT_DISTANCE_ATTENUATION_ARB',
'GL_MAX_VERTEX_UNITS_ARB', 'GL_ACTIVE_VERTEX_UNITS_ARB',
'GL_WEIGHT_SUM_UNITY_ARB', 'GL_VERTEX_BLEND_ARB', 'GL_CURRENT_WEIGHT_ARB',
'GL_WEIGHT_ARRAY_TYPE_ARB', 'GL_WEIGHT_ARRAY_STRIDE_ARB',
'GL_WEIGHT_ARRAY_SIZE_ARB', 'GL_WEIGHT_ARRAY_POINTER_ARB',
'GL_WEIGHT_ARRAY_ARB', 'GL_MODELVIEW0_ARB', 'GL_MODELVIEW1_ARB',
'GL_MODELVIEW2_ARB', 'GL_MODELVIEW3_ARB', 'GL_MODELVIEW4_ARB',
'GL_MODELVIEW5_ARB', 'GL_MODELVIEW6_ARB', 'GL_MODELVIEW7_ARB',
'GL_MODELVIEW8_ARB', 'GL_MODELVIEW9_ARB', 'GL_MODELVIEW10_ARB',
'GL_MODELVIEW11_ARB', 'GL_MODELVIEW12_ARB', 'GL_MODELVIEW13_ARB',
'GL_MODELVIEW14_ARB', 'GL_MODELVIEW15_ARB', 'GL_MODELVIEW16_ARB',
'GL_MODELVIEW17_ARB', 'GL_MODELVIEW18_ARB', 'GL_MODELVIEW19_ARB',
'GL_MODELVIEW20_ARB', 'GL_MODELVIEW21_ARB', 'GL_MODELVIEW22_ARB',
'GL_MODELVIEW23_ARB', 'GL_MODELVIEW24_ARB', 'GL_MODELVIEW25_ARB',
'GL_MODELVIEW26_ARB', 'GL_MODELVIEW27_ARB', 'GL_MODELVIEW28_ARB',
'GL_MODELVIEW29_ARB', 'GL_MODELVIEW30_ARB', 'GL_MODELVIEW31_ARB',
'GL_MATRIX_PALETTE_ARB', 'GL_MAX_MATRIX_PALETTE_STACK_DEPTH_ARB',
'GL_MAX_PALETTE_MATRICES_ARB', 'GL_CURRENT_PALETTE_MATRIX_ARB',
'GL_MATRIX_INDEX_ARRAY_ARB', 'GL_CURRENT_MATRIX_INDEX_ARB',
'GL_MATRIX_INDEX_ARRAY_SIZE_ARB', 'GL_MATRIX_INDEX_ARRAY_TYPE_ARB',
'GL_MATRIX_INDEX_ARRAY_STRIDE_ARB', 'GL_MATRIX_INDEX_ARRAY_POINTER_ARB',
'GL_COMBINE_ARB', 'GL_COMBINE_RGB_ARB', 'GL_COMBINE_ALPHA_ARB',
'GL_SOURCE0_RGB_ARB', 'GL_SOURCE1_RGB_ARB', 'GL_SOURCE2_RGB_ARB',
'GL_SOURCE0_ALPHA_ARB', 'GL_SOURCE1_ALPHA_ARB', 'GL_SOURCE2_ALPHA_ARB',
'GL_OPERAND0_RGB_ARB', 'GL_OPERAND1_RGB_ARB', 'GL_OPERAND2_RGB_ARB',
'GL_OPERAND0_ALPHA_ARB', 'GL_OPERAND1_ALPHA_ARB', 'GL_OPERAND2_ALPHA_ARB',
'GL_RGB_SCALE_ARB', 'GL_ADD_SIGNED_ARB', 'GL_INTERPOLATE_ARB',
'GL_SUBTRACT_ARB', 'GL_CONSTANT_ARB', 'GL_PRIMARY_COLOR_ARB',
'GL_PREVIOUS_ARB', 'GL_DOT3_RGB_ARB', 'GL_DOT3_RGBA_ARB',
'GL_MIRRORED_REPEAT_ARB', 'GL_DEPTH_COMPONENT16_ARB',
'GL_DEPTH_COMPONENT24_ARB', 'GL_DEPTH_COMPONENT32_ARB',
'GL_TEXTURE_DEPTH_SIZE_ARB', 'GL_DEPTH_TEXTURE_MODE_ARB',
'GL_TEXTURE_COMPARE_MODE_ARB', 'GL_TEXTURE_COMPARE_FUNC_ARB',
'GL_COMPARE_R_TO_TEXTURE_ARB', 'GL_TEXTURE_COMPARE_FAIL_VALUE_ARB',
'GL_COLOR_SUM_ARB', 'GL_VERTEX_PROGRAM_ARB',
'GL_VERTEX_ATTRIB_ARRAY_ENABLED_ARB', 'GL_VERTEX_ATTRIB_ARRAY_SIZE_ARB',
'GL_VERTEX_ATTRIB_ARRAY_STRIDE_ARB', 'GL_VERTEX_ATTRIB_ARRAY_TYPE_ARB',
'GL_CURRENT_VERTEX_ATTRIB_ARB', 'GL_PROGRAM_LENGTH_ARB',
'GL_PROGRAM_STRING_ARB', 'GL_MAX_PROGRAM_MATRIX_STACK_DEPTH_ARB',
'GL_MAX_PROGRAM_MATRICES_ARB', 'GL_CURRENT_MATRIX_STACK_DEPTH_ARB',
'GL_CURRENT_MATRIX_ARB', 'GL_VERTEX_PROGRAM_POINT_SIZE_ARB',
'GL_VERTEX_PROGRAM_TWO_SIDE_ARB', 'GL_VERTEX_ATTRIB_ARRAY_POINTER_ARB',
'GL_PROGRAM_ERROR_POSITION_ARB', 'GL_PROGRAM_BINDING_ARB',
'GL_MAX_VERTEX_ATTRIBS_ARB', 'GL_VERTEX_ATTRIB_ARRAY_NORMALIZED_ARB',
'GL_PROGRAM_ERROR_STRING_ARB', 'GL_PROGRAM_FORMAT_ASCII_ARB',
'GL_PROGRAM_FORMAT_ARB', 'GL_PROGRAM_INSTRUCTIONS_ARB',
'GL_MAX_PROGRAM_INSTRUCTIONS_ARB', 'GL_PROGRAM_NATIVE_INSTRUCTIONS_ARB',
'GL_MAX_PROGRAM_NATIVE_INSTRUCTIONS_ARB', 'GL_PROGRAM_TEMPORARIES_ARB',
'GL_MAX_PROGRAM_TEMPORARIES_ARB', 'GL_PROGRAM_NATIVE_TEMPORARIES_ARB',
'GL_MAX_PROGRAM_NATIVE_TEMPORARIES_ARB', 'GL_PROGRAM_PARAMETERS_ARB',
'GL_MAX_PROGRAM_PARAMETERS_ARB', 'GL_PROGRAM_NATIVE_PARAMETERS_ARB',
'GL_MAX_PROGRAM_NATIVE_PARAMETERS_ARB', 'GL_PROGRAM_ATTRIBS_ARB',
'GL_MAX_PROGRAM_ATTRIBS_ARB', 'GL_PROGRAM_NATIVE_ATTRIBS_ARB',
'GL_MAX_PROGRAM_NATIVE_ATTRIBS_ARB', 'GL_PROGRAM_ADDRESS_REGISTERS_ARB',
'GL_MAX_PROGRAM_ADDRESS_REGISTERS_ARB',
'GL_PROGRAM_NATIVE_ADDRESS_REGISTERS_ARB',
'GL_MAX_PROGRAM_NATIVE_ADDRESS_REGISTERS_ARB',
'GL_MAX_PROGRAM_LOCAL_PARAMETERS_ARB', 'GL_MAX_PROGRAM_ENV_PARAMETERS_ARB',
'GL_PROGRAM_UNDER_NATIVE_LIMITS_ARB', 'GL_TRANSPOSE_CURRENT_MATRIX_ARB',
'GL_MATRIX0_ARB', 'GL_MATRIX1_ARB', 'GL_MATRIX2_ARB', 'GL_MATRIX3_ARB',
'GL_MATRIX4_ARB', 'GL_MATRIX5_ARB', 'GL_MATRIX6_ARB', 'GL_MATRIX7_ARB',
'GL_MATRIX8_ARB', 'GL_MATRIX9_ARB', 'GL_MATRIX10_ARB', 'GL_MATRIX11_ARB',
'GL_MATRIX12_ARB', 'GL_MATRIX13_ARB', 'GL_MATRIX14_ARB', 'GL_MATRIX15_ARB',
'GL_MATRIX16_ARB', 'GL_MATRIX17_ARB', 'GL_MATRIX18_ARB', 'GL_MATRIX19_ARB',
'GL_MATRIX20_ARB', 'GL_MATRIX21_ARB', 'GL_MATRIX22_ARB', 'GL_MATRIX23_ARB',
'GL_MATRIX24_ARB', 'GL_MATRIX25_ARB', 'GL_MATRIX26_ARB', 'GL_MATRIX27_ARB',
'GL_MATRIX28_ARB', 'GL_MATRIX29_ARB', 'GL_MATRIX30_ARB', 'GL_MATRIX31_ARB',
'GL_FRAGMENT_PROGRAM_ARB', 'GL_PROGRAM_ALU_INSTRUCTIONS_ARB',
'GL_PROGRAM_TEX_INSTRUCTIONS_ARB', 'GL_PROGRAM_TEX_INDIRECTIONS_ARB',
'GL_PROGRAM_NATIVE_ALU_INSTRUCTIONS_ARB',
'GL_PROGRAM_NATIVE_TEX_INSTRUCTIONS_ARB',
'GL_PROGRAM_NATIVE_TEX_INDIRECTIONS_ARB',
'GL_MAX_PROGRAM_ALU_INSTRUCTIONS_ARB', 'GL_MAX_PROGRAM_TEX_INSTRUCTIONS_ARB',
'GL_MAX_PROGRAM_TEX_INDIRECTIONS_ARB',
'GL_MAX_PROGRAM_NATIVE_ALU_INSTRUCTIONS_ARB',
'GL_MAX_PROGRAM_NATIVE_TEX_INSTRUCTIONS_ARB',
'GL_MAX_PROGRAM_NATIVE_TEX_INDIRECTIONS_ARB', 'GL_MAX_TEXTURE_COORDS_ARB',
'GL_MAX_TEXTURE_IMAGE_UNITS_ARB', 'GL_BUFFER_SIZE_ARB', 'GL_BUFFER_USAGE_ARB',
'GL_ARRAY_BUFFER_ARB', 'GL_ELEMENT_ARRAY_BUFFER_ARB',
'GL_ARRAY_BUFFER_BINDING_ARB', 'GL_ELEMENT_ARRAY_BUFFER_BINDING_ARB',
'GL_VERTEX_ARRAY_BUFFER_BINDING_ARB', 'GL_NORMAL_ARRAY_BUFFER_BINDING_ARB',
'GL_COLOR_ARRAY_BUFFER_BINDING_ARB', 'GL_INDEX_ARRAY_BUFFER_BINDING_ARB',
'GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING_ARB',
'GL_EDGE_FLAG_ARRAY_BUFFER_BINDING_ARB',
'GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING_ARB',
'GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING_ARB',
'GL_WEIGHT_ARRAY_BUFFER_BINDING_ARB',
'GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING_ARB', 'GL_READ_ONLY_ARB',
'GL_WRITE_ONLY_ARB', 'GL_READ_WRITE_ARB', 'GL_BUFFER_ACCESS_ARB',
'GL_BUFFER_MAPPED_ARB', 'GL_BUFFER_MAP_POINTER_ARB', 'GL_STREAM_DRAW_ARB',
'GL_STREAM_READ_ARB', 'GL_STREAM_COPY_ARB', 'GL_STATIC_DRAW_ARB',
'GL_STATIC_READ_ARB', 'GL_STATIC_COPY_ARB', 'GL_DYNAMIC_DRAW_ARB',
'GL_DYNAMIC_READ_ARB', 'GL_DYNAMIC_COPY_ARB', 'GL_QUERY_COUNTER_BITS_ARB',
'GL_CURRENT_QUERY_ARB', 'GL_QUERY_RESULT_ARB',
'GL_QUERY_RESULT_AVAILABLE_ARB', 'GL_SAMPLES_PASSED_ARB',
'GL_PROGRAM_OBJECT_ARB', 'GL_SHADER_OBJECT_ARB', 'GL_OBJECT_TYPE_ARB',
'GL_OBJECT_SUBTYPE_ARB', 'GL_FLOAT_VEC2_ARB', 'GL_FLOAT_VEC3_ARB',
'GL_FLOAT_VEC4_ARB', 'GL_INT_VEC2_ARB', 'GL_INT_VEC3_ARB', 'GL_INT_VEC4_ARB',
'GL_BOOL_ARB', 'GL_BOOL_VEC2_ARB', 'GL_BOOL_VEC3_ARB', 'GL_BOOL_VEC4_ARB',
'GL_FLOAT_MAT2_ARB', 'GL_FLOAT_MAT3_ARB', 'GL_FLOAT_MAT4_ARB',
'GL_SAMPLER_1D_ARB', 'GL_SAMPLER_2D_ARB', 'GL_SAMPLER_3D_ARB',
'GL_SAMPLER_CUBE_ARB', 'GL_SAMPLER_1D_SHADOW_ARB', 'GL_SAMPLER_2D_SHADOW_ARB',
'GL_SAMPLER_2D_RECT_ARB', 'GL_SAMPLER_2D_RECT_SHADOW_ARB',
'GL_OBJECT_DELETE_STATUS_ARB', 'GL_OBJECT_COMPILE_STATUS_ARB',
'GL_OBJECT_LINK_STATUS_ARB', 'GL_OBJECT_VALIDATE_STATUS_ARB',
'GL_OBJECT_INFO_LOG_LENGTH_ARB', 'GL_OBJECT_ATTACHED_OBJECTS_ARB',
'GL_OBJECT_ACTIVE_UNIFORMS_ARB', 'GL_OBJECT_ACTIVE_UNIFORM_MAX_LENGTH_ARB',
'GL_OBJECT_SHADER_SOURCE_LENGTH_ARB', 'GL_VERTEX_SHADER_ARB',
'GL_MAX_VERTEX_UNIFORM_COMPONENTS_ARB', 'GL_MAX_VARYING_FLOATS_ARB',
'GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS_ARB',
'GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS_ARB', 'GL_OBJECT_ACTIVE_ATTRIBUTES_ARB',
'GL_OBJECT_ACTIVE_ATTRIBUTE_MAX_LENGTH_ARB', 'GL_FRAGMENT_SHADER_ARB',
'GL_MAX_FRAGMENT_UNIFORM_COMPONENTS_ARB',
'GL_FRAGMENT_SHADER_DERIVATIVE_HINT_ARB', 'GL_SHADING_LANGUAGE_VERSION_ARB',
'GL_POINT_SPRITE_ARB', 'GL_COORD_REPLACE_ARB', 'GL_MAX_DRAW_BUFFERS_ARB',
'GL_DRAW_BUFFER0_ARB', 'GL_DRAW_BUFFER1_ARB', 'GL_DRAW_BUFFER2_ARB',
'GL_DRAW_BUFFER3_ARB', 'GL_DRAW_BUFFER4_ARB', 'GL_DRAW_BUFFER5_ARB',
'GL_DRAW_BUFFER6_ARB', 'GL_DRAW_BUFFER7_ARB', 'GL_DRAW_BUFFER8_ARB',
'GL_DRAW_BUFFER9_ARB', 'GL_DRAW_BUFFER10_ARB', 'GL_DRAW_BUFFER11_ARB',
'GL_DRAW_BUFFER12_ARB', 'GL_DRAW_BUFFER13_ARB', 'GL_DRAW_BUFFER14_ARB',
'GL_DRAW_BUFFER15_ARB', 'GL_TEXTURE_RECTANGLE_ARB',
'GL_TEXTURE_BINDING_RECTANGLE_ARB', 'GL_PROXY_TEXTURE_RECTANGLE_ARB',
'GL_MAX_RECTANGLE_TEXTURE_SIZE_ARB', 'GL_RGBA_FLOAT_MODE_ARB',
'GL_CLAMP_VERTEX_COLOR_ARB', 'GL_CLAMP_FRAGMENT_COLOR_ARB',
'GL_CLAMP_READ_COLOR_ARB', 'GL_FIXED_ONLY_ARB', 'GL_HALF_FLOAT_ARB',
'GL_TEXTURE_RED_TYPE_ARB', 'GL_TEXTURE_GREEN_TYPE_ARB',
'GL_TEXTURE_BLUE_TYPE_ARB', 'GL_TEXTURE_ALPHA_TYPE_ARB',
'GL_TEXTURE_LUMINANCE_TYPE_ARB', 'GL_TEXTURE_INTENSITY_TYPE_ARB',
'GL_TEXTURE_DEPTH_TYPE_ARB', 'GL_UNSIGNED_NORMALIZED_ARB', 'GL_RGBA32F_ARB',
'GL_RGB32F_ARB', 'GL_ALPHA32F_ARB', 'GL_INTENSITY32F_ARB',
'GL_LUMINANCE32F_ARB', 'GL_LUMINANCE_ALPHA32F_ARB', 'GL_RGBA16F_ARB',
'GL_RGB16F_ARB', 'GL_ALPHA16F_ARB', 'GL_INTENSITY16F_ARB',
'GL_LUMINANCE16F_ARB', 'GL_LUMINANCE_ALPHA16F_ARB',
'GL_PIXEL_PACK_BUFFER_ARB', 'GL_PIXEL_UNPACK_BUFFER_ARB',
'GL_PIXEL_PACK_BUFFER_BINDING_ARB', 'GL_PIXEL_UNPACK_BUFFER_BINDING_ARB',
'GL_DEPTH_COMPONENT32F', 'GL_DEPTH32F_STENCIL8',
'GL_FLOAT_32_UNSIGNED_INT_24_8_REV', 'GL_INVALID_FRAMEBUFFER_OPERATION',
'GL_FRAMEBUFFER_ATTACHMENT_COLOR_ENCODING',
'GL_FRAMEBUFFER_ATTACHMENT_COMPONENT_TYPE',
'GL_FRAMEBUFFER_ATTACHMENT_RED_SIZE', 'GL_FRAMEBUFFER_ATTACHMENT_GREEN_SIZE',
'GL_FRAMEBUFFER_ATTACHMENT_BLUE_SIZE', 'GL_FRAMEBUFFER_ATTACHMENT_ALPHA_SIZE',
'GL_FRAMEBUFFER_ATTACHMENT_DEPTH_SIZE',
'GL_FRAMEBUFFER_ATTACHMENT_STENCIL_SIZE', 'GL_FRAMEBUFFER_DEFAULT',
'GL_FRAMEBUFFER_UNDEFINED', 'GL_DEPTH_STENCIL_ATTACHMENT', 'GL_INDEX',
'GL_MAX_RENDERBUFFER_SIZE', 'GL_DEPTH_STENCIL', 'GL_UNSIGNED_INT_24_8',
'GL_DEPTH24_STENCIL8', 'GL_TEXTURE_STENCIL_SIZE', 'GL_FRAMEBUFFER_BINDING',
'GL_DRAW_FRAMEBUFFER_BINDING', 'GL_RENDERBUFFER_BINDING',
'GL_READ_FRAMEBUFFER', 'GL_DRAW_FRAMEBUFFER', 'GL_READ_FRAMEBUFFER_BINDING',
'GL_RENDERBUFFER_SAMPLES', 'GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE',
'GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME',
'GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL',
'GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE',
'GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LAYER', 'GL_FRAMEBUFFER_COMPLETE',
'GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT',
'GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT',
'GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER',
'GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER', 'GL_FRAMEBUFFER_UNSUPPORTED',
'GL_MAX_COLOR_ATTACHMENTS', 'GL_COLOR_ATTACHMENT0', 'GL_COLOR_ATTACHMENT1',
'GL_COLOR_ATTACHMENT2', 'GL_COLOR_ATTACHMENT3', 'GL_COLOR_ATTACHMENT4',
'GL_COLOR_ATTACHMENT5', 'GL_COLOR_ATTACHMENT6', 'GL_COLOR_ATTACHMENT7',
'GL_COLOR_ATTACHMENT8', 'GL_COLOR_ATTACHMENT9', 'GL_COLOR_ATTACHMENT10',
'GL_COLOR_ATTACHMENT11', 'GL_COLOR_ATTACHMENT12', 'GL_COLOR_ATTACHMENT13',
'GL_COLOR_ATTACHMENT14', 'GL_COLOR_ATTACHMENT15', 'GL_DEPTH_ATTACHMENT',
'GL_STENCIL_ATTACHMENT', 'GL_FRAMEBUFFER', 'GL_RENDERBUFFER',
'GL_RENDERBUFFER_WIDTH', 'GL_RENDERBUFFER_HEIGHT',
'GL_RENDERBUFFER_INTERNAL_FORMAT', 'GL_STENCIL_INDEX1', 'GL_STENCIL_INDEX4',
'GL_STENCIL_INDEX8', 'GL_STENCIL_INDEX16', 'GL_RENDERBUFFER_RED_SIZE',
'GL_RENDERBUFFER_GREEN_SIZE', 'GL_RENDERBUFFER_BLUE_SIZE',
'GL_RENDERBUFFER_ALPHA_SIZE', 'GL_RENDERBUFFER_DEPTH_SIZE',
'GL_RENDERBUFFER_STENCIL_SIZE', 'GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE',
'GL_MAX_SAMPLES', 'GL_FRAMEBUFFER_SRGB', 'GL_LINES_ADJACENCY_ARB',
'GL_LINE_STRIP_ADJACENCY_ARB', 'GL_TRIANGLES_ADJACENCY_ARB',
'GL_TRIANGLE_STRIP_ADJACENCY_ARB', 'GL_PROGRAM_POINT_SIZE_ARB',
'GL_MAX_GEOMETRY_TEXTURE_IMAGE_UNITS_ARB',
'GL_FRAMEBUFFER_ATTACHMENT_LAYERED_ARB',
'GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS_ARB',
'GL_FRAMEBUFFER_INCOMPLETE_LAYER_COUNT_ARB', 'GL_GEOMETRY_SHADER_ARB',
'GL_GEOMETRY_VERTICES_OUT_ARB', 'GL_GEOMETRY_INPUT_TYPE_ARB',
'GL_GEOMETRY_OUTPUT_TYPE_ARB', 'GL_MAX_GEOMETRY_VARYING_COMPONENTS_ARB',
'GL_MAX_VERTEX_VARYING_COMPONENTS_ARB',
'GL_MAX_GEOMETRY_UNIFORM_COMPONENTS_ARB',
'GL_MAX_GEOMETRY_OUTPUT_VERTICES_ARB',
'GL_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS_ARB', 'GL_HALF_FLOAT',
'GL_MAP_READ_BIT', 'GL_MAP_WRITE_BIT', 'GL_MAP_INVALIDATE_RANGE_BIT',
'GL_MAP_INVALIDATE_BUFFER_BIT', 'GL_MAP_FLUSH_EXPLICIT_BIT',
'GL_MAP_UNSYNCHRONIZED_BIT', 'GL_TEXTURE_BUFFER_ARB',
'GL_MAX_TEXTURE_BUFFER_SIZE_ARB', 'GL_TEXTURE_BINDING_BUFFER_ARB',
'GL_TEXTURE_BUFFER_DATA_STORE_BINDING_ARB', 'GL_TEXTURE_BUFFER_FORMAT_ARB',
'GL_COMPRESSED_RED_RGTC1', 'GL_COMPRESSED_SIGNED_RED_RGTC1',
'GL_COMPRESSED_RG_RGTC2', 'GL_COMPRESSED_SIGNED_RG_RGTC2', 'GL_RG',
'GL_RG_INTEGER', 'GL_R8', 'GL_R16', 'GL_RG8', 'GL_RG16', 'GL_R16F', 'GL_R32F',
'GL_RG16F', 'GL_RG32F', 'GL_R8I', 'GL_R8UI', 'GL_R16I', 'GL_R16UI', 'GL_R32I',
'GL_R32UI', 'GL_RG8I', 'GL_RG8UI', 'GL_RG16I', 'GL_RG16UI', 'GL_RG32I',
'GL_RG32UI', 'GL_VERTEX_ARRAY_BINDING', 'GL_ABGR_EXT',
'GL_CONSTANT_COLOR_EXT', 'GL_ONE_MINUS_CONSTANT_COLOR_EXT',
'GL_CONSTANT_ALPHA_EXT', 'GL_ONE_MINUS_CONSTANT_ALPHA_EXT',
'GL_BLEND_COLOR_EXT', 'GL_POLYGON_OFFSET_EXT', 'GL_POLYGON_OFFSET_FACTOR_EXT',
'GL_POLYGON_OFFSET_BIAS_EXT', 'GL_ALPHA4_EXT', 'GL_ALPHA8_EXT',
'GL_ALPHA12_EXT', 'GL_ALPHA16_EXT', 'GL_LUMINANCE4_EXT', 'GL_LUMINANCE8_EXT',
'GL_LUMINANCE12_EXT', 'GL_LUMINANCE16_EXT', 'GL_LUMINANCE4_ALPHA4_EXT',
'GL_LUMINANCE6_ALPHA2_EXT', 'GL_LUMINANCE8_ALPHA8_EXT',
'GL_LUMINANCE12_ALPHA4_EXT', 'GL_LUMINANCE12_ALPHA12_EXT',
'GL_LUMINANCE16_ALPHA16_EXT', 'GL_INTENSITY_EXT', 'GL_INTENSITY4_EXT',
'GL_INTENSITY8_EXT', 'GL_INTENSITY12_EXT', 'GL_INTENSITY16_EXT',
'GL_RGB2_EXT', 'GL_RGB4_EXT', 'GL_RGB5_EXT', 'GL_RGB8_EXT', 'GL_RGB10_EXT',
'GL_RGB12_EXT', 'GL_RGB16_EXT', 'GL_RGBA2_EXT', 'GL_RGBA4_EXT',
'GL_RGB5_A1_EXT', 'GL_RGBA8_EXT', 'GL_RGB10_A2_EXT', 'GL_RGBA12_EXT',
'GL_RGBA16_EXT', 'GL_TEXTURE_RED_SIZE_EXT', 'GL_TEXTURE_GREEN_SIZE_EXT',
'GL_TEXTURE_BLUE_SIZE_EXT', 'GL_TEXTURE_ALPHA_SIZE_EXT',
'GL_TEXTURE_LUMINANCE_SIZE_EXT', 'GL_TEXTURE_INTENSITY_SIZE_EXT',
'GL_REPLACE_EXT', 'GL_PROXY_TEXTURE_1D_EXT', 'GL_PROXY_TEXTURE_2D_EXT',
'GL_TEXTURE_TOO_LARGE_EXT', 'GL_PACK_SKIP_IMAGES_EXT',
'GL_PACK_IMAGE_HEIGHT_EXT', 'GL_UNPACK_SKIP_IMAGES_EXT',
'GL_UNPACK_IMAGE_HEIGHT_EXT', 'GL_TEXTURE_3D_EXT', 'GL_PROXY_TEXTURE_3D_EXT',
'GL_TEXTURE_DEPTH_EXT', 'GL_TEXTURE_WRAP_R_EXT', 'GL_MAX_3D_TEXTURE_SIZE_EXT',
'GL_FILTER4_SGIS', 'GL_TEXTURE_FILTER4_SIZE_SGIS', 'GL_HISTOGRAM_EXT',
'GL_PROXY_HISTOGRAM_EXT', 'GL_HISTOGRAM_WIDTH_EXT', 'GL_HISTOGRAM_FORMAT_EXT',
'GL_HISTOGRAM_RED_SIZE_EXT', 'GL_HISTOGRAM_GREEN_SIZE_EXT',
'GL_HISTOGRAM_BLUE_SIZE_EXT', 'GL_HISTOGRAM_ALPHA_SIZE_EXT',
'GL_HISTOGRAM_LUMINANCE_SIZE_EXT', 'GL_HISTOGRAM_SINK_EXT', 'GL_MINMAX_EXT',
'GL_MINMAX_FORMAT_EXT', 'GL_MINMAX_SINK_EXT', 'GL_TABLE_TOO_LARGE_EXT',
'GL_CONVOLUTION_1D_EXT', 'GL_CONVOLUTION_2D_EXT', 'GL_SEPARABLE_2D_EXT',
'GL_CONVOLUTION_BORDER_MODE_EXT', 'GL_CONVOLUTION_FILTER_SCALE_EXT',
'GL_CONVOLUTION_FILTER_BIAS_EXT', 'GL_REDUCE_EXT',
'GL_CONVOLUTION_FORMAT_EXT', 'GL_CONVOLUTION_WIDTH_EXT',
'GL_CONVOLUTION_HEIGHT_EXT', 'GL_MAX_CONVOLUTION_WIDTH_EXT',
'GL_MAX_CONVOLUTION_HEIGHT_EXT', 'GL_POST_CONVOLUTION_RED_SCALE_EXT',
'GL_POST_CONVOLUTION_GREEN_SCALE_EXT', 'GL_POST_CONVOLUTION_BLUE_SCALE_EXT',
'GL_POST_CONVOLUTION_ALPHA_SCALE_EXT', 'GL_POST_CONVOLUTION_RED_BIAS_EXT',
'GL_POST_CONVOLUTION_GREEN_BIAS_EXT', 'GL_POST_CONVOLUTION_BLUE_BIAS_EXT',
'GL_POST_CONVOLUTION_ALPHA_BIAS_EXT', 'GL_COLOR_MATRIX_SGI',
'GL_COLOR_MATRIX_STACK_DEPTH_SGI', 'GL_MAX_COLOR_MATRIX_STACK_DEPTH_SGI',
'GL_POST_COLOR_MATRIX_RED_SCALE_SGI', 'GL_POST_COLOR_MATRIX_GREEN_SCALE_SGI',
'GL_POST_COLOR_MATRIX_BLUE_SCALE_SGI', 'GL_POST_COLOR_MATRIX_ALPHA_SCALE_SGI',
'GL_POST_COLOR_MATRIX_RED_BIAS_SGI', 'GL_POST_COLOR_MATRIX_GREEN_BIAS_SGI',
'GL_POST_COLOR_MATRIX_BLUE_BIAS_SGI', 'GL_POST_COLOR_MATRIX_ALPHA_BIAS_SGI',
'GL_COLOR_TABLE_SGI', 'GL_POST_CONVOLUTION_COLOR_TABLE_SGI',
'GL_POST_COLOR_MATRIX_COLOR_TABLE_SGI', 'GL_PROXY_COLOR_TABLE_SGI',
'GL_PROXY_POST_CONVOLUTION_COLOR_TABLE_SGI',
'GL_PROXY_POST_COLOR_MATRIX_COLOR_TABLE_SGI', 'GL_COLOR_TABLE_SCALE_SGI',
'GL_COLOR_TABLE_BIAS_SGI', 'GL_COLOR_TABLE_FORMAT_SGI',
'GL_COLOR_TABLE_WIDTH_SGI', 'GL_COLOR_TABLE_RED_SIZE_SGI',
'GL_COLOR_TABLE_GREEN_SIZE_SGI', 'GL_COLOR_TABLE_BLUE_SIZE_SGI',
'GL_COLOR_TABLE_ALPHA_SIZE_SGI', 'GL_COLOR_TABLE_LUMINANCE_SIZE_SGI',
'GL_COLOR_TABLE_INTENSITY_SIZE_SGI', 'GL_PIXEL_TEXTURE_SGIS',
'GL_PIXEL_FRAGMENT_RGB_SOURCE_SGIS', 'GL_PIXEL_FRAGMENT_ALPHA_SOURCE_SGIS',
'GL_PIXEL_GROUP_COLOR_SGIS', 'GL_PIXEL_TEX_GEN_SGIX',
'GL_PIXEL_TEX_GEN_MODE_SGIX', 'GL_PACK_SKIP_VOLUMES_SGIS',
'GL_PACK_IMAGE_DEPTH_SGIS', 'GL_UNPACK_SKIP_VOLUMES_SGIS',
'GL_UNPACK_IMAGE_DEPTH_SGIS', 'GL_TEXTURE_4D_SGIS',
'GL_PROXY_TEXTURE_4D_SGIS', 'GL_TEXTURE_4DSIZE_SGIS',
'GL_TEXTURE_WRAP_Q_SGIS', 'GL_MAX_4D_TEXTURE_SIZE_SGIS',
'GL_TEXTURE_4D_BINDING_SGIS', 'GL_TEXTURE_COLOR_TABLE_SGI',
'GL_PROXY_TEXTURE_COLOR_TABLE_SGI', 'GL_CMYK_EXT', 'GL_CMYKA_EXT',
'GL_PACK_CMYK_HINT_EXT', 'GL_UNPACK_CMYK_HINT_EXT', 'GL_TEXTURE_PRIORITY_EXT',
'GL_TEXTURE_RESIDENT_EXT', 'GL_TEXTURE_1D_BINDING_EXT',
'GL_TEXTURE_2D_BINDING_EXT', 'GL_TEXTURE_3D_BINDING_EXT',
'GL_DETAIL_TEXTURE_2D_SGIS', 'GL_DETAIL_TEXTURE_2D_BINDING_SGIS',
'GL_LINEAR_DETAIL_SGIS', 'GL_LINEAR_DETAIL_ALPHA_SGIS',
'GL_LINEAR_DETAIL_COLOR_SGIS', 'GL_DETAIL_TEXTURE_LEVEL_SGIS',
'GL_DETAIL_TEXTURE_MODE_SGIS', 'GL_DETAIL_TEXTURE_FUNC_POINTS_SGIS',
'GL_LINEAR_SHARPEN_SGIS', 'GL_LINEAR_SHARPEN_ALPHA_SGIS',
'GL_LINEAR_SHARPEN_COLOR_SGIS', 'GL_SHARPEN_TEXTURE_FUNC_POINTS_SGIS',
'GL_UNSIGNED_BYTE_3_3_2_EXT', 'GL_UNSIGNED_SHORT_4_4_4_4_EXT',
'GL_UNSIGNED_SHORT_5_5_5_1_EXT', 'GL_UNSIGNED_INT_8_8_8_8_EXT',
'GL_UNSIGNED_INT_10_10_10_2_EXT', 'GL_TEXTURE_MIN_LOD_SGIS',
'GL_TEXTURE_MAX_LOD_SGIS', 'GL_TEXTURE_BASE_LEVEL_SGIS',
'GL_TEXTURE_MAX_LEVEL_SGIS', 'GL_MULTISAMPLE_SGIS',
'GL_SAMPLE_ALPHA_TO_MASK_SGIS', 'GL_SAMPLE_ALPHA_TO_ONE_SGIS',
'GL_SAMPLE_MASK_SGIS', 'GL_1PASS_SGIS', 'GL_2PASS_0_SGIS', 'GL_2PASS_1_SGIS',
'GL_4PASS_0_SGIS', 'GL_4PASS_1_SGIS', 'GL_4PASS_2_SGIS', 'GL_4PASS_3_SGIS',
'GL_SAMPLE_BUFFERS_SGIS', 'GL_SAMPLES_SGIS', 'GL_SAMPLE_MASK_VALUE_SGIS',
'GL_SAMPLE_MASK_INVERT_SGIS', 'GL_SAMPLE_PATTERN_SGIS',
'GL_RESCALE_NORMAL_EXT', 'GL_VERTEX_ARRAY_EXT', 'GL_NORMAL_ARRAY_EXT',
'GL_COLOR_ARRAY_EXT', 'GL_INDEX_ARRAY_EXT', 'GL_TEXTURE_COORD_ARRAY_EXT',
'GL_EDGE_FLAG_ARRAY_EXT', 'GL_VERTEX_ARRAY_SIZE_EXT',
'GL_VERTEX_ARRAY_TYPE_EXT', 'GL_VERTEX_ARRAY_STRIDE_EXT',
'GL_VERTEX_ARRAY_COUNT_EXT', 'GL_NORMAL_ARRAY_TYPE_EXT',
'GL_NORMAL_ARRAY_STRIDE_EXT', 'GL_NORMAL_ARRAY_COUNT_EXT',
'GL_COLOR_ARRAY_SIZE_EXT', 'GL_COLOR_ARRAY_TYPE_EXT',
'GL_COLOR_ARRAY_STRIDE_EXT', 'GL_COLOR_ARRAY_COUNT_EXT',
'GL_INDEX_ARRAY_TYPE_EXT', 'GL_INDEX_ARRAY_STRIDE_EXT',
'GL_INDEX_ARRAY_COUNT_EXT', 'GL_TEXTURE_COORD_ARRAY_SIZE_EXT',
'GL_TEXTURE_COORD_ARRAY_TYPE_EXT', 'GL_TEXTURE_COORD_ARRAY_STRIDE_EXT',
'GL_TEXTURE_COORD_ARRAY_COUNT_EXT', 'GL_EDGE_FLAG_ARRAY_STRIDE_EXT',
'GL_EDGE_FLAG_ARRAY_COUNT_EXT', 'GL_VERTEX_ARRAY_POINTER_EXT',
'GL_NORMAL_ARRAY_POINTER_EXT', 'GL_COLOR_ARRAY_POINTER_EXT',
'GL_INDEX_ARRAY_POINTER_EXT', 'GL_TEXTURE_COORD_ARRAY_POINTER_EXT',
'GL_EDGE_FLAG_ARRAY_POINTER_EXT', 'GL_GENERATE_MIPMAP_SGIS',
'GL_GENERATE_MIPMAP_HINT_SGIS', 'GL_LINEAR_CLIPMAP_LINEAR_SGIX',
'GL_TEXTURE_CLIPMAP_CENTER_SGIX', 'GL_TEXTURE_CLIPMAP_FRAME_SGIX',
'GL_TEXTURE_CLIPMAP_OFFSET_SGIX', 'GL_TEXTURE_CLIPMAP_VIRTUAL_DEPTH_SGIX',
'GL_TEXTURE_CLIPMAP_LOD_OFFSET_SGIX', 'GL_TEXTURE_CLIPMAP_DEPTH_SGIX',
'GL_MAX_CLIPMAP_DEPTH_SGIX', 'GL_MAX_CLIPMAP_VIRTUAL_DEPTH_SGIX',
'GL_NEAREST_CLIPMAP_NEAREST_SGIX', 'GL_NEAREST_CLIPMAP_LINEAR_SGIX',
'GL_LINEAR_CLIPMAP_NEAREST_SGIX', 'GL_TEXTURE_COMPARE_SGIX',
'GL_TEXTURE_COMPARE_OPERATOR_SGIX', 'GL_TEXTURE_LEQUAL_R_SGIX',
'GL_TEXTURE_GEQUAL_R_SGIX', 'GL_CLAMP_TO_EDGE_SGIS',
'GL_CLAMP_TO_BORDER_SGIS', 'GL_FUNC_ADD_EXT', 'GL_MIN_EXT', 'GL_MAX_EXT',
'GL_BLEND_EQUATION_EXT', 'GL_FUNC_SUBTRACT_EXT',
'GL_FUNC_REVERSE_SUBTRACT_EXT', 'GL_INTERLACE_SGIX',
'GL_PIXEL_TILE_BEST_ALIGNMENT_SGIX', 'GL_PIXEL_TILE_CACHE_INCREMENT_SGIX',
'GL_PIXEL_TILE_WIDTH_SGIX', 'GL_PIXEL_TILE_HEIGHT_SGIX',
'GL_PIXEL_TILE_GRID_WIDTH_SGIX', 'GL_PIXEL_TILE_GRID_HEIGHT_SGIX',
'GL_PIXEL_TILE_GRID_DEPTH_SGIX', 'GL_PIXEL_TILE_CACHE_SIZE_SGIX',
'GL_DUAL_ALPHA4_SGIS', 'GL_DUAL_ALPHA8_SGIS', 'GL_DUAL_ALPHA12_SGIS',
'GL_DUAL_ALPHA16_SGIS', 'GL_DUAL_LUMINANCE4_SGIS', 'GL_DUAL_LUMINANCE8_SGIS',
'GL_DUAL_LUMINANCE12_SGIS', 'GL_DUAL_LUMINANCE16_SGIS',
'GL_DUAL_INTENSITY4_SGIS', 'GL_DUAL_INTENSITY8_SGIS',
'GL_DUAL_INTENSITY12_SGIS', 'GL_DUAL_INTENSITY16_SGIS',
'GL_DUAL_LUMINANCE_ALPHA4_SGIS', 'GL_DUAL_LUMINANCE_ALPHA8_SGIS',
'GL_QUAD_ALPHA4_SGIS', 'GL_QUAD_ALPHA8_SGIS', 'GL_QUAD_LUMINANCE4_SGIS',
'GL_QUAD_LUMINANCE8_SGIS', 'GL_QUAD_INTENSITY4_SGIS',
'GL_QUAD_INTENSITY8_SGIS', 'GL_DUAL_TEXTURE_SELECT_SGIS',
'GL_QUAD_TEXTURE_SELECT_SGIS', 'GL_SPRITE_SGIX', 'GL_SPRITE_MODE_SGIX',
'GL_SPRITE_AXIS_SGIX', 'GL_SPRITE_TRANSLATION_SGIX', 'GL_SPRITE_AXIAL_SGIX',
'GL_SPRITE_OBJECT_ALIGNED_SGIX', 'GL_SPRITE_EYE_ALIGNED_SGIX',
'GL_TEXTURE_MULTI_BUFFER_HINT_SGIX', 'GL_POINT_SIZE_MIN_EXT',
'GL_POINT_SIZE_MAX_EXT', 'GL_POINT_FADE_THRESHOLD_SIZE_EXT',
'GL_DISTANCE_ATTENUATION_EXT', 'GL_POINT_SIZE_MIN_SGIS',
'GL_POINT_SIZE_MAX_SGIS', 'GL_POINT_FADE_THRESHOLD_SIZE_SGIS',
'GL_DISTANCE_ATTENUATION_SGIS', 'GL_INSTRUMENT_BUFFER_POINTER_SGIX',
'GL_INSTRUMENT_MEASUREMENTS_SGIX', 'GL_POST_TEXTURE_FILTER_BIAS_SGIX',
'GL_POST_TEXTURE_FILTER_SCALE_SGIX', 'GL_POST_TEXTURE_FILTER_BIAS_RANGE_SGIX',
'GL_POST_TEXTURE_FILTER_SCALE_RANGE_SGIX', 'GL_FRAMEZOOM_SGIX',
'GL_FRAMEZOOM_FACTOR_SGIX', 'GL_MAX_FRAMEZOOM_FACTOR_SGIX',
'GL_TEXTURE_DEFORMATION_BIT_SGIX', 'GL_GEOMETRY_DEFORMATION_BIT_SGIX',
'GL_GEOMETRY_DEFORMATION_SGIX', 'GL_TEXTURE_DEFORMATION_SGIX',
'GL_DEFORMATIONS_MASK_SGIX', 'GL_MAX_DEFORMATION_ORDER_SGIX',
'GL_REFERENCE_PLANE_SGIX', 'GL_REFERENCE_PLANE_EQUATION_SGIX',
'GL_DEPTH_COMPONENT16_SGIX', 'GL_DEPTH_COMPONENT24_SGIX',
'GL_DEPTH_COMPONENT32_SGIX', 'GL_FOG_FUNC_SGIS', 'GL_FOG_FUNC_POINTS_SGIS',
'GL_MAX_FOG_FUNC_POINTS_SGIS', 'GL_FOG_OFFSET_SGIX',
'GL_FOG_OFFSET_VALUE_SGIX', 'GL_IMAGE_SCALE_X_HP', 'GL_IMAGE_SCALE_Y_HP',
'GL_IMAGE_TRANSLATE_X_HP', 'GL_IMAGE_TRANSLATE_Y_HP',
'GL_IMAGE_ROTATE_ANGLE_HP', 'GL_IMAGE_ROTATE_ORIGIN_X_HP',
'GL_IMAGE_ROTATE_ORIGIN_Y_HP', 'GL_IMAGE_MAG_FILTER_HP',
'GL_IMAGE_MIN_FILTER_HP', 'GL_IMAGE_CUBIC_WEIGHT_HP', 'GL_CUBIC_HP',
'GL_AVERAGE_HP', 'GL_IMAGE_TRANSFORM_2D_HP',
'GL_POST_IMAGE_TRANSFORM_COLOR_TABLE_HP',
'GL_PROXY_POST_IMAGE_TRANSFORM_COLOR_TABLE_HP', 'GL_IGNORE_BORDER_HP',
'GL_CONSTANT_BORDER_HP', 'GL_REPLICATE_BORDER_HP',
'GL_CONVOLUTION_BORDER_COLOR_HP', 'GL_TEXTURE_ENV_BIAS_SGIX',
'GL_VERTEX_DATA_HINT_PGI', 'GL_VERTEX_CONSISTENT_HINT_PGI',
'GL_MATERIAL_SIDE_HINT_PGI', 'GL_MAX_VERTEX_HINT_PGI', 'GL_COLOR3_BIT_PGI',
'GL_COLOR4_BIT_PGI', 'GL_EDGEFLAG_BIT_PGI', 'GL_INDEX_BIT_PGI',
'GL_MAT_AMBIENT_BIT_PGI', 'GL_MAT_AMBIENT_AND_DIFFUSE_BIT_PGI',
'GL_MAT_DIFFUSE_BIT_PGI', 'GL_MAT_EMISSION_BIT_PGI',
'GL_MAT_COLOR_INDEXES_BIT_PGI', 'GL_MAT_SHININESS_BIT_PGI',
'GL_MAT_SPECULAR_BIT_PGI', 'GL_NORMAL_BIT_PGI', 'GL_TEXCOORD1_BIT_PGI',
'GL_TEXCOORD2_BIT_PGI', 'GL_TEXCOORD3_BIT_PGI', 'GL_TEXCOORD4_BIT_PGI',
'GL_VERTEX23_BIT_PGI', 'GL_VERTEX4_BIT_PGI',
'GL_PREFER_DOUBLEBUFFER_HINT_PGI', 'GL_CONSERVE_MEMORY_HINT_PGI',
'GL_RECLAIM_MEMORY_HINT_PGI', 'GL_NATIVE_GRAPHICS_HANDLE_PGI',
'GL_NATIVE_GRAPHICS_BEGIN_HINT_PGI', 'GL_NATIVE_GRAPHICS_END_HINT_PGI',
'GL_ALWAYS_FAST_HINT_PGI', 'GL_ALWAYS_SOFT_HINT_PGI',
'GL_ALLOW_DRAW_OBJ_HINT_PGI', 'GL_ALLOW_DRAW_WIN_HINT_PGI',
'GL_ALLOW_DRAW_FRG_HINT_PGI', 'GL_ALLOW_DRAW_MEM_HINT_PGI',
'GL_STRICT_DEPTHFUNC_HINT_PGI', 'GL_STRICT_LIGHTING_HINT_PGI',
'GL_STRICT_SCISSOR_HINT_PGI', 'GL_FULL_STIPPLE_HINT_PGI',
'GL_CLIP_NEAR_HINT_PGI', 'GL_CLIP_FAR_HINT_PGI', 'GL_WIDE_LINE_HINT_PGI',
'GL_BACK_NORMALS_HINT_PGI', 'GL_COLOR_INDEX1_EXT', 'GL_COLOR_INDEX2_EXT',
'GL_COLOR_INDEX4_EXT', 'GL_COLOR_INDEX8_EXT', 'GL_COLOR_INDEX12_EXT',
'GL_COLOR_INDEX16_EXT', 'GL_TEXTURE_INDEX_SIZE_EXT',
'GL_CLIP_VOLUME_CLIPPING_HINT_EXT', 'GL_LIST_PRIORITY_SGIX',
'GL_IR_INSTRUMENT1_SGIX', 'GL_CALLIGRAPHIC_FRAGMENT_SGIX',
'GL_TEXTURE_LOD_BIAS_S_SGIX', 'GL_TEXTURE_LOD_BIAS_T_SGIX',
'GL_TEXTURE_LOD_BIAS_R_SGIX', 'GL_SHADOW_AMBIENT_SGIX',
'GL_INDEX_MATERIAL_EXT', 'GL_INDEX_MATERIAL_PARAMETER_EXT',
'GL_INDEX_MATERIAL_FACE_EXT', 'GL_INDEX_TEST_EXT', 'GL_INDEX_TEST_FUNC_EXT',
'GL_INDEX_TEST_REF_EXT', 'GL_IUI_V2F_EXT', 'GL_IUI_V3F_EXT',
'GL_IUI_N3F_V2F_EXT', 'GL_IUI_N3F_V3F_EXT', 'GL_T2F_IUI_V2F_EXT',
'GL_T2F_IUI_V3F_EXT', 'GL_T2F_IUI_N3F_V2F_EXT', 'GL_T2F_IUI_N3F_V3F_EXT',
'GL_ARRAY_ELEMENT_LOCK_FIRST_EXT', 'GL_ARRAY_ELEMENT_LOCK_COUNT_EXT',
'GL_CULL_VERTEX_EXT', 'GL_CULL_VERTEX_EYE_POSITION_EXT',
'GL_CULL_VERTEX_OBJECT_POSITION_EXT', 'GL_YCRCB_422_SGIX',
'GL_YCRCB_444_SGIX', 'GL_FRAGMENT_LIGHTING_SGIX',
'GL_FRAGMENT_COLOR_MATERIAL_SGIX', 'GL_FRAGMENT_COLOR_MATERIAL_FACE_SGIX',
'GL_FRAGMENT_COLOR_MATERIAL_PARAMETER_SGIX', 'GL_MAX_FRAGMENT_LIGHTS_SGIX',
'GL_MAX_ACTIVE_LIGHTS_SGIX', 'GL_CURRENT_RASTER_NORMAL_SGIX',
'GL_LIGHT_ENV_MODE_SGIX', 'GL_FRAGMENT_LIGHT_MODEL_LOCAL_VIEWER_SGIX',
'GL_FRAGMENT_LIGHT_MODEL_TWO_SIDE_SGIX',
'GL_FRAGMENT_LIGHT_MODEL_AMBIENT_SGIX',
'GL_FRAGMENT_LIGHT_MODEL_NORMAL_INTERPOLATION_SGIX',
'GL_FRAGMENT_LIGHT0_SGIX', 'GL_FRAGMENT_LIGHT1_SGIX',
'GL_FRAGMENT_LIGHT2_SGIX', 'GL_FRAGMENT_LIGHT3_SGIX',
'GL_FRAGMENT_LIGHT4_SGIX', 'GL_FRAGMENT_LIGHT5_SGIX',
'GL_FRAGMENT_LIGHT6_SGIX', 'GL_FRAGMENT_LIGHT7_SGIX',
'GL_RASTER_POSITION_UNCLIPPED_IBM', 'GL_TEXTURE_LIGHTING_MODE_HP',
'GL_TEXTURE_POST_SPECULAR_HP', 'GL_TEXTURE_PRE_SPECULAR_HP',
'GL_MAX_ELEMENTS_VERTICES_EXT', 'GL_MAX_ELEMENTS_INDICES_EXT', 'GL_PHONG_WIN',
'GL_PHONG_HINT_WIN', 'GL_FOG_SPECULAR_TEXTURE_WIN',
'GL_FRAGMENT_MATERIAL_EXT', 'GL_FRAGMENT_NORMAL_EXT', 'GL_FRAGMENT_COLOR_EXT',
'GL_ATTENUATION_EXT', 'GL_SHADOW_ATTENUATION_EXT',
'GL_TEXTURE_APPLICATION_MODE_EXT', 'GL_TEXTURE_LIGHT_EXT',
'GL_TEXTURE_MATERIAL_FACE_EXT', 'GL_TEXTURE_MATERIAL_PARAMETER_EXT',
'GL_ALPHA_MIN_SGIX', 'GL_ALPHA_MAX_SGIX', 'GL_PIXEL_TEX_GEN_Q_CEILING_SGIX',
'GL_PIXEL_TEX_GEN_Q_ROUND_SGIX', 'GL_PIXEL_TEX_GEN_Q_FLOOR_SGIX',
'GL_PIXEL_TEX_GEN_ALPHA_REPLACE_SGIX',
'GL_PIXEL_TEX_GEN_ALPHA_NO_REPLACE_SGIX', 'GL_PIXEL_TEX_GEN_ALPHA_LS_SGIX',
'GL_PIXEL_TEX_GEN_ALPHA_MS_SGIX', 'GL_BGR_EXT', 'GL_BGRA_EXT',
'GL_ASYNC_MARKER_SGIX', 'GL_ASYNC_TEX_IMAGE_SGIX',
'GL_ASYNC_DRAW_PIXELS_SGIX', 'GL_ASYNC_READ_PIXELS_SGIX',
'GL_MAX_ASYNC_TEX_IMAGE_SGIX', 'GL_MAX_ASYNC_DRAW_PIXELS_SGIX',
'GL_MAX_ASYNC_READ_PIXELS_SGIX', 'GL_ASYNC_HISTOGRAM_SGIX',
'GL_MAX_ASYNC_HISTOGRAM_SGIX', 'GL_PARALLEL_ARRAYS_INTEL',
'GL_VERTEX_ARRAY_PARALLEL_POINTERS_INTEL',
'GL_NORMAL_ARRAY_PARALLEL_POINTERS_INTEL',
'GL_COLOR_ARRAY_PARALLEL_POINTERS_INTEL',
'GL_TEXTURE_COORD_ARRAY_PARALLEL_POINTERS_INTEL', 'GL_OCCLUSION_TEST_HP',
'GL_OCCLUSION_TEST_RESULT_HP', 'GL_PIXEL_TRANSFORM_2D_EXT',
'GL_PIXEL_MAG_FILTER_EXT', 'GL_PIXEL_MIN_FILTER_EXT',
'GL_PIXEL_CUBIC_WEIGHT_EXT', 'GL_CUBIC_EXT', 'GL_AVERAGE_EXT',
'GL_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT',
'GL_MAX_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT',
'GL_PIXEL_TRANSFORM_2D_MATRIX_EXT', 'GL_SHARED_TEXTURE_PALETTE_EXT',
'GL_LIGHT_MODEL_COLOR_CONTROL_EXT', 'GL_SINGLE_COLOR_EXT',
'GL_SEPARATE_SPECULAR_COLOR_EXT', 'GL_COLOR_SUM_EXT',
'GL_CURRENT_SECONDARY_COLOR_EXT', 'GL_SECONDARY_COLOR_ARRAY_SIZE_EXT',
'GL_SECONDARY_COLOR_ARRAY_TYPE_EXT', 'GL_SECONDARY_COLOR_ARRAY_STRIDE_EXT',
'GL_SECONDARY_COLOR_ARRAY_POINTER_EXT', 'GL_SECONDARY_COLOR_ARRAY_EXT',
'GL_PERTURB_EXT', 'GL_TEXTURE_NORMAL_EXT', 'GL_FOG_COORDINATE_SOURCE_EXT',
'GL_FOG_COORDINATE_EXT', 'GL_FRAGMENT_DEPTH_EXT',
'GL_CURRENT_FOG_COORDINATE_EXT', 'GL_FOG_COORDINATE_ARRAY_TYPE_EXT',
'GL_FOG_COORDINATE_ARRAY_STRIDE_EXT', 'GL_FOG_COORDINATE_ARRAY_POINTER_EXT',
'GL_FOG_COORDINATE_ARRAY_EXT', 'GL_SCREEN_COORDINATES_REND',
'GL_INVERTED_SCREEN_W_REND', 'GL_TANGENT_ARRAY_EXT', 'GL_BINORMAL_ARRAY_EXT',
'GL_CURRENT_TANGENT_EXT', 'GL_CURRENT_BINORMAL_EXT',
'GL_TANGENT_ARRAY_TYPE_EXT', 'GL_TANGENT_ARRAY_STRIDE_EXT',
'GL_BINORMAL_ARRAY_TYPE_EXT', 'GL_BINORMAL_ARRAY_STRIDE_EXT',
'GL_TANGENT_ARRAY_POINTER_EXT', 'GL_BINORMAL_ARRAY_POINTER_EXT',
'GL_MAP1_TANGENT_EXT', 'GL_MAP2_TANGENT_EXT', 'GL_MAP1_BINORMAL_EXT',
'GL_MAP2_BINORMAL_EXT', 'GL_COMBINE_EXT', 'GL_COMBINE_RGB_EXT',
'GL_COMBINE_ALPHA_EXT', 'GL_RGB_SCALE_EXT', 'GL_ADD_SIGNED_EXT',
'GL_INTERPOLATE_EXT', 'GL_CONSTANT_EXT', 'GL_PRIMARY_COLOR_EXT',
'GL_PREVIOUS_EXT', 'GL_SOURCE0_RGB_EXT', 'GL_SOURCE1_RGB_EXT',
'GL_SOURCE2_RGB_EXT', 'GL_SOURCE0_ALPHA_EXT', 'GL_SOURCE1_ALPHA_EXT',
'GL_SOURCE2_ALPHA_EXT', 'GL_OPERAND0_RGB_EXT', 'GL_OPERAND1_RGB_EXT',
'GL_OPERAND2_RGB_EXT', 'GL_OPERAND0_ALPHA_EXT', 'GL_OPERAND1_ALPHA_EXT',
'GL_OPERAND2_ALPHA_EXT', 'GL_LIGHT_MODEL_SPECULAR_VECTOR_APPLE',
'GL_TRANSFORM_HINT_APPLE', 'GL_FOG_SCALE_SGIX', 'GL_FOG_SCALE_VALUE_SGIX',
'GL_UNPACK_CONSTANT_DATA_SUNX', 'GL_TEXTURE_CONSTANT_DATA_SUNX',
'GL_GLOBAL_ALPHA_SUN', 'GL_GLOBAL_ALPHA_FACTOR_SUN', 'GL_RESTART_SUN',
'GL_REPLACE_MIDDLE_SUN', 'GL_REPLACE_OLDEST_SUN', 'GL_TRIANGLE_LIST_SUN',
'GL_REPLACEMENT_CODE_SUN', 'GL_REPLACEMENT_CODE_ARRAY_SUN',
'GL_REPLACEMENT_CODE_ARRAY_TYPE_SUN', 'GL_REPLACEMENT_CODE_ARRAY_STRIDE_SUN',
'GL_REPLACEMENT_CODE_ARRAY_POINTER_SUN', 'GL_R1UI_V3F_SUN',
'GL_R1UI_C4UB_V3F_SUN', 'GL_R1UI_C3F_V3F_SUN', 'GL_R1UI_N3F_V3F_SUN',
'GL_R1UI_C4F_N3F_V3F_SUN', 'GL_R1UI_T2F_V3F_SUN', 'GL_R1UI_T2F_N3F_V3F_SUN',
'GL_R1UI_T2F_C4F_N3F_V3F_SUN', 'GL_BLEND_DST_RGB_EXT', 'GL_BLEND_SRC_RGB_EXT',
'GL_BLEND_DST_ALPHA_EXT', 'GL_BLEND_SRC_ALPHA_EXT', 'GL_RED_MIN_CLAMP_INGR',
'GL_GREEN_MIN_CLAMP_INGR', 'GL_BLUE_MIN_CLAMP_INGR',
'GL_ALPHA_MIN_CLAMP_INGR', 'GL_RED_MAX_CLAMP_INGR', 'GL_GREEN_MAX_CLAMP_INGR',
'GL_BLUE_MAX_CLAMP_INGR', 'GL_ALPHA_MAX_CLAMP_INGR', 'GL_INTERLACE_READ_INGR',
'GL_INCR_WRAP_EXT', 'GL_DECR_WRAP_EXT', 'GL_422_EXT', 'GL_422_REV_EXT',
'GL_422_AVERAGE_EXT', 'GL_422_REV_AVERAGE_EXT', 'GL_NORMAL_MAP_NV',
'GL_REFLECTION_MAP_NV', 'GL_NORMAL_MAP_EXT', 'GL_REFLECTION_MAP_EXT',
'GL_TEXTURE_CUBE_MAP_EXT', 'GL_TEXTURE_BINDING_CUBE_MAP_EXT',
'GL_TEXTURE_CUBE_MAP_POSITIVE_X_EXT', 'GL_TEXTURE_CUBE_MAP_NEGATIVE_X_EXT',
'GL_TEXTURE_CUBE_MAP_POSITIVE_Y_EXT', 'GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_EXT',
'GL_TEXTURE_CUBE_MAP_POSITIVE_Z_EXT', 'GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_EXT',
'GL_PROXY_TEXTURE_CUBE_MAP_EXT', 'GL_MAX_CUBE_MAP_TEXTURE_SIZE_EXT',
'GL_WRAP_BORDER_SUN', 'GL_MAX_TEXTURE_LOD_BIAS_EXT',
'GL_TEXTURE_FILTER_CONTROL_EXT', 'GL_TEXTURE_LOD_BIAS_EXT',
'GL_TEXTURE_MAX_ANISOTROPY_EXT', 'GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT',
'GL_MODELVIEW0_STACK_DEPTH_EXT', 'GL_MODELVIEW1_STACK_DEPTH_EXT',
'GL_MODELVIEW0_MATRIX_EXT', 'GL_MODELVIEW1_MATRIX_EXT',
'GL_VERTEX_WEIGHTING_EXT', 'GL_MODELVIEW0_EXT', 'GL_MODELVIEW1_EXT',
'GL_CURRENT_VERTEX_WEIGHT_EXT', 'GL_VERTEX_WEIGHT_ARRAY_EXT',
'GL_VERTEX_WEIGHT_ARRAY_SIZE_EXT', 'GL_VERTEX_WEIGHT_ARRAY_TYPE_EXT',
'GL_VERTEX_WEIGHT_ARRAY_STRIDE_EXT', 'GL_VERTEX_WEIGHT_ARRAY_POINTER_EXT',
'GL_MAX_SHININESS_NV', 'GL_MAX_SPOT_EXPONENT_NV', 'GL_VERTEX_ARRAY_RANGE_NV',
'GL_VERTEX_ARRAY_RANGE_LENGTH_NV', 'GL_VERTEX_ARRAY_RANGE_VALID_NV',
'GL_MAX_VERTEX_ARRAY_RANGE_ELEMENT_NV', 'GL_VERTEX_ARRAY_RANGE_POINTER_NV',
'GL_REGISTER_COMBINERS_NV', 'GL_VARIABLE_A_NV', 'GL_VARIABLE_B_NV',
'GL_VARIABLE_C_NV', 'GL_VARIABLE_D_NV', 'GL_VARIABLE_E_NV',
'GL_VARIABLE_F_NV', 'GL_VARIABLE_G_NV', 'GL_CONSTANT_COLOR0_NV',
'GL_CONSTANT_COLOR1_NV', 'GL_PRIMARY_COLOR_NV', 'GL_SECONDARY_COLOR_NV',
'GL_SPARE0_NV', 'GL_SPARE1_NV', 'GL_DISCARD_NV', 'GL_E_TIMES_F_NV',
'GL_SPARE0_PLUS_SECONDARY_COLOR_NV', 'GL_UNSIGNED_IDENTITY_NV',
'GL_UNSIGNED_INVERT_NV', 'GL_EXPAND_NORMAL_NV', 'GL_EXPAND_NEGATE_NV',
'GL_HALF_BIAS_NORMAL_NV', 'GL_HALF_BIAS_NEGATE_NV', 'GL_SIGNED_IDENTITY_NV',
'GL_SIGNED_NEGATE_NV', 'GL_SCALE_BY_TWO_NV', 'GL_SCALE_BY_FOUR_NV',
'GL_SCALE_BY_ONE_HALF_NV', 'GL_BIAS_BY_NEGATIVE_ONE_HALF_NV',
'GL_COMBINER_INPUT_NV', 'GL_COMBINER_MAPPING_NV',
'GL_COMBINER_COMPONENT_USAGE_NV', 'GL_COMBINER_AB_DOT_PRODUCT_NV',
'GL_COMBINER_CD_DOT_PRODUCT_NV', 'GL_COMBINER_MUX_SUM_NV',
'GL_COMBINER_SCALE_NV', 'GL_COMBINER_BIAS_NV', 'GL_COMBINER_AB_OUTPUT_NV',
'GL_COMBINER_CD_OUTPUT_NV', 'GL_COMBINER_SUM_OUTPUT_NV',
'GL_MAX_GENERAL_COMBINERS_NV', 'GL_NUM_GENERAL_COMBINERS_NV',
'GL_COLOR_SUM_CLAMP_NV', 'GL_COMBINER0_NV', 'GL_COMBINER1_NV',
'GL_COMBINER2_NV', 'GL_COMBINER3_NV', 'GL_COMBINER4_NV', 'GL_COMBINER5_NV',
'GL_COMBINER6_NV', 'GL_COMBINER7_NV', 'GL_FOG_DISTANCE_MODE_NV',
'GL_EYE_RADIAL_NV', 'GL_EYE_PLANE_ABSOLUTE_NV', 'GL_EMBOSS_LIGHT_NV',
'GL_EMBOSS_CONSTANT_NV', 'GL_EMBOSS_MAP_NV', 'GL_COMBINE4_NV',
'GL_SOURCE3_RGB_NV', 'GL_SOURCE3_ALPHA_NV', 'GL_OPERAND3_RGB_NV',
'GL_OPERAND3_ALPHA_NV', 'GL_COMPRESSED_RGB_S3TC_DXT1_EXT',
'GL_COMPRESSED_RGBA_S3TC_DXT1_EXT', 'GL_COMPRESSED_RGBA_S3TC_DXT3_EXT',
'GL_COMPRESSED_RGBA_S3TC_DXT5_EXT', 'GL_CULL_VERTEX_IBM',
'GL_VERTEX_ARRAY_LIST_IBM', 'GL_NORMAL_ARRAY_LIST_IBM',
'GL_COLOR_ARRAY_LIST_IBM', 'GL_INDEX_ARRAY_LIST_IBM',
'GL_TEXTURE_COORD_ARRAY_LIST_IBM', 'GL_EDGE_FLAG_ARRAY_LIST_IBM',
'GL_FOG_COORDINATE_ARRAY_LIST_IBM', 'GL_SECONDARY_COLOR_ARRAY_LIST_IBM',
'GL_VERTEX_ARRAY_LIST_STRIDE_IBM', 'GL_NORMAL_ARRAY_LIST_STRIDE_IBM',
'GL_COLOR_ARRAY_LIST_STRIDE_IBM', 'GL_INDEX_ARRAY_LIST_STRIDE_IBM',
'GL_TEXTURE_COORD_ARRAY_LIST_STRIDE_IBM',
'GL_EDGE_FLAG_ARRAY_LIST_STRIDE_IBM',
'GL_FOG_COORDINATE_ARRAY_LIST_STRIDE_IBM',
'GL_SECONDARY_COLOR_ARRAY_LIST_STRIDE_IBM', 'GL_PACK_SUBSAMPLE_RATE_SGIX',
'GL_UNPACK_SUBSAMPLE_RATE_SGIX', 'GL_PIXEL_SUBSAMPLE_4444_SGIX',
'GL_PIXEL_SUBSAMPLE_2424_SGIX', 'GL_PIXEL_SUBSAMPLE_4242_SGIX',
'GL_YCRCB_SGIX', 'GL_YCRCBA_SGIX', 'GL_DEPTH_PASS_INSTRUMENT_SGIX',
'GL_DEPTH_PASS_INSTRUMENT_COUNTERS_SGIX', 'GL_DEPTH_PASS_INSTRUMENT_MAX_SGIX',
'GL_COMPRESSED_RGB_FXT1_3DFX', 'GL_COMPRESSED_RGBA_FXT1_3DFX',
'GL_MULTISAMPLE_3DFX', 'GL_SAMPLE_BUFFERS_3DFX', 'GL_SAMPLES_3DFX',
'GL_MULTISAMPLE_BIT_3DFX', 'GL_MULTISAMPLE_EXT',
'GL_SAMPLE_ALPHA_TO_MASK_EXT', 'GL_SAMPLE_ALPHA_TO_ONE_EXT',
'GL_SAMPLE_MASK_EXT', 'GL_1PASS_EXT', 'GL_2PASS_0_EXT', 'GL_2PASS_1_EXT',
'GL_4PASS_0_EXT', 'GL_4PASS_1_EXT', 'GL_4PASS_2_EXT', 'GL_4PASS_3_EXT',
'GL_SAMPLE_BUFFERS_EXT', 'GL_SAMPLES_EXT', 'GL_SAMPLE_MASK_VALUE_EXT',
'GL_SAMPLE_MASK_INVERT_EXT', 'GL_SAMPLE_PATTERN_EXT',
'GL_MULTISAMPLE_BIT_EXT', 'GL_VERTEX_PRECLIP_SGIX',
'GL_VERTEX_PRECLIP_HINT_SGIX', 'GL_CONVOLUTION_HINT_SGIX',
'GL_PACK_RESAMPLE_SGIX', 'GL_UNPACK_RESAMPLE_SGIX',
'GL_RESAMPLE_REPLICATE_SGIX', 'GL_RESAMPLE_ZERO_FILL_SGIX',
'GL_RESAMPLE_DECIMATE_SGIX', 'GL_EYE_DISTANCE_TO_POINT_SGIS',
'GL_OBJECT_DISTANCE_TO_POINT_SGIS', 'GL_EYE_DISTANCE_TO_LINE_SGIS',
'GL_OBJECT_DISTANCE_TO_LINE_SGIS', 'GL_EYE_POINT_SGIS',
'GL_OBJECT_POINT_SGIS', 'GL_EYE_LINE_SGIS', 'GL_OBJECT_LINE_SGIS',
'GL_TEXTURE_COLOR_WRITEMASK_SGIS', 'GL_DOT3_RGB_EXT', 'GL_DOT3_RGBA_EXT',
'GL_MIRROR_CLAMP_ATI', 'GL_MIRROR_CLAMP_TO_EDGE_ATI', 'GL_ALL_COMPLETED_NV',
'GL_FENCE_STATUS_NV', 'GL_FENCE_CONDITION_NV', 'GL_MIRRORED_REPEAT_IBM',
'GL_EVAL_2D_NV', 'GL_EVAL_TRIANGULAR_2D_NV', 'GL_MAP_TESSELLATION_NV',
'GL_MAP_ATTRIB_U_ORDER_NV', 'GL_MAP_ATTRIB_V_ORDER_NV',
'GL_EVAL_FRACTIONAL_TESSELLATION_NV', 'GL_EVAL_VERTEX_ATTRIB0_NV',
'GL_EVAL_VERTEX_ATTRIB1_NV', 'GL_EVAL_VERTEX_ATTRIB2_NV',
'GL_EVAL_VERTEX_ATTRIB3_NV', 'GL_EVAL_VERTEX_ATTRIB4_NV',
'GL_EVAL_VERTEX_ATTRIB5_NV', 'GL_EVAL_VERTEX_ATTRIB6_NV',
'GL_EVAL_VERTEX_ATTRIB7_NV', 'GL_EVAL_VERTEX_ATTRIB8_NV',
'GL_EVAL_VERTEX_ATTRIB9_NV', 'GL_EVAL_VERTEX_ATTRIB10_NV',
'GL_EVAL_VERTEX_ATTRIB11_NV', 'GL_EVAL_VERTEX_ATTRIB12_NV',
'GL_EVAL_VERTEX_ATTRIB13_NV', 'GL_EVAL_VERTEX_ATTRIB14_NV',
'GL_EVAL_VERTEX_ATTRIB15_NV', 'GL_MAX_MAP_TESSELLATION_NV',
'GL_MAX_RATIONAL_EVAL_ORDER_NV', 'GL_DEPTH_STENCIL_NV',
'GL_UNSIGNED_INT_24_8_NV', 'GL_DEPTH_STENCIL_EXT', 'GL_DEPTH24_STENCIL8_EXT',
'GL_TEXTURE_STENCIL_SIZE_EXT', 'GL_UNSIGNED_INT_24_8_EXT',
'GL_PER_STAGE_CONSTANTS_NV', 'GL_TEXTURE_RECTANGLE_NV',
'GL_TEXTURE_BINDING_RECTANGLE_NV', 'GL_PROXY_TEXTURE_RECTANGLE_NV',
'GL_MAX_RECTANGLE_TEXTURE_SIZE_NV', 'GL_OFFSET_TEXTURE_RECTANGLE_NV',
'GL_OFFSET_TEXTURE_RECTANGLE_SCALE_NV', 'GL_DOT_PRODUCT_TEXTURE_RECTANGLE_NV',
'GL_RGBA_UNSIGNED_DOT_PRODUCT_MAPPING_NV', 'GL_UNSIGNED_INT_S8_S8_8_8_NV',
'GL_UNSIGNED_INT_8_8_S8_S8_REV_NV', 'GL_DSDT_MAG_INTENSITY_NV',
'GL_SHADER_CONSISTENT_NV', 'GL_TEXTURE_SHADER_NV', 'GL_SHADER_OPERATION_NV',
'GL_CULL_MODES_NV', 'GL_OFFSET_TEXTURE_MATRIX_NV',
'GL_OFFSET_TEXTURE_SCALE_NV', 'GL_OFFSET_TEXTURE_BIAS_NV',
'GL_OFFSET_TEXTURE_2D_MATRIX_NV', 'GL_OFFSET_TEXTURE_2D_SCALE_NV',
'GL_OFFSET_TEXTURE_2D_BIAS_NV', 'GL_PREVIOUS_TEXTURE_INPUT_NV',
'GL_CONST_EYE_NV', 'GL_PASS_THROUGH_NV', 'GL_CULL_FRAGMENT_NV',
'GL_OFFSET_TEXTURE_2D_NV', 'GL_DEPENDENT_AR_TEXTURE_2D_NV',
'GL_DEPENDENT_GB_TEXTURE_2D_NV', 'GL_DOT_PRODUCT_NV',
'GL_DOT_PRODUCT_DEPTH_REPLACE_NV', 'GL_DOT_PRODUCT_TEXTURE_2D_NV',
'GL_DOT_PRODUCT_TEXTURE_CUBE_MAP_NV', 'GL_DOT_PRODUCT_DIFFUSE_CUBE_MAP_NV',
'GL_DOT_PRODUCT_REFLECT_CUBE_MAP_NV',
'GL_DOT_PRODUCT_CONST_EYE_REFLECT_CUBE_MAP_NV', 'GL_HILO_NV', 'GL_DSDT_NV',
'GL_DSDT_MAG_NV', 'GL_DSDT_MAG_VIB_NV', 'GL_HILO16_NV', 'GL_SIGNED_HILO_NV',
'GL_SIGNED_HILO16_NV', 'GL_SIGNED_RGBA_NV', 'GL_SIGNED_RGBA8_NV',
'GL_SIGNED_RGB_NV', 'GL_SIGNED_RGB8_NV', 'GL_SIGNED_LUMINANCE_NV',
'GL_SIGNED_LUMINANCE8_NV', 'GL_SIGNED_LUMINANCE_ALPHA_NV',
'GL_SIGNED_LUMINANCE8_ALPHA8_NV', 'GL_SIGNED_ALPHA_NV', 'GL_SIGNED_ALPHA8_NV',
'GL_SIGNED_INTENSITY_NV', 'GL_SIGNED_INTENSITY8_NV', 'GL_DSDT8_NV',
'GL_DSDT8_MAG8_NV', 'GL_DSDT8_MAG8_INTENSITY8_NV',
'GL_SIGNED_RGB_UNSIGNED_ALPHA_NV', 'GL_SIGNED_RGB8_UNSIGNED_ALPHA8_NV',
'GL_HI_SCALE_NV', 'GL_LO_SCALE_NV', 'GL_DS_SCALE_NV', 'GL_DT_SCALE_NV',
'GL_MAGNITUDE_SCALE_NV', 'GL_VIBRANCE_SCALE_NV', 'GL_HI_BIAS_NV',
'GL_LO_BIAS_NV', 'GL_DS_BIAS_NV', 'GL_DT_BIAS_NV', 'GL_MAGNITUDE_BIAS_NV',
'GL_VIBRANCE_BIAS_NV', 'GL_TEXTURE_BORDER_VALUES_NV', 'GL_TEXTURE_HI_SIZE_NV',
'GL_TEXTURE_LO_SIZE_NV', 'GL_TEXTURE_DS_SIZE_NV', 'GL_TEXTURE_DT_SIZE_NV',
'GL_TEXTURE_MAG_SIZE_NV', 'GL_DOT_PRODUCT_TEXTURE_3D_NV',
'GL_VERTEX_ARRAY_RANGE_WITHOUT_FLUSH_NV', 'GL_VERTEX_PROGRAM_NV',
'GL_VERTEX_STATE_PROGRAM_NV', 'GL_ATTRIB_ARRAY_SIZE_NV',
'GL_ATTRIB_ARRAY_STRIDE_NV', 'GL_ATTRIB_ARRAY_TYPE_NV',
'GL_CURRENT_ATTRIB_NV', 'GL_PROGRAM_LENGTH_NV', 'GL_PROGRAM_STRING_NV',
'GL_MODELVIEW_PROJECTION_NV', 'GL_IDENTITY_NV', 'GL_INVERSE_NV',
'GL_TRANSPOSE_NV', 'GL_INVERSE_TRANSPOSE_NV',
'GL_MAX_TRACK_MATRIX_STACK_DEPTH_NV', 'GL_MAX_TRACK_MATRICES_NV',
'GL_MATRIX0_NV', 'GL_MATRIX1_NV', 'GL_MATRIX2_NV', 'GL_MATRIX3_NV',
'GL_MATRIX4_NV', 'GL_MATRIX5_NV', 'GL_MATRIX6_NV', 'GL_MATRIX7_NV',
'GL_CURRENT_MATRIX_STACK_DEPTH_NV', 'GL_CURRENT_MATRIX_NV',
'GL_VERTEX_PROGRAM_POINT_SIZE_NV', 'GL_VERTEX_PROGRAM_TWO_SIDE_NV',
'GL_PROGRAM_PARAMETER_NV', 'GL_ATTRIB_ARRAY_POINTER_NV',
'GL_PROGRAM_TARGET_NV', 'GL_PROGRAM_RESIDENT_NV', 'GL_TRACK_MATRIX_NV',
'GL_TRACK_MATRIX_TRANSFORM_NV', 'GL_VERTEX_PROGRAM_BINDING_NV',
'GL_PROGRAM_ERROR_POSITION_NV', 'GL_VERTEX_ATTRIB_ARRAY0_NV',
'GL_VERTEX_ATTRIB_ARRAY1_NV', 'GL_VERTEX_ATTRIB_ARRAY2_NV',
'GL_VERTEX_ATTRIB_ARRAY3_NV', 'GL_VERTEX_ATTRIB_ARRAY4_NV',
'GL_VERTEX_ATTRIB_ARRAY5_NV', 'GL_VERTEX_ATTRIB_ARRAY6_NV',
'GL_VERTEX_ATTRIB_ARRAY7_NV', 'GL_VERTEX_ATTRIB_ARRAY8_NV',
'GL_VERTEX_ATTRIB_ARRAY9_NV', 'GL_VERTEX_ATTRIB_ARRAY10_NV',
'GL_VERTEX_ATTRIB_ARRAY11_NV', 'GL_VERTEX_ATTRIB_ARRAY12_NV',
'GL_VERTEX_ATTRIB_ARRAY13_NV', 'GL_VERTEX_ATTRIB_ARRAY14_NV',
'GL_VERTEX_ATTRIB_ARRAY15_NV', 'GL_MAP1_VERTEX_ATTRIB0_4_NV',
'GL_MAP1_VERTEX_ATTRIB1_4_NV', 'GL_MAP1_VERTEX_ATTRIB2_4_NV',
'GL_MAP1_VERTEX_ATTRIB3_4_NV', 'GL_MAP1_VERTEX_ATTRIB4_4_NV',
'GL_MAP1_VERTEX_ATTRIB5_4_NV', 'GL_MAP1_VERTEX_ATTRIB6_4_NV',
'GL_MAP1_VERTEX_ATTRIB7_4_NV', 'GL_MAP1_VERTEX_ATTRIB8_4_NV',
'GL_MAP1_VERTEX_ATTRIB9_4_NV', 'GL_MAP1_VERTEX_ATTRIB10_4_NV',
'GL_MAP1_VERTEX_ATTRIB11_4_NV', 'GL_MAP1_VERTEX_ATTRIB12_4_NV',
'GL_MAP1_VERTEX_ATTRIB13_4_NV', 'GL_MAP1_VERTEX_ATTRIB14_4_NV',
'GL_MAP1_VERTEX_ATTRIB15_4_NV', 'GL_MAP2_VERTEX_ATTRIB0_4_NV',
'GL_MAP2_VERTEX_ATTRIB1_4_NV', 'GL_MAP2_VERTEX_ATTRIB2_4_NV',
'GL_MAP2_VERTEX_ATTRIB3_4_NV', 'GL_MAP2_VERTEX_ATTRIB4_4_NV',
'GL_MAP2_VERTEX_ATTRIB5_4_NV', 'GL_MAP2_VERTEX_ATTRIB6_4_NV',
'GL_MAP2_VERTEX_ATTRIB7_4_NV', 'GL_MAP2_VERTEX_ATTRIB8_4_NV',
'GL_MAP2_VERTEX_ATTRIB9_4_NV', 'GL_MAP2_VERTEX_ATTRIB10_4_NV',
'GL_MAP2_VERTEX_ATTRIB11_4_NV', 'GL_MAP2_VERTEX_ATTRIB12_4_NV',
'GL_MAP2_VERTEX_ATTRIB13_4_NV', 'GL_MAP2_VERTEX_ATTRIB14_4_NV',
'GL_MAP2_VERTEX_ATTRIB15_4_NV', 'GL_TEXTURE_MAX_CLAMP_S_SGIX',
'GL_TEXTURE_MAX_CLAMP_T_SGIX', 'GL_TEXTURE_MAX_CLAMP_R_SGIX',
'GL_SCALEBIAS_HINT_SGIX', 'GL_INTERLACE_OML', 'GL_INTERLACE_READ_OML',
'GL_FORMAT_SUBSAMPLE_24_24_OML', 'GL_FORMAT_SUBSAMPLE_244_244_OML',
'GL_PACK_RESAMPLE_OML', 'GL_UNPACK_RESAMPLE_OML', 'GL_RESAMPLE_REPLICATE_OML',
'GL_RESAMPLE_ZERO_FILL_OML', 'GL_RESAMPLE_AVERAGE_OML',
'GL_RESAMPLE_DECIMATE_OML', 'GL_DEPTH_STENCIL_TO_RGBA_NV',
'GL_DEPTH_STENCIL_TO_BGRA_NV', 'GL_BUMP_ROT_MATRIX_ATI',
'GL_BUMP_ROT_MATRIX_SIZE_ATI', 'GL_BUMP_NUM_TEX_UNITS_ATI',
'GL_BUMP_TEX_UNITS_ATI', 'GL_DUDV_ATI', 'GL_DU8DV8_ATI', 'GL_BUMP_ENVMAP_ATI',
'GL_BUMP_TARGET_ATI', 'GL_FRAGMENT_SHADER_ATI', 'GL_REG_0_ATI',
'GL_REG_1_ATI', 'GL_REG_2_ATI', 'GL_REG_3_ATI', 'GL_REG_4_ATI',
'GL_REG_5_ATI', 'GL_REG_6_ATI', 'GL_REG_7_ATI', 'GL_REG_8_ATI',
'GL_REG_9_ATI', 'GL_REG_10_ATI', 'GL_REG_11_ATI', 'GL_REG_12_ATI',
'GL_REG_13_ATI', 'GL_REG_14_ATI', 'GL_REG_15_ATI', 'GL_REG_16_ATI',
'GL_REG_17_ATI', 'GL_REG_18_ATI', 'GL_REG_19_ATI', 'GL_REG_20_ATI',
'GL_REG_21_ATI', 'GL_REG_22_ATI', 'GL_REG_23_ATI', 'GL_REG_24_ATI',
'GL_REG_25_ATI', 'GL_REG_26_ATI', 'GL_REG_27_ATI', 'GL_REG_28_ATI',
'GL_REG_29_ATI', 'GL_REG_30_ATI', 'GL_REG_31_ATI', 'GL_CON_0_ATI',
'GL_CON_1_ATI', 'GL_CON_2_ATI', 'GL_CON_3_ATI', 'GL_CON_4_ATI',
'GL_CON_5_ATI', 'GL_CON_6_ATI', 'GL_CON_7_ATI', 'GL_CON_8_ATI',
'GL_CON_9_ATI', 'GL_CON_10_ATI', 'GL_CON_11_ATI', 'GL_CON_12_ATI',
'GL_CON_13_ATI', 'GL_CON_14_ATI', 'GL_CON_15_ATI', 'GL_CON_16_ATI',
'GL_CON_17_ATI', 'GL_CON_18_ATI', 'GL_CON_19_ATI', 'GL_CON_20_ATI',
'GL_CON_21_ATI', 'GL_CON_22_ATI', 'GL_CON_23_ATI', 'GL_CON_24_ATI',
'GL_CON_25_ATI', 'GL_CON_26_ATI', 'GL_CON_27_ATI', 'GL_CON_28_ATI',
'GL_CON_29_ATI', 'GL_CON_30_ATI', 'GL_CON_31_ATI', 'GL_MOV_ATI', 'GL_ADD_ATI',
'GL_MUL_ATI', 'GL_SUB_ATI', 'GL_DOT3_ATI', 'GL_DOT4_ATI', 'GL_MAD_ATI',
'GL_LERP_ATI', 'GL_CND_ATI', 'GL_CND0_ATI', 'GL_DOT2_ADD_ATI',
'GL_SECONDARY_INTERPOLATOR_ATI', 'GL_NUM_FRAGMENT_REGISTERS_ATI',
'GL_NUM_FRAGMENT_CONSTANTS_ATI', 'GL_NUM_PASSES_ATI',
'GL_NUM_INSTRUCTIONS_PER_PASS_ATI', 'GL_NUM_INSTRUCTIONS_TOTAL_ATI',
'GL_NUM_INPUT_INTERPOLATOR_COMPONENTS_ATI', 'GL_NUM_LOOPBACK_COMPONENTS_ATI',
'GL_COLOR_ALPHA_PAIRING_ATI', 'GL_SWIZZLE_STR_ATI', 'GL_SWIZZLE_STQ_ATI',
'GL_SWIZZLE_STR_DR_ATI', 'GL_SWIZZLE_STQ_DQ_ATI', 'GL_SWIZZLE_STRQ_ATI',
'GL_SWIZZLE_STRQ_DQ_ATI', 'GL_RED_BIT_ATI', 'GL_GREEN_BIT_ATI',
'GL_BLUE_BIT_ATI', 'GL_2X_BIT_ATI', 'GL_4X_BIT_ATI', 'GL_8X_BIT_ATI',
'GL_HALF_BIT_ATI', 'GL_QUARTER_BIT_ATI', 'GL_EIGHTH_BIT_ATI',
'GL_SATURATE_BIT_ATI', 'GL_COMP_BIT_ATI', 'GL_NEGATE_BIT_ATI',
'GL_BIAS_BIT_ATI', 'GL_PN_TRIANGLES_ATI',
'GL_MAX_PN_TRIANGLES_TESSELATION_LEVEL_ATI', 'GL_PN_TRIANGLES_POINT_MODE_ATI',
'GL_PN_TRIANGLES_NORMAL_MODE_ATI', 'GL_PN_TRIANGLES_TESSELATION_LEVEL_ATI',
'GL_PN_TRIANGLES_POINT_MODE_LINEAR_ATI',
'GL_PN_TRIANGLES_POINT_MODE_CUBIC_ATI',
'GL_PN_TRIANGLES_NORMAL_MODE_LINEAR_ATI',
'GL_PN_TRIANGLES_NORMAL_MODE_QUADRATIC_ATI', 'GL_STATIC_ATI',
'GL_DYNAMIC_ATI', 'GL_PRESERVE_ATI', 'GL_DISCARD_ATI',
'GL_OBJECT_BUFFER_SIZE_ATI', 'GL_OBJECT_BUFFER_USAGE_ATI',
'GL_ARRAY_OBJECT_BUFFER_ATI', 'GL_ARRAY_OBJECT_OFFSET_ATI',
'GL_VERTEX_SHADER_EXT', 'GL_VERTEX_SHADER_BINDING_EXT', 'GL_OP_INDEX_EXT',
'GL_OP_NEGATE_EXT', 'GL_OP_DOT3_EXT', 'GL_OP_DOT4_EXT', 'GL_OP_MUL_EXT',
'GL_OP_ADD_EXT', 'GL_OP_MADD_EXT', 'GL_OP_FRAC_EXT', 'GL_OP_MAX_EXT',
'GL_OP_MIN_EXT', 'GL_OP_SET_GE_EXT', 'GL_OP_SET_LT_EXT', 'GL_OP_CLAMP_EXT',
'GL_OP_FLOOR_EXT', 'GL_OP_ROUND_EXT', 'GL_OP_EXP_BASE_2_EXT',
'GL_OP_LOG_BASE_2_EXT', 'GL_OP_POWER_EXT', 'GL_OP_RECIP_EXT',
'GL_OP_RECIP_SQRT_EXT', 'GL_OP_SUB_EXT', 'GL_OP_CROSS_PRODUCT_EXT',
'GL_OP_MULTIPLY_MATRIX_EXT', 'GL_OP_MOV_EXT', 'GL_OUTPUT_VERTEX_EXT',
'GL_OUTPUT_COLOR0_EXT', 'GL_OUTPUT_COLOR1_EXT',
'GL_OUTPUT_TEXTURE_COORD0_EXT', 'GL_OUTPUT_TEXTURE_COORD1_EXT',
'GL_OUTPUT_TEXTURE_COORD2_EXT', 'GL_OUTPUT_TEXTURE_COORD3_EXT',
'GL_OUTPUT_TEXTURE_COORD4_EXT', 'GL_OUTPUT_TEXTURE_COORD5_EXT',
'GL_OUTPUT_TEXTURE_COORD6_EXT', 'GL_OUTPUT_TEXTURE_COORD7_EXT',
'GL_OUTPUT_TEXTURE_COORD8_EXT', 'GL_OUTPUT_TEXTURE_COORD9_EXT',
'GL_OUTPUT_TEXTURE_COORD10_EXT', 'GL_OUTPUT_TEXTURE_COORD11_EXT',
'GL_OUTPUT_TEXTURE_COORD12_EXT', 'GL_OUTPUT_TEXTURE_COORD13_EXT',
'GL_OUTPUT_TEXTURE_COORD14_EXT', 'GL_OUTPUT_TEXTURE_COORD15_EXT',
'GL_OUTPUT_TEXTURE_COORD16_EXT', 'GL_OUTPUT_TEXTURE_COORD17_EXT',
'GL_OUTPUT_TEXTURE_COORD18_EXT', 'GL_OUTPUT_TEXTURE_COORD19_EXT',
'GL_OUTPUT_TEXTURE_COORD20_EXT', 'GL_OUTPUT_TEXTURE_COORD21_EXT',
'GL_OUTPUT_TEXTURE_COORD22_EXT', 'GL_OUTPUT_TEXTURE_COORD23_EXT',
'GL_OUTPUT_TEXTURE_COORD24_EXT', 'GL_OUTPUT_TEXTURE_COORD25_EXT',
'GL_OUTPUT_TEXTURE_COORD26_EXT', 'GL_OUTPUT_TEXTURE_COORD27_EXT',
'GL_OUTPUT_TEXTURE_COORD28_EXT', 'GL_OUTPUT_TEXTURE_COORD29_EXT',
'GL_OUTPUT_TEXTURE_COORD30_EXT', 'GL_OUTPUT_TEXTURE_COORD31_EXT',
'GL_OUTPUT_FOG_EXT', 'GL_SCALAR_EXT', 'GL_VECTOR_EXT', 'GL_MATRIX_EXT',
'GL_VARIANT_EXT', 'GL_INVARIANT_EXT', 'GL_LOCAL_CONSTANT_EXT', 'GL_LOCAL_EXT',
'GL_MAX_VERTEX_SHADER_INSTRUCTIONS_EXT', 'GL_MAX_VERTEX_SHADER_VARIANTS_EXT',
'GL_MAX_VERTEX_SHADER_INVARIANTS_EXT',
'GL_MAX_VERTEX_SHADER_LOCAL_CONSTANTS_EXT', 'GL_MAX_VERTEX_SHADER_LOCALS_EXT',
'GL_MAX_OPTIMIZED_VERTEX_SHADER_INSTRUCTIONS_EXT',
'GL_MAX_OPTIMIZED_VERTEX_SHADER_VARIANTS_EXT',
'GL_MAX_OPTIMIZED_VERTEX_SHADER_LOCAL_CONSTANTS_EXT',
'GL_MAX_OPTIMIZED_VERTEX_SHADER_INVARIANTS_EXT',
'GL_MAX_OPTIMIZED_VERTEX_SHADER_LOCALS_EXT',
'GL_VERTEX_SHADER_INSTRUCTIONS_EXT', 'GL_VERTEX_SHADER_VARIANTS_EXT',
'GL_VERTEX_SHADER_INVARIANTS_EXT', 'GL_VERTEX_SHADER_LOCAL_CONSTANTS_EXT',
'GL_VERTEX_SHADER_LOCALS_EXT', 'GL_VERTEX_SHADER_OPTIMIZED_EXT', 'GL_X_EXT',
'GL_Y_EXT', 'GL_Z_EXT', 'GL_W_EXT', 'GL_NEGATIVE_X_EXT', 'GL_NEGATIVE_Y_EXT',
'GL_NEGATIVE_Z_EXT', 'GL_NEGATIVE_W_EXT', 'GL_ZERO_EXT', 'GL_ONE_EXT',
'GL_NEGATIVE_ONE_EXT', 'GL_NORMALIZED_RANGE_EXT', 'GL_FULL_RANGE_EXT',
'GL_CURRENT_VERTEX_EXT', 'GL_MVP_MATRIX_EXT', 'GL_VARIANT_VALUE_EXT',
'GL_VARIANT_DATATYPE_EXT', 'GL_VARIANT_ARRAY_STRIDE_EXT',
'GL_VARIANT_ARRAY_TYPE_EXT', 'GL_VARIANT_ARRAY_EXT',
'GL_VARIANT_ARRAY_POINTER_EXT', 'GL_INVARIANT_VALUE_EXT',
'GL_INVARIANT_DATATYPE_EXT', 'GL_LOCAL_CONSTANT_VALUE_EXT',
'GL_LOCAL_CONSTANT_DATATYPE_EXT', 'GL_MAX_VERTEX_STREAMS_ATI',
'GL_VERTEX_STREAM0_ATI', 'GL_VERTEX_STREAM1_ATI', 'GL_VERTEX_STREAM2_ATI',
'GL_VERTEX_STREAM3_ATI', 'GL_VERTEX_STREAM4_ATI', 'GL_VERTEX_STREAM5_ATI',
'GL_VERTEX_STREAM6_ATI', 'GL_VERTEX_STREAM7_ATI', 'GL_VERTEX_SOURCE_ATI',
'GL_ELEMENT_ARRAY_ATI', 'GL_ELEMENT_ARRAY_TYPE_ATI',
'GL_ELEMENT_ARRAY_POINTER_ATI', 'GL_QUAD_MESH_SUN', 'GL_TRIANGLE_MESH_SUN',
'GL_SLICE_ACCUM_SUN', 'GL_MULTISAMPLE_FILTER_HINT_NV', 'GL_DEPTH_CLAMP_NV',
'GL_PIXEL_COUNTER_BITS_NV', 'GL_CURRENT_OCCLUSION_QUERY_ID_NV',
'GL_PIXEL_COUNT_NV', 'GL_PIXEL_COUNT_AVAILABLE_NV', 'GL_POINT_SPRITE_NV',
'GL_COORD_REPLACE_NV', 'GL_POINT_SPRITE_R_MODE_NV',
'GL_OFFSET_PROJECTIVE_TEXTURE_2D_NV',
'GL_OFFSET_PROJECTIVE_TEXTURE_2D_SCALE_NV',
'GL_OFFSET_PROJECTIVE_TEXTURE_RECTANGLE_NV',
'GL_OFFSET_PROJECTIVE_TEXTURE_RECTANGLE_SCALE_NV',
'GL_OFFSET_HILO_TEXTURE_2D_NV', 'GL_OFFSET_HILO_TEXTURE_RECTANGLE_NV',
'GL_OFFSET_HILO_PROJECTIVE_TEXTURE_2D_NV',
'GL_OFFSET_HILO_PROJECTIVE_TEXTURE_RECTANGLE_NV',
'GL_DEPENDENT_HILO_TEXTURE_2D_NV', 'GL_DEPENDENT_RGB_TEXTURE_3D_NV',
'GL_DEPENDENT_RGB_TEXTURE_CUBE_MAP_NV', 'GL_DOT_PRODUCT_PASS_THROUGH_NV',
'GL_DOT_PRODUCT_TEXTURE_1D_NV', 'GL_DOT_PRODUCT_AFFINE_DEPTH_REPLACE_NV',
'GL_HILO8_NV', 'GL_SIGNED_HILO8_NV', 'GL_FORCE_BLUE_TO_ONE_NV',
'GL_STENCIL_TEST_TWO_SIDE_EXT', 'GL_ACTIVE_STENCIL_FACE_EXT',
'GL_TEXT_FRAGMENT_SHADER_ATI', 'GL_UNPACK_CLIENT_STORAGE_APPLE',
'GL_ELEMENT_ARRAY_APPLE', 'GL_ELEMENT_ARRAY_TYPE_APPLE',
'GL_ELEMENT_ARRAY_POINTER_APPLE', 'GL_DRAW_PIXELS_APPLE', 'GL_FENCE_APPLE',
'GL_VERTEX_ARRAY_BINDING_APPLE', 'GL_VERTEX_ARRAY_RANGE_APPLE',
'GL_VERTEX_ARRAY_RANGE_LENGTH_APPLE', 'GL_VERTEX_ARRAY_STORAGE_HINT_APPLE',
'GL_VERTEX_ARRAY_RANGE_POINTER_APPLE', 'GL_STORAGE_CACHED_APPLE',
'GL_STORAGE_SHARED_APPLE', 'GL_YCBCR_422_APPLE',
'GL_UNSIGNED_SHORT_8_8_APPLE', 'GL_UNSIGNED_SHORT_8_8_REV_APPLE',
'GL_RGB_S3TC', 'GL_RGB4_S3TC', 'GL_RGBA_S3TC', 'GL_RGBA4_S3TC',
'GL_MAX_DRAW_BUFFERS_ATI', 'GL_DRAW_BUFFER0_ATI', 'GL_DRAW_BUFFER1_ATI',
'GL_DRAW_BUFFER2_ATI', 'GL_DRAW_BUFFER3_ATI', 'GL_DRAW_BUFFER4_ATI',
'GL_DRAW_BUFFER5_ATI', 'GL_DRAW_BUFFER6_ATI', 'GL_DRAW_BUFFER7_ATI',
'GL_DRAW_BUFFER8_ATI', 'GL_DRAW_BUFFER9_ATI', 'GL_DRAW_BUFFER10_ATI',
'GL_DRAW_BUFFER11_ATI', 'GL_DRAW_BUFFER12_ATI', 'GL_DRAW_BUFFER13_ATI',
'GL_DRAW_BUFFER14_ATI', 'GL_DRAW_BUFFER15_ATI', 'GL_TYPE_RGBA_FLOAT_ATI',
'GL_COLOR_CLEAR_UNCLAMPED_VALUE_ATI', 'GL_MODULATE_ADD_ATI',
'GL_MODULATE_SIGNED_ADD_ATI', 'GL_MODULATE_SUBTRACT_ATI',
'GL_RGBA_FLOAT32_ATI', 'GL_RGB_FLOAT32_ATI', 'GL_ALPHA_FLOAT32_ATI',
'GL_INTENSITY_FLOAT32_ATI', 'GL_LUMINANCE_FLOAT32_ATI',
'GL_LUMINANCE_ALPHA_FLOAT32_ATI', 'GL_RGBA_FLOAT16_ATI', 'GL_RGB_FLOAT16_ATI',
'GL_ALPHA_FLOAT16_ATI', 'GL_INTENSITY_FLOAT16_ATI',
'GL_LUMINANCE_FLOAT16_ATI', 'GL_LUMINANCE_ALPHA_FLOAT16_ATI', 'GL_FLOAT_R_NV',
'GL_FLOAT_RG_NV', 'GL_FLOAT_RGB_NV', 'GL_FLOAT_RGBA_NV', 'GL_FLOAT_R16_NV',
'GL_FLOAT_R32_NV', 'GL_FLOAT_RG16_NV', 'GL_FLOAT_RG32_NV',
'GL_FLOAT_RGB16_NV', 'GL_FLOAT_RGB32_NV', 'GL_FLOAT_RGBA16_NV',
'GL_FLOAT_RGBA32_NV', 'GL_TEXTURE_FLOAT_COMPONENTS_NV',
'GL_FLOAT_CLEAR_COLOR_VALUE_NV', 'GL_FLOAT_RGBA_MODE_NV',
'GL_MAX_FRAGMENT_PROGRAM_LOCAL_PARAMETERS_NV', 'GL_FRAGMENT_PROGRAM_NV',
'GL_MAX_TEXTURE_COORDS_NV', 'GL_MAX_TEXTURE_IMAGE_UNITS_NV',
'GL_FRAGMENT_PROGRAM_BINDING_NV', 'GL_PROGRAM_ERROR_STRING_NV',
'GL_HALF_FLOAT_NV', 'GL_WRITE_PIXEL_DATA_RANGE_NV',
'GL_READ_PIXEL_DATA_RANGE_NV', 'GL_WRITE_PIXEL_DATA_RANGE_LENGTH_NV',
'GL_READ_PIXEL_DATA_RANGE_LENGTH_NV', 'GL_WRITE_PIXEL_DATA_RANGE_POINTER_NV',
'GL_READ_PIXEL_DATA_RANGE_POINTER_NV', 'GL_PRIMITIVE_RESTART_NV',
'GL_PRIMITIVE_RESTART_INDEX_NV', 'GL_TEXTURE_UNSIGNED_REMAP_MODE_NV',
'GL_STENCIL_BACK_FUNC_ATI', 'GL_STENCIL_BACK_FAIL_ATI',
'GL_STENCIL_BACK_PASS_DEPTH_FAIL_ATI', 'GL_STENCIL_BACK_PASS_DEPTH_PASS_ATI',
'GL_IMPLEMENTATION_COLOR_READ_TYPE_OES',
'GL_IMPLEMENTATION_COLOR_READ_FORMAT_OES', 'GL_DEPTH_BOUNDS_TEST_EXT',
'GL_DEPTH_BOUNDS_EXT', 'GL_MIRROR_CLAMP_EXT', 'GL_MIRROR_CLAMP_TO_EDGE_EXT',
'GL_MIRROR_CLAMP_TO_BORDER_EXT', 'GL_BLEND_EQUATION_RGB_EXT',
'GL_BLEND_EQUATION_ALPHA_EXT', 'GL_PACK_INVERT_MESA',
'GL_UNSIGNED_SHORT_8_8_MESA', 'GL_UNSIGNED_SHORT_8_8_REV_MESA',
'GL_YCBCR_MESA', 'GL_PIXEL_PACK_BUFFER_EXT', 'GL_PIXEL_UNPACK_BUFFER_EXT',
'GL_PIXEL_PACK_BUFFER_BINDING_EXT', 'GL_PIXEL_UNPACK_BUFFER_BINDING_EXT',
'GL_MAX_PROGRAM_EXEC_INSTRUCTIONS_NV', 'GL_MAX_PROGRAM_CALL_DEPTH_NV',
'GL_MAX_PROGRAM_IF_DEPTH_NV', 'GL_MAX_PROGRAM_LOOP_DEPTH_NV',
'GL_MAX_PROGRAM_LOOP_COUNT_NV', 'GL_INVALID_FRAMEBUFFER_OPERATION_EXT',
'GL_MAX_RENDERBUFFER_SIZE_EXT', 'GL_FRAMEBUFFER_BINDING_EXT',
'GL_RENDERBUFFER_BINDING_EXT', 'GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE_EXT',
'GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME_EXT',
'GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL_EXT',
'GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE_EXT',
'GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_3D_ZOFFSET_EXT',
'GL_FRAMEBUFFER_COMPLETE_EXT', 'GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT_EXT',
'GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT_EXT',
'GL_FRAMEBUFFER_INCOMPLETE_DUPLICATE_ATTACHMENT_EXT',
'GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_EXT',
'GL_FRAMEBUFFER_INCOMPLETE_FORMATS_EXT',
'GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER_EXT',
'GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER_EXT', 'GL_FRAMEBUFFER_UNSUPPORTED_EXT',
'GL_MAX_COLOR_ATTACHMENTS_EXT', 'GL_COLOR_ATTACHMENT0_EXT',
'GL_COLOR_ATTACHMENT1_EXT', 'GL_COLOR_ATTACHMENT2_EXT',
'GL_COLOR_ATTACHMENT3_EXT', 'GL_COLOR_ATTACHMENT4_EXT',
'GL_COLOR_ATTACHMENT5_EXT', 'GL_COLOR_ATTACHMENT6_EXT',
'GL_COLOR_ATTACHMENT7_EXT', 'GL_COLOR_ATTACHMENT8_EXT',
'GL_COLOR_ATTACHMENT9_EXT', 'GL_COLOR_ATTACHMENT10_EXT',
'GL_COLOR_ATTACHMENT11_EXT', 'GL_COLOR_ATTACHMENT12_EXT',
'GL_COLOR_ATTACHMENT13_EXT', 'GL_COLOR_ATTACHMENT14_EXT',
'GL_COLOR_ATTACHMENT15_EXT', 'GL_DEPTH_ATTACHMENT_EXT',
'GL_STENCIL_ATTACHMENT_EXT', 'GL_FRAMEBUFFER_EXT', 'GL_RENDERBUFFER_EXT',
'GL_RENDERBUFFER_WIDTH_EXT', 'GL_RENDERBUFFER_HEIGHT_EXT',
'GL_RENDERBUFFER_INTERNAL_FORMAT_EXT', 'GL_STENCIL_INDEX_EXT',
'GL_STENCIL_INDEX1_EXT', 'GL_STENCIL_INDEX4_EXT', 'GL_STENCIL_INDEX8_EXT',
'GL_STENCIL_INDEX16_EXT', 'GL_RENDERBUFFER_RED_SIZE_EXT',
'GL_RENDERBUFFER_GREEN_SIZE_EXT', 'GL_RENDERBUFFER_BLUE_SIZE_EXT',
'GL_RENDERBUFFER_ALPHA_SIZE_EXT', 'GL_RENDERBUFFER_DEPTH_SIZE_EXT',
'GL_RENDERBUFFER_STENCIL_SIZE_EXT', 'GL_CG_VERTEX_SHADER_EXT',
'GL_CG_FRAGMENT_SHADER_EXT', 'GL_TIME_ELAPSED_EXT', 'GL_TEXTURE_BUFFER_EXT',
'GL_MAX_TEXTURE_BUFFER_SIZE_EXT', 'GL_TEXTURE_BINDING_BUFFER_EXT',
'GL_TEXTURE_BUFFER_DATA_STORE_BINDING_EXT', 'GL_TEXTURE_BUFFER_FORMAT_EXT',
'GL_SAMPLER_1D_ARRAY_EXT', 'GL_SAMPLER_2D_ARRAY_EXT', 'GL_SAMPLER_BUFFER_EXT',
'GL_SAMPLER_1D_ARRAY_SHADOW_EXT', 'GL_SAMPLER_2D_ARRAY_SHADOW_EXT',
'GL_SAMPLER_CUBE_SHADOW_EXT', 'GL_UNSIGNED_INT_VEC2_EXT',
'GL_UNSIGNED_INT_VEC3_EXT', 'GL_UNSIGNED_INT_VEC4_EXT',
'GL_INT_SAMPLER_1D_EXT', 'GL_INT_SAMPLER_2D_EXT', 'GL_INT_SAMPLER_3D_EXT',
'GL_INT_SAMPLER_CUBE_EXT', 'GL_INT_SAMPLER_2D_RECT_EXT',
'GL_INT_SAMPLER_1D_ARRAY_EXT', 'GL_INT_SAMPLER_2D_ARRAY_EXT',
'GL_INT_SAMPLER_BUFFER_EXT', 'GL_UNSIGNED_INT_SAMPLER_1D_EXT',
'GL_UNSIGNED_INT_SAMPLER_2D_EXT', 'GL_UNSIGNED_INT_SAMPLER_3D_EXT',
'GL_UNSIGNED_INT_SAMPLER_CUBE_EXT', 'GL_UNSIGNED_INT_SAMPLER_2D_RECT_EXT',
'GL_UNSIGNED_INT_SAMPLER_1D_ARRAY_EXT',
'GL_UNSIGNED_INT_SAMPLER_2D_ARRAY_EXT', 'GL_UNSIGNED_INT_SAMPLER_BUFFER_EXT',
'GL_VERTEX_ATTRIB_ARRAY_INTEGER_EXT', 'GL_GEOMETRY_SHADER_EXT',
'GL_MAX_GEOMETRY_VARYING_COMPONENTS_EXT',
'GL_MAX_VERTEX_VARYING_COMPONENTS_EXT', 'GL_MAX_VARYING_COMPONENTS_EXT',
'GL_MAX_GEOMETRY_UNIFORM_COMPONENTS_EXT',
'GL_MAX_GEOMETRY_OUTPUT_VERTICES_EXT',
'GL_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS_EXT', 'GL_GEOMETRY_VERTICES_OUT_EXT',
'GL_GEOMETRY_INPUT_TYPE_EXT', 'GL_GEOMETRY_OUTPUT_TYPE_EXT',
'GL_MAX_GEOMETRY_TEXTURE_IMAGE_UNITS_EXT', 'GL_LINES_ADJACENCY_EXT',
'GL_LINE_STRIP_ADJACENCY_EXT', 'GL_TRIANGLES_ADJACENCY_EXT',
'GL_TRIANGLE_STRIP_ADJACENCY_EXT', 'GL_FRAMEBUFFER_ATTACHMENT_LAYERED_EXT',
'GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS_EXT',
'GL_FRAMEBUFFER_INCOMPLETE_LAYER_COUNT_EXT',
'GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LAYER_EXT', 'GL_PROGRAM_POINT_SIZE_EXT',
'GL_MAX_VERTEX_BINDABLE_UNIFORMS_EXT',
'GL_MAX_FRAGMENT_BINDABLE_UNIFORMS_EXT',
'GL_MAX_GEOMETRY_BINDABLE_UNIFORMS_EXT', 'GL_MAX_BINDABLE_UNIFORM_SIZE_EXT',
'GL_UNIFORM_BUFFER_EXT', 'GL_UNIFORM_BUFFER_BINDING_EXT',
'GL_FRAMEBUFFER_SRGB_EXT', 'GL_FRAMEBUFFER_SRGB_CAPABLE_EXT',
'GL_RGB9_E5_EXT', 'GL_UNSIGNED_INT_5_9_9_9_REV_EXT',
'GL_TEXTURE_SHARED_SIZE_EXT', 'GL_R11F_G11F_B10F_EXT',
'GL_UNSIGNED_INT_10F_11F_11F_REV_EXT', 'GL_RGBA_SIGNED_COMPONENTS_EXT',
'GL_TEXTURE_1D_ARRAY_EXT', 'GL_PROXY_TEXTURE_1D_ARRAY_EXT',
'GL_TEXTURE_2D_ARRAY_EXT', 'GL_PROXY_TEXTURE_2D_ARRAY_EXT',
'GL_TEXTURE_BINDING_1D_ARRAY_EXT', 'GL_TEXTURE_BINDING_2D_ARRAY_EXT',
'GL_MAX_ARRAY_TEXTURE_LAYERS_EXT', 'GL_COMPARE_REF_DEPTH_TO_TEXTURE_EXT',
'GL_RGBA32UI_EXT', 'GL_RGB32UI_EXT', 'GL_ALPHA32UI_EXT',
'GL_INTENSITY32UI_EXT', 'GL_LUMINANCE32UI_EXT', 'GL_LUMINANCE_ALPHA32UI_EXT',
'GL_RGBA16UI_EXT', 'GL_RGB16UI_EXT', 'GL_ALPHA16UI_EXT',
'GL_INTENSITY16UI_EXT', 'GL_LUMINANCE16UI_EXT', 'GL_LUMINANCE_ALPHA16UI_EXT',
'GL_RGBA8UI_EXT', 'GL_RGB8UI_EXT', 'GL_ALPHA8UI_EXT', 'GL_INTENSITY8UI_EXT',
'GL_LUMINANCE8UI_EXT', 'GL_LUMINANCE_ALPHA8UI_EXT', 'GL_RGBA32I_EXT',
'GL_RGB32I_EXT', 'GL_ALPHA32I_EXT', 'GL_INTENSITY32I_EXT',
'GL_LUMINANCE32I_EXT', 'GL_LUMINANCE_ALPHA32I_EXT', 'GL_RGBA16I_EXT',
'GL_RGB16I_EXT', 'GL_ALPHA16I_EXT', 'GL_INTENSITY16I_EXT',
'GL_LUMINANCE16I_EXT', 'GL_LUMINANCE_ALPHA16I_EXT', 'GL_RGBA8I_EXT',
'GL_RGB8I_EXT', 'GL_ALPHA8I_EXT', 'GL_INTENSITY8I_EXT', 'GL_LUMINANCE8I_EXT',
'GL_LUMINANCE_ALPHA8I_EXT', 'GL_RED_INTEGER_EXT', 'GL_GREEN_INTEGER_EXT',
'GL_BLUE_INTEGER_EXT', 'GL_ALPHA_INTEGER_EXT', 'GL_RGB_INTEGER_EXT',
'GL_RGBA_INTEGER_EXT', 'GL_BGR_INTEGER_EXT', 'GL_BGRA_INTEGER_EXT',
'GL_LUMINANCE_INTEGER_EXT', 'GL_LUMINANCE_ALPHA_INTEGER_EXT',
'GL_RGBA_INTEGER_MODE_EXT', 'GL_DEPTH_COMPONENT32F_NV',
'GL_DEPTH32F_STENCIL8_NV', 'GL_FLOAT_32_UNSIGNED_INT_24_8_REV_NV',
'GL_DEPTH_BUFFER_FLOAT_MODE_NV', 'GL_COMPRESSED_LUMINANCE_LATC1_EXT',
'GL_COMPRESSED_SIGNED_LUMINANCE_LATC1_EXT',
'GL_COMPRESSED_LUMINANCE_ALPHA_LATC2_EXT',
'GL_COMPRESSED_SIGNED_LUMINANCE_ALPHA_LATC2_EXT', 'GL_BACK_PRIMARY_COLOR_NV',
'GL_BACK_SECONDARY_COLOR_NV', 'GL_TEXTURE_COORD_NV', 'GL_CLIP_DISTANCE_NV',
'GL_VERTEX_ID_NV', 'GL_PRIMITIVE_ID_NV', 'GL_GENERIC_ATTRIB_NV',
'GL_TRANSFORM_FEEDBACK_ATTRIBS_NV', 'GL_TRANSFORM_FEEDBACK_BUFFER_MODE_NV',
'GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS_NV', 'GL_ACTIVE_VARYINGS_NV',
'GL_ACTIVE_VARYING_MAX_LENGTH_NV', 'GL_TRANSFORM_FEEDBACK_VARYINGS_NV',
'GL_TRANSFORM_FEEDBACK_BUFFER_START_NV',
'GL_TRANSFORM_FEEDBACK_BUFFER_SIZE_NV', 'GL_TRANSFORM_FEEDBACK_RECORD_NV',
'GL_PRIMITIVES_GENERATED_NV', 'GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN_NV',
'GL_RASTERIZER_DISCARD_NV',
'GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_ATTRIBS_NV',
'GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS_NV', 'GL_INTERLEAVED_ATTRIBS_NV',
'GL_SEPARATE_ATTRIBS_NV', 'GL_TRANSFORM_FEEDBACK_BUFFER_NV',
'GL_TRANSFORM_FEEDBACK_BUFFER_BINDING_NV', 'GL_GEOMETRY_PROGRAM_NV',
'GL_MAX_PROGRAM_OUTPUT_VERTICES_NV',
'GL_MAX_PROGRAM_TOTAL_OUTPUT_COMPONENTS_NV', 'GL_MIN_PROGRAM_TEXEL_OFFSET_NV',
'GL_MAX_PROGRAM_TEXEL_OFFSET_NV', 'GL_PROGRAM_ATTRIB_COMPONENTS_NV',
'GL_PROGRAM_RESULT_COMPONENTS_NV', 'GL_MAX_PROGRAM_ATTRIB_COMPONENTS_NV',
'GL_MAX_PROGRAM_RESULT_COMPONENTS_NV', 'GL_MAX_PROGRAM_GENERIC_ATTRIBS_NV',
'GL_MAX_PROGRAM_GENERIC_RESULTS_NV',
'GL_MAX_PROGRAM_PARAMETER_BUFFER_BINDINGS_NV',
'GL_MAX_PROGRAM_PARAMETER_BUFFER_SIZE_NV',
'GL_VERTEX_PROGRAM_PARAMETER_BUFFER_NV',
'GL_GEOMETRY_PROGRAM_PARAMETER_BUFFER_NV',
'GL_FRAGMENT_PROGRAM_PARAMETER_BUFFER_NV',
'GL_RENDERBUFFER_COVERAGE_SAMPLES_NV', 'GL_RENDERBUFFER_COLOR_SAMPLES_NV',
'GL_MAX_RENDERBUFFER_COVERAGE_SAMPLES_NV',
'GL_MAX_RENDERBUFFER_COLOR_SAMPLES_NV',
'GL_MAX_MULTISAMPLE_COVERAGE_MODES_NV', 'GL_MULTISAMPLE_COVERAGE_MODES_NV',
'GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE_EXT', 'GL_MAX_SAMPLES_EXT',
'GL_RENDERBUFFER_SAMPLES_EXT', 'GL_READ_FRAMEBUFFER_EXT',
'GL_DRAW_FRAMEBUFFER_EXT', 'GL_DRAW_FRAMEBUFFER_BINDING_EXT',
'GL_READ_FRAMEBUFFER_BINDING_EXT', 'GL_COMPRESSED_RED_RGTC1_EXT',
'GL_COMPRESSED_SIGNED_RED_RGTC1_EXT', 'GL_COMPRESSED_RED_GREEN_RGTC2_EXT',
'GL_COMPRESSED_SIGNED_RED_GREEN_RGTC2_EXT', 'GL_FRAME_NV', 'GL_FIELDS_NV',
'GL_CURRENT_TIME_NV', 'GL_NUM_FILL_STREAMS_NV', 'GL_PRESENT_TIME_NV',
'GL_PRESENT_DURATION_NV', 'GL_QUERY_WAIT_NV', 'GL_QUERY_NO_WAIT_NV',
'GL_QUERY_BY_REGION_WAIT_NV', 'GL_QUERY_BY_REGION_NO_WAIT_NV',
'GL_TRANSFORM_FEEDBACK_BUFFER_EXT', 'GL_TRANSFORM_FEEDBACK_BUFFER_START_EXT',
'GL_TRANSFORM_FEEDBACK_BUFFER_SIZE_EXT',
'GL_TRANSFORM_FEEDBACK_BUFFER_BINDING_EXT', 'GL_INTERLEAVED_ATTRIBS_EXT',
'GL_SEPARATE_ATTRIBS_EXT', 'GL_PRIMITIVES_GENERATED_EXT',
'GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN_EXT', 'GL_RASTERIZER_DISCARD_EXT',
'GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_COMPONENTS_EXT',
'GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS_EXT',
'GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS_EXT',
'GL_TRANSFORM_FEEDBACK_VARYINGS_EXT', 'GL_TRANSFORM_FEEDBACK_BUFFER_MODE_EXT',
'GL_TRANSFORM_FEEDBACK_VARYING_MAX_LENGTH_EXT', 'GL_PROGRAM_MATRIX_EXT',
'GL_TRANSPOSE_PROGRAM_MATRIX_EXT', 'GL_PROGRAM_MATRIX_STACK_DEPTH_EXT',
'GL_TEXTURE_SWIZZLE_R_EXT', 'GL_TEXTURE_SWIZZLE_G_EXT',
'GL_TEXTURE_SWIZZLE_B_EXT', 'GL_TEXTURE_SWIZZLE_A_EXT',
'GL_TEXTURE_SWIZZLE_RGBA_EXT', 'GL_SAMPLE_POSITION_NV', 'GL_SAMPLE_MASK_NV',
'GL_SAMPLE_MASK_VALUE_NV', 'GL_TEXTURE_BINDING_RENDERBUFFER_NV',
'GL_TEXTURE_RENDERBUFFER_DATA_STORE_BINDING_NV',
'GL_MAX_SAMPLE_MASK_WORDS_NV', 'GL_TEXTURE_RENDERBUFFER_NV',
'GL_SAMPLER_RENDERBUFFER_NV', 'GL_INT_SAMPLER_RENDERBUFFER_NV',
'GL_UNSIGNED_INT_SAMPLER_RENDERBUFFER_NV', 'GL_TRANSFORM_FEEDBACK_NV',
'GL_TRANSFORM_FEEDBACK_BUFFER_PAUSED_NV',
'GL_TRANSFORM_FEEDBACK_BUFFER_ACTIVE_NV', 'GL_TRANSFORM_FEEDBACK_BINDING_NV',
'GL_VERTEX_ATTRIB_ARRAY_UNIFIED_NV', 'GL_ELEMENT_ARRAY_UNIFIED_NV',
'GL_VERTEX_ATTRIB_ARRAY_ADDRESS_NV', 'GL_VERTEX_ARRAY_ADDRESS_NV',
'GL_NORMAL_ARRAY_ADDRESS_NV', 'GL_COLOR_ARRAY_ADDRESS_NV',
'GL_INDEX_ARRAY_ADDRESS_NV', 'GL_TEXTURE_COORD_ARRAY_ADDRESS_NV',
'GL_EDGE_FLAG_ARRAY_ADDRESS_NV', 'GL_SECONDARY_COLOR_ARRAY_ADDRESS_NV',
'GL_FOG_COORD_ARRAY_ADDRESS_NV', 'GL_ELEMENT_ARRAY_ADDRESS_NV',
'GL_VERTEX_ATTRIB_ARRAY_LENGTH_NV', 'GL_VERTEX_ARRAY_LENGTH_NV',
'GL_NORMAL_ARRAY_LENGTH_NV', 'GL_COLOR_ARRAY_LENGTH_NV',
'GL_INDEX_ARRAY_LENGTH_NV', 'GL_TEXTURE_COORD_ARRAY_LENGTH_NV',
'GL_EDGE_FLAG_ARRAY_LENGTH_NV', 'GL_SECONDARY_COLOR_ARRAY_LENGTH_NV',
'GL_FOG_COORD_ARRAY_LENGTH_NV', 'GL_ELEMENT_ARRAY_LENGTH_NV',
'GL_BUFFER_GPU_ADDRESS_NV', 'GL_GPU_ADDRESS_NV',
'GL_MAX_SHADER_BUFFER_ADDRESS_NV', 'GLchar', 'GLintptr', 'GLsizeiptr',
'GLintptrARB', 'GLsizeiptrARB', 'GLcharARB', 'GLhandleARB', 'GLhalfARB',
'GLhalfNV', 'GLint64EXT', 'GLuint64EXT', 'GL_VERSION_1_4',
'glBlendFuncSeparate', 'glFogCoordf', 'glFogCoordfv', 'glFogCoordd',
'glFogCoorddv', 'glFogCoordPointer', 'glMultiDrawArrays',
'glMultiDrawElements', 'glPointParameterf', 'glPointParameterfv',
'glPointParameteri', 'glPointParameteriv', 'glSecondaryColor3b',
'glSecondaryColor3bv', 'glSecondaryColor3d', 'glSecondaryColor3dv',
'glSecondaryColor3f', 'glSecondaryColor3fv', 'glSecondaryColor3i',
'glSecondaryColor3iv', 'glSecondaryColor3s', 'glSecondaryColor3sv',
'glSecondaryColor3ub', 'glSecondaryColor3ubv', 'glSecondaryColor3ui',
'glSecondaryColor3uiv', 'glSecondaryColor3us', 'glSecondaryColor3usv',
'glSecondaryColorPointer', 'glWindowPos2d', 'glWindowPos2dv', 'glWindowPos2f',
'glWindowPos2fv', 'glWindowPos2i', 'glWindowPos2iv', 'glWindowPos2s',
'glWindowPos2sv', 'glWindowPos3d', 'glWindowPos3dv', 'glWindowPos3f',
'glWindowPos3fv', 'glWindowPos3i', 'glWindowPos3iv', 'glWindowPos3s',
'glWindowPos3sv', 'PFNGLBLENDFUNCSEPARATEPROC', 'PFNGLFOGCOORDFPROC',
'PFNGLFOGCOORDFVPROC', 'PFNGLFOGCOORDDPROC', 'PFNGLFOGCOORDDVPROC',
'PFNGLFOGCOORDPOINTERPROC', 'PFNGLMULTIDRAWARRAYSPROC',
'PFNGLMULTIDRAWELEMENTSPROC', 'PFNGLPOINTPARAMETERFPROC',
'PFNGLPOINTPARAMETERFVPROC', 'PFNGLPOINTPARAMETERIPROC',
'PFNGLPOINTPARAMETERIVPROC', 'PFNGLSECONDARYCOLOR3BPROC',
'PFNGLSECONDARYCOLOR3BVPROC', 'PFNGLSECONDARYCOLOR3DPROC',
'PFNGLSECONDARYCOLOR3DVPROC', 'PFNGLSECONDARYCOLOR3FPROC',
'PFNGLSECONDARYCOLOR3FVPROC', 'PFNGLSECONDARYCOLOR3IPROC',
'PFNGLSECONDARYCOLOR3IVPROC', 'PFNGLSECONDARYCOLOR3SPROC',
'PFNGLSECONDARYCOLOR3SVPROC', 'PFNGLSECONDARYCOLOR3UBPROC',
'PFNGLSECONDARYCOLOR3UBVPROC', 'PFNGLSECONDARYCOLOR3UIPROC',
'PFNGLSECONDARYCOLOR3UIVPROC', 'PFNGLSECONDARYCOLOR3USPROC',
'PFNGLSECONDARYCOLOR3USVPROC', 'PFNGLSECONDARYCOLORPOINTERPROC',
'PFNGLWINDOWPOS2DPROC', 'PFNGLWINDOWPOS2DVPROC', 'PFNGLWINDOWPOS2FPROC',
'PFNGLWINDOWPOS2FVPROC', 'PFNGLWINDOWPOS2IPROC', 'PFNGLWINDOWPOS2IVPROC',
'PFNGLWINDOWPOS2SPROC', 'PFNGLWINDOWPOS2SVPROC', 'PFNGLWINDOWPOS3DPROC',
'PFNGLWINDOWPOS3DVPROC', 'PFNGLWINDOWPOS3FPROC', 'PFNGLWINDOWPOS3FVPROC',
'PFNGLWINDOWPOS3IPROC', 'PFNGLWINDOWPOS3IVPROC', 'PFNGLWINDOWPOS3SPROC',
'PFNGLWINDOWPOS3SVPROC', 'GL_VERSION_1_5', 'glGenQueries', 'glDeleteQueries',
'glIsQuery', 'glBeginQuery', 'glEndQuery', 'glGetQueryiv',
'glGetQueryObjectiv', 'glGetQueryObjectuiv', 'glBindBuffer',
'glDeleteBuffers', 'glGenBuffers', 'glIsBuffer', 'glBufferData',
'glBufferSubData', 'glGetBufferSubData', 'glMapBuffer', 'glUnmapBuffer',
'glGetBufferParameteriv', 'glGetBufferPointerv', 'PFNGLGENQUERIESPROC',
'PFNGLDELETEQUERIESPROC', 'PFNGLISQUERYPROC', 'PFNGLBEGINQUERYPROC',
'PFNGLENDQUERYPROC', 'PFNGLGETQUERYIVPROC', 'PFNGLGETQUERYOBJECTIVPROC',
'PFNGLGETQUERYOBJECTUIVPROC', 'PFNGLBINDBUFFERPROC', 'PFNGLDELETEBUFFERSPROC',
'PFNGLGENBUFFERSPROC', 'PFNGLISBUFFERPROC', 'PFNGLBUFFERDATAPROC',
'PFNGLBUFFERSUBDATAPROC', 'PFNGLGETBUFFERSUBDATAPROC', 'PFNGLMAPBUFFERPROC',
'PFNGLUNMAPBUFFERPROC', 'PFNGLGETBUFFERPARAMETERIVPROC',
'PFNGLGETBUFFERPOINTERVPROC', 'GL_VERSION_2_0', 'glBlendEquationSeparate',
'glDrawBuffers', 'glStencilOpSeparate', 'glStencilFuncSeparate',
'glStencilMaskSeparate', 'glAttachShader', 'glBindAttribLocation',
'glCompileShader', 'glCreateProgram', 'glCreateShader', 'glDeleteProgram',
'glDeleteShader', 'glDetachShader', 'glDisableVertexAttribArray',
'glEnableVertexAttribArray', 'glGetActiveAttrib', 'glGetActiveUniform',
'glGetAttachedShaders', 'glGetAttribLocation', 'glGetProgramiv',
'glGetProgramInfoLog', 'glGetShaderiv', 'glGetShaderInfoLog',
'glGetShaderSource', 'glGetUniformLocation', 'glGetUniformfv',
'glGetUniformiv', 'glGetVertexAttribdv', 'glGetVertexAttribfv',
'glGetVertexAttribiv', 'glGetVertexAttribPointerv', 'glIsProgram',
'glIsShader', 'glLinkProgram', 'glShaderSource', 'glUseProgram',
'glUniform1f', 'glUniform2f', 'glUniform3f', 'glUniform4f', 'glUniform1i',
'glUniform2i', 'glUniform3i', 'glUniform4i', 'glUniform1fv', 'glUniform2fv',
'glUniform3fv', 'glUniform4fv', 'glUniform1iv', 'glUniform2iv',
'glUniform3iv', 'glUniform4iv', 'glUniformMatrix2fv', 'glUniformMatrix3fv',
'glUniformMatrix4fv', 'glValidateProgram', 'glVertexAttrib1d',
'glVertexAttrib1dv', 'glVertexAttrib1f', 'glVertexAttrib1fv',
'glVertexAttrib1s', 'glVertexAttrib1sv', 'glVertexAttrib2d',
'glVertexAttrib2dv', 'glVertexAttrib2f', 'glVertexAttrib2fv',
'glVertexAttrib2s', 'glVertexAttrib2sv', 'glVertexAttrib3d',
'glVertexAttrib3dv', 'glVertexAttrib3f', 'glVertexAttrib3fv',
'glVertexAttrib3s', 'glVertexAttrib3sv', 'glVertexAttrib4Nbv',
'glVertexAttrib4Niv', 'glVertexAttrib4Nsv', 'glVertexAttrib4Nub',
'glVertexAttrib4Nubv', 'glVertexAttrib4Nuiv', 'glVertexAttrib4Nusv',
'glVertexAttrib4bv', 'glVertexAttrib4d', 'glVertexAttrib4dv',
'glVertexAttrib4f', 'glVertexAttrib4fv', 'glVertexAttrib4iv',
'glVertexAttrib4s', 'glVertexAttrib4sv', 'glVertexAttrib4ubv',
'glVertexAttrib4uiv', 'glVertexAttrib4usv', 'glVertexAttribPointer',
'PFNGLBLENDEQUATIONSEPARATEPROC', 'PFNGLDRAWBUFFERSPROC',
'PFNGLSTENCILOPSEPARATEPROC', 'PFNGLSTENCILFUNCSEPARATEPROC',
'PFNGLSTENCILMASKSEPARATEPROC', 'PFNGLATTACHSHADERPROC',
'PFNGLBINDATTRIBLOCATIONPROC', 'PFNGLCOMPILESHADERPROC',
'PFNGLCREATEPROGRAMPROC', 'PFNGLCREATESHADERPROC', 'PFNGLDELETEPROGRAMPROC',
'PFNGLDELETESHADERPROC', 'PFNGLDETACHSHADERPROC',
'PFNGLDISABLEVERTEXATTRIBARRAYPROC', 'PFNGLENABLEVERTEXATTRIBARRAYPROC',
'PFNGLGETACTIVEATTRIBPROC', 'PFNGLGETACTIVEUNIFORMPROC',
'PFNGLGETATTACHEDSHADERSPROC', 'PFNGLGETATTRIBLOCATIONPROC',
'PFNGLGETPROGRAMIVPROC', 'PFNGLGETPROGRAMINFOLOGPROC', 'PFNGLGETSHADERIVPROC',
'PFNGLGETSHADERINFOLOGPROC', 'PFNGLGETSHADERSOURCEPROC',
'PFNGLGETUNIFORMLOCATIONPROC', 'PFNGLGETUNIFORMFVPROC',
'PFNGLGETUNIFORMIVPROC', 'PFNGLGETVERTEXATTRIBDVPROC',
'PFNGLGETVERTEXATTRIBFVPROC', 'PFNGLGETVERTEXATTRIBIVPROC',
'PFNGLGETVERTEXATTRIBPOINTERVPROC', 'PFNGLISPROGRAMPROC', 'PFNGLISSHADERPROC',
'PFNGLLINKPROGRAMPROC', 'PFNGLSHADERSOURCEPROC', 'PFNGLUSEPROGRAMPROC',
'PFNGLUNIFORM1FPROC', 'PFNGLUNIFORM2FPROC', 'PFNGLUNIFORM3FPROC',
'PFNGLUNIFORM4FPROC', 'PFNGLUNIFORM1IPROC', 'PFNGLUNIFORM2IPROC',
'PFNGLUNIFORM3IPROC', 'PFNGLUNIFORM4IPROC', 'PFNGLUNIFORM1FVPROC',
'PFNGLUNIFORM2FVPROC', 'PFNGLUNIFORM3FVPROC', 'PFNGLUNIFORM4FVPROC',
'PFNGLUNIFORM1IVPROC', 'PFNGLUNIFORM2IVPROC', 'PFNGLUNIFORM3IVPROC',
'PFNGLUNIFORM4IVPROC', 'PFNGLUNIFORMMATRIX2FVPROC',
'PFNGLUNIFORMMATRIX3FVPROC', 'PFNGLUNIFORMMATRIX4FVPROC',
'PFNGLVALIDATEPROGRAMPROC', 'PFNGLVERTEXATTRIB1DPROC',
'PFNGLVERTEXATTRIB1DVPROC', 'PFNGLVERTEXATTRIB1FPROC',
'PFNGLVERTEXATTRIB1FVPROC', 'PFNGLVERTEXATTRIB1SPROC',
'PFNGLVERTEXATTRIB1SVPROC', 'PFNGLVERTEXATTRIB2DPROC',
'PFNGLVERTEXATTRIB2DVPROC', 'PFNGLVERTEXATTRIB2FPROC',
'PFNGLVERTEXATTRIB2FVPROC', 'PFNGLVERTEXATTRIB2SPROC',
'PFNGLVERTEXATTRIB2SVPROC', 'PFNGLVERTEXATTRIB3DPROC',
'PFNGLVERTEXATTRIB3DVPROC', 'PFNGLVERTEXATTRIB3FPROC',
'PFNGLVERTEXATTRIB3FVPROC', 'PFNGLVERTEXATTRIB3SPROC',
'PFNGLVERTEXATTRIB3SVPROC', 'PFNGLVERTEXATTRIB4NBVPROC',
'PFNGLVERTEXATTRIB4NIVPROC', 'PFNGLVERTEXATTRIB4NSVPROC',
'PFNGLVERTEXATTRIB4NUBPROC', 'PFNGLVERTEXATTRIB4NUBVPROC',
'PFNGLVERTEXATTRIB4NUIVPROC', 'PFNGLVERTEXATTRIB4NUSVPROC',
'PFNGLVERTEXATTRIB4BVPROC', 'PFNGLVERTEXATTRIB4DPROC',
'PFNGLVERTEXATTRIB4DVPROC', 'PFNGLVERTEXATTRIB4FPROC',
'PFNGLVERTEXATTRIB4FVPROC', 'PFNGLVERTEXATTRIB4IVPROC',
'PFNGLVERTEXATTRIB4SPROC', 'PFNGLVERTEXATTRIB4SVPROC',
'PFNGLVERTEXATTRIB4UBVPROC', 'PFNGLVERTEXATTRIB4UIVPROC',
'PFNGLVERTEXATTRIB4USVPROC', 'PFNGLVERTEXATTRIBPOINTERPROC', 'GL_VERSION_2_1',
'glUniformMatrix2x3fv', 'glUniformMatrix3x2fv', 'glUniformMatrix2x4fv',
'glUniformMatrix4x2fv', 'glUniformMatrix3x4fv', 'glUniformMatrix4x3fv',
'PFNGLUNIFORMMATRIX2X3FVPROC', 'PFNGLUNIFORMMATRIX3X2FVPROC',
'PFNGLUNIFORMMATRIX2X4FVPROC', 'PFNGLUNIFORMMATRIX4X2FVPROC',
'PFNGLUNIFORMMATRIX3X4FVPROC', 'PFNGLUNIFORMMATRIX4X3FVPROC',
'GL_VERSION_3_0', 'glColorMaski', 'glGetBooleani_v', 'glGetIntegeri_v',
'glEnablei', 'glDisablei', 'glIsEnabledi', 'glBeginTransformFeedback',
'glEndTransformFeedback', 'glBindBufferRange', 'glBindBufferBase',
'glTransformFeedbackVaryings', 'glGetTransformFeedbackVarying',
'glClampColor', 'glBeginConditionalRender', 'glEndConditionalRender',
'glVertexAttribI1i', 'glVertexAttribI2i', 'glVertexAttribI3i',
'glVertexAttribI4i', 'glVertexAttribI1ui', 'glVertexAttribI2ui',
'glVertexAttribI3ui', 'glVertexAttribI4ui', 'glVertexAttribI1iv',
'glVertexAttribI2iv', 'glVertexAttribI3iv', 'glVertexAttribI4iv',
'glVertexAttribI1uiv', 'glVertexAttribI2uiv', 'glVertexAttribI3uiv',
'glVertexAttribI4uiv', 'glVertexAttribI4bv', 'glVertexAttribI4sv',
'glVertexAttribI4ubv', 'glVertexAttribI4usv', 'glVertexAttribIPointer',
'glGetVertexAttribIiv', 'glGetVertexAttribIuiv', 'glGetUniformuiv',
'glBindFragDataLocation', 'glGetFragDataLocation', 'glUniform1ui',
'glUniform2ui', 'glUniform3ui', 'glUniform4ui', 'glUniform1uiv',
'glUniform2uiv', 'glUniform3uiv', 'glUniform4uiv', 'glTexParameterIiv',
'glTexParameterIuiv', 'glGetTexParameterIiv', 'glGetTexParameterIuiv',
'glClearBufferiv', 'glClearBufferuiv', 'glClearBufferfv', 'glClearBufferfi',
'glGetStringi', 'PFNGLCOLORMASKIPROC', 'PFNGLGETBOOLEANI_VPROC',
'PFNGLGETINTEGERI_VPROC', 'PFNGLENABLEIPROC', 'PFNGLDISABLEIPROC',
'PFNGLISENABLEDIPROC', 'PFNGLBEGINTRANSFORMFEEDBACKPROC',
'PFNGLENDTRANSFORMFEEDBACKPROC', 'PFNGLBINDBUFFERRANGEPROC',
'PFNGLBINDBUFFERBASEPROC', 'PFNGLTRANSFORMFEEDBACKVARYINGSPROC',
'PFNGLGETTRANSFORMFEEDBACKVARYINGPROC', 'PFNGLCLAMPCOLORPROC',
'PFNGLBEGINCONDITIONALRENDERPROC', 'PFNGLENDCONDITIONALRENDERPROC',
'PFNGLVERTEXATTRIBI1IPROC', 'PFNGLVERTEXATTRIBI2IPROC',
'PFNGLVERTEXATTRIBI3IPROC', 'PFNGLVERTEXATTRIBI4IPROC',
'PFNGLVERTEXATTRIBI1UIPROC', 'PFNGLVERTEXATTRIBI2UIPROC',
'PFNGLVERTEXATTRIBI3UIPROC', 'PFNGLVERTEXATTRIBI4UIPROC',
'PFNGLVERTEXATTRIBI1IVPROC', 'PFNGLVERTEXATTRIBI2IVPROC',
'PFNGLVERTEXATTRIBI3IVPROC', 'PFNGLVERTEXATTRIBI4IVPROC',
'PFNGLVERTEXATTRIBI1UIVPROC', 'PFNGLVERTEXATTRIBI2UIVPROC',
'PFNGLVERTEXATTRIBI3UIVPROC', 'PFNGLVERTEXATTRIBI4UIVPROC',
'PFNGLVERTEXATTRIBI4BVPROC', 'PFNGLVERTEXATTRIBI4SVPROC',
'PFNGLVERTEXATTRIBI4UBVPROC', 'PFNGLVERTEXATTRIBI4USVPROC',
'PFNGLVERTEXATTRIBIPOINTERPROC', 'PFNGLGETVERTEXATTRIBIIVPROC',
'PFNGLGETVERTEXATTRIBIUIVPROC', 'PFNGLGETUNIFORMUIVPROC',
'PFNGLBINDFRAGDATALOCATIONPROC', 'PFNGLGETFRAGDATALOCATIONPROC',
'PFNGLUNIFORM1UIPROC', 'PFNGLUNIFORM2UIPROC', 'PFNGLUNIFORM3UIPROC',
'PFNGLUNIFORM4UIPROC', 'PFNGLUNIFORM1UIVPROC', 'PFNGLUNIFORM2UIVPROC',
'PFNGLUNIFORM3UIVPROC', 'PFNGLUNIFORM4UIVPROC', 'PFNGLTEXPARAMETERIIVPROC',
'PFNGLTEXPARAMETERIUIVPROC', 'PFNGLGETTEXPARAMETERIIVPROC',
'PFNGLGETTEXPARAMETERIUIVPROC', 'PFNGLCLEARBUFFERIVPROC',
'PFNGLCLEARBUFFERUIVPROC', 'PFNGLCLEARBUFFERFVPROC', 'PFNGLCLEARBUFFERFIPROC',
'PFNGLGETSTRINGIPROC', 'GL_ARB_transpose_matrix', 'glLoadTransposeMatrixfARB',
'glLoadTransposeMatrixdARB', 'glMultTransposeMatrixfARB',
'glMultTransposeMatrixdARB', 'PFNGLLOADTRANSPOSEMATRIXFARBPROC',
'PFNGLLOADTRANSPOSEMATRIXDARBPROC', 'PFNGLMULTTRANSPOSEMATRIXFARBPROC',
'PFNGLMULTTRANSPOSEMATRIXDARBPROC', 'GL_ARB_multisample',
'glSampleCoverageARB', 'PFNGLSAMPLECOVERAGEARBPROC', 'GL_ARB_texture_env_add',
'GL_ARB_texture_cube_map', 'GL_ARB_texture_compression',
'glCompressedTexImage3DARB', 'glCompressedTexImage2DARB',
'glCompressedTexImage1DARB', 'glCompressedTexSubImage3DARB',
'glCompressedTexSubImage2DARB', 'glCompressedTexSubImage1DARB',
'glGetCompressedTexImageARB', 'PFNGLCOMPRESSEDTEXIMAGE3DARBPROC',
'PFNGLCOMPRESSEDTEXIMAGE2DARBPROC', 'PFNGLCOMPRESSEDTEXIMAGE1DARBPROC',
'PFNGLCOMPRESSEDTEXSUBIMAGE3DARBPROC', 'PFNGLCOMPRESSEDTEXSUBIMAGE2DARBPROC',
'PFNGLCOMPRESSEDTEXSUBIMAGE1DARBPROC', 'PFNGLGETCOMPRESSEDTEXIMAGEARBPROC',
'GL_ARB_texture_border_clamp', 'GL_ARB_point_parameters',
'glPointParameterfARB', 'glPointParameterfvARB',
'PFNGLPOINTPARAMETERFARBPROC', 'PFNGLPOINTPARAMETERFVARBPROC',
'GL_ARB_vertex_blend', 'glWeightbvARB', 'glWeightsvARB', 'glWeightivARB',
'glWeightfvARB', 'glWeightdvARB', 'glWeightubvARB', 'glWeightusvARB',
'glWeightuivARB', 'glWeightPointerARB', 'glVertexBlendARB',
'PFNGLWEIGHTBVARBPROC', 'PFNGLWEIGHTSVARBPROC', 'PFNGLWEIGHTIVARBPROC',
'PFNGLWEIGHTFVARBPROC', 'PFNGLWEIGHTDVARBPROC', 'PFNGLWEIGHTUBVARBPROC',
'PFNGLWEIGHTUSVARBPROC', 'PFNGLWEIGHTUIVARBPROC', 'PFNGLWEIGHTPOINTERARBPROC',
'PFNGLVERTEXBLENDARBPROC', 'GL_ARB_matrix_palette',
'glCurrentPaletteMatrixARB', 'glMatrixIndexubvARB', 'glMatrixIndexusvARB',
'glMatrixIndexuivARB', 'glMatrixIndexPointerARB',
'PFNGLCURRENTPALETTEMATRIXARBPROC', 'PFNGLMATRIXINDEXUBVARBPROC',
'PFNGLMATRIXINDEXUSVARBPROC', 'PFNGLMATRIXINDEXUIVARBPROC',
'PFNGLMATRIXINDEXPOINTERARBPROC', 'GL_ARB_texture_env_combine',
'GL_ARB_texture_env_crossbar', 'GL_ARB_texture_env_dot3',
'GL_ARB_texture_mirrored_repeat', 'GL_ARB_depth_texture', 'GL_ARB_shadow',
'GL_ARB_shadow_ambient', 'GL_ARB_window_pos', 'glWindowPos2dARB',
'glWindowPos2dvARB', 'glWindowPos2fARB', 'glWindowPos2fvARB',
'glWindowPos2iARB', 'glWindowPos2ivARB', 'glWindowPos2sARB',
'glWindowPos2svARB', 'glWindowPos3dARB', 'glWindowPos3dvARB',
'glWindowPos3fARB', 'glWindowPos3fvARB', 'glWindowPos3iARB',
'glWindowPos3ivARB', 'glWindowPos3sARB', 'glWindowPos3svARB',
'PFNGLWINDOWPOS2DARBPROC', 'PFNGLWINDOWPOS2DVARBPROC',
'PFNGLWINDOWPOS2FARBPROC', 'PFNGLWINDOWPOS2FVARBPROC',
'PFNGLWINDOWPOS2IARBPROC', 'PFNGLWINDOWPOS2IVARBPROC',
'PFNGLWINDOWPOS2SARBPROC', 'PFNGLWINDOWPOS2SVARBPROC',
'PFNGLWINDOWPOS3DARBPROC', 'PFNGLWINDOWPOS3DVARBPROC',
'PFNGLWINDOWPOS3FARBPROC', 'PFNGLWINDOWPOS3FVARBPROC',
'PFNGLWINDOWPOS3IARBPROC', 'PFNGLWINDOWPOS3IVARBPROC',
'PFNGLWINDOWPOS3SARBPROC', 'PFNGLWINDOWPOS3SVARBPROC',
'GL_ARB_vertex_program', 'glVertexAttrib1dARB', 'glVertexAttrib1dvARB',
'glVertexAttrib1fARB', 'glVertexAttrib1fvARB', 'glVertexAttrib1sARB',
'glVertexAttrib1svARB', 'glVertexAttrib2dARB', 'glVertexAttrib2dvARB',
'glVertexAttrib2fARB', 'glVertexAttrib2fvARB', 'glVertexAttrib2sARB',
'glVertexAttrib2svARB', 'glVertexAttrib3dARB', 'glVertexAttrib3dvARB',
'glVertexAttrib3fARB', 'glVertexAttrib3fvARB', 'glVertexAttrib3sARB',
'glVertexAttrib3svARB', 'glVertexAttrib4NbvARB', 'glVertexAttrib4NivARB',
'glVertexAttrib4NsvARB', 'glVertexAttrib4NubARB', 'glVertexAttrib4NubvARB',
'glVertexAttrib4NuivARB', 'glVertexAttrib4NusvARB', 'glVertexAttrib4bvARB',
'glVertexAttrib4dARB', 'glVertexAttrib4dvARB', 'glVertexAttrib4fARB',
'glVertexAttrib4fvARB', 'glVertexAttrib4ivARB', 'glVertexAttrib4sARB',
'glVertexAttrib4svARB', 'glVertexAttrib4ubvARB', 'glVertexAttrib4uivARB',
'glVertexAttrib4usvARB', 'glVertexAttribPointerARB',
'glEnableVertexAttribArrayARB', 'glDisableVertexAttribArrayARB',
'glProgramStringARB', 'glBindProgramARB', 'glDeleteProgramsARB',
'glGenProgramsARB', 'glProgramEnvParameter4dARB',
'glProgramEnvParameter4dvARB', 'glProgramEnvParameter4fARB',
'glProgramEnvParameter4fvARB', 'glProgramLocalParameter4dARB',
'glProgramLocalParameter4dvARB', 'glProgramLocalParameter4fARB',
'glProgramLocalParameter4fvARB', 'glGetProgramEnvParameterdvARB',
'glGetProgramEnvParameterfvARB', 'glGetProgramLocalParameterdvARB',
'glGetProgramLocalParameterfvARB', 'glGetProgramivARB',
'glGetProgramStringARB', 'glGetVertexAttribdvARB', 'glGetVertexAttribfvARB',
'glGetVertexAttribivARB', 'glGetVertexAttribPointervARB', 'glIsProgramARB',
'PFNGLVERTEXATTRIB1DARBPROC', 'PFNGLVERTEXATTRIB1DVARBPROC',
'PFNGLVERTEXATTRIB1FARBPROC', 'PFNGLVERTEXATTRIB1FVARBPROC',
'PFNGLVERTEXATTRIB1SARBPROC', 'PFNGLVERTEXATTRIB1SVARBPROC',
'PFNGLVERTEXATTRIB2DARBPROC', 'PFNGLVERTEXATTRIB2DVARBPROC',
'PFNGLVERTEXATTRIB2FARBPROC', 'PFNGLVERTEXATTRIB2FVARBPROC',
'PFNGLVERTEXATTRIB2SARBPROC', 'PFNGLVERTEXATTRIB2SVARBPROC',
'PFNGLVERTEXATTRIB3DARBPROC', 'PFNGLVERTEXATTRIB3DVARBPROC',
'PFNGLVERTEXATTRIB3FARBPROC', 'PFNGLVERTEXATTRIB3FVARBPROC',
'PFNGLVERTEXATTRIB3SARBPROC', 'PFNGLVERTEXATTRIB3SVARBPROC',
'PFNGLVERTEXATTRIB4NBVARBPROC', 'PFNGLVERTEXATTRIB4NIVARBPROC',
'PFNGLVERTEXATTRIB4NSVARBPROC', 'PFNGLVERTEXATTRIB4NUBARBPROC',
'PFNGLVERTEXATTRIB4NUBVARBPROC', 'PFNGLVERTEXATTRIB4NUIVARBPROC',
'PFNGLVERTEXATTRIB4NUSVARBPROC', 'PFNGLVERTEXATTRIB4BVARBPROC',
'PFNGLVERTEXATTRIB4DARBPROC', 'PFNGLVERTEXATTRIB4DVARBPROC',
'PFNGLVERTEXATTRIB4FARBPROC', 'PFNGLVERTEXATTRIB4FVARBPROC',
'PFNGLVERTEXATTRIB4IVARBPROC', 'PFNGLVERTEXATTRIB4SARBPROC',
'PFNGLVERTEXATTRIB4SVARBPROC', 'PFNGLVERTEXATTRIB4UBVARBPROC',
'PFNGLVERTEXATTRIB4UIVARBPROC', 'PFNGLVERTEXATTRIB4USVARBPROC',
'PFNGLVERTEXATTRIBPOINTERARBPROC', 'PFNGLENABLEVERTEXATTRIBARRAYARBPROC',
'PFNGLDISABLEVERTEXATTRIBARRAYARBPROC', 'PFNGLPROGRAMSTRINGARBPROC',
'PFNGLBINDPROGRAMARBPROC', 'PFNGLDELETEPROGRAMSARBPROC',
'PFNGLGENPROGRAMSARBPROC', 'PFNGLPROGRAMENVPARAMETER4DARBPROC',
'PFNGLPROGRAMENVPARAMETER4DVARBPROC', 'PFNGLPROGRAMENVPARAMETER4FARBPROC',
'PFNGLPROGRAMENVPARAMETER4FVARBPROC', 'PFNGLPROGRAMLOCALPARAMETER4DARBPROC',
'PFNGLPROGRAMLOCALPARAMETER4DVARBPROC', 'PFNGLPROGRAMLOCALPARAMETER4FARBPROC',
'PFNGLPROGRAMLOCALPARAMETER4FVARBPROC',
'PFNGLGETPROGRAMENVPARAMETERDVARBPROC',
'PFNGLGETPROGRAMENVPARAMETERFVARBPROC',
'PFNGLGETPROGRAMLOCALPARAMETERDVARBPROC',
'PFNGLGETPROGRAMLOCALPARAMETERFVARBPROC', 'PFNGLGETPROGRAMIVARBPROC',
'PFNGLGETPROGRAMSTRINGARBPROC', 'PFNGLGETVERTEXATTRIBDVARBPROC',
'PFNGLGETVERTEXATTRIBFVARBPROC', 'PFNGLGETVERTEXATTRIBIVARBPROC',
'PFNGLGETVERTEXATTRIBPOINTERVARBPROC', 'PFNGLISPROGRAMARBPROC',
'GL_ARB_fragment_program', 'GL_ARB_vertex_buffer_object', 'glBindBufferARB',
'glDeleteBuffersARB', 'glGenBuffersARB', 'glIsBufferARB', 'glBufferDataARB',
'glBufferSubDataARB', 'glGetBufferSubDataARB', 'glMapBufferARB',
'glUnmapBufferARB', 'glGetBufferParameterivARB', 'glGetBufferPointervARB',
'PFNGLBINDBUFFERARBPROC', 'PFNGLDELETEBUFFERSARBPROC',
'PFNGLGENBUFFERSARBPROC', 'PFNGLISBUFFERARBPROC', 'PFNGLBUFFERDATAARBPROC',
'PFNGLBUFFERSUBDATAARBPROC', 'PFNGLGETBUFFERSUBDATAARBPROC',
'PFNGLMAPBUFFERARBPROC', 'PFNGLUNMAPBUFFERARBPROC',
'PFNGLGETBUFFERPARAMETERIVARBPROC', 'PFNGLGETBUFFERPOINTERVARBPROC',
'GL_ARB_occlusion_query', 'glGenQueriesARB', 'glDeleteQueriesARB',
'glIsQueryARB', 'glBeginQueryARB', 'glEndQueryARB', 'glGetQueryivARB',
'glGetQueryObjectivARB', 'glGetQueryObjectuivARB', 'PFNGLGENQUERIESARBPROC',
'PFNGLDELETEQUERIESARBPROC', 'PFNGLISQUERYARBPROC', 'PFNGLBEGINQUERYARBPROC',
'PFNGLENDQUERYARBPROC', 'PFNGLGETQUERYIVARBPROC',
'PFNGLGETQUERYOBJECTIVARBPROC', 'PFNGLGETQUERYOBJECTUIVARBPROC',
'GL_ARB_shader_objects', 'glDeleteObjectARB', 'glGetHandleARB',
'glDetachObjectARB', 'glCreateShaderObjectARB', 'glShaderSourceARB',
'glCompileShaderARB', 'glCreateProgramObjectARB', 'glAttachObjectARB',
'glLinkProgramARB', 'glUseProgramObjectARB', 'glValidateProgramARB',
'glUniform1fARB', 'glUniform2fARB', 'glUniform3fARB', 'glUniform4fARB',
'glUniform1iARB', 'glUniform2iARB', 'glUniform3iARB', 'glUniform4iARB',
'glUniform1fvARB', 'glUniform2fvARB', 'glUniform3fvARB', 'glUniform4fvARB',
'glUniform1ivARB', 'glUniform2ivARB', 'glUniform3ivARB', 'glUniform4ivARB',
'glUniformMatrix2fvARB', 'glUniformMatrix3fvARB', 'glUniformMatrix4fvARB',
'glGetObjectParameterfvARB', 'glGetObjectParameterivARB', 'glGetInfoLogARB',
'glGetAttachedObjectsARB', 'glGetUniformLocationARB', 'glGetActiveUniformARB',
'glGetUniformfvARB', 'glGetUniformivARB', 'glGetShaderSourceARB',
'PFNGLDELETEOBJECTARBPROC', 'PFNGLGETHANDLEARBPROC',
'PFNGLDETACHOBJECTARBPROC', 'PFNGLCREATESHADEROBJECTARBPROC',
'PFNGLSHADERSOURCEARBPROC', 'PFNGLCOMPILESHADERARBPROC',
'PFNGLCREATEPROGRAMOBJECTARBPROC', 'PFNGLATTACHOBJECTARBPROC',
'PFNGLLINKPROGRAMARBPROC', 'PFNGLUSEPROGRAMOBJECTARBPROC',
'PFNGLVALIDATEPROGRAMARBPROC', 'PFNGLUNIFORM1FARBPROC',
'PFNGLUNIFORM2FARBPROC', 'PFNGLUNIFORM3FARBPROC', 'PFNGLUNIFORM4FARBPROC',
'PFNGLUNIFORM1IARBPROC', 'PFNGLUNIFORM2IARBPROC', 'PFNGLUNIFORM3IARBPROC',
'PFNGLUNIFORM4IARBPROC', 'PFNGLUNIFORM1FVARBPROC', 'PFNGLUNIFORM2FVARBPROC',
'PFNGLUNIFORM3FVARBPROC', 'PFNGLUNIFORM4FVARBPROC', 'PFNGLUNIFORM1IVARBPROC',
'PFNGLUNIFORM2IVARBPROC', 'PFNGLUNIFORM3IVARBPROC', 'PFNGLUNIFORM4IVARBPROC',
'PFNGLUNIFORMMATRIX2FVARBPROC', 'PFNGLUNIFORMMATRIX3FVARBPROC',
'PFNGLUNIFORMMATRIX4FVARBPROC', 'PFNGLGETOBJECTPARAMETERFVARBPROC',
'PFNGLGETOBJECTPARAMETERIVARBPROC', 'PFNGLGETINFOLOGARBPROC',
'PFNGLGETATTACHEDOBJECTSARBPROC', 'PFNGLGETUNIFORMLOCATIONARBPROC',
'PFNGLGETACTIVEUNIFORMARBPROC', 'PFNGLGETUNIFORMFVARBPROC',
'PFNGLGETUNIFORMIVARBPROC', 'PFNGLGETSHADERSOURCEARBPROC',
'GL_ARB_vertex_shader', 'glBindAttribLocationARB', 'glGetActiveAttribARB',
'glGetAttribLocationARB', 'PFNGLBINDATTRIBLOCATIONARBPROC',
'PFNGLGETACTIVEATTRIBARBPROC', 'PFNGLGETATTRIBLOCATIONARBPROC',
'GL_ARB_fragment_shader', 'GL_ARB_shading_language_100',
'GL_ARB_texture_non_power_of_two', 'GL_ARB_point_sprite',
'GL_ARB_fragment_program_shadow', 'GL_ARB_draw_buffers', 'glDrawBuffersARB',
'PFNGLDRAWBUFFERSARBPROC', 'GL_ARB_texture_rectangle',
'GL_ARB_color_buffer_float', 'glClampColorARB', 'PFNGLCLAMPCOLORARBPROC',
'GL_ARB_half_float_pixel', 'GL_ARB_texture_float',
'GL_ARB_pixel_buffer_object', 'GL_ARB_depth_buffer_float',
'GL_ARB_draw_instanced', 'glDrawArraysInstancedARB',
'glDrawElementsInstancedARB', 'PFNGLDRAWARRAYSINSTANCEDARBPROC',
'PFNGLDRAWELEMENTSINSTANCEDARBPROC', 'GL_ARB_framebuffer_object',
'glIsRenderbuffer', 'glBindRenderbuffer', 'glDeleteRenderbuffers',
'glGenRenderbuffers', 'glRenderbufferStorage', 'glGetRenderbufferParameteriv',
'glIsFramebuffer', 'glBindFramebuffer', 'glDeleteFramebuffers',
'glGenFramebuffers', 'glCheckFramebufferStatus', 'glFramebufferTexture1D',
'glFramebufferTexture2D', 'glFramebufferTexture3D',
'glFramebufferRenderbuffer', 'glGetFramebufferAttachmentParameteriv',
'glGenerateMipmap', 'glBlitFramebuffer', 'glRenderbufferStorageMultisample',
'glFramebufferTextureLayer', 'PFNGLISRENDERBUFFERPROC',
'PFNGLBINDRENDERBUFFERPROC', 'PFNGLDELETERENDERBUFFERSPROC',
'PFNGLGENRENDERBUFFERSPROC', 'PFNGLRENDERBUFFERSTORAGEPROC',
'PFNGLGETRENDERBUFFERPARAMETERIVPROC', 'PFNGLISFRAMEBUFFERPROC',
'PFNGLBINDFRAMEBUFFERPROC', 'PFNGLDELETEFRAMEBUFFERSPROC',
'PFNGLGENFRAMEBUFFERSPROC', 'PFNGLCHECKFRAMEBUFFERSTATUSPROC',
'PFNGLFRAMEBUFFERTEXTURE1DPROC', 'PFNGLFRAMEBUFFERTEXTURE2DPROC',
'PFNGLFRAMEBUFFERTEXTURE3DPROC', 'PFNGLFRAMEBUFFERRENDERBUFFERPROC',
'PFNGLGETFRAMEBUFFERATTACHMENTPARAMETERIVPROC', 'PFNGLGENERATEMIPMAPPROC',
'PFNGLBLITFRAMEBUFFERPROC', 'PFNGLRENDERBUFFERSTORAGEMULTISAMPLEPROC',
'PFNGLFRAMEBUFFERTEXTURELAYERPROC', 'GL_ARB_framebuffer_sRGB',
'GL_ARB_geometry_shader4', 'glProgramParameteriARB',
'glFramebufferTextureARB', 'glFramebufferTextureLayerARB',
'glFramebufferTextureFaceARB', 'PFNGLPROGRAMPARAMETERIARBPROC',
'PFNGLFRAMEBUFFERTEXTUREARBPROC', 'PFNGLFRAMEBUFFERTEXTURELAYERARBPROC',
'PFNGLFRAMEBUFFERTEXTUREFACEARBPROC', 'GL_ARB_half_float_vertex',
'GL_ARB_instanced_arrays', 'glVertexAttribDivisorARB',
'PFNGLVERTEXATTRIBDIVISORARBPROC', 'GL_ARB_map_buffer_range',
'glMapBufferRange', 'glFlushMappedBufferRange', 'PFNGLMAPBUFFERRANGEPROC',
'PFNGLFLUSHMAPPEDBUFFERRANGEPROC', 'GL_ARB_texture_buffer_object',
'glTexBufferARB', 'PFNGLTEXBUFFERARBPROC', 'GL_ARB_texture_compression_rgtc',
'GL_ARB_texture_rg', 'GL_ARB_vertex_array_object', 'glBindVertexArray',
'glDeleteVertexArrays', 'glGenVertexArrays', 'glIsVertexArray',
'PFNGLBINDVERTEXARRAYPROC', 'PFNGLDELETEVERTEXARRAYSPROC',
'PFNGLGENVERTEXARRAYSPROC', 'PFNGLISVERTEXARRAYPROC', 'GL_EXT_abgr',
'GL_EXT_blend_color', 'glBlendColorEXT', 'PFNGLBLENDCOLOREXTPROC',
'GL_EXT_polygon_offset', 'glPolygonOffsetEXT', 'PFNGLPOLYGONOFFSETEXTPROC',
'GL_EXT_texture', 'GL_EXT_texture3D', 'glTexImage3DEXT', 'glTexSubImage3DEXT',
'PFNGLTEXIMAGE3DEXTPROC', 'PFNGLTEXSUBIMAGE3DEXTPROC',
'GL_SGIS_texture_filter4', 'glGetTexFilterFuncSGIS', 'glTexFilterFuncSGIS',
'PFNGLGETTEXFILTERFUNCSGISPROC', 'PFNGLTEXFILTERFUNCSGISPROC',
'GL_EXT_subtexture', 'glTexSubImage1DEXT', 'glTexSubImage2DEXT',
'PFNGLTEXSUBIMAGE1DEXTPROC', 'PFNGLTEXSUBIMAGE2DEXTPROC',
'GL_EXT_copy_texture', 'glCopyTexImage1DEXT', 'glCopyTexImage2DEXT',
'glCopyTexSubImage1DEXT', 'glCopyTexSubImage2DEXT', 'glCopyTexSubImage3DEXT',
'PFNGLCOPYTEXIMAGE1DEXTPROC', 'PFNGLCOPYTEXIMAGE2DEXTPROC',
'PFNGLCOPYTEXSUBIMAGE1DEXTPROC', 'PFNGLCOPYTEXSUBIMAGE2DEXTPROC',
'PFNGLCOPYTEXSUBIMAGE3DEXTPROC', 'GL_EXT_histogram', 'glGetHistogramEXT',
'glGetHistogramParameterfvEXT', 'glGetHistogramParameterivEXT',
'glGetMinmaxEXT', 'glGetMinmaxParameterfvEXT', 'glGetMinmaxParameterivEXT',
'glHistogramEXT', 'glMinmaxEXT', 'glResetHistogramEXT', 'glResetMinmaxEXT',
'PFNGLGETHISTOGRAMEXTPROC', 'PFNGLGETHISTOGRAMPARAMETERFVEXTPROC',
'PFNGLGETHISTOGRAMPARAMETERIVEXTPROC', 'PFNGLGETMINMAXEXTPROC',
'PFNGLGETMINMAXPARAMETERFVEXTPROC', 'PFNGLGETMINMAXPARAMETERIVEXTPROC',
'PFNGLHISTOGRAMEXTPROC', 'PFNGLMINMAXEXTPROC', 'PFNGLRESETHISTOGRAMEXTPROC',
'PFNGLRESETMINMAXEXTPROC', 'GL_EXT_convolution', 'glConvolutionFilter1DEXT',
'glConvolutionFilter2DEXT', 'glConvolutionParameterfEXT',
'glConvolutionParameterfvEXT', 'glConvolutionParameteriEXT',
'glConvolutionParameterivEXT', 'glCopyConvolutionFilter1DEXT',
'glCopyConvolutionFilter2DEXT', 'glGetConvolutionFilterEXT',
'glGetConvolutionParameterfvEXT', 'glGetConvolutionParameterivEXT',
'glGetSeparableFilterEXT', 'glSeparableFilter2DEXT',
'PFNGLCONVOLUTIONFILTER1DEXTPROC', 'PFNGLCONVOLUTIONFILTER2DEXTPROC',
'PFNGLCONVOLUTIONPARAMETERFEXTPROC', 'PFNGLCONVOLUTIONPARAMETERFVEXTPROC',
'PFNGLCONVOLUTIONPARAMETERIEXTPROC', 'PFNGLCONVOLUTIONPARAMETERIVEXTPROC',
'PFNGLCOPYCONVOLUTIONFILTER1DEXTPROC', 'PFNGLCOPYCONVOLUTIONFILTER2DEXTPROC',
'PFNGLGETCONVOLUTIONFILTEREXTPROC', 'PFNGLGETCONVOLUTIONPARAMETERFVEXTPROC',
'PFNGLGETCONVOLUTIONPARAMETERIVEXTPROC', 'PFNGLGETSEPARABLEFILTEREXTPROC',
'PFNGLSEPARABLEFILTER2DEXTPROC', 'GL_EXT_color_matrix', 'GL_SGI_color_table',
'glColorTableSGI', 'glColorTableParameterfvSGI', 'glColorTableParameterivSGI',
'glCopyColorTableSGI', 'glGetColorTableSGI', 'glGetColorTableParameterfvSGI',
'glGetColorTableParameterivSGI', 'PFNGLCOLORTABLESGIPROC',
'PFNGLCOLORTABLEPARAMETERFVSGIPROC', 'PFNGLCOLORTABLEPARAMETERIVSGIPROC',
'PFNGLCOPYCOLORTABLESGIPROC', 'PFNGLGETCOLORTABLESGIPROC',
'PFNGLGETCOLORTABLEPARAMETERFVSGIPROC',
'PFNGLGETCOLORTABLEPARAMETERIVSGIPROC', 'GL_SGIX_pixel_texture',
'glPixelTexGenSGIX', 'PFNGLPIXELTEXGENSGIXPROC', 'GL_SGIS_pixel_texture',
'glPixelTexGenParameteriSGIS', 'glPixelTexGenParameterivSGIS',
'glPixelTexGenParameterfSGIS', 'glPixelTexGenParameterfvSGIS',
'glGetPixelTexGenParameterivSGIS', 'glGetPixelTexGenParameterfvSGIS',
'PFNGLPIXELTEXGENPARAMETERISGISPROC', 'PFNGLPIXELTEXGENPARAMETERIVSGISPROC',
'PFNGLPIXELTEXGENPARAMETERFSGISPROC', 'PFNGLPIXELTEXGENPARAMETERFVSGISPROC',
'PFNGLGETPIXELTEXGENPARAMETERIVSGISPROC',
'PFNGLGETPIXELTEXGENPARAMETERFVSGISPROC', 'GL_SGIS_texture4D',
'glTexImage4DSGIS', 'glTexSubImage4DSGIS', 'PFNGLTEXIMAGE4DSGISPROC',
'PFNGLTEXSUBIMAGE4DSGISPROC', 'GL_SGI_texture_color_table', 'GL_EXT_cmyka',
'GL_EXT_texture_object', 'glAreTexturesResidentEXT', 'glBindTextureEXT',
'glDeleteTexturesEXT', 'glGenTexturesEXT', 'glIsTextureEXT',
'glPrioritizeTexturesEXT', 'PFNGLARETEXTURESRESIDENTEXTPROC',
'PFNGLBINDTEXTUREEXTPROC', 'PFNGLDELETETEXTURESEXTPROC',
'PFNGLGENTEXTURESEXTPROC', 'PFNGLISTEXTUREEXTPROC',
'PFNGLPRIORITIZETEXTURESEXTPROC', 'GL_SGIS_detail_texture',
'glDetailTexFuncSGIS', 'glGetDetailTexFuncSGIS', 'PFNGLDETAILTEXFUNCSGISPROC',
'PFNGLGETDETAILTEXFUNCSGISPROC', 'GL_SGIS_sharpen_texture',
'glSharpenTexFuncSGIS', 'glGetSharpenTexFuncSGIS',
'PFNGLSHARPENTEXFUNCSGISPROC', 'PFNGLGETSHARPENTEXFUNCSGISPROC',
'GL_EXT_packed_pixels', 'GL_SGIS_texture_lod', 'GL_SGIS_multisample',
'glSampleMaskSGIS', 'glSamplePatternSGIS', 'PFNGLSAMPLEMASKSGISPROC',
'PFNGLSAMPLEPATTERNSGISPROC', 'GL_EXT_rescale_normal', 'GL_EXT_vertex_array',
'glArrayElementEXT', 'glColorPointerEXT', 'glDrawArraysEXT',
'glEdgeFlagPointerEXT', 'glGetPointervEXT', 'glIndexPointerEXT',
'glNormalPointerEXT', 'glTexCoordPointerEXT', 'glVertexPointerEXT',
'PFNGLARRAYELEMENTEXTPROC', 'PFNGLCOLORPOINTEREXTPROC',
'PFNGLDRAWARRAYSEXTPROC', 'PFNGLEDGEFLAGPOINTEREXTPROC',
'PFNGLGETPOINTERVEXTPROC', 'PFNGLINDEXPOINTEREXTPROC',
'PFNGLNORMALPOINTEREXTPROC', 'PFNGLTEXCOORDPOINTEREXTPROC',
'PFNGLVERTEXPOINTEREXTPROC', 'GL_EXT_misc_attribute',
'GL_SGIS_generate_mipmap', 'GL_SGIX_clipmap', 'GL_SGIX_shadow',
'GL_SGIS_texture_edge_clamp', 'GL_SGIS_texture_border_clamp',
'GL_EXT_blend_minmax', 'glBlendEquationEXT', 'PFNGLBLENDEQUATIONEXTPROC',
'GL_EXT_blend_subtract', 'GL_EXT_blend_logic_op', 'GL_SGIX_interlace',
'GL_SGIX_pixel_tiles', 'GL_SGIX_texture_select', 'GL_SGIX_sprite',
'glSpriteParameterfSGIX', 'glSpriteParameterfvSGIX', 'glSpriteParameteriSGIX',
'glSpriteParameterivSGIX', 'PFNGLSPRITEPARAMETERFSGIXPROC',
'PFNGLSPRITEPARAMETERFVSGIXPROC', 'PFNGLSPRITEPARAMETERISGIXPROC',
'PFNGLSPRITEPARAMETERIVSGIXPROC', 'GL_SGIX_texture_multi_buffer',
'GL_EXT_point_parameters', 'glPointParameterfEXT', 'glPointParameterfvEXT',
'PFNGLPOINTPARAMETERFEXTPROC', 'PFNGLPOINTPARAMETERFVEXTPROC',
'GL_SGIS_point_parameters', 'glPointParameterfSGIS', 'glPointParameterfvSGIS',
'PFNGLPOINTPARAMETERFSGISPROC', 'PFNGLPOINTPARAMETERFVSGISPROC',
'GL_SGIX_instruments', 'glGetInstrumentsSGIX', 'glInstrumentsBufferSGIX',
'glPollInstrumentsSGIX', 'glReadInstrumentsSGIX', 'glStartInstrumentsSGIX',
'glStopInstrumentsSGIX', 'PFNGLGETINSTRUMENTSSGIXPROC',
'PFNGLINSTRUMENTSBUFFERSGIXPROC', 'PFNGLPOLLINSTRUMENTSSGIXPROC',
'PFNGLREADINSTRUMENTSSGIXPROC', 'PFNGLSTARTINSTRUMENTSSGIXPROC',
'PFNGLSTOPINSTRUMENTSSGIXPROC', 'GL_SGIX_texture_scale_bias',
'GL_SGIX_framezoom', 'glFrameZoomSGIX', 'PFNGLFRAMEZOOMSGIXPROC',
'GL_SGIX_tag_sample_buffer', 'glTagSampleBufferSGIX',
'PFNGLTAGSAMPLEBUFFERSGIXPROC', 'GL_SGIX_polynomial_ffd',
'glDeformationMap3dSGIX', 'glDeformationMap3fSGIX', 'glDeformSGIX',
'glLoadIdentityDeformationMapSGIX', 'PFNGLDEFORMATIONMAP3DSGIXPROC',
'PFNGLDEFORMATIONMAP3FSGIXPROC', 'PFNGLDEFORMSGIXPROC',
'PFNGLLOADIDENTITYDEFORMATIONMAPSGIXPROC', 'GL_SGIX_reference_plane',
'glReferencePlaneSGIX', 'PFNGLREFERENCEPLANESGIXPROC', 'GL_SGIX_flush_raster',
'glFlushRasterSGIX', 'PFNGLFLUSHRASTERSGIXPROC', 'GL_SGIX_depth_texture',
'GL_SGIS_fog_function', 'glFogFuncSGIS', 'glGetFogFuncSGIS',
'PFNGLFOGFUNCSGISPROC', 'PFNGLGETFOGFUNCSGISPROC', 'GL_SGIX_fog_offset',
'GL_HP_image_transform', 'glImageTransformParameteriHP',
'glImageTransformParameterfHP', 'glImageTransformParameterivHP',
'glImageTransformParameterfvHP', 'glGetImageTransformParameterivHP',
'glGetImageTransformParameterfvHP', 'PFNGLIMAGETRANSFORMPARAMETERIHPPROC',
'PFNGLIMAGETRANSFORMPARAMETERFHPPROC', 'PFNGLIMAGETRANSFORMPARAMETERIVHPPROC',
'PFNGLIMAGETRANSFORMPARAMETERFVHPPROC',
'PFNGLGETIMAGETRANSFORMPARAMETERIVHPPROC',
'PFNGLGETIMAGETRANSFORMPARAMETERFVHPPROC', 'GL_HP_convolution_border_modes',
'GL_SGIX_texture_add_env', 'GL_EXT_color_subtable', 'glColorSubTableEXT',
'glCopyColorSubTableEXT', 'PFNGLCOLORSUBTABLEEXTPROC',
'PFNGLCOPYCOLORSUBTABLEEXTPROC', 'GL_PGI_vertex_hints', 'GL_PGI_misc_hints',
'glHintPGI', 'PFNGLHINTPGIPROC', 'GL_EXT_paletted_texture', 'glColorTableEXT',
'glGetColorTableEXT', 'glGetColorTableParameterivEXT',
'glGetColorTableParameterfvEXT', 'PFNGLCOLORTABLEEXTPROC',
'PFNGLGETCOLORTABLEEXTPROC', 'PFNGLGETCOLORTABLEPARAMETERIVEXTPROC',
'PFNGLGETCOLORTABLEPARAMETERFVEXTPROC', 'GL_EXT_clip_volume_hint',
'GL_SGIX_list_priority', 'glGetListParameterfvSGIX',
'glGetListParameterivSGIX', 'glListParameterfSGIX', 'glListParameterfvSGIX',
'glListParameteriSGIX', 'glListParameterivSGIX',
'PFNGLGETLISTPARAMETERFVSGIXPROC', 'PFNGLGETLISTPARAMETERIVSGIXPROC',
'PFNGLLISTPARAMETERFSGIXPROC', 'PFNGLLISTPARAMETERFVSGIXPROC',
'PFNGLLISTPARAMETERISGIXPROC', 'PFNGLLISTPARAMETERIVSGIXPROC',
'GL_SGIX_ir_instrument1', 'GL_SGIX_calligraphic_fragment',
'GL_SGIX_texture_lod_bias', 'GL_SGIX_shadow_ambient', 'GL_EXT_index_texture',
'GL_EXT_index_material', 'glIndexMaterialEXT', 'PFNGLINDEXMATERIALEXTPROC',
'GL_EXT_index_func', 'glIndexFuncEXT', 'PFNGLINDEXFUNCEXTPROC',
'GL_EXT_index_array_formats', 'GL_EXT_compiled_vertex_array',
'glLockArraysEXT', 'glUnlockArraysEXT', 'PFNGLLOCKARRAYSEXTPROC',
'PFNGLUNLOCKARRAYSEXTPROC', 'GL_EXT_cull_vertex', 'glCullParameterdvEXT',
'glCullParameterfvEXT', 'PFNGLCULLPARAMETERDVEXTPROC',
'PFNGLCULLPARAMETERFVEXTPROC', 'GL_SGIX_ycrcb', 'GL_SGIX_fragment_lighting',
'glFragmentColorMaterialSGIX', 'glFragmentLightfSGIX',
'glFragmentLightfvSGIX', 'glFragmentLightiSGIX', 'glFragmentLightivSGIX',
'glFragmentLightModelfSGIX', 'glFragmentLightModelfvSGIX',
'glFragmentLightModeliSGIX', 'glFragmentLightModelivSGIX',
'glFragmentMaterialfSGIX', 'glFragmentMaterialfvSGIX',
'glFragmentMaterialiSGIX', 'glFragmentMaterialivSGIX',
'glGetFragmentLightfvSGIX', 'glGetFragmentLightivSGIX',
'glGetFragmentMaterialfvSGIX', 'glGetFragmentMaterialivSGIX',
'glLightEnviSGIX', 'PFNGLFRAGMENTCOLORMATERIALSGIXPROC',
'PFNGLFRAGMENTLIGHTFSGIXPROC', 'PFNGLFRAGMENTLIGHTFVSGIXPROC',
'PFNGLFRAGMENTLIGHTISGIXPROC', 'PFNGLFRAGMENTLIGHTIVSGIXPROC',
'PFNGLFRAGMENTLIGHTMODELFSGIXPROC', 'PFNGLFRAGMENTLIGHTMODELFVSGIXPROC',
'PFNGLFRAGMENTLIGHTMODELISGIXPROC', 'PFNGLFRAGMENTLIGHTMODELIVSGIXPROC',
'PFNGLFRAGMENTMATERIALFSGIXPROC', 'PFNGLFRAGMENTMATERIALFVSGIXPROC',
'PFNGLFRAGMENTMATERIALISGIXPROC', 'PFNGLFRAGMENTMATERIALIVSGIXPROC',
'PFNGLGETFRAGMENTLIGHTFVSGIXPROC', 'PFNGLGETFRAGMENTLIGHTIVSGIXPROC',
'PFNGLGETFRAGMENTMATERIALFVSGIXPROC', 'PFNGLGETFRAGMENTMATERIALIVSGIXPROC',
'PFNGLLIGHTENVISGIXPROC', 'GL_IBM_rasterpos_clip', 'GL_HP_texture_lighting',
'GL_EXT_draw_range_elements', 'glDrawRangeElementsEXT',
'PFNGLDRAWRANGEELEMENTSEXTPROC', 'GL_WIN_phong_shading',
'GL_WIN_specular_fog', 'GL_EXT_light_texture', 'glApplyTextureEXT',
'glTextureLightEXT', 'glTextureMaterialEXT', 'PFNGLAPPLYTEXTUREEXTPROC',
'PFNGLTEXTURELIGHTEXTPROC', 'PFNGLTEXTUREMATERIALEXTPROC',
'GL_SGIX_blend_alpha_minmax', 'GL_EXT_bgra', 'GL_SGIX_async',
'glAsyncMarkerSGIX', 'glFinishAsyncSGIX', 'glPollAsyncSGIX',
'glGenAsyncMarkersSGIX', 'glDeleteAsyncMarkersSGIX', 'glIsAsyncMarkerSGIX',
'PFNGLASYNCMARKERSGIXPROC', 'PFNGLFINISHASYNCSGIXPROC',
'PFNGLPOLLASYNCSGIXPROC', 'PFNGLGENASYNCMARKERSSGIXPROC',
'PFNGLDELETEASYNCMARKERSSGIXPROC', 'PFNGLISASYNCMARKERSGIXPROC',
'GL_SGIX_async_pixel', 'GL_SGIX_async_histogram', 'GL_INTEL_parallel_arrays',
'glVertexPointervINTEL', 'glNormalPointervINTEL', 'glColorPointervINTEL',
'glTexCoordPointervINTEL', 'PFNGLVERTEXPOINTERVINTELPROC',
'PFNGLNORMALPOINTERVINTELPROC', 'PFNGLCOLORPOINTERVINTELPROC',
'PFNGLTEXCOORDPOINTERVINTELPROC', 'GL_HP_occlusion_test',
'GL_EXT_pixel_transform', 'glPixelTransformParameteriEXT',
'glPixelTransformParameterfEXT', 'glPixelTransformParameterivEXT',
'glPixelTransformParameterfvEXT', 'PFNGLPIXELTRANSFORMPARAMETERIEXTPROC',
'PFNGLPIXELTRANSFORMPARAMETERFEXTPROC',
'PFNGLPIXELTRANSFORMPARAMETERIVEXTPROC',
'PFNGLPIXELTRANSFORMPARAMETERFVEXTPROC', 'GL_EXT_pixel_transform_color_table',
'GL_EXT_shared_texture_palette', 'GL_EXT_separate_specular_color',
'GL_EXT_secondary_color', 'glSecondaryColor3bEXT', 'glSecondaryColor3bvEXT',
'glSecondaryColor3dEXT', 'glSecondaryColor3dvEXT', 'glSecondaryColor3fEXT',
'glSecondaryColor3fvEXT', 'glSecondaryColor3iEXT', 'glSecondaryColor3ivEXT',
'glSecondaryColor3sEXT', 'glSecondaryColor3svEXT', 'glSecondaryColor3ubEXT',
'glSecondaryColor3ubvEXT', 'glSecondaryColor3uiEXT',
'glSecondaryColor3uivEXT', 'glSecondaryColor3usEXT',
'glSecondaryColor3usvEXT', 'glSecondaryColorPointerEXT',
'PFNGLSECONDARYCOLOR3BEXTPROC', 'PFNGLSECONDARYCOLOR3BVEXTPROC',
'PFNGLSECONDARYCOLOR3DEXTPROC', 'PFNGLSECONDARYCOLOR3DVEXTPROC',
'PFNGLSECONDARYCOLOR3FEXTPROC', 'PFNGLSECONDARYCOLOR3FVEXTPROC',
'PFNGLSECONDARYCOLOR3IEXTPROC', 'PFNGLSECONDARYCOLOR3IVEXTPROC',
'PFNGLSECONDARYCOLOR3SEXTPROC', 'PFNGLSECONDARYCOLOR3SVEXTPROC',
'PFNGLSECONDARYCOLOR3UBEXTPROC', 'PFNGLSECONDARYCOLOR3UBVEXTPROC',
'PFNGLSECONDARYCOLOR3UIEXTPROC', 'PFNGLSECONDARYCOLOR3UIVEXTPROC',
'PFNGLSECONDARYCOLOR3USEXTPROC', 'PFNGLSECONDARYCOLOR3USVEXTPROC',
'PFNGLSECONDARYCOLORPOINTEREXTPROC', 'GL_EXT_texture_perturb_normal',
'glTextureNormalEXT', 'PFNGLTEXTURENORMALEXTPROC', 'GL_EXT_multi_draw_arrays',
'glMultiDrawArraysEXT', 'glMultiDrawElementsEXT',
'PFNGLMULTIDRAWARRAYSEXTPROC', 'PFNGLMULTIDRAWELEMENTSEXTPROC',
'GL_EXT_fog_coord', 'glFogCoordfEXT', 'glFogCoordfvEXT', 'glFogCoorddEXT',
'glFogCoorddvEXT', 'glFogCoordPointerEXT', 'PFNGLFOGCOORDFEXTPROC',
'PFNGLFOGCOORDFVEXTPROC', 'PFNGLFOGCOORDDEXTPROC', 'PFNGLFOGCOORDDVEXTPROC',
'PFNGLFOGCOORDPOINTEREXTPROC', 'GL_REND_screen_coordinates',
'GL_EXT_coordinate_frame', 'glTangent3bEXT', 'glTangent3bvEXT',
'glTangent3dEXT', 'glTangent3dvEXT', 'glTangent3fEXT', 'glTangent3fvEXT',
'glTangent3iEXT', 'glTangent3ivEXT', 'glTangent3sEXT', 'glTangent3svEXT',
'glBinormal3bEXT', 'glBinormal3bvEXT', 'glBinormal3dEXT', 'glBinormal3dvEXT',
'glBinormal3fEXT', 'glBinormal3fvEXT', 'glBinormal3iEXT', 'glBinormal3ivEXT',
'glBinormal3sEXT', 'glBinormal3svEXT', 'glTangentPointerEXT',
'glBinormalPointerEXT', 'PFNGLTANGENT3BEXTPROC', 'PFNGLTANGENT3BVEXTPROC',
'PFNGLTANGENT3DEXTPROC', 'PFNGLTANGENT3DVEXTPROC', 'PFNGLTANGENT3FEXTPROC',
'PFNGLTANGENT3FVEXTPROC', 'PFNGLTANGENT3IEXTPROC', 'PFNGLTANGENT3IVEXTPROC',
'PFNGLTANGENT3SEXTPROC', 'PFNGLTANGENT3SVEXTPROC', 'PFNGLBINORMAL3BEXTPROC',
'PFNGLBINORMAL3BVEXTPROC', 'PFNGLBINORMAL3DEXTPROC',
'PFNGLBINORMAL3DVEXTPROC', 'PFNGLBINORMAL3FEXTPROC',
'PFNGLBINORMAL3FVEXTPROC', 'PFNGLBINORMAL3IEXTPROC',
'PFNGLBINORMAL3IVEXTPROC', 'PFNGLBINORMAL3SEXTPROC',
'PFNGLBINORMAL3SVEXTPROC', 'PFNGLTANGENTPOINTEREXTPROC',
'PFNGLBINORMALPOINTEREXTPROC', 'GL_EXT_texture_env_combine',
'GL_APPLE_specular_vector', 'GL_APPLE_transform_hint', 'GL_SGIX_fog_scale',
'GL_SUNX_constant_data', 'glFinishTextureSUNX', 'PFNGLFINISHTEXTURESUNXPROC',
'GL_SUN_global_alpha', 'glGlobalAlphaFactorbSUN', 'glGlobalAlphaFactorsSUN',
'glGlobalAlphaFactoriSUN', 'glGlobalAlphaFactorfSUN',
'glGlobalAlphaFactordSUN', 'glGlobalAlphaFactorubSUN',
'glGlobalAlphaFactorusSUN', 'glGlobalAlphaFactoruiSUN',
'PFNGLGLOBALALPHAFACTORBSUNPROC', 'PFNGLGLOBALALPHAFACTORSSUNPROC',
'PFNGLGLOBALALPHAFACTORISUNPROC', 'PFNGLGLOBALALPHAFACTORFSUNPROC',
'PFNGLGLOBALALPHAFACTORDSUNPROC', 'PFNGLGLOBALALPHAFACTORUBSUNPROC',
'PFNGLGLOBALALPHAFACTORUSSUNPROC', 'PFNGLGLOBALALPHAFACTORUISUNPROC',
'GL_SUN_triangle_list', 'glReplacementCodeuiSUN', 'glReplacementCodeusSUN',
'glReplacementCodeubSUN', 'glReplacementCodeuivSUN',
'glReplacementCodeusvSUN', 'glReplacementCodeubvSUN',
'glReplacementCodePointerSUN', 'PFNGLREPLACEMENTCODEUISUNPROC',
'PFNGLREPLACEMENTCODEUSSUNPROC', 'PFNGLREPLACEMENTCODEUBSUNPROC',
'PFNGLREPLACEMENTCODEUIVSUNPROC', 'PFNGLREPLACEMENTCODEUSVSUNPROC',
'PFNGLREPLACEMENTCODEUBVSUNPROC', 'PFNGLREPLACEMENTCODEPOINTERSUNPROC',
'GL_SUN_vertex', 'glColor4ubVertex2fSUN', 'glColor4ubVertex2fvSUN',
'glColor4ubVertex3fSUN', 'glColor4ubVertex3fvSUN', 'glColor3fVertex3fSUN',
'glColor3fVertex3fvSUN', 'glNormal3fVertex3fSUN', 'glNormal3fVertex3fvSUN',
'glColor4fNormal3fVertex3fSUN', 'glColor4fNormal3fVertex3fvSUN',
'glTexCoord2fVertex3fSUN', 'glTexCoord2fVertex3fvSUN',
'glTexCoord4fVertex4fSUN', 'glTexCoord4fVertex4fvSUN',
'glTexCoord2fColor4ubVertex3fSUN', 'glTexCoord2fColor4ubVertex3fvSUN',
'glTexCoord2fColor3fVertex3fSUN', 'glTexCoord2fColor3fVertex3fvSUN',
'glTexCoord2fNormal3fVertex3fSUN', 'glTexCoord2fNormal3fVertex3fvSUN',
'glTexCoord2fColor4fNormal3fVertex3fSUN',
'glTexCoord2fColor4fNormal3fVertex3fvSUN',
'glTexCoord4fColor4fNormal3fVertex4fSUN',
'glTexCoord4fColor4fNormal3fVertex4fvSUN', 'glReplacementCodeuiVertex3fSUN',
'glReplacementCodeuiVertex3fvSUN', 'glReplacementCodeuiColor4ubVertex3fSUN',
'glReplacementCodeuiColor4ubVertex3fvSUN',
'glReplacementCodeuiColor3fVertex3fSUN',
'glReplacementCodeuiColor3fVertex3fvSUN',
'glReplacementCodeuiNormal3fVertex3fSUN',
'glReplacementCodeuiNormal3fVertex3fvSUN',
'glReplacementCodeuiColor4fNormal3fVertex3fSUN',
'glReplacementCodeuiColor4fNormal3fVertex3fvSUN',
'glReplacementCodeuiTexCoord2fVertex3fSUN',
'glReplacementCodeuiTexCoord2fVertex3fvSUN',
'glReplacementCodeuiTexCoord2fNormal3fVertex3fSUN',
'glReplacementCodeuiTexCoord2fNormal3fVertex3fvSUN',
'glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fSUN',
'glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fvSUN',
'PFNGLCOLOR4UBVERTEX2FSUNPROC', 'PFNGLCOLOR4UBVERTEX2FVSUNPROC',
'PFNGLCOLOR4UBVERTEX3FSUNPROC', 'PFNGLCOLOR4UBVERTEX3FVSUNPROC',
'PFNGLCOLOR3FVERTEX3FSUNPROC', 'PFNGLCOLOR3FVERTEX3FVSUNPROC',
'PFNGLNORMAL3FVERTEX3FSUNPROC', 'PFNGLNORMAL3FVERTEX3FVSUNPROC',
'PFNGLCOLOR4FNORMAL3FVERTEX3FSUNPROC', 'PFNGLCOLOR4FNORMAL3FVERTEX3FVSUNPROC',
'PFNGLTEXCOORD2FVERTEX3FSUNPROC', 'PFNGLTEXCOORD2FVERTEX3FVSUNPROC',
'PFNGLTEXCOORD4FVERTEX4FSUNPROC', 'PFNGLTEXCOORD4FVERTEX4FVSUNPROC',
'PFNGLTEXCOORD2FCOLOR4UBVERTEX3FSUNPROC',
'PFNGLTEXCOORD2FCOLOR4UBVERTEX3FVSUNPROC',
'PFNGLTEXCOORD2FCOLOR3FVERTEX3FSUNPROC',
'PFNGLTEXCOORD2FCOLOR3FVERTEX3FVSUNPROC',
'PFNGLTEXCOORD2FNORMAL3FVERTEX3FSUNPROC',
'PFNGLTEXCOORD2FNORMAL3FVERTEX3FVSUNPROC',
'PFNGLTEXCOORD2FCOLOR4FNORMAL3FVERTEX3FSUNPROC',
'PFNGLTEXCOORD2FCOLOR4FNORMAL3FVERTEX3FVSUNPROC',
'PFNGLTEXCOORD4FCOLOR4FNORMAL3FVERTEX4FSUNPROC',
'PFNGLTEXCOORD4FCOLOR4FNORMAL3FVERTEX4FVSUNPROC',
'PFNGLREPLACEMENTCODEUIVERTEX3FSUNPROC',
'PFNGLREPLACEMENTCODEUIVERTEX3FVSUNPROC',
'PFNGLREPLACEMENTCODEUICOLOR4UBVERTEX3FSUNPROC',
'PFNGLREPLACEMENTCODEUICOLOR4UBVERTEX3FVSUNPROC',
'PFNGLREPLACEMENTCODEUICOLOR3FVERTEX3FSUNPROC',
'PFNGLREPLACEMENTCODEUICOLOR3FVERTEX3FVSUNPROC',
'PFNGLREPLACEMENTCODEUINORMAL3FVERTEX3FSUNPROC',
'PFNGLREPLACEMENTCODEUINORMAL3FVERTEX3FVSUNPROC',
'PFNGLREPLACEMENTCODEUICOLOR4FNORMAL3FVERTEX3FSUNPROC',
'PFNGLREPLACEMENTCODEUICOLOR4FNORMAL3FVERTEX3FVSUNPROC',
'PFNGLREPLACEMENTCODEUITEXCOORD2FVERTEX3FSUNPROC',
'PFNGLREPLACEMENTCODEUITEXCOORD2FVERTEX3FVSUNPROC',
'PFNGLREPLACEMENTCODEUITEXCOORD2FNORMAL3FVERTEX3FSUNPROC',
'PFNGLREPLACEMENTCODEUITEXCOORD2FNORMAL3FVERTEX3FVSUNPROC',
'PFNGLREPLACEMENTCODEUITEXCOORD2FCOLOR4FNORMAL3FVERTEX3FSUNPROC',
'PFNGLREPLACEMENTCODEUITEXCOORD2FCOLOR4FNORMAL3FVERTEX3FVSUNPROC',
'GL_EXT_blend_func_separate', 'glBlendFuncSeparateEXT',
'PFNGLBLENDFUNCSEPARATEEXTPROC', 'GL_INGR_blend_func_separate',
'glBlendFuncSeparateINGR', 'PFNGLBLENDFUNCSEPARATEINGRPROC',
'GL_INGR_color_clamp', 'GL_INGR_interlace_read', 'GL_EXT_stencil_wrap',
'GL_EXT_422_pixels', 'GL_NV_texgen_reflection',
'GL_SUN_convolution_border_modes', 'GL_EXT_texture_env_add',
'GL_EXT_texture_lod_bias', 'GL_EXT_texture_filter_anisotropic',
'GL_EXT_vertex_weighting', 'glVertexWeightfEXT', 'glVertexWeightfvEXT',
'glVertexWeightPointerEXT', 'PFNGLVERTEXWEIGHTFEXTPROC',
'PFNGLVERTEXWEIGHTFVEXTPROC', 'PFNGLVERTEXWEIGHTPOINTEREXTPROC',
'GL_NV_light_max_exponent', 'GL_NV_vertex_array_range',
'glFlushVertexArrayRangeNV', 'glVertexArrayRangeNV',
'PFNGLFLUSHVERTEXARRAYRANGENVPROC', 'PFNGLVERTEXARRAYRANGENVPROC',
'GL_NV_register_combiners', 'glCombinerParameterfvNV',
'glCombinerParameterfNV', 'glCombinerParameterivNV', 'glCombinerParameteriNV',
'glCombinerInputNV', 'glCombinerOutputNV', 'glFinalCombinerInputNV',
'glGetCombinerInputParameterfvNV', 'glGetCombinerInputParameterivNV',
'glGetCombinerOutputParameterfvNV', 'glGetCombinerOutputParameterivNV',
'glGetFinalCombinerInputParameterfvNV',
'glGetFinalCombinerInputParameterivNV', 'PFNGLCOMBINERPARAMETERFVNVPROC',
'PFNGLCOMBINERPARAMETERFNVPROC', 'PFNGLCOMBINERPARAMETERIVNVPROC',
'PFNGLCOMBINERPARAMETERINVPROC', 'PFNGLCOMBINERINPUTNVPROC',
'PFNGLCOMBINEROUTPUTNVPROC', 'PFNGLFINALCOMBINERINPUTNVPROC',
'PFNGLGETCOMBINERINPUTPARAMETERFVNVPROC',
'PFNGLGETCOMBINERINPUTPARAMETERIVNVPROC',
'PFNGLGETCOMBINEROUTPUTPARAMETERFVNVPROC',
'PFNGLGETCOMBINEROUTPUTPARAMETERIVNVPROC',
'PFNGLGETFINALCOMBINERINPUTPARAMETERFVNVPROC',
'PFNGLGETFINALCOMBINERINPUTPARAMETERIVNVPROC', 'GL_NV_fog_distance',
'GL_NV_texgen_emboss', 'GL_NV_blend_square', 'GL_NV_texture_env_combine4',
'GL_MESA_resize_buffers', 'glResizeBuffersMESA', 'PFNGLRESIZEBUFFERSMESAPROC',
'GL_MESA_window_pos', 'glWindowPos2dMESA', 'glWindowPos2dvMESA',
'glWindowPos2fMESA', 'glWindowPos2fvMESA', 'glWindowPos2iMESA',
'glWindowPos2ivMESA', 'glWindowPos2sMESA', 'glWindowPos2svMESA',
'glWindowPos3dMESA', 'glWindowPos3dvMESA', 'glWindowPos3fMESA',
'glWindowPos3fvMESA', 'glWindowPos3iMESA', 'glWindowPos3ivMESA',
'glWindowPos3sMESA', 'glWindowPos3svMESA', 'glWindowPos4dMESA',
'glWindowPos4dvMESA', 'glWindowPos4fMESA', 'glWindowPos4fvMESA',
'glWindowPos4iMESA', 'glWindowPos4ivMESA', 'glWindowPos4sMESA',
'glWindowPos4svMESA', 'PFNGLWINDOWPOS2DMESAPROC', 'PFNGLWINDOWPOS2DVMESAPROC',
'PFNGLWINDOWPOS2FMESAPROC', 'PFNGLWINDOWPOS2FVMESAPROC',
'PFNGLWINDOWPOS2IMESAPROC', 'PFNGLWINDOWPOS2IVMESAPROC',
'PFNGLWINDOWPOS2SMESAPROC', 'PFNGLWINDOWPOS2SVMESAPROC',
'PFNGLWINDOWPOS3DMESAPROC', 'PFNGLWINDOWPOS3DVMESAPROC',
'PFNGLWINDOWPOS3FMESAPROC', 'PFNGLWINDOWPOS3FVMESAPROC',
'PFNGLWINDOWPOS3IMESAPROC', 'PFNGLWINDOWPOS3IVMESAPROC',
'PFNGLWINDOWPOS3SMESAPROC', 'PFNGLWINDOWPOS3SVMESAPROC',
'PFNGLWINDOWPOS4DMESAPROC', 'PFNGLWINDOWPOS4DVMESAPROC',
'PFNGLWINDOWPOS4FMESAPROC', 'PFNGLWINDOWPOS4FVMESAPROC',
'PFNGLWINDOWPOS4IMESAPROC', 'PFNGLWINDOWPOS4IVMESAPROC',
'PFNGLWINDOWPOS4SMESAPROC', 'PFNGLWINDOWPOS4SVMESAPROC',
'GL_EXT_texture_compression_s3tc', 'GL_IBM_cull_vertex',
'GL_IBM_multimode_draw_arrays', 'glMultiModeDrawArraysIBM',
'glMultiModeDrawElementsIBM', 'PFNGLMULTIMODEDRAWARRAYSIBMPROC',
'PFNGLMULTIMODEDRAWELEMENTSIBMPROC', 'GL_IBM_vertex_array_lists',
'glColorPointerListIBM', 'glSecondaryColorPointerListIBM',
'glEdgeFlagPointerListIBM', 'glFogCoordPointerListIBM',
'glIndexPointerListIBM', 'glNormalPointerListIBM', 'glTexCoordPointerListIBM',
'glVertexPointerListIBM', 'PFNGLCOLORPOINTERLISTIBMPROC',
'PFNGLSECONDARYCOLORPOINTERLISTIBMPROC', 'PFNGLEDGEFLAGPOINTERLISTIBMPROC',
'PFNGLFOGCOORDPOINTERLISTIBMPROC', 'PFNGLINDEXPOINTERLISTIBMPROC',
'PFNGLNORMALPOINTERLISTIBMPROC', 'PFNGLTEXCOORDPOINTERLISTIBMPROC',
'PFNGLVERTEXPOINTERLISTIBMPROC', 'GL_SGIX_subsample', 'GL_SGIX_ycrcba',
'GL_SGIX_ycrcb_subsample', 'GL_SGIX_depth_pass_instrument',
'GL_3DFX_texture_compression_FXT1', 'GL_3DFX_multisample', 'GL_3DFX_tbuffer',
'glTbufferMask3DFX', 'PFNGLTBUFFERMASK3DFXPROC', 'GL_EXT_multisample',
'glSampleMaskEXT', 'glSamplePatternEXT', 'PFNGLSAMPLEMASKEXTPROC',
'PFNGLSAMPLEPATTERNEXTPROC', 'GL_SGIX_vertex_preclip',
'GL_SGIX_convolution_accuracy', 'GL_SGIX_resample',
'GL_SGIS_point_line_texgen', 'GL_SGIS_texture_color_mask',
'glTextureColorMaskSGIS', 'PFNGLTEXTURECOLORMASKSGISPROC',
'GL_SGIX_igloo_interface', 'glIglooInterfaceSGIX',
'PFNGLIGLOOINTERFACESGIXPROC', 'GL_EXT_texture_env_dot3',
'GL_ATI_texture_mirror_once', 'GL_NV_fence', 'glDeleteFencesNV',
'glGenFencesNV', 'glIsFenceNV', 'glTestFenceNV', 'glGetFenceivNV',
'glFinishFenceNV', 'glSetFenceNV', 'PFNGLDELETEFENCESNVPROC',
'PFNGLGENFENCESNVPROC', 'PFNGLISFENCENVPROC', 'PFNGLTESTFENCENVPROC',
'PFNGLGETFENCEIVNVPROC', 'PFNGLFINISHFENCENVPROC', 'PFNGLSETFENCENVPROC',
'GL_NV_evaluators', 'glMapControlPointsNV', 'glMapParameterivNV',
'glMapParameterfvNV', 'glGetMapControlPointsNV', 'glGetMapParameterivNV',
'glGetMapParameterfvNV', 'glGetMapAttribParameterivNV',
'glGetMapAttribParameterfvNV', 'glEvalMapsNV', 'PFNGLMAPCONTROLPOINTSNVPROC',
'PFNGLMAPPARAMETERIVNVPROC', 'PFNGLMAPPARAMETERFVNVPROC',
'PFNGLGETMAPCONTROLPOINTSNVPROC', 'PFNGLGETMAPPARAMETERIVNVPROC',
'PFNGLGETMAPPARAMETERFVNVPROC', 'PFNGLGETMAPATTRIBPARAMETERIVNVPROC',
'PFNGLGETMAPATTRIBPARAMETERFVNVPROC', 'PFNGLEVALMAPSNVPROC',
'GL_NV_packed_depth_stencil', 'GL_EXT_packed_depth_stencil',
'GL_NV_register_combiners2', 'glCombinerStageParameterfvNV',
'glGetCombinerStageParameterfvNV', 'PFNGLCOMBINERSTAGEPARAMETERFVNVPROC',
'PFNGLGETCOMBINERSTAGEPARAMETERFVNVPROC', 'GL_NV_texture_compression_vtc',
'GL_NV_texture_rectangle', 'GL_NV_texture_shader', 'GL_NV_texture_shader2',
'GL_NV_vertex_array_range2', 'GL_NV_vertex_program',
'glAreProgramsResidentNV', 'glBindProgramNV', 'glDeleteProgramsNV',
'glExecuteProgramNV', 'glGenProgramsNV', 'glGetProgramParameterdvNV',
'glGetProgramParameterfvNV', 'glGetProgramivNV', 'glGetProgramStringNV',
'glGetTrackMatrixivNV', 'glGetVertexAttribdvNV', 'glGetVertexAttribfvNV',
'glGetVertexAttribivNV', 'glGetVertexAttribPointervNV', 'glIsProgramNV',
'glLoadProgramNV', 'glProgramParameter4dNV', 'glProgramParameter4dvNV',
'glProgramParameter4fNV', 'glProgramParameter4fvNV',
'glProgramParameters4dvNV', 'glProgramParameters4fvNV',
'glRequestResidentProgramsNV', 'glTrackMatrixNV', 'glVertexAttribPointerNV',
'glVertexAttrib1dNV', 'glVertexAttrib1dvNV', 'glVertexAttrib1fNV',
'glVertexAttrib1fvNV', 'glVertexAttrib1sNV', 'glVertexAttrib1svNV',
'glVertexAttrib2dNV', 'glVertexAttrib2dvNV', 'glVertexAttrib2fNV',
'glVertexAttrib2fvNV', 'glVertexAttrib2sNV', 'glVertexAttrib2svNV',
'glVertexAttrib3dNV', 'glVertexAttrib3dvNV', 'glVertexAttrib3fNV',
'glVertexAttrib3fvNV', 'glVertexAttrib3sNV', 'glVertexAttrib3svNV',
'glVertexAttrib4dNV', 'glVertexAttrib4dvNV', 'glVertexAttrib4fNV',
'glVertexAttrib4fvNV', 'glVertexAttrib4sNV', 'glVertexAttrib4svNV',
'glVertexAttrib4ubNV', 'glVertexAttrib4ubvNV', 'glVertexAttribs1dvNV',
'glVertexAttribs1fvNV', 'glVertexAttribs1svNV', 'glVertexAttribs2dvNV',
'glVertexAttribs2fvNV', 'glVertexAttribs2svNV', 'glVertexAttribs3dvNV',
'glVertexAttribs3fvNV', 'glVertexAttribs3svNV', 'glVertexAttribs4dvNV',
'glVertexAttribs4fvNV', 'glVertexAttribs4svNV', 'glVertexAttribs4ubvNV',
'PFNGLAREPROGRAMSRESIDENTNVPROC', 'PFNGLBINDPROGRAMNVPROC',
'PFNGLDELETEPROGRAMSNVPROC', 'PFNGLEXECUTEPROGRAMNVPROC',
'PFNGLGENPROGRAMSNVPROC', 'PFNGLGETPROGRAMPARAMETERDVNVPROC',
'PFNGLGETPROGRAMPARAMETERFVNVPROC', 'PFNGLGETPROGRAMIVNVPROC',
'PFNGLGETPROGRAMSTRINGNVPROC', 'PFNGLGETTRACKMATRIXIVNVPROC',
'PFNGLGETVERTEXATTRIBDVNVPROC', 'PFNGLGETVERTEXATTRIBFVNVPROC',
'PFNGLGETVERTEXATTRIBIVNVPROC', 'PFNGLGETVERTEXATTRIBPOINTERVNVPROC',
'PFNGLISPROGRAMNVPROC', 'PFNGLLOADPROGRAMNVPROC',
'PFNGLPROGRAMPARAMETER4DNVPROC', 'PFNGLPROGRAMPARAMETER4DVNVPROC',
'PFNGLPROGRAMPARAMETER4FNVPROC', 'PFNGLPROGRAMPARAMETER4FVNVPROC',
'PFNGLPROGRAMPARAMETERS4DVNVPROC', 'PFNGLPROGRAMPARAMETERS4FVNVPROC',
'PFNGLREQUESTRESIDENTPROGRAMSNVPROC', 'PFNGLTRACKMATRIXNVPROC',
'PFNGLVERTEXATTRIBPOINTERNVPROC', 'PFNGLVERTEXATTRIB1DNVPROC',
'PFNGLVERTEXATTRIB1DVNVPROC', 'PFNGLVERTEXATTRIB1FNVPROC',
'PFNGLVERTEXATTRIB1FVNVPROC', 'PFNGLVERTEXATTRIB1SNVPROC',
'PFNGLVERTEXATTRIB1SVNVPROC', 'PFNGLVERTEXATTRIB2DNVPROC',
'PFNGLVERTEXATTRIB2DVNVPROC', 'PFNGLVERTEXATTRIB2FNVPROC',
'PFNGLVERTEXATTRIB2FVNVPROC', 'PFNGLVERTEXATTRIB2SNVPROC',
'PFNGLVERTEXATTRIB2SVNVPROC', 'PFNGLVERTEXATTRIB3DNVPROC',
'PFNGLVERTEXATTRIB3DVNVPROC', 'PFNGLVERTEXATTRIB3FNVPROC',
'PFNGLVERTEXATTRIB3FVNVPROC', 'PFNGLVERTEXATTRIB3SNVPROC',
'PFNGLVERTEXATTRIB3SVNVPROC', 'PFNGLVERTEXATTRIB4DNVPROC',
'PFNGLVERTEXATTRIB4DVNVPROC', 'PFNGLVERTEXATTRIB4FNVPROC',
'PFNGLVERTEXATTRIB4FVNVPROC', 'PFNGLVERTEXATTRIB4SNVPROC',
'PFNGLVERTEXATTRIB4SVNVPROC', 'PFNGLVERTEXATTRIB4UBNVPROC',
'PFNGLVERTEXATTRIB4UBVNVPROC', 'PFNGLVERTEXATTRIBS1DVNVPROC',
'PFNGLVERTEXATTRIBS1FVNVPROC', 'PFNGLVERTEXATTRIBS1SVNVPROC',
'PFNGLVERTEXATTRIBS2DVNVPROC', 'PFNGLVERTEXATTRIBS2FVNVPROC',
'PFNGLVERTEXATTRIBS2SVNVPROC', 'PFNGLVERTEXATTRIBS3DVNVPROC',
'PFNGLVERTEXATTRIBS3FVNVPROC', 'PFNGLVERTEXATTRIBS3SVNVPROC',
'PFNGLVERTEXATTRIBS4DVNVPROC', 'PFNGLVERTEXATTRIBS4FVNVPROC',
'PFNGLVERTEXATTRIBS4SVNVPROC', 'PFNGLVERTEXATTRIBS4UBVNVPROC',
'GL_SGIX_texture_coordinate_clamp', 'GL_SGIX_scalebias_hint',
'GL_OML_interlace', 'GL_OML_subsample', 'GL_OML_resample',
'GL_NV_copy_depth_to_color', 'GL_ATI_envmap_bumpmap',
'glTexBumpParameterivATI', 'glTexBumpParameterfvATI',
'glGetTexBumpParameterivATI', 'glGetTexBumpParameterfvATI',
'PFNGLTEXBUMPPARAMETERIVATIPROC', 'PFNGLTEXBUMPPARAMETERFVATIPROC',
'PFNGLGETTEXBUMPPARAMETERIVATIPROC', 'PFNGLGETTEXBUMPPARAMETERFVATIPROC',
'GL_ATI_fragment_shader', 'glGenFragmentShadersATI',
'glBindFragmentShaderATI', 'glDeleteFragmentShaderATI',
'glBeginFragmentShaderATI', 'glEndFragmentShaderATI', 'glPassTexCoordATI',
'glSampleMapATI', 'glColorFragmentOp1ATI', 'glColorFragmentOp2ATI',
'glColorFragmentOp3ATI', 'glAlphaFragmentOp1ATI', 'glAlphaFragmentOp2ATI',
'glAlphaFragmentOp3ATI', 'glSetFragmentShaderConstantATI',
'PFNGLGENFRAGMENTSHADERSATIPROC', 'PFNGLBINDFRAGMENTSHADERATIPROC',
'PFNGLDELETEFRAGMENTSHADERATIPROC', 'PFNGLBEGINFRAGMENTSHADERATIPROC',
'PFNGLENDFRAGMENTSHADERATIPROC', 'PFNGLPASSTEXCOORDATIPROC',
'PFNGLSAMPLEMAPATIPROC', 'PFNGLCOLORFRAGMENTOP1ATIPROC',
'PFNGLCOLORFRAGMENTOP2ATIPROC', 'PFNGLCOLORFRAGMENTOP3ATIPROC',
'PFNGLALPHAFRAGMENTOP1ATIPROC', 'PFNGLALPHAFRAGMENTOP2ATIPROC',
'PFNGLALPHAFRAGMENTOP3ATIPROC', 'PFNGLSETFRAGMENTSHADERCONSTANTATIPROC',
'GL_ATI_pn_triangles', 'glPNTrianglesiATI', 'glPNTrianglesfATI',
'PFNGLPNTRIANGLESIATIPROC', 'PFNGLPNTRIANGLESFATIPROC',
'GL_ATI_vertex_array_object', 'glNewObjectBufferATI', 'glIsObjectBufferATI',
'glUpdateObjectBufferATI', 'glGetObjectBufferfvATI', 'glGetObjectBufferivATI',
'glFreeObjectBufferATI', 'glArrayObjectATI', 'glGetArrayObjectfvATI',
'glGetArrayObjectivATI', 'glVariantArrayObjectATI',
'glGetVariantArrayObjectfvATI', 'glGetVariantArrayObjectivATI',
'PFNGLNEWOBJECTBUFFERATIPROC', 'PFNGLISOBJECTBUFFERATIPROC',
'PFNGLUPDATEOBJECTBUFFERATIPROC', 'PFNGLGETOBJECTBUFFERFVATIPROC',
'PFNGLGETOBJECTBUFFERIVATIPROC', 'PFNGLFREEOBJECTBUFFERATIPROC',
'PFNGLARRAYOBJECTATIPROC', 'PFNGLGETARRAYOBJECTFVATIPROC',
'PFNGLGETARRAYOBJECTIVATIPROC', 'PFNGLVARIANTARRAYOBJECTATIPROC',
'PFNGLGETVARIANTARRAYOBJECTFVATIPROC', 'PFNGLGETVARIANTARRAYOBJECTIVATIPROC',
'GL_EXT_vertex_shader', 'glBeginVertexShaderEXT', 'glEndVertexShaderEXT',
'glBindVertexShaderEXT', 'glGenVertexShadersEXT', 'glDeleteVertexShaderEXT',
'glShaderOp1EXT', 'glShaderOp2EXT', 'glShaderOp3EXT', 'glSwizzleEXT',
'glWriteMaskEXT', 'glInsertComponentEXT', 'glExtractComponentEXT',
'glGenSymbolsEXT', 'glSetInvariantEXT', 'glSetLocalConstantEXT',
'glVariantbvEXT', 'glVariantsvEXT', 'glVariantivEXT', 'glVariantfvEXT',
'glVariantdvEXT', 'glVariantubvEXT', 'glVariantusvEXT', 'glVariantuivEXT',
'glVariantPointerEXT', 'glEnableVariantClientStateEXT',
'glDisableVariantClientStateEXT', 'glBindLightParameterEXT',
'glBindMaterialParameterEXT', 'glBindTexGenParameterEXT',
'glBindTextureUnitParameterEXT', 'glBindParameterEXT',
'glIsVariantEnabledEXT', 'glGetVariantBooleanvEXT', 'glGetVariantIntegervEXT',
'glGetVariantFloatvEXT', 'glGetVariantPointervEXT',
'glGetInvariantBooleanvEXT', 'glGetInvariantIntegervEXT',
'glGetInvariantFloatvEXT', 'glGetLocalConstantBooleanvEXT',
'glGetLocalConstantIntegervEXT', 'glGetLocalConstantFloatvEXT',
'PFNGLBEGINVERTEXSHADEREXTPROC', 'PFNGLENDVERTEXSHADEREXTPROC',
'PFNGLBINDVERTEXSHADEREXTPROC', 'PFNGLGENVERTEXSHADERSEXTPROC',
'PFNGLDELETEVERTEXSHADEREXTPROC', 'PFNGLSHADEROP1EXTPROC',
'PFNGLSHADEROP2EXTPROC', 'PFNGLSHADEROP3EXTPROC', 'PFNGLSWIZZLEEXTPROC',
'PFNGLWRITEMASKEXTPROC', 'PFNGLINSERTCOMPONENTEXTPROC',
'PFNGLEXTRACTCOMPONENTEXTPROC', 'PFNGLGENSYMBOLSEXTPROC',
'PFNGLSETINVARIANTEXTPROC', 'PFNGLSETLOCALCONSTANTEXTPROC',
'PFNGLVARIANTBVEXTPROC', 'PFNGLVARIANTSVEXTPROC', 'PFNGLVARIANTIVEXTPROC',
'PFNGLVARIANTFVEXTPROC', 'PFNGLVARIANTDVEXTPROC', 'PFNGLVARIANTUBVEXTPROC',
'PFNGLVARIANTUSVEXTPROC', 'PFNGLVARIANTUIVEXTPROC',
'PFNGLVARIANTPOINTEREXTPROC', 'PFNGLENABLEVARIANTCLIENTSTATEEXTPROC',
'PFNGLDISABLEVARIANTCLIENTSTATEEXTPROC', 'PFNGLBINDLIGHTPARAMETEREXTPROC',
'PFNGLBINDMATERIALPARAMETEREXTPROC', 'PFNGLBINDTEXGENPARAMETEREXTPROC',
'PFNGLBINDTEXTUREUNITPARAMETEREXTPROC', 'PFNGLBINDPARAMETEREXTPROC',
'PFNGLISVARIANTENABLEDEXTPROC', 'PFNGLGETVARIANTBOOLEANVEXTPROC',
'PFNGLGETVARIANTINTEGERVEXTPROC', 'PFNGLGETVARIANTFLOATVEXTPROC',
'PFNGLGETVARIANTPOINTERVEXTPROC', 'PFNGLGETINVARIANTBOOLEANVEXTPROC',
'PFNGLGETINVARIANTINTEGERVEXTPROC', 'PFNGLGETINVARIANTFLOATVEXTPROC',
'PFNGLGETLOCALCONSTANTBOOLEANVEXTPROC',
'PFNGLGETLOCALCONSTANTINTEGERVEXTPROC', 'PFNGLGETLOCALCONSTANTFLOATVEXTPROC',
'GL_ATI_vertex_streams', 'glVertexStream1sATI', 'glVertexStream1svATI',
'glVertexStream1iATI', 'glVertexStream1ivATI', 'glVertexStream1fATI',
'glVertexStream1fvATI', 'glVertexStream1dATI', 'glVertexStream1dvATI',
'glVertexStream2sATI', 'glVertexStream2svATI', 'glVertexStream2iATI',
'glVertexStream2ivATI', 'glVertexStream2fATI', 'glVertexStream2fvATI',
'glVertexStream2dATI', 'glVertexStream2dvATI', 'glVertexStream3sATI',
'glVertexStream3svATI', 'glVertexStream3iATI', 'glVertexStream3ivATI',
'glVertexStream3fATI', 'glVertexStream3fvATI', 'glVertexStream3dATI',
'glVertexStream3dvATI', 'glVertexStream4sATI', 'glVertexStream4svATI',
'glVertexStream4iATI', 'glVertexStream4ivATI', 'glVertexStream4fATI',
'glVertexStream4fvATI', 'glVertexStream4dATI', 'glVertexStream4dvATI',
'glNormalStream3bATI', 'glNormalStream3bvATI', 'glNormalStream3sATI',
'glNormalStream3svATI', 'glNormalStream3iATI', 'glNormalStream3ivATI',
'glNormalStream3fATI', 'glNormalStream3fvATI', 'glNormalStream3dATI',
'glNormalStream3dvATI', 'glClientActiveVertexStreamATI',
'glVertexBlendEnviATI', 'glVertexBlendEnvfATI', 'PFNGLVERTEXSTREAM1SATIPROC',
'PFNGLVERTEXSTREAM1SVATIPROC', 'PFNGLVERTEXSTREAM1IATIPROC',
'PFNGLVERTEXSTREAM1IVATIPROC', 'PFNGLVERTEXSTREAM1FATIPROC',
'PFNGLVERTEXSTREAM1FVATIPROC', 'PFNGLVERTEXSTREAM1DATIPROC',
'PFNGLVERTEXSTREAM1DVATIPROC', 'PFNGLVERTEXSTREAM2SATIPROC',
'PFNGLVERTEXSTREAM2SVATIPROC', 'PFNGLVERTEXSTREAM2IATIPROC',
'PFNGLVERTEXSTREAM2IVATIPROC', 'PFNGLVERTEXSTREAM2FATIPROC',
'PFNGLVERTEXSTREAM2FVATIPROC', 'PFNGLVERTEXSTREAM2DATIPROC',
'PFNGLVERTEXSTREAM2DVATIPROC', 'PFNGLVERTEXSTREAM3SATIPROC',
'PFNGLVERTEXSTREAM3SVATIPROC', 'PFNGLVERTEXSTREAM3IATIPROC',
'PFNGLVERTEXSTREAM3IVATIPROC', 'PFNGLVERTEXSTREAM3FATIPROC',
'PFNGLVERTEXSTREAM3FVATIPROC', 'PFNGLVERTEXSTREAM3DATIPROC',
'PFNGLVERTEXSTREAM3DVATIPROC', 'PFNGLVERTEXSTREAM4SATIPROC',
'PFNGLVERTEXSTREAM4SVATIPROC', 'PFNGLVERTEXSTREAM4IATIPROC',
'PFNGLVERTEXSTREAM4IVATIPROC', 'PFNGLVERTEXSTREAM4FATIPROC',
'PFNGLVERTEXSTREAM4FVATIPROC', 'PFNGLVERTEXSTREAM4DATIPROC',
'PFNGLVERTEXSTREAM4DVATIPROC', 'PFNGLNORMALSTREAM3BATIPROC',
'PFNGLNORMALSTREAM3BVATIPROC', 'PFNGLNORMALSTREAM3SATIPROC',
'PFNGLNORMALSTREAM3SVATIPROC', 'PFNGLNORMALSTREAM3IATIPROC',
'PFNGLNORMALSTREAM3IVATIPROC', 'PFNGLNORMALSTREAM3FATIPROC',
'PFNGLNORMALSTREAM3FVATIPROC', 'PFNGLNORMALSTREAM3DATIPROC',
'PFNGLNORMALSTREAM3DVATIPROC', 'PFNGLCLIENTACTIVEVERTEXSTREAMATIPROC',
'PFNGLVERTEXBLENDENVIATIPROC', 'PFNGLVERTEXBLENDENVFATIPROC',
'GL_ATI_element_array', 'glElementPointerATI', 'glDrawElementArrayATI',
'glDrawRangeElementArrayATI', 'PFNGLELEMENTPOINTERATIPROC',
'PFNGLDRAWELEMENTARRAYATIPROC', 'PFNGLDRAWRANGEELEMENTARRAYATIPROC',
'GL_SUN_mesh_array', 'glDrawMeshArraysSUN', 'PFNGLDRAWMESHARRAYSSUNPROC',
'GL_SUN_slice_accum', 'GL_NV_multisample_filter_hint', 'GL_NV_depth_clamp',
'GL_NV_occlusion_query', 'glGenOcclusionQueriesNV',
'glDeleteOcclusionQueriesNV', 'glIsOcclusionQueryNV',
'glBeginOcclusionQueryNV', 'glEndOcclusionQueryNV', 'glGetOcclusionQueryivNV',
'glGetOcclusionQueryuivNV', 'PFNGLGENOCCLUSIONQUERIESNVPROC',
'PFNGLDELETEOCCLUSIONQUERIESNVPROC', 'PFNGLISOCCLUSIONQUERYNVPROC',
'PFNGLBEGINOCCLUSIONQUERYNVPROC', 'PFNGLENDOCCLUSIONQUERYNVPROC',
'PFNGLGETOCCLUSIONQUERYIVNVPROC', 'PFNGLGETOCCLUSIONQUERYUIVNVPROC',
'GL_NV_point_sprite', 'glPointParameteriNV', 'glPointParameterivNV',
'PFNGLPOINTPARAMETERINVPROC', 'PFNGLPOINTPARAMETERIVNVPROC',
'GL_NV_texture_shader3', 'GL_NV_vertex_program1_1', 'GL_EXT_shadow_funcs',
'GL_EXT_stencil_two_side', 'glActiveStencilFaceEXT',
'PFNGLACTIVESTENCILFACEEXTPROC', 'GL_ATI_text_fragment_shader',
'GL_APPLE_client_storage', 'GL_APPLE_element_array', 'glElementPointerAPPLE',
'glDrawElementArrayAPPLE', 'glDrawRangeElementArrayAPPLE',
'glMultiDrawElementArrayAPPLE', 'glMultiDrawRangeElementArrayAPPLE',
'PFNGLELEMENTPOINTERAPPLEPROC', 'PFNGLDRAWELEMENTARRAYAPPLEPROC',
'PFNGLDRAWRANGEELEMENTARRAYAPPLEPROC', 'PFNGLMULTIDRAWELEMENTARRAYAPPLEPROC',
'PFNGLMULTIDRAWRANGEELEMENTARRAYAPPLEPROC', 'GL_APPLE_fence',
'glGenFencesAPPLE', 'glDeleteFencesAPPLE', 'glSetFenceAPPLE',
'glIsFenceAPPLE', 'glTestFenceAPPLE', 'glFinishFenceAPPLE',
'glTestObjectAPPLE', 'glFinishObjectAPPLE', 'PFNGLGENFENCESAPPLEPROC',
'PFNGLDELETEFENCESAPPLEPROC', 'PFNGLSETFENCEAPPLEPROC',
'PFNGLISFENCEAPPLEPROC', 'PFNGLTESTFENCEAPPLEPROC',
'PFNGLFINISHFENCEAPPLEPROC', 'PFNGLTESTOBJECTAPPLEPROC',
'PFNGLFINISHOBJECTAPPLEPROC', 'GL_APPLE_vertex_array_object',
'glBindVertexArrayAPPLE', 'glDeleteVertexArraysAPPLE',
'glGenVertexArraysAPPLE', 'glIsVertexArrayAPPLE',
'PFNGLBINDVERTEXARRAYAPPLEPROC', 'PFNGLDELETEVERTEXARRAYSAPPLEPROC',
'PFNGLGENVERTEXARRAYSAPPLEPROC', 'PFNGLISVERTEXARRAYAPPLEPROC',
'GL_APPLE_vertex_array_range', 'glVertexArrayRangeAPPLE',
'glFlushVertexArrayRangeAPPLE', 'glVertexArrayParameteriAPPLE',
'PFNGLVERTEXARRAYRANGEAPPLEPROC', 'PFNGLFLUSHVERTEXARRAYRANGEAPPLEPROC',
'PFNGLVERTEXARRAYPARAMETERIAPPLEPROC', 'GL_APPLE_ycbcr_422', 'GL_S3_s3tc',
'GL_ATI_draw_buffers', 'glDrawBuffersATI', 'PFNGLDRAWBUFFERSATIPROC',
'GL_ATI_pixel_format_float', 'GL_ATI_texture_env_combine3',
'GL_ATI_texture_float', 'GL_NV_float_buffer', 'GL_NV_fragment_program',
'glProgramNamedParameter4fNV', 'glProgramNamedParameter4dNV',
'glProgramNamedParameter4fvNV', 'glProgramNamedParameter4dvNV',
'glGetProgramNamedParameterfvNV', 'glGetProgramNamedParameterdvNV',
'PFNGLPROGRAMNAMEDPARAMETER4FNVPROC', 'PFNGLPROGRAMNAMEDPARAMETER4DNVPROC',
'PFNGLPROGRAMNAMEDPARAMETER4FVNVPROC', 'PFNGLPROGRAMNAMEDPARAMETER4DVNVPROC',
'PFNGLGETPROGRAMNAMEDPARAMETERFVNVPROC',
'PFNGLGETPROGRAMNAMEDPARAMETERDVNVPROC', 'GL_NV_half_float', 'glVertex2hNV',
'glVertex2hvNV', 'glVertex3hNV', 'glVertex3hvNV', 'glVertex4hNV',
'glVertex4hvNV', 'glNormal3hNV', 'glNormal3hvNV', 'glColor3hNV',
'glColor3hvNV', 'glColor4hNV', 'glColor4hvNV', 'glTexCoord1hNV',
'glTexCoord1hvNV', 'glTexCoord2hNV', 'glTexCoord2hvNV', 'glTexCoord3hNV',
'glTexCoord3hvNV', 'glTexCoord4hNV', 'glTexCoord4hvNV', 'glMultiTexCoord1hNV',
'glMultiTexCoord1hvNV', 'glMultiTexCoord2hNV', 'glMultiTexCoord2hvNV',
'glMultiTexCoord3hNV', 'glMultiTexCoord3hvNV', 'glMultiTexCoord4hNV',
'glMultiTexCoord4hvNV', 'glFogCoordhNV', 'glFogCoordhvNV',
'glSecondaryColor3hNV', 'glSecondaryColor3hvNV', 'glVertexWeighthNV',
'glVertexWeighthvNV', 'glVertexAttrib1hNV', 'glVertexAttrib1hvNV',
'glVertexAttrib2hNV', 'glVertexAttrib2hvNV', 'glVertexAttrib3hNV',
'glVertexAttrib3hvNV', 'glVertexAttrib4hNV', 'glVertexAttrib4hvNV',
'glVertexAttribs1hvNV', 'glVertexAttribs2hvNV', 'glVertexAttribs3hvNV',
'glVertexAttribs4hvNV', 'PFNGLVERTEX2HNVPROC', 'PFNGLVERTEX2HVNVPROC',
'PFNGLVERTEX3HNVPROC', 'PFNGLVERTEX3HVNVPROC', 'PFNGLVERTEX4HNVPROC',
'PFNGLVERTEX4HVNVPROC', 'PFNGLNORMAL3HNVPROC', 'PFNGLNORMAL3HVNVPROC',
'PFNGLCOLOR3HNVPROC', 'PFNGLCOLOR3HVNVPROC', 'PFNGLCOLOR4HNVPROC',
'PFNGLCOLOR4HVNVPROC', 'PFNGLTEXCOORD1HNVPROC', 'PFNGLTEXCOORD1HVNVPROC',
'PFNGLTEXCOORD2HNVPROC', 'PFNGLTEXCOORD2HVNVPROC', 'PFNGLTEXCOORD3HNVPROC',
'PFNGLTEXCOORD3HVNVPROC', 'PFNGLTEXCOORD4HNVPROC', 'PFNGLTEXCOORD4HVNVPROC',
'PFNGLMULTITEXCOORD1HNVPROC', 'PFNGLMULTITEXCOORD1HVNVPROC',
'PFNGLMULTITEXCOORD2HNVPROC', 'PFNGLMULTITEXCOORD2HVNVPROC',
'PFNGLMULTITEXCOORD3HNVPROC', 'PFNGLMULTITEXCOORD3HVNVPROC',
'PFNGLMULTITEXCOORD4HNVPROC', 'PFNGLMULTITEXCOORD4HVNVPROC',
'PFNGLFOGCOORDHNVPROC', 'PFNGLFOGCOORDHVNVPROC',
'PFNGLSECONDARYCOLOR3HNVPROC', 'PFNGLSECONDARYCOLOR3HVNVPROC',
'PFNGLVERTEXWEIGHTHNVPROC', 'PFNGLVERTEXWEIGHTHVNVPROC',
'PFNGLVERTEXATTRIB1HNVPROC', 'PFNGLVERTEXATTRIB1HVNVPROC',
'PFNGLVERTEXATTRIB2HNVPROC', 'PFNGLVERTEXATTRIB2HVNVPROC',
'PFNGLVERTEXATTRIB3HNVPROC', 'PFNGLVERTEXATTRIB3HVNVPROC',
'PFNGLVERTEXATTRIB4HNVPROC', 'PFNGLVERTEXATTRIB4HVNVPROC',
'PFNGLVERTEXATTRIBS1HVNVPROC', 'PFNGLVERTEXATTRIBS2HVNVPROC',
'PFNGLVERTEXATTRIBS3HVNVPROC', 'PFNGLVERTEXATTRIBS4HVNVPROC',
'GL_NV_pixel_data_range', 'glPixelDataRangeNV', 'glFlushPixelDataRangeNV',
'PFNGLPIXELDATARANGENVPROC', 'PFNGLFLUSHPIXELDATARANGENVPROC',
'GL_NV_primitive_restart', 'glPrimitiveRestartNV',
'glPrimitiveRestartIndexNV', 'PFNGLPRIMITIVERESTARTNVPROC',
'PFNGLPRIMITIVERESTARTINDEXNVPROC', 'GL_NV_texture_expand_normal',
'GL_NV_vertex_program2', 'GL_ATI_map_object_buffer', 'glMapObjectBufferATI',
'glUnmapObjectBufferATI', 'PFNGLMAPOBJECTBUFFERATIPROC',
'PFNGLUNMAPOBJECTBUFFERATIPROC', 'GL_ATI_separate_stencil',
'glStencilOpSeparateATI', 'glStencilFuncSeparateATI',
'PFNGLSTENCILOPSEPARATEATIPROC', 'PFNGLSTENCILFUNCSEPARATEATIPROC',
'GL_ATI_vertex_attrib_array_object', 'glVertexAttribArrayObjectATI',
'glGetVertexAttribArrayObjectfvATI', 'glGetVertexAttribArrayObjectivATI',
'PFNGLVERTEXATTRIBARRAYOBJECTATIPROC',
'PFNGLGETVERTEXATTRIBARRAYOBJECTFVATIPROC',
'PFNGLGETVERTEXATTRIBARRAYOBJECTIVATIPROC', 'GL_OES_read_format',
'GL_EXT_depth_bounds_test', 'glDepthBoundsEXT', 'PFNGLDEPTHBOUNDSEXTPROC',
'GL_EXT_texture_mirror_clamp', 'GL_EXT_blend_equation_separate',
'glBlendEquationSeparateEXT', 'PFNGLBLENDEQUATIONSEPARATEEXTPROC',
'GL_MESA_pack_invert', 'GL_MESA_ycbcr_texture', 'GL_EXT_pixel_buffer_object',
'GL_NV_fragment_program_option', 'GL_NV_fragment_program2',
'GL_NV_vertex_program2_option', 'GL_NV_vertex_program3',
'GL_EXT_framebuffer_object', 'glIsRenderbufferEXT', 'glBindRenderbufferEXT',
'glDeleteRenderbuffersEXT', 'glGenRenderbuffersEXT',
'glRenderbufferStorageEXT', 'glGetRenderbufferParameterivEXT',
'glIsFramebufferEXT', 'glBindFramebufferEXT', 'glDeleteFramebuffersEXT',
'glGenFramebuffersEXT', 'glCheckFramebufferStatusEXT',
'glFramebufferTexture1DEXT', 'glFramebufferTexture2DEXT',
'glFramebufferTexture3DEXT', 'glFramebufferRenderbufferEXT',
'glGetFramebufferAttachmentParameterivEXT', 'glGenerateMipmapEXT',
'PFNGLISRENDERBUFFEREXTPROC', 'PFNGLBINDRENDERBUFFEREXTPROC',
'PFNGLDELETERENDERBUFFERSEXTPROC', 'PFNGLGENRENDERBUFFERSEXTPROC',
'PFNGLRENDERBUFFERSTORAGEEXTPROC', 'PFNGLGETRENDERBUFFERPARAMETERIVEXTPROC',
'PFNGLISFRAMEBUFFEREXTPROC', 'PFNGLBINDFRAMEBUFFEREXTPROC',
'PFNGLDELETEFRAMEBUFFERSEXTPROC', 'PFNGLGENFRAMEBUFFERSEXTPROC',
'PFNGLCHECKFRAMEBUFFERSTATUSEXTPROC', 'PFNGLFRAMEBUFFERTEXTURE1DEXTPROC',
'PFNGLFRAMEBUFFERTEXTURE2DEXTPROC', 'PFNGLFRAMEBUFFERTEXTURE3DEXTPROC',
'PFNGLFRAMEBUFFERRENDERBUFFEREXTPROC',
'PFNGLGETFRAMEBUFFERATTACHMENTPARAMETERIVEXTPROC',
'PFNGLGENERATEMIPMAPEXTPROC', 'GL_GREMEDY_string_marker',
'glStringMarkerGREMEDY', 'PFNGLSTRINGMARKERGREMEDYPROC', 'GL_EXT_Cg_shader',
'GL_EXT_timer_query', 'glGetQueryObjecti64vEXT', 'glGetQueryObjectui64vEXT',
'PFNGLGETQUERYOBJECTI64VEXTPROC', 'PFNGLGETQUERYOBJECTUI64VEXTPROC',
'GL_EXT_texture_buffer_object', 'glTexBufferEXT', 'PFNGLTEXBUFFEREXTPROC',
'GL_NV_transform_feedback', 'glBeginTransformFeedbackNV',
'glEndTransformFeedbackNV', 'glTransformFeedbackAttribsNV',
'glBindBufferRangeNV', 'glBindBufferOffsetNV', 'glBindBufferBaseNV',
'glTransformFeedbackVaryingsNV', 'glActiveVaryingNV',
'glGetVaryingLocationNV', 'glGetActiveVaryingNV',
'glGetTransformFeedbackVaryingNV', 'PFNGLBEGINTRANSFORMFEEDBACKNVPROC',
'PFNGLENDTRANSFORMFEEDBACKNVPROC', 'PFNGLTRANSFORMFEEDBACKATTRIBSNVPROC',
'PFNGLBINDBUFFERRANGENVPROC', 'PFNGLBINDBUFFEROFFSETNVPROC',
'PFNGLBINDBUFFERBASENVPROC', 'PFNGLTRANSFORMFEEDBACKVARYINGSNVPROC',
'PFNGLACTIVEVARYINGNVPROC', 'PFNGLGETVARYINGLOCATIONNVPROC',
'PFNGLGETACTIVEVARYINGNVPROC', 'PFNGLGETTRANSFORMFEEDBACKVARYINGNVPROC',
'GL_NV_depth_buffer_float', 'glDepthRangedNV', 'glClearDepthdNV',
'glDepthBoundsdNV', 'PFNGLDEPTHRANGEDNVPROC', 'PFNGLCLEARDEPTHDNVPROC',
'PFNGLDEPTHBOUNDSDNVPROC', 'GL_EXT_texture_compression_latc',
'GL_EXT_framebuffer_sRGB', 'GL_EXT_texture_shared_exponent',
'GL_EXT_packed_float', 'GL_EXT_texture_array', 'GL_EXT_draw_buffers2',
'glColorMaskIndexedEXT', 'glGetBooleanIndexedvEXT', 'glGetIntegerIndexedvEXT',
'glEnableIndexedEXT', 'glDisableIndexedEXT', 'glIsEnabledIndexedEXT',
'PFNGLCOLORMASKINDEXEDEXTPROC', 'PFNGLGETBOOLEANINDEXEDVEXTPROC',
'PFNGLGETINTEGERINDEXEDVEXTPROC', 'PFNGLENABLEINDEXEDEXTPROC',
'PFNGLDISABLEINDEXEDEXTPROC', 'PFNGLISENABLEDINDEXEDEXTPROC',
'GL_EXT_texture_integer', 'glTexParameterIivEXT', 'glTexParameterIuivEXT',
'glGetTexParameterIivEXT', 'glGetTexParameterIuivEXT', 'glClearColorIiEXT',
'glClearColorIuiEXT', 'PFNGLTEXPARAMETERIIVEXTPROC',
'PFNGLTEXPARAMETERIUIVEXTPROC', 'PFNGLGETTEXPARAMETERIIVEXTPROC',
'PFNGLGETTEXPARAMETERIUIVEXTPROC', 'PFNGLCLEARCOLORIIEXTPROC',
'PFNGLCLEARCOLORIUIEXTPROC', 'GL_EXT_bindable_uniform', 'glUniformBufferEXT',
'glGetUniformBufferSizeEXT', 'glGetUniformOffsetEXT',
'PFNGLUNIFORMBUFFEREXTPROC', 'PFNGLGETUNIFORMBUFFERSIZEEXTPROC',
'PFNGLGETUNIFORMOFFSETEXTPROC', 'GL_EXT_gpu_shader4', 'glGetUniformuivEXT',
'glBindFragDataLocationEXT', 'glGetFragDataLocationEXT', 'glUniform1uiEXT',
'glUniform2uiEXT', 'glUniform3uiEXT', 'glUniform4uiEXT', 'glUniform1uivEXT',
'glUniform2uivEXT', 'glUniform3uivEXT', 'glUniform4uivEXT',
'glVertexAttribI1iEXT', 'glVertexAttribI2iEXT', 'glVertexAttribI3iEXT',
'glVertexAttribI4iEXT', 'glVertexAttribI1uiEXT', 'glVertexAttribI2uiEXT',
'glVertexAttribI3uiEXT', 'glVertexAttribI4uiEXT', 'glVertexAttribI1ivEXT',
'glVertexAttribI2ivEXT', 'glVertexAttribI3ivEXT', 'glVertexAttribI4ivEXT',
'glVertexAttribI1uivEXT', 'glVertexAttribI2uivEXT', 'glVertexAttribI3uivEXT',
'glVertexAttribI4uivEXT', 'glVertexAttribI4bvEXT', 'glVertexAttribI4svEXT',
'glVertexAttribI4ubvEXT', 'glVertexAttribI4usvEXT',
'glVertexAttribIPointerEXT', 'glGetVertexAttribIivEXT',
'glGetVertexAttribIuivEXT', 'PFNGLGETUNIFORMUIVEXTPROC',
'PFNGLBINDFRAGDATALOCATIONEXTPROC', 'PFNGLGETFRAGDATALOCATIONEXTPROC',
'PFNGLUNIFORM1UIEXTPROC', 'PFNGLUNIFORM2UIEXTPROC', 'PFNGLUNIFORM3UIEXTPROC',
'PFNGLUNIFORM4UIEXTPROC', 'PFNGLUNIFORM1UIVEXTPROC',
'PFNGLUNIFORM2UIVEXTPROC', 'PFNGLUNIFORM3UIVEXTPROC',
'PFNGLUNIFORM4UIVEXTPROC', 'PFNGLVERTEXATTRIBI1IEXTPROC',
'PFNGLVERTEXATTRIBI2IEXTPROC', 'PFNGLVERTEXATTRIBI3IEXTPROC',
'PFNGLVERTEXATTRIBI4IEXTPROC', 'PFNGLVERTEXATTRIBI1UIEXTPROC',
'PFNGLVERTEXATTRIBI2UIEXTPROC', 'PFNGLVERTEXATTRIBI3UIEXTPROC',
'PFNGLVERTEXATTRIBI4UIEXTPROC', 'PFNGLVERTEXATTRIBI1IVEXTPROC',
'PFNGLVERTEXATTRIBI2IVEXTPROC', 'PFNGLVERTEXATTRIBI3IVEXTPROC',
'PFNGLVERTEXATTRIBI4IVEXTPROC', 'PFNGLVERTEXATTRIBI1UIVEXTPROC',
'PFNGLVERTEXATTRIBI2UIVEXTPROC', 'PFNGLVERTEXATTRIBI3UIVEXTPROC',
'PFNGLVERTEXATTRIBI4UIVEXTPROC', 'PFNGLVERTEXATTRIBI4BVEXTPROC',
'PFNGLVERTEXATTRIBI4SVEXTPROC', 'PFNGLVERTEXATTRIBI4UBVEXTPROC',
'PFNGLVERTEXATTRIBI4USVEXTPROC', 'PFNGLVERTEXATTRIBIPOINTEREXTPROC',
'PFNGLGETVERTEXATTRIBIIVEXTPROC', 'PFNGLGETVERTEXATTRIBIUIVEXTPROC',
'GL_EXT_geometry_shader4', 'glProgramParameteriEXT',
'glFramebufferTextureEXT', 'glFramebufferTextureLayerEXT',
'glFramebufferTextureFaceEXT', 'PFNGLPROGRAMPARAMETERIEXTPROC',
'PFNGLFRAMEBUFFERTEXTUREEXTPROC', 'PFNGLFRAMEBUFFERTEXTURELAYEREXTPROC',
'PFNGLFRAMEBUFFERTEXTUREFACEEXTPROC', 'GL_NV_geometry_program4',
'glProgramVertexLimitNV', 'PFNGLPROGRAMVERTEXLIMITNVPROC',
'GL_NV_gpu_program4', 'glProgramLocalParameterI4iNV',
'glProgramLocalParameterI4ivNV', 'glProgramLocalParametersI4ivNV',
'glProgramLocalParameterI4uiNV', 'glProgramLocalParameterI4uivNV',
'glProgramLocalParametersI4uivNV', 'glProgramEnvParameterI4iNV',
'glProgramEnvParameterI4ivNV', 'glProgramEnvParametersI4ivNV',
'glProgramEnvParameterI4uiNV', 'glProgramEnvParameterI4uivNV',
'glProgramEnvParametersI4uivNV', 'glGetProgramLocalParameterIivNV',
'glGetProgramLocalParameterIuivNV', 'glGetProgramEnvParameterIivNV',
'glGetProgramEnvParameterIuivNV', 'PFNGLPROGRAMLOCALPARAMETERI4INVPROC',
'PFNGLPROGRAMLOCALPARAMETERI4IVNVPROC',
'PFNGLPROGRAMLOCALPARAMETERSI4IVNVPROC',
'PFNGLPROGRAMLOCALPARAMETERI4UINVPROC',
'PFNGLPROGRAMLOCALPARAMETERI4UIVNVPROC',
'PFNGLPROGRAMLOCALPARAMETERSI4UIVNVPROC', 'PFNGLPROGRAMENVPARAMETERI4INVPROC',
'PFNGLPROGRAMENVPARAMETERI4IVNVPROC', 'PFNGLPROGRAMENVPARAMETERSI4IVNVPROC',
'PFNGLPROGRAMENVPARAMETERI4UINVPROC', 'PFNGLPROGRAMENVPARAMETERI4UIVNVPROC',
'PFNGLPROGRAMENVPARAMETERSI4UIVNVPROC',
'PFNGLGETPROGRAMLOCALPARAMETERIIVNVPROC',
'PFNGLGETPROGRAMLOCALPARAMETERIUIVNVPROC',
'PFNGLGETPROGRAMENVPARAMETERIIVNVPROC',
'PFNGLGETPROGRAMENVPARAMETERIUIVNVPROC', 'GL_NV_parameter_buffer_object',
'glProgramBufferParametersfvNV', 'glProgramBufferParametersIivNV',
'glProgramBufferParametersIuivNV', 'PFNGLPROGRAMBUFFERPARAMETERSFVNVPROC',
'PFNGLPROGRAMBUFFERPARAMETERSIIVNVPROC',
'PFNGLPROGRAMBUFFERPARAMETERSIUIVNVPROC', 'GL_EXT_framebuffer_multisample',
'glRenderbufferStorageMultisampleEXT',
'PFNGLRENDERBUFFERSTORAGEMULTISAMPLEEXTPROC',
'GL_NV_framebuffer_multisample_coverage',
'glRenderbufferStorageMultisampleCoverageNV',
'PFNGLRENDERBUFFERSTORAGEMULTISAMPLECOVERAGENVPROC',
'GL_EXT_framebuffer_blit', 'glBlitFramebufferEXT',
'PFNGLBLITFRAMEBUFFEREXTPROC', 'GL_EXT_draw_instanced',
'glDrawArraysInstancedEXT', 'glDrawElementsInstancedEXT',
'PFNGLDRAWARRAYSINSTANCEDEXTPROC', 'PFNGLDRAWELEMENTSINSTANCEDEXTPROC',
'GL_EXT_texture_compression_rgtc', 'GL_NV_present_video',
'glPresentFrameKeyedNV', 'glPresentFrameDualFillNV', 'glGetVideoivNV',
'glGetVideouivNV', 'glGetVideoi64vNV', 'glGetVideoui64vNV',
'PFNGLPRESENTFRAMEKEYEDNVPROC', 'PFNGLPRESENTFRAMEDUALFILLNVPROC',
'PFNGLGETVIDEOIVNVPROC', 'PFNGLGETVIDEOUIVNVPROC', 'PFNGLGETVIDEOI64VNVPROC',
'PFNGLGETVIDEOUI64VNVPROC', 'GL_NV_conditional_render',
'glBeginConditionalRenderNV', 'glEndConditionalRenderNV',
'PFNGLBEGINCONDITIONALRENDERNVPROC', 'PFNGLENDCONDITIONALRENDERNVPROC',
'GL_EXT_transform_feedback', 'glBeginTransformFeedbackEXT',
'glEndTransformFeedbackEXT', 'glBindBufferRangeEXT', 'glBindBufferOffsetEXT',
'glBindBufferBaseEXT', 'glTransformFeedbackVaryingsEXT',
'glGetTransformFeedbackVaryingEXT', 'PFNGLBEGINTRANSFORMFEEDBACKEXTPROC',
'PFNGLENDTRANSFORMFEEDBACKEXTPROC', 'PFNGLBINDBUFFERRANGEEXTPROC',
'PFNGLBINDBUFFEROFFSETEXTPROC', 'PFNGLBINDBUFFERBASEEXTPROC',
'PFNGLTRANSFORMFEEDBACKVARYINGSEXTPROC',
'PFNGLGETTRANSFORMFEEDBACKVARYINGEXTPROC', 'GL_EXT_direct_state_access',
'glClientAttribDefaultEXT', 'glPushClientAttribDefaultEXT',
'glMatrixLoadfEXT', 'glMatrixLoaddEXT', 'glMatrixMultfEXT',
'glMatrixMultdEXT', 'glMatrixLoadIdentityEXT', 'glMatrixRotatefEXT',
'glMatrixRotatedEXT', 'glMatrixScalefEXT', 'glMatrixScaledEXT',
'glMatrixTranslatefEXT', 'glMatrixTranslatedEXT', 'glMatrixFrustumEXT',
'glMatrixOrthoEXT', 'glMatrixPopEXT', 'glMatrixPushEXT',
'glMatrixLoadTransposefEXT', 'glMatrixLoadTransposedEXT',
'glMatrixMultTransposefEXT', 'glMatrixMultTransposedEXT',
'glTextureParameterfEXT', 'glTextureParameterfvEXT', 'glTextureParameteriEXT',
'glTextureParameterivEXT', 'glTextureImage1DEXT', 'glTextureImage2DEXT',
'glTextureSubImage1DEXT', 'glTextureSubImage2DEXT', 'glCopyTextureImage1DEXT',
'glCopyTextureImage2DEXT', 'glCopyTextureSubImage1DEXT',
'glCopyTextureSubImage2DEXT', 'glGetTextureImageEXT',
'glGetTextureParameterfvEXT', 'glGetTextureParameterivEXT',
'glGetTextureLevelParameterfvEXT', 'glGetTextureLevelParameterivEXT',
'glTextureImage3DEXT', 'glTextureSubImage3DEXT', 'glCopyTextureSubImage3DEXT',
'glMultiTexParameterfEXT', 'glMultiTexParameterfvEXT',
'glMultiTexParameteriEXT', 'glMultiTexParameterivEXT', 'glMultiTexImage1DEXT',
'glMultiTexImage2DEXT', 'glMultiTexSubImage1DEXT', 'glMultiTexSubImage2DEXT',
'glCopyMultiTexImage1DEXT', 'glCopyMultiTexImage2DEXT',
'glCopyMultiTexSubImage1DEXT', 'glCopyMultiTexSubImage2DEXT',
'glGetMultiTexImageEXT', 'glGetMultiTexParameterfvEXT',
'glGetMultiTexParameterivEXT', 'glGetMultiTexLevelParameterfvEXT',
'glGetMultiTexLevelParameterivEXT', 'glMultiTexImage3DEXT',
'glMultiTexSubImage3DEXT', 'glCopyMultiTexSubImage3DEXT',
'glBindMultiTextureEXT', 'glEnableClientStateIndexedEXT',
'glDisableClientStateIndexedEXT', 'glMultiTexCoordPointerEXT',
'glMultiTexEnvfEXT', 'glMultiTexEnvfvEXT', 'glMultiTexEnviEXT',
'glMultiTexEnvivEXT', 'glMultiTexGendEXT', 'glMultiTexGendvEXT',
'glMultiTexGenfEXT', 'glMultiTexGenfvEXT', 'glMultiTexGeniEXT',
'glMultiTexGenivEXT', 'glGetMultiTexEnvfvEXT', 'glGetMultiTexEnvivEXT',
'glGetMultiTexGendvEXT', 'glGetMultiTexGenfvEXT', 'glGetMultiTexGenivEXT',
'glGetFloatIndexedvEXT', 'glGetDoubleIndexedvEXT', 'glGetPointerIndexedvEXT',
'glCompressedTextureImage3DEXT', 'glCompressedTextureImage2DEXT',
'glCompressedTextureImage1DEXT', 'glCompressedTextureSubImage3DEXT',
'glCompressedTextureSubImage2DEXT', 'glCompressedTextureSubImage1DEXT',
'glGetCompressedTextureImageEXT', 'glCompressedMultiTexImage3DEXT',
'glCompressedMultiTexImage2DEXT', 'glCompressedMultiTexImage1DEXT',
'glCompressedMultiTexSubImage3DEXT', 'glCompressedMultiTexSubImage2DEXT',
'glCompressedMultiTexSubImage1DEXT', 'glGetCompressedMultiTexImageEXT',
'glNamedProgramStringEXT', 'glNamedProgramLocalParameter4dEXT',
'glNamedProgramLocalParameter4dvEXT', 'glNamedProgramLocalParameter4fEXT',
'glNamedProgramLocalParameter4fvEXT', 'glGetNamedProgramLocalParameterdvEXT',
'glGetNamedProgramLocalParameterfvEXT', 'glGetNamedProgramivEXT',
'glGetNamedProgramStringEXT', 'glNamedProgramLocalParameters4fvEXT',
'glNamedProgramLocalParameterI4iEXT', 'glNamedProgramLocalParameterI4ivEXT',
'glNamedProgramLocalParametersI4ivEXT', 'glNamedProgramLocalParameterI4uiEXT',
'glNamedProgramLocalParameterI4uivEXT',
'glNamedProgramLocalParametersI4uivEXT',
'glGetNamedProgramLocalParameterIivEXT',
'glGetNamedProgramLocalParameterIuivEXT', 'glTextureParameterIivEXT',
'glTextureParameterIuivEXT', 'glGetTextureParameterIivEXT',
'glGetTextureParameterIuivEXT', 'glMultiTexParameterIivEXT',
'glMultiTexParameterIuivEXT', 'glGetMultiTexParameterIivEXT',
'glGetMultiTexParameterIuivEXT', 'glProgramUniform1fEXT',
'glProgramUniform2fEXT', 'glProgramUniform3fEXT', 'glProgramUniform4fEXT',
'glProgramUniform1iEXT', 'glProgramUniform2iEXT', 'glProgramUniform3iEXT',
'glProgramUniform4iEXT', 'glProgramUniform1fvEXT', 'glProgramUniform2fvEXT',
'glProgramUniform3fvEXT', 'glProgramUniform4fvEXT', 'glProgramUniform1ivEXT',
'glProgramUniform2ivEXT', 'glProgramUniform3ivEXT', 'glProgramUniform4ivEXT',
'glProgramUniformMatrix2fvEXT', 'glProgramUniformMatrix3fvEXT',
'glProgramUniformMatrix4fvEXT', 'glProgramUniformMatrix2x3fvEXT',
'glProgramUniformMatrix3x2fvEXT', 'glProgramUniformMatrix2x4fvEXT',
'glProgramUniformMatrix4x2fvEXT', 'glProgramUniformMatrix3x4fvEXT',
'glProgramUniformMatrix4x3fvEXT', 'glProgramUniform1uiEXT',
'glProgramUniform2uiEXT', 'glProgramUniform3uiEXT', 'glProgramUniform4uiEXT',
'glProgramUniform1uivEXT', 'glProgramUniform2uivEXT',
'glProgramUniform3uivEXT', 'glProgramUniform4uivEXT', 'glNamedBufferDataEXT',
'glNamedBufferSubDataEXT', 'glMapNamedBufferEXT', 'glUnmapNamedBufferEXT',
'glGetNamedBufferParameterivEXT', 'glGetNamedBufferPointervEXT',
'glGetNamedBufferSubDataEXT', 'glTextureBufferEXT', 'glMultiTexBufferEXT',
'glNamedRenderbufferStorageEXT', 'glGetNamedRenderbufferParameterivEXT',
'glCheckNamedFramebufferStatusEXT', 'glNamedFramebufferTexture1DEXT',
'glNamedFramebufferTexture2DEXT', 'glNamedFramebufferTexture3DEXT',
'glNamedFramebufferRenderbufferEXT',
'glGetNamedFramebufferAttachmentParameterivEXT', 'glGenerateTextureMipmapEXT',
'glGenerateMultiTexMipmapEXT', 'glFramebufferDrawBufferEXT',
'glFramebufferDrawBuffersEXT', 'glFramebufferReadBufferEXT',
'glGetFramebufferParameterivEXT', 'glNamedRenderbufferStorageMultisampleEXT',
'glNamedRenderbufferStorageMultisampleCoverageEXT',
'glNamedFramebufferTextureEXT', 'glNamedFramebufferTextureLayerEXT',
'glNamedFramebufferTextureFaceEXT', 'glTextureRenderbufferEXT',
'glMultiTexRenderbufferEXT', 'PFNGLCLIENTATTRIBDEFAULTEXTPROC',
'PFNGLPUSHCLIENTATTRIBDEFAULTEXTPROC', 'PFNGLMATRIXLOADFEXTPROC',
'PFNGLMATRIXLOADDEXTPROC', 'PFNGLMATRIXMULTFEXTPROC',
'PFNGLMATRIXMULTDEXTPROC', 'PFNGLMATRIXLOADIDENTITYEXTPROC',
'PFNGLMATRIXROTATEFEXTPROC', 'PFNGLMATRIXROTATEDEXTPROC',
'PFNGLMATRIXSCALEFEXTPROC', 'PFNGLMATRIXSCALEDEXTPROC',
'PFNGLMATRIXTRANSLATEFEXTPROC', 'PFNGLMATRIXTRANSLATEDEXTPROC',
'PFNGLMATRIXFRUSTUMEXTPROC', 'PFNGLMATRIXORTHOEXTPROC',
'PFNGLMATRIXPOPEXTPROC', 'PFNGLMATRIXPUSHEXTPROC',
'PFNGLMATRIXLOADTRANSPOSEFEXTPROC', 'PFNGLMATRIXLOADTRANSPOSEDEXTPROC',
'PFNGLMATRIXMULTTRANSPOSEFEXTPROC', 'PFNGLMATRIXMULTTRANSPOSEDEXTPROC',
'PFNGLTEXTUREPARAMETERFEXTPROC', 'PFNGLTEXTUREPARAMETERFVEXTPROC',
'PFNGLTEXTUREPARAMETERIEXTPROC', 'PFNGLTEXTUREPARAMETERIVEXTPROC',
'PFNGLTEXTUREIMAGE1DEXTPROC', 'PFNGLTEXTUREIMAGE2DEXTPROC',
'PFNGLTEXTURESUBIMAGE1DEXTPROC', 'PFNGLTEXTURESUBIMAGE2DEXTPROC',
'PFNGLCOPYTEXTUREIMAGE1DEXTPROC', 'PFNGLCOPYTEXTUREIMAGE2DEXTPROC',
'PFNGLCOPYTEXTURESUBIMAGE1DEXTPROC', 'PFNGLCOPYTEXTURESUBIMAGE2DEXTPROC',
'PFNGLGETTEXTUREIMAGEEXTPROC', 'PFNGLGETTEXTUREPARAMETERFVEXTPROC',
'PFNGLGETTEXTUREPARAMETERIVEXTPROC', 'PFNGLGETTEXTURELEVELPARAMETERFVEXTPROC',
'PFNGLGETTEXTURELEVELPARAMETERIVEXTPROC', 'PFNGLTEXTUREIMAGE3DEXTPROC',
'PFNGLTEXTURESUBIMAGE3DEXTPROC', 'PFNGLCOPYTEXTURESUBIMAGE3DEXTPROC',
'PFNGLMULTITEXPARAMETERFEXTPROC', 'PFNGLMULTITEXPARAMETERFVEXTPROC',
'PFNGLMULTITEXPARAMETERIEXTPROC', 'PFNGLMULTITEXPARAMETERIVEXTPROC',
'PFNGLMULTITEXIMAGE1DEXTPROC', 'PFNGLMULTITEXIMAGE2DEXTPROC',
'PFNGLMULTITEXSUBIMAGE1DEXTPROC', 'PFNGLMULTITEXSUBIMAGE2DEXTPROC',
'PFNGLCOPYMULTITEXIMAGE1DEXTPROC', 'PFNGLCOPYMULTITEXIMAGE2DEXTPROC',
'PFNGLCOPYMULTITEXSUBIMAGE1DEXTPROC', 'PFNGLCOPYMULTITEXSUBIMAGE2DEXTPROC',
'PFNGLGETMULTITEXIMAGEEXTPROC', 'PFNGLGETMULTITEXPARAMETERFVEXTPROC',
'PFNGLGETMULTITEXPARAMETERIVEXTPROC',
'PFNGLGETMULTITEXLEVELPARAMETERFVEXTPROC',
'PFNGLGETMULTITEXLEVELPARAMETERIVEXTPROC', 'PFNGLMULTITEXIMAGE3DEXTPROC',
'PFNGLMULTITEXSUBIMAGE3DEXTPROC', 'PFNGLCOPYMULTITEXSUBIMAGE3DEXTPROC',
'PFNGLBINDMULTITEXTUREEXTPROC', 'PFNGLENABLECLIENTSTATEINDEXEDEXTPROC',
'PFNGLDISABLECLIENTSTATEINDEXEDEXTPROC', 'PFNGLMULTITEXCOORDPOINTEREXTPROC',
'PFNGLMULTITEXENVFEXTPROC', 'PFNGLMULTITEXENVFVEXTPROC',
'PFNGLMULTITEXENVIEXTPROC', 'PFNGLMULTITEXENVIVEXTPROC',
'PFNGLMULTITEXGENDEXTPROC', 'PFNGLMULTITEXGENDVEXTPROC',
'PFNGLMULTITEXGENFEXTPROC', 'PFNGLMULTITEXGENFVEXTPROC',
'PFNGLMULTITEXGENIEXTPROC', 'PFNGLMULTITEXGENIVEXTPROC',
'PFNGLGETMULTITEXENVFVEXTPROC', 'PFNGLGETMULTITEXENVIVEXTPROC',
'PFNGLGETMULTITEXGENDVEXTPROC', 'PFNGLGETMULTITEXGENFVEXTPROC',
'PFNGLGETMULTITEXGENIVEXTPROC', 'PFNGLGETFLOATINDEXEDVEXTPROC',
'PFNGLGETDOUBLEINDEXEDVEXTPROC', 'PFNGLGETPOINTERINDEXEDVEXTPROC',
'PFNGLCOMPRESSEDTEXTUREIMAGE3DEXTPROC',
'PFNGLCOMPRESSEDTEXTUREIMAGE2DEXTPROC',
'PFNGLCOMPRESSEDTEXTUREIMAGE1DEXTPROC',
'PFNGLCOMPRESSEDTEXTURESUBIMAGE3DEXTPROC',
'PFNGLCOMPRESSEDTEXTURESUBIMAGE2DEXTPROC',
'PFNGLCOMPRESSEDTEXTURESUBIMAGE1DEXTPROC',
'PFNGLGETCOMPRESSEDTEXTUREIMAGEEXTPROC',
'PFNGLCOMPRESSEDMULTITEXIMAGE3DEXTPROC',
'PFNGLCOMPRESSEDMULTITEXIMAGE2DEXTPROC',
'PFNGLCOMPRESSEDMULTITEXIMAGE1DEXTPROC',
'PFNGLCOMPRESSEDMULTITEXSUBIMAGE3DEXTPROC',
'PFNGLCOMPRESSEDMULTITEXSUBIMAGE2DEXTPROC',
'PFNGLCOMPRESSEDMULTITEXSUBIMAGE1DEXTPROC',
'PFNGLGETCOMPRESSEDMULTITEXIMAGEEXTPROC', 'PFNGLNAMEDPROGRAMSTRINGEXTPROC',
'PFNGLNAMEDPROGRAMLOCALPARAMETER4DEXTPROC',
'PFNGLNAMEDPROGRAMLOCALPARAMETER4DVEXTPROC',
'PFNGLNAMEDPROGRAMLOCALPARAMETER4FEXTPROC',
'PFNGLNAMEDPROGRAMLOCALPARAMETER4FVEXTPROC',
'PFNGLGETNAMEDPROGRAMLOCALPARAMETERDVEXTPROC',
'PFNGLGETNAMEDPROGRAMLOCALPARAMETERFVEXTPROC',
'PFNGLGETNAMEDPROGRAMIVEXTPROC', 'PFNGLGETNAMEDPROGRAMSTRINGEXTPROC',
'PFNGLNAMEDPROGRAMLOCALPARAMETERS4FVEXTPROC',
'PFNGLNAMEDPROGRAMLOCALPARAMETERI4IEXTPROC',
'PFNGLNAMEDPROGRAMLOCALPARAMETERI4IVEXTPROC',
'PFNGLNAMEDPROGRAMLOCALPARAMETERSI4IVEXTPROC',
'PFNGLNAMEDPROGRAMLOCALPARAMETERI4UIEXTPROC',
'PFNGLNAMEDPROGRAMLOCALPARAMETERI4UIVEXTPROC',
'PFNGLNAMEDPROGRAMLOCALPARAMETERSI4UIVEXTPROC',
'PFNGLGETNAMEDPROGRAMLOCALPARAMETERIIVEXTPROC',
'PFNGLGETNAMEDPROGRAMLOCALPARAMETERIUIVEXTPROC',
'PFNGLTEXTUREPARAMETERIIVEXTPROC', 'PFNGLTEXTUREPARAMETERIUIVEXTPROC',
'PFNGLGETTEXTUREPARAMETERIIVEXTPROC', 'PFNGLGETTEXTUREPARAMETERIUIVEXTPROC',
'PFNGLMULTITEXPARAMETERIIVEXTPROC', 'PFNGLMULTITEXPARAMETERIUIVEXTPROC',
'PFNGLGETMULTITEXPARAMETERIIVEXTPROC', 'PFNGLGETMULTITEXPARAMETERIUIVEXTPROC',
'PFNGLPROGRAMUNIFORM1FEXTPROC', 'PFNGLPROGRAMUNIFORM2FEXTPROC',
'PFNGLPROGRAMUNIFORM3FEXTPROC', 'PFNGLPROGRAMUNIFORM4FEXTPROC',
'PFNGLPROGRAMUNIFORM1IEXTPROC', 'PFNGLPROGRAMUNIFORM2IEXTPROC',
'PFNGLPROGRAMUNIFORM3IEXTPROC', 'PFNGLPROGRAMUNIFORM4IEXTPROC',
'PFNGLPROGRAMUNIFORM1FVEXTPROC', 'PFNGLPROGRAMUNIFORM2FVEXTPROC',
'PFNGLPROGRAMUNIFORM3FVEXTPROC', 'PFNGLPROGRAMUNIFORM4FVEXTPROC',
'PFNGLPROGRAMUNIFORM1IVEXTPROC', 'PFNGLPROGRAMUNIFORM2IVEXTPROC',
'PFNGLPROGRAMUNIFORM3IVEXTPROC', 'PFNGLPROGRAMUNIFORM4IVEXTPROC',
'PFNGLPROGRAMUNIFORMMATRIX2FVEXTPROC', 'PFNGLPROGRAMUNIFORMMATRIX3FVEXTPROC',
'PFNGLPROGRAMUNIFORMMATRIX4FVEXTPROC',
'PFNGLPROGRAMUNIFORMMATRIX2X3FVEXTPROC',
'PFNGLPROGRAMUNIFORMMATRIX3X2FVEXTPROC',
'PFNGLPROGRAMUNIFORMMATRIX2X4FVEXTPROC',
'PFNGLPROGRAMUNIFORMMATRIX4X2FVEXTPROC',
'PFNGLPROGRAMUNIFORMMATRIX3X4FVEXTPROC',
'PFNGLPROGRAMUNIFORMMATRIX4X3FVEXTPROC', 'PFNGLPROGRAMUNIFORM1UIEXTPROC',
'PFNGLPROGRAMUNIFORM2UIEXTPROC', 'PFNGLPROGRAMUNIFORM3UIEXTPROC',
'PFNGLPROGRAMUNIFORM4UIEXTPROC', 'PFNGLPROGRAMUNIFORM1UIVEXTPROC',
'PFNGLPROGRAMUNIFORM2UIVEXTPROC', 'PFNGLPROGRAMUNIFORM3UIVEXTPROC',
'PFNGLPROGRAMUNIFORM4UIVEXTPROC', 'PFNGLNAMEDBUFFERDATAEXTPROC',
'PFNGLNAMEDBUFFERSUBDATAEXTPROC', 'PFNGLMAPNAMEDBUFFEREXTPROC',
'PFNGLUNMAPNAMEDBUFFEREXTPROC', 'PFNGLGETNAMEDBUFFERPARAMETERIVEXTPROC',
'PFNGLGETNAMEDBUFFERPOINTERVEXTPROC', 'PFNGLGETNAMEDBUFFERSUBDATAEXTPROC',
'PFNGLTEXTUREBUFFEREXTPROC', 'PFNGLMULTITEXBUFFEREXTPROC',
'PFNGLNAMEDRENDERBUFFERSTORAGEEXTPROC',
'PFNGLGETNAMEDRENDERBUFFERPARAMETERIVEXTPROC',
'PFNGLCHECKNAMEDFRAMEBUFFERSTATUSEXTPROC',
'PFNGLNAMEDFRAMEBUFFERTEXTURE1DEXTPROC',
'PFNGLNAMEDFRAMEBUFFERTEXTURE2DEXTPROC',
'PFNGLNAMEDFRAMEBUFFERTEXTURE3DEXTPROC',
'PFNGLNAMEDFRAMEBUFFERRENDERBUFFEREXTPROC',
'PFNGLGETNAMEDFRAMEBUFFERATTACHMENTPARAMETERIVEXTPROC',
'PFNGLGENERATETEXTUREMIPMAPEXTPROC', 'PFNGLGENERATEMULTITEXMIPMAPEXTPROC',
'PFNGLFRAMEBUFFERDRAWBUFFEREXTPROC', 'PFNGLFRAMEBUFFERDRAWBUFFERSEXTPROC',
'PFNGLFRAMEBUFFERREADBUFFEREXTPROC', 'PFNGLGETFRAMEBUFFERPARAMETERIVEXTPROC',
'PFNGLNAMEDRENDERBUFFERSTORAGEMULTISAMPLEEXTPROC',
'PFNGLNAMEDRENDERBUFFERSTORAGEMULTISAMPLECOVERAGEEXTPROC',
'PFNGLNAMEDFRAMEBUFFERTEXTUREEXTPROC',
'PFNGLNAMEDFRAMEBUFFERTEXTURELAYEREXTPROC',
'PFNGLNAMEDFRAMEBUFFERTEXTUREFACEEXTPROC', 'PFNGLTEXTURERENDERBUFFEREXTPROC',
'PFNGLMULTITEXRENDERBUFFEREXTPROC', 'GL_EXT_vertex_array_bgra',
'GL_EXT_texture_swizzle', 'GL_NV_explicit_multisample',
'glGetMultisamplefvNV', 'glSampleMaskIndexedNV', 'glTexRenderbufferNV',
'PFNGLGETMULTISAMPLEFVNVPROC', 'PFNGLSAMPLEMASKINDEXEDNVPROC',
'PFNGLTEXRENDERBUFFERNVPROC', 'GL_NV_transform_feedback2',
'glBindTransformFeedbackNV', 'glDeleteTransformFeedbacksNV',
'glGenTransformFeedbacksNV', 'glIsTransformFeedbackNV',
'glPauseTransformFeedbackNV', 'glResumeTransformFeedbackNV',
'glDrawTransformFeedbackNV', 'PFNGLBINDTRANSFORMFEEDBACKNVPROC',
'PFNGLDELETETRANSFORMFEEDBACKSNVPROC', 'PFNGLGENTRANSFORMFEEDBACKSNVPROC',
'PFNGLISTRANSFORMFEEDBACKNVPROC', 'PFNGLPAUSETRANSFORMFEEDBACKNVPROC',
'PFNGLRESUMETRANSFORMFEEDBACKNVPROC', 'PFNGLDRAWTRANSFORMFEEDBACKNVPROC',
'GL_NV_vertex_buffer_unified_memory', 'glBufferAddressRangeNV',
'glVertexFormatNV', 'glNormalFormatNV', 'glColorFormatNV', 'glIndexFormatNV',
'glTexCoordFormatNV', 'glEdgeFlagFormatNV', 'glSecondaryColorFormatNV',
'glFogCoordFormatNV', 'glVertexAttribFormatNV', 'glVertexAttribIFormatNV',
'glGetIntegerui64i_vNV', 'PFNGLBUFFERADDRESSRANGENVPROC',
'PFNGLVERTEXFORMATNVPROC', 'PFNGLNORMALFORMATNVPROC',
'PFNGLCOLORFORMATNVPROC', 'PFNGLINDEXFORMATNVPROC',
'PFNGLTEXCOORDFORMATNVPROC', 'PFNGLEDGEFLAGFORMATNVPROC',
'PFNGLSECONDARYCOLORFORMATNVPROC', 'PFNGLFOGCOORDFORMATNVPROC',
'PFNGLVERTEXATTRIBFORMATNVPROC', 'PFNGLVERTEXATTRIBIFORMATNVPROC',
'PFNGLGETINTEGERUI64I_VNVPROC', 'GL_NV_shader_buffer_load',
'glGetBufferParameterui64vNV', 'glGetIntegerui64vNV',
'glGetNamedBufferParameterui64vNV', 'glIsBufferResidentNV',
'glIsNamedBufferResidentNV', 'glMakeBufferNonResidentNV',
'glMakeBufferResidentNV', 'glMakeNamedBufferNonResidentNV',
'glMakeNamedBufferResidentNV', 'glUniformui64NV', 'glUniformui64vNV',
'glGetUniformui64vNV', 'glProgramUniformui64NV', 'glProgramUniformui64vNV',
'PFNGLGETBUFFERPARAMETERUI64VNVPROC', 'PFNGLGETINTEGERUI64VNVPROC',
'PFNGLGETNAMEDBUFFERPARAMETERUI64VNVPROC', 'PFNGLISBUFFERRESIDENTNVPROC',
'PFNGLISNAMEDBUFFERRESIDENTNVPROC', 'PFNGLMAKEBUFFERNONRESIDENTNVPROC',
'PFNGLMAKEBUFFERRESIDENTNVPROC', 'PFNGLMAKENAMEDBUFFERNONRESIDENTNVPROC',
'PFNGLMAKENAMEDBUFFERRESIDENTNVPROC', 'PFNGLUNIFORMUI64NVPROC',
'PFNGLUNIFORMUI64VNVPROC', 'PFNGLGETUNIFORMUI64VNVPROC',
'PFNGLPROGRAMUNIFORMUI64NVPROC', 'PFNGLPROGRAMUNIFORMUI64VNVPROC']
# END GENERATED CONTENT (do not edit above this line)







